'use strict';

// eslint-disable-next-line eslint-comments/disable-enable-pair
/* eslint-disable @typescript-eslint/naming-convention */
let LevelResolver = /*#__PURE__*/function (LevelResolver) {
  LevelResolver[LevelResolver["silent"] = 0] = "silent";
  LevelResolver[LevelResolver["error"] = 1] = "error";
  LevelResolver[LevelResolver["warn"] = 2] = "warn";
  LevelResolver[LevelResolver["info"] = 3] = "info";
  LevelResolver[LevelResolver["debug"] = 4] = "debug";
  return LevelResolver;
}({});
const shouldLog = (currentLevel, level) => LevelResolver[level] <= currentLevel;

/**
 * A Logger using console output.
 *
 * @param logLevel - The logger level name
 * @param prefix - An optional logger message prefix
 * @param output - The output to print logs, using by default the global console object
 *
 * @internal
 */
class ConsoleLogger {
  constructor(logLevel, prefix = '', output = console) {
    this.logLevel = logLevel;
    this.prefix = prefix;
    this.output = output;
    this.level = LevelResolver[this.logLevel];
  }
  makeMethod(method) {
    return message => {
      if (shouldLog(this.level, method)) {
        this.output[method](this.prefix ? `${this.prefix} ${message}` : message);
      }
    };
  }
  debug = this.makeMethod('debug');
  error = this.makeMethod('error');
  info = this.makeMethod('info');
  warn = this.makeMethod('warn');
}

let sdkLogger = /*#__PURE__*/new ConsoleLogger('silent');

/**
 * Sets a logger to be used within the SDK.
 *
 * @param logger - The Logger instance
 *
 * @public
 */
const setLogger = logger => {
  sdkLogger = logger;
};

/**
 * Sets the logger to console logger with given logLevel (log is disabled by default).
 *
 * @param logLevel - The Log level (default to 'warn')
 * @param prefix - A Log message prefix (default to 'scaleway-sdk-js:')
 *
 * @public
 */
const enableConsoleLogger = (logLevel = 'warn', prefix = 'scaleway-sdk-js:') => setLogger(new ConsoleLogger(logLevel, prefix));

/**
 * Returns the active SDK logger.
 *
 * @internal
 */
const getLogger = () => sdkLogger;

/**
 * Adds an header to a request through an interceptor.
 *
 * @param key - The header key
 * @param value - The header value
 * @returns The Request interceptor
 *
 * @internal
 */
const addHeaderInterceptor = (key, value) => ({
  request
}) => {
  const clone = request.clone();
  if (value !== undefined) {
    clone.headers.append(key, value);
  }
  return clone;
};

/**
 * Adds asynchronously an header to a request through an interceptor.
 *
 * @param key - The header key
 * @param value - The header value as a Promise
 * @returns The Request interceptor
 *
 * @internal
 */
const addAsyncHeaderInterceptor = (key, getter) => async request => addHeaderInterceptor(key, await getter())(request);

const isAccessKeyRegex = /^SCW[A-Z0-9]{17}$/i;
const isRegionRegex = /^[a-z]{2}-[a-z]{3}$/i;
const isUUIDRegex = /^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/i;
const isZoneRegex = /^[a-z]{2}-[a-z]{3}-[1-9]$/i;

/** Returns true if the given string has a valid UUID format. */
const isUUID = str => isUUIDRegex.test(str);

/** Returns true if the given string has a valid Scaleway access key format. */
const isAccessKey = str => isAccessKeyRegex.test(str);

/** Returns true if the given string has a valid Scaleway secret key format. */
const isSecretKey = str => isUUID(str);

/** Returns true if the given string has a valid Scaleway organization ID format. */
const isOrganizationId = str => isUUID(str);

/** Returns true if the given string has a valid Scaleway project ID format. */
const isProjectId = str => isUUID(str);

/** Returns true if the given string has a valid region format. */
const isRegion = str => isRegionRegex.test(str);

/** Returns true if the given string has a valid zone format. */
const isZone = str => isZoneRegex.test(str);

/** Returns true if the given string has a valid URL format and starts by `http(s):`. */
const isURL = str => {
  let url;
  try {
    url = new URL(str);
  } catch {
    return false;
  }
  return url.protocol === 'http:' || url.protocol === 'https:';
};

/**
 * Holds access key and secret key.
 *
 * @public
 */

/**
 * Holds default values of a Scaleway profile.
 *
 * @public
 */

/**
 * Holds values of a Scaleway profile.
 *
 * @public
 */

/**
 * Verifies that the payload contains both the accessKey and the secretKey.
 *
 * @param obj - The secrets
 * @returns Whether the secrets are not empty.
 *
 * @internal
 */
const hasAuthenticationSecrets = obj => typeof obj.accessKey === 'string' && obj.accessKey !== '' && typeof obj.secretKey === 'string' && obj.secretKey !== '';

/**
 * Asserts the format of secrets.
 *
 * @param obj - The secrets
 * @returns Whether the secrets use a valid format
 *
 * @throws Error
 * Thrown if either the accessKey or the secretKey has en invalid format.
 *
 * @internal
 */
function assertValidAuthenticationSecrets(obj) {
  if (!(obj.accessKey && obj.secretKey)) {
    throw new Error(`Invalid secrets, accessKey & secretKey must be defined. See https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/`);
  }
  if (!isAccessKey(obj.accessKey)) {
    throw new Error(`Invalid access key format '${obj.accessKey}', expected SCWXXXXXXXXXXXXXXXXX format. See https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/`);
  }
  if (!isSecretKey(obj.secretKey)) {
    throw new Error(`Invalid secret key format '${obj.secretKey}', expected a UUID: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx. See https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/`);
  }
}

const SESSION_HEADER_KEY = 'x-session-token';
const AUTH_HEADER_KEY = 'x-auth-token';
/**
 * Authenticates with a session token.
 *
 * @param getToken - The token accessor
 * @returns The request interceptor
 *
 * @deprecated Please use addAsyncHeaderInterceptor instead.
 *
 * @internal
 */
const authenticateWithSessionToken = getToken => addAsyncHeaderInterceptor(SESSION_HEADER_KEY, getToken);

/**
 * Authenticates with a secrets.
 *
 * @param getToken - The secrets
 * @returns The request interceptor
 *
 * @throws Error
 * Thrown if the secrets are invalid.
 *
 * @internal
 */
const authenticateWithSecrets = secrets => {
  assertValidAuthenticationSecrets(secrets);
  return addHeaderInterceptor(AUTH_HEADER_KEY, secrets.secretKey);
};

/**
 * Obfuscates a token.
 *
 * @param key - The token
 * @returns The obfuscated token
 *
 * @internal
 */
const obfuscateToken = key => `${key.substring(0, 5)}xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx`;

/**
 * Obfuscates an UUID.
 *
 * @param key - The UUID
 * @returns The obfuscated UUID
 *
 * @internal
 */
const obfuscateUUID = key => `${key.substring(0, 8)}-xxxx-xxxx-xxxx-xxxxxxxxxxxx`;
/**
 * Obfuscates headers entry.
 *
 * @param array - The header entry
 * @returns The obfuscated entry
 *
 * @internal
 */
const obfuscateAuthHeadersEntry = ([name, value]) => {
  if (name === SESSION_HEADER_KEY) return [name, obfuscateToken(value)];
  if (name === AUTH_HEADER_KEY) return [name, obfuscateUUID(value)];
  return [name, value];
};

/**
 * A factory to build {@link Settings}.
 *
 * @public
 */

/**
 * Instantiates the SDK from a configuration {@link Profile}.
 *
 * @param profile - The profile
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
const withProfile = profile => settings => {
  const newSettings = {
    ...settings
  };
  if (profile.apiURL) {
    newSettings.apiURL = profile.apiURL;
  }
  if (profile.defaultOrganizationId) {
    newSettings.defaultOrganizationId = profile.defaultOrganizationId;
  }
  if (profile.defaultProjectId) {
    newSettings.defaultProjectId = profile.defaultProjectId;
  }
  if (profile.defaultRegion) {
    newSettings.defaultRegion = profile.defaultRegion;
  }
  if (profile.defaultZone) {
    newSettings.defaultZone = profile.defaultZone;
  }
  if (hasAuthenticationSecrets(profile)) {
    newSettings.interceptors = [{
      request: authenticateWithSecrets(profile)
    }, ...newSettings.interceptors];
  }
  return newSettings;
};

/**
 * Instantiates the SDK with a different HTTP client.
 *
 * @param httpClient - A fetch compatible HTTP client
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
const withHTTPClient = httpClient => settings => ({
  ...settings,
  httpClient
});

/**
 * Instantiates the SDK with a default page size.
 *
 * @param defaultPageSize - The default page size
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
const withDefaultPageSize = defaultPageSize => settings => ({
  ...settings,
  defaultPageSize
});

/**
 * Instantiates the SDK with a different default user agent.
 *
 * @param userAgent - The default user agent
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
const withUserAgent = userAgent => settings => ({
  ...settings,
  userAgent
});

/**
 * Instantiates the SDK with an additional user agent.
 *
 * @param userAgent - The suffix to append to default user agent
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
const withUserAgentSuffix = userAgent => settings => ({
  ...settings,
  userAgent: settings.userAgent ? `${settings.userAgent} ${userAgent}` : userAgent
});

/**
 * Instantiates the SDK with additional interceptors.
 *
 * @param interceptors - The additional {@link NetworkInterceptors} interceptors
 * @returns A factory {@link ClientConfig}
 *
 * @remarks
 * It doesn't override the existing interceptors, but instead push more to the list.
 * This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @example
 * ```
 * withAdditionalInterceptors([
 *   {
 *     request: ({ request }) => {
 *       console.log(`Do something with ${JSON.stringify(request)}`)
 *       return request
 *     },
 *     response: ({ response }) => {
 *       console.log(`Do something with ${JSON.stringify(response)}`)
 *       return response
 *     },
 *     responseError: async ({
 *       request,
 *       error,
 *     }: {
 *       request: Request
 *       error: unknown
 *     }) => {
 *       console.log(
 *         `Do something with ${JSON.stringify(request)} and ${JSON.stringify(
 *           error,
 *         )}`,
 *       )
 *       throw error // or return Promise.resolve(someData)
 *     },
 *   },
 * ])
 * ```
 *
 * @public
 */
const withAdditionalInterceptors = interceptors => settings => ({
  ...settings,
  interceptors: settings.interceptors.concat(interceptors)
});

/**
 * Instantiates the SDK with legacy interceptors.
 */
/* eslint-disable deprecation/deprecation */
const withLegacyInterceptors = () => settings => {
  if (!settings.requestInterceptors && !settings.responseInterceptors) {
    return settings;
  }
  const allInterceptors = settings.interceptors.concat((settings.requestInterceptors ?? []).map(obj => ({
    request: obj
  })), (settings.responseInterceptors ?? []).map(obj => ({
    response: obj
  })));
  return {
    ...settings,
    interceptors: allInterceptors
  };
};
/* eslint-enable deprecation/deprecation */

/**
 * Holds default values of settings.
 *
 * @public
 */

/**
 * Settings hold the values of all client options.
 *
 * @public
 */

/**
 * Validates the content of a {@link Settings} object.
 *
 * @throws Error
 * Thrown if {@link Settings} aren't valid.
 *
 * @internal
 */
const assertValidSettings = obj => {
  // Default Organization ID.
  if (obj.defaultOrganizationId !== undefined) {
    if (typeof obj.defaultOrganizationId !== 'string' || obj.defaultOrganizationId.length === 0) {
      throw new Error('Default organization ID cannot be empty');
    }
    if (!isOrganizationId(obj.defaultOrganizationId)) {
      throw new Error(`Invalid organization ID format '${obj.defaultOrganizationId}', expected a UUID: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx`);
    }
  }

  // Default Project ID.
  if (obj.defaultProjectId !== undefined) {
    if (typeof obj.defaultProjectId !== 'string' || obj.defaultProjectId.length === 0) {
      throw new Error('Default project ID cannot be empty');
    }
    if (!isProjectId(obj.defaultProjectId)) {
      throw new Error(`Invalid project ID format '${obj.defaultProjectId}', expected a UUID: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx`);
    }
  }

  // Default Region.
  if (obj.defaultRegion && !isRegion(obj.defaultRegion)) {
    throw new Error(`Invalid default region format '${obj.defaultRegion}'`);
  }

  // Default Zone.
  if (obj.defaultZone && !isZone(obj.defaultZone)) {
    throw new Error(`Invalid default zone format '${obj.defaultZone}'`);
  }

  // API URL.
  if (!isURL(obj.apiURL)) {
    throw new Error(`Invalid URL ${obj.apiURL}`);
  }
  if (obj.apiURL.endsWith('/')) {
    throw new Error(`Invalid URL ${obj.apiURL}: it should not have a trailing slash`);
  }

  // HTTP Client.
  if (typeof obj.httpClient !== typeof fetch) {
    throw new Error(`Invalid HTTP Client`);
  }

  // Default Page Size.
  if (obj.defaultPageSize !== undefined && (typeof obj.defaultPageSize !== 'number' || Number.isNaN(obj.defaultPageSize) || obj.defaultPageSize <= 0)) {
    throw new Error(`Invalid defaultPageSize ${obj.defaultPageSize}: it should be a number above 0`);
  }

  // User Agent.
  if (typeof obj.userAgent !== 'string') {
    throw new Error(`Invalid User-Agent`);
  }
};

const version = 'v2.25.0';
const userAgent = `scaleway-sdk-js/${version}`;

const isBrowser = () =>
// eslint-disable-next-line @typescript-eslint/prefer-optional-chain
typeof window !== 'undefined' && typeof window.document !== 'undefined';

/**
 * Composes request interceptors.
 *
 * @param interceptors - A list of request interceptors
 * @returns An async composed interceptor
 *
 * @internal
 */
const composeRequestInterceptors = interceptors => async request => interceptors.reduce(async (asyncResult, interceptor) => interceptor({
  request: await asyncResult
}), Promise.resolve(request));

/**
 * Composes response interceptors.
 *
 * @param interceptors - A list of response interceptors
 * @returns An async composed interceptor
 *
 * @internal
 */
const composeResponseInterceptors = interceptors => async response => interceptors.reduce(async (asyncResult, interceptor) => interceptor({
  response: await asyncResult
}), Promise.resolve(response));

/**
 * Compose response error interceptors.
 *
 * @internal
 */
const composeResponseErrorInterceptors = interceptors => async (request, error) => {
  let prevError = error;
  for (const interceptor of interceptors) {
    try {
      const res = await interceptor({
        request,
        error: prevError
      });
      return res;
    } catch (err) {
      prevError = err;
    }
  }
  throw prevError;
};

/**
 * Converts a string to PascalCase.
 *
 * @param str - The input string
 * @returns The string in PascalCase
 *
 * @internal
 */
const toPascalCase = str => str.replace(/\w+/g, word => `${word[0].toUpperCase()}${word.slice(1).toLowerCase()}`);

/**
 * Converts a Headers entry to string.
 *
 * @param entry - The header entry as a string tuple
 * @returns A serialized string
 *
 * @internal
 */
const serializeHeadersEntry = ([name, value]) => `${toPascalCase(name)}: ${value}`;

/**
 * Converts Headers to safe to log strings (with obfuscated auth secrets).
 *
 * @param headers - The Headers
 * @returns Serialized headers strings
 *
 * @internal
 */
const serializeHeaders = headers => Array.from(headers.entries(), serializeHeadersEntry);

/**
 * Dumps a Request into a readable string.
 *
 * @param request - The request
 * @returns The readable string
 *
 * @internal
 */
const dumpRequest = async request => [`${request.method.toUpperCase()}: ${request.url}`, ...serializeHeaders(request.headers), await request.clone().text()].join('\r\n');

/**
 * Dumps a Response into a readable string.
 *
 * @param response - The response
 * @returns The readable string
 *
 * @internal
 */
const dumpResponse = async response => [`HTTP ${response.status} ${response.ok ? 'OK' : 'NOK'}`, ...serializeHeaders(response.headers), await response.clone().text()].join('\r\n');

/**
 * Mapper of an header entry.
 *
 * @internal
 */

/**
 * HTTP Request with obfuscated secrets.
 *
 * @internal
 */
class ObfuscatedRequest extends Request {
  constructor(request, obfuscate) {
    super(request);
    this.request = request;
    this.obfuscate = obfuscate;
  }
  get headers() {
    return new Headers(Array.from(this.request.headers, this.obfuscate));
  }
  clone() {
    return new ObfuscatedRequest(this.request, this.obfuscate);
  }
}

/**
 * Creates an interceptor to obfuscate the requests.
 *
 * @param obfuscate - The Header entries obfuscator mapper
 * @returns The obfuscated Request
 *
 * @internal
 */
const obfuscateInterceptor = obfuscate => ({
  request
}) => new ObfuscatedRequest(request, obfuscate);
const identity = ({
  request
}) => request;

/**
 * Creates an interceptor to log the requests.
 *
 * @param identifier - The request identifier
 * @param obfuscate - The obfuscation interceptor
 * @returns The interceptor
 *
 * @internal
 */
const logRequest = (identifier, obfuscate = identity) => async ({
  request
}) => {
  if (shouldLog(LevelResolver[getLogger().logLevel], 'debug')) {
    getLogger().debug(`--------------- Scaleway SDK REQUEST ${identifier} ---------------
${await dumpRequest(await obfuscate({
      request
    }))}
---------------------------------------------------------`);
  }
  return request;
};

/**
 * Creates an interceptor to log the responses.
 *
 * @param identifier - The request identifier
 * @returns The interceptor
 *
 * @internal
 */
const logResponse = identifier => async ({
  response
}) => {
  if (shouldLog(LevelResolver[getLogger().logLevel], 'debug')) {
    getLogger().debug(`--------------- Scaleway SDK RESPONSE ${identifier} ---------------
${await dumpResponse(response)}
---------------------------------------------------------`);
  }
  return response;
};

/**
 * Validates an object is of type Response without using `instanceof`.
 *
 * @remarks Check issue #509 for more context.
 *
 * @internal
 */
const isResponse = obj => obj !== null && obj !== undefined && typeof obj === 'object' && 'status' in obj && typeof obj.status === 'number' && 'statusText' in obj && typeof obj.statusText === 'string' && 'headers' in obj && typeof obj.headers === 'object' && 'body' in obj && typeof obj.body !== 'undefined';

/**
 * Validates an unknown object is a JSON Object.
 *
 * @internal
 */
const isJSONObject = obj => {
  const objT = typeof obj;
  return obj !== undefined && obj !== null && objT !== 'string' && objT !== 'number' && objT !== 'boolean' && !Array.isArray(obj) && objT === 'object';
};

/**
 * Verifies the object is a record of string to string[].
 *
 * @param obj - The object
 * @returns Whether the object is of the expected type
 *
 * @internal
 */
const isRecordOfStringArray = obj => {
  if (!isJSONObject(obj)) {
    return false;
  }
  for (const elt of Object.values(obj)) {
    if (!Array.isArray(elt) || Object.values(elt).find(x => typeof x !== 'string') !== undefined) {
      return false;
    }
  }
  return true;
};

/**
 * Builds the default message for {@link ScalewayError}.
 *
 * @param status - The response code
 * @param body - The response body
 * @returns The error message
 *
 * @internal
 */
const buildDefaultMessage = (status, body) => {
  const message = [`http error ${status}`];
  if (typeof body === 'string') {
    message.push(body);
  } else if (isJSONObject(body)) {
    if (typeof body.resource === 'string') {
      message.push(`resource ${body.resource}`);
    }
    if (typeof body.message === 'string') {
      message.push(body.message);
    }
    if (body.fields && isRecordOfStringArray(body.fields)) {
      message.push(Object.entries(body.fields).map(([name, list]) => `${name} (${list.join(', ')})`).join(', '));
    }
  }
  return message.join(': ');
};

/**
 * Scaleway error.
 *
 * @public
 */
class ScalewayError extends Error {
  /** The message originating from the payload. */

  constructor( /** The response status. */
  status, /** The response payload. */
  body, /** The augmented message. */
  message = buildDefaultMessage(status, body)) {
    super(message); // 'Error' breaks prototype chain here
    this.status = status;
    this.body = body;
    this.message = message;
    this.name = 'ScalewayError';
    this.rawMessage = typeof body === 'object' && typeof body.message === 'string' ? body.message : undefined;
    Object.setPrototypeOf(this, new.target.prototype); // restore prototype chain
  }
  static fromJSON(status, obj) {
    return new ScalewayError(status, obj);
  }
  toString() {
    return `${this.name}: ${this.message}`;
  }
}

/**
 * Details of an {@link InvalidArgumentsError} error.
 *
 * @public
 */

/**
 * Build the default message for {@link InvalidArgumentsError}.
 *
 * @param list - The list of {@link InvalidArgumentsErrorDetails}
 * @returns The error message
 *
 * @internal
 */
const buildMessage$5 = list => {
  const invalidArgs = list.reduce((acc, details) => {
    let readableReason = '';
    switch (details.reason) {
      case 'required':
        readableReason = `is required`;
        break;
      case 'format':
        readableReason = `is wrongly formatted`;
        break;
      case 'constraint':
        readableReason = `does not respect constraint`;
        break;
      default:
        readableReason = `is invalid for unexpected reason`;
        break;
    }
    if (details.helpMessage && details.helpMessage.length > 0) {
      readableReason = readableReason.concat(`, `, details.helpMessage);
    }
    acc.push(`${details.argumentName} ${readableReason}`);
    return acc;
  }, []);
  return `invalid argument(s): ${invalidArgs.join('; ')}`;
};

/**
 * InvalidArguments error happens when one or many fields are invalid in the request message.
 *
 * @public
 */
class InvalidArgumentsError extends ScalewayError {
  constructor(status, body, details) {
    super(status, body, buildMessage$5(details));
    this.status = status;
    this.body = body;
    this.details = details;
    this.name = 'InvalidArgumentsError';
  }
  static fromJSON(status, obj) {
    if (!Array.isArray(obj.details)) return null;
    return new InvalidArgumentsError(status, obj, obj.details.reduce((list, detail) => isJSONObject(detail) && typeof detail.argument_name === 'string' && typeof detail.reason === 'string' ? list.concat({
      argumentName: detail.argument_name,
      helpMessage: typeof detail.help_message === 'string' ? detail.help_message : undefined,
      reason: detail.reason
    }) : list, []));
  }
}

/**
 * Scope of an {@link QuotasExceededErrorDetails} error.
 *
 * @public
 */

/**
 * Details of an {@link QuotasExceededError} error.
 *
 * @public
 */

/**
 * Build the default message for {@link QuotasExceededError}.
 *
 * @param list - The list of {@link QuotasExceededErrorDetails}
 * @returns The error message
 *
 * @internal
 */
const buildMessage$4 = list => `quota(s) exceeded: ${list.map(details => {
  const message = `${details.resource} has reached its quota (${details.current}/${details.quota})`;
  return details.scope ? `${message} for ${details.scope.kind} '${details.scope.id}'` : message;
}).join('; ')}`;
const buildScope = detail => {
  if (typeof detail.organization_id === 'string' && detail.organization_id.length) {
    return {
      id: detail.organization_id,
      kind: 'organization'
    };
  }
  if (typeof detail.project_id === 'string' && detail.project_id.length) {
    return {
      id: detail.project_id,
      kind: 'project'
    };
  }
  return undefined;
};

/**
 * QuotasExceeded error happens when one or many resource exceed quotas during the creation of a resource.
 *
 * @public
 */
class QuotasExceededError extends ScalewayError {
  constructor(status, body, list) {
    super(status, body, buildMessage$4(list));
    this.status = status;
    this.body = body;
    this.list = list;
    this.name = 'QuotasExceededError';
  }
  static fromJSON(status, obj) {
    if (!Array.isArray(obj.details)) return null;
    return new QuotasExceededError(status, obj, obj.details.reduce((list, detail) => isJSONObject(detail) && typeof detail.resource === 'string' && typeof detail.quota === 'number' && typeof detail.current === 'number' ? list.concat({
      current: detail.current,
      quota: detail.quota,
      resource: detail.resource,
      scope: buildScope(detail)
    }) : list, []));
  }
}

/**
 * InvalidRequest error is only returned by the instance API.
 *
 * @public
 */
class InvalidRequestMapper {
  static fromJSON(status, obj) {
    if (typeof obj.message === 'string' && obj.message.toLowerCase().includes('quota exceeded for this resource')) {
      return new QuotasExceededError(status, obj, [{
        current: 0,
        quota: 0,
        resource: typeof obj.resource === 'string' ? obj.resource : ''
      }]);
    }
    const fields = obj.fields && isRecordOfStringArray(obj.fields) ? obj.fields : {};
    const fieldsMessages = Object.entries(fields);
    if (fieldsMessages.length) {
      return new InvalidArgumentsError(status, obj, fieldsMessages.map(([argumentName, messages]) => messages.map(helpMessage => ({
        argumentName,
        helpMessage,
        reason: 'constraint'
      }))).flat());
    }
    return new ScalewayError(status, obj);
  }
}

/**
 * ResourceNotFound error happens when getting a resource that does not exist anymore.
 *
 * @public
 */
class ResourceNotFoundError extends ScalewayError {
  constructor(status, body, resource, resourceId) {
    super(status, body, `resource ${resource} with ID ${resourceId} is not found`);
    this.status = status;
    this.body = body;
    this.resource = resource;
    this.resourceId = resourceId;
    this.name = 'ResourceNotFoundError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.resource !== 'string' || typeof obj.resource_id !== 'string') {
      return null;
    }
    return new ResourceNotFoundError(status, obj, obj.resource, obj.resource_id);
  }
}

/**
 * UnknownResource error is only returned by the instance API.
 *
 * @public
 */
class UnknownResourceMapper {
  static fromJSON(status, obj) {
    // Split the message
    // Note: some errors uses ' and not "
    // Examples: `"111..." not found` or `Security Group '111...' not found`
    const messageParts = typeof obj.message === 'string' ? obj.message.split(/"|'/) : [];
    if (messageParts.length === 3 && isUUID(messageParts[1])) {
      return new ResourceNotFoundError(status, obj,
      // transform `Security group ` to `security_group`
      // `.replaceAll()` may be too recent to use yet.
      // that's why we're using `.split(' ').join('_')` for now.
      messageParts[0].trim().toLowerCase().split(' ').join('_'), messageParts[1]);
    }
    return new ScalewayError(status, obj);
  }
}

/**
 * AlreadyExists error is used when a resource already exists.
 *
 * @public
 */
class AlreadyExistsError extends ScalewayError {
  constructor(status, body, resource, resourceId, helpMessage) {
    super(status, body, `resource ${resource} with ID ${resourceId} already exists: ${helpMessage}`);
    this.status = status;
    this.body = body;
    this.resource = resource;
    this.resourceId = resourceId;
    this.helpMessage = helpMessage;
    this.name = 'AlreadyExistsError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.resource !== 'string' || typeof obj.resource_id !== 'string' || typeof obj.help_message !== 'string') {
      return null;
    }
    return new AlreadyExistsError(status, obj, obj.resource, obj.resource_id, obj.help_message);
  }
}

/**
 * Build the default message for {@link DeniedAuthenticationError}.
 *
 * @param method - The authentication method
 * @param reason - The deny reason
 * @returns The error message
 *
 * @internal
 */
const buildMessage$3 = (method, reason) => {
  let reasonDesc;
  switch (reason) {
    case 'invalid_argument':
      reasonDesc = `invalid ${method} format or empty value`;
      break;
    case 'not_found':
      reasonDesc = `${method} does not exist`;
      break;
    case 'expired':
      reasonDesc = `${method} is expired`;
      break;
    default:
      reasonDesc = `unknown reason for ${method}`;
  }
  return `denied authentication: ${reasonDesc}`;
};

/**
 * DeniedAuthentication error is used by the API Gateway auth service to deny a request.
 *
 * @public
 */
class DeniedAuthenticationError extends ScalewayError {
  constructor(status, body, method, reason) {
    super(status, body, buildMessage$3(method, reason));
    this.status = status;
    this.body = body;
    this.method = method;
    this.reason = reason;
    this.name = 'DeniedAuthenticationError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.method !== 'string' || typeof obj.reason !== 'string') {
      return null;
    }
    return new DeniedAuthenticationError(status, obj, obj.method, obj.reason);
  }
}

/**
 * OutOfStock error happens when stocks are empty for the resource.
 *
 * @public
 */
class OutOfStockError extends ScalewayError {
  constructor(status, body, resource) {
    super(status, body, `resource ${resource} is out of stock`);
    this.status = status;
    this.body = body;
    this.resource = resource;
    this.name = 'OutOfStockError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.resource !== 'string') return null;
    return new OutOfStockError(status, obj, obj.resource);
  }
}

/**
 * Details of an {@link PermissionsDeniedError} error.
 *
 * @public
 */

/**
 * Build the default message for {@link PermissionsDeniedError}.
 *
 * @param list - The list of {@link PermissionsDeniedErrorDetails}
 * @returns The error message
 *
 * @internal
 */
const buildMessage$2 = list => `insufficient permissions: ${list.map(({
  action,
  resource
}) => `${action} ${resource}`).join('; ')}`;

/**
 * PermissionsDenied error happens when one or many permissions are not accorded to the user making the request.
 *
 * @public
 */
class PermissionsDeniedError extends ScalewayError {
  constructor(status, body, list) {
    super(status, body, buildMessage$2(list));
    this.status = status;
    this.body = body;
    this.list = list;
    this.name = 'PermissionsDeniedError';
  }
  static fromJSON(status, obj) {
    if (!Array.isArray(obj.details)) return null;
    return new PermissionsDeniedError(status, obj, obj.details.reduce((list, detail) => isJSONObject(detail) && typeof detail.resource === 'string' && typeof detail.action === 'string' ? list.concat({
      action: detail.action,
      resource: detail.resource
    }) : list, []));
  }
}

/**
 * Build the default message for {@link PreconditionFailedError}.
 *
 * @param precondition - The precondition
 * @param helpMessage - The message which should help the user to fix the root cause
 * @returns The error message
 *
 * @internal
 */
const buildMessage$1 = (precondition, helpMessage) => {
  let message = `precondition failed: ${precondition}`;
  if (typeof helpMessage === 'string' && helpMessage.length > 0) {
    message = message.concat(', ', helpMessage);
  }
  return message;
};

/**
 * PreconditionFailed error is used when a precondition is not respected.
 *
 * @public
 */
class PreconditionFailedError extends ScalewayError {
  constructor(status, body, precondition, helpMessage) {
    super(status, body, buildMessage$1(precondition, helpMessage));
    this.status = status;
    this.body = body;
    this.precondition = precondition;
    this.helpMessage = helpMessage;
    this.name = 'PreconditionFailedError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.precondition !== 'string' || typeof obj.help_message !== 'string') {
      return null;
    }
    return new PreconditionFailedError(status, obj, obj.precondition, obj.help_message);
  }
}

/**
 * ResourceExpired error happens when trying to access a resource that has expired.
 *
 * @public
 */
class ResourceExpiredError extends ScalewayError {
  constructor(status, body, resource, resourceId, expiredSince) {
    super(status, body, `resource ${resource} with ID ${resourceId} expired since ${expiredSince.toISOString()}`);
    this.status = status;
    this.body = body;
    this.resource = resource;
    this.resourceId = resourceId;
    this.expiredSince = expiredSince;
    this.name = 'ResourceExpiredError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.resource !== 'string' || typeof obj.resource_id !== 'string' || typeof obj.expired_since !== 'string') {
      return null;
    }
    return new ResourceExpiredError(status, obj, obj.resource, obj.resource_id, new Date(obj.expired_since));
  }
}

/**
 * ResourceLocked error happens when a resource is locked by trust and safety.
 *
 * @public
 */
class ResourceLockedError extends ScalewayError {
  constructor(status, body, resource, resourceId) {
    super(status, body, `resource ${resource} with ID ${resourceId} is locked`);
    this.status = status;
    this.body = body;
    this.resource = resource;
    this.resourceId = resourceId;
    this.name = 'ResourceLockedError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.resource !== 'string' || typeof obj.resource_id !== 'string') {
      return null;
    }
    return new ResourceLockedError(status, obj, obj.resource, obj.resource_id);
  }
}

/**
 * Quota policy of {@link TooManyRequestsError}.
 *
 * @public
 */

/**
 * Build the default message for {@link TooManyRequestsError}.
 *
 * @internal
 */
const buildMessage = (helpMessage, limit, resetSeconds, resetAt) => {
  const details = [];
  if (limit) {
    if (limit.windowSeconds) {
      details.push(`quota is ${limit.quota} for ${limit.windowSeconds}s`);
    } else {
      details.push(`quota is ${limit.quota}`);
    }
  }
  if (resetSeconds) {
    details.push(`resets in ${resetSeconds}s`);
  } else if (resetAt) {
    details.push(`resets at ${resetAt.toISOString()}`);
  }
  let output = `too many requests`;
  if (details.length > 0) {
    output += ` (${details.join(', ')})`;
  }
  if (helpMessage.length > 0) {
    output += `: ${helpMessage}`;
  }
  return output;
};

/**
 * TooManyRequestsError error happens when fetching too many times a resource.
 *
 * @public
 */
class TooManyRequestsError extends ScalewayError {
  constructor(status, body, helpMessage, limit, /** The number of seconds until the quota resets */
  resetSeconds, /** The timestamp when the quota resets */
  resetAt) {
    super(status, body, buildMessage(helpMessage, limit, resetSeconds, resetAt));
    this.status = status;
    this.body = body;
    this.helpMessage = helpMessage;
    this.limit = limit;
    this.resetSeconds = resetSeconds;
    this.resetAt = resetAt;
    this.name = 'TooManyRequestsError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.help_message !== 'string') return null;
    let limit;
    if (isJSONObject(obj.limit) && typeof obj.limit.quota === 'number') {
      limit = {
        quota: obj.limit.quota,
        windowSeconds: typeof obj.limit.window_seconds === 'number' ? obj.limit.window_seconds : undefined
      };
    }
    return new TooManyRequestsError(status, obj, obj.help_message, limit, typeof obj.reset_seconds === 'number' ? obj.reset_seconds : undefined, typeof obj.reset_at === 'string' ? new Date(obj.reset_at) : undefined);
  }
}

/**
 * TransientState error happens when trying to perform an action on a resource in a transient state.
 *
 * @public
 */
class TransientStateError extends ScalewayError {
  constructor(status, body, resource, resourceId, currentState) {
    super(status, body, `resource ${resource} with ID ${resourceId} is in a transient state: ${currentState}`);
    this.status = status;
    this.body = body;
    this.resource = resource;
    this.resourceId = resourceId;
    this.currentState = currentState;
    this.name = 'TransientStateError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.resource !== 'string' || typeof obj.resource_id !== 'string' || typeof obj.current_state !== 'string') {
      return null;
    }
    return new TransientStateError(status, obj, obj.resource, obj.resource_id, obj.current_state);
  }
}

/**
 * Unmarshals a standard error from raw body.
 *
 * @param type - The error type
 * @param status - The status code
 * @param body - The error response
 * @returns The standard error if found
 *
 * @internal
 */
const unmarshalStandardError = (type, status, body) => {
  let error;
  switch (type) {
    case 'denied_authentication':
      error = DeniedAuthenticationError;
      break;
    case 'invalid_arguments':
      error = InvalidArgumentsError;
      break;
    case 'out_of_stock':
      error = OutOfStockError;
      break;
    case 'permissions_denied':
      error = PermissionsDeniedError;
      break;
    case 'precondition_failed':
      error = PreconditionFailedError;
      break;
    case 'quotas_exceeded':
      error = QuotasExceededError;
      break;
    case 'expired':
      error = ResourceExpiredError;
      break;
    case 'not_found':
      error = ResourceNotFoundError;
      break;
    case 'locked':
      error = ResourceLockedError;
      break;
    case 'transient_state':
      error = TransientStateError;
      break;
    case 'already_exists':
      error = AlreadyExistsError;
      break;
    case 'too_many_requests':
      error = TooManyRequestsError;
      break;
    default:
      return null;
  }
  return error.fromJSON(status, body);
};

/**
 * Unmarshals a non-standard error from raw body.
 *
 * @param type - The error type
 * @param status - The status code
 * @param body - The error response
 * @returns The non-standard error if found
 *
 * @internal
 */
const unmarshalNonStandardError = (type, status, body) => {
  switch (type) {
    case 'unknown_resource':
      return UnknownResourceMapper.fromJSON(status, body);
    case 'invalid_request_error':
      return InvalidRequestMapper.fromJSON(status, body);
    default:
      return null;
  }
};

/**
 * Parses Scaleway error from raw body.
 *
 * @param status - The status code
 * @param body - The error response
 * @returns The resolved error
 *
 * @internal
 */
const parseScalewayError = (status, body) => {
  const parsableError = typeof body.type === 'string' && (unmarshalStandardError(body.type, status, body) ?? unmarshalNonStandardError(body.type, status, body));
  return parsableError || new ScalewayError(status, body);
};

const X_TOTAL_COUNT_HEADER_KEY = 'x-total-count';
const TOTAL_COUNT_RES_KEY = 'total_count';

/**
 * Fixes the totalCount property for old APIs.
 *
 * @internal
 */
const fixLegacyTotalCount = (obj, headers) => {
  const headerVal = headers.get(X_TOTAL_COUNT_HEADER_KEY);
  if (!headerVal) {
    return obj;
  }
  const totalCount = parseInt(headerVal, 10);
  if (Number.isNaN(totalCount)) {
    return obj;
  }
  if (isJSONObject(obj) && !(TOTAL_COUNT_RES_KEY in obj)) {
    return Object.assign(obj, {
      [TOTAL_COUNT_RES_KEY]: totalCount
    });
  }
  return obj;
};

/**
 * Makes response parser.
 *
 * @param unmarshaller - The response payload unmarshaller
 * @returns An async converter of HTTP Response to desired result
 *
 * @throws {@link ScalewayError}
 * Thrown by the API if the request couldn't be completed.
 *
 * @throws TypeError
 * Thrown if the response parameter isn't of the expected type.
 *
 * @throws Error
 * JSON parsing could trigger an error.
 *
 * @internal
 */
const responseParser = (unmarshaller, responseType) => async response => {
  if (!isResponse(response)) {
    throw new TypeError('Invalid response object');
  }
  if (response.ok) {
    if (response.status === 204) return unmarshaller(undefined);
    const contentType = response.headers.get('Content-Type');
    try {
      if (responseType === 'json' && contentType === 'application/json') {
        return unmarshaller(fixLegacyTotalCount(await response.json(), response.headers));
      }
      if (responseType === 'blob') {
        return unmarshaller(await response.blob());
      }
      return unmarshaller(await response.text());
    } catch (err) {
      throw new ScalewayError(response.status, `could not parse '${contentType ?? ''}' response${err instanceof Error ? `: ${err.message}` : ''}`);
    }
  }
  const error = await response.clone().json().catch(() => response.text());
  if (isJSONObject(error)) throw parseScalewayError(response.status, error);
  throw new ScalewayError(response.status, typeof error === 'string' ? error : 'cannot read error response body');
};

/**
 * Builds Request from {@link ScwRequest} & {@link Settings}.
 *
 * @param request - A scaleway request
 * @param settings - The settings
 * @returns A fetch Request
 *
 * @internal
 */
const buildRequest = (request, settings) => {
  let {
    path
  } = request;
  if (request.urlParams instanceof URLSearchParams) {
    path = path.concat(`?${request.urlParams.toString()}`);
  }
  return new Request(`${settings.apiURL}${path}`, {
    body: request.body,
    headers: {
      Accept: 'application/json',
      ...( /* istanbul ignore next */!isBrowser() ? {
        'User-Agent': settings.userAgent
      } : {}),
      ...request.headers
    },
    method: request.method
  });
};
const asIs = response => response;
/**
 * Builds a resource fetcher.
 *
 * @param settings - The {@link Settings} object
 * @param httpClient - The HTTP client that should be used to call the API
 * @returns The fetcher
 *
 * @internal
 */
const buildFetcher = (settings, httpClient) => {
  let requestNumber = 0;
  const prepareRequest = requestId => composeRequestInterceptors([...settings.interceptors.map(obj => obj.request).filter(obj => obj), logRequest(requestId, obfuscateInterceptor(obfuscateAuthHeadersEntry))]);
  const prepareResponse = requestId => composeResponseInterceptors([...settings.interceptors.map(obj => obj.response).filter(obj => obj), logResponse(requestId)]);
  const prepareResponseErrors = () => composeResponseErrorInterceptors(settings.interceptors.map(obj => obj.responseError).filter(obj => obj));
  return async (request, unwrapper = asIs) => {
    const requestId = `${requestNumber += 1}`;
    const reqInterceptors = prepareRequest(requestId);
    const finalRequest = await reqInterceptors(buildRequest(request, settings));
    try {
      const response = await httpClient(finalRequest);
      const resInterceptors = prepareResponse(requestId);
      const finalResponse = await resInterceptors(response);
      const resUnmarshaller = responseParser(unwrapper, request.responseType ?? 'json');
      const unmarshaledResponse = await resUnmarshaller(finalResponse);
      return unmarshaledResponse;
    } catch (err) {
      const resErrorInterceptors = prepareResponseErrors();
      const handledError = await resErrorInterceptors(finalRequest, err);
      return unwrapper(handledError);
    }
  };
};

/** Default {@link Settings} values. */
const DEFAULT_SETTINGS = {
  apiURL: 'https://api.scaleway.com',
  httpClient: fetch,
  interceptors: [],
  userAgent
};

/**
 * Scaleway client.
 */

/**
 * Creates a Scaleway client with advanced options.
 * You can either use existing factories
 * (like `withProfile`, `withUserAgentSuffix`, etc)
 * or write your own using the interface `ClientConfig`.
 *
 * @example
 * Creates a client with factories:
 * ```
 * createAdvancedClient(
 *   (obj: Settings) => ({
 *     ...obj,
 *     defaultPageSize: 100 ,
 *     httpClient: myFetchWrapper,
 *   }),
 *   withUserAgentSuffix('bot-name/1.0'),
 * )
 * ```
 *
 * @throws Error
 * Thrown if the setup fails.
 *
 * @public
 */
const createAdvancedClient = (...configs) => {
  const settings = configs.concat([withLegacyInterceptors()]).reduce((currentSettings, config) => config(currentSettings), DEFAULT_SETTINGS);
  assertValidSettings(settings);
  getLogger().info(`init Scaleway SDK version ${version}`);
  return {
    fetch: buildFetcher(settings, settings.httpClient),
    settings
  };
};

/**
 * Creates a Scaleway client with a profile.
 *
 * @example
 * Creates a client with credentials & default values (see https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/):
 * ```
 * import { createClient } from '@scaleway/sdk'
 *
 * createClient({
 *   accessKey: 'SCWXXXXXXXXXXXXXXXXX',
 *   secretKey: 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx',
 *   defaultProjectId: 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx',
 *   defaultRegion: 'fr-par',
 *   defaultZone: 'fr-par-1',
 * })
 * ```
 *
 * @example
 * Creates a client by loading values from the environment (see https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/)
 * or the config file created by CLI `scw init` (see https://www.scaleway.com/en/cli/):
 * ```
 * import { loadProfileFromConfigurationFile } from '@scaleway/configuration-loader'
 * import { createClient } from '@scaleway/sdk'
 *
 * createClient({
 *   ...await loadProfileFromConfigurationFile(),
 *   defaultZone: 'fr-par-3',
 * })
 * ```
 *
 * @throws Error
 * Thrown if the setup fails.
 *
 * @public
 */
const createClient = (profile = {}) => createAdvancedClient(withProfile(profile));

var index$B = /*#__PURE__*/Object.freeze({
  __proto__: null,
  AlreadyExistsError: AlreadyExistsError,
  DeniedAuthenticationError: DeniedAuthenticationError,
  InvalidArgumentsError: InvalidArgumentsError,
  OutOfStockError: OutOfStockError,
  PermissionsDeniedError: PermissionsDeniedError,
  PreconditionFailedError: PreconditionFailedError,
  QuotasExceededError: QuotasExceededError,
  ResourceExpiredError: ResourceExpiredError,
  ResourceLockedError: ResourceLockedError,
  ResourceNotFoundError: ResourceNotFoundError,
  ScalewayError: ScalewayError,
  TooManyRequestsError: TooManyRequestsError,
  TransientStateError: TransientStateError
});

/**
 * Sleep for a specified number of time.
 *
 * @param ms - The number of milliseconds
 * @returns The sleep promise
 *
 * @internal
 */
const sleep = ms => new Promise(resolve => {
  setTimeout(resolve, ms);
});

const DEFAULT_TIMEOUT_SECONDS = 300;
const DEFAULT_MIN_DELAY_SECONDS = 1;
const DEFAULT_MAX_DELAY_SECONDS = 30;

/**
 * Creates an exponential backoff interval strategy.
 *
 * @param minDelay - The minimum delay before the next try in seconds
 * @param maxDelay - The maximum delay before the next try in seconds
 * @returns An exponential backoff generator
 *
 * @internal
 */
function* createExponentialBackoffStrategy(minDelay, maxDelay) {
  if (minDelay < 1 || maxDelay < 1 || minDelay > maxDelay) {
    throw new Error('Waiter: minDelay must be >= 1 and maxDelay must be >= minDelay');
  }
  let attempt = 1;
  const ceiling = Math.log(maxDelay / minDelay) / Math.log(2) + 1;
  const randomInRange = (min, max) => min + Math.random() * (max - min);
  // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
  while (true) {
    if (attempt > ceiling) {
      yield maxDelay;
    } else {
      yield randomInRange(minDelay, minDelay * 2 ** (attempt - 1));
    }
    attempt += 1;
  }
}

/**
 * Tries a specific logic several times until it succeeds, timeouts, or throws an exception.
 *
 * @param retry - The function to retry logic between each interval
 * @param strategy - A generated interval strategy iterator
 * @param timeout - The maximum time elapsed before timeout error
 *
 * @throws An timeout exception or error thrown by the logic being run
 *
 * @internal
 */
const tryAtIntervals = async (retry, strategy, timeout = DEFAULT_TIMEOUT_SECONDS) => {
  const timeoutTimestamp = Date.now() + timeout * 1000;
  let retryCount = 0;
  while (Date.now() <= timeoutTimestamp) {
    const delay = strategy.next(retryCount += 1).value * 1000;
    // Break if timeout has been reached
    if (timeoutTimestamp <= Date.now() + delay) break;
    // Wait before the next retry
    await sleep(delay);
    // Retry
    const {
      value,
      done
    } = await retry();
    if (done) return value;
  }
  throw new Error(`Timeout after ${timeout}s`);
};

/**
 * Represents the condition to stop waiting for a resource.
 *
 * @public
 */

/**
 * The options to wait until a resource is ready.
 *
 * @public
 */

/**
 * Fetches resource several times until an expected condition is reached, timeouts, or throws an exception.
 *
 * @param stop - The condition to stop waiting
 * @param fetcher - The method to retrieve resource
 * @param request - The resource request options
 * @param options - The retry strategy options
 * @param strategy - An optional custom strategy
 *
 * @returns A promise of resource
 *
 * @public
 */
const waitForResource = (stop, fetcher, request, options, strategy = createExponentialBackoffStrategy(options?.minDelay ?? DEFAULT_MIN_DELAY_SECONDS, options?.maxDelay ?? DEFAULT_MAX_DELAY_SECONDS)) => tryAtIntervals(async () => {
  const value = await fetcher(request);
  return {
    done: await stop(value),
    value
  };
}, strategy, options?.timeout);

/**
 * Abstract class to instantiate API from a {@link Client}.
 *
 * @internal
 */
let API$z = class API {
  constructor(client) {
    this.client = client;
  }
};

/**
 * Returns the parameter if it's valid as path parameter
 * (string and not empty, or number), else throws an exception.
 *
 * @param name - The parameter name
 * @param param - The parameter value
 * @returns The parameter value
 *
 * @throws TypeError
 * Thrown if the parameter is invalid.
 *
 * @internal
 */
function validatePathParam(name, param) {
  if (typeof param === 'string' && param.length > 0) return param;
  if (typeof param === 'number') return param.toString();
  throw new TypeError(`param ${name} cannot be empty in request`);
}

/**
 * Resolves the ideal parameter and value amongst an optional list.
 *
 * @param list - The list to be looking into
 * @param isRequired - If at least one "one-of" should be found, false by default
 * @returns The parameter and value
 *
 * @throws TypeError
 * Thrown if isRequired is true, and no value or default value is specified.
 *
 * @internal
 */
const resolveOneOf = (list, isRequired = false) => {
  const elt = list.find(obj => obj.value !== undefined) ?? list.find(obj => obj.default !== undefined);
  const value = elt?.value ?? elt?.default;
  if (elt && value !== undefined) {
    return {
      [elt.param]: value
    };
  }
  if (isRequired) {
    const keyList = list.map(obj => obj.param).join(' or ');
    throw new TypeError(`one of ${keyList} must be indicated in the request`);
  }
  return {};
};
/**
 * Filters defined parameters tuples and converts them to URLSearchParams.
 *
 * @param paramTuples - The key/value pairs
 * @returns URLSearchParams
 *
 * @internal
 */
const urlParams = (...paramTuples) => {
  const params = new URLSearchParams();
  for (const [key, value] of paramTuples) {
    if (typeof key === 'string' && value != null) {
      if (Array.isArray(value)) {
        for (const innerValue of value) {
          if (innerValue !== null) {
            params.append(key, innerValue instanceof Date ? innerValue.toISOString() : innerValue.toString());
          }
        }
      } else {
        params.append(key, value instanceof Date ? value.toISOString() : value.toString());
      }
    }
  }
  return params;
};

/**
 * Unmarshals data to Date object.
 *
 * @internal
 */
const unmarshalDate = data => {
  if (typeof data !== 'string') {
    return undefined;
  }
  const date = new Date(data);
  if (Number.isNaN(date.getTime())) {
    return undefined;
  }
  return date;
};

/**
 * Unmarshals array of object.
 *
 * @internal
 */
const unmarshalArrayOfObject = (data, unmarshaller, emptyFallback = true) => {
  if (!Array.isArray(data)) {
    return emptyFallback ? [] : undefined;
  }
  return data.map(elt => unmarshaller(elt));
};

/**
 * Unmarshals map of object.
 *
 * @internal
 */
const unmarshalMapOfObject = (data, unmarshaller, emptyFallback = true) => {
  if (!data || typeof data !== 'object' || !(data instanceof Object) || Array.isArray(data)) {
    return emptyFallback ? {} : undefined;
  }
  return Object.entries(data).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: unmarshaller(value)
  }), {});
};

/**
 * Unmarshals {@link Money}
 *
 * @internal
 */
const unmarshalMoney = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Money' failed as data isn't a dictionary.`);
  }
  return {
    currencyCode: data.currency_code,
    nanos: data.nanos,
    units: data.units
  };
};

/**
 * Unmarshals {@link ServiceInfo}.
 *
 * @internal
 */
const unmarshalServiceInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServiceInfo' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    documentationUrl: data.documentation_url,
    name: data.name,
    version: data.version
  };
};

/**
 * Unmarshals {@link ScwFile}.
 *
 * @internal
 */
const unmarshalScwFile = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ScwFile' failed as data isn't a dictionary.`);
  }
  return {
    content: data.content,
    contentType: data.content_type,
    name: data.name
  };
};

/**
 * Unmarshals {@link TimeSeriesPoint}
 *
 * @remarks To optimize the size of this message,
 * the JSON is compressed in an array instead of a dictionary.
 * Example: `["2019-08-08T15:00:00Z", 0.2]`.
 *
 * @internal
 */
const unmarshalTimeSeriesPoint = data => {
  if (!Array.isArray(data)) {
    throw new TypeError(`Unmarshalling the type 'TimeSeriesPoint' failed as data isn't an array.`);
  }
  return {
    timestamp: unmarshalDate(data[0]),
    value: data[1]
  };
};

/**
 * Unmarshals {@link TimeSeries}
 *
 * @internal
 */
const unmarshalTimeSeries = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TimeSeries' failed as data isn't a dictionary.`);
  }
  return {
    metadata: data.metadata,
    name: data.name,
    points: unmarshalArrayOfObject(data.points, unmarshalTimeSeriesPoint)
  };
};

/**
 * Marshals {@link ScwFile}.
 *
 * @internal
 */
const marshalScwFile = obj => ({
  content: obj.content,
  content_type: obj.contentType,
  name: obj.name
});

/**
 * Marshals {@link Money}
 *
 * @internal
 */
const marshalMoney = obj => ({
  currency_code: obj.currencyCode,
  nanos: obj.nanos,
  units: obj.units
});

/**
 * Marshals {@link TimeSeriesPoint}
 *
 * @internal
 */
const marshalTimeSeriesPoint = obj => ({
  timestamp: obj.timestamp?.toISOString(),
  value: obj.value
});

/**
 * Marshals {@link TimeSeries}
 *
 * @internal
 */
const marshalTimeSeries = obj => ({
  metadata: obj.metadata,
  name: obj.name,
  points: obj.points.map(elt => marshalTimeSeriesPoint(elt))
});

const extract = key => result => result[key];
function* pages(key, fetcher, request, firstPage) {
  if (!Array.isArray(firstPage[key])) {
    throw new Error(`Property ${key} is not a list in paginated result`);
  }
  const getList = extract(key);
  let page = request.page || 1;
  if (page === 1) {
    yield Promise.resolve(getList(firstPage));
    page += 1;
  }
  const {
    length
  } = firstPage[key];
  if (!length) return;
  const {
    totalCount
  } = firstPage;
  while (page <= Math.floor((totalCount + length - 1) / length)) {
    yield fetcher({
      ...request,
      page
    }).then(getList);
    page += 1;
  }
}

/**
 * Fetches a paginated resource.
 *
 * @param key - The resource key of values list
 * @param fetcher - The method to retrieve paginated resources
 * @param request - A request with pagination options
 * @param initial - The first page
 * @returns An async generator of resources arrays
 */
async function* fetchPaginated(key, fetcher, request, initial = fetcher(request)) {
  yield* pages(key, fetcher, request, await initial);
}

/**
 * Fetches all paginated resource.
 *
 * @param key - The resource key of values list
 * @param fetcher - The method to retrieve paginated resources
 * @param request - A request with pagination options
 * @param initial - The first page
 * @returns A resources array Promise
 */
const fetchAll = async (key, fetcher, request, initial = fetcher(request)) => (await Promise.all(Array.from(pages(key, fetcher, request, await initial)))).flat();

/**
 * Enriches a listing method with helpers.
 *
 * @param key - The resource key of values list
 * @param fetcher - The method to retrieve paginated resources
 * @param request - A request with pagination options
 * @returns A resource Promise with the pagination helpers
 *
 * @internal
 */
const enrichForPagination = (key, fetcher, request) => {
  const firstPage = fetcher(request);
  return Object.assign(firstPage, {
    all: () => fetchAll(key, fetcher, request, firstPage),
    [Symbol.asyncIterator]: () => fetchPaginated(key, fetcher, request, firstPage)
  });
};

const ADJECTIVES = ['admiring', 'adoring', 'affectionate', 'agitated', 'amazing', 'angry', 'awesome', 'beautiful', 'blissful', 'bold', 'boring', 'brave', 'busy', 'charming', 'clever', 'cool', 'compassionate', 'competent', 'condescending', 'confident', 'cranky', 'crazy', 'dazzling', 'determined', 'distracted', 'dreamy', 'eager', 'ecstatic', 'elastic', 'elated', 'elegant', 'eloquent', 'epic', 'exciting', 'fervent', 'festive', 'flamboyant', 'focused', 'friendly', 'frosty', 'funny', 'gallant', 'gifted', 'goofy', 'gracious', 'great', 'happy', 'hardcore', 'heuristic', 'hopeful', 'hungry', 'infallible', 'inspiring', 'interesting', 'intelligent', 'jolly', 'jovial', 'keen', 'kind', 'laughing', 'loving', 'lucid', 'magical', 'mystifying', 'modest', 'musing', 'naughty', 'nervous', 'nice', 'nifty', 'nostalgic', 'objective', 'optimistic', 'peaceful', 'pedantic', 'pensive', 'practical', 'priceless', 'quirky', 'quizzical', 'recursing', 'relaxed', 'reverent', 'romantic', 'sad', 'serene', 'sharp', 'silly', 'sleepy', 'stoic', 'strange', 'stupefied', 'suspicious', 'sweet', 'tender', 'thirsty', 'trusting', 'unruffled', 'upbeat', 'vibrant', 'vigilant', 'vigorous', 'wizardly', 'wonderful', 'xenodochial', 'youthful', 'zealous', 'zen'];
const NAMES = ['albattani', 'allen', 'almeida', 'antonelli', 'agnesi', 'archimedes', 'ardinghelli', 'aryabhata', 'austin', 'babbage', 'banach', 'banzai', 'bardeen', 'bartik', 'bassi', 'beaver', 'bell', 'benz', 'bhabha', 'bhaskara', 'black', 'blackburn', 'blackwell', 'bohr', 'booth', 'borg', 'bose', 'bouman', 'boyd', 'brahmagupta', 'brattain', 'brown', 'buck', 'burnell', 'cannon', 'carson', 'cartwright', 'carver', 'cerf', 'chandrasekhar', 'chaplygin', 'chatelet', 'chatterjee', 'chebyshev', 'cohen', 'chaum', 'clarke', 'colden', 'cori', 'cray', 'curran', 'curie', 'darwin', 'davinci', 'dewdney', 'dhawan', 'diffie', 'dijkstra', 'dirac', 'driscoll', 'dubinsky', 'easley', 'edison', 'einstein', 'elbakyan', 'elgamal', 'elion', 'ellis', 'engelbart', 'euclid', 'euler', 'faraday', 'feistel', 'fermat', 'fermi', 'feynman', 'franklin', 'gagarin', 'galileo', 'galois', 'ganguly', 'gates', 'gauss', 'germain', 'goldberg', 'goldstine', 'goldwasser', 'golick', 'goodall', 'gould', 'greider', 'grothendieck', 'haibt', 'hamilton', 'haslett', 'hawking', 'hellman', 'heisenberg', 'hermann', 'herschel', 'hertz', 'heyrovsky', 'hodgkin', 'hofstadter', 'hoover', 'hopper', 'hugle', 'hypatia', 'ishizaka', 'jackson', 'jang', 'jemison', 'jennings', 'jepsen', 'johnson', 'joliot', 'jones', 'kalam', 'kapitsa', 'kare', 'keldysh', 'keller', 'kepler', 'khayyam', 'khorana', 'kilby', 'kirch', 'knuth', 'kowalevski', 'lalande', 'lamarr', 'lamport', 'leakey', 'leavitt', 'lederberg', 'lehmann', 'lewin', 'lichterman', 'liskov', 'lovelace', 'lumiere', 'mahavira', 'margulis', 'matsumoto', 'maxwell', 'mayer', 'mccarthy', 'mcclintock', 'mclaren', 'mclean', 'mcnulty', 'mendel', 'mendeleev', 'meitner', 'meninsky', 'merkle', 'mestorf', 'mirzakhani', 'montalcini', 'moore', 'morse', 'murdock', 'moser', 'napier', 'nash', 'neumann', 'newton', 'niel', 'nightingale', 'nobel', 'noether', 'northcutt', 'noyce', 'panini', 'pare', 'pascal', 'pasteur', 'payne', 'perlman', 'pike', 'poincare', 'poitras', 'proskuriakova', 'ptolemy', 'raman', 'ramanujan', 'ride', 'ritchie', 'rhodes', 'robinson', 'roentgen', 'rosalind', 'rubin', 'saha', 'sammet', 'sanderson', 'satoshi', 'shamir', 'shannon', 'shaw', 'shirley', 'shockley', 'shtern', 'sinoussi', 'snyder', 'solomon', 'spence', 'stonebraker', 'sutherland', 'swanson', 'swartz', 'swirles', 'taussig', 'tereshkova', 'tesla', 'tharp', 'thompson', 'torvalds', 'tu', 'turing', 'varahamihira', 'vaughan', 'visvesvaraya', 'volhard', 'villani', 'wescoff', 'wilbur', 'wiles', 'williams', 'williamson', 'wilson', 'wing', 'wozniak', 'wright', 'wu', 'yalow', 'yonath', 'zhukovsky'];
const randomName = (prefix = '', separator = '-') => {
  const name = `${ADJECTIVES[Math.floor(Math.random() * ADJECTIVES.length)] ?? ''}${separator}${NAMES[Math.floor(Math.random() * NAMES.length)] ?? ''}`;
  if (name === `boring${separator}wozniak`) {
    return randomName(prefix, separator);
  }
  return prefix.length > 0 ? `${prefix}${separator}${name}` : name;
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalProject$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Project' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListProjectsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListProjectsResponse' failed as data isn't a dictionary.`);
  }
  return {
    projects: unmarshalArrayOfObject(data.projects, unmarshalProject$1),
    totalCount: data.total_count
  };
};
const marshalCreateProjectRequest = (request, defaults) => ({
  description: request.description,
  name: request.name || randomName('proj'),
  organization_id: request.organizationId ?? defaults.defaultOrganizationId
});
const marshalUpdateProjectRequest = (request, defaults) => ({
  description: request.description,
  name: request.name
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$y = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Account API.
 *
 * User related data. This API allows you to manage projects.
 */
let API$y = class API extends API$z {
  /**
   * Create a new Project for an Organization. Deprecated in favor of Account
   * API v3. Generate a new Project for an Organization, specifying its
   * configuration including name and description.
   *
   * @deprecated
   * @param request - The request {@link CreateProjectRequest}
   * @returns A Promise of Project
   */
  createProject = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateProjectRequest(request, this.client.settings)),
    headers: jsonContentHeaders$y,
    method: 'POST',
    path: `/account/v2/projects`
  }, unmarshalProject$1);
  pageOfListProjects = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/account/v2/projects`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_ids', request.projectIds])
  }, unmarshalListProjectsResponse$1);

  /**
   * List all Projects of an Organization. Deprecated in favor of Account API
   * v3. List all Projects of an Organization. The response will include the
   * total number of Projects as well as their associated Organizations, names
   * and IDs. Other information include the creation and update date of the
   * Project.
   *
   * @deprecated
   * @param request - The request {@link ListProjectsRequest}
   * @returns A Promise of ListProjectsResponse
   */
  listProjects = (request = {}) => enrichForPagination('projects', this.pageOfListProjects, request);

  /**
   * Get an existing Project. Deprecated in favor of Account API v3. Retrieve
   * information about an existing Project, specified by its Project ID. Its
   * full details, including ID, name and description, are returned in the
   * response object.
   *
   * @deprecated
   * @param request - The request {@link GetProjectRequest}
   * @returns A Promise of Project
   */
  getProject = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/account/v2/projects/${validatePathParam('projectId', request.projectId ?? this.client.settings.defaultProjectId)}`
  }, unmarshalProject$1);

  /**
   * Delete an existing Project. Deprecated in favor of Account API v3. Delete
   * an existing Project, specified by its Project ID. The Project needs to be
   * empty (meaning there are no resources left in it) to be deleted
   * effectively. Note that deleting a Project is permanent, and cannot be
   * undone.
   *
   * @deprecated
   * @param request - The request {@link DeleteProjectRequest}
   */
  deleteProject = (request = {}) => this.client.fetch({
    method: 'DELETE',
    path: `/account/v2/projects/${validatePathParam('projectId', request.projectId ?? this.client.settings.defaultProjectId)}`
  });

  /**
   * Update Project. Deprecated in favor of Account API v3. Update the
   * parameters of an existing Project, specified by its Project ID. These
   * parameters include the name and description.
   *
   * @deprecated
   * @param request - The request {@link UpdateProjectRequest}
   * @returns A Promise of Project
   */
  updateProject = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalUpdateProjectRequest(request, this.client.settings)),
    headers: jsonContentHeaders$y,
    method: 'PATCH',
    path: `/account/v2/projects/${validatePathParam('projectId', request.projectId ?? this.client.settings.defaultProjectId)}`
  }, unmarshalProject$1);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$y = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$y
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalProject = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Project' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListProjectsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListProjectsResponse' failed as data isn't a dictionary.`);
  }
  return {
    projects: unmarshalArrayOfObject(data.projects, unmarshalProject),
    totalCount: data.total_count
  };
};
const marshalProjectApiCreateProjectRequest = (request, defaults) => ({
  description: request.description,
  name: request.name || randomName('proj'),
  organization_id: request.organizationId ?? defaults.defaultOrganizationId
});
const marshalProjectApiUpdateProjectRequest = (request, defaults) => ({
  description: request.description,
  name: request.name
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$x = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Account API.
 *
 * This API allows you to manage projects.
 */
class ProjectAPI extends API$z {
  /**
   * Create a new Project for an Organization. Generate a new Project for an
   * Organization, specifying its configuration including name and description.
   *
   * @param request - The request {@link ProjectApiCreateProjectRequest}
   * @returns A Promise of Project
   */
  createProject = request => this.client.fetch({
    body: JSON.stringify(marshalProjectApiCreateProjectRequest(request, this.client.settings)),
    headers: jsonContentHeaders$x,
    method: 'POST',
    path: `/account/v3/projects`
  }, unmarshalProject);
  pageOfListProjects = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/account/v3/projects`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_ids', request.projectIds])
  }, unmarshalListProjectsResponse);

  /**
   * List all Projects of an Organization. List all Projects of an Organization.
   * The response will include the total number of Projects as well as their
   * associated Organizations, names, and IDs. Other information includes the
   * creation and update date of the Project.
   *
   * @param request - The request {@link ProjectApiListProjectsRequest}
   * @returns A Promise of ListProjectsResponse
   */
  listProjects = (request = {}) => enrichForPagination('projects', this.pageOfListProjects, request);

  /**
   * Get an existing Project. Retrieve information about an existing Project,
   * specified by its Project ID. Its full details, including ID, name and
   * description, are returned in the response object.
   *
   * @param request - The request {@link ProjectApiGetProjectRequest}
   * @returns A Promise of Project
   */
  getProject = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/account/v3/projects/${validatePathParam('projectId', request.projectId ?? this.client.settings.defaultProjectId)}`
  }, unmarshalProject);

  /**
   * Delete an existing Project. Delete an existing Project, specified by its
   * Project ID. The Project needs to be empty (meaning there are no resources
   * left in it) to be deleted effectively. Note that deleting a Project is
   * permanent, and cannot be undone.
   *
   * @param request - The request {@link ProjectApiDeleteProjectRequest}
   */
  deleteProject = (request = {}) => this.client.fetch({
    method: 'DELETE',
    path: `/account/v3/projects/${validatePathParam('projectId', request.projectId ?? this.client.settings.defaultProjectId)}`
  });

  /**
   * Update Project. Update the parameters of an existing Project, specified by
   * its Project ID. These parameters include the name and description.
   *
   * @param request - The request {@link ProjectApiUpdateProjectRequest}
   * @returns A Promise of Project
   */
  updateProject = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalProjectApiUpdateProjectRequest(request, this.client.settings)),
    headers: jsonContentHeaders$x,
    method: 'PATCH',
    path: `/account/v3/projects/${validatePathParam('projectId', request.projectId ?? this.client.settings.defaultProjectId)}`
  }, unmarshalProject);
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const ProjectApiCreateProjectRequest = {
  description: {
    maxLength: 200
  },
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\._\- ]+$/
  }
};
const ProjectApiListProjectsRequest = {
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\._\- ]+$/
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ProjectApiUpdateProjectRequest = {
  description: {
    maxLength: 200
  },
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\._\- ]+$/
  }
};

var validationRules_gen$i = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ProjectApiCreateProjectRequest: ProjectApiCreateProjectRequest,
  ProjectApiListProjectsRequest: ProjectApiListProjectsRequest,
  ProjectApiUpdateProjectRequest: ProjectApiUpdateProjectRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$x = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ProjectAPI: ProjectAPI,
  ValidationRules: validationRules_gen$i
});

var index$A = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v2: index_gen$y,
  v3: index_gen$x
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link ServerStatus}. */
const SERVER_TRANSIENT_STATUSES$3 = ['starting', 'rebooting', 'updating', 'locking', 'unlocking', 'reinstalling'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalOS$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OS' failed as data isn't a dictionary.`);
  }
  return {
    compatibleServerTypes: data.compatible_server_types,
    family: data.family,
    id: data.id,
    imageUrl: data.image_url,
    isBeta: data.is_beta,
    label: data.label,
    name: data.name,
    version: data.version,
    xcodeVersion: data.xcode_version
  };
};
const unmarshalServerTypeCPU = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerTypeCPU' failed as data isn't a dictionary.`);
  }
  return {
    coreCount: data.core_count,
    name: data.name
  };
};
const unmarshalServerTypeDisk = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerTypeDisk' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    type: data.type
  };
};
const unmarshalServerTypeMemory = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerTypeMemory' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    type: data.type
  };
};
const unmarshalServerType$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerType' failed as data isn't a dictionary.`);
  }
  return {
    cpu: data.cpu ? unmarshalServerTypeCPU(data.cpu) : undefined,
    defaultOs: data.default_os ? unmarshalOS$2(data.default_os) : undefined,
    disk: data.disk ? unmarshalServerTypeDisk(data.disk) : undefined,
    memory: data.memory ? unmarshalServerTypeMemory(data.memory) : undefined,
    minimumLeaseDuration: data.minimum_lease_duration,
    name: data.name,
    stock: data.stock
  };
};
const unmarshalServer$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Server' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    deletableAt: unmarshalDate(data.deletable_at),
    id: data.id,
    ip: data.ip,
    name: data.name,
    organizationId: data.organization_id,
    os: data.os ? unmarshalOS$2(data.os) : undefined,
    projectId: data.project_id,
    status: data.status,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    vncUrl: data.vnc_url,
    zone: data.zone
  };
};
const unmarshalListOSResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListOSResponse' failed as data isn't a dictionary.`);
  }
  return {
    os: unmarshalArrayOfObject(data.os, unmarshalOS$2),
    totalCount: data.total_count
  };
};
const unmarshalListServerTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServerTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    serverTypes: unmarshalArrayOfObject(data.server_types, unmarshalServerType$1)
  };
};
const unmarshalListServersResponse$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalServer$3),
    totalCount: data.total_count
  };
};
const marshalCreateServerRequest$3 = (request, defaults) => ({
  name: request.name || randomName('as'),
  os_id: request.osId,
  project_id: request.projectId ?? defaults.defaultProjectId,
  type: request.type
});
const marshalReinstallServerRequest = (request, defaults) => ({
  os_id: request.osId
});
const marshalUpdateServerRequest$3 = (request, defaults) => ({
  name: request.name
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$w = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Apple silicon API. */
let API$x = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-3'];

  /**
   * List server types. List all technical details about Apple silicon server
   * types available in the specified zone. Since there is only one Availability
   * Zone for Apple silicon servers, the targeted value is `fr-par-3`.
   *
   * @param request - The request {@link ListServerTypesRequest}
   * @returns A Promise of ListServerTypesResponse
   */
  listServerTypes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/server-types`
  }, unmarshalListServerTypesResponse);

  /**
   * Get a server type. Get technical details (CPU, disk size etc.) of a server
   * type.
   *
   * @param request - The request {@link GetServerTypeRequest}
   * @returns A Promise of ServerType
   */
  getServerType = request => this.client.fetch({
    method: 'GET',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/server-type/${validatePathParam('serverType', request.serverType)}`
  }, unmarshalServerType$1);

  /**
   * Create a server. Create a new server in the targeted zone, specifying its
   * configuration including name and type.
   *
   * @param request - The request {@link CreateServerRequest}
   * @returns A Promise of Server
   */
  createServer = request => this.client.fetch({
    body: JSON.stringify(marshalCreateServerRequest$3(request, this.client.settings)),
    headers: jsonContentHeaders$w,
    method: 'POST',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`
  }, unmarshalServer$3);
  pageOfListServers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListServersResponse$3);

  /**
   * List all servers. List all servers in the specified zone. By default,
   * returned servers in the list are ordered by creation date in ascending
   * order, though this can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListServersRequest}
   * @returns A Promise of ListServersResponse
   */
  listServers = (request = {}) => enrichForPagination('servers', this.pageOfListServers, request);
  pageOfListOS = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/os`,
    urlParams: urlParams(['name', request.name], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['server_type', request.serverType])
  }, unmarshalListOSResponse$2);

  /**
   * List all Operating Systems (OS). List all Operating Systems (OS). The
   * response will include the total number of OS as well as their associated
   * IDs, names and labels.
   *
   * @param request - The request {@link ListOSRequest}
   * @returns A Promise of ListOSResponse
   */
  listOS = (request = {}) => enrichForPagination('os', this.pageOfListOS, request);

  /**
   * Get an Operating System (OS). Get an Operating System (OS). The response
   * will include the OS's unique ID as well as its name and label.
   *
   * @param request - The request {@link GetOSRequest}
   * @returns A Promise of OS
   */
  getOS = request => this.client.fetch({
    method: 'GET',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/os/${validatePathParam('osId', request.osId)}`
  }, unmarshalOS$2);

  /**
   * Get a server. Retrieve information about an existing Apple silicon server,
   * specified by its server ID. Its full details, including name, status and IP
   * address, are returned in the response object.
   *
   * @param request - The request {@link GetServerRequest}
   * @returns A Promise of Server
   */
  getServer = request => this.client.fetch({
    method: 'GET',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalServer$3);

  /**
   * Waits for {@link Server} to be in a final state.
   *
   * @param request - The request {@link GetServerRequest}
   * @param options - The waiting options
   * @returns A Promise of Server
   */
  waitForServer = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SERVER_TRANSIENT_STATUSES$3.includes(res.status))), this.getServer, request, options);

  /**
   * Update a server. Update the parameters of an existing Apple silicon server,
   * specified by its server ID.
   *
   * @param request - The request {@link UpdateServerRequest}
   * @returns A Promise of Server
   */
  updateServer = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateServerRequest$3(request, this.client.settings)),
    headers: jsonContentHeaders$w,
    method: 'PATCH',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalServer$3);

  /**
   * Delete a server. Delete an existing Apple silicon server, specified by its
   * server ID. Deleting a server is permanent, and cannot be undone. Note that
   * the minimum allocation period for Apple silicon-as-a-service is 24 hours,
   * meaning you cannot delete your server prior to that.
   *
   * @param request - The request {@link DeleteServerRequest}
   */
  deleteServer = request => this.client.fetch({
    method: 'DELETE',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  });

  /**
   * Reboot a server. Reboot an existing Apple silicon server, specified by its
   * server ID.
   *
   * @param request - The request {@link RebootServerRequest}
   * @returns A Promise of Server
   */
  rebootServer = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$w,
    method: 'POST',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/reboot`
  }, unmarshalServer$3);

  /**
   * Reinstall a server. Reinstall an existing Apple silicon server (specified
   * by its server ID) from a new image (OS). All the data on the disk is
   * deleted and all configuration is reset to the defailt configuration values
   * of the image (OS).
   *
   * @param request - The request {@link ReinstallServerRequest}
   * @returns A Promise of Server
   */
  reinstallServer = request => this.client.fetch({
    body: JSON.stringify(marshalReinstallServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$w,
    method: 'POST',
    path: `/apple-silicon/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/reinstall`
  }, unmarshalServer$3);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$w = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$x,
  SERVER_TRANSIENT_STATUSES: SERVER_TRANSIENT_STATUSES$3
});

var index$z = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$w
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link ServerInstallStatus}. */
const SERVER_INSTALL_TRANSIENT_STATUSES$1 = ['to_install', 'installing'];

/** Lists transient statutes of the enum {@link ServerPrivateNetworkStatus}. */
const SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES = ['attaching', 'detaching'];

/** Lists transient statutes of the enum {@link ServerStatus}. */
const SERVER_TRANSIENT_STATUSES$2 = ['delivering', 'stopping', 'starting', 'deleting', 'ordered', 'resetting'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalIP$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'IP' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    id: data.id,
    reverse: data.reverse,
    reverseStatus: data.reverse_status,
    reverseStatusMessage: data.reverse_status_message,
    version: data.version
  };
};
const unmarshalOSOSField = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OSOSField' failed as data isn't a dictionary.`);
  }
  return {
    defaultValue: data.default_value,
    editable: data.editable,
    required: data.required
  };
};
const unmarshalOS$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OS' failed as data isn't a dictionary.`);
  }
  return {
    allowed: data.allowed,
    enabled: data.enabled,
    id: data.id,
    licenseRequired: data.license_required,
    logoUrl: data.logo_url,
    name: data.name,
    password: data.password ? unmarshalOSOSField(data.password) : undefined,
    servicePassword: data.service_password ? unmarshalOSOSField(data.service_password) : undefined,
    serviceUser: data.service_user ? unmarshalOSOSField(data.service_user) : undefined,
    ssh: data.ssh ? unmarshalOSOSField(data.ssh) : undefined,
    user: data.user ? unmarshalOSOSField(data.user) : undefined,
    version: data.version
  };
};
const unmarshalCPU$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CPU' failed as data isn't a dictionary.`);
  }
  return {
    benchmark: data.benchmark,
    coreCount: data.core_count,
    frequency: data.frequency,
    name: data.name,
    threadCount: data.thread_count
  };
};
const unmarshalDisk$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Disk' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    type: data.type
  };
};
const unmarshalMemory$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Memory' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    frequency: data.frequency,
    isEcc: data.is_ecc,
    type: data.type
  };
};
const unmarshalOfferOptionOffer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferOptionOffer' failed as data isn't a dictionary.`);
  }
  return {
    enabled: data.enabled,
    id: data.id,
    manageable: data.manageable,
    name: data.name,
    osId: data.os_id,
    price: data.price ? unmarshalMoney(data.price) : undefined,
    subscriptionPeriod: data.subscription_period
  };
};
const unmarshalPersistentMemory$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PersistentMemory' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    frequency: data.frequency,
    type: data.type
  };
};
const unmarshalRaidController$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RaidController' failed as data isn't a dictionary.`);
  }
  return {
    model: data.model,
    raidLevel: data.raid_level
  };
};
const unmarshalOffer$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Offer' failed as data isn't a dictionary.`);
  }
  return {
    bandwidth: data.bandwidth,
    commercialRange: data.commercial_range,
    cpus: unmarshalArrayOfObject(data.cpus, unmarshalCPU$1),
    disks: unmarshalArrayOfObject(data.disks, unmarshalDisk$1),
    enable: data.enable,
    fee: data.fee ? unmarshalMoney(data.fee) : undefined,
    id: data.id,
    incompatibleOsIds: data.incompatible_os_ids,
    memories: unmarshalArrayOfObject(data.memories, unmarshalMemory$1),
    name: data.name,
    operationPath: data.operation_path,
    options: unmarshalArrayOfObject(data.options, unmarshalOfferOptionOffer),
    persistentMemories: unmarshalArrayOfObject(data.persistent_memories, unmarshalPersistentMemory$1),
    pricePerHour: data.price_per_hour ? unmarshalMoney(data.price_per_hour) : undefined,
    pricePerMonth: data.price_per_month ? unmarshalMoney(data.price_per_month) : undefined,
    privateBandwidth: data.private_bandwidth,
    quotaName: data.quota_name,
    raidControllers: unmarshalArrayOfObject(data.raid_controllers, unmarshalRaidController$1),
    sharedBandwidth: data.shared_bandwidth,
    stock: data.stock,
    subscriptionPeriod: data.subscription_period,
    tags: data.tags
  };
};
const unmarshalOption = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Option' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    manageable: data.manageable,
    name: data.name
  };
};
const unmarshalServerPrivateNetwork = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerPrivateNetwork' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    privateNetworkId: data.private_network_id,
    projectId: data.project_id,
    serverId: data.server_id,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at),
    vlan: data.vlan
  };
};
const unmarshalServerInstall$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerInstall' failed as data isn't a dictionary.`);
  }
  return {
    hostname: data.hostname,
    osId: data.os_id,
    serviceUrl: data.service_url,
    serviceUser: data.service_user,
    sshKeyIds: data.ssh_key_ids,
    status: data.status,
    user: data.user
  };
};
const unmarshalServerOption$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerOption' failed as data isn't a dictionary.`);
  }
  return {
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    manageable: data.manageable,
    name: data.name,
    status: data.status
  };
};
const unmarshalServerRescueServer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerRescueServer' failed as data isn't a dictionary.`);
  }
  return {
    password: data.password,
    user: data.user
  };
};
const unmarshalServer$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Server' failed as data isn't a dictionary.`);
  }
  return {
    bootType: data.boot_type,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    domain: data.domain,
    id: data.id,
    install: data.install ? unmarshalServerInstall$1(data.install) : undefined,
    ips: unmarshalArrayOfObject(data.ips, unmarshalIP$3),
    name: data.name,
    offerId: data.offer_id,
    offerName: data.offer_name,
    options: unmarshalArrayOfObject(data.options, unmarshalServerOption$1),
    organizationId: data.organization_id,
    pingStatus: data.ping_status,
    projectId: data.project_id,
    rescueServer: data.rescue_server ? unmarshalServerRescueServer(data.rescue_server) : undefined,
    status: data.status,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalSetting = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Setting' failed as data isn't a dictionary.`);
  }
  return {
    enabled: data.enabled,
    id: data.id,
    projectId: data.project_id,
    type: data.type
  };
};
const unmarshalBMCAccess$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BMCAccess' failed as data isn't a dictionary.`);
  }
  return {
    expiresAt: unmarshalDate(data.expires_at),
    login: data.login,
    password: data.password,
    url: data.url
  };
};
const unmarshalGetServerMetricsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetServerMetricsResponse' failed as data isn't a dictionary.`);
  }
  return {
    pings: data.pings ? unmarshalTimeSeries(data.pings) : undefined
  };
};
const unmarshalListOSResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListOSResponse' failed as data isn't a dictionary.`);
  }
  return {
    os: unmarshalArrayOfObject(data.os, unmarshalOS$1),
    totalCount: data.total_count
  };
};
const unmarshalListOffersResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListOffersResponse' failed as data isn't a dictionary.`);
  }
  return {
    offers: unmarshalArrayOfObject(data.offers, unmarshalOffer$2),
    totalCount: data.total_count
  };
};
const unmarshalListOptionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListOptionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    options: unmarshalArrayOfObject(data.options, unmarshalOption),
    totalCount: data.total_count
  };
};
const unmarshalServerEvent$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerEvent' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListServerEventsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServerEventsResponse' failed as data isn't a dictionary.`);
  }
  return {
    events: unmarshalArrayOfObject(data.events, unmarshalServerEvent$1),
    totalCount: data.total_count
  };
};
const unmarshalListServerPrivateNetworksResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServerPrivateNetworksResponse' failed as data isn't a dictionary.`);
  }
  return {
    serverPrivateNetworks: unmarshalArrayOfObject(data.server_private_networks, unmarshalServerPrivateNetwork),
    totalCount: data.total_count
  };
};
const unmarshalListServersResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalServer$2),
    totalCount: data.total_count
  };
};
const unmarshalListSettingsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSettingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    settings: unmarshalArrayOfObject(data.settings, unmarshalSetting),
    totalCount: data.total_count
  };
};
const unmarshalSetServerPrivateNetworksResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetServerPrivateNetworksResponse' failed as data isn't a dictionary.`);
  }
  return {
    serverPrivateNetworks: unmarshalArrayOfObject(data.server_private_networks, unmarshalServerPrivateNetwork)
  };
};
const marshalAddOptionServerRequest = (request, defaults) => ({
  expires_at: request.expiresAt
});
const marshalCreateServerRequestInstall = (request, defaults) => ({
  hostname: request.hostname,
  os_id: request.osId,
  password: request.password,
  service_password: request.servicePassword,
  service_user: request.serviceUser,
  ssh_key_ids: request.sshKeyIds,
  user: request.user
});
const marshalCreateServerRequest$2 = (request, defaults) => ({
  description: request.description,
  install: request.install !== undefined ? marshalCreateServerRequestInstall(request.install) : undefined,
  name: request.name,
  offer_id: request.offerId,
  option_ids: request.optionIds,
  tags: request.tags,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalInstallServerRequest$1 = (request, defaults) => ({
  hostname: request.hostname,
  os_id: request.osId,
  password: request.password,
  service_password: request.servicePassword,
  service_user: request.serviceUser,
  ssh_key_ids: request.sshKeyIds,
  user: request.user
});
const marshalPrivateNetworkApiAddServerPrivateNetworkRequest = (request, defaults) => ({
  private_network_id: request.privateNetworkId
});
const marshalPrivateNetworkApiSetServerPrivateNetworksRequest = (request, defaults) => ({
  private_network_ids: request.privateNetworkIds
});
const marshalRebootServerRequest = (request, defaults) => ({
  boot_type: request.bootType
});
const marshalStartBMCAccessRequest$1 = (request, defaults) => ({
  ip: request.ip
});
const marshalStartServerRequest = (request, defaults) => ({
  boot_type: request.bootType
});
const marshalUpdateIPRequest$2 = (request, defaults) => ({
  reverse: request.reverse
});
const marshalUpdateServerRequest$2 = (request, defaults) => ({
  description: request.description,
  name: request.name,
  tags: request.tags
});
const marshalUpdateSettingRequest = (request, defaults) => ({
  enabled: request.enabled
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$v = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Elastic Metal API.
 *
 * This API allows to manage your Elastic Metal server.
 */
let API$w = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'nl-ams-1', 'nl-ams-2'];
  pageOfListServers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`,
    urlParams: urlParams(['name', request.name], ['option_id', request.optionId], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['status', request.status], ['tags', request.tags])
  }, unmarshalListServersResponse$2);

  /**
   * List Elastic Metal servers for an Organization. List Elastic Metal servers
   * for a specific Organization.
   *
   * @param request - The request {@link ListServersRequest}
   * @returns A Promise of ListServersResponse
   */
  listServers = (request = {}) => enrichForPagination('servers', this.pageOfListServers, request);

  /**
   * Get a specific Elastic Metal server. Get full details of an existing
   * Elastic Metal server associated with the ID.
   *
   * @param request - The request {@link GetServerRequest}
   * @returns A Promise of Server
   */
  getServer = request => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalServer$2);

  /**
   * Waits for {@link Server} to be in a final state.
   *
   * @param request - The request {@link GetServerRequest}
   * @param options - The waiting options
   * @returns A Promise of Server
   */
  waitForServer = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SERVER_TRANSIENT_STATUSES$2.includes(res.status))), this.getServer, request, options);

  /**
   * Create an Elastic Metal server. Create a new Elastic Metal server. Once the
   * server is created, proceed with the [installation of an OS](#post-3e949e).
   *
   * @param request - The request {@link CreateServerRequest}
   * @returns A Promise of Server
   */
  createServer = request => this.client.fetch({
    body: JSON.stringify(marshalCreateServerRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'POST',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`
  }, unmarshalServer$2);

  /**
   * Update an Elastic Metal server. Update the server associated with the ID.
   * You can update parameters such as the server's name, tags and description.
   * Any parameters left null in the request body are not updated.
   *
   * @param request - The request {@link UpdateServerRequest}
   * @returns A Promise of Server
   */
  updateServer = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateServerRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'PATCH',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalServer$2);

  /**
   * Install an Elastic Metal server. Install an Operating System (OS) on the
   * Elastic Metal server with a specific ID.
   *
   * @param request - The request {@link InstallServerRequest}
   * @returns A Promise of Server
   */
  installServer = request => this.client.fetch({
    body: JSON.stringify(marshalInstallServerRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'POST',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/install`
  }, unmarshalServer$2);

  /**
   * Return server metrics. Get the ping status of the server associated with
   * the ID.
   *
   * @param request - The request {@link GetServerMetricsRequest}
   * @returns A Promise of GetServerMetricsResponse
   */
  getServerMetrics = request => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/metrics`
  }, unmarshalGetServerMetricsResponse);

  /**
   * Delete an Elastic Metal server. Delete the server associated with the ID.
   *
   * @param request - The request {@link DeleteServerRequest}
   * @returns A Promise of Server
   */
  deleteServer = request => this.client.fetch({
    method: 'DELETE',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalServer$2);

  /**
   * Reboot an Elastic Metal server. Reboot the Elastic Metal server associated
   * with the ID, use the `boot_type` `rescue` to reboot the server in rescue
   * mode.
   *
   * @param request - The request {@link RebootServerRequest}
   * @returns A Promise of Server
   */
  rebootServer = request => this.client.fetch({
    body: JSON.stringify(marshalRebootServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'POST',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/reboot`
  }, unmarshalServer$2);

  /**
   * Start an Elastic Metal server. Start the server associated with the ID.
   *
   * @param request - The request {@link StartServerRequest}
   * @returns A Promise of Server
   */
  startServer = request => this.client.fetch({
    body: JSON.stringify(marshalStartServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'POST',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/start`
  }, unmarshalServer$2);

  /**
   * Stop an Elastic Metal server. Stop the server associated with the ID. The
   * server remains allocated to your account and all data remains on the local
   * storage of the server.
   *
   * @param request - The request {@link StopServerRequest}
   * @returns A Promise of Server
   */
  stopServer = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$v,
    method: 'POST',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/stop`
  }, unmarshalServer$2);
  pageOfListServerEvents = request => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/events`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListServerEventsResponse$1);

  /**
   * List server events. List event (i.e. start/stop/reboot) associated to the
   * server ID.
   *
   * @param request - The request {@link ListServerEventsRequest}
   * @returns A Promise of ListServerEventsResponse
   */
  listServerEvents = request => enrichForPagination('events', this.pageOfListServerEvents, request);

  /**
   * Start BMC access. Start BMC (Baseboard Management Controller) access
   * associated with the ID. The BMC (Baseboard Management Controller) access is
   * available one hour after the installation of the server. You need first to
   * create an option Remote Access. You will find the ID and the price with a
   * call to listOffers
   * (https://developers.scaleway.com/en/products/baremetal/api/#get-78db92).
   * Then add the option
   * https://developers.scaleway.com/en/products/baremetal/api/#post-b14abd.
   * After adding the BMC option, you need to Get Remote Access to get the
   * login/password
   * https://developers.scaleway.com/en/products/baremetal/api/#get-cefc0f. Do
   * not forget to delete the Option after use.
   *
   * @param request - The request {@link StartBMCAccessRequest}
   * @returns A Promise of BMCAccess
   */
  startBMCAccess = request => this.client.fetch({
    body: JSON.stringify(marshalStartBMCAccessRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'POST',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/bmc-access`
  }, unmarshalBMCAccess$1);

  /**
   * Get BMC access. Get the BMC (Baseboard Management Controller) access
   * associated with the ID, including the URL and login information needed to
   * connect.
   *
   * @param request - The request {@link GetBMCAccessRequest}
   * @returns A Promise of BMCAccess
   */
  getBMCAccess = request => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/bmc-access`
  }, unmarshalBMCAccess$1);

  /**
   * Stop BMC access. Stop BMC (Baseboard Management Controller) access
   * associated with the ID.
   *
   * @param request - The request {@link StopBMCAccessRequest}
   */
  stopBMCAccess = request => this.client.fetch({
    method: 'DELETE',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/bmc-access`
  });

  /**
   * Update IP. Configure the IP address associated with the server ID and IP
   * ID. You can use this method to set a reverse DNS for an IP address.
   *
   * @param request - The request {@link UpdateIPRequest}
   * @returns A Promise of IP
   */
  updateIP = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateIPRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'PATCH',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIP$3);

  /**
   * Add server option. Add an option, such as Private Networks, to a specific
   * server.
   *
   * @param request - The request {@link AddOptionServerRequest}
   * @returns A Promise of Server
   */
  addOptionServer = request => this.client.fetch({
    body: JSON.stringify(marshalAddOptionServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'POST',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/options/${validatePathParam('optionId', request.optionId)}`
  }, unmarshalServer$2);

  /**
   * Delete server option. Delete an option from a specific server.
   *
   * @param request - The request {@link DeleteOptionServerRequest}
   * @returns A Promise of Server
   */
  deleteOptionServer = request => this.client.fetch({
    method: 'DELETE',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/options/${validatePathParam('optionId', request.optionId)}`
  }, unmarshalServer$2);
  pageOfListOffers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/offers`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['subscription_period', request.subscriptionPeriod])
  }, unmarshalListOffersResponse$2);

  /**
   * List offers. List all available Elastic Metal server configurations.
   *
   * @param request - The request {@link ListOffersRequest}
   * @returns A Promise of ListOffersResponse
   */
  listOffers = (request = {}) => enrichForPagination('offers', this.pageOfListOffers, request);

  /**
   * Get offer. Get details of an offer identified by its offer ID.
   *
   * @param request - The request {@link GetOfferRequest}
   * @returns A Promise of Offer
   */
  getOffer = request => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/offers/${validatePathParam('offerId', request.offerId)}`
  }, unmarshalOffer$2);

  /**
   * Get option. Return specific option for the ID.
   *
   * @param request - The request {@link GetOptionRequest}
   * @returns A Promise of Option
   */
  getOption = request => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/options/${validatePathParam('optionId', request.optionId)}`
  }, unmarshalOption);
  pageOfListOptions = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/options`,
    urlParams: urlParams(['name', request.name], ['offer_id', request.offerId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListOptionsResponse);

  /**
   * List options. List all options matching with filters.
   *
   * @param request - The request {@link ListOptionsRequest}
   * @returns A Promise of ListOptionsResponse
   */
  listOptions = (request = {}) => enrichForPagination('options', this.pageOfListOptions, request);
  pageOfListSettings = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/settings`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListSettingsResponse);

  /**
   * List all settings. Return all settings for a Project ID.
   *
   * @param request - The request {@link ListSettingsRequest}
   * @returns A Promise of ListSettingsResponse
   */
  listSettings = (request = {}) => enrichForPagination('settings', this.pageOfListSettings, request);

  /**
   * Update setting. Update a setting for a Project ID (enable or disable).
   *
   * @param request - The request {@link UpdateSettingRequest}
   * @returns A Promise of Setting
   */
  updateSetting = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSettingRequest(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'PATCH',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/settings/${validatePathParam('settingId', request.settingId)}`
  }, unmarshalSetting);
  pageOfListOS = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/os`,
    urlParams: urlParams(['offer_id', request.offerId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListOSResponse$1);

  /**
   * List available OSes. List all OSes that are available for installation on
   * Elastic Metal servers.
   *
   * @param request - The request {@link ListOSRequest}
   * @returns A Promise of ListOSResponse
   */
  listOS = (request = {}) => enrichForPagination('os', this.pageOfListOS, request);

  /**
   * Get OS with an ID. Return the specific OS for the ID.
   *
   * @param request - The request {@link GetOSRequest}
   * @returns A Promise of OS
   */
  getOS = request => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/os/${validatePathParam('osId', request.osId)}`
  }, unmarshalOS$1);
};

/** Elastic Metal Private Network API. */
class PrivateNetworkAPI extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-2'];

  /**
   * Add a server to a Private Network.
   *
   * @param request - The request
   *   {@link PrivateNetworkApiAddServerPrivateNetworkRequest}
   * @returns A Promise of ServerPrivateNetwork
   */
  addServerPrivateNetwork = request => this.client.fetch({
    body: JSON.stringify(marshalPrivateNetworkApiAddServerPrivateNetworkRequest(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'POST',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private-networks`
  }, unmarshalServerPrivateNetwork);

  /**
   * Set multiple Private Networks on a server.
   *
   * @param request - The request
   *   {@link PrivateNetworkApiSetServerPrivateNetworksRequest}
   * @returns A Promise of SetServerPrivateNetworksResponse
   */
  setServerPrivateNetworks = request => this.client.fetch({
    body: JSON.stringify(marshalPrivateNetworkApiSetServerPrivateNetworksRequest(request, this.client.settings)),
    headers: jsonContentHeaders$v,
    method: 'PUT',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private-networks`
  }, unmarshalSetServerPrivateNetworksResponse);
  pageOfListServerPrivateNetworks = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/server-private-networks`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['private_network_id', request.privateNetworkId], ['project_id', request.projectId], ['server_id', request.serverId])
  }, unmarshalListServerPrivateNetworksResponse);

  /**
   * List the Private Networks of a server.
   *
   * @param request - The request
   *   {@link PrivateNetworkApiListServerPrivateNetworksRequest}
   * @returns A Promise of ListServerPrivateNetworksResponse
   */
  listServerPrivateNetworks = (request = {}) => enrichForPagination('serverPrivateNetworks', this.pageOfListServerPrivateNetworks, request);

  /**
   * Delete a Private Network.
   *
   * @param request - The request
   *   {@link PrivateNetworkApiDeleteServerPrivateNetworkRequest}
   */
  deleteServerPrivateNetwork = request => this.client.fetch({
    method: 'DELETE',
    path: `/baremetal/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}`
  });
}

class BaremetalV1UtilsAPI extends API$w {
  /**
   * Waits for {@link ServerInstall} to be in a final state.
   *
   * @param request - The request {@link GetServerRequest}
   * @param options - The waiting options
   * @returns A Promise of ServerInstall
   */
  waitForServerInstall = (request, options) => tryAtIntervals(async () => {
    const value = await this.getServer(request).then(server => {
      if (!server.install) {
        throw new Error(`Server creation has not begun for server ${request.serverId}`);
      }
      return server.install;
    });
    return {
      done: !SERVER_INSTALL_TRANSIENT_STATUSES$1.includes(value.status),
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateServerRequest = {
  description: {
    maxLength: 255
  },
  name: {
    minLength: 1
  }
};
const CreateServerRequestInstall = {
  hostname: {
    maxLength: 255
  },
  password: {
    maxLength: 255
  },
  servicePassword: {
    maxLength: 255
  },
  serviceUser: {
    maxLength: 255
  },
  user: {
    maxLength: 255
  }
};
const InstallServerRequest$1 = {
  hostname: {
    maxLength: 255
  },
  password: {
    maxLength: 255
  },
  servicePassword: {
    maxLength: 255
  },
  serviceUser: {
    maxLength: 255
  },
  user: {
    maxLength: 255
  }
};
const ListOSRequest$1 = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListOffersRequest$1 = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListOptionsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListServerEventsRequest$1 = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListServersRequest$1 = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListSettingsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const UpdateServerRequest$1 = {
  description: {
    maxLength: 255
  },
  name: {
    maxLength: 255
  }
};

var validationRules_gen$h = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateServerRequest: CreateServerRequest,
  CreateServerRequestInstall: CreateServerRequestInstall,
  InstallServerRequest: InstallServerRequest$1,
  ListOSRequest: ListOSRequest$1,
  ListOffersRequest: ListOffersRequest$1,
  ListOptionsRequest: ListOptionsRequest,
  ListServerEventsRequest: ListServerEventsRequest$1,
  ListServersRequest: ListServersRequest$1,
  ListSettingsRequest: ListSettingsRequest,
  UpdateServerRequest: UpdateServerRequest$1
});

var index$y = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: BaremetalV1UtilsAPI,
  PrivateNetworkAPI: PrivateNetworkAPI,
  SERVER_INSTALL_TRANSIENT_STATUSES: SERVER_INSTALL_TRANSIENT_STATUSES$1,
  SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES: SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES,
  SERVER_TRANSIENT_STATUSES: SERVER_TRANSIENT_STATUSES$2,
  ValidationRules: validationRules_gen$h
});

var index$x = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index$y
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalGetConsumptionResponseConsumption = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetConsumptionResponseConsumption' failed as data isn't a dictionary.`);
  }
  return {
    category: data.category,
    description: data.description,
    operationPath: data.operation_path,
    projectId: data.project_id,
    value: data.value ? unmarshalMoney(data.value) : undefined
  };
};
const unmarshalGetConsumptionResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetConsumptionResponse' failed as data isn't a dictionary.`);
  }
  return {
    consumptions: unmarshalArrayOfObject(data.consumptions, unmarshalGetConsumptionResponseConsumption),
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalDiscountCoupon$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DiscountCoupon' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description
  };
};
const unmarshalDiscountFilter$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DiscountFilter' failed as data isn't a dictionary.`);
  }
  return {
    type: data.type,
    value: data.value
  };
};
const unmarshalDiscount$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Discount' failed as data isn't a dictionary.`);
  }
  return {
    coupon: data.coupon ? unmarshalDiscountCoupon$1(data.coupon) : undefined,
    creationDate: unmarshalDate(data.creation_date),
    description: data.description,
    filters: unmarshalArrayOfObject(data.filters, unmarshalDiscountFilter$1),
    id: data.id,
    mode: data.mode,
    organizationId: data.organization_id,
    startDate: unmarshalDate(data.start_date),
    stopDate: unmarshalDate(data.stop_date),
    value: data.value,
    valueRemaining: data.value_remaining,
    valueUsed: data.value_used
  };
};
const unmarshalListDiscountsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDiscountsResponse' failed as data isn't a dictionary.`);
  }
  return {
    discounts: unmarshalArrayOfObject(data.discounts, unmarshalDiscount$1),
    totalCount: data.total_count
  };
};
const unmarshalInvoice$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Invoice' failed as data isn't a dictionary.`);
  }
  return {
    dueDate: unmarshalDate(data.due_date),
    id: data.id,
    invoiceType: data.invoice_type,
    issuedDate: unmarshalDate(data.issued_date),
    number: data.number,
    startDate: unmarshalDate(data.start_date),
    totalTaxed: data.total_taxed ? unmarshalMoney(data.total_taxed) : undefined,
    totalUntaxed: data.total_untaxed ? unmarshalMoney(data.total_untaxed) : undefined
  };
};
const unmarshalListInvoicesResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInvoicesResponse' failed as data isn't a dictionary.`);
  }
  return {
    invoices: unmarshalArrayOfObject(data.invoices, unmarshalInvoice$2),
    totalCount: data.total_count
  };
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
/**
 * Billing API.
 *
 * This API allows you to query your consumption.
 */
let API$v = class API extends API$z {
  /**
   * Get current month's consumption. The consumption reflects the amount of
   * money you have spent for the products you have used. The consumption value
   * is monetary and is not computed in real time.
   *
   * @param request - The request {@link GetConsumptionRequest}
   * @returns A Promise of GetConsumptionResponse
   */
  getConsumption = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/billing/v2alpha1/consumption`,
    urlParams: urlParams(['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId])
  }, unmarshalGetConsumptionResponse);
  pageOfListInvoices = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/billing/v2alpha1/invoices`,
    urlParams: urlParams(['invoice_type', request.invoiceType], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['started_after', request.startedAfter], ['started_before', request.startedBefore])
  }, unmarshalListInvoicesResponse$2);

  /**
   * List invoices. List all your invoices, filtering by `start_date` and
   * `invoice_type`. Each invoice has its own ID.
   *
   * @param request - The request {@link ListInvoicesRequest}
   * @returns A Promise of ListInvoicesResponse
   */
  listInvoices = (request = {}) => enrichForPagination('invoices', this.pageOfListInvoices, request);

  /**
   * Download an invoice. Download a specific invoice, specified by its ID.
   *
   * @param request - The request {@link DownloadInvoiceRequest}
   * @returns A Promise of Blob
   */
  downloadInvoice = request => this.client.fetch({
    method: 'GET',
    path: `/billing/v2alpha1/invoices/${validatePathParam('invoiceId', request.invoiceId)}/download`,
    urlParams: urlParams(['dl', 1], ['file_type', request.fileType]),
    responseType: 'blob'
  });
  pageOfListDiscounts = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/billing/v2alpha1/discounts`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDiscountsResponse$1);

  /**
   * List all user's discounts. List all discounts for an organization and
   * usable categories/products/offers/references/regions/zones where the
   * discount can be applied.
   *
   * @param request - The request {@link ListDiscountsRequest}
   * @returns A Promise of ListDiscountsResponse
   */
  listDiscounts = (request = {}) => enrichForPagination('discounts', this.pageOfListDiscounts, request);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$v = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$v
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalInvoice$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Invoice' failed as data isn't a dictionary.`);
  }
  return {
    billingPeriod: unmarshalDate(data.billing_period),
    dueDate: unmarshalDate(data.due_date),
    id: data.id,
    issuedDate: unmarshalDate(data.issued_date),
    number: data.number,
    organizationId: data.organization_id,
    organizationName: data.organization_name,
    sellerName: data.seller_name,
    startDate: unmarshalDate(data.start_date),
    state: data.state,
    stopDate: unmarshalDate(data.stop_date),
    totalDiscount: data.total_discount ? unmarshalMoney(data.total_discount) : undefined,
    totalTax: data.total_tax ? unmarshalMoney(data.total_tax) : undefined,
    totalTaxed: data.total_taxed ? unmarshalMoney(data.total_taxed) : undefined,
    totalUndiscount: data.total_undiscount ? unmarshalMoney(data.total_undiscount) : undefined,
    totalUntaxed: data.total_untaxed ? unmarshalMoney(data.total_untaxed) : undefined,
    type: data.type
  };
};
const unmarshalListConsumptionsResponseConsumption = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListConsumptionsResponseConsumption' failed as data isn't a dictionary.`);
  }
  return {
    billedQuantity: data.billed_quantity,
    categoryName: data.category_name,
    productName: data.product_name,
    projectId: data.project_id,
    resourceName: data.resource_name,
    sku: data.sku,
    unit: data.unit,
    value: data.value ? unmarshalMoney(data.value) : undefined
  };
};
const unmarshalListConsumptionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListConsumptionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    consumptions: unmarshalArrayOfObject(data.consumptions, unmarshalListConsumptionsResponseConsumption),
    totalCount: data.total_count,
    totalDiscountUntaxedValue: data.total_discount_untaxed_value,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalDiscountCoupon = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DiscountCoupon' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description
  };
};
const unmarshalDiscountFilter = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DiscountFilter' failed as data isn't a dictionary.`);
  }
  return {
    type: data.type,
    value: data.value
  };
};
const unmarshalDiscount = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Discount' failed as data isn't a dictionary.`);
  }
  return {
    coupon: data.coupon ? unmarshalDiscountCoupon(data.coupon) : undefined,
    creationDate: unmarshalDate(data.creation_date),
    description: data.description,
    filters: unmarshalArrayOfObject(data.filters, unmarshalDiscountFilter),
    id: data.id,
    mode: data.mode,
    organizationId: data.organization_id,
    startDate: unmarshalDate(data.start_date),
    stopDate: unmarshalDate(data.stop_date),
    value: data.value,
    valueRemaining: data.value_remaining,
    valueUsed: data.value_used
  };
};
const unmarshalListDiscountsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDiscountsResponse' failed as data isn't a dictionary.`);
  }
  return {
    discounts: unmarshalArrayOfObject(data.discounts, unmarshalDiscount),
    totalCount: data.total_count
  };
};
const unmarshalListInvoicesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInvoicesResponse' failed as data isn't a dictionary.`);
  }
  return {
    invoices: unmarshalArrayOfObject(data.invoices, unmarshalInvoice$1),
    totalCount: data.total_count
  };
};
const unmarshalListTaxesResponseTax = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTaxesResponseTax' failed as data isn't a dictionary.`);
  }
  return {
    currency: data.currency,
    description: data.description,
    rate: data.rate,
    totalTaxValue: data.total_tax_value
  };
};
const unmarshalListTaxesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTaxesResponse' failed as data isn't a dictionary.`);
  }
  return {
    taxes: unmarshalArrayOfObject(data.taxes, unmarshalListTaxesResponseTax),
    totalCount: data.total_count,
    updatedAt: unmarshalDate(data.updated_at)
  };
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
/**
 * Billing API.
 *
 * This API allows you to query your consumption.
 */
let API$u = class API extends API$z {
  pageOfListConsumptions = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/billing/v2beta1/consumptions`,
    urlParams: urlParams(['billing_period', request.billingPeriod], ['category_name', request.categoryName], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ...Object.entries(resolveOneOf([{
      default: this.client.settings.defaultOrganizationId,
      param: 'organization_id',
      value: request.organizationId
    }, {
      default: this.client.settings.defaultProjectId,
      param: 'project_id',
      value: request.projectId
    }])))
  }, unmarshalListConsumptionsResponse);

  /**
   * Get monthly consumption. Consumption allows you to retrieve your past or
   * current consumption cost, by project or category.
   *
   * @param request - The request {@link ListConsumptionsRequest}
   * @returns A Promise of ListConsumptionsResponse
   */
  listConsumptions = (request = {}) => enrichForPagination('consumptions', this.pageOfListConsumptions, request);
  pageOfListTaxes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/billing/v2beta1/taxes`,
    urlParams: urlParams(['billing_period', request.billingPeriod], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListTaxesResponse);

  /**
   * Get monthly consumption taxes. Consumption Tax allows you to retrieve your
   * past or current tax charges, by project or category.
   *
   * @param request - The request {@link ListTaxesRequest}
   * @returns A Promise of ListTaxesResponse
   */
  listTaxes = (request = {}) => enrichForPagination('taxes', this.pageOfListTaxes, request);
  pageOfListInvoices = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/billing/v2beta1/invoices`,
    urlParams: urlParams(['billing_period_start_after', request.billingPeriodStartAfter], ['billing_period_start_before', request.billingPeriodStartBefore], ['invoice_type', request.invoiceType], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListInvoicesResponse$1);

  /**
   * List invoices. List all your invoices, filtering by `start_date` and
   * `invoice_type`. Each invoice has its own ID.
   *
   * @param request - The request {@link ListInvoicesRequest}
   * @returns A Promise of ListInvoicesResponse
   */
  listInvoices = (request = {}) => enrichForPagination('invoices', this.pageOfListInvoices, request);

  /**
   * Export invoices. Export invoices in a CSV file.
   *
   * @param request - The request {@link ExportInvoicesRequest}
   * @returns A Promise of Blob
   */
  exportInvoices = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/billing/v2beta1/export-invoices`,
    urlParams: urlParams(['dl', 1], ['billing_period_start_after', request.billingPeriodStartAfter], ['billing_period_start_before', request.billingPeriodStartBefore], ['file_type', request.fileType], ['invoice_type', request.invoiceType], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize]),
    responseType: 'blob'
  });

  /**
   * Get an invoice. Get a specific invoice, specified by its ID.
   *
   * @param request - The request {@link GetInvoiceRequest}
   * @returns A Promise of Invoice
   */
  getInvoice = request => this.client.fetch({
    method: 'GET',
    path: `/billing/v2beta1/invoices/${validatePathParam('invoiceId', request.invoiceId)}`
  }, unmarshalInvoice$1);

  /**
   * Download an invoice. Download a specific invoice, specified by its ID.
   *
   * @param request - The request {@link DownloadInvoiceRequest}
   * @returns A Promise of Blob
   */
  downloadInvoice = request => this.client.fetch({
    method: 'GET',
    path: `/billing/v2beta1/invoices/${validatePathParam('invoiceId', request.invoiceId)}/download`,
    urlParams: urlParams(['dl', 1], ['file_type', request.fileType]),
    responseType: 'blob'
  });
  pageOfListDiscounts = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/billing/v2beta1/discounts`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDiscountsResponse);

  /**
   * List discounts. List all discounts for your organization and usable
   * categories, products, offers, references, regions and zones where the
   * discount can be applied.
   *
   * @param request - The request {@link ListDiscountsRequest}
   * @returns A Promise of ListDiscountsResponse
   */
  listDiscounts = (request = {}) => enrichForPagination('discounts', this.pageOfListDiscounts, request);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$u = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$u
});

var index$w = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v2alpha1: index_gen$v,
  v2beta1: index_gen$u
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link ReferenceStatus}. */
const REFERENCE_TRANSIENT_STATUSES = ['attaching', 'detaching', 'creating'];

/** Lists transient statutes of the enum {@link SnapshotStatus}. */
const SNAPSHOT_TRANSIENT_STATUSES$3 = ['creating', 'deleting'];

/** Lists transient statutes of the enum {@link VolumeStatus}. */
const VOLUME_TRANSIENT_STATUSES$1 = ['creating', 'deleting', 'resizing', 'snapshotting'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalReference = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Reference' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    productResourceId: data.product_resource_id,
    productResourceType: data.product_resource_type,
    status: data.status,
    type: data.type
  };
};
const unmarshalSnapshotParentVolume = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SnapshotParentVolume' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name,
    status: data.status,
    type: data.type
  };
};
const unmarshalSnapshot$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Snapshot' failed as data isn't a dictionary.`);
  }
  return {
    class: data.class,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    parentVolume: data.parent_volume ? unmarshalSnapshotParentVolume(data.parent_volume) : undefined,
    projectId: data.project_id,
    references: unmarshalArrayOfObject(data.references, unmarshalReference),
    size: data.size,
    status: data.status,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalVolumeSpecifications = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'VolumeSpecifications' failed as data isn't a dictionary.`);
  }
  return {
    class: data.class,
    perfIops: data.perf_iops
  };
};
const unmarshalVolume$4 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Volume' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    lastDetachedAt: unmarshalDate(data.last_detached_at),
    name: data.name,
    parentSnapshotId: data.parent_snapshot_id,
    projectId: data.project_id,
    references: unmarshalArrayOfObject(data.references, unmarshalReference),
    size: data.size,
    specs: data.specs ? unmarshalVolumeSpecifications(data.specs) : undefined,
    status: data.status,
    tags: data.tags,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalListSnapshotsResponse$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSnapshotsResponse' failed as data isn't a dictionary.`);
  }
  return {
    snapshots: unmarshalArrayOfObject(data.snapshots, unmarshalSnapshot$3),
    totalCount: data.total_count
  };
};
const unmarshalVolumeType$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'VolumeType' failed as data isn't a dictionary.`);
  }
  return {
    pricing: data.pricing ? unmarshalMoney(data.pricing) : undefined,
    snapshotPricing: data.snapshot_pricing ? unmarshalMoney(data.snapshot_pricing) : undefined,
    specs: data.specs ? unmarshalVolumeSpecifications(data.specs) : undefined,
    type: data.type
  };
};
const unmarshalListVolumeTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListVolumeTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    volumeTypes: unmarshalArrayOfObject(data.volume_types, unmarshalVolumeType$1)
  };
};
const unmarshalListVolumesResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListVolumesResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    volumes: unmarshalArrayOfObject(data.volumes, unmarshalVolume$4)
  };
};
const marshalCreateSnapshotRequest$3 = (request, defaults) => ({
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags,
  volume_id: request.volumeId
});
const marshalCreateVolumeRequestFromEmpty = (request, defaults) => ({
  size: request.size
});
const marshalCreateVolumeRequestFromSnapshot = (request, defaults) => ({
  size: request.size,
  snapshot_id: request.snapshotId
});
const marshalCreateVolumeRequest$2 = (request, defaults) => ({
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags,
  ...resolveOneOf([{
    param: 'from_empty',
    value: request.fromEmpty !== undefined ? marshalCreateVolumeRequestFromEmpty(request.fromEmpty) : undefined
  }, {
    param: 'from_snapshot',
    value: request.fromSnapshot !== undefined ? marshalCreateVolumeRequestFromSnapshot(request.fromSnapshot) : undefined
  }]),
  ...resolveOneOf([{
    param: 'perf_iops',
    value: request.perfIops
  }])
});
const marshalUpdateSnapshotRequest$3 = (request, defaults) => ({
  name: request.name,
  tags: request.tags
});
const marshalUpdateVolumeRequest$2 = (request, defaults) => ({
  name: request.name,
  perf_iops: request.perfIops,
  size: request.size,
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$u = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Scaleway Block Storage API.
 *
 * This API allows you to use and manage your Block Storage volumes.
 */
let API$t = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'nl-ams-1', 'nl-ams-3', 'pl-waw-3'];
  pageOfListVolumeTypes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volume-types`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListVolumeTypesResponse);

  /**
   * List volume types. List all available volume types in a specified zone. The
   * volume types listed are ordered by name in ascending order.
   *
   * @param request - The request {@link ListVolumeTypesRequest}
   * @returns A Promise of ListVolumeTypesResponse
   */
  listVolumeTypes = (request = {}) => enrichForPagination('volumeTypes', this.pageOfListVolumeTypes, request);
  pageOfListVolumes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['product_resource_id', request.productResourceId], ['project_id', request.projectId])
  }, unmarshalListVolumesResponse$2);

  /**
   * List volumes. List all existing volumes in a specified zone. By default,
   * the volumes listed are ordered by creation date in ascending order. This
   * can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListVolumesRequest}
   * @returns A Promise of ListVolumesResponse
   */
  listVolumes = (request = {}) => enrichForPagination('volumes', this.pageOfListVolumes, request);

  /**
   * Create a volume. To create a new volume from scratch, you must specify
   * `from_empty` and the `size`. To create a volume from an existing snapshot,
   * specify `from_snapshot` and the `snapshot_id` in the request payload
   * instead, size is optional and can be specified if you need to extend the
   * original size. The volume will take on the same volume class and underlying
   * IOPS limitations as the original snapshot.
   *
   * @param request - The request {@link CreateVolumeRequest}
   * @returns A Promise of Volume
   */
  createVolume = request => this.client.fetch({
    body: JSON.stringify(marshalCreateVolumeRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$u,
    method: 'POST',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes`
  }, unmarshalVolume$4);

  /**
   * Get a volume. Retrieve technical information about a specific volume.
   * Details such as size, type, and status are returned in the response.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @returns A Promise of Volume
   */
  getVolume = request => this.client.fetch({
    method: 'GET',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalVolume$4);

  /**
   * Waits for {@link Volume} to be in a final state.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @param options - The waiting options
   * @returns A Promise of Volume
   */
  waitForVolume = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!VOLUME_TRANSIENT_STATUSES$1.includes(res.status))), this.getVolume, request, options);

  /**
   * Delete a detached volume. You must specify the `volume_id` of the volume
   * you want to delete. The volume must not be in the `in_use` status.
   *
   * @param request - The request {@link DeleteVolumeRequest}
   */
  deleteVolume = request => this.client.fetch({
    method: 'DELETE',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  });

  /**
   * Update a volume. Update the technical details of a volume, such as its
   * name, tags, or its new size and `volume_type` (within the same Block
   * Storage class). You can only resize a volume to a larger size. It is
   * currently not possible to change your Block Storage Class.
   *
   * @param request - The request {@link UpdateVolumeRequest}
   * @returns A Promise of Volume
   */
  updateVolume = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateVolumeRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$u,
    method: 'PATCH',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalVolume$4);
  pageOfListSnapshots = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['volume_id', request.volumeId])
  }, unmarshalListSnapshotsResponse$3);

  /**
   * List all snapshots. List all available snapshots in a specified zone. By
   * default, the snapshots listed are ordered by creation date in ascending
   * order. This can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListSnapshotsRequest}
   * @returns A Promise of ListSnapshotsResponse
   */
  listSnapshots = (request = {}) => enrichForPagination('snapshots', this.pageOfListSnapshots, request);

  /**
   * Get a snapshot. Retrieve technical information about a specific snapshot.
   * Details such as size, volume type, and status are returned in the
   * response.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  getSnapshot = request => this.client.fetch({
    method: 'GET',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot$3);

  /**
   * Waits for {@link Snapshot} to be in a final state.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @param options - The waiting options
   * @returns A Promise of Snapshot
   */
  waitForSnapshot = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SNAPSHOT_TRANSIENT_STATUSES$3.includes(res.status))), this.getSnapshot, request, options);

  /**
   * Create a snapshot of a volume. To create a snapshot, the volume must be in
   * the `in_use` or the `available` status. If your volume is in a transient
   * state, you need to wait until the end of the current operation.
   *
   * @param request - The request {@link CreateSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  createSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSnapshotRequest$3(request, this.client.settings)),
    headers: jsonContentHeaders$u,
    method: 'POST',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots`
  }, unmarshalSnapshot$3);

  /**
   * Delete a snapshot. You must specify the `snapshot_id` of the snapshot you
   * want to delete. The snapshot must not be in use.
   *
   * @param request - The request {@link DeleteSnapshotRequest}
   */
  deleteSnapshot = request => this.client.fetch({
    method: 'DELETE',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  });

  /**
   * Update a snapshot. Update the name or tags of the snapshot.
   *
   * @param request - The request {@link UpdateSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  updateSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSnapshotRequest$3(request, this.client.settings)),
    headers: jsonContentHeaders$u,
    method: 'PATCH',
    path: `/block/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot$3);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateSnapshotRequest = {
  name: {
    minLength: 1
  }
};
const CreateVolumeRequest = {
  name: {
    minLength: 1
  }
};
const ListSnapshotsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListVolumeTypesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListVolumesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const Reference = {
  productResourceType: {
    minLength: 1
  }
};

var validationRules_gen$g = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateSnapshotRequest: CreateSnapshotRequest,
  CreateVolumeRequest: CreateVolumeRequest,
  ListSnapshotsRequest: ListSnapshotsRequest,
  ListVolumeTypesRequest: ListVolumeTypesRequest,
  ListVolumesRequest: ListVolumesRequest,
  Reference: Reference
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$t = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$t,
  REFERENCE_TRANSIENT_STATUSES: REFERENCE_TRANSIENT_STATUSES,
  SNAPSHOT_TRANSIENT_STATUSES: SNAPSHOT_TRANSIENT_STATUSES$3,
  VOLUME_TRANSIENT_STATUSES: VOLUME_TRANSIENT_STATUSES$1,
  ValidationRules: validationRules_gen$g
});

var index$v = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$t
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalContactPointEmail$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactPointEmail' failed as data isn't a dictionary.`);
  }
  return {
    to: data.to
  };
};
const unmarshalContactPoint$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactPoint' failed as data isn't a dictionary.`);
  }
  return {
    email: data.email ? unmarshalContactPointEmail$1(data.email) : undefined,
    region: data.region
  };
};
const unmarshalDataSource = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DataSource' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    origin: data.origin,
    projectId: data.project_id,
    region: data.region,
    synchronizedWithGrafana: data.synchronized_with_grafana,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    url: data.url
  };
};
const unmarshalGrafanaProductDashboard$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GrafanaProductDashboard' failed as data isn't a dictionary.`);
  }
  return {
    name: data.name,
    tags: data.tags,
    title: data.title,
    url: data.url,
    variables: data.variables
  };
};
const unmarshalGrafanaUser$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GrafanaUser' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    login: data.login,
    password: data.password,
    role: data.role
  };
};
const unmarshalPlan$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Plan' failed as data isn't a dictionary.`);
  }
  return {
    logsIngestionPrice: data.logs_ingestion_price,
    monthlyPrice: data.monthly_price,
    name: data.name,
    retentionLogsInterval: data.retention_logs_interval,
    retentionMetricsInterval: data.retention_metrics_interval,
    retentionTracesInterval: data.retention_traces_interval,
    sampleIngestionPrice: data.sample_ingestion_price,
    tracesIngestionPrice: data.traces_ingestion_price
  };
};
const unmarshalToken$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Token' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    projectId: data.project_id,
    region: data.region,
    scopes: data.scopes,
    secretKey: data.secret_key,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalAlertManager = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AlertManager' failed as data isn't a dictionary.`);
  }
  return {
    alertManagerEnabled: data.alert_manager_enabled,
    alertManagerUrl: data.alert_manager_url,
    managedAlertsEnabled: data.managed_alerts_enabled,
    region: data.region
  };
};
const unmarshalGrafana = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Grafana' failed as data isn't a dictionary.`);
  }
  return {
    grafanaUrl: data.grafana_url
  };
};
const unmarshalListContactPointsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListContactPointsResponse' failed as data isn't a dictionary.`);
  }
  return {
    contactPoints: unmarshalArrayOfObject(data.contact_points, unmarshalContactPoint$1),
    hasAdditionalContactPoints: data.has_additional_contact_points,
    hasAdditionalReceivers: data.has_additional_receivers,
    totalCount: data.total_count
  };
};
const unmarshalListDataSourcesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDataSourcesResponse' failed as data isn't a dictionary.`);
  }
  return {
    dataSources: unmarshalArrayOfObject(data.data_sources, unmarshalDataSource),
    totalCount: data.total_count
  };
};
const unmarshalListGrafanaProductDashboardsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListGrafanaProductDashboardsResponse' failed as data isn't a dictionary.`);
  }
  return {
    dashboards: unmarshalArrayOfObject(data.dashboards, unmarshalGrafanaProductDashboard$1),
    totalCount: data.total_count
  };
};
const unmarshalListGrafanaUsersResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListGrafanaUsersResponse' failed as data isn't a dictionary.`);
  }
  return {
    grafanaUsers: unmarshalArrayOfObject(data.grafana_users, unmarshalGrafanaUser$1),
    totalCount: data.total_count
  };
};
const unmarshalListPlansResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPlansResponse' failed as data isn't a dictionary.`);
  }
  return {
    plans: unmarshalArrayOfObject(data.plans, unmarshalPlan$1),
    totalCount: data.total_count
  };
};
const unmarshalListTokensResponse$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTokensResponse' failed as data isn't a dictionary.`);
  }
  return {
    tokens: unmarshalArrayOfObject(data.tokens, unmarshalToken$3),
    totalCount: data.total_count
  };
};
const unmarshalUsage = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Usage' failed as data isn't a dictionary.`);
  }
  return {
    dataSourceId: data.data_source_id,
    dataSourceOrigin: data.data_source_origin,
    dataSourceType: data.data_source_type,
    interval: data.interval,
    projectId: data.project_id,
    quantityOverInterval: data.quantity_over_interval,
    region: data.region,
    unit: data.unit
  };
};
const unmarshalUsageOverview = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UsageOverview' failed as data isn't a dictionary.`);
  }
  return {
    externalLogsUsage: data.external_logs_usage ? unmarshalUsage(data.external_logs_usage) : undefined,
    externalMetricsUsage: data.external_metrics_usage ? unmarshalUsage(data.external_metrics_usage) : undefined,
    externalTracesUsage: data.external_traces_usage ? unmarshalUsage(data.external_traces_usage) : undefined,
    scalewayLogsUsage: data.scaleway_logs_usage ? unmarshalUsage(data.scaleway_logs_usage) : undefined,
    scalewayMetricsUsage: data.scaleway_metrics_usage ? unmarshalUsage(data.scaleway_metrics_usage) : undefined
  };
};
const marshalGlobalApiCreateGrafanaUserRequest = (request, defaults) => ({
  login: request.login,
  project_id: request.projectId ?? defaults.defaultProjectId,
  role: request.role
});
const marshalGlobalApiResetGrafanaUserPasswordRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalGlobalApiSelectPlanRequest = (request, defaults) => ({
  plan_name: request.planName,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalGlobalApiSyncGrafanaDataSourcesRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalContactPointEmail$1 = (request, defaults) => ({
  to: request.to
});
const marshalRegionalApiCreateContactPointRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId,
  ...resolveOneOf([{
    param: 'email',
    value: request.email !== undefined ? marshalContactPointEmail$1(request.email) : undefined
  }])
});
const marshalRegionalApiCreateDataSourceRequest = (request, defaults) => ({
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId,
  type: request.type
});
const marshalRegionalApiCreateTokenRequest = (request, defaults) => ({
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId,
  token_scopes: request.tokenScopes !== undefined ? request.tokenScopes : undefined
});
const marshalRegionalApiDeleteContactPointRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId,
  ...resolveOneOf([{
    param: 'email',
    value: request.email !== undefined ? marshalContactPointEmail$1(request.email) : undefined
  }])
});
const marshalRegionalApiDisableAlertManagerRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalRegionalApiDisableManagedAlertsRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalRegionalApiEnableAlertManagerRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalRegionalApiEnableManagedAlertsRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalRegionalApiTriggerTestAlertRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$t = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Cockpit Global API.
 *
 * The Cockpit Global API allows you to manage your Cockpit's Grafana and plans.
 */
class GlobalAPI extends API$z {
  /**
   * Get your Cockpit's Grafana. Retrieve information on your Cockpit's Grafana,
   * specified by the ID of the Project the Cockpit belongs to. The output
   * returned displays the URL to access your Cockpit's Grafana.
   *
   * @param request - The request {@link GlobalApiGetGrafanaRequest}
   * @returns A Promise of Grafana
   */
  getGrafana = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/grafana`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalGrafana);

  /**
   * Synchronize Grafana data sources. Trigger the synchronization of all your
   * data sources and the alert manager in the relevant regions. The alert
   * manager will only be synchronized if you have enabled it.
   *
   * @param request - The request {@link GlobalApiSyncGrafanaDataSourcesRequest}
   */
  syncGrafanaDataSources = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalGlobalApiSyncGrafanaDataSourcesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/grafana/sync-data-sources`
  });

  /**
   * Create a Grafana user. Create a Grafana user to connect to your Cockpit's
   * Grafana. Upon creation, your user password displays only once, so make sure
   * that you save it. Each Grafana user is associated with a role: viewer or
   * editor. A viewer can only view dashboards, whereas an editor can create and
   * edit dashboards. Note that the `admin` username is not available for
   * creation.
   *
   * @param request - The request {@link GlobalApiCreateGrafanaUserRequest}
   * @returns A Promise of GrafanaUser
   */
  createGrafanaUser = request => this.client.fetch({
    body: JSON.stringify(marshalGlobalApiCreateGrafanaUserRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/grafana/users`
  }, unmarshalGrafanaUser$1);
  pageOfListGrafanaUsers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/grafana/users`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListGrafanaUsersResponse$1);

  /**
   * List Grafana users. List all Grafana users created in your Cockpit's
   * Grafana. By default, the Grafana users returned in the list are ordered in
   * ascending order.
   *
   * @param request - The request {@link GlobalApiListGrafanaUsersRequest}
   * @returns A Promise of ListGrafanaUsersResponse
   */
  listGrafanaUsers = (request = {}) => enrichForPagination('grafanaUsers', this.pageOfListGrafanaUsers, request);

  /**
   * Delete a Grafana user. Delete a Grafana user from your Cockpit's Grafana,
   * specified by the ID of the Project the Cockpit belongs to, and the ID of
   * the Grafana user.
   *
   * @param request - The request {@link GlobalApiDeleteGrafanaUserRequest}
   */
  deleteGrafanaUser = request => this.client.fetch({
    method: 'DELETE',
    path: `/cockpit/v1/grafana/users/${validatePathParam('grafanaUserId', request.grafanaUserId)}`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  });

  /**
   * Reset a Grafana user password. Reset the password of a Grafana user,
   * specified by the ID of the Project the Cockpit belongs to, and the ID of
   * the Grafana user. A new password regenerates and only displays once. Make
   * sure that you save it.
   *
   * @param request - The request
   *   {@link GlobalApiResetGrafanaUserPasswordRequest}
   * @returns A Promise of GrafanaUser
   */
  resetGrafanaUserPassword = request => this.client.fetch({
    body: JSON.stringify(marshalGlobalApiResetGrafanaUserPasswordRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/grafana/users/${validatePathParam('grafanaUserId', request.grafanaUserId)}/reset-password`
  }, unmarshalGrafanaUser$1);
  pageOfListGrafanaProductDashboards = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/grafana/product-dashboards`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['tags', request.tags])
  }, unmarshalListGrafanaProductDashboardsResponse$1);

  /**
   * List Scaleway resources dashboards. Retrieve a list of available dashboards
   * in Grafana, for all Scaleway resources which are integrated with Cockpit.
   *
   * @param request - The request
   *   {@link GlobalApiListGrafanaProductDashboardsRequest}
   * @returns A Promise of ListGrafanaProductDashboardsResponse
   */
  listGrafanaProductDashboards = (request = {}) => enrichForPagination('dashboards', this.pageOfListGrafanaProductDashboards, request);

  /**
   * Get Scaleway resource dashboard. Retrieve information about the dashboard
   * of a Scaleway resource in Grafana, specified by the ID of the Project the
   * Cockpit belongs to, and the name of the dashboard.
   *
   * @param request - The request
   *   {@link GlobalApiGetGrafanaProductDashboardRequest}
   * @returns A Promise of GrafanaProductDashboard
   */
  getGrafanaProductDashboard = request => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/grafana/product-dashboards/${validatePathParam('dashboardName', request.dashboardName)}`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalGrafanaProductDashboard$1);
  pageOfListPlans = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/plans`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListPlansResponse$1);

  /**
   * List plan types. Retrieve a list of available pricing plan types.
   *
   * @param request - The request {@link GlobalApiListPlansRequest}
   * @returns A Promise of ListPlansResponse
   */
  listPlans = (request = {}) => enrichForPagination('plans', this.pageOfListPlans, request);

  /**
   * Apply a pricing plan. Apply a pricing plan on a given Project. You must
   * specify the ID of the pricing plan type. Note that you will be billed for
   * the plan you apply.
   *
   * @param request - The request {@link GlobalApiSelectPlanRequest}
   * @returns A Promise of Plan
   */
  selectPlan = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalGlobalApiSelectPlanRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'PATCH',
    path: `/cockpit/v1/plans`
  }, unmarshalPlan$1);

  /**
   * Get current plan. Retrieve a pricing plan for the given Project, specified
   * by the ID of the Project.
   *
   * @param request - The request {@link GlobalApiGetCurrentPlanRequest}
   * @returns A Promise of Plan
   */
  getCurrentPlan = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/current-plan`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalPlan$1);
}

/**
 * Cockpit Regional API.
 *
 * The Cockpit Regional API allows you to create data sources and tokens to
 * store and query data types such as metrics, logs, and traces. You can also
 * push your data into Cockpit, and send alerts to your contact points when your
 * resources may require your attention, using the regional Alert manager.
 */
class RegionalAPI extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];

  /**
   * Create a data source. You must specify the data source type upon creation.
   * Available data source types include:
   *
   * - Metrics
   * - Logs
   * - Traces The name of the data source will then be used as reference to name
   *   the associated Grafana data source.
   *
   * @param request - The request {@link RegionalApiCreateDataSourceRequest}
   * @returns A Promise of DataSource
   */
  createDataSource = request => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiCreateDataSourceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/data-sources`
  }, unmarshalDataSource);

  /**
   * Get a data source. Retrieve information about a given data source,
   * specified by the data source ID. The data source's information such as its
   * name, type, URL, origin, and retention period, is returned.
   *
   * @param request - The request {@link RegionalApiGetDataSourceRequest}
   * @returns A Promise of DataSource
   */
  getDataSource = request => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/data-sources/${validatePathParam('dataSourceId', request.dataSourceId)}`
  }, unmarshalDataSource);

  /**
   * Delete a data source. Delete a given data source, specified by the data
   * source ID. Note that deleting a data source is irreversible, and cannot be
   * undone.
   *
   * @param request - The request {@link RegionalApiDeleteDataSourceRequest}
   */
  deleteDataSource = request => this.client.fetch({
    method: 'DELETE',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/data-sources/${validatePathParam('dataSourceId', request.dataSourceId)}`
  });
  pageOfListDataSources = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/data-sources`,
    urlParams: urlParams(['order_by', request.orderBy], ['origin', request.origin], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['types', request.types])
  }, unmarshalListDataSourcesResponse);

  /**
   * List data sources. Retrieve the list of data sources available in the
   * specified region. By default, the data sources returned in the list are
   * ordered by creation date, in ascending order. You can list data sources by
   * Project, type and origin.
   *
   * @param request - The request {@link RegionalApiListDataSourcesRequest}
   * @returns A Promise of ListDataSourcesResponse
   */
  listDataSources = (request = {}) => enrichForPagination('dataSources', this.pageOfListDataSources, request);

  /**
   * Get data source usage overview. Retrieve the data source usage overview per
   * type for the specified Project.
   *
   * @param request - The request {@link RegionalApiGetUsageOverviewRequest}
   * @returns A Promise of UsageOverview
   */
  getUsageOverview = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/usage-overview`,
    urlParams: urlParams(['interval', request.interval], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalUsageOverview);

  /**
   * Create a token. Give your token the relevant scopes to ensure it has the
   * right permissions to interact with your data sources and the Alert manager.
   * Make sure that you create your token in the same regions as the data
   * sources you want to use it for. Upon creation, your token's secret key
   * display only once. Make sure that you save it.
   *
   * @param request - The request {@link RegionalApiCreateTokenRequest}
   * @returns A Promise of Token
   */
  createToken = request => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiCreateTokenRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens`
  }, unmarshalToken$3);
  pageOfListTokens = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['token_scopes', request.tokenScopes])
  }, unmarshalListTokensResponse$3);

  /**
   * List tokens. Retrieve a list of all tokens in the specified region. By
   * default, tokens returned in the list are ordered by creation date, in
   * ascending order. You can filter tokens by Project ID and token scopes.
   *
   * @param request - The request {@link RegionalApiListTokensRequest}
   * @returns A Promise of ListTokensResponse
   */
  listTokens = (request = {}) => enrichForPagination('tokens', this.pageOfListTokens, request);

  /**
   * Get a token. Retrieve information about a given token, specified by the
   * token ID. The token's information such as its scopes, is returned.
   *
   * @param request - The request {@link RegionalApiGetTokenRequest}
   * @returns A Promise of Token
   */
  getToken = request => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens/${validatePathParam('tokenId', request.tokenId)}`
  }, unmarshalToken$3);

  /**
   * Delete a token. Delete a given token, specified by the token ID. Deleting a
   * token is irreversible and cannot be undone.
   *
   * @param request - The request {@link RegionalApiDeleteTokenRequest}
   */
  deleteToken = request => this.client.fetch({
    method: 'DELETE',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens/${validatePathParam('tokenId', request.tokenId)}`
  });

  /**
   * Get the Alert manager. Retrieve information about the Alert manager which
   * is unique per Project and region. By default the Alert manager is disabled.
   * The output returned displays a URL to access the Alert manager, and whether
   * the Alert manager and managed alerts are enabled.
   *
   * @param request - The request {@link RegionalApiGetAlertManagerRequest}
   * @returns A Promise of AlertManager
   */
  getAlertManager = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalAlertManager);

  /**
   * Enable the Alert manager. Enabling the Alert manager allows you to enable
   * managed alerts and create contact points in the specified Project and
   * region, to be notified when your Scaleway resources may require your
   * attention.
   *
   * @param request - The request {@link RegionalApiEnableAlertManagerRequest}
   * @returns A Promise of AlertManager
   */
  enableAlertManager = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiEnableAlertManagerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager/enable`
  }, unmarshalAlertManager);

  /**
   * Disable the Alert manager. Disabling the Alert manager deletes the contact
   * points you have created and disables managed alerts in the specified
   * Project and region.
   *
   * @param request - The request {@link RegionalApiDisableAlertManagerRequest}
   * @returns A Promise of AlertManager
   */
  disableAlertManager = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiDisableAlertManagerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager/disable`
  }, unmarshalAlertManager);

  /**
   * Create a contact point. Contact points are email addresses associated with
   * the default receiver, that the Alert manager sends alerts to. The source of
   * the alerts are data sources within the same Project and region as the Alert
   * manager. If you need to receive alerts for other receivers, you can create
   * additional contact points and receivers in Grafana. Make sure that you
   * select the Scaleway Alert manager.
   *
   * @param request - The request {@link RegionalApiCreateContactPointRequest}
   * @returns A Promise of ContactPoint
   */
  createContactPoint = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiCreateContactPointRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager/contact-points`
  }, unmarshalContactPoint$1);
  pageOfListContactPoints = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager/contact-points`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListContactPointsResponse$1);

  /**
   * List contact points. Retrieve a list of contact points for the specified
   * Project. The response lists all contact points and receivers created in
   * Grafana or via the API.
   *
   * @param request - The request {@link RegionalApiListContactPointsRequest}
   * @returns A Promise of ListContactPointsResponse
   */
  listContactPoints = (request = {}) => enrichForPagination('contactPoints', this.pageOfListContactPoints, request);

  /**
   * Delete a contact point. Delete a contact point associated with the default
   * receiver.
   *
   * @param request - The request {@link RegionalApiDeleteContactPointRequest}
   */
  deleteContactPoint = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiDeleteContactPointRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager/contact-points/delete`
  });

  /**
   * Enable managed alerts. Enable the sending of managed alerts for the
   * specified Project. Managed alerts are predefined alerts that apply to
   * Scaleway recources integrated with Cockpit by default.
   *
   * @param request - The request {@link RegionalApiEnableManagedAlertsRequest}
   * @returns A Promise of AlertManager
   */
  enableManagedAlerts = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiEnableManagedAlertsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager/managed-alerts/enable`
  }, unmarshalAlertManager);

  /**
   * Disable managed alerts. Disable the sending of managed alerts for the
   * specified Project.
   *
   * @param request - The request {@link RegionalApiDisableManagedAlertsRequest}
   * @returns A Promise of AlertManager
   */
  disableManagedAlerts = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiDisableManagedAlertsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager/managed-alerts/disable`
  }, unmarshalAlertManager);

  /**
   * Trigger a test alert. Send a test alert to the Alert manager to make sure
   * your contact points get notified.
   *
   * @param request - The request {@link RegionalApiTriggerTestAlertRequest}
   */
  triggerTestAlert = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalRegionalApiTriggerTestAlertRequest(request, this.client.settings)),
    headers: jsonContentHeaders$t,
    method: 'POST',
    path: `/cockpit/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/alert-manager/trigger-test-alert`
  });
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const RegionalApiCreateDataSourceRequest = {
  name: {
    maxLength: 50,
    minLength: 3,
    pattern: /^[A-Za-z0-9-_. ]+$/
  }
};
const RegionalApiCreateTokenRequest = {
  name: {
    maxLength: 50,
    minLength: 3,
    pattern: /^[A-Za-z0-9-_]+$/
  }
};
const RegionalApiListContactPointsRequest = {
  page: {
    greaterThanOrEqual: 1
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 1000
  }
};
const RegionalApiListDataSourcesRequest = {
  page: {
    greaterThanOrEqual: 1
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 1000
  }
};
const RegionalApiListTokensRequest = {
  page: {
    greaterThanOrEqual: 1
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 1000
  }
};

var validationRules_gen$f = /*#__PURE__*/Object.freeze({
  __proto__: null,
  RegionalApiCreateDataSourceRequest: RegionalApiCreateDataSourceRequest,
  RegionalApiCreateTokenRequest: RegionalApiCreateTokenRequest,
  RegionalApiListContactPointsRequest: RegionalApiListContactPointsRequest,
  RegionalApiListDataSourcesRequest: RegionalApiListDataSourcesRequest,
  RegionalApiListTokensRequest: RegionalApiListTokensRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$s = /*#__PURE__*/Object.freeze({
  __proto__: null,
  GlobalAPI: GlobalAPI,
  RegionalAPI: RegionalAPI,
  ValidationRules: validationRules_gen$f
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link CockpitStatus}. */
const COCKPIT_TRANSIENT_STATUSES = ['creating', 'deleting', 'updating'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalContactPointEmail = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactPointEmail' failed as data isn't a dictionary.`);
  }
  return {
    to: data.to
  };
};
const unmarshalContactPoint = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactPoint' failed as data isn't a dictionary.`);
  }
  return {
    email: data.email ? unmarshalContactPointEmail(data.email) : undefined
  };
};
const unmarshalDatasource = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Datasource' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    isManagedByScaleway: data.is_managed_by_scaleway,
    name: data.name,
    projectId: data.project_id,
    type: data.type,
    url: data.url
  };
};
const unmarshalGrafanaProductDashboard = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GrafanaProductDashboard' failed as data isn't a dictionary.`);
  }
  return {
    dashboardName: data.dashboard_name,
    tags: data.tags,
    title: data.title,
    url: data.url,
    variables: data.variables
  };
};
const unmarshalGrafanaUser = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GrafanaUser' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    login: data.login,
    password: data.password,
    role: data.role
  };
};
const unmarshalTokenScopes = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TokenScopes' failed as data isn't a dictionary.`);
  }
  return {
    queryLogs: data.query_logs,
    queryMetrics: data.query_metrics,
    queryTraces: data.query_traces,
    setupAlerts: data.setup_alerts,
    setupLogsRules: data.setup_logs_rules,
    setupMetricsRules: data.setup_metrics_rules,
    writeLogs: data.write_logs,
    writeMetrics: data.write_metrics,
    writeTraces: data.write_traces
  };
};
const unmarshalToken$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Token' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    projectId: data.project_id,
    scopes: data.scopes ? unmarshalTokenScopes(data.scopes) : undefined,
    secretKey: data.secret_key,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalCockpitEndpoints = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CockpitEndpoints' failed as data isn't a dictionary.`);
  }
  return {
    alertmanagerUrl: data.alertmanager_url,
    grafanaUrl: data.grafana_url,
    logsUrl: data.logs_url,
    metricsUrl: data.metrics_url,
    tracesUrl: data.traces_url
  };
};
const unmarshalPlan = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Plan' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    logsIngestionPrice: data.logs_ingestion_price,
    name: data.name,
    retentionLogsInterval: data.retention_logs_interval,
    retentionMetricsInterval: data.retention_metrics_interval,
    retentionPrice: data.retention_price,
    retentionTracesInterval: data.retention_traces_interval,
    sampleIngestionPrice: data.sample_ingestion_price,
    tracesIngestionPrice: data.traces_ingestion_price
  };
};
const unmarshalCockpit = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Cockpit' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    endpoints: data.endpoints ? unmarshalCockpitEndpoints(data.endpoints) : undefined,
    managedAlertsEnabled: data.managed_alerts_enabled,
    plan: data.plan ? unmarshalPlan(data.plan) : undefined,
    projectId: data.project_id,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalCockpitMetrics = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CockpitMetrics' failed as data isn't a dictionary.`);
  }
  return {
    timeseries: unmarshalArrayOfObject(data.timeseries, unmarshalTimeSeries)
  };
};
const unmarshalListContactPointsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListContactPointsResponse' failed as data isn't a dictionary.`);
  }
  return {
    contactPoints: unmarshalArrayOfObject(data.contact_points, unmarshalContactPoint),
    hasAdditionalContactPoints: data.has_additional_contact_points,
    hasAdditionalReceivers: data.has_additional_receivers,
    totalCount: data.total_count
  };
};
const unmarshalListDatasourcesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDatasourcesResponse' failed as data isn't a dictionary.`);
  }
  return {
    datasources: unmarshalArrayOfObject(data.datasources, unmarshalDatasource),
    totalCount: data.total_count
  };
};
const unmarshalListGrafanaProductDashboardsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListGrafanaProductDashboardsResponse' failed as data isn't a dictionary.`);
  }
  return {
    dashboards: unmarshalArrayOfObject(data.dashboards, unmarshalGrafanaProductDashboard),
    totalCount: data.total_count
  };
};
const unmarshalListGrafanaUsersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListGrafanaUsersResponse' failed as data isn't a dictionary.`);
  }
  return {
    grafanaUsers: unmarshalArrayOfObject(data.grafana_users, unmarshalGrafanaUser),
    totalCount: data.total_count
  };
};
const unmarshalListPlansResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPlansResponse' failed as data isn't a dictionary.`);
  }
  return {
    plans: unmarshalArrayOfObject(data.plans, unmarshalPlan),
    totalCount: data.total_count
  };
};
const unmarshalListTokensResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTokensResponse' failed as data isn't a dictionary.`);
  }
  return {
    tokens: unmarshalArrayOfObject(data.tokens, unmarshalToken$2),
    totalCount: data.total_count
  };
};
const unmarshalSelectPlanResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SelectPlanResponse' failed as data isn't a dictionary.`);
  }
  return {};
};
const marshalActivateCockpitRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalContactPointEmail = (request, defaults) => ({
  to: request.to
});
const marshalContactPoint = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'email',
    value: request.email !== undefined ? marshalContactPointEmail(request.email) : undefined
  }])
});
const marshalCreateContactPointRequest = (request, defaults) => ({
  contact_point: request.contactPoint !== undefined ? marshalContactPoint(request.contactPoint) : undefined,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalCreateDatasourceRequest = (request, defaults) => ({
  is_default: request.isDefault,
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId,
  type: request.type
});
const marshalCreateGrafanaUserRequest = (request, defaults) => ({
  login: request.login,
  project_id: request.projectId ?? defaults.defaultProjectId,
  role: request.role
});
const marshalTokenScopes = (request, defaults) => ({
  query_logs: request.queryLogs,
  query_metrics: request.queryMetrics,
  query_traces: request.queryTraces,
  setup_alerts: request.setupAlerts,
  setup_logs_rules: request.setupLogsRules,
  setup_metrics_rules: request.setupMetricsRules,
  write_logs: request.writeLogs,
  write_metrics: request.writeMetrics,
  write_traces: request.writeTraces
});
const marshalCreateTokenRequest$2 = (request, defaults) => ({
  name: request.name || randomName('token'),
  project_id: request.projectId ?? defaults.defaultProjectId,
  scopes: request.scopes !== undefined ? marshalTokenScopes(request.scopes) : undefined
});
const marshalDeactivateCockpitRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalDeleteContactPointRequest = (request, defaults) => ({
  contact_point: request.contactPoint !== undefined ? marshalContactPoint(request.contactPoint) : undefined,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalDeleteGrafanaUserRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalDisableManagedAlertsRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalEnableManagedAlertsRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalResetGrafanaUserPasswordRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalSelectPlanRequest = (request, defaults) => ({
  plan_id: request.planId,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalTriggerTestAlertRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$s = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Cockpit API.
 *
 * The Cockpit API allows you to activate your Cockpit to store metrics and
 * logs. It also provides you with a dedicated Grafana for dashboarding to
 * visualize your metrics and logs.
 */
let API$s = class API extends API$z {
  /**
   * Activate the Cockpit of a given Project specified by the Project ID.
   *
   * @param request - The request {@link ActivateCockpitRequest}
   * @returns A Promise of Cockpit
   */
  activateCockpit = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalActivateCockpitRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/activate`
  }, unmarshalCockpit);

  /**
   * Retrieve the Cockpit of a given Project specified by the Project ID.
   *
   * @param request - The request {@link GetCockpitRequest}
   * @returns A Promise of Cockpit
   */
  getCockpit = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/cockpit`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalCockpit);

  /**
   * Waits for {@link Cockpit} to be in a final state.
   *
   * @param request - The request {@link GetCockpitRequest}
   * @param options - The waiting options
   * @returns A Promise of Cockpit
   */
  waitForCockpit = (request = {}, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!COCKPIT_TRANSIENT_STATUSES.includes(res.status))), this.getCockpit, request, options);

  /**
   * Retrieve metrics from your Cockpit specified by the ID of the Project the
   * Cockpit belongs to.
   *
   * @param request - The request {@link GetCockpitMetricsRequest}
   * @returns A Promise of CockpitMetrics
   */
  getCockpitMetrics = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/cockpit/metrics`,
    urlParams: urlParams(['end_date', request.endDate], ['metric_name', request.metricName], ['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['start_date', request.startDate])
  }, unmarshalCockpitMetrics);

  /**
   * Deactivate the Cockpit of a given Project specified by the Project ID.
   *
   * @param request - The request {@link DeactivateCockpitRequest}
   * @returns A Promise of Cockpit
   */
  deactivateCockpit = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalDeactivateCockpitRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/deactivate`
  }, unmarshalCockpit);

  /**
   * Create a data source for a given Project specified by the Project ID and
   * the data source type.
   *
   * @param request - The request {@link CreateDatasourceRequest}
   * @returns A Promise of Datasource
   */
  createDatasource = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDatasourceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/datasources`
  }, unmarshalDatasource);

  /**
   * Delete a given data source specified by the data source ID.
   *
   * @param request - The request {@link DeleteDatasourceRequest}
   */
  deleteDatasource = request => this.client.fetch({
    method: 'DELETE',
    path: `/cockpit/v1beta1/datasources/${validatePathParam('datasourceId', request.datasourceId)}`
  });
  pageOfListDatasources = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/datasources`,
    urlParams: urlParams(['is_managed_by_scaleway', request.isManagedByScaleway], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['types', request.types])
  }, unmarshalListDatasourcesResponse);

  /**
   * Get a list of data sources for the specified Project ID.
   *
   * @param request - The request {@link ListDatasourcesRequest}
   * @returns A Promise of ListDatasourcesResponse
   */
  listDatasources = (request = {}) => enrichForPagination('datasources', this.pageOfListDatasources, request);

  /**
   * Create a token in a given Project specified by the Project ID.
   *
   * @param request - The request {@link CreateTokenRequest}
   * @returns A Promise of Token
   */
  createToken = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateTokenRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/tokens`
  }, unmarshalToken$2);
  pageOfListTokens = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/tokens`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListTokensResponse$2);

  /**
   * Get a list of tokens in a given Project specified by the Project ID.
   *
   * @param request - The request {@link ListTokensRequest}
   * @returns A Promise of ListTokensResponse
   */
  listTokens = (request = {}) => enrichForPagination('tokens', this.pageOfListTokens, request);

  /**
   * Retrieve a given token specified by the token ID.
   *
   * @param request - The request {@link GetTokenRequest}
   * @returns A Promise of Token
   */
  getToken = request => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/tokens/${validatePathParam('tokenId', request.tokenId)}`
  }, unmarshalToken$2);

  /**
   * Delete a given token specified by the token ID.
   *
   * @param request - The request {@link DeleteTokenRequest}
   */
  deleteToken = request => this.client.fetch({
    method: 'DELETE',
    path: `/cockpit/v1beta1/tokens/${validatePathParam('tokenId', request.tokenId)}`
  });

  /**
   * Create a contact point associated with the default receiver, to receive
   * alerts.
   *
   * @param request - The request {@link CreateContactPointRequest}
   * @returns A Promise of ContactPoint
   */
  createContactPoint = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateContactPointRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/contact-points`
  }, unmarshalContactPoint);
  pageOfListContactPoints = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/contact-points`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListContactPointsResponse);

  /**
   * Get a list of contact points created for a given Cockpit, specified by the
   * ID of the Project the Cockpit belongs to.
   *
   * @param request - The request {@link ListContactPointsRequest}
   * @returns A Promise of ListContactPointsResponse
   */
  listContactPoints = (request = {}) => enrichForPagination('contactPoints', this.pageOfListContactPoints, request);

  /**
   * Delete a contact point associated with the default receiver.
   *
   * @param request - The request {@link DeleteContactPointRequest}
   */
  deleteContactPoint = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalDeleteContactPointRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/delete-contact-point`
  });

  /**
   * Enable the sending of managed alerts for a given Cockpit, specified by the
   * ID of the Project the Cockpit belongs to.
   *
   * @param request - The request {@link EnableManagedAlertsRequest}
   */
  enableManagedAlerts = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalEnableManagedAlertsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/enable-managed-alerts`
  });

  /**
   * Disable the sending of managed alerts for a given Cockpit, specified by the
   * ID of the Project the Cockpit belongs to.
   *
   * @param request - The request {@link DisableManagedAlertsRequest}
   */
  disableManagedAlerts = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalDisableManagedAlertsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/disable-managed-alerts`
  });

  /**
   * Send a test alert to make sure your contact points get notified when an
   * actual alert is triggered.
   *
   * @param request - The request {@link TriggerTestAlertRequest}
   */
  triggerTestAlert = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalTriggerTestAlertRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/trigger-test-alert`
  });

  /**
   * Create a Grafana user for your Cockpit's Grafana. Make sure you save the
   * automatically-generated password and the Grafana user ID.
   *
   * @param request - The request {@link CreateGrafanaUserRequest}
   * @returns A Promise of GrafanaUser
   */
  createGrafanaUser = request => this.client.fetch({
    body: JSON.stringify(marshalCreateGrafanaUserRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/grafana-users`
  }, unmarshalGrafanaUser);
  pageOfListGrafanaUsers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/grafana-users`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListGrafanaUsersResponse);

  /**
   * Get a list of all Grafana users created in your Cockpit's Grafana.
   *
   * @param request - The request {@link ListGrafanaUsersRequest}
   * @returns A Promise of ListGrafanaUsersResponse
   */
  listGrafanaUsers = (request = {}) => enrichForPagination('grafanaUsers', this.pageOfListGrafanaUsers, request);

  /**
   * Delete a Grafana user from your Cockpit's Grafana, specified by the ID of
   * the Project the Cockpit belongs to, and the ID of the Grafana user.
   *
   * @param request - The request {@link DeleteGrafanaUserRequest}
   */
  deleteGrafanaUser = request => this.client.fetch({
    body: JSON.stringify(marshalDeleteGrafanaUserRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/grafana-users/${validatePathParam('grafanaUserId', request.grafanaUserId)}/delete`
  });

  /**
   * Reset the password of a Grafana user, specified by the ID of the Project
   * the Cockpit belongs to, and the ID of the Grafana user.
   *
   * @param request - The request {@link ResetGrafanaUserPasswordRequest}
   * @returns A Promise of GrafanaUser
   */
  resetGrafanaUserPassword = request => this.client.fetch({
    body: JSON.stringify(marshalResetGrafanaUserPasswordRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/grafana-users/${validatePathParam('grafanaUserId', request.grafanaUserId)}/reset-password`
  }, unmarshalGrafanaUser);
  pageOfListPlans = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/plans`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListPlansResponse);

  /**
   * Get a list of all pricing plans available.
   *
   * @param request - The request {@link ListPlansRequest}
   * @returns A Promise of ListPlansResponse
   */
  listPlans = (request = {}) => enrichForPagination('plans', this.pageOfListPlans, request);

  /**
   * Select your chosen pricing plan for your Cockpit, specifying the Cockpit's
   * Project ID and the pricing plan's ID in the request.
   *
   * @param request - The request {@link SelectPlanRequest}
   * @returns A Promise of SelectPlanResponse
   */
  selectPlan = request => this.client.fetch({
    body: JSON.stringify(marshalSelectPlanRequest(request, this.client.settings)),
    headers: jsonContentHeaders$s,
    method: 'POST',
    path: `/cockpit/v1beta1/select-plan`
  }, unmarshalSelectPlanResponse);
  pageOfListGrafanaProductDashboards = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/grafana-product-dashboards`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['tags', request.tags])
  }, unmarshalListGrafanaProductDashboardsResponse);

  /**
   * List product dashboards. Get a list of available product dashboards.
   *
   * @param request - The request {@link ListGrafanaProductDashboardsRequest}
   * @returns A Promise of ListGrafanaProductDashboardsResponse
   */
  listGrafanaProductDashboards = (request = {}) => enrichForPagination('dashboards', this.pageOfListGrafanaProductDashboards, request);

  /**
   * Get a product dashboard. Get a product dashboard specified by the dashboard
   * ID.
   *
   * @param request - The request {@link GetGrafanaProductDashboardRequest}
   * @returns A Promise of GrafanaProductDashboard
   */
  getGrafanaProductDashboard = request => this.client.fetch({
    method: 'GET',
    path: `/cockpit/v1beta1/grafana-product-dashboards/${validatePathParam('dashboardName', request.dashboardName)}`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalGrafanaProductDashboard);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$r = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$s,
  COCKPIT_TRANSIENT_STATUSES: COCKPIT_TRANSIENT_STATUSES
});

var index$u = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$s,
  v1beta1: index_gen$r
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link ContainerStatus}. */
const CONTAINER_TRANSIENT_STATUSES = ['deleting', 'creating', 'pending'];

/** Lists transient statutes of the enum {@link CronStatus}. */
const CRON_TRANSIENT_STATUSES$1 = ['deleting', 'creating', 'pending'];

/** Lists transient statutes of the enum {@link DomainStatus}. */
const DOMAIN_TRANSIENT_STATUSES$3 = ['deleting', 'creating', 'pending'];

/** Lists transient statutes of the enum {@link NamespaceStatus}. */
const NAMESPACE_TRANSIENT_STATUSES$2 = ['deleting', 'creating', 'pending'];

/** Lists transient statutes of the enum {@link TokenStatus}. */
const TOKEN_TRANSIENT_STATUSES$1 = ['deleting', 'creating'];

/** Lists transient statutes of the enum {@link TriggerStatus}. */
const TRIGGER_TRANSIENT_STATUSES$1 = ['deleting', 'creating', 'pending'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalSecretHashedValue$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SecretHashedValue' failed as data isn't a dictionary.`);
  }
  return {
    hashedValue: data.hashed_value,
    key: data.key
  };
};
const unmarshalContainer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Container' failed as data isn't a dictionary.`);
  }
  return {
    cpuLimit: data.cpu_limit,
    description: data.description,
    domainName: data.domain_name,
    environmentVariables: data.environment_variables,
    errorMessage: data.error_message,
    httpOption: data.http_option,
    id: data.id,
    maxConcurrency: data.max_concurrency,
    maxScale: data.max_scale,
    memoryLimit: data.memory_limit,
    minScale: data.min_scale,
    name: data.name,
    namespaceId: data.namespace_id,
    port: data.port,
    privacy: data.privacy,
    protocol: data.protocol,
    region: data.region,
    registryImage: data.registry_image,
    secretEnvironmentVariables: unmarshalArrayOfObject(data.secret_environment_variables, unmarshalSecretHashedValue$1),
    status: data.status,
    timeout: data.timeout
  };
};
const unmarshalCron$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Cron' failed as data isn't a dictionary.`);
  }
  return {
    args: data.args,
    containerId: data.container_id,
    id: data.id,
    name: data.name,
    schedule: data.schedule,
    status: data.status
  };
};
const unmarshalDomain$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Domain' failed as data isn't a dictionary.`);
  }
  return {
    containerId: data.container_id,
    errorMessage: data.error_message,
    hostname: data.hostname,
    id: data.id,
    status: data.status,
    url: data.url
  };
};
const unmarshalNamespace$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Namespace' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    environmentVariables: data.environment_variables,
    errorMessage: data.error_message,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    region: data.region,
    registryEndpoint: data.registry_endpoint,
    registryNamespaceId: data.registry_namespace_id,
    secretEnvironmentVariables: unmarshalArrayOfObject(data.secret_environment_variables, unmarshalSecretHashedValue$1),
    status: data.status
  };
};
const unmarshalToken$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Token' failed as data isn't a dictionary.`);
  }
  return {
    containerId: data.container_id,
    description: data.description,
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    namespaceId: data.namespace_id,
    publicKey: data.public_key,
    status: data.status,
    token: data.token
  };
};
const unmarshalTriggerMnqNatsClientConfig$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TriggerMnqNatsClientConfig' failed as data isn't a dictionary.`);
  }
  return {
    mnqCredentialId: data.mnq_credential_id,
    mnqNatsAccountId: data.mnq_nats_account_id,
    mnqProjectId: data.mnq_project_id,
    mnqRegion: data.mnq_region,
    subject: data.subject
  };
};
const unmarshalTriggerMnqSqsClientConfig$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TriggerMnqSqsClientConfig' failed as data isn't a dictionary.`);
  }
  return {
    mnqCredentialId: data.mnq_credential_id,
    mnqProjectId: data.mnq_project_id,
    mnqRegion: data.mnq_region,
    queue: data.queue
  };
};
const unmarshalTriggerSqsClientConfig$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TriggerSqsClientConfig' failed as data isn't a dictionary.`);
  }
  return {
    accessKey: data.access_key,
    endpoint: data.endpoint,
    queueUrl: data.queue_url,
    secretKey: data.secret_key
  };
};
const unmarshalTrigger$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Trigger' failed as data isn't a dictionary.`);
  }
  return {
    containerId: data.container_id,
    description: data.description,
    errorMessage: data.error_message,
    id: data.id,
    inputType: data.input_type,
    name: data.name,
    scwNatsConfig: data.scw_nats_config ? unmarshalTriggerMnqNatsClientConfig$1(data.scw_nats_config) : undefined,
    scwSqsConfig: data.scw_sqs_config ? unmarshalTriggerMnqSqsClientConfig$1(data.scw_sqs_config) : undefined,
    sqsConfig: data.sqs_config ? unmarshalTriggerSqsClientConfig$1(data.sqs_config) : undefined,
    status: data.status
  };
};
const unmarshalListContainersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListContainersResponse' failed as data isn't a dictionary.`);
  }
  return {
    containers: unmarshalArrayOfObject(data.containers, unmarshalContainer),
    totalCount: data.total_count
  };
};
const unmarshalListCronsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListCronsResponse' failed as data isn't a dictionary.`);
  }
  return {
    crons: unmarshalArrayOfObject(data.crons, unmarshalCron$1),
    totalCount: data.total_count
  };
};
const unmarshalListDomainsResponse$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary.`);
  }
  return {
    domains: unmarshalArrayOfObject(data.domains, unmarshalDomain$3),
    totalCount: data.total_count
  };
};
const unmarshalListNamespacesResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNamespacesResponse' failed as data isn't a dictionary.`);
  }
  return {
    namespaces: unmarshalArrayOfObject(data.namespaces, unmarshalNamespace$2),
    totalCount: data.total_count
  };
};
const unmarshalListTokensResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTokensResponse' failed as data isn't a dictionary.`);
  }
  return {
    tokens: unmarshalArrayOfObject(data.tokens, unmarshalToken$1),
    totalCount: data.total_count
  };
};
const unmarshalListTriggersResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTriggersResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    triggers: unmarshalArrayOfObject(data.triggers, unmarshalTrigger$1)
  };
};
const marshalSecret$1 = (request, defaults) => ({
  key: request.key,
  value: request.value
});
const marshalCreateContainerRequest = (request, defaults) => ({
  cpu_limit: request.cpuLimit,
  description: request.description,
  environment_variables: request.environmentVariables,
  http_option: request.httpOption,
  max_concurrency: request.maxConcurrency,
  max_scale: request.maxScale,
  memory_limit: request.memoryLimit,
  min_scale: request.minScale,
  name: request.name,
  namespace_id: request.namespaceId,
  port: request.port,
  privacy: request.privacy,
  protocol: request.protocol,
  registry_image: request.registryImage,
  secret_environment_variables: request.secretEnvironmentVariables !== undefined ? request.secretEnvironmentVariables.map(elt => marshalSecret$1(elt)) : undefined,
  timeout: request.timeout
});
const marshalCreateCronRequest$1 = (request, defaults) => ({
  args: request.args,
  container_id: request.containerId,
  name: request.name,
  schedule: request.schedule
});
const marshalCreateDomainRequest$2 = (request, defaults) => ({
  container_id: request.containerId,
  hostname: request.hostname
});
const marshalCreateNamespaceRequest$2 = (request, defaults) => ({
  description: request.description,
  environment_variables: request.environmentVariables,
  name: request.name || randomName('cns'),
  project_id: request.projectId ?? defaults.defaultProjectId,
  secret_environment_variables: request.secretEnvironmentVariables !== undefined ? request.secretEnvironmentVariables.map(elt => marshalSecret$1(elt)) : undefined
});
const marshalCreateTokenRequest$1 = (request, defaults) => ({
  description: request.description,
  expires_at: request.expiresAt,
  ...resolveOneOf([{
    param: 'container_id',
    value: request.containerId
  }, {
    param: 'namespace_id',
    value: request.namespaceId
  }])
});
const marshalCreateTriggerRequestMnqNatsClientConfig$1 = (request, defaults) => ({
  mnq_nats_account_id: request.mnqNatsAccountId,
  mnq_project_id: request.mnqProjectId,
  mnq_region: request.mnqRegion,
  subject: request.subject
});
const marshalCreateTriggerRequestMnqSqsClientConfig$1 = (request, defaults) => ({
  mnq_project_id: request.mnqProjectId,
  mnq_region: request.mnqRegion,
  queue: request.queue
});
const marshalCreateTriggerRequestSqsClientConfig$1 = (request, defaults) => ({
  access_key: request.accessKey,
  endpoint: request.endpoint,
  queue_url: request.queueUrl,
  secret_key: request.secretKey
});
const marshalCreateTriggerRequest$1 = (request, defaults) => ({
  container_id: request.containerId,
  description: request.description,
  name: request.name,
  ...resolveOneOf([{
    param: 'scw_sqs_config',
    value: request.scwSqsConfig !== undefined ? marshalCreateTriggerRequestMnqSqsClientConfig$1(request.scwSqsConfig) : undefined
  }, {
    param: 'scw_nats_config',
    value: request.scwNatsConfig !== undefined ? marshalCreateTriggerRequestMnqNatsClientConfig$1(request.scwNatsConfig) : undefined
  }, {
    param: 'sqs_config',
    value: request.sqsConfig !== undefined ? marshalCreateTriggerRequestSqsClientConfig$1(request.sqsConfig) : undefined
  }])
});
const marshalUpdateContainerRequest = (request, defaults) => ({
  cpu_limit: request.cpuLimit,
  description: request.description,
  environment_variables: request.environmentVariables,
  http_option: request.httpOption,
  max_concurrency: request.maxConcurrency,
  max_scale: request.maxScale,
  memory_limit: request.memoryLimit,
  min_scale: request.minScale,
  port: request.port,
  privacy: request.privacy,
  protocol: request.protocol,
  redeploy: request.redeploy,
  registry_image: request.registryImage,
  secret_environment_variables: request.secretEnvironmentVariables !== undefined ? request.secretEnvironmentVariables.map(elt => marshalSecret$1(elt)) : undefined,
  timeout: request.timeout
});
const marshalUpdateCronRequest$1 = (request, defaults) => ({
  args: request.args,
  container_id: request.containerId,
  name: request.name,
  schedule: request.schedule
});
const marshalUpdateNamespaceRequest$2 = (request, defaults) => ({
  description: request.description,
  environment_variables: request.environmentVariables,
  secret_environment_variables: request.secretEnvironmentVariables !== undefined ? request.secretEnvironmentVariables.map(elt => marshalSecret$1(elt)) : undefined
});
const marshalUpdateTriggerRequestSqsClientConfig$1 = (request, defaults) => ({
  access_key: request.accessKey,
  secret_key: request.secretKey
});
const marshalUpdateTriggerRequest$1 = (request, defaults) => ({
  description: request.description,
  name: request.name,
  ...resolveOneOf([{
    param: 'sqs_config',
    value: request.sqsConfig !== undefined ? marshalUpdateTriggerRequestSqsClientConfig$1(request.sqsConfig) : undefined
  }])
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$r = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Serverless Containers API. */
let API$r = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];
  pageOfListNamespaces = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListNamespacesResponse$2);

  /**
   * List all your namespaces. List all namespaces in a specified region.
   *
   * @param request - The request {@link ListNamespacesRequest}
   * @returns A Promise of ListNamespacesResponse
   */
  listNamespaces = (request = {}) => enrichForPagination('namespaces', this.pageOfListNamespaces, request);

  /**
   * Get a namespace. Get the namespace associated with the specified ID.
   *
   * @param request - The request {@link GetNamespaceRequest}
   * @returns A Promise of Namespace
   */
  getNamespace = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace$2);

  /**
   * Waits for {@link Namespace} to be in a final state.
   *
   * @param request - The request {@link GetNamespaceRequest}
   * @param options - The waiting options
   * @returns A Promise of Namespace
   */
  waitForNamespace = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!NAMESPACE_TRANSIENT_STATUSES$2.includes(res.status))), this.getNamespace, request, options);

  /**
   * Create a new namespace. Create a new namespace in a specified region.
   *
   * @param request - The request {@link CreateNamespaceRequest}
   * @returns A Promise of Namespace
   */
  createNamespace = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateNamespaceRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'POST',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces`
  }, unmarshalNamespace$2);

  /**
   * Update an existing namespace. Update the space associated with the
   * specified ID.
   *
   * @param request - The request {@link UpdateNamespaceRequest}
   * @returns A Promise of Namespace
   */
  updateNamespace = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateNamespaceRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'PATCH',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace$2);

  /**
   * Delete an existing namespace. Delete the namespace associated with the
   * specified ID.
   *
   * @param request - The request {@link DeleteNamespaceRequest}
   * @returns A Promise of Namespace
   */
  deleteNamespace = request => this.client.fetch({
    method: 'DELETE',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace$2);
  pageOfListContainers = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/containers`,
    urlParams: urlParams(['name', request.name], ['namespace_id', request.namespaceId], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListContainersResponse);

  /**
   * List all your containers. List all containers for a specified region.
   *
   * @param request - The request {@link ListContainersRequest}
   * @returns A Promise of ListContainersResponse
   */
  listContainers = request => enrichForPagination('containers', this.pageOfListContainers, request);

  /**
   * Get a container. Get the container associated with the specified ID.
   *
   * @param request - The request {@link GetContainerRequest}
   * @returns A Promise of Container
   */
  getContainer = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/containers/${validatePathParam('containerId', request.containerId)}`
  }, unmarshalContainer);

  /**
   * Waits for {@link Container} to be in a final state.
   *
   * @param request - The request {@link GetContainerRequest}
   * @param options - The waiting options
   * @returns A Promise of Container
   */
  waitForContainer = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!CONTAINER_TRANSIENT_STATUSES.includes(res.status))), this.getContainer, request, options);

  /**
   * Create a new container. Create a new container in the specified region.
   *
   * @param request - The request {@link CreateContainerRequest}
   * @returns A Promise of Container
   */
  createContainer = request => this.client.fetch({
    body: JSON.stringify(marshalCreateContainerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'POST',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/containers`
  }, unmarshalContainer);

  /**
   * Update an existing container. Update the container associated with the
   * specified ID.
   *
   * @param request - The request {@link UpdateContainerRequest}
   * @returns A Promise of Container
   */
  updateContainer = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateContainerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'PATCH',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/containers/${validatePathParam('containerId', request.containerId)}`
  }, unmarshalContainer);

  /**
   * Delete a container. Delete the container associated with the specified ID.
   *
   * @param request - The request {@link DeleteContainerRequest}
   * @returns A Promise of Container
   */
  deleteContainer = request => this.client.fetch({
    method: 'DELETE',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/containers/${validatePathParam('containerId', request.containerId)}`
  }, unmarshalContainer);

  /**
   * Deploy a container. Deploy a container associated with the specified ID.
   *
   * @param request - The request {@link DeployContainerRequest}
   * @returns A Promise of Container
   */
  deployContainer = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$r,
    method: 'POST',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/containers/${validatePathParam('containerId', request.containerId)}/deploy`
  }, unmarshalContainer);
  pageOfListCrons = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons`,
    urlParams: urlParams(['container_id', request.containerId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListCronsResponse$1);

  /**
   * List all your crons.
   *
   * @param request - The request {@link ListCronsRequest}
   * @returns A Promise of ListCronsResponse
   */
  listCrons = request => enrichForPagination('crons', this.pageOfListCrons, request);

  /**
   * Get a cron. Get the cron associated with the specified ID.
   *
   * @param request - The request {@link GetCronRequest}
   * @returns A Promise of Cron
   */
  getCron = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons/${validatePathParam('cronId', request.cronId)}`
  }, unmarshalCron$1);

  /**
   * Waits for {@link Cron} to be in a final state.
   *
   * @param request - The request {@link GetCronRequest}
   * @param options - The waiting options
   * @returns A Promise of Cron
   */
  waitForCron = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!CRON_TRANSIENT_STATUSES$1.includes(res.status))), this.getCron, request, options);

  /**
   * Create a new cron.
   *
   * @param request - The request {@link CreateCronRequest}
   * @returns A Promise of Cron
   */
  createCron = request => this.client.fetch({
    body: JSON.stringify(marshalCreateCronRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'POST',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons`
  }, unmarshalCron$1);

  /**
   * Update an existing cron. Update the cron associated with the specified ID.
   *
   * @param request - The request {@link UpdateCronRequest}
   * @returns A Promise of Cron
   */
  updateCron = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateCronRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'PATCH',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons/${validatePathParam('cronId', request.cronId)}`
  }, unmarshalCron$1);

  /**
   * Delete an existing cron. Delete the cron associated with the specified ID.
   *
   * @param request - The request {@link DeleteCronRequest}
   * @returns A Promise of Cron
   */
  deleteCron = request => this.client.fetch({
    method: 'DELETE',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons/${validatePathParam('cronId', request.cronId)}`
  }, unmarshalCron$1);
  pageOfListDomains = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains`,
    urlParams: urlParams(['container_id', request.containerId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDomainsResponse$3);

  /**
   * List all domain name bindings. List all domain name bindings in a specified
   * region.
   *
   * @param request - The request {@link ListDomainsRequest}
   * @returns A Promise of ListDomainsResponse
   */
  listDomains = request => enrichForPagination('domains', this.pageOfListDomains, request);

  /**
   * Get a domain name binding. Get a domain name binding for the container with
   * the specified ID.
   *
   * @param request - The request {@link GetDomainRequest}
   * @returns A Promise of Domain
   */
  getDomain = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domainId', request.domainId)}`
  }, unmarshalDomain$3);

  /**
   * Waits for {@link Domain} to be in a final state.
   *
   * @param request - The request {@link GetDomainRequest}
   * @param options - The waiting options
   * @returns A Promise of Domain
   */
  waitForDomain = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!DOMAIN_TRANSIENT_STATUSES$3.includes(res.status))), this.getDomain, request, options);

  /**
   * Create a domain name binding. Create a domain name binding for the
   * container with the specified ID.
   *
   * @param request - The request {@link CreateDomainRequest}
   * @returns A Promise of Domain
   */
  createDomain = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDomainRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'POST',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains`
  }, unmarshalDomain$3);

  /**
   * Delete a domain name binding. Delete the domain name binding with the
   * specific ID.
   *
   * @param request - The request {@link DeleteDomainRequest}
   * @returns A Promise of Domain
   */
  deleteDomain = request => this.client.fetch({
    method: 'DELETE',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domainId', request.domainId)}`
  }, unmarshalDomain$3);

  /**
   * Create a JWT token. Deprecated in favor of CreateToken.
   *
   * @deprecated
   * @param request - The request {@link IssueJWTRequest}
   * @returns A Promise of Token
   */
  issueJWT = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/issue-jwt`,
    urlParams: urlParams(['expires_at', request.expiresAt], ...Object.entries(resolveOneOf([{
      param: 'container_id',
      value: request.containerId
    }, {
      param: 'namespace_id',
      value: request.namespaceId
    }])))
  }, unmarshalToken$1);

  /**
   * Create a new revocable token.
   *
   * @param request - The request {@link CreateTokenRequest}
   * @returns A Promise of Token
   */
  createToken = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateTokenRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'POST',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens`
  }, unmarshalToken$1);

  /**
   * Get a token. Get a token with a specified ID.
   *
   * @param request - The request {@link GetTokenRequest}
   * @returns A Promise of Token
   */
  getToken = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens/${validatePathParam('tokenId', request.tokenId)}`
  }, unmarshalToken$1);

  /**
   * Waits for {@link Token} to be in a final state.
   *
   * @param request - The request {@link GetTokenRequest}
   * @param options - The waiting options
   * @returns A Promise of Token
   */
  waitForToken = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!TOKEN_TRANSIENT_STATUSES$1.includes(res.status))), this.getToken, request, options);
  pageOfListTokens = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens`,
    urlParams: urlParams(['container_id', request.containerId], ['namespace_id', request.namespaceId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListTokensResponse$1);

  /**
   * List all tokens. List all tokens belonging to a specified Organization or
   * Project.
   *
   * @param request - The request {@link ListTokensRequest}
   * @returns A Promise of ListTokensResponse
   */
  listTokens = (request = {}) => enrichForPagination('tokens', this.pageOfListTokens, request);

  /**
   * Delete a token. Delete a token with a specified ID.
   *
   * @param request - The request {@link DeleteTokenRequest}
   * @returns A Promise of Token
   */
  deleteToken = request => this.client.fetch({
    method: 'DELETE',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens/${validatePathParam('tokenId', request.tokenId)}`
  }, unmarshalToken$1);

  /**
   * Create a trigger. Create a new trigger for a specified container.
   *
   * @param request - The request {@link CreateTriggerRequest}
   * @returns A Promise of Trigger
   */
  createTrigger = request => this.client.fetch({
    body: JSON.stringify(marshalCreateTriggerRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'POST',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers`
  }, unmarshalTrigger$1);

  /**
   * Get a trigger. Get a trigger with a specified ID.
   *
   * @param request - The request {@link GetTriggerRequest}
   * @returns A Promise of Trigger
   */
  getTrigger = request => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers/${validatePathParam('triggerId', request.triggerId)}`
  }, unmarshalTrigger$1);

  /**
   * Waits for {@link Trigger} to be in a final state.
   *
   * @param request - The request {@link GetTriggerRequest}
   * @param options - The waiting options
   * @returns A Promise of Trigger
   */
  waitForTrigger = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!TRIGGER_TRANSIENT_STATUSES$1.includes(res.status))), this.getTrigger, request, options);
  pageOfListTriggers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ...Object.entries(resolveOneOf([{
      param: 'container_id',
      value: request.containerId
    }, {
      param: 'namespace_id',
      value: request.namespaceId
    }, {
      default: this.client.settings.defaultProjectId,
      param: 'project_id',
      value: request.projectId
    }])))
  }, unmarshalListTriggersResponse$1);

  /**
   * List all triggers. List all triggers belonging to a specified Organization
   * or Project.
   *
   * @param request - The request {@link ListTriggersRequest}
   * @returns A Promise of ListTriggersResponse
   */
  listTriggers = (request = {}) => enrichForPagination('triggers', this.pageOfListTriggers, request);

  /**
   * Update a trigger. Update a trigger with a specified ID.
   *
   * @param request - The request {@link UpdateTriggerRequest}
   * @returns A Promise of Trigger
   */
  updateTrigger = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateTriggerRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$r,
    method: 'PATCH',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers/${validatePathParam('triggerId', request.triggerId)}`
  }, unmarshalTrigger$1);

  /**
   * Delete a trigger. Delete a trigger with a specified ID.
   *
   * @param request - The request {@link DeleteTriggerRequest}
   * @returns A Promise of Trigger
   */
  deleteTrigger = request => this.client.fetch({
    method: 'DELETE',
    path: `/containers/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers/${validatePathParam('triggerId', request.triggerId)}`
  }, unmarshalTrigger$1);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateTriggerRequest$1 = {
  description: {
    maxLength: 255
  },
  name: {
    maxLength: 50,
    minLength: 1
  }
};
const CreateTriggerRequestMnqNatsClientConfig$1 = {
  mnqRegion: {
    maxLength: 20,
    minLength: 1
  },
  subject: {
    maxLength: 100,
    minLength: 1,
    pattern: /^[^\s]+$/
  }
};
const CreateTriggerRequestMnqSqsClientConfig$1 = {
  mnqRegion: {
    maxLength: 20,
    minLength: 1
  },
  queue: {
    maxLength: 100,
    minLength: 1
  }
};
const UpdateTriggerRequest$1 = {
  description: {
    maxLength: 255
  },
  name: {
    maxLength: 50,
    minLength: 1
  }
};

var validationRules_gen$e = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateTriggerRequest: CreateTriggerRequest$1,
  CreateTriggerRequestMnqNatsClientConfig: CreateTriggerRequestMnqNatsClientConfig$1,
  CreateTriggerRequestMnqSqsClientConfig: CreateTriggerRequestMnqSqsClientConfig$1,
  UpdateTriggerRequest: UpdateTriggerRequest$1
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$q = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$r,
  CONTAINER_TRANSIENT_STATUSES: CONTAINER_TRANSIENT_STATUSES,
  CRON_TRANSIENT_STATUSES: CRON_TRANSIENT_STATUSES$1,
  DOMAIN_TRANSIENT_STATUSES: DOMAIN_TRANSIENT_STATUSES$3,
  NAMESPACE_TRANSIENT_STATUSES: NAMESPACE_TRANSIENT_STATUSES$2,
  TOKEN_TRANSIENT_STATUSES: TOKEN_TRANSIENT_STATUSES$1,
  TRIGGER_TRANSIENT_STATUSES: TRIGGER_TRANSIENT_STATUSES$1,
  ValidationRules: validationRules_gen$e
});

var index$t = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1beta1: index_gen$q
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link BMCAccessStatus}. */
const BMC_ACCESS_TRANSIENT_STATUSES = ['creating', 'deleting'];

/** Lists transient statutes of the enum {@link IPv6BlockDelegationStatus}. */
const I_PV6_BLOCK_DELEGATION_TRANSIENT_STATUSES = ['updating'];

/** Lists transient statutes of the enum {@link RpnGroupMemberStatus}. */
const RPN_GROUP_MEMBER_TRANSIENT_STATUSES = ['creating', 'deleting'];

/** Lists transient statutes of the enum {@link RpnSanStatus}. */
const RPN_SAN_TRANSIENT_STATUSES = ['creating', 'deleting'];

/** Lists transient statutes of the enum {@link RpnV2GroupStatus}. */
const RPN_V2_GROUP_TRANSIENT_STATUSES = ['creating', 'updating', 'deleting'];

/** Lists transient statutes of the enum {@link RpnV2MemberStatus}. */
const RPN_V2_MEMBER_TRANSIENT_STATUSES = ['creating', 'updating', 'deleting'];

/** Lists transient statutes of the enum {@link ServerInstallStatus}. */
const SERVER_INSTALL_TRANSIENT_STATUSES = ['booting', 'setting_up_raid', 'partitioning', 'formatting', 'installing', 'configuring', 'configuring_bootloader', 'rebooting'];

/** Lists transient statutes of the enum {@link ServerStatus}. */
const SERVER_TRANSIENT_STATUSES$1 = ['delivering', 'installing'];

/** Lists transient statutes of the enum {@link ServiceProvisioningStatus}. */
const SERVICE_PROVISIONING_TRANSIENT_STATUSES = ['delivering', 'expiring'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalIP$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'IP' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    cidr: data.cidr,
    gateway: data.gateway,
    ipId: data.ip_id,
    netmask: data.netmask,
    reverse: data.reverse,
    semantic: data.semantic,
    status: data.status,
    version: data.version
  };
};
const unmarshalCPU = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CPU' failed as data isn't a dictionary.`);
  }
  return {
    coreCount: data.core_count,
    frequency: data.frequency,
    name: data.name,
    threadCount: data.thread_count
  };
};
const unmarshalDisk = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Disk' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    type: data.type
  };
};
const unmarshalMemory = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Memory' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    frequency: data.frequency,
    isEcc: data.is_ecc,
    type: data.type
  };
};
const unmarshalPersistentMemory = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PersistentMemory' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    frequency: data.frequency,
    model: data.model
  };
};
const unmarshalRaidController = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RaidController' failed as data isn't a dictionary.`);
  }
  return {
    model: data.model,
    raidLevel: data.raid_level
  };
};
const unmarshalOfferAntiDosInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferAntiDosInfo' failed as data isn't a dictionary.`);
  }
  return {
    type: data.type
  };
};
const unmarshalOfferBackupInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferBackupInfo' failed as data isn't a dictionary.`);
  }
  return {
    size: data.size
  };
};
const unmarshalOfferBandwidthInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferBandwidthInfo' failed as data isn't a dictionary.`);
  }
  return {
    speed: data.speed
  };
};
const unmarshalOfferFailoverBlockInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferFailoverBlockInfo' failed as data isn't a dictionary.`);
  }
  return {
    onetimeFees: data.onetime_fees ? unmarshalOffer$1(data.onetime_fees) : undefined
  };
};
const unmarshalOfferFailoverIpInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferFailoverIpInfo' failed as data isn't a dictionary.`);
  }
  return {
    onetimeFees: data.onetime_fees ? unmarshalOffer$1(data.onetime_fees) : undefined
  };
};
const unmarshalOfferLicenseInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferLicenseInfo' failed as data isn't a dictionary.`);
  }
  return {
    boundToIp: data.bound_to_ip
  };
};
const unmarshalOfferRPNInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferRPNInfo' failed as data isn't a dictionary.`);
  }
  return {
    speed: data.speed
  };
};
const unmarshalOfferSANInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferSANInfo' failed as data isn't a dictionary.`);
  }
  return {
    deviceType: data.device_type,
    ha: data.ha,
    size: data.size
  };
};
const unmarshalOfferServerInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferServerInfo' failed as data isn't a dictionary.`);
  }
  return {
    availableOptions: unmarshalArrayOfObject(data.available_options, unmarshalOffer$1),
    bandwidth: data.bandwidth,
    commercialRange: data.commercial_range,
    connectivity: data.connectivity,
    cpus: unmarshalArrayOfObject(data.cpus, unmarshalCPU),
    disks: unmarshalArrayOfObject(data.disks, unmarshalDisk),
    memories: unmarshalArrayOfObject(data.memories, unmarshalMemory),
    onetimeFees: data.onetime_fees ? unmarshalOffer$1(data.onetime_fees) : undefined,
    persistentMemories: unmarshalArrayOfObject(data.persistent_memories, unmarshalPersistentMemory),
    raidControllers: unmarshalArrayOfObject(data.raid_controllers, unmarshalRaidController),
    rpnVersion: data.rpn_version,
    stock: data.stock,
    stockByDatacenter: data.stock_by_datacenter
  };
};
const unmarshalOfferServiceLevelInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferServiceLevelInfo' failed as data isn't a dictionary.`);
  }
  return {
    antidos: data.antidos,
    availableOptions: unmarshalArrayOfObject(data.available_options, unmarshalOffer$1),
    customization: data.customization,
    extraFailoverQuota: data.extra_failover_quota,
    git: data.git,
    highRpnBandwidth: data.high_rpn_bandwidth,
    prioritySupport: data.priority_support,
    salesSupport: data.sales_support,
    sla: data.sla,
    supportPhone: data.support_phone,
    supportTicket: data.support_ticket
  };
};
const unmarshalOfferStorageInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferStorageInfo' failed as data isn't a dictionary.`);
  }
  return {
    maxQuota: data.max_quota,
    size: data.size
  };
};
const unmarshalOffer$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Offer' failed as data isn't a dictionary.`);
  }
  return {
    antidosInfo: data.antidos_info ? unmarshalOfferAntiDosInfo(data.antidos_info) : undefined,
    backupInfo: data.backup_info ? unmarshalOfferBackupInfo(data.backup_info) : undefined,
    bandwidthInfo: data.bandwidth_info ? unmarshalOfferBandwidthInfo(data.bandwidth_info) : undefined,
    catalog: data.catalog,
    failoverBlockInfo: data.failover_block_info ? unmarshalOfferFailoverBlockInfo(data.failover_block_info) : undefined,
    failoverIpInfo: data.failover_ip_info ? unmarshalOfferFailoverIpInfo(data.failover_ip_info) : undefined,
    id: data.id,
    licenseInfo: data.license_info ? unmarshalOfferLicenseInfo(data.license_info) : undefined,
    name: data.name,
    paymentFrequency: data.payment_frequency,
    pricing: data.pricing ? unmarshalMoney(data.pricing) : undefined,
    rpnInfo: data.rpn_info ? unmarshalOfferRPNInfo(data.rpn_info) : undefined,
    sanInfo: data.san_info ? unmarshalOfferSANInfo(data.san_info) : undefined,
    serverInfo: data.server_info ? unmarshalOfferServerInfo(data.server_info) : undefined,
    serviceLevelInfo: data.service_level_info ? unmarshalOfferServiceLevelInfo(data.service_level_info) : undefined,
    storageInfo: data.storage_info ? unmarshalOfferStorageInfo(data.storage_info) : undefined,
    usbStorageInfo: data.usb_storage_info ? unmarshalOfferStorageInfo(data.usb_storage_info) : undefined
  };
};
const unmarshalOS = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OS' failed as data isn't a dictionary.`);
  }
  return {
    allowCustomPartitioning: data.allow_custom_partitioning,
    allowSshKeys: data.allow_ssh_keys,
    allowedFilesystems: data.allowed_filesystems,
    arch: data.arch,
    displayName: data.display_name,
    hostnameMaxLength: data.hostname_max_length,
    hostnameRegex: data.hostname_regex,
    id: data.id,
    licenseOffers: unmarshalArrayOfObject(data.license_offers, unmarshalOffer$1),
    maxPartitions: data.max_partitions,
    name: data.name,
    panelPasswordRegex: data.panel_password_regex,
    passwordRegex: data.password_regex,
    releasedAt: unmarshalDate(data.released_at),
    requiresAdminPassword: data.requires_admin_password,
    requiresLicense: data.requires_license,
    requiresPanelPassword: data.requires_panel_password,
    requiresUser: data.requires_user,
    requiresValidHostname: data.requires_valid_hostname,
    type: data.type,
    version: data.version
  };
};
const unmarshalRpnSan = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnSan' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    datacenterName: data.datacenter_name,
    deliveredAt: unmarshalDate(data.delivered_at),
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    iqn: data.iqn,
    iqnSuffix: data.iqn_suffix,
    offer: data.offer ? unmarshalOffer$1(data.offer) : undefined,
    offerId: data.offer_id,
    offerName: data.offer_name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    rpnv1Compatible: data.rpnv1_compatible,
    rpnv1Implicit: data.rpnv1_implicit,
    serverHostname: data.server_hostname,
    status: data.status,
    storageSize: data.storage_size,
    terminatedAt: unmarshalDate(data.terminated_at)
  };
};
const unmarshalRpnGroup = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnGroup' failed as data isn't a dictionary.`);
  }
  return {
    active: data.active,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    membersCount: data.members_count,
    name: data.name,
    organizationId: data.organization_id,
    owner: data.owner,
    projectId: data.project_id,
    type: data.type
  };
};
const unmarshalNetworkInterface = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NetworkInterface' failed as data isn't a dictionary.`);
  }
  return {
    cardId: data.card_id,
    deviceId: data.device_id,
    ips: unmarshalArrayOfObject(data.ips, unmarshalIP$2),
    mac: data.mac,
    type: data.type
  };
};
const unmarshalServerLocation$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerLocation' failed as data isn't a dictionary.`);
  }
  return {
    datacenterName: data.datacenter_name,
    rack: data.rack,
    room: data.room
  };
};
const unmarshalServerOption = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerOption' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    expiredAt: unmarshalDate(data.expired_at),
    offer: data.offer ? unmarshalOffer$1(data.offer) : undefined,
    options: unmarshalArrayOfObject(data.options, unmarshalServerOption),
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalServiceLevel = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServiceLevel' failed as data isn't a dictionary.`);
  }
  return {
    level: data.level,
    offerId: data.offer_id
  };
};
const unmarshalServer$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Server' failed as data isn't a dictionary.`);
  }
  return {
    abuseContact: data.abuse_contact,
    createdAt: unmarshalDate(data.created_at),
    expiredAt: unmarshalDate(data.expired_at),
    hasBmc: data.has_bmc,
    hostname: data.hostname,
    id: data.id,
    interfaces: unmarshalArrayOfObject(data.interfaces, unmarshalNetworkInterface),
    ipv6Slaac: data.ipv6_slaac,
    isOutsourced: data.is_outsourced,
    isRpnv2Member: data.is_rpnv2_member,
    level: data.level ? unmarshalServiceLevel(data.level) : undefined,
    location: data.location ? unmarshalServerLocation$1(data.location) : undefined,
    offer: data.offer ? unmarshalOffer$1(data.offer) : undefined,
    options: unmarshalArrayOfObject(data.options, unmarshalServerOption),
    organizationId: data.organization_id,
    os: data.os ? unmarshalOS(data.os) : undefined,
    projectId: data.project_id,
    qinq: data.qinq,
    rebootedAt: unmarshalDate(data.rebooted_at),
    rescueOs: data.rescue_os ? unmarshalOS(data.rescue_os) : undefined,
    status: data.status,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalRpnV2GroupSubnet = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnV2GroupSubnet' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    cidr: data.cidr
  };
};
const unmarshalRpnV2Group = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnV2Group' failed as data isn't a dictionary.`);
  }
  return {
    compatibleRpnv1: data.compatible_rpnv1,
    gateway: data.gateway,
    id: data.id,
    membersCount: data.members_count,
    name: data.name,
    organizationId: data.organization_id,
    owner: data.owner,
    projectId: data.project_id,
    rpnv1Group: data.rpnv1_group ? unmarshalRpnGroup(data.rpnv1_group) : undefined,
    status: data.status,
    subnet: data.subnet ? unmarshalRpnV2GroupSubnet(data.subnet) : undefined,
    type: data.type
  };
};
const unmarshalService = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Service' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    deliveredAt: unmarshalDate(data.delivered_at),
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    offer: data.offer ? unmarshalOffer$1(data.offer) : undefined,
    provisioningStatus: data.provisioning_status,
    resourceId: data.resource_id,
    terminatedAt: unmarshalDate(data.terminated_at),
    type: data.type
  };
};
const unmarshalFailoverBlock = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'FailoverBlock' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    cidr: data.cidr,
    gatewayIp: data.gateway_ip,
    id: data.id,
    ipVersion: data.ip_version,
    nameservers: data.nameservers,
    netmask: data.netmask
  };
};
const unmarshalFailoverIP = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'FailoverIP' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    block: data.block ? unmarshalFailoverBlock(data.block) : undefined,
    cidr: data.cidr,
    gatewayIp: data.gateway_ip,
    id: data.id,
    ipVersion: data.ip_version,
    mac: data.mac,
    netmask: data.netmask,
    reverse: data.reverse,
    serverId: data.server_id,
    serverZone: data.server_zone,
    status: data.status,
    type: data.type
  };
};
const unmarshalBMCAccess = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BMCAccess' failed as data isn't a dictionary.`);
  }
  return {
    expiresAt: unmarshalDate(data.expires_at),
    login: data.login,
    password: data.password,
    status: data.status,
    url: data.url
  };
};
const unmarshalBackup = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Backup' failed as data isn't a dictionary.`);
  }
  return {
    aclEnabled: data.acl_enabled,
    autologin: data.autologin,
    id: data.id,
    login: data.login,
    quotaFiles: data.quota_files,
    quotaFilesUsed: data.quota_files_used,
    quotaSpace: data.quota_space,
    quotaSpaceUsed: data.quota_space_used,
    server: data.server,
    status: data.status
  };
};
const unmarshalCanOrderResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CanOrderResponse' failed as data isn't a dictionary.`);
  }
  return {
    billingOk: data.billing_ok,
    canOrder: data.can_order,
    emailConfirmed: data.email_confirmed,
    message: data.message,
    paymentMode: data.payment_mode,
    phoneConfirmed: data.phone_confirmed,
    quotaOk: data.quota_ok,
    userConfirmed: data.user_confirmed
  };
};
const unmarshalCreateFailoverIPsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateFailoverIPsResponse' failed as data isn't a dictionary.`);
  }
  return {
    services: unmarshalArrayOfObject(data.services, unmarshalService),
    totalCount: data.total_count
  };
};
const unmarshalGetIPv6BlockQuotasResponseQuota = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetIPv6BlockQuotasResponseQuota' failed as data isn't a dictionary.`);
  }
  return {
    cidr: data.cidr,
    quota: data.quota
  };
};
const unmarshalGetIPv6BlockQuotasResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetIPv6BlockQuotasResponse' failed as data isn't a dictionary.`);
  }
  return {
    quotas: unmarshalArrayOfObject(data.quotas, unmarshalGetIPv6BlockQuotasResponseQuota),
    totalCount: data.total_count
  };
};
const unmarshalGetRemainingQuotaResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetRemainingQuotaResponse' failed as data isn't a dictionary.`);
  }
  return {
    failoverBlockQuota: data.failover_block_quota,
    failoverBlockRemainingQuota: data.failover_block_remaining_quota,
    failoverIpQuota: data.failover_ip_quota,
    failoverIpRemainingQuota: data.failover_ip_remaining_quota
  };
};
const unmarshalGetRpnStatusResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetRpnStatusResponse' failed as data isn't a dictionary.`);
  }
  return {
    operationsLeft: data.operations_left,
    status: data.status
  };
};
const unmarshalIPv6Block = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'IPv6Block' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    cidr: data.cidr,
    delegationStatus: data.delegation_status,
    duid: data.duid,
    id: data.id,
    nameservers: data.nameservers,
    subnets: unmarshalArrayOfObject(data.subnets, unmarshalIPv6Block)
  };
};
const unmarshalInvoice = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Invoice' failed as data isn't a dictionary.`);
  }
  return {
    content: data.content,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    paidAt: unmarshalDate(data.paid_at),
    paymentMethod: data.payment_method,
    status: data.status,
    totalWithTaxes: data.total_with_taxes ? unmarshalMoney(data.total_with_taxes) : undefined,
    totalWithoutTaxes: data.total_without_taxes ? unmarshalMoney(data.total_without_taxes) : undefined,
    transactionId: data.transaction_id
  };
};
const unmarshalListFailoverIPsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListFailoverIPsResponse' failed as data isn't a dictionary.`);
  }
  return {
    failoverIps: unmarshalArrayOfObject(data.failover_ips, unmarshalFailoverIP),
    totalCount: data.total_count
  };
};
const unmarshalListIPv6BlockSubnetsAvailableResponseSubnet = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListIPv6BlockSubnetsAvailableResponseSubnet' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    cidr: data.cidr
  };
};
const unmarshalListIPv6BlockSubnetsAvailableResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListIPv6BlockSubnetsAvailableResponse' failed as data isn't a dictionary.`);
  }
  return {
    subnetAvailables: unmarshalArrayOfObject(data.subnet_availables, unmarshalListIPv6BlockSubnetsAvailableResponseSubnet),
    totalCount: data.total_count
  };
};
const unmarshalInvoiceSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'InvoiceSummary' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    paidAt: unmarshalDate(data.paid_at),
    paymentMethod: data.payment_method,
    status: data.status,
    totalWithTaxes: data.total_with_taxes ? unmarshalMoney(data.total_with_taxes) : undefined,
    totalWithoutTaxes: data.total_without_taxes ? unmarshalMoney(data.total_without_taxes) : undefined,
    transactionId: data.transaction_id
  };
};
const unmarshalListInvoicesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInvoicesResponse' failed as data isn't a dictionary.`);
  }
  return {
    invoices: unmarshalArrayOfObject(data.invoices, unmarshalInvoiceSummary),
    totalCount: data.total_count
  };
};
const unmarshalRpnSanIpRpnV2Group = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnSanIpRpnV2Group' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name
  };
};
const unmarshalRpnSanIpServer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnSanIpServer' failed as data isn't a dictionary.`);
  }
  return {
    datacenterName: data.datacenter_name,
    hostname: data.hostname,
    id: data.id
  };
};
const unmarshalRpnSanIp = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnSanIp' failed as data isn't a dictionary.`);
  }
  return {
    ip: data.ip ? unmarshalIP$2(data.ip) : undefined,
    rpnv2Group: data.rpnv2_group ? unmarshalRpnSanIpRpnV2Group(data.rpnv2_group) : undefined,
    server: data.server ? unmarshalRpnSanIpServer(data.server) : undefined,
    type: data.type
  };
};
const unmarshalListIpsResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListIpsResponse' failed as data isn't a dictionary.`);
  }
  return {
    ips: unmarshalArrayOfObject(data.ips, unmarshalRpnSanIp),
    totalCount: data.total_count
  };
};
const unmarshalListOSResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListOSResponse' failed as data isn't a dictionary.`);
  }
  return {
    os: unmarshalArrayOfObject(data.os, unmarshalOS),
    totalCount: data.total_count
  };
};
const unmarshalListOffersResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListOffersResponse' failed as data isn't a dictionary.`);
  }
  return {
    offers: unmarshalArrayOfObject(data.offers, unmarshalOffer$1),
    totalCount: data.total_count
  };
};
const unmarshalRefundSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RefundSummary' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    method: data.method,
    refundedAt: unmarshalDate(data.refunded_at),
    status: data.status,
    totalWithTaxes: data.total_with_taxes ? unmarshalMoney(data.total_with_taxes) : undefined,
    totalWithoutTaxes: data.total_without_taxes ? unmarshalMoney(data.total_without_taxes) : undefined
  };
};
const unmarshalListRefundsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRefundsResponse' failed as data isn't a dictionary.`);
  }
  return {
    refunds: unmarshalArrayOfObject(data.refunds, unmarshalRefundSummary),
    totalCount: data.total_count
  };
};
const unmarshalRpnSanServer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnSanServer' failed as data isn't a dictionary.`);
  }
  return {
    datacenterName: data.datacenter_name,
    hostname: data.hostname,
    id: data.id,
    sans: unmarshalArrayOfObject(data.sans, unmarshalRpnSan),
    zone: data.zone
  };
};
const unmarshalListRpnCapableSanServersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnCapableSanServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    sanServers: unmarshalArrayOfObject(data.san_servers, unmarshalRpnSanServer),
    totalCount: data.total_count
  };
};
const unmarshalListRpnCapableServersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnCapableServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalServer$1),
    totalCount: data.total_count
  };
};
const unmarshalRpnGroupMember = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnGroupMember' failed as data isn't a dictionary.`);
  }
  return {
    groupId: data.group_id,
    groupName: data.group_name,
    groupOwner: data.group_owner,
    id: data.id,
    owner: data.owner,
    sanServer: data.san_server ? unmarshalRpnSanServer(data.san_server) : undefined,
    server: data.server ? unmarshalServer$1(data.server) : undefined,
    speed: data.speed,
    status: data.status
  };
};
const unmarshalListRpnGroupMembersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnGroupMembersResponse' failed as data isn't a dictionary.`);
  }
  return {
    members: unmarshalArrayOfObject(data.members, unmarshalRpnGroupMember),
    totalCount: data.total_count
  };
};
const unmarshalListRpnGroupsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnGroupsResponse' failed as data isn't a dictionary.`);
  }
  return {
    rpnGroups: unmarshalArrayOfObject(data.rpn_groups, unmarshalRpnGroup),
    totalCount: data.total_count
  };
};
const unmarshalListRpnInvitesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnInvitesResponse' failed as data isn't a dictionary.`);
  }
  return {
    members: unmarshalArrayOfObject(data.members, unmarshalRpnGroupMember),
    totalCount: data.total_count
  };
};
const unmarshalRpnSanSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnSanSummary' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    datacenterName: data.datacenter_name,
    deliveredAt: unmarshalDate(data.delivered_at),
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    iqnSuffix: data.iqn_suffix,
    offerId: data.offer_id,
    offerName: data.offer_name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    rpnv1Compatible: data.rpnv1_compatible,
    rpnv1Implicit: data.rpnv1_implicit,
    serverHostname: data.server_hostname,
    status: data.status,
    storageSize: data.storage_size,
    terminatedAt: unmarshalDate(data.terminated_at)
  };
};
const unmarshalListRpnSansResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnSansResponse' failed as data isn't a dictionary.`);
  }
  return {
    rpnSans: unmarshalArrayOfObject(data.rpn_sans, unmarshalRpnSanSummary),
    totalCount: data.total_count
  };
};
const unmarshalRpnServerCapability = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnServerCapability' failed as data isn't a dictionary.`);
  }
  return {
    canJoinQinqGroup: data.can_join_qinq_group,
    canJoinRpnv2Group: data.can_join_rpnv2_group,
    compatibleQinq: data.compatible_qinq,
    datacenterName: data.datacenter_name,
    hostname: data.hostname,
    id: data.id,
    ipAddress: data.ip_address,
    rpnVersion: data.rpn_version,
    rpnv1GroupCount: data.rpnv1_group_count,
    rpnv2GroupCount: data.rpnv2_group_count,
    zone: data.zone
  };
};
const unmarshalListRpnServerCapabilitiesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnServerCapabilitiesResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalRpnServerCapability),
    totalCount: data.total_count
  };
};
const unmarshalListRpnV2CapableResourcesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnV2CapableResourcesResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalServer$1),
    totalCount: data.total_count
  };
};
const unmarshalRpnV2Member = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RpnV2Member' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    rpnv1Group: data.rpnv1_group ? unmarshalRpnGroup(data.rpnv1_group) : undefined,
    server: data.server ? unmarshalServer$1(data.server) : undefined,
    speed: data.speed,
    status: data.status,
    vlan: data.vlan
  };
};
const unmarshalLog$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Log' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    createdAt: unmarshalDate(data.created_at),
    finishedAt: unmarshalDate(data.finished_at),
    group: data.group ? unmarshalRpnV2Group(data.group) : undefined,
    id: data.id,
    member: data.member ? unmarshalRpnV2Member(data.member) : undefined,
    status: data.status
  };
};
const unmarshalListRpnV2GroupLogsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnV2GroupLogsResponse' failed as data isn't a dictionary.`);
  }
  return {
    logs: unmarshalArrayOfObject(data.logs, unmarshalLog$1),
    totalCount: data.total_count
  };
};
const unmarshalListRpnV2GroupsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnV2GroupsResponse' failed as data isn't a dictionary.`);
  }
  return {
    rpnGroups: unmarshalArrayOfObject(data.rpn_groups, unmarshalRpnV2Group),
    totalCount: data.total_count
  };
};
const unmarshalListRpnV2MembersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRpnV2MembersResponse' failed as data isn't a dictionary.`);
  }
  return {
    members: unmarshalArrayOfObject(data.members, unmarshalRpnV2Member),
    totalCount: data.total_count
  };
};
const unmarshalServerDisk = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerDisk' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    connector: data.connector,
    id: data.id,
    isAddon: data.is_addon,
    type: data.type
  };
};
const unmarshalListServerDisksResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServerDisksResponse' failed as data isn't a dictionary.`);
  }
  return {
    disks: unmarshalArrayOfObject(data.disks, unmarshalServerDisk),
    totalCount: data.total_count
  };
};
const unmarshalServerEvent = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerEvent' failed as data isn't a dictionary.`);
  }
  return {
    date: unmarshalDate(data.date),
    description: data.description,
    eventId: data.event_id
  };
};
const unmarshalListServerEventsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServerEventsResponse' failed as data isn't a dictionary.`);
  }
  return {
    events: unmarshalArrayOfObject(data.events, unmarshalServerEvent),
    totalCount: data.total_count
  };
};
const unmarshalServerSummary$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerSummary' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    datacenterName: data.datacenter_name,
    expiredAt: unmarshalDate(data.expired_at),
    hostname: data.hostname,
    id: data.id,
    interfaces: unmarshalArrayOfObject(data.interfaces, unmarshalNetworkInterface),
    isOutsourced: data.is_outsourced,
    level: data.level ? unmarshalServiceLevel(data.level) : undefined,
    offerId: data.offer_id,
    offerName: data.offer_name,
    organizationId: data.organization_id,
    osId: data.os_id,
    projectId: data.project_id,
    qinq: data.qinq,
    rpnVersion: data.rpn_version,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalListServersResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalServerSummary$1),
    totalCount: data.total_count
  };
};
const unmarshalListServicesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServicesResponse' failed as data isn't a dictionary.`);
  }
  return {
    services: unmarshalArrayOfObject(data.services, unmarshalService),
    totalCount: data.total_count
  };
};
const unmarshalListSubscribableServerOptionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSubscribableServerOptionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    serverOptions: unmarshalArrayOfObject(data.server_options, unmarshalOffer$1),
    totalCount: data.total_count
  };
};
const unmarshalRaidArray = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RaidArray' failed as data isn't a dictionary.`);
  }
  return {
    disks: unmarshalArrayOfObject(data.disks, unmarshalServerDisk),
    raidLevel: data.raid_level
  };
};
const unmarshalRaid = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Raid' failed as data isn't a dictionary.`);
  }
  return {
    raidArrays: unmarshalArrayOfObject(data.raid_arrays, unmarshalRaidArray)
  };
};
const unmarshalRefund = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Refund' failed as data isn't a dictionary.`);
  }
  return {
    content: data.content,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    method: data.method,
    refundedAt: unmarshalDate(data.refunded_at),
    status: data.status,
    totalWithTaxes: data.total_with_taxes ? unmarshalMoney(data.total_with_taxes) : undefined,
    totalWithoutTaxes: data.total_without_taxes ? unmarshalMoney(data.total_without_taxes) : undefined
  };
};
const unmarshalRescue = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Rescue' failed as data isn't a dictionary.`);
  }
  return {
    login: data.login,
    osId: data.os_id,
    password: data.password,
    protocol: data.protocol
  };
};
const unmarshalPartition = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Partition' failed as data isn't a dictionary.`);
  }
  return {
    capacity: data.capacity,
    connectors: data.connectors,
    fileSystem: data.file_system,
    mountPoint: data.mount_point,
    raidLevel: data.raid_level,
    type: data.type
  };
};
const unmarshalServerDefaultPartitioning = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerDefaultPartitioning' failed as data isn't a dictionary.`);
  }
  return {
    partitions: unmarshalArrayOfObject(data.partitions, unmarshalPartition)
  };
};
const unmarshalServerInstall = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerInstall' failed as data isn't a dictionary.`);
  }
  return {
    hostname: data.hostname,
    osId: data.os_id,
    panelUrl: data.panel_url,
    partitions: unmarshalArrayOfObject(data.partitions, unmarshalPartition),
    sshKeyIds: data.ssh_key_ids,
    status: data.status,
    userLogin: data.user_login
  };
};
const unmarshalSubscribeStorageOptionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SubscribeStorageOptionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    services: unmarshalArrayOfObject(data.services, unmarshalService)
  };
};
const marshalAttachFailoverIPToMacAddressRequest = (request, defaults) => ({
  mac: request.mac,
  type: request.type
});
const marshalAttachFailoverIPsRequest = (request, defaults) => ({
  fips_ids: request.fipsIds,
  server_id: request.serverId
});
const marshalCreateFailoverIPsRequest = (request, defaults) => ({
  offer_id: request.offerId,
  project_id: request.projectId ?? defaults.defaultProjectId,
  quantity: request.quantity
});
const marshalCreateServerRequest$1 = (request, defaults) => ({
  datacenter_name: request.datacenterName,
  offer_id: request.offerId,
  project_id: request.projectId ?? defaults.defaultProjectId,
  server_option_ids: request.serverOptionIds
});
const marshalDetachFailoverIPsRequest = (request, defaults) => ({
  fips_ids: request.fipsIds
});
const marshalIPv6BlockApiCreateIPv6BlockRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalIPv6BlockApiCreateIPv6BlockSubnetRequest = (request, defaults) => ({
  address: request.address,
  cidr: request.cidr
});
const marshalIPv6BlockApiUpdateIPv6BlockRequest = (request, defaults) => ({
  nameservers: request.nameservers
});
const marshalInstallPartition = (request, defaults) => ({
  capacity: request.capacity,
  connectors: request.connectors,
  file_system: request.fileSystem,
  mount_point: request.mountPoint,
  raid_level: request.raidLevel
});
const marshalInstallServerRequest = (request, defaults) => ({
  hostname: request.hostname,
  ip_id: request.ipId,
  license_offer_id: request.licenseOfferId,
  os_id: request.osId,
  panel_password: request.panelPassword,
  partitions: request.partitions !== undefined ? request.partitions.map(elt => marshalInstallPartition(elt)) : undefined,
  root_password: request.rootPassword,
  ssh_key_ids: request.sshKeyIds,
  user_login: request.userLogin,
  user_password: request.userPassword
});
const marshalRpnSanApiAddIpRequest = (request, defaults) => ({
  ip_ids: request.ipIds
});
const marshalRpnSanApiCreateRpnSanRequest = (request, defaults) => ({
  offer_id: request.offerId,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalRpnSanApiRemoveIpRequest = (request, defaults) => ({
  ip_ids: request.ipIds
});
const marshalRpnV1ApiAddRpnGroupMembersRequest = (request, defaults) => ({
  san_server_ids: request.sanServerIds,
  server_ids: request.serverIds
});
const marshalRpnV1ApiCreateRpnGroupRequest = (request, defaults) => ({
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId,
  san_server_ids: request.sanServerIds,
  server_ids: request.serverIds
});
const marshalRpnV1ApiDeleteRpnGroupMembersRequest = (request, defaults) => ({
  member_ids: request.memberIds
});
const marshalRpnV1ApiLeaveRpnGroupRequest = (request, defaults) => ({
  member_ids: request.memberIds,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalRpnV1ApiRpnGroupInviteRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId,
  server_ids: request.serverIds
});
const marshalRpnV1ApiUpdateRpnGroupNameRequest = (request, defaults) => ({
  name: request.name
});
const marshalRpnV2ApiAddRpnV2MembersRequest = (request, defaults) => ({
  servers: request.servers
});
const marshalRpnV2ApiCreateRpnV2GroupRequest = (request, defaults) => ({
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId,
  servers: request.servers,
  type: request.type
});
const marshalRpnV2ApiDeleteRpnV2MembersRequest = (request, defaults) => ({
  member_ids: request.memberIds
});
const marshalRpnV2ApiEnableRpnV2GroupCompatibilityRequest = (request, defaults) => ({
  rpnv1_group_id: request.rpnv1GroupId
});
const marshalRpnV2ApiUpdateRpnV2GroupNameRequest = (request, defaults) => ({
  name: request.name
});
const marshalRpnV2ApiUpdateRpnV2VlanForMembersRequest = (request, defaults) => ({
  member_ids: request.memberIds,
  vlan: request.vlan
});
const marshalStartBMCAccessRequest = (request, defaults) => ({
  ip: request.ip
});
const marshalStartRescueRequest = (request, defaults) => ({
  os_id: request.osId
});
const marshalSubscribeServerOptionRequest = (request, defaults) => ({
  option_id: request.optionId
});
const marshalSubscribeStorageOptionsRequest = (request, defaults) => ({
  options_ids: request.optionsIds
});
const marshalUpdatableRaidArray = (request, defaults) => ({
  disk_ids: request.diskIds,
  raid_level: request.raidLevel
});
const marshalUpdateRaidRequest = (request, defaults) => ({
  raid_arrays: request.raidArrays.map(elt => marshalUpdatableRaidArray(elt))
});
const marshalUpdateReverseRequest = (request, defaults) => ({
  reverse: request.reverse
});
const marshalUpdateServerBackupRequest = (request, defaults) => ({
  acl_enabled: request.aclEnabled,
  autologin: request.autologin,
  password: request.password
});
const marshalUpdateServerRequest$1 = (request, defaults) => ({
  enable_ipv6: request.enableIpv6,
  hostname: request.hostname
});
const marshalUpdateServerTagsRequest = (request, defaults) => ({
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$q = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Dedibox Phoenix API. */
let API$q = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'nl-ams-1'];
  pageOfListServers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['search', request.search])
  }, unmarshalListServersResponse$1);

  /**
   * List baremetal servers for project.
   *
   * @param request - The request {@link ListServersRequest}
   * @returns A Promise of ListServersResponse
   */
  listServers = (request = {}) => enrichForPagination('servers', this.pageOfListServers, request);

  /**
   * Get a specific baremetal server. Get the server associated with the given
   * ID.
   *
   * @param request - The request {@link GetServerRequest}
   * @returns A Promise of Server
   */
  getServer = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalServer$1);

  /**
   * Waits for {@link Server} to be in a final state.
   *
   * @param request - The request {@link GetServerRequest}
   * @param options - The waiting options
   * @returns A Promise of Server
   */
  waitForServer = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SERVER_TRANSIENT_STATUSES$1.includes(res.status))), this.getServer, request, options);
  getServerBackup = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/backups`
  }, unmarshalBackup);
  updateServerBackup = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateServerBackupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/backups`
  }, unmarshalBackup);
  pageOfListSubscribableServerOptions = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/subscribable-server-options`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListSubscribableServerOptionsResponse);

  /**
   * List subscribable server options. List subscribable options associated to
   * the given server ID.
   *
   * @param request - The request {@link ListSubscribableServerOptionsRequest}
   * @returns A Promise of ListSubscribableServerOptionsResponse
   */
  listSubscribableServerOptions = request => enrichForPagination('serverOptions', this.pageOfListSubscribableServerOptions, request);

  /**
   * Subscribe server option. Subscribe option for the given server ID.
   *
   * @param request - The request {@link SubscribeServerOptionRequest}
   * @returns A Promise of Service
   */
  subscribeServerOption = request => this.client.fetch({
    body: JSON.stringify(marshalSubscribeServerOptionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/subscribe-server-option`
  }, unmarshalService);

  /**
   * Create a baremetal server. Create a new baremetal server. The order return
   * you a service ID to follow the provisionning status you could call
   * GetService.
   *
   * @param request - The request {@link CreateServerRequest}
   * @returns A Promise of Service
   */
  createServer = request => this.client.fetch({
    body: JSON.stringify(marshalCreateServerRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`
  }, unmarshalService);

  /**
   * Subscribe storage server option. Subscribe storage option for the given
   * server ID.
   *
   * @param request - The request {@link SubscribeStorageOptionsRequest}
   * @returns A Promise of SubscribeStorageOptionsResponse
   */
  subscribeStorageOptions = request => this.client.fetch({
    body: JSON.stringify(marshalSubscribeStorageOptionsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/subscribe-storage-options`
  }, unmarshalSubscribeStorageOptionsResponse);

  /**
   * Update a baremetal server. Update the server associated with the given ID.
   *
   * @param request - The request {@link UpdateServerRequest}
   * @returns A Promise of Server
   */
  updateServer = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateServerRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalServer$1);
  updateServerTags = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateServerTagsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/tags`
  }, unmarshalServer$1);

  /**
   * Reboot a baremetal server. Reboot the server associated with the given ID,
   * use boot param to reboot in rescue.
   *
   * @param request - The request {@link RebootServerRequest}
   */
  rebootServer = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/reboot`
  });

  /**
   * Start a baremetal server. Start the server associated with the given ID.
   *
   * @param request - The request {@link StartServerRequest}
   */
  startServer = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/start`
  });

  /**
   * Stop a baremetal server. Stop the server associated with the given ID.
   *
   * @param request - The request {@link StopServerRequest}
   */
  stopServer = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/stop`
  });

  /**
   * Delete a baremetal server. Delete the server associated with the given ID.
   *
   * @param request - The request {@link DeleteServerRequest}
   */
  deleteServer = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  });
  pageOfListServerEvents = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/events`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListServerEventsResponse);

  /**
   * List server events. List events associated to the given server ID.
   *
   * @param request - The request {@link ListServerEventsRequest}
   * @returns A Promise of ListServerEventsResponse
   */
  listServerEvents = request => enrichForPagination('events', this.pageOfListServerEvents, request);
  pageOfListServerDisks = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/disks`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListServerDisksResponse);

  /**
   * List server disks. List disks associated to the given server ID.
   *
   * @param request - The request {@link ListServerDisksRequest}
   * @returns A Promise of ListServerDisksResponse
   */
  listServerDisks = request => enrichForPagination('disks', this.pageOfListServerDisks, request);
  getOrderedService = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ordered-services/${validatePathParam('orderedServiceId', request.orderedServiceId)}`
  }, unmarshalService);

  /**
   * Get a specific service. Get the service associated with the given ID.
   *
   * @param request - The request {@link GetServiceRequest}
   * @returns A Promise of Service
   */
  getService = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/services/${validatePathParam('serviceId', request.serviceId)}`
  }, unmarshalService);

  /**
   * Delete a specific service. Delete the service associated with the given ID.
   *
   * @param request - The request {@link DeleteServiceRequest}
   * @returns A Promise of Service
   */
  deleteService = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/services/${validatePathParam('serviceId', request.serviceId)}`
  }, unmarshalService);
  pageOfListServices = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/services`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListServicesResponse);

  /**
   * List services.
   *
   * @param request - The request {@link ListServicesRequest}
   * @returns A Promise of ListServicesResponse
   */
  listServices = (request = {}) => enrichForPagination('services', this.pageOfListServices, request);

  /**
   * Install a baremetal server. Install an OS on the server associated with the
   * given ID.
   *
   * @param request - The request {@link InstallServerRequest}
   * @returns A Promise of ServerInstall
   */
  installServer = request => this.client.fetch({
    body: JSON.stringify(marshalInstallServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/install`
  }, unmarshalServerInstall);

  /**
   * Get a specific server installation status. Get the server installation
   * status associated with the given server ID.
   *
   * @param request - The request {@link GetServerInstallRequest}
   * @returns A Promise of ServerInstall
   */
  getServerInstall = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/install`
  }, unmarshalServerInstall);

  /**
   * Waits for {@link ServerInstall} to be in a final state.
   *
   * @param request - The request {@link GetServerInstallRequest}
   * @param options - The waiting options
   * @returns A Promise of ServerInstall
   */
  waitForServerInstall = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SERVER_INSTALL_TRANSIENT_STATUSES.includes(res.status))), this.getServerInstall, request, options);

  /**
   * Cancels the current (running) server installation. Cancels the current
   * server installation associated with the given server ID.
   *
   * @param request - The request {@link CancelServerInstallRequest}
   */
  cancelServerInstall = request => this.client.fetch({
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/cancel-install`
  });

  /**
   * Get server default partitioning. Get the server default partitioning schema
   * associated with the given server ID and OS ID.
   *
   * @param request - The request {@link GetServerDefaultPartitioningRequest}
   * @returns A Promise of ServerDefaultPartitioning
   */
  getServerDefaultPartitioning = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/partitioning/${validatePathParam('osId', request.osId)}`
  }, unmarshalServerDefaultPartitioning);

  /**
   * Start BMC (Baseboard Management Controller) access for a given baremetal
   * server. Start BMC (Baseboard Management Controller) access associated with
   * the given ID. The BMC (Baseboard Management Controller) access is available
   * one hour after the installation of the server.
   *
   * @param request - The request {@link StartBMCAccessRequest}
   */
  startBMCAccess = request => this.client.fetch({
    body: JSON.stringify(marshalStartBMCAccessRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/bmc-access`
  });

  /**
   * Get BMC (Baseboard Management Controller) access for a given baremetal
   * server. Get the BMC (Baseboard Management Controller) access associated
   * with the given ID.
   *
   * @param request - The request {@link GetBMCAccessRequest}
   * @returns A Promise of BMCAccess
   */
  getBMCAccess = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/bmc-access`
  }, unmarshalBMCAccess);

  /**
   * Waits for {@link BMCAccess} to be in a final state.
   *
   * @param request - The request {@link GetBMCAccessRequest}
   * @param options - The waiting options
   * @returns A Promise of BMCAccess
   */
  waitForBMCAccess = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!BMC_ACCESS_TRANSIENT_STATUSES.includes(res.status))), this.getBMCAccess, request, options);

  /**
   * Stop BMC (Baseboard Management Controller) access for a given baremetal
   * server. Stop BMC (Baseboard Management Controller) access associated with
   * the given ID.
   *
   * @param request - The request {@link StopBMCAccessRequest}
   */
  stopBMCAccess = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/bmc-access`
  });
  pageOfListOffers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/offers`,
    urlParams: urlParams(['available_only', request.availableOnly], ['catalog', request.catalog], ['commercial_range', request.commercialRange], ['is_failover_block', request.isFailoverBlock], ['is_failover_ip', request.isFailoverIp], ['is_rpn_san', request.isRpnSan], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['sold_in', request.soldIn && request.soldIn.length > 0 ? request.soldIn.join(',') : undefined])
  }, unmarshalListOffersResponse$1);

  /**
   * List offers. List all available server offers.
   *
   * @param request - The request {@link ListOffersRequest}
   * @returns A Promise of ListOffersResponse
   */
  listOffers = (request = {}) => enrichForPagination('offers', this.pageOfListOffers, request);

  /**
   * Get offer. Return specific offer for the given ID.
   *
   * @param request - The request {@link GetOfferRequest}
   * @returns A Promise of Offer
   */
  getOffer = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/offers/${validatePathParam('offerId', request.offerId)}`,
    urlParams: urlParams(['project_id', request.projectId])
  }, unmarshalOffer$1);
  pageOfListOS = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/os`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['server_id', request.serverId], ['type', request.type])
  }, unmarshalListOSResponse);

  /**
   * List all available OS that can be install on a baremetal server.
   *
   * @param request - The request {@link ListOSRequest}
   * @returns A Promise of ListOSResponse
   */
  listOS = request => enrichForPagination('os', this.pageOfListOS, request);

  /**
   * Get an OS with a given ID. Return specific OS for the given ID.
   *
   * @param request - The request {@link GetOSRequest}
   * @returns A Promise of OS
   */
  getOS = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/os/${validatePathParam('osId', request.osId)}`,
    urlParams: urlParams(['project_id', request.projectId], ['server_id', request.serverId])
  }, unmarshalOS);

  /**
   * Update reverse of ip. Update reverse of ip associated with the given ID.
   *
   * @param request - The request {@link UpdateReverseRequest}
   * @returns A Promise of IP
   */
  updateReverse = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateReverseRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/reverses/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIP$2);

  /**
   * Order failover IPs. Order X failover IPs.
   *
   * @param request - The request {@link CreateFailoverIPsRequest}
   * @returns A Promise of CreateFailoverIPsResponse
   */
  createFailoverIPs = request => this.client.fetch({
    body: JSON.stringify(marshalCreateFailoverIPsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/failover-ips`
  }, unmarshalCreateFailoverIPsResponse);

  /**
   * Attach failovers on baremetal server. Attach failovers on the server
   * associated with the given ID.
   *
   * @param request - The request {@link AttachFailoverIPsRequest}
   */
  attachFailoverIPs = request => this.client.fetch({
    body: JSON.stringify(marshalAttachFailoverIPsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/failover-ips/attach`
  });

  /**
   * Detach failovers on baremetal server. Detach failovers on the server
   * associated with the given ID.
   *
   * @param request - The request {@link DetachFailoverIPsRequest}
   */
  detachFailoverIPs = request => this.client.fetch({
    body: JSON.stringify(marshalDetachFailoverIPsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/failover-ips/detach`
  });

  /**
   * Attach a failover IP to a MAC address.
   *
   * @param request - The request {@link AttachFailoverIPToMacAddressRequest}
   * @returns A Promise of IP
   */
  attachFailoverIPToMacAddress = request => this.client.fetch({
    body: JSON.stringify(marshalAttachFailoverIPToMacAddressRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/failover-ips/${validatePathParam('ipId', request.ipId)}/attach-to-mac-address`
  }, unmarshalIP$2);

  /**
   * Detach a failover IP from a MAC address.
   *
   * @param request - The request {@link DetachFailoverIPFromMacAddressRequest}
   * @returns A Promise of IP
   */
  detachFailoverIPFromMacAddress = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/failover-ips/${validatePathParam('ipId', request.ipId)}/detach-from-mac-address`
  }, unmarshalIP$2);

  /**
   * Delete a failover server. Delete the failover associated with the given ID.
   *
   * @param request - The request {@link DeleteFailoverIPRequest}
   */
  deleteFailoverIP = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/failover-ips/${validatePathParam('ipId', request.ipId)}`
  });
  pageOfListFailoverIPs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/failover-ips`,
    urlParams: urlParams(['only_available', request.onlyAvailable], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['search', request.search])
  }, unmarshalListFailoverIPsResponse);

  /**
   * List failovers for project. List failovers servers for project.
   *
   * @param request - The request {@link ListFailoverIPsRequest}
   * @returns A Promise of ListFailoverIPsResponse
   */
  listFailoverIPs = (request = {}) => enrichForPagination('failoverIps', this.pageOfListFailoverIPs, request);

  /**
   * Get a specific baremetal server. Get the server associated with the given
   * ID.
   *
   * @param request - The request {@link GetFailoverIPRequest}
   * @returns A Promise of FailoverIP
   */
  getFailoverIP = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/failover-ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalFailoverIP);

  /**
   * Get remaining quota.
   *
   * @param request - The request {@link GetRemainingQuotaRequest}
   * @returns A Promise of GetRemainingQuotaResponse
   */
  getRemainingQuota = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/remaining-quota`,
    urlParams: urlParams(['project_id', request.projectId])
  }, unmarshalGetRemainingQuotaResponse);

  /**
   * Get raid. Return raid for the given server ID.
   *
   * @param request - The request {@link GetRaidRequest}
   * @returns A Promise of Raid
   */
  getRaid = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/raid`
  }, unmarshalRaid);

  /**
   * Update RAID. Update RAID associated with the given server ID.
   *
   * @param request - The request {@link UpdateRaidRequest}
   */
  updateRaid = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateRaidRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/update-raid`
  });

  /**
   * Start in rescue baremetal server. Start in rescue the server associated
   * with the given ID.
   *
   * @param request - The request {@link StartRescueRequest}
   * @returns A Promise of Rescue
   */
  startRescue = request => this.client.fetch({
    body: JSON.stringify(marshalStartRescueRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/rescue`
  }, unmarshalRescue);

  /**
   * Get rescue information. Return rescue information for the given server ID.
   *
   * @param request - The request {@link GetRescueRequest}
   * @returns A Promise of Rescue
   */
  getRescue = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/rescue`
  }, unmarshalRescue);

  /**
   * Stop rescue on baremetal server. Stop rescue on the server associated with
   * the given ID.
   *
   * @param request - The request {@link StopRescueRequest}
   */
  stopRescue = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/rescue`
  });
};

/** Dedibox Phoenix Billing API. */
class BillingAPI extends API$z {
  pageOfListInvoices = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/invoices`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListInvoicesResponse);
  listInvoices = (request = {}) => enrichForPagination('invoices', this.pageOfListInvoices, request);
  getInvoice = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/invoices/${validatePathParam('invoiceId', request.invoiceId)}`
  }, unmarshalInvoice);
  downloadInvoice = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/invoices/${validatePathParam('invoiceId', request.invoiceId)}/download`,
    urlParams: urlParams(['dl', 1]),
    responseType: 'blob'
  });
  pageOfListRefunds = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/refunds`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRefundsResponse);
  listRefunds = (request = {}) => enrichForPagination('refunds', this.pageOfListRefunds, request);
  getRefund = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/refunds/${validatePathParam('refundId', request.refundId)}`
  }, unmarshalRefund);
  downloadRefund = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/refunds/${validatePathParam('refundId', request.refundId)}/download`,
    urlParams: urlParams(['dl', 1]),
    responseType: 'blob'
  });
  canOrder = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/can-order`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalCanOrderResponse);
}

/** Dedibox Phoenix IPv6 Block API. */
class IPv6BlockAPI extends API$z {
  /**
   * Get IPv6 block quota. Get IPv6 block quota with the given project ID. /48
   * one per organization. /56 link to your number of server. /64 link to your
   * number of failover IP.
   *
   * @param request - The request {@link IPv6BlockApiGetIPv6BlockQuotasRequest}
   * @returns A Promise of GetIPv6BlockQuotasResponse
   */
  getIPv6BlockQuotas = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/ipv6-block-quotas`,
    urlParams: urlParams(['project_id', request.projectId])
  }, unmarshalGetIPv6BlockQuotasResponse);

  /**
   * Create IPv6 block for baremetal server. Create IPv6 block associated with
   * the given project ID.
   *
   * @param request - The request {@link IPv6BlockApiCreateIPv6BlockRequest}
   * @returns A Promise of IPv6Block
   */
  createIPv6Block = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalIPv6BlockApiCreateIPv6BlockRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/ipv6-block`
  }, unmarshalIPv6Block);

  /**
   * Get a specific IPv6 block. Get the IPv6 block associated with the given ID.
   *
   * @param request - The request {@link IPv6BlockApiGetIPv6BlockRequest}
   * @returns A Promise of IPv6Block
   */
  getIPv6Block = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/ipv6-block`,
    urlParams: urlParams(['project_id', request.projectId])
  }, unmarshalIPv6Block);

  /**
   * Update IPv6 block. Update DNS associated to IPv6 block. If DNS is used,
   * minimum of 2 is necessary and maximum of 5 (no duplicate).
   *
   * @param request - The request {@link IPv6BlockApiUpdateIPv6BlockRequest}
   * @returns A Promise of IPv6Block
   */
  updateIPv6Block = request => this.client.fetch({
    body: JSON.stringify(marshalIPv6BlockApiUpdateIPv6BlockRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/ipv6-blocks/${validatePathParam('blockId', request.blockId)}`
  }, unmarshalIPv6Block);

  /**
   * Delete IPv6 block. Delete IPv6 block subnet with the given ID.
   *
   * @param request - The request {@link IPv6BlockApiDeleteIPv6BlockRequest}
   */
  deleteIPv6Block = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/ipv6-blocks/${validatePathParam('blockId', request.blockId)}`
  });

  /**
   * Create IPv6 block subnet. Create IPv6 block subnet for the given IP ID. /48
   * could create subnet in /56 (quota link to your number of server). /56 could
   * create subnet in /64 (quota link to your number of failover IP).
   *
   * @param request - The request
   *   {@link IPv6BlockApiCreateIPv6BlockSubnetRequest}
   * @returns A Promise of IPv6Block
   */
  createIPv6BlockSubnet = request => this.client.fetch({
    body: JSON.stringify(marshalIPv6BlockApiCreateIPv6BlockSubnetRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/ipv6-blocks/${validatePathParam('blockId', request.blockId)}/subnets`
  }, unmarshalIPv6Block);

  /**
   * List available IPv6 block subnets. List all available IPv6 block subnets
   * for given IP ID.
   *
   * @param request - The request
   *   {@link IPv6BlockApiListIPv6BlockSubnetsAvailableRequest}
   * @returns A Promise of ListIPv6BlockSubnetsAvailableResponse
   */
  listIPv6BlockSubnetsAvailable = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/ipv6-blocks/${validatePathParam('blockId', request.blockId)}/subnets`
  }, unmarshalListIPv6BlockSubnetsAvailableResponse);
}

/** Dedibox Phoenix RPN API. */
class RpnAPI extends API$z {
  pageOfListRpnServerCapabilities = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpn/server-capabilities`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRpnServerCapabilitiesResponse);
  listRpnServerCapabilities = (request = {}) => enrichForPagination('servers', this.pageOfListRpnServerCapabilities, request);
  getRpnStatus = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpn/status`,
    urlParams: urlParams(['project_id', request.projectId], ['rpnv1_group_id', request.rpnv1GroupId], ['rpnv2_group_id', request.rpnv2GroupId])
  }, unmarshalGetRpnStatusResponse);
}

/** Dedibox Phoenix RPN SAN API. */
class RpnSanAPI extends API$z {
  pageOfListRpnSans = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpn-sans`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRpnSansResponse);
  listRpnSans = (request = {}) => enrichForPagination('rpnSans', this.pageOfListRpnSans, request);
  getRpnSan = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpn-sans/${validatePathParam('rpnSanId', request.rpnSanId)}`
  }, unmarshalRpnSan);

  /**
   * Waits for {@link RpnSan} to be in a final state.
   *
   * @param request - The request {@link RpnSanApiGetRpnSanRequest}
   * @param options - The waiting options
   * @returns A Promise of RpnSan
   */
  waitForRpnSan = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!RPN_SAN_TRANSIENT_STATUSES.includes(res.status))), this.getRpnSan, request, options);
  deleteRpnSan = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/rpn-sans/${validatePathParam('rpnSanId', request.rpnSanId)}`
  });
  createRpnSan = request => this.client.fetch({
    body: JSON.stringify(marshalRpnSanApiCreateRpnSanRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpn-sans`
  }, unmarshalService);
  listIps = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpn-sans/${validatePathParam('rpnSanId', request.rpnSanId)}/ips`,
    urlParams: urlParams(['type', request.type])
  }, unmarshalListIpsResponse$2);
  addIp = request => this.client.fetch({
    body: JSON.stringify(marshalRpnSanApiAddIpRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpn-sans/${validatePathParam('rpnSanId', request.rpnSanId)}/ips`
  });
  removeIp = request => this.client.fetch({
    body: JSON.stringify(marshalRpnSanApiRemoveIpRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'DELETE',
    path: `/dedibox/v1/rpn-sans/${validatePathParam('rpnSanId', request.rpnSanId)}/ips`
  });
  listAvailableIps = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpn-sans/${validatePathParam('rpnSanId', request.rpnSanId)}/available-ips`,
    urlParams: urlParams(['type', request.type])
  }, unmarshalListIpsResponse$2);
}

/** Dedibox Phoenix RPN v1 API. */
class RpnV1API extends API$z {
  pageOfListRpnGroups = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv1/groups`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRpnGroupsResponse);
  listRpnGroups = (request = {}) => enrichForPagination('rpnGroups', this.pageOfListRpnGroups, request);
  getRpnGroup = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv1/groups/${validatePathParam('groupId', request.groupId)}`
  }, unmarshalRpnGroup);
  createRpnGroup = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV1ApiCreateRpnGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpnv1/groups`
  }, unmarshalRpnGroup);
  deleteRpnGroup = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/rpnv1/groups/${validatePathParam('groupId', request.groupId)}`
  });
  updateRpnGroupName = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV1ApiUpdateRpnGroupNameRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/rpnv1/groups/${validatePathParam('groupId', request.groupId)}`
  }, unmarshalRpnGroup);
  pageOfListRpnGroupMembers = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv1/groups/${validatePathParam('groupId', request.groupId)}/members`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRpnGroupMembersResponse);
  listRpnGroupMembers = request => enrichForPagination('members', this.pageOfListRpnGroupMembers, request);
  rpnGroupInvite = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV1ApiRpnGroupInviteRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpnv1/groups/${validatePathParam('groupId', request.groupId)}/invite`
  });
  leaveRpnGroup = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV1ApiLeaveRpnGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpnv1/groups/${validatePathParam('groupId', request.groupId)}/leave`
  });
  addRpnGroupMembers = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV1ApiAddRpnGroupMembersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpnv1/groups/${validatePathParam('groupId', request.groupId)}/members`
  }, unmarshalRpnGroup);
  deleteRpnGroupMembers = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV1ApiDeleteRpnGroupMembersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'DELETE',
    path: `/dedibox/v1/rpnv1/groups/${validatePathParam('groupId', request.groupId)}/members`
  }, unmarshalRpnGroup);
  pageOfListRpnCapableServers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv1/capable-servers`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRpnCapableServersResponse);
  listRpnCapableServers = (request = {}) => enrichForPagination('servers', this.pageOfListRpnCapableServers, request);
  pageOfListRpnCapableSanServers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv1/capable-san-servers`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRpnCapableSanServersResponse);
  listRpnCapableSanServers = (request = {}) => enrichForPagination('sanServers', this.pageOfListRpnCapableSanServers, request);
  pageOfListRpnInvites = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv1/invites`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListRpnInvitesResponse);
  listRpnInvites = (request = {}) => enrichForPagination('members', this.pageOfListRpnInvites, request);
  acceptRpnInvite = request => this.client.fetch({
    method: 'POST',
    path: `/dedibox/v1/rpnv1/invites/${validatePathParam('memberId', request.memberId)}/accept`
  });
  refuseRpnInvite = request => this.client.fetch({
    method: 'POST',
    path: `/dedibox/v1/rpnv1/invites/${validatePathParam('memberId', request.memberId)}/refuse`
  });
}

/** Dedibox Phoenix RPN v2 API. */
class RpnV2API extends API$z {
  pageOfListRpnV2Groups = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv2/groups`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRpnV2GroupsResponse);
  listRpnV2Groups = (request = {}) => enrichForPagination('rpnGroups', this.pageOfListRpnV2Groups, request);
  pageOfListRpnV2Members = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}/members`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['type', request.type])
  }, unmarshalListRpnV2MembersResponse);
  listRpnV2Members = request => enrichForPagination('members', this.pageOfListRpnV2Members, request);
  getRpnV2Group = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}`
  }, unmarshalRpnV2Group);

  /**
   * Waits for {@link RpnV2Group} to be in a final state.
   *
   * @param request - The request {@link RpnV2ApiGetRpnV2GroupRequest}
   * @param options - The waiting options
   * @returns A Promise of RpnV2Group
   */
  waitForRpnV2Group = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!RPN_V2_GROUP_TRANSIENT_STATUSES.includes(res.status))), this.getRpnV2Group, request, options);
  createRpnV2Group = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV2ApiCreateRpnV2GroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpnv2/groups`
  }, unmarshalRpnV2Group);
  deleteRpnV2Group = request => this.client.fetch({
    method: 'DELETE',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}`
  });
  updateRpnV2GroupName = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV2ApiUpdateRpnV2GroupNameRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}`
  }, unmarshalRpnV2Group);
  addRpnV2Members = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV2ApiAddRpnV2MembersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}/members`
  });
  deleteRpnV2Members = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV2ApiDeleteRpnV2MembersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'DELETE',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}/members`
  });
  pageOfListRpnV2CapableResources = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv2/groups/capable`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRpnV2CapableResourcesResponse);
  listRpnV2CapableResources = (request = {}) => enrichForPagination('servers', this.pageOfListRpnV2CapableResources, request);
  pageOfListRpnV2GroupLogs = request => this.client.fetch({
    method: 'GET',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}/logs`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListRpnV2GroupLogsResponse);
  listRpnV2GroupLogs = request => enrichForPagination('logs', this.pageOfListRpnV2GroupLogs, request);
  updateRpnV2VlanForMembers = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV2ApiUpdateRpnV2VlanForMembersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'PATCH',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}/vlan`
  });
  enableRpnV2GroupCompatibility = request => this.client.fetch({
    body: JSON.stringify(marshalRpnV2ApiEnableRpnV2GroupCompatibilityRequest(request, this.client.settings)),
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}/enable-compatibility`
  });
  disableRpnV2GroupCompatibility = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$q,
    method: 'POST',
    path: `/dedibox/v1/rpnv2/groups/${validatePathParam('groupId', request.groupId)}/disable-compatibility`
  });
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const AttachFailoverIPToMacAddressRequest = {
  ipId: {
    greaterThan: 0
  }
};
const AttachFailoverIPsRequest = {
  serverId: {
    greaterThan: 0
  }
};
const BillingApiDownloadInvoiceRequest = {
  invoiceId: {
    greaterThan: 0
  }
};
const BillingApiDownloadRefundRequest = {
  refundId: {
    greaterThan: 0
  }
};
const BillingApiGetInvoiceRequest = {
  invoiceId: {
    greaterThan: 0
  }
};
const BillingApiGetRefundRequest = {
  refundId: {
    greaterThan: 0
  }
};
const BillingApiListInvoicesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const BillingApiListRefundsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const CancelServerInstallRequest = {
  serverId: {
    greaterThan: 0
  }
};
const DeleteFailoverIPRequest = {
  ipId: {
    greaterThan: 0
  }
};
const DeleteServerRequest = {
  serverId: {
    greaterThan: 0
  }
};
const DeleteServiceRequest = {
  serviceId: {
    greaterThan: 0
  }
};
const DetachFailoverIPFromMacAddressRequest = {
  ipId: {
    greaterThan: 0
  }
};
const GetBMCAccessRequest = {
  serverId: {
    greaterThan: 0
  }
};
const GetFailoverIPRequest = {
  ipId: {
    greaterThan: 0
  }
};
const GetOSRequest = {
  osId: {
    greaterThan: 0
  },
  serverId: {
    greaterThan: 0
  }
};
const GetOfferRequest = {
  offerId: {
    greaterThan: 0
  }
};
const GetOrderedServiceRequest = {
  orderedServiceId: {
    greaterThan: 0
  }
};
const GetRaidRequest = {
  serverId: {
    greaterThan: 0
  }
};
const GetRescueRequest = {
  serverId: {
    greaterThan: 0
  }
};
const GetServerBackupRequest = {
  serverId: {
    greaterThan: 0
  }
};
const GetServerInstallRequest = {
  serverId: {
    greaterThan: 0
  }
};
const GetServerRequest = {
  serverId: {
    greaterThan: 0
  }
};
const GetServiceRequest = {
  serviceId: {
    greaterThan: 0
  }
};
const IPv6Block = {
  id: {
    greaterThan: 0
  }
};
const IPv6BlockApiCreateIPv6BlockSubnetRequest = {
  blockId: {
    greaterThan: 0
  }
};
const IPv6BlockApiDeleteIPv6BlockRequest = {
  blockId: {
    greaterThan: 0
  }
};
const IPv6BlockApiListIPv6BlockSubnetsAvailableRequest = {
  blockId: {
    greaterThan: 0
  }
};
const IPv6BlockApiUpdateIPv6BlockRequest = {
  blockId: {
    greaterThan: 0
  }
};
const InstallServerRequest = {
  osId: {
    greaterThan: 0
  },
  serverId: {
    greaterThan: 0
  }
};
const ListFailoverIPsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListOSRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  },
  serverId: {
    greaterThan: 0
  }
};
const ListOffersRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListServerDisksRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  },
  serverId: {
    greaterThan: 0
  }
};
const ListServerEventsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  },
  serverId: {
    greaterThan: 0
  }
};
const ListServersRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListServicesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListSubscribableServerOptionsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  },
  serverId: {
    greaterThan: 0
  }
};
const RebootServerRequest = {
  serverId: {
    greaterThan: 0
  }
};
const RpnApiListRpnServerCapabilitiesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnSanApiAddIpRequest = {
  rpnSanId: {
    greaterThan: 0
  }
};
const RpnSanApiDeleteRpnSanRequest = {
  rpnSanId: {
    greaterThan: 0
  }
};
const RpnSanApiGetRpnSanRequest = {
  rpnSanId: {
    greaterThan: 0
  }
};
const RpnSanApiListAvailableIpsRequest = {
  rpnSanId: {
    greaterThan: 0
  }
};
const RpnSanApiListIpsRequest = {
  rpnSanId: {
    greaterThan: 0
  }
};
const RpnSanApiListRpnSansRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnSanApiRemoveIpRequest = {
  rpnSanId: {
    greaterThan: 0
  }
};
const RpnV1ApiAcceptRpnInviteRequest = {
  memberId: {
    greaterThan: 0
  }
};
const RpnV1ApiAddRpnGroupMembersRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV1ApiDeleteRpnGroupMembersRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV1ApiDeleteRpnGroupRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV1ApiGetRpnGroupRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV1ApiLeaveRpnGroupRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV1ApiListRpnCapableSanServersRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV1ApiListRpnCapableServersRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV1ApiListRpnGroupMembersRequest = {
  groupId: {
    greaterThan: 0
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV1ApiListRpnGroupsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV1ApiListRpnInvitesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV1ApiRefuseRpnInviteRequest = {
  memberId: {
    greaterThan: 0
  }
};
const RpnV1ApiRpnGroupInviteRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV1ApiUpdateRpnGroupNameRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV2ApiAddRpnV2MembersRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV2ApiDeleteRpnV2GroupRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV2ApiDeleteRpnV2MembersRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV2ApiDisableRpnV2GroupCompatibilityRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV2ApiEnableRpnV2GroupCompatibilityRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV2ApiGetRpnV2GroupRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV2ApiListRpnV2CapableResourcesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV2ApiListRpnV2GroupLogsRequest = {
  groupId: {
    greaterThan: 0
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV2ApiListRpnV2GroupsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV2ApiListRpnV2MembersRequest = {
  groupId: {
    greaterThan: 0
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const RpnV2ApiUpdateRpnV2GroupNameRequest = {
  groupId: {
    greaterThan: 0
  }
};
const RpnV2ApiUpdateRpnV2VlanForMembersRequest = {
  groupId: {
    greaterThan: 0
  }
};
const StartRescueRequest = {
  serverId: {
    greaterThan: 0
  }
};
const StartServerRequest = {
  serverId: {
    greaterThan: 0
  }
};
const StopBMCAccessRequest = {
  serverId: {
    greaterThan: 0
  }
};
const StopRescueRequest = {
  serverId: {
    greaterThan: 0
  }
};
const StopServerRequest = {
  serverId: {
    greaterThan: 0
  }
};
const SubscribeServerOptionRequest = {
  optionId: {
    greaterThan: 0
  },
  serverId: {
    greaterThan: 0
  }
};
const SubscribeStorageOptionsRequest = {
  serverId: {
    greaterThan: 0
  }
};
const UpdateRaidRequest = {
  serverId: {
    greaterThan: 0
  }
};
const UpdateReverseRequest = {
  ipId: {
    greaterThan: 0
  }
};
const UpdateServerBackupRequest = {
  serverId: {
    greaterThan: 0
  }
};
const UpdateServerRequest = {
  serverId: {
    greaterThan: 0
  }
};
const UpdateServerTagsRequest = {
  serverId: {
    greaterThan: 0
  }
};

var validationRules_gen$d = /*#__PURE__*/Object.freeze({
  __proto__: null,
  AttachFailoverIPToMacAddressRequest: AttachFailoverIPToMacAddressRequest,
  AttachFailoverIPsRequest: AttachFailoverIPsRequest,
  BillingApiDownloadInvoiceRequest: BillingApiDownloadInvoiceRequest,
  BillingApiDownloadRefundRequest: BillingApiDownloadRefundRequest,
  BillingApiGetInvoiceRequest: BillingApiGetInvoiceRequest,
  BillingApiGetRefundRequest: BillingApiGetRefundRequest,
  BillingApiListInvoicesRequest: BillingApiListInvoicesRequest,
  BillingApiListRefundsRequest: BillingApiListRefundsRequest,
  CancelServerInstallRequest: CancelServerInstallRequest,
  DeleteFailoverIPRequest: DeleteFailoverIPRequest,
  DeleteServerRequest: DeleteServerRequest,
  DeleteServiceRequest: DeleteServiceRequest,
  DetachFailoverIPFromMacAddressRequest: DetachFailoverIPFromMacAddressRequest,
  GetBMCAccessRequest: GetBMCAccessRequest,
  GetFailoverIPRequest: GetFailoverIPRequest,
  GetOSRequest: GetOSRequest,
  GetOfferRequest: GetOfferRequest,
  GetOrderedServiceRequest: GetOrderedServiceRequest,
  GetRaidRequest: GetRaidRequest,
  GetRescueRequest: GetRescueRequest,
  GetServerBackupRequest: GetServerBackupRequest,
  GetServerInstallRequest: GetServerInstallRequest,
  GetServerRequest: GetServerRequest,
  GetServiceRequest: GetServiceRequest,
  IPv6Block: IPv6Block,
  IPv6BlockApiCreateIPv6BlockSubnetRequest: IPv6BlockApiCreateIPv6BlockSubnetRequest,
  IPv6BlockApiDeleteIPv6BlockRequest: IPv6BlockApiDeleteIPv6BlockRequest,
  IPv6BlockApiListIPv6BlockSubnetsAvailableRequest: IPv6BlockApiListIPv6BlockSubnetsAvailableRequest,
  IPv6BlockApiUpdateIPv6BlockRequest: IPv6BlockApiUpdateIPv6BlockRequest,
  InstallServerRequest: InstallServerRequest,
  ListFailoverIPsRequest: ListFailoverIPsRequest,
  ListOSRequest: ListOSRequest,
  ListOffersRequest: ListOffersRequest,
  ListServerDisksRequest: ListServerDisksRequest,
  ListServerEventsRequest: ListServerEventsRequest,
  ListServersRequest: ListServersRequest,
  ListServicesRequest: ListServicesRequest,
  ListSubscribableServerOptionsRequest: ListSubscribableServerOptionsRequest,
  RebootServerRequest: RebootServerRequest,
  RpnApiListRpnServerCapabilitiesRequest: RpnApiListRpnServerCapabilitiesRequest,
  RpnSanApiAddIpRequest: RpnSanApiAddIpRequest,
  RpnSanApiDeleteRpnSanRequest: RpnSanApiDeleteRpnSanRequest,
  RpnSanApiGetRpnSanRequest: RpnSanApiGetRpnSanRequest,
  RpnSanApiListAvailableIpsRequest: RpnSanApiListAvailableIpsRequest,
  RpnSanApiListIpsRequest: RpnSanApiListIpsRequest,
  RpnSanApiListRpnSansRequest: RpnSanApiListRpnSansRequest,
  RpnSanApiRemoveIpRequest: RpnSanApiRemoveIpRequest,
  RpnV1ApiAcceptRpnInviteRequest: RpnV1ApiAcceptRpnInviteRequest,
  RpnV1ApiAddRpnGroupMembersRequest: RpnV1ApiAddRpnGroupMembersRequest,
  RpnV1ApiDeleteRpnGroupMembersRequest: RpnV1ApiDeleteRpnGroupMembersRequest,
  RpnV1ApiDeleteRpnGroupRequest: RpnV1ApiDeleteRpnGroupRequest,
  RpnV1ApiGetRpnGroupRequest: RpnV1ApiGetRpnGroupRequest,
  RpnV1ApiLeaveRpnGroupRequest: RpnV1ApiLeaveRpnGroupRequest,
  RpnV1ApiListRpnCapableSanServersRequest: RpnV1ApiListRpnCapableSanServersRequest,
  RpnV1ApiListRpnCapableServersRequest: RpnV1ApiListRpnCapableServersRequest,
  RpnV1ApiListRpnGroupMembersRequest: RpnV1ApiListRpnGroupMembersRequest,
  RpnV1ApiListRpnGroupsRequest: RpnV1ApiListRpnGroupsRequest,
  RpnV1ApiListRpnInvitesRequest: RpnV1ApiListRpnInvitesRequest,
  RpnV1ApiRefuseRpnInviteRequest: RpnV1ApiRefuseRpnInviteRequest,
  RpnV1ApiRpnGroupInviteRequest: RpnV1ApiRpnGroupInviteRequest,
  RpnV1ApiUpdateRpnGroupNameRequest: RpnV1ApiUpdateRpnGroupNameRequest,
  RpnV2ApiAddRpnV2MembersRequest: RpnV2ApiAddRpnV2MembersRequest,
  RpnV2ApiDeleteRpnV2GroupRequest: RpnV2ApiDeleteRpnV2GroupRequest,
  RpnV2ApiDeleteRpnV2MembersRequest: RpnV2ApiDeleteRpnV2MembersRequest,
  RpnV2ApiDisableRpnV2GroupCompatibilityRequest: RpnV2ApiDisableRpnV2GroupCompatibilityRequest,
  RpnV2ApiEnableRpnV2GroupCompatibilityRequest: RpnV2ApiEnableRpnV2GroupCompatibilityRequest,
  RpnV2ApiGetRpnV2GroupRequest: RpnV2ApiGetRpnV2GroupRequest,
  RpnV2ApiListRpnV2CapableResourcesRequest: RpnV2ApiListRpnV2CapableResourcesRequest,
  RpnV2ApiListRpnV2GroupLogsRequest: RpnV2ApiListRpnV2GroupLogsRequest,
  RpnV2ApiListRpnV2GroupsRequest: RpnV2ApiListRpnV2GroupsRequest,
  RpnV2ApiListRpnV2MembersRequest: RpnV2ApiListRpnV2MembersRequest,
  RpnV2ApiUpdateRpnV2GroupNameRequest: RpnV2ApiUpdateRpnV2GroupNameRequest,
  RpnV2ApiUpdateRpnV2VlanForMembersRequest: RpnV2ApiUpdateRpnV2VlanForMembersRequest,
  StartRescueRequest: StartRescueRequest,
  StartServerRequest: StartServerRequest,
  StopBMCAccessRequest: StopBMCAccessRequest,
  StopRescueRequest: StopRescueRequest,
  StopServerRequest: StopServerRequest,
  SubscribeServerOptionRequest: SubscribeServerOptionRequest,
  SubscribeStorageOptionsRequest: SubscribeStorageOptionsRequest,
  UpdateRaidRequest: UpdateRaidRequest,
  UpdateReverseRequest: UpdateReverseRequest,
  UpdateServerBackupRequest: UpdateServerBackupRequest,
  UpdateServerRequest: UpdateServerRequest,
  UpdateServerTagsRequest: UpdateServerTagsRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$p = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$q,
  BMC_ACCESS_TRANSIENT_STATUSES: BMC_ACCESS_TRANSIENT_STATUSES,
  BillingAPI: BillingAPI,
  IPv6BlockAPI: IPv6BlockAPI,
  I_PV6_BLOCK_DELEGATION_TRANSIENT_STATUSES: I_PV6_BLOCK_DELEGATION_TRANSIENT_STATUSES,
  RPN_GROUP_MEMBER_TRANSIENT_STATUSES: RPN_GROUP_MEMBER_TRANSIENT_STATUSES,
  RPN_SAN_TRANSIENT_STATUSES: RPN_SAN_TRANSIENT_STATUSES,
  RPN_V2_GROUP_TRANSIENT_STATUSES: RPN_V2_GROUP_TRANSIENT_STATUSES,
  RPN_V2_MEMBER_TRANSIENT_STATUSES: RPN_V2_MEMBER_TRANSIENT_STATUSES,
  RpnAPI: RpnAPI,
  RpnSanAPI: RpnSanAPI,
  RpnV1API: RpnV1API,
  RpnV2API: RpnV2API,
  SERVER_INSTALL_TRANSIENT_STATUSES: SERVER_INSTALL_TRANSIENT_STATUSES,
  SERVER_TRANSIENT_STATUSES: SERVER_TRANSIENT_STATUSES$1,
  SERVICE_PROVISIONING_TRANSIENT_STATUSES: SERVICE_PROVISIONING_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen$d
});

var index$s = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$p
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link InstanceLogStatus}. */
const INSTANCE_LOG_TRANSIENT_STATUSES$1 = ['creating'];

/** Lists transient statutes of the enum {@link InstanceStatus}. */
const INSTANCE_TRANSIENT_STATUSES$2 = ['provisioning', 'configuring', 'deleting', 'autohealing', 'initializing', 'backuping', 'snapshotting', 'restarting'];

/** Lists transient statutes of the enum {@link MaintenanceStatus}. */
const MAINTENANCE_TRANSIENT_STATUSES$1 = ['ongoing'];

/** Lists transient statutes of the enum {@link ReadReplicaStatus}. */
const READ_REPLICA_TRANSIENT_STATUSES$1 = ['provisioning', 'initializing', 'deleting', 'configuring', 'promoting'];

/** Lists transient statutes of the enum {@link SnapshotStatus}. */
const SNAPSHOT_TRANSIENT_STATUSES$2 = ['creating', 'restoring', 'deleting'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalEndpointDirectAccessDetails$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EndpointDirectAccessDetails' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalEndpointLoadBalancerDetails$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EndpointLoadBalancerDetails' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalEndpointPrivateNetworkDetails$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EndpointPrivateNetworkDetails' failed as data isn't a dictionary.`);
  }
  return {
    privateNetworkId: data.private_network_id,
    serviceIp: data.service_ip,
    zone: data.zone
  };
};
const unmarshalEndpoint$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Endpoint' failed as data isn't a dictionary.`);
  }
  return {
    directAccess: data.direct_access ? unmarshalEndpointDirectAccessDetails$1(data.direct_access) : undefined,
    hostname: data.hostname,
    id: data.id,
    ip: data.ip,
    loadBalancer: data.load_balancer ? unmarshalEndpointLoadBalancerDetails$1(data.load_balancer) : undefined,
    name: data.name,
    port: data.port,
    privateNetwork: data.private_network ? unmarshalEndpointPrivateNetworkDetails$2(data.private_network) : undefined
  };
};
const unmarshalMaintenance$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Maintenance' failed as data isn't a dictionary.`);
  }
  return {
    closedAt: unmarshalDate(data.closed_at),
    forcedAt: unmarshalDate(data.forced_at),
    reason: data.reason,
    startsAt: unmarshalDate(data.starts_at),
    status: data.status,
    stopsAt: unmarshalDate(data.stops_at)
  };
};
const unmarshalReadReplica$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ReadReplica' failed as data isn't a dictionary.`);
  }
  return {
    endpoints: unmarshalArrayOfObject(data.endpoints, unmarshalEndpoint$3),
    id: data.id,
    region: data.region,
    sameZone: data.same_zone,
    status: data.status
  };
};
const unmarshalDatabase$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Database' failed as data isn't a dictionary.`);
  }
  return {
    managed: data.managed,
    name: data.name,
    owner: data.owner,
    size: data.size
  };
};
const unmarshalInstanceLog$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'InstanceLog' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    downloadUrl: data.download_url,
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    nodeName: data.node_name,
    region: data.region,
    status: data.status
  };
};
const unmarshalBackupSchedule$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BackupSchedule' failed as data isn't a dictionary.`);
  }
  return {
    disabled: data.disabled,
    frequency: data.frequency,
    nextRunAt: unmarshalDate(data.next_run_at),
    retention: data.retention
  };
};
const unmarshalInstanceSetting$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'InstanceSetting' failed as data isn't a dictionary.`);
  }
  return {
    name: data.name,
    value: data.value
  };
};
const unmarshalLogsPolicy$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'LogsPolicy' failed as data isn't a dictionary.`);
  }
  return {
    maxAgeRetention: data.max_age_retention,
    totalDiskRetention: data.total_disk_retention
  };
};
const unmarshalUpgradableVersion$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpgradableVersion' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    minorVersion: data.minor_version,
    name: data.name,
    version: data.version
  };
};
const unmarshalVolume$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Volume' failed as data isn't a dictionary.`);
  }
  return {
    class: data.class,
    size: data.size,
    type: data.type
  };
};
const unmarshalInstance$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Instance' failed as data isn't a dictionary.`);
  }
  return {
    backupSameRegion: data.backup_same_region,
    backupSchedule: data.backup_schedule ? unmarshalBackupSchedule$1(data.backup_schedule) : undefined,
    createdAt: unmarshalDate(data.created_at),
    endpoint: data.endpoint ? unmarshalEndpoint$3(data.endpoint) : undefined,
    endpoints: unmarshalArrayOfObject(data.endpoints, unmarshalEndpoint$3),
    engine: data.engine,
    id: data.id,
    initSettings: unmarshalArrayOfObject(data.init_settings, unmarshalInstanceSetting$1),
    isHaCluster: data.is_ha_cluster,
    logsPolicy: data.logs_policy ? unmarshalLogsPolicy$1(data.logs_policy) : undefined,
    maintenances: unmarshalArrayOfObject(data.maintenances, unmarshalMaintenance$1),
    name: data.name,
    nodeType: data.node_type,
    organizationId: data.organization_id,
    projectId: data.project_id,
    readReplicas: unmarshalArrayOfObject(data.read_replicas, unmarshalReadReplica$1),
    region: data.region,
    settings: unmarshalArrayOfObject(data.settings, unmarshalInstanceSetting$1),
    status: data.status,
    tags: data.tags,
    upgradableVersion: unmarshalArrayOfObject(data.upgradable_version, unmarshalUpgradableVersion$1),
    volume: data.volume ? unmarshalVolume$3(data.volume) : undefined
  };
};
const unmarshalPrivilege$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Privilege' failed as data isn't a dictionary.`);
  }
  return {
    databaseName: data.database_name,
    permission: data.permission,
    userName: data.user_name
  };
};
const unmarshalSnapshotVolumeType$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SnapshotVolumeType' failed as data isn't a dictionary.`);
  }
  return {
    class: data.class,
    type: data.type
  };
};
const unmarshalSnapshot$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Snapshot' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    instanceId: data.instance_id,
    instanceName: data.instance_name,
    name: data.name,
    nodeType: data.node_type,
    region: data.region,
    size: data.size,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at),
    volumeType: data.volume_type ? unmarshalSnapshotVolumeType$1(data.volume_type) : undefined
  };
};
const unmarshalUser$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'User' failed as data isn't a dictionary.`);
  }
  return {
    isAdmin: data.is_admin,
    name: data.name
  };
};
const unmarshalACLRule$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ACLRule' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    description: data.description,
    direction: data.direction,
    ip: data.ip,
    port: data.port,
    protocol: data.protocol
  };
};
const unmarshalAddInstanceACLRulesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AddInstanceACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$3)
  };
};
const unmarshalAddInstanceSettingsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AddInstanceSettingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    settings: unmarshalArrayOfObject(data.settings, unmarshalInstanceSetting$1)
  };
};
const unmarshalDeleteInstanceACLRulesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeleteInstanceACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$3)
  };
};
const unmarshalDeleteInstanceSettingsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeleteInstanceSettingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    settings: unmarshalArrayOfObject(data.settings, unmarshalInstanceSetting$1)
  };
};
const unmarshalInstanceMetrics$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'InstanceMetrics' failed as data isn't a dictionary.`);
  }
  return {
    timeseries: unmarshalArrayOfObject(data.timeseries, unmarshalTimeSeries)
  };
};
const unmarshalEngineSetting$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EngineSetting' failed as data isn't a dictionary.`);
  }
  return {
    defaultValue: data.default_value,
    description: data.description,
    floatMax: data.float_max,
    floatMin: data.float_min,
    hotConfigurable: data.hot_configurable,
    intMax: data.int_max,
    intMin: data.int_min,
    name: data.name,
    propertyType: data.property_type,
    stringConstraint: data.string_constraint,
    unit: data.unit
  };
};
const unmarshalEngineVersion$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EngineVersion' failed as data isn't a dictionary.`);
  }
  return {
    availableInitSettings: unmarshalArrayOfObject(data.available_init_settings, unmarshalEngineSetting$1),
    availableSettings: unmarshalArrayOfObject(data.available_settings, unmarshalEngineSetting$1),
    beta: data.beta,
    disabled: data.disabled,
    endOfLife: unmarshalDate(data.end_of_life),
    name: data.name,
    version: data.version
  };
};
const unmarshalDatabaseEngine$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DatabaseEngine' failed as data isn't a dictionary.`);
  }
  return {
    logoUrl: data.logo_url,
    name: data.name,
    region: data.region,
    versions: unmarshalArrayOfObject(data.versions, unmarshalEngineVersion$1)
  };
};
const unmarshalListDatabaseEnginesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDatabaseEnginesResponse' failed as data isn't a dictionary.`);
  }
  return {
    engines: unmarshalArrayOfObject(data.engines, unmarshalDatabaseEngine$1),
    totalCount: data.total_count
  };
};
const unmarshalListDatabasesResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDatabasesResponse' failed as data isn't a dictionary.`);
  }
  return {
    databases: unmarshalArrayOfObject(data.databases, unmarshalDatabase$2),
    totalCount: data.total_count
  };
};
const unmarshalListInstanceACLRulesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstanceACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$3),
    totalCount: data.total_count
  };
};
const unmarshalListInstanceLogsDetailsResponseInstanceLogDetail$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstanceLogsDetailsResponseInstanceLogDetail' failed as data isn't a dictionary.`);
  }
  return {
    logName: data.log_name,
    size: data.size
  };
};
const unmarshalListInstanceLogsDetailsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstanceLogsDetailsResponse' failed as data isn't a dictionary.`);
  }
  return {
    details: unmarshalArrayOfObject(data.details, unmarshalListInstanceLogsDetailsResponseInstanceLogDetail$1)
  };
};
const unmarshalListInstanceLogsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstanceLogsResponse' failed as data isn't a dictionary.`);
  }
  return {
    instanceLogs: unmarshalArrayOfObject(data.instance_logs, unmarshalInstanceLog$1)
  };
};
const unmarshalListInstancesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstancesResponse' failed as data isn't a dictionary.`);
  }
  return {
    instances: unmarshalArrayOfObject(data.instances, unmarshalInstance$2),
    totalCount: data.total_count
  };
};
const unmarshalNodeTypeVolumeConstraintSizes$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NodeTypeVolumeConstraintSizes' failed as data isn't a dictionary.`);
  }
  return {
    maxSize: data.max_size,
    minSize: data.min_size
  };
};
const unmarshalNodeTypeVolumeType$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NodeTypeVolumeType' failed as data isn't a dictionary.`);
  }
  return {
    chunkSize: data.chunk_size,
    class: data.class,
    description: data.description,
    maxSize: data.max_size,
    minSize: data.min_size,
    type: data.type
  };
};
const unmarshalNodeType$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NodeType' failed as data isn't a dictionary.`);
  }
  return {
    availableVolumeTypes: unmarshalArrayOfObject(data.available_volume_types, unmarshalNodeTypeVolumeType$1),
    beta: data.beta,
    description: data.description,
    disabled: data.disabled,
    generation: data.generation,
    instanceRange: data.instance_range,
    isBssdCompatible: data.is_bssd_compatible,
    isHaRequired: data.is_ha_required,
    memory: data.memory,
    name: data.name,
    region: data.region,
    stockStatus: data.stock_status,
    vcpus: data.vcpus,
    volumeConstraint: data.volume_constraint ? unmarshalNodeTypeVolumeConstraintSizes$1(data.volume_constraint) : undefined
  };
};
const unmarshalListNodeTypesResponse$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNodeTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    nodeTypes: unmarshalArrayOfObject(data.node_types, unmarshalNodeType$3),
    totalCount: data.total_count
  };
};
const unmarshalListPrivilegesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPrivilegesResponse' failed as data isn't a dictionary.`);
  }
  return {
    privileges: unmarshalArrayOfObject(data.privileges, unmarshalPrivilege$1),
    totalCount: data.total_count
  };
};
const unmarshalListSnapshotsResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSnapshotsResponse' failed as data isn't a dictionary.`);
  }
  return {
    snapshots: unmarshalArrayOfObject(data.snapshots, unmarshalSnapshot$2),
    totalCount: data.total_count
  };
};
const unmarshalListUsersResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListUsersResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    users: unmarshalArrayOfObject(data.users, unmarshalUser$2)
  };
};
const unmarshalSetInstanceACLRulesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetInstanceACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$3)
  };
};
const unmarshalSetInstanceSettingsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetInstanceSettingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    settings: unmarshalArrayOfObject(data.settings, unmarshalInstanceSetting$1)
  };
};
const marshalACLRuleRequest$2 = (request, defaults) => ({
  description: request.description,
  ip: request.ip
});
const marshalAddInstanceACLRulesRequest$1 = (request, defaults) => ({
  rules: request.rules.map(elt => marshalACLRuleRequest$2(elt))
});
const marshalInstanceSetting$1 = (request, defaults) => ({
  name: request.name,
  value: request.value
});
const marshalAddInstanceSettingsRequest$1 = (request, defaults) => ({
  settings: request.settings.map(elt => marshalInstanceSetting$1(elt))
});
const marshalCloneInstanceRequest$1 = (request, defaults) => ({
  name: request.name,
  node_type: request.nodeType
});
const marshalCreateDatabaseRequest$2 = (request, defaults) => ({
  name: request.name
});
const marshalEndpointSpecPrivateNetworkIpamConfig$1 = (request, defaults) => ({});
const marshalEndpointSpecLoadBalancer$1 = (request, defaults) => ({});
const marshalEndpointSpecPrivateNetwork$2 = (request, defaults) => ({
  private_network_id: request.privateNetworkId,
  ...resolveOneOf([{
    param: 'service_ip',
    value: request.serviceIp
  }, {
    param: 'ipam_config',
    value: request.ipamConfig !== undefined ? marshalEndpointSpecPrivateNetworkIpamConfig$1(request.ipamConfig) : undefined
  }])
});
const marshalEndpointSpec$3 = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'load_balancer',
    value: request.loadBalancer !== undefined ? marshalEndpointSpecLoadBalancer$1(request.loadBalancer) : undefined
  }, {
    param: 'private_network',
    value: request.privateNetwork !== undefined ? marshalEndpointSpecPrivateNetwork$2(request.privateNetwork) : undefined
  }])
});
const marshalCreateEndpointRequest$2 = (request, defaults) => ({
  endpoint_spec: request.endpointSpec !== undefined ? marshalEndpointSpec$3(request.endpointSpec) : undefined
});
const marshalCreateInstanceFromSnapshotRequest$1 = (request, defaults) => ({
  instance_name: request.instanceName,
  is_ha_cluster: request.isHaCluster,
  node_type: request.nodeType
});
const marshalCreateInstanceRequest$1 = (request, defaults) => ({
  backup_same_region: request.backupSameRegion,
  disable_backup: request.disableBackup,
  engine: request.engine,
  init_endpoints: request.initEndpoints !== undefined ? request.initEndpoints.map(elt => marshalEndpointSpec$3(elt)) : undefined,
  init_settings: request.initSettings !== undefined ? request.initSettings.map(elt => marshalInstanceSetting$1(elt)) : undefined,
  is_ha_cluster: request.isHaCluster,
  name: request.name || randomName('ins'),
  node_type: request.nodeType,
  password: request.password,
  tags: request.tags,
  user_name: request.userName,
  volume_size: request.volumeSize,
  volume_type: request.volumeType,
  ...resolveOneOf([{
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }, {
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }])
});
const marshalReadReplicaEndpointSpecPrivateNetworkIpamConfig$1 = (request, defaults) => ({});
const marshalReadReplicaEndpointSpecDirectAccess$1 = (request, defaults) => ({});
const marshalReadReplicaEndpointSpecPrivateNetwork$1 = (request, defaults) => ({
  private_network_id: request.privateNetworkId,
  ...resolveOneOf([{
    param: 'service_ip',
    value: request.serviceIp
  }, {
    param: 'ipam_config',
    value: request.ipamConfig !== undefined ? marshalReadReplicaEndpointSpecPrivateNetworkIpamConfig$1(request.ipamConfig) : undefined
  }])
});
const marshalReadReplicaEndpointSpec$1 = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'direct_access',
    value: request.directAccess !== undefined ? marshalReadReplicaEndpointSpecDirectAccess$1(request.directAccess) : undefined
  }, {
    param: 'private_network',
    value: request.privateNetwork !== undefined ? marshalReadReplicaEndpointSpecPrivateNetwork$1(request.privateNetwork) : undefined
  }])
});
const marshalCreateReadReplicaEndpointRequest$1 = (request, defaults) => ({
  endpoint_spec: request.endpointSpec.map(elt => marshalReadReplicaEndpointSpec$1(elt))
});
const marshalCreateReadReplicaRequest$1 = (request, defaults) => ({
  endpoint_spec: request.endpointSpec !== undefined ? request.endpointSpec.map(elt => marshalReadReplicaEndpointSpec$1(elt)) : undefined,
  instance_id: request.instanceId,
  same_zone: request.sameZone
});
const marshalCreateSnapshotRequest$2 = (request, defaults) => ({
  expires_at: request.expiresAt,
  name: request.name || randomName('snp')
});
const marshalCreateUserRequest$2 = (request, defaults) => ({
  is_admin: request.isAdmin,
  name: request.name,
  password: request.password
});
const marshalDeleteInstanceACLRulesRequest$1 = (request, defaults) => ({
  acl_rule_ips: request.aclRuleIps
});
const marshalDeleteInstanceSettingsRequest$1 = (request, defaults) => ({
  setting_names: request.settingNames
});
const marshalMigrateEndpointRequest$1 = (request, defaults) => ({
  instance_id: request.instanceId
});
const marshalPurgeInstanceLogsRequest$1 = (request, defaults) => ({
  log_name: request.logName
});
const marshalSetInstanceACLRulesRequest$1 = (request, defaults) => ({
  rules: request.rules.map(elt => marshalACLRuleRequest$2(elt))
});
const marshalSetInstanceSettingsRequest$1 = (request, defaults) => ({
  settings: request.settings.map(elt => marshalInstanceSetting$1(elt))
});
const marshalSetPrivilegeRequest$1 = (request, defaults) => ({
  database_name: request.databaseName,
  permission: request.permission,
  user_name: request.userName
});
const marshalLogsPolicy$1 = (request, defaults) => ({
  max_age_retention: request.maxAgeRetention,
  total_disk_retention: request.totalDiskRetention
});
const marshalUpdateInstanceRequest$1 = (request, defaults) => ({
  backup_same_region: request.backupSameRegion,
  backup_schedule_frequency: request.backupScheduleFrequency,
  backup_schedule_retention: request.backupScheduleRetention,
  backup_schedule_start_hour: request.backupScheduleStartHour,
  is_backup_schedule_disabled: request.isBackupScheduleDisabled,
  logs_policy: request.logsPolicy !== undefined ? marshalLogsPolicy$1(request.logsPolicy) : undefined,
  name: request.name,
  tags: request.tags
});
const marshalUpdateSnapshotRequest$2 = (request, defaults) => ({
  expires_at: request.expiresAt,
  name: request.name
});
const marshalUpdateUserRequest$2 = (request, defaults) => ({
  is_admin: request.isAdmin,
  password: request.password
});
const marshalUpgradeInstanceRequestMajorUpgradeWorkflow$1 = (request, defaults) => ({
  upgradable_version_id: request.upgradableVersionId,
  with_endpoints: request.withEndpoints
});
const marshalUpgradeInstanceRequest$1 = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'node_type',
    value: request.nodeType
  }, {
    param: 'enable_ha',
    value: request.enableHa
  }, {
    param: 'volume_size',
    value: request.volumeSize
  }, {
    param: 'volume_type',
    value: request.volumeType
  }, {
    param: 'upgradable_version_id',
    value: request.upgradableVersionId
  }, {
    param: 'major_upgrade_workflow',
    value: request.majorUpgradeWorkflow !== undefined ? marshalUpgradeInstanceRequestMajorUpgradeWorkflow$1(request.majorUpgradeWorkflow) : undefined
  }])
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$p = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Managed Document Databases API. */
let API$p = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];
  pageOfListDatabaseEngines = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/database-engines`,
    urlParams: urlParams(['name', request.name], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['version', request.version])
  }, unmarshalListDatabaseEnginesResponse$1);

  /**
   * List available database engines. List the FerretDB database engines
   * available at Scaleway.
   *
   * @param request - The request {@link ListDatabaseEnginesRequest}
   * @returns A Promise of ListDatabaseEnginesResponse
   */
  listDatabaseEngines = (request = {}) => enrichForPagination('engines', this.pageOfListDatabaseEngines, request);
  pageOfListNodeTypes = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/node-types`,
    urlParams: urlParams(['include_disabled_types', request.includeDisabledTypes], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListNodeTypesResponse$3);

  /**
   * List available node types. List all available node types. By default, the
   * node types returned in the list are ordered by creation date in ascending
   * order, though this can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListNodeTypesRequest}
   * @returns A Promise of ListNodeTypesResponse
   */
  listNodeTypes = request => enrichForPagination('nodeTypes', this.pageOfListNodeTypes, request);

  /**
   * Upgrade a Database Instance. Upgrade your current Database Instance
   * specifications like node type, high availability, volume, or the database
   * engine version. Note that upon upgrade the `enable_ha` parameter can only
   * be set to `true`.
   *
   * @param request - The request {@link UpgradeInstanceRequest}
   * @returns A Promise of Instance
   */
  upgradeInstance = request => this.client.fetch({
    body: JSON.stringify(marshalUpgradeInstanceRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/upgrade`
  }, unmarshalInstance$2);
  pageOfListInstances = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalListInstancesResponse$1);

  /**
   * List Database Instances. List all Database Instances in the specified
   * region, for a given Scaleway Organization or Scaleway Project. By default,
   * the Database Instances returned in the list are ordered by creation date in
   * ascending order, though this can be modified via the order_by field. You
   * can define additional parameters for your query, such as `tags` and `name`.
   * For the `name` parameter, the value you include will be checked against the
   * whole name string to see if it includes the string you put in the
   * parameter.
   *
   * @param request - The request {@link ListInstancesRequest}
   * @returns A Promise of ListInstancesResponse
   */
  listInstances = (request = {}) => enrichForPagination('instances', this.pageOfListInstances, request);

  /**
   * Get a Database Instance. Retrieve information about a given Database
   * Instance, specified by the `region` and `instance_id` parameters. Its full
   * details, including name, status, IP address and port, are returned in the
   * response object.
   *
   * @param request - The request {@link GetInstanceRequest}
   * @returns A Promise of Instance
   */
  getInstance = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}`
  }, unmarshalInstance$2);

  /**
   * Waits for {@link Instance} to be in a final state.
   *
   * @param request - The request {@link GetInstanceRequest}
   * @param options - The waiting options
   * @returns A Promise of Instance
   */
  waitForInstance = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!INSTANCE_TRANSIENT_STATUSES$2.includes(res.status))), this.getInstance, request, options);

  /**
   * Create a Database Instance. Create a new Database Instance. You must set
   * the `engine`, `user_name`, `password` and `node_type` parameters.
   * Optionally, you can specify the volume type and size.
   *
   * @param request - The request {@link CreateInstanceRequest}
   * @returns A Promise of Instance
   */
  createInstance = request => this.client.fetch({
    body: JSON.stringify(marshalCreateInstanceRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances`
  }, unmarshalInstance$2);

  /**
   * Update a Database Instance. Update the parameters of a Database Instance,
   * including name, tags and backup schedule details.
   *
   * @param request - The request {@link UpdateInstanceRequest}
   * @returns A Promise of Instance
   */
  updateInstance = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateInstanceRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'PATCH',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}`
  }, unmarshalInstance$2);

  /**
   * Delete a Database Instance. Delete a given Database Instance, specified by
   * the `region` and `instance_id` parameters. Deleting a Database Instance is
   * permanent, and cannot be undone. Note that upon deletion all your data will
   * be lost.
   *
   * @param request - The request {@link DeleteInstanceRequest}
   * @returns A Promise of Instance
   */
  deleteInstance = request => this.client.fetch({
    method: 'DELETE',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}`
  }, unmarshalInstance$2);

  /**
   * Clone a Database Instance. Clone a given Database Instance, specified by
   * the `region` and `instance_id` parameters. The clone feature allows you to
   * create a new Database Instance from an existing one. The clone includes all
   * existing databases, users and permissions. You can create a clone on a
   * Database Instance bigger than your current one.
   *
   * @param request - The request {@link CloneInstanceRequest}
   * @returns A Promise of Instance
   */
  cloneInstance = request => this.client.fetch({
    body: JSON.stringify(marshalCloneInstanceRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/clone`
  }, unmarshalInstance$2);

  /**
   * Restart Database Instance. Restart a given Database Instance, specified by
   * the `region` and `instance_id` parameters. The status of the Database
   * Instance returned in the response.
   *
   * @param request - The request {@link RestartInstanceRequest}
   * @returns A Promise of Instance
   */
  restartInstance = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/restart`
  }, unmarshalInstance$2);

  /**
   * Get the TLS certificate of a Database Instance. Retrieve information about
   * the TLS certificate of a given Database Instance. Details like name and
   * content are returned in the response.
   *
   * @param request - The request {@link GetInstanceCertificateRequest}
   * @returns A Promise of Blob
   */
  getInstanceCertificate = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/certificate`,
    urlParams: urlParams(['dl', 1]),
    responseType: 'blob'
  });

  /**
   * Renew the TLS certificate of a Database Instance. Renew a TLS for a
   * Database Instance. Renewing a certificate means that you will not be able
   * to connect to your Database Instance using the previous certificate. You
   * will also need to download and update the new certificate for all database
   * clients.
   *
   * @param request - The request {@link RenewInstanceCertificateRequest}
   */
  renewInstanceCertificate = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/renew-certificate`
  });

  /**
   * Get Database Instance metrics. Retrieve the time series metrics of a given
   * Database Instance. You can define the period from which to retrieve metrics
   * by specifying the `start_date` and `end_date`.
   *
   * @param request - The request {@link GetInstanceMetricsRequest}
   * @returns A Promise of InstanceMetrics
   */
  getInstanceMetrics = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/metrics`,
    urlParams: urlParams(['end_date', request.endDate], ['metric_name', request.metricName], ['start_date', request.startDate])
  }, unmarshalInstanceMetrics$1);

  /**
   * Create a Read Replica. Create a new Read Replica of a Database Instance.
   * You must specify the `region` and the `instance_id`. You can only create a
   * maximum of 3 Read Replicas per Database Instance.
   *
   * @param request - The request {@link CreateReadReplicaRequest}
   * @returns A Promise of ReadReplica
   */
  createReadReplica = request => this.client.fetch({
    body: JSON.stringify(marshalCreateReadReplicaRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas`
  }, unmarshalReadReplica$1);

  /**
   * Get a Read Replica. Retrieve information about a Database Instance Read
   * Replica. Full details about the Read Replica, like `endpoints`, `status`
   * and `region` are returned in the response.
   *
   * @param request - The request {@link GetReadReplicaRequest}
   * @returns A Promise of ReadReplica
   */
  getReadReplica = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}`
  }, unmarshalReadReplica$1);

  /**
   * Waits for {@link ReadReplica} to be in a final state.
   *
   * @param request - The request {@link GetReadReplicaRequest}
   * @param options - The waiting options
   * @returns A Promise of ReadReplica
   */
  waitForReadReplica = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!READ_REPLICA_TRANSIENT_STATUSES$1.includes(res.status))), this.getReadReplica, request, options);

  /**
   * Delete a Read Replica. Delete a Read Replica of a Database Instance. You
   * must specify the `region` and `read_replica_id` parameters of the Read
   * Replica you want to delete.
   *
   * @param request - The request {@link DeleteReadReplicaRequest}
   * @returns A Promise of ReadReplica
   */
  deleteReadReplica = request => this.client.fetch({
    method: 'DELETE',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}`
  }, unmarshalReadReplica$1);

  /**
   * Resync a Read Replica. When you resync a Read Replica, first it is reset,
   * then its data is resynchronized from the primary node. Your Read Replica
   * remains unavailable during the resync process. The duration of this process
   * is proportional to the size of your Database Instance. The configured
   * endpoints do not change.
   *
   * @param request - The request {@link ResetReadReplicaRequest}
   * @returns A Promise of ReadReplica
   */
  resetReadReplica = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}/reset`
  }, unmarshalReadReplica$1);

  /**
   * Promote a Read Replica. Promote a Read Replica to Database Instance
   * automatically.
   *
   * @param request - The request {@link PromoteReadReplicaRequest}
   * @returns A Promise of Instance
   */
  promoteReadReplica = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}/promote`
  }, unmarshalInstance$2);

  /**
   * Create an endpoint for a Read Replica. Create a new endpoint for a Read
   * Replica. Read Replicas can have at most one direct access and one Private
   * Network endpoint.
   *
   * @param request - The request {@link CreateReadReplicaEndpointRequest}
   * @returns A Promise of ReadReplica
   */
  createReadReplicaEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalCreateReadReplicaEndpointRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}/endpoints`
  }, unmarshalReadReplica$1);

  /**
   * List available logs of a Database Instance. List the available logs of a
   * Database Instance. By default, the logs returned in the list are ordered by
   * creation date in ascending order, though this can be modified via the
   * order_by field.
   *
   * @param request - The request {@link ListInstanceLogsRequest}
   * @returns A Promise of ListInstanceLogsResponse
   */
  listInstanceLogs = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/logs`,
    urlParams: urlParams(['order_by', request.orderBy])
  }, unmarshalListInstanceLogsResponse$1);

  /**
   * Get given logs of a Database Instance. Retrieve information about the logs
   * of a Database Instance. Specify the `instance_log_id` and `region` in your
   * request to get information such as `download_url`, `status`, `expires_at`
   * and `created_at` about your logs in the response.
   *
   * @param request - The request {@link GetInstanceLogRequest}
   * @returns A Promise of InstanceLog
   */
  getInstanceLog = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/logs/${validatePathParam('instanceLogId', request.instanceLogId)}`
  }, unmarshalInstanceLog$1);

  /**
   * Waits for {@link InstanceLog} to be in a final state.
   *
   * @param request - The request {@link GetInstanceLogRequest}
   * @param options - The waiting options
   * @returns A Promise of InstanceLog
   */
  waitForInstanceLog = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!INSTANCE_LOG_TRANSIENT_STATUSES$1.includes(res.status))), this.getInstanceLog, request, options);

  /**
   * Purge remote Database Instance logs. Purge a given remote log from a
   * Database Instance. You can specify the `log_name` of the log you wish to
   * clean from your Database Instance.
   *
   * @param request - The request {@link PurgeInstanceLogsRequest}
   */
  purgeInstanceLogs = request => this.client.fetch({
    body: JSON.stringify(marshalPurgeInstanceLogsRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/purge-logs`
  });

  /**
   * List remote Database Instance logs details. List remote log details. By
   * default, the details returned in the list are ordered by creation date in
   * ascending order, though this can be modified via the order_by field.
   *
   * @param request - The request {@link ListInstanceLogsDetailsRequest}
   * @returns A Promise of ListInstanceLogsDetailsResponse
   */
  listInstanceLogsDetails = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/logs-details`
  }, unmarshalListInstanceLogsDetailsResponse$1);

  /**
   * Add Database Instance advanced settings. Add an advanced setting to a
   * Database Instance. You must set the `name` and the `value` of each
   * setting.
   *
   * @param request - The request {@link AddInstanceSettingsRequest}
   * @returns A Promise of AddInstanceSettingsResponse
   */
  addInstanceSettings = request => this.client.fetch({
    body: JSON.stringify(marshalAddInstanceSettingsRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/settings`
  }, unmarshalAddInstanceSettingsResponse$1);

  /**
   * Delete Database Instance advanced settings. Delete an advanced setting in a
   * Database Instance. You must specify the names of the settings you want to
   * delete in the request.
   *
   * @param request - The request {@link DeleteInstanceSettingsRequest}
   * @returns A Promise of DeleteInstanceSettingsResponse
   */
  deleteInstanceSettings = request => this.client.fetch({
    body: JSON.stringify(marshalDeleteInstanceSettingsRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'DELETE',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/settings`
  }, unmarshalDeleteInstanceSettingsResponse$1);

  /**
   * Set Database Instance advanced settings. Update an advanced setting for a
   * Database Instance. Settings added upon database engine initalization can
   * only be defined once, and cannot, therefore, be updated.
   *
   * @param request - The request {@link SetInstanceSettingsRequest}
   * @returns A Promise of SetInstanceSettingsResponse
   */
  setInstanceSettings = request => this.client.fetch({
    body: JSON.stringify(marshalSetInstanceSettingsRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'PUT',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/settings`
  }, unmarshalSetInstanceSettingsResponse$1);
  pageOfListInstanceACLRules = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/acls`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListInstanceACLRulesResponse$1);

  /**
   * List ACL rules of a Database Instance. List the ACL rules for a given
   * Database Instance. The response is an array of ACL objects, each one
   * representing an ACL that denies, allows or redirects traffic based on
   * certain conditions.
   *
   * @param request - The request {@link ListInstanceACLRulesRequest}
   * @returns A Promise of ListInstanceACLRulesResponse
   */
  listInstanceACLRules = request => enrichForPagination('rules', this.pageOfListInstanceACLRules, request);

  /**
   * Add an ACL rule to a Database Instance. Add an additional ACL rule to a
   * Database Instance.
   *
   * @param request - The request {@link AddInstanceACLRulesRequest}
   * @returns A Promise of AddInstanceACLRulesResponse
   */
  addInstanceACLRules = request => this.client.fetch({
    body: JSON.stringify(marshalAddInstanceACLRulesRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/acls`
  }, unmarshalAddInstanceACLRulesResponse$1);

  /**
   * Set ACL rules for a Database Instance. Replace all the ACL rules of a
   * Database Instance.
   *
   * @param request - The request {@link SetInstanceACLRulesRequest}
   * @returns A Promise of SetInstanceACLRulesResponse
   */
  setInstanceACLRules = request => this.client.fetch({
    body: JSON.stringify(marshalSetInstanceACLRulesRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'PUT',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/acls`
  }, unmarshalSetInstanceACLRulesResponse$1);

  /**
   * Delete ACL rules of a Database Instance. Delete one or more ACL rules of a
   * Database Instance.
   *
   * @param request - The request {@link DeleteInstanceACLRulesRequest}
   * @returns A Promise of DeleteInstanceACLRulesResponse
   */
  deleteInstanceACLRules = request => this.client.fetch({
    body: JSON.stringify(marshalDeleteInstanceACLRulesRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'DELETE',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/acls`
  }, unmarshalDeleteInstanceACLRulesResponse$1);
  pageOfListUsers = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/users`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListUsersResponse$2);

  /**
   * List users of a Database Instance. List all users of a given Database
   * Instance. By default, the users returned in the list are ordered by
   * creation date in ascending order, though this can be modified via the
   * order_by field.
   *
   * @param request - The request {@link ListUsersRequest}
   * @returns A Promise of ListUsersResponse
   */
  listUsers = request => enrichForPagination('users', this.pageOfListUsers, request);

  /**
   * Create a user for a Database Instance. Create a new user for a Database
   * Instance. You must define the `name`, `password` and `is_admin`
   * parameters.
   *
   * @param request - The request {@link CreateUserRequest}
   * @returns A Promise of User
   */
  createUser = request => this.client.fetch({
    body: JSON.stringify(marshalCreateUserRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/users`
  }, unmarshalUser$2);

  /**
   * Update a user on a Database Instance. Update the parameters of a user on a
   * Database Instance. You can update the `password` and `is_admin` parameters,
   * but you cannot change the name of the user.
   *
   * @param request - The request {@link UpdateUserRequest}
   * @returns A Promise of User
   */
  updateUser = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateUserRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'PATCH',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/users/${validatePathParam('name', request.name)}`
  }, unmarshalUser$2);

  /**
   * Delete a user on a Database Instance. Delete a given user on a Database
   * Instance. You must specify, in the endpoint, the `region`, `instance_id`
   * and `name` parameters of the user you want to delete.
   *
   * @param request - The request {@link DeleteUserRequest}
   */
  deleteUser = request => this.client.fetch({
    method: 'DELETE',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/users/${validatePathParam('name', request.name)}`
  });
  pageOfListDatabases = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/databases`,
    urlParams: urlParams(['managed', request.managed], ['name', request.name], ['order_by', request.orderBy], ['owner', request.owner], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDatabasesResponse$2);

  /**
   * List databases in a Database Instance. List all databases of a given
   * Database Instance. By default, the databases returned in the list are
   * ordered by creation date in ascending order, though this can be modified
   * via the order_by field. You can define additional parameters for your
   * query, such as `name`, `managed` and `owner`.
   *
   * @param request - The request {@link ListDatabasesRequest}
   * @returns A Promise of ListDatabasesResponse
   */
  listDatabases = request => enrichForPagination('databases', this.pageOfListDatabases, request);

  /**
   * Create a database in a Database Instance. Create a new database. You must
   * define the `name` parameter in the request.
   *
   * @param request - The request {@link CreateDatabaseRequest}
   * @returns A Promise of Database
   */
  createDatabase = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDatabaseRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/databases`
  }, unmarshalDatabase$2);

  /**
   * Delete a database in a Database Instance. Delete a given database on a
   * Database Instance. You must specify, in the endpoint, the `region`,
   * `instance_id` and `name` parameters of the database you want to delete.
   *
   * @param request - The request {@link DeleteDatabaseRequest}
   */
  deleteDatabase = request => this.client.fetch({
    method: 'DELETE',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/databases/${validatePathParam('name', request.name)}`
  });
  pageOfListPrivileges = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/privileges`,
    urlParams: urlParams(['database_name', request.databaseName], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['user_name', request.userName])
  }, unmarshalListPrivilegesResponse$1);

  /**
   * List user privileges for a database. List privileges of a user on a
   * database. By default, the details returned in the list are ordered by
   * creation date in ascending order, though this can be modified via the
   * order_by field. You can define additional parameters for your query, such
   * as `database_name` and `user_name`.
   *
   * @param request - The request {@link ListPrivilegesRequest}
   * @returns A Promise of ListPrivilegesResponse
   */
  listPrivileges = request => enrichForPagination('privileges', this.pageOfListPrivileges, request);

  /**
   * Set user privileges for a database. Set the privileges of a user on a
   * database. You must define `database_name`, `user_name` and `permission` in
   * the request body.
   *
   * @param request - The request {@link SetPrivilegeRequest}
   * @returns A Promise of Privilege
   */
  setPrivilege = request => this.client.fetch({
    body: JSON.stringify(marshalSetPrivilegeRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'PUT',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/privileges`
  }, unmarshalPrivilege$1);
  pageOfListSnapshots = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots`,
    urlParams: urlParams(['instance_id', request.instanceId], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListSnapshotsResponse$2);

  /**
   * List snapshots. List snapshots. You can include the `instance_id` or
   * `project_id` in your query to get the list of snapshots for specific
   * Database Instances and/or Projects. By default, the details returned in the
   * list are ordered by creation date in ascending order, though this can be
   * modified via the `order_by` field.
   *
   * @param request - The request {@link ListSnapshotsRequest}
   * @returns A Promise of ListSnapshotsResponse
   */
  listSnapshots = (request = {}) => enrichForPagination('snapshots', this.pageOfListSnapshots, request);

  /**
   * Get a Database Instance snapshot. Retrieve information about a given
   * snapshot, specified by its `snapshot_id` and `region`. Full details about
   * the snapshot, like size and expiration date, are returned in the response.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  getSnapshot = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot$2);

  /**
   * Waits for {@link Snapshot} to be in a final state.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @param options - The waiting options
   * @returns A Promise of Snapshot
   */
  waitForSnapshot = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SNAPSHOT_TRANSIENT_STATUSES$2.includes(res.status))), this.getSnapshot, request, options);

  /**
   * Create a Database Instance snapshot. Create a new snapshot of a Database
   * Instance. You must define the `name` parameter in the request.
   *
   * @param request - The request {@link CreateSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  createSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSnapshotRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/snapshots`
  }, unmarshalSnapshot$2);

  /**
   * Update a Database Instance snapshot. Update the parameters of a snapshot of
   * a Database Instance. You can update the `name` and `expires_at`
   * parameters.
   *
   * @param request - The request {@link UpdateSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  updateSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSnapshotRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'PATCH',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot$2);

  /**
   * Delete a Database Instance snapshot. Delete a given snapshot of a Database
   * Instance. You must specify, in the endpoint, the `region` and `snapshot_id`
   * parameters of the snapshot you want to delete.
   *
   * @param request - The request {@link DeleteSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  deleteSnapshot = request => this.client.fetch({
    method: 'DELETE',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot$2);

  /**
   * Create a new Database Instance from a snapshot. Restore a snapshot. When
   * you restore a snapshot, a new Instance is created and billed to your
   * account. Note that is possible to select a larger node type for your new
   * Database Instance. However, the Block volume size will be the same as the
   * size of the restored snapshot. All Instance settings will be restored if
   * you chose a node type with the same or more memory size than the initial
   * Instance. Settings will be reset to the default if your node type has less
   * memory.
   *
   * @param request - The request {@link CreateInstanceFromSnapshotRequest}
   * @returns A Promise of Instance
   */
  createInstanceFromSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalCreateInstanceFromSnapshotRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}/create-instance`
  }, unmarshalInstance$2);

  /**
   * Create a new Database Instance endpoint. Create a new endpoint for a
   * Database Instance. You can add `load_balancer` and `private_network`
   * specifications to the body of the request.
   *
   * @param request - The request {@link CreateEndpointRequest}
   * @returns A Promise of Endpoint
   */
  createEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalCreateEndpointRequest$2(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/endpoints`
  }, unmarshalEndpoint$3);

  /**
   * Delete a Database Instance endpoint. Delete the endpoint of a Database
   * Instance. You must specify the `region` and `endpoint_id` parameters of the
   * endpoint you want to delete. Note that might need to update any environment
   * configurations that point to the deleted endpoint.
   *
   * @param request - The request {@link DeleteEndpointRequest}
   */
  deleteEndpoint = request => this.client.fetch({
    method: 'DELETE',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  });

  /**
   * Get a Database Instance endpoint. Retrieve information about a Database
   * Instance endpoint. Full details about the endpoint, like `ip`, `port`,
   * `private_network` and `load_balancer` specifications are returned in the
   * response.
   *
   * @param request - The request {@link GetEndpointRequest}
   * @returns A Promise of Endpoint
   */
  getEndpoint = request => this.client.fetch({
    method: 'GET',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  }, unmarshalEndpoint$3);

  /**
   * Migrate Database Instance endpoint. Migrate an existing Database Instance
   * endpoint to another Database Instance.
   *
   * @param request - The request {@link MigrateEndpointRequest}
   * @returns A Promise of Endpoint
   */
  migrateEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalMigrateEndpointRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints/${validatePathParam('endpointId', request.endpointId)}/migrate`
  }, unmarshalEndpoint$3);

  /**
   * Apply an instance maintenance. Apply a pending instance maintenance on your
   * instance. This action can generate some service interruption.
   *
   * @param request - The request {@link ApplyInstanceMaintenanceRequest}
   * @returns A Promise of Maintenance
   */
  applyInstanceMaintenance = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$p,
    method: 'POST',
    path: `/document-db/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/apply-maintenance`
  }, unmarshalMaintenance$1);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const UpdateInstanceRequest$1 = {
  backupScheduleStartHour: {
    lessThanOrEqual: 23
  }
};

var validationRules_gen$c = /*#__PURE__*/Object.freeze({
  __proto__: null,
  UpdateInstanceRequest: UpdateInstanceRequest$1
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$o = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$p,
  INSTANCE_LOG_TRANSIENT_STATUSES: INSTANCE_LOG_TRANSIENT_STATUSES$1,
  INSTANCE_TRANSIENT_STATUSES: INSTANCE_TRANSIENT_STATUSES$2,
  MAINTENANCE_TRANSIENT_STATUSES: MAINTENANCE_TRANSIENT_STATUSES$1,
  READ_REPLICA_TRANSIENT_STATUSES: READ_REPLICA_TRANSIENT_STATUSES$1,
  SNAPSHOT_TRANSIENT_STATUSES: SNAPSHOT_TRANSIENT_STATUSES$2,
  ValidationRules: validationRules_gen$c
});

var index$r = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1beta1: index_gen$o
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link DNSZoneStatus}. */
const DNS_ZONE_TRANSIENT_STATUSES = ['pending'];

/** Lists transient statutes of the enum {@link DomainFeatureStatus}. */
const DOMAIN_FEATURE_TRANSIENT_STATUSES = ['enabling', 'disabling'];

/**
 * Lists transient statutes of the enum
 * {@link DomainRegistrationStatusTransferStatus}.
 */
const DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES = ['pending', 'processing'];

/** Lists transient statutes of the enum {@link DomainStatus}. */
const DOMAIN_TRANSIENT_STATUSES$2 = ['creating', 'renewing', 'xfering', 'expiring', 'updating', 'checking', 'deleting'];

/** Lists transient statutes of the enum {@link HostStatus}. */
const HOST_TRANSIENT_STATUSES = ['updating', 'deleting'];

/** Lists transient statutes of the enum {@link SSLCertificateStatus}. */
const SSL_CERTIFICATE_TRANSIENT_STATUSES = ['pending'];

/** Lists transient statutes of the enum {@link TaskStatus}. */
const TASK_TRANSIENT_STATUSES$1 = ['pending'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalContactExtensionFRAssociationInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactExtensionFRAssociationInfo' failed as data isn't a dictionary.`);
  }
  return {
    publicationJo: unmarshalDate(data.publication_jo),
    publicationJoPage: data.publication_jo_page
  };
};
const unmarshalContactExtensionFRCodeAuthAfnicInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactExtensionFRCodeAuthAfnicInfo' failed as data isn't a dictionary.`);
  }
  return {
    codeAuthAfnic: data.code_auth_afnic
  };
};
const unmarshalContactExtensionFRDunsInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactExtensionFRDunsInfo' failed as data isn't a dictionary.`);
  }
  return {
    dunsId: data.duns_id,
    localId: data.local_id
  };
};
const unmarshalContactExtensionFRIndividualInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactExtensionFRIndividualInfo' failed as data isn't a dictionary.`);
  }
  return {
    whoisOptIn: data.whois_opt_in
  };
};
const unmarshalContactExtensionFRTrademarkInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactExtensionFRTrademarkInfo' failed as data isn't a dictionary.`);
  }
  return {
    trademarkInpi: data.trademark_inpi
  };
};
const unmarshalContactExtensionEU = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactExtensionEU' failed as data isn't a dictionary.`);
  }
  return {
    europeanCitizenship: data.european_citizenship
  };
};
const unmarshalContactExtensionFR = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactExtensionFR' failed as data isn't a dictionary.`);
  }
  return {
    associationInfo: data.association_info ? unmarshalContactExtensionFRAssociationInfo(data.association_info) : undefined,
    codeAuthAfnicInfo: data.code_auth_afnic_info ? unmarshalContactExtensionFRCodeAuthAfnicInfo(data.code_auth_afnic_info) : undefined,
    dunsInfo: data.duns_info ? unmarshalContactExtensionFRDunsInfo(data.duns_info) : undefined,
    individualInfo: data.individual_info ? unmarshalContactExtensionFRIndividualInfo(data.individual_info) : undefined,
    mode: data.mode,
    trademarkInfo: data.trademark_info ? unmarshalContactExtensionFRTrademarkInfo(data.trademark_info) : undefined
  };
};
const unmarshalContactExtensionNL = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactExtensionNL' failed as data isn't a dictionary.`);
  }
  return {
    legalForm: data.legal_form,
    legalFormRegistrationNumber: data.legal_form_registration_number
  };
};
const unmarshalContactQuestion = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactQuestion' failed as data isn't a dictionary.`);
  }
  return {
    answer: data.answer,
    question: data.question
  };
};
const unmarshalContact = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Contact' failed as data isn't a dictionary.`);
  }
  return {
    addressLine1: data.address_line_1,
    addressLine2: data.address_line_2,
    city: data.city,
    companyIdentificationCode: data.company_identification_code,
    companyName: data.company_name,
    country: data.country,
    email: data.email,
    emailAlt: data.email_alt,
    emailStatus: data.email_status,
    extensionEu: data.extension_eu ? unmarshalContactExtensionEU(data.extension_eu) : undefined,
    extensionFr: data.extension_fr ? unmarshalContactExtensionFR(data.extension_fr) : undefined,
    extensionNl: data.extension_nl ? unmarshalContactExtensionNL(data.extension_nl) : undefined,
    faxNumber: data.fax_number,
    firstname: data.firstname,
    id: data.id,
    lang: data.lang,
    lastname: data.lastname,
    legalForm: data.legal_form,
    phoneNumber: data.phone_number,
    questions: data.questions ? unmarshalArrayOfObject(data.questions, unmarshalContactQuestion) : undefined,
    resale: data.resale,
    state: data.state,
    status: data.status,
    vatIdentificationCode: data.vat_identification_code,
    whoisOptIn: data.whois_opt_in,
    zip: data.zip
  };
};
const unmarshalDNSZone = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DNSZone' failed as data isn't a dictionary.`);
  }
  return {
    domain: data.domain,
    linkedProducts: data.linked_products,
    message: data.message,
    ns: data.ns,
    nsDefault: data.ns_default,
    nsMaster: data.ns_master,
    projectId: data.project_id,
    status: data.status,
    subdomain: data.subdomain,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalHost = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Host' failed as data isn't a dictionary.`);
  }
  return {
    domain: data.domain,
    ips: data.ips,
    name: data.name,
    status: data.status
  };
};
const unmarshalSSLCertificate = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SSLCertificate' failed as data isn't a dictionary.`);
  }
  return {
    alternativeDnsZones: data.alternative_dns_zones,
    certificateChain: data.certificate_chain,
    createdAt: unmarshalDate(data.created_at),
    dnsZone: data.dns_zone,
    expiredAt: unmarshalDate(data.expired_at),
    privateKey: data.private_key,
    status: data.status
  };
};
const unmarshalCheckContactsCompatibilityResponseContactCheckResult = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CheckContactsCompatibilityResponseContactCheckResult' failed as data isn't a dictionary.`);
  }
  return {
    compatible: data.compatible,
    errorMessage: data.error_message
  };
};
const unmarshalCheckContactsCompatibilityResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CheckContactsCompatibilityResponse' failed as data isn't a dictionary.`);
  }
  return {
    administrativeCheckResult: data.administrative_check_result ? unmarshalCheckContactsCompatibilityResponseContactCheckResult(data.administrative_check_result) : undefined,
    compatible: data.compatible,
    ownerCheckResult: data.owner_check_result ? unmarshalCheckContactsCompatibilityResponseContactCheckResult(data.owner_check_result) : undefined,
    technicalCheckResult: data.technical_check_result ? unmarshalCheckContactsCompatibilityResponseContactCheckResult(data.technical_check_result) : undefined
  };
};
const unmarshalClearDNSZoneRecordsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClearDNSZoneRecordsResponse' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalDeleteDNSZoneResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeleteDNSZoneResponse' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalDeleteExternalDomainResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeleteExternalDomainResponse' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalDeleteSSLCertificateResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeleteSSLCertificateResponse' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalDSRecordPublicKey = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DSRecordPublicKey' failed as data isn't a dictionary.`);
  }
  return {
    key: data.key
  };
};
const unmarshalDSRecordDigest = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DSRecordDigest' failed as data isn't a dictionary.`);
  }
  return {
    digest: data.digest,
    publicKey: data.public_key ? unmarshalDSRecordPublicKey(data.public_key) : undefined,
    type: data.type
  };
};
const unmarshalDSRecord = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DSRecord' failed as data isn't a dictionary.`);
  }
  return {
    algorithm: data.algorithm,
    digest: data.digest ? unmarshalDSRecordDigest(data.digest) : undefined,
    keyId: data.key_id,
    publicKey: data.public_key ? unmarshalDSRecordPublicKey(data.public_key) : undefined
  };
};
const unmarshalTldOffer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TldOffer' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    operationPath: data.operation_path,
    price: data.price ? unmarshalMoney(data.price) : undefined
  };
};
const unmarshalDomainDNSSEC = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainDNSSEC' failed as data isn't a dictionary.`);
  }
  return {
    dsRecords: unmarshalArrayOfObject(data.ds_records, unmarshalDSRecord),
    status: data.status
  };
};
const unmarshalDomainRegistrationStatusExternalDomain = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRegistrationStatusExternalDomain' failed as data isn't a dictionary.`);
  }
  return {
    validationToken: data.validation_token
  };
};
const unmarshalDomainRegistrationStatusTransfer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRegistrationStatusTransfer' failed as data isn't a dictionary.`);
  }
  return {
    status: data.status,
    voteCurrentOwner: data.vote_current_owner,
    voteNewOwner: data.vote_new_owner
  };
};
const unmarshalTld = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Tld' failed as data isn't a dictionary.`);
  }
  return {
    dnssecSupport: data.dnssec_support,
    durationInYearsMax: data.duration_in_years_max,
    durationInYearsMin: data.duration_in_years_min,
    idnSupport: data.idn_support,
    name: data.name,
    offers: unmarshalMapOfObject(data.offers, unmarshalTldOffer),
    specifications: data.specifications
  };
};
const unmarshalDomain$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Domain' failed as data isn't a dictionary.`);
  }
  return {
    administrativeContact: data.administrative_contact ? unmarshalContact(data.administrative_contact) : undefined,
    autoRenewStatus: data.auto_renew_status,
    dnsZones: unmarshalArrayOfObject(data.dns_zones, unmarshalDNSZone),
    dnssec: data.dnssec ? unmarshalDomainDNSSEC(data.dnssec) : undefined,
    domain: data.domain,
    eppCode: data.epp_code,
    expiredAt: unmarshalDate(data.expired_at),
    externalDomainRegistrationStatus: data.external_domain_registration_status ? unmarshalDomainRegistrationStatusExternalDomain(data.external_domain_registration_status) : undefined,
    isExternal: data.is_external,
    linkedProducts: data.linked_products,
    organizationId: data.organization_id,
    ownerContact: data.owner_contact ? unmarshalContact(data.owner_contact) : undefined,
    pendingTrade: data.pending_trade,
    projectId: data.project_id,
    registrar: data.registrar,
    status: data.status,
    technicalContact: data.technical_contact ? unmarshalContact(data.technical_contact) : undefined,
    tld: data.tld ? unmarshalTld(data.tld) : undefined,
    transferRegistrationStatus: data.transfer_registration_status ? unmarshalDomainRegistrationStatusTransfer(data.transfer_registration_status) : undefined,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalGetDNSZoneTsigKeyResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetDNSZoneTsigKeyResponse' failed as data isn't a dictionary.`);
  }
  return {
    algorithm: data.algorithm,
    key: data.key,
    name: data.name
  };
};
const unmarshalDomainRecordGeoIPConfigMatch = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecordGeoIPConfigMatch' failed as data isn't a dictionary.`);
  }
  return {
    continents: data.continents,
    countries: data.countries,
    data: data.data
  };
};
const unmarshalDomainRecordViewConfigView = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecordViewConfigView' failed as data isn't a dictionary.`);
  }
  return {
    data: data.data,
    subnet: data.subnet
  };
};
const unmarshalDomainRecordWeightedConfigWeightedIP = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecordWeightedConfigWeightedIP' failed as data isn't a dictionary.`);
  }
  return {
    ip: data.ip,
    weight: data.weight
  };
};
const unmarshalDomainRecordGeoIPConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecordGeoIPConfig' failed as data isn't a dictionary.`);
  }
  return {
    default: data.default,
    matches: unmarshalArrayOfObject(data.matches, unmarshalDomainRecordGeoIPConfigMatch)
  };
};
const unmarshalDomainRecordHTTPServiceConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecordHTTPServiceConfig' failed as data isn't a dictionary.`);
  }
  return {
    ips: data.ips,
    mustContain: data.must_contain,
    strategy: data.strategy,
    url: data.url,
    userAgent: data.user_agent
  };
};
const unmarshalDomainRecordViewConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecordViewConfig' failed as data isn't a dictionary.`);
  }
  return {
    views: unmarshalArrayOfObject(data.views, unmarshalDomainRecordViewConfigView)
  };
};
const unmarshalDomainRecordWeightedConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecordWeightedConfig' failed as data isn't a dictionary.`);
  }
  return {
    weightedIps: unmarshalArrayOfObject(data.weighted_ips, unmarshalDomainRecordWeightedConfigWeightedIP)
  };
};
const unmarshalDomainRecord = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecord' failed as data isn't a dictionary.`);
  }
  return {
    comment: data.comment,
    data: data.data,
    geoIpConfig: data.geo_ip_config ? unmarshalDomainRecordGeoIPConfig(data.geo_ip_config) : undefined,
    httpServiceConfig: data.http_service_config ? unmarshalDomainRecordHTTPServiceConfig(data.http_service_config) : undefined,
    id: data.id,
    name: data.name,
    priority: data.priority,
    ttl: data.ttl,
    type: data.type,
    viewConfig: data.view_config ? unmarshalDomainRecordViewConfig(data.view_config) : undefined,
    weightedConfig: data.weighted_config ? unmarshalDomainRecordWeightedConfig(data.weighted_config) : undefined
  };
};
const unmarshalRecordIdentifier = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RecordIdentifier' failed as data isn't a dictionary.`);
  }
  return {
    data: data.data,
    name: data.name,
    ttl: data.ttl,
    type: data.type
  };
};
const unmarshalRecordChangeAdd = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RecordChangeAdd' failed as data isn't a dictionary.`);
  }
  return {
    records: unmarshalArrayOfObject(data.records, unmarshalDomainRecord)
  };
};
const unmarshalRecordChangeClear = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RecordChangeClear' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalRecordChangeDelete = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RecordChangeDelete' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    idFields: data.id_fields ? unmarshalRecordIdentifier(data.id_fields) : undefined
  };
};
const unmarshalRecordChangeSet = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RecordChangeSet' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    idFields: data.id_fields ? unmarshalRecordIdentifier(data.id_fields) : undefined,
    records: unmarshalArrayOfObject(data.records, unmarshalDomainRecord)
  };
};
const unmarshalRecordChange = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RecordChange' failed as data isn't a dictionary.`);
  }
  return {
    add: data.add ? unmarshalRecordChangeAdd(data.add) : undefined,
    clear: data.clear ? unmarshalRecordChangeClear(data.clear) : undefined,
    delete: data.delete ? unmarshalRecordChangeDelete(data.delete) : undefined,
    set: data.set ? unmarshalRecordChangeSet(data.set) : undefined
  };
};
const unmarshalGetDNSZoneVersionDiffResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetDNSZoneVersionDiffResponse' failed as data isn't a dictionary.`);
  }
  return {
    changes: unmarshalArrayOfObject(data.changes, unmarshalRecordChange)
  };
};
const unmarshalGetDomainAuthCodeResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetDomainAuthCodeResponse' failed as data isn't a dictionary.`);
  }
  return {
    authCode: data.auth_code
  };
};
const unmarshalImportProviderDNSZoneResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ImportProviderDNSZoneResponse' failed as data isn't a dictionary.`);
  }
  return {
    records: unmarshalArrayOfObject(data.records, unmarshalDomainRecord)
  };
};
const unmarshalImportRawDNSZoneResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ImportRawDNSZoneResponse' failed as data isn't a dictionary.`);
  }
  return {
    records: unmarshalArrayOfObject(data.records, unmarshalDomainRecord)
  };
};
const unmarshalContactRolesRoles = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactRolesRoles' failed as data isn't a dictionary.`);
  }
  return {
    isAdministrative: data.is_administrative,
    isOwner: data.is_owner,
    isTechnical: data.is_technical
  };
};
const unmarshalContactRoles = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ContactRoles' failed as data isn't a dictionary.`);
  }
  return {
    contact: data.contact ? unmarshalContact(data.contact) : undefined,
    roles: unmarshalMapOfObject(data.roles, unmarshalContactRolesRoles)
  };
};
const unmarshalListContactsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListContactsResponse' failed as data isn't a dictionary.`);
  }
  return {
    contacts: unmarshalArrayOfObject(data.contacts, unmarshalContactRoles),
    totalCount: data.total_count
  };
};
const unmarshalNameserver$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Nameserver' failed as data isn't a dictionary.`);
  }
  return {
    ip: data.ip,
    name: data.name
  };
};
const unmarshalListDNSZoneNameserversResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDNSZoneNameserversResponse' failed as data isn't a dictionary.`);
  }
  return {
    ns: unmarshalArrayOfObject(data.ns, unmarshalNameserver$1)
  };
};
const unmarshalListDNSZoneRecordsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDNSZoneRecordsResponse' failed as data isn't a dictionary.`);
  }
  return {
    records: unmarshalArrayOfObject(data.records, unmarshalDomainRecord),
    totalCount: data.total_count
  };
};
const unmarshalListDNSZoneVersionRecordsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDNSZoneVersionRecordsResponse' failed as data isn't a dictionary.`);
  }
  return {
    records: unmarshalArrayOfObject(data.records, unmarshalDomainRecord),
    totalCount: data.total_count
  };
};
const unmarshalDNSZoneVersion = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DNSZoneVersion' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id
  };
};
const unmarshalListDNSZoneVersionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDNSZoneVersionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    versions: unmarshalArrayOfObject(data.versions, unmarshalDNSZoneVersion)
  };
};
const unmarshalListDNSZonesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDNSZonesResponse' failed as data isn't a dictionary.`);
  }
  return {
    dnsZones: unmarshalArrayOfObject(data.dns_zones, unmarshalDNSZone),
    totalCount: data.total_count
  };
};
const unmarshalListDomainHostsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDomainHostsResponse' failed as data isn't a dictionary.`);
  }
  return {
    hosts: unmarshalArrayOfObject(data.hosts, unmarshalHost),
    totalCount: data.total_count
  };
};
const unmarshalDomainSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainSummary' failed as data isn't a dictionary.`);
  }
  return {
    autoRenewStatus: data.auto_renew_status,
    createdAt: unmarshalDate(data.created_at),
    dnssecStatus: data.dnssec_status,
    domain: data.domain,
    eppCode: data.epp_code,
    expiredAt: unmarshalDate(data.expired_at),
    externalDomainRegistrationStatus: data.external_domain_registration_status ? unmarshalDomainRegistrationStatusExternalDomain(data.external_domain_registration_status) : undefined,
    isExternal: data.is_external,
    organizationId: data.organization_id,
    pendingTrade: data.pending_trade,
    projectId: data.project_id,
    registrar: data.registrar,
    status: data.status,
    transferRegistrationStatus: data.transfer_registration_status ? unmarshalDomainRegistrationStatusTransfer(data.transfer_registration_status) : undefined,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListDomainsResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary.`);
  }
  return {
    domains: unmarshalArrayOfObject(data.domains, unmarshalDomainSummary),
    totalCount: data.total_count
  };
};
const unmarshalRenewableDomain = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RenewableDomain' failed as data isn't a dictionary.`);
  }
  return {
    domain: data.domain,
    estimatedDeleteAt: unmarshalDate(data.estimated_delete_at),
    expiredAt: unmarshalDate(data.expired_at),
    limitRedemptionAt: unmarshalDate(data.limit_redemption_at),
    limitRenewAt: unmarshalDate(data.limit_renew_at),
    organizationId: data.organization_id,
    projectId: data.project_id,
    renewableDurationInYears: data.renewable_duration_in_years,
    status: data.status,
    tld: data.tld ? unmarshalTld(data.tld) : undefined
  };
};
const unmarshalListRenewableDomainsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRenewableDomainsResponse' failed as data isn't a dictionary.`);
  }
  return {
    domains: unmarshalArrayOfObject(data.domains, unmarshalRenewableDomain),
    totalCount: data.total_count
  };
};
const unmarshalListSSLCertificatesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSSLCertificatesResponse' failed as data isn't a dictionary.`);
  }
  return {
    certificates: unmarshalArrayOfObject(data.certificates, unmarshalSSLCertificate),
    totalCount: data.total_count
  };
};
const unmarshalTask$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Task' failed as data isn't a dictionary.`);
  }
  return {
    contactIdentifier: data.contact_identifier,
    domain: data.domain,
    id: data.id,
    message: data.message,
    organizationId: data.organization_id,
    projectId: data.project_id,
    startedAt: unmarshalDate(data.started_at),
    status: data.status,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListTasksResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTasksResponse' failed as data isn't a dictionary.`);
  }
  return {
    tasks: unmarshalArrayOfObject(data.tasks, unmarshalTask$1),
    totalCount: data.total_count
  };
};
const unmarshalListTldsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTldsResponse' failed as data isn't a dictionary.`);
  }
  return {
    tlds: unmarshalArrayOfObject(data.tlds, unmarshalTld),
    totalCount: data.total_count
  };
};
const unmarshalOrderResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OrderResponse' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    domains: data.domains,
    organizationId: data.organization_id,
    projectId: data.project_id,
    taskId: data.task_id
  };
};
const unmarshalRefreshDNSZoneResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RefreshDNSZoneResponse' failed as data isn't a dictionary.`);
  }
  return {
    dnsZones: unmarshalArrayOfObject(data.dns_zones, unmarshalDNSZone)
  };
};
const unmarshalRegisterExternalDomainResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RegisterExternalDomainResponse' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    domain: data.domain,
    organizationId: data.organization_id,
    projectId: data.project_id,
    validationToken: data.validation_token
  };
};
const unmarshalRestoreDNSZoneVersionResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RestoreDNSZoneVersionResponse' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalAvailableDomain = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AvailableDomain' failed as data isn't a dictionary.`);
  }
  return {
    available: data.available,
    domain: data.domain,
    tld: data.tld ? unmarshalTld(data.tld) : undefined
  };
};
const unmarshalSearchAvailableDomainsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SearchAvailableDomainsResponse' failed as data isn't a dictionary.`);
  }
  return {
    availableDomains: unmarshalArrayOfObject(data.available_domains, unmarshalAvailableDomain)
  };
};
const unmarshalUpdateDNSZoneNameserversResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateDNSZoneNameserversResponse' failed as data isn't a dictionary.`);
  }
  return {
    ns: unmarshalArrayOfObject(data.ns, unmarshalNameserver$1)
  };
};
const unmarshalUpdateDNSZoneRecordsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateDNSZoneRecordsResponse' failed as data isn't a dictionary.`);
  }
  return {
    records: unmarshalArrayOfObject(data.records, unmarshalDomainRecord)
  };
};
const marshalCloneDNSZoneRequest = (request, defaults) => ({
  dest_dns_zone: request.destDnsZone,
  overwrite: request.overwrite,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalCreateDNSZoneRequest = (request, defaults) => ({
  domain: request.domain,
  project_id: request.projectId ?? defaults.defaultProjectId,
  subdomain: request.subdomain
});
const marshalCreateSSLCertificateRequest = (request, defaults) => ({
  alternative_dns_zones: request.alternativeDnsZones,
  dns_zone: request.dnsZone
});
const marshalImportProviderDNSZoneRequestOnlineV1 = (request, defaults) => ({
  token: request.token
});
const marshalImportProviderDNSZoneRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'online_v1',
    value: request.onlineV1 !== undefined ? marshalImportProviderDNSZoneRequestOnlineV1(request.onlineV1) : undefined
  }])
});
const marshalImportRawDNSZoneRequestTsigKey = (request, defaults) => ({
  algorithm: request.algorithm,
  key: request.key,
  name: request.name
});
const marshalImportRawDNSZoneRequestAXFRSource = (request, defaults) => ({
  name_server: request.nameServer,
  tsig_key: request.tsigKey !== undefined ? marshalImportRawDNSZoneRequestTsigKey(request.tsigKey) : undefined
});
const marshalImportRawDNSZoneRequestBindSource = (request, defaults) => ({
  content: request.content
});
const marshalImportRawDNSZoneRequest = (request, defaults) => ({
  content: request.content,
  format: request.format,
  project_id: request.projectId ?? defaults.defaultProjectId,
  ...resolveOneOf([{
    param: 'bind_source',
    value: request.bindSource !== undefined ? marshalImportRawDNSZoneRequestBindSource(request.bindSource) : undefined
  }, {
    param: 'axfr_source',
    value: request.axfrSource !== undefined ? marshalImportRawDNSZoneRequestAXFRSource(request.axfrSource) : undefined
  }])
});
const marshalRefreshDNSZoneRequest = (request, defaults) => ({
  recreate_dns_zone: request.recreateDnsZone,
  recreate_sub_dns_zone: request.recreateSubDnsZone
});
const marshalContactExtensionFRAssociationInfo = (request, defaults) => ({
  publication_jo: request.publicationJo,
  publication_jo_page: request.publicationJoPage
});
const marshalContactExtensionFRCodeAuthAfnicInfo = (request, defaults) => ({
  code_auth_afnic: request.codeAuthAfnic
});
const marshalContactExtensionFRDunsInfo = (request, defaults) => ({
  duns_id: request.dunsId,
  local_id: request.localId
});
const marshalContactExtensionFRIndividualInfo = (request, defaults) => ({
  whois_opt_in: request.whoisOptIn
});
const marshalContactExtensionFRTrademarkInfo = (request, defaults) => ({
  trademark_inpi: request.trademarkInpi
});
const marshalContactExtensionEU = (request, defaults) => ({
  european_citizenship: request.europeanCitizenship
});
const marshalContactExtensionFR = (request, defaults) => ({
  mode: request.mode,
  ...resolveOneOf([{
    param: 'individual_info',
    value: request.individualInfo !== undefined ? marshalContactExtensionFRIndividualInfo(request.individualInfo) : undefined
  }, {
    param: 'duns_info',
    value: request.dunsInfo !== undefined ? marshalContactExtensionFRDunsInfo(request.dunsInfo) : undefined
  }, {
    param: 'association_info',
    value: request.associationInfo !== undefined ? marshalContactExtensionFRAssociationInfo(request.associationInfo) : undefined
  }, {
    param: 'trademark_info',
    value: request.trademarkInfo !== undefined ? marshalContactExtensionFRTrademarkInfo(request.trademarkInfo) : undefined
  }, {
    param: 'code_auth_afnic_info',
    value: request.codeAuthAfnicInfo !== undefined ? marshalContactExtensionFRCodeAuthAfnicInfo(request.codeAuthAfnicInfo) : undefined
  }])
});
const marshalContactExtensionNL = (request, defaults) => ({
  legal_form: request.legalForm,
  legal_form_registration_number: request.legalFormRegistrationNumber
});
const marshalContactQuestion = (request, defaults) => ({
  answer: request.answer,
  question: request.question
});
const marshalNewContact = (request, defaults) => ({
  address_line_1: request.addressLine1,
  address_line_2: request.addressLine2,
  city: request.city,
  company_identification_code: request.companyIdentificationCode,
  company_name: request.companyName,
  country: request.country,
  email: request.email,
  email_alt: request.emailAlt,
  extension_eu: request.extensionEu !== undefined ? marshalContactExtensionEU(request.extensionEu) : undefined,
  extension_fr: request.extensionFr !== undefined ? marshalContactExtensionFR(request.extensionFr) : undefined,
  extension_nl: request.extensionNl !== undefined ? marshalContactExtensionNL(request.extensionNl) : undefined,
  fax_number: request.faxNumber,
  firstname: request.firstname,
  lang: request.lang,
  lastname: request.lastname,
  legal_form: request.legalForm,
  phone_number: request.phoneNumber,
  questions: request.questions !== undefined ? request.questions.map(elt => marshalContactQuestion(elt)) : undefined,
  resale: request.resale,
  state: request.state,
  vat_identification_code: request.vatIdentificationCode,
  whois_opt_in: request.whoisOptIn,
  zip: request.zip
});
const marshalRegistrarApiBuyDomainsRequest = (request, defaults) => ({
  domains: request.domains,
  duration_in_years: request.durationInYears,
  project_id: request.projectId ?? defaults.defaultProjectId,
  ...resolveOneOf([{
    param: 'owner_contact_id',
    value: request.ownerContactId
  }, {
    param: 'owner_contact',
    value: request.ownerContact !== undefined ? marshalNewContact(request.ownerContact) : undefined
  }]),
  ...resolveOneOf([{
    param: 'administrative_contact_id',
    value: request.administrativeContactId
  }, {
    param: 'administrative_contact',
    value: request.administrativeContact !== undefined ? marshalNewContact(request.administrativeContact) : undefined
  }]),
  ...resolveOneOf([{
    param: 'technical_contact_id',
    value: request.technicalContactId
  }, {
    param: 'technical_contact',
    value: request.technicalContact !== undefined ? marshalNewContact(request.technicalContact) : undefined
  }])
});
const marshalRegistrarApiCheckContactsCompatibilityRequest = (request, defaults) => ({
  domains: request.domains,
  tlds: request.tlds,
  ...resolveOneOf([{
    param: 'owner_contact_id',
    value: request.ownerContactId
  }, {
    param: 'owner_contact',
    value: request.ownerContact !== undefined ? marshalNewContact(request.ownerContact) : undefined
  }]),
  ...resolveOneOf([{
    param: 'administrative_contact_id',
    value: request.administrativeContactId
  }, {
    param: 'administrative_contact',
    value: request.administrativeContact !== undefined ? marshalNewContact(request.administrativeContact) : undefined
  }]),
  ...resolveOneOf([{
    param: 'technical_contact_id',
    value: request.technicalContactId
  }, {
    param: 'technical_contact',
    value: request.technicalContact !== undefined ? marshalNewContact(request.technicalContact) : undefined
  }])
});
const marshalRegistrarApiCreateDomainHostRequest = (request, defaults) => ({
  ips: request.ips,
  name: request.name
});
const marshalDSRecordPublicKey = (request, defaults) => ({
  key: request.key
});
const marshalDSRecordDigest = (request, defaults) => ({
  digest: request.digest,
  public_key: request.publicKey !== undefined ? marshalDSRecordPublicKey(request.publicKey) : undefined,
  type: request.type
});
const marshalDSRecord = (request, defaults) => ({
  algorithm: request.algorithm,
  key_id: request.keyId,
  ...resolveOneOf([{
    param: 'digest',
    value: request.digest !== undefined ? marshalDSRecordDigest(request.digest) : undefined
  }, {
    param: 'public_key',
    value: request.publicKey !== undefined ? marshalDSRecordPublicKey(request.publicKey) : undefined
  }])
});
const marshalRegistrarApiEnableDomainDNSSECRequest = (request, defaults) => ({
  ds_record: request.dsRecord !== undefined ? marshalDSRecord(request.dsRecord) : undefined
});
const marshalRegistrarApiRegisterExternalDomainRequest = (request, defaults) => ({
  domain: request.domain,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalRegistrarApiRenewDomainsRequest = (request, defaults) => ({
  domains: request.domains,
  duration_in_years: request.durationInYears,
  force_late_renewal: request.forceLateRenewal
});
const marshalRegistrarApiTradeDomainRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId,
  ...resolveOneOf([{
    param: 'new_owner_contact_id',
    value: request.newOwnerContactId
  }, {
    param: 'new_owner_contact',
    value: request.newOwnerContact !== undefined ? marshalNewContact(request.newOwnerContact) : undefined
  }])
});
const marshalTransferInDomainRequestTransferRequest = (request, defaults) => ({
  auth_code: request.authCode,
  domain: request.domain
});
const marshalRegistrarApiTransferInDomainRequest = (request, defaults) => ({
  domains: request.domains.map(elt => marshalTransferInDomainRequestTransferRequest(elt)),
  project_id: request.projectId ?? defaults.defaultProjectId,
  ...resolveOneOf([{
    param: 'owner_contact_id',
    value: request.ownerContactId
  }, {
    param: 'owner_contact',
    value: request.ownerContact !== undefined ? marshalNewContact(request.ownerContact) : undefined
  }]),
  ...resolveOneOf([{
    param: 'administrative_contact_id',
    value: request.administrativeContactId
  }, {
    param: 'administrative_contact',
    value: request.administrativeContact !== undefined ? marshalNewContact(request.administrativeContact) : undefined
  }]),
  ...resolveOneOf([{
    param: 'technical_contact_id',
    value: request.technicalContactId
  }, {
    param: 'technical_contact',
    value: request.technicalContact !== undefined ? marshalNewContact(request.technicalContact) : undefined
  }])
});
const marshalUpdateContactRequestQuestion = (request, defaults) => ({
  answer: request.answer,
  question: request.question
});
const marshalRegistrarApiUpdateContactRequest = (request, defaults) => ({
  address_line_1: request.addressLine1,
  address_line_2: request.addressLine2,
  city: request.city,
  company_identification_code: request.companyIdentificationCode,
  country: request.country,
  email: request.email,
  email_alt: request.emailAlt,
  extension_eu: request.extensionEu !== undefined ? marshalContactExtensionEU(request.extensionEu) : undefined,
  extension_fr: request.extensionFr !== undefined ? marshalContactExtensionFR(request.extensionFr) : undefined,
  extension_nl: request.extensionNl !== undefined ? marshalContactExtensionNL(request.extensionNl) : undefined,
  fax_number: request.faxNumber,
  lang: request.lang,
  phone_number: request.phoneNumber,
  questions: request.questions !== undefined ? request.questions.map(elt => marshalUpdateContactRequestQuestion(elt)) : undefined,
  resale: request.resale,
  state: request.state,
  vat_identification_code: request.vatIdentificationCode,
  whois_opt_in: request.whoisOptIn,
  zip: request.zip
});
const marshalRegistrarApiUpdateDomainHostRequest = (request, defaults) => ({
  ips: request.ips
});
const marshalRegistrarApiUpdateDomainRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'technical_contact_id',
    value: request.technicalContactId
  }, {
    param: 'technical_contact',
    value: request.technicalContact !== undefined ? marshalNewContact(request.technicalContact) : undefined
  }]),
  ...resolveOneOf([{
    param: 'owner_contact_id',
    value: request.ownerContactId
  }, {
    param: 'owner_contact',
    value: request.ownerContact !== undefined ? marshalNewContact(request.ownerContact) : undefined
  }]),
  ...resolveOneOf([{
    param: 'administrative_contact_id',
    value: request.administrativeContactId
  }, {
    param: 'administrative_contact',
    value: request.administrativeContact !== undefined ? marshalNewContact(request.administrativeContact) : undefined
  }])
});
const marshalNameserver = (request, defaults) => ({
  ip: request.ip,
  name: request.name
});
const marshalUpdateDNSZoneNameserversRequest = (request, defaults) => ({
  ns: request.ns.map(elt => marshalNameserver(elt))
});
const marshalDomainRecordGeoIPConfigMatch = (request, defaults) => ({
  continents: request.continents,
  countries: request.countries,
  data: request.data
});
const marshalDomainRecordViewConfigView = (request, defaults) => ({
  data: request.data,
  subnet: request.subnet
});
const marshalDomainRecordWeightedConfigWeightedIP = (request, defaults) => ({
  ip: request.ip,
  weight: request.weight
});
const marshalDomainRecordGeoIPConfig = (request, defaults) => ({
  default: request.default,
  matches: request.matches.map(elt => marshalDomainRecordGeoIPConfigMatch(elt))
});
const marshalDomainRecordHTTPServiceConfig = (request, defaults) => ({
  ips: request.ips,
  must_contain: request.mustContain,
  strategy: request.strategy,
  url: request.url,
  user_agent: request.userAgent
});
const marshalDomainRecordViewConfig = (request, defaults) => ({
  views: request.views.map(elt => marshalDomainRecordViewConfigView(elt))
});
const marshalDomainRecordWeightedConfig = (request, defaults) => ({
  weighted_ips: request.weightedIps.map(elt => marshalDomainRecordWeightedConfigWeightedIP(elt))
});
const marshalDomainRecord = (request, defaults) => ({
  comment: request.comment,
  data: request.data,
  id: request.id,
  name: request.name,
  priority: request.priority,
  ttl: request.ttl,
  type: request.type,
  ...resolveOneOf([{
    param: 'geo_ip_config',
    value: request.geoIpConfig !== undefined ? marshalDomainRecordGeoIPConfig(request.geoIpConfig) : undefined
  }, {
    param: 'http_service_config',
    value: request.httpServiceConfig !== undefined ? marshalDomainRecordHTTPServiceConfig(request.httpServiceConfig) : undefined
  }, {
    param: 'weighted_config',
    value: request.weightedConfig !== undefined ? marshalDomainRecordWeightedConfig(request.weightedConfig) : undefined
  }, {
    param: 'view_config',
    value: request.viewConfig !== undefined ? marshalDomainRecordViewConfig(request.viewConfig) : undefined
  }])
});
const marshalRecordIdentifier = (request, defaults) => ({
  data: request.data,
  name: request.name,
  ttl: request.ttl,
  type: request.type
});
const marshalRecordChangeAdd = (request, defaults) => ({
  records: request.records.map(elt => marshalDomainRecord(elt))
});
const marshalRecordChangeClear = (request, defaults) => ({});
const marshalRecordChangeDelete = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'id',
    value: request.id
  }, {
    param: 'id_fields',
    value: request.idFields !== undefined ? marshalRecordIdentifier(request.idFields) : undefined
  }])
});
const marshalRecordChangeSet = (request, defaults) => ({
  records: request.records.map(elt => marshalDomainRecord(elt)),
  ...resolveOneOf([{
    param: 'id',
    value: request.id
  }, {
    param: 'id_fields',
    value: request.idFields !== undefined ? marshalRecordIdentifier(request.idFields) : undefined
  }])
});
const marshalRecordChange = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'add',
    value: request.add !== undefined ? marshalRecordChangeAdd(request.add) : undefined
  }, {
    param: 'set',
    value: request.set !== undefined ? marshalRecordChangeSet(request.set) : undefined
  }, {
    param: 'delete',
    value: request.delete !== undefined ? marshalRecordChangeDelete(request.delete) : undefined
  }, {
    param: 'clear',
    value: request.clear !== undefined ? marshalRecordChangeClear(request.clear) : undefined
  }])
});
const marshalUpdateDNSZoneRecordsRequest = (request, defaults) => ({
  changes: request.changes.map(elt => marshalRecordChange(elt)),
  disallow_new_zone_creation: request.disallowNewZoneCreation,
  return_all_records: request.returnAllRecords,
  serial: request.serial
});
const marshalUpdateDNSZoneRequest = (request, defaults) => ({
  new_dns_zone: request.newDnsZone,
  project_id: request.projectId ?? defaults.defaultProjectId
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$o = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Domains and DNS API.
 *
 * Manage your domains, DNS zones and records with the Domains and DNS API.
 */
let API$o = class API extends API$z {
  pageOfListDNSZones = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/dns-zones`,
    urlParams: urlParams(['created_after', request.createdAfter], ['created_before', request.createdBefore], ['dns_zone', request.dnsZone], ['dns_zones', request.dnsZones], ['domain', request.domain], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['updated_after', request.updatedAfter], ['updated_before', request.updatedBefore])
  }, unmarshalListDNSZonesResponse);

  /**
   * List DNS zones. Retrieve the list of DNS zones you can manage and filter
   * DNS zones associated with specific domain names.
   *
   * @param request - The request {@link ListDNSZonesRequest}
   * @returns A Promise of ListDNSZonesResponse
   */
  listDNSZones = request => enrichForPagination('dnsZones', this.pageOfListDNSZones, request);

  /**
   * Create a DNS zone. Create a new DNS zone specified by the domain name, the
   * subdomain and the Project ID.
   *
   * @param request - The request {@link CreateDNSZoneRequest}
   * @returns A Promise of DNSZone
   */
  createDNSZone = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDNSZoneRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/dns-zones`
  }, unmarshalDNSZone);

  /**
   * Update a DNS zone. Update the name and/or the Organizations for a DNS zone.
   *
   * @param request - The request {@link UpdateDNSZoneRequest}
   * @returns A Promise of DNSZone
   */
  updateDNSZone = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDNSZoneRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'PATCH',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}`
  }, unmarshalDNSZone);

  /**
   * Clone a DNS zone. Clone an existing DNS zone with all its records into a
   * new DNS zone.
   *
   * @param request - The request {@link CloneDNSZoneRequest}
   * @returns A Promise of DNSZone
   */
  cloneDNSZone = request => this.client.fetch({
    body: JSON.stringify(marshalCloneDNSZoneRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/clone`
  }, unmarshalDNSZone);

  /**
   * Delete a DNS zone. Delete a DNS zone and all its records.
   *
   * @param request - The request {@link DeleteDNSZoneRequest}
   * @returns A Promise of DeleteDNSZoneResponse
   */
  deleteDNSZone = request => this.client.fetch({
    method: 'DELETE',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalDeleteDNSZoneResponse);
  pageOfListDNSZoneRecords = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/records`,
    urlParams: urlParams(['id', request.id], ['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['type', request.type])
  }, unmarshalListDNSZoneRecordsResponse);

  /**
   * List records within a DNS zone. Retrieve a list of DNS records within a DNS
   * zone that has default name servers. You can filter records by type and
   * name.
   *
   * @param request - The request {@link ListDNSZoneRecordsRequest}
   * @returns A Promise of ListDNSZoneRecordsResponse
   */
  listDNSZoneRecords = request => enrichForPagination('records', this.pageOfListDNSZoneRecords, request);

  /**
   * Update records within a DNS zone. Update records within a DNS zone that has
   * default name servers and perform several actions on your records.
   *
   * Actions include:
   *
   * - Add: allows you to add a new record or add a new IP to an existing A
   *   record, for example
   * - Set: allows you to edit a record or edit an IP from an existing A record,
   *   for example
   * - Delete: allows you to delete a record or delete an IP from an existing A
   *   record, for example
   * - Clear: allows you to delete all records from a DNS zone
   *
   * All edits will be versioned.
   *
   * @param request - The request {@link UpdateDNSZoneRecordsRequest}
   * @returns A Promise of UpdateDNSZoneRecordsResponse
   */
  updateDNSZoneRecords = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDNSZoneRecordsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'PATCH',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/records`
  }, unmarshalUpdateDNSZoneRecordsResponse);

  /**
   * List name servers within a DNS zone. Retrieve a list of name servers within
   * a DNS zone and their optional glue records.
   *
   * @param request - The request {@link ListDNSZoneNameserversRequest}
   * @returns A Promise of ListDNSZoneNameserversResponse
   */
  listDNSZoneNameservers = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/nameservers`,
    urlParams: urlParams(['project_id', request.projectId])
  }, unmarshalListDNSZoneNameserversResponse);

  /**
   * Update name servers within a DNS zone. Update name servers within a DNS
   * zone and set optional glue records.
   *
   * @param request - The request {@link UpdateDNSZoneNameserversRequest}
   * @returns A Promise of UpdateDNSZoneNameserversResponse
   */
  updateDNSZoneNameservers = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDNSZoneNameserversRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'PUT',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/nameservers`
  }, unmarshalUpdateDNSZoneNameserversResponse);

  /**
   * Clear records within a DNS zone. Delete all records within a DNS zone that
   * has default name servers.<br/> All edits will be versioned.
   *
   * @param request - The request {@link ClearDNSZoneRecordsRequest}
   * @returns A Promise of ClearDNSZoneRecordsResponse
   */
  clearDNSZoneRecords = request => this.client.fetch({
    method: 'DELETE',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/records`
  }, unmarshalClearDNSZoneRecordsResponse);

  /**
   * Export a raw DNS zone. Export a DNS zone with default name servers, in a
   * specific format.
   *
   * @param request - The request {@link ExportRawDNSZoneRequest}
   * @returns A Promise of Blob
   */
  exportRawDNSZone = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/raw`,
    urlParams: urlParams(['dl', 1], ['format', request.format]),
    responseType: 'blob'
  });

  /**
   * Import a raw DNS zone. Import and replace the format of records from a
   * given provider, with default name servers.
   *
   * @param request - The request {@link ImportRawDNSZoneRequest}
   * @returns A Promise of ImportRawDNSZoneResponse
   */
  importRawDNSZone = request => this.client.fetch({
    body: JSON.stringify(marshalImportRawDNSZoneRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/raw`
  }, unmarshalImportRawDNSZoneResponse);

  /**
   * Import a DNS zone from another provider. Import and replace the format of
   * records from a given provider, with default name servers.
   *
   * @param request - The request {@link ImportProviderDNSZoneRequest}
   * @returns A Promise of ImportProviderDNSZoneResponse
   */
  importProviderDNSZone = request => this.client.fetch({
    body: JSON.stringify(marshalImportProviderDNSZoneRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/import-provider`
  }, unmarshalImportProviderDNSZoneResponse);

  /**
   * Refresh a DNS zone. Refresh an SOA DNS zone to reload the records in the
   * DNS zone and update the SOA serial. You can recreate the given DNS zone and
   * its sub DNS zone if needed.
   *
   * @param request - The request {@link RefreshDNSZoneRequest}
   * @returns A Promise of RefreshDNSZoneResponse
   */
  refreshDNSZone = request => this.client.fetch({
    body: JSON.stringify(marshalRefreshDNSZoneRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/refresh`
  }, unmarshalRefreshDNSZoneResponse);
  pageOfListDNSZoneVersions = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/versions`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDNSZoneVersionsResponse);

  /**
   * List versions of a DNS zone. Retrieve a list of a DNS zone's versions.<br/>
   * The maximum version count is 100. If the count reaches this limit, the
   * oldest version will be deleted after each new modification.
   *
   * @param request - The request {@link ListDNSZoneVersionsRequest}
   * @returns A Promise of ListDNSZoneVersionsResponse
   */
  listDNSZoneVersions = request => enrichForPagination('versions', this.pageOfListDNSZoneVersions, request);
  pageOfListDNSZoneVersionRecords = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/dns-zones/version/${validatePathParam('dnsZoneVersionId', request.dnsZoneVersionId)}`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDNSZoneVersionRecordsResponse);

  /**
   * List records from a given version of a specific DNS zone. Retrieve a list
   * of records from a specific DNS zone version.
   *
   * @param request - The request {@link ListDNSZoneVersionRecordsRequest}
   * @returns A Promise of ListDNSZoneVersionRecordsResponse
   */
  listDNSZoneVersionRecords = request => enrichForPagination('records', this.pageOfListDNSZoneVersionRecords, request);

  /**
   * Access differences from a specific DNS zone version. Access a previous DNS
   * zone version to see the differences from another specific version.
   *
   * @param request - The request {@link GetDNSZoneVersionDiffRequest}
   * @returns A Promise of GetDNSZoneVersionDiffResponse
   */
  getDNSZoneVersionDiff = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/dns-zones/version/${validatePathParam('dnsZoneVersionId', request.dnsZoneVersionId)}/diff`
  }, unmarshalGetDNSZoneVersionDiffResponse);

  /**
   * Restore a DNS zone version. Restore and activate a version of a specific
   * DNS zone.
   *
   * @param request - The request {@link RestoreDNSZoneVersionRequest}
   * @returns A Promise of RestoreDNSZoneVersionResponse
   */
  restoreDNSZoneVersion = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/dns-zones/version/${validatePathParam('dnsZoneVersionId', request.dnsZoneVersionId)}/restore`
  }, unmarshalRestoreDNSZoneVersionResponse);

  /**
   * Get a DNS zone's TLS certificate. Get the DNS zone's TLS certificate. If
   * you do not have a certificate, the ouptut returns `no certificate found`.
   *
   * @param request - The request {@link GetSSLCertificateRequest}
   * @returns A Promise of SSLCertificate
   */
  getSSLCertificate = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/ssl-certificates/${validatePathParam('dnsZone', request.dnsZone)}`
  }, unmarshalSSLCertificate);

  /**
   * Waits for {@link SSLCertificate} to be in a final state.
   *
   * @param request - The request {@link GetSSLCertificateRequest}
   * @param options - The waiting options
   * @returns A Promise of SSLCertificate
   */
  waitForSSLCertificate = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SSL_CERTIFICATE_TRANSIENT_STATUSES.includes(res.status))), this.getSSLCertificate, request, options);

  /**
   * Create or get the DNS zone's TLS certificate. Create a new TLS certificate
   * or retrieve information about an existing TLS certificate.
   *
   * @param request - The request {@link CreateSSLCertificateRequest}
   * @returns A Promise of SSLCertificate
   */
  createSSLCertificate = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSSLCertificateRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/ssl-certificates`
  }, unmarshalSSLCertificate);
  pageOfListSSLCertificates = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/ssl-certificates`,
    urlParams: urlParams(['dns_zone', request.dnsZone], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListSSLCertificatesResponse);

  /**
   * List a user's TLS certificates. List all the TLS certificates a user has
   * created, specified by the user's Project ID and the DNS zone.
   *
   * @param request - The request {@link ListSSLCertificatesRequest}
   * @returns A Promise of ListSSLCertificatesResponse
   */
  listSSLCertificates = request => enrichForPagination('certificates', this.pageOfListSSLCertificates, request);

  /**
   * Delete a TLS certificate. Delete an existing TLS certificate specified by
   * its DNS zone. Deleting a TLS certificate is permanent and cannot be
   * undone.
   *
   * @param request - The request {@link DeleteSSLCertificateRequest}
   * @returns A Promise of DeleteSSLCertificateResponse
   */
  deleteSSLCertificate = request => this.client.fetch({
    method: 'DELETE',
    path: `/domain/v2beta1/ssl-certificates/${validatePathParam('dnsZone', request.dnsZone)}`
  }, unmarshalDeleteSSLCertificateResponse);

  /**
   * Get the DNS zone's TSIG key. Retrieve information about the TSIG key of a
   * given DNS zone to allow AXFR requests.
   *
   * @param request - The request {@link GetDNSZoneTsigKeyRequest}
   * @returns A Promise of GetDNSZoneTsigKeyResponse
   */
  getDNSZoneTsigKey = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/tsig-key`
  }, unmarshalGetDNSZoneTsigKeyResponse);

  /**
   * Delete the DNS zone's TSIG key. Delete an existing TSIG key specified by
   * its DNS zone. Deleting a TSIG key is permanent and cannot be undone.
   *
   * @param request - The request {@link DeleteDNSZoneTsigKeyRequest}
   */
  deleteDNSZoneTsigKey = request => this.client.fetch({
    method: 'DELETE',
    path: `/domain/v2beta1/dns-zones/${validatePathParam('dnsZone', request.dnsZone)}/tsig-key`
  });
};

/**
 * Domains and DNS - Registrar API.
 *
 * Manage your domains and contacts.
 */
class RegistrarAPI extends API$z {
  pageOfListTasks = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/tasks`,
    urlParams: urlParams(['domain', request.domain], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['statuses', request.statuses], ['types', request.types])
  }, unmarshalListTasksResponse);

  /**
   * List tasks. List all operations performed on the account. You can filter
   * the list of tasks by domain name.
   *
   * @param request - The request {@link RegistrarApiListTasksRequest}
   * @returns A Promise of ListTasksResponse
   */
  listTasks = (request = {}) => enrichForPagination('tasks', this.pageOfListTasks, request);

  /**
   * Purchase domains. Request the registration of domain names. You can provide
   * a domain's already existing contact or a new contact.
   *
   * @param request - The request {@link RegistrarApiBuyDomainsRequest}
   * @returns A Promise of OrderResponse
   */
  buyDomains = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiBuyDomainsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/buy-domains`
  }, unmarshalOrderResponse);

  /**
   * Renew domains. Request the renewal of one or more domain names.
   *
   * @param request - The request {@link RegistrarApiRenewDomainsRequest}
   * @returns A Promise of OrderResponse
   */
  renewDomains = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiRenewDomainsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/renew-domains`
  }, unmarshalOrderResponse);

  /**
   * Transfer a domain. Request the transfer of a domain from another registrar
   * to Scaleway Domains and DNS.
   *
   * @param request - The request {@link RegistrarApiTransferInDomainRequest}
   * @returns A Promise of OrderResponse
   */
  transferInDomain = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiTransferInDomainRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/transfer-domains`
  }, unmarshalOrderResponse);

  /**
   * Trade a domain's contact. Request to change a domain's contact owner.<br/>
   * If you specify the `organization_id` of the domain's new owner, the contact
   * will change from the current owner's Scaleway account to the new owner's
   * Scaleway account.<br/> If the new owner's current contact information is
   * not available, the first ever contact they have created for previous
   * domains is taken into account to operate the change.<br/> If the new owner
   * has never created a contact to register domains before, an error message
   * displays.
   *
   * @param request - The request {@link RegistrarApiTradeDomainRequest}
   * @returns A Promise of OrderResponse
   */
  tradeDomain = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiTradeDomainRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/trade`
  }, unmarshalOrderResponse);

  /**
   * Register an external domain. Request the registration of an external domain
   * name.
   *
   * @param request - The request
   *   {@link RegistrarApiRegisterExternalDomainRequest}
   * @returns A Promise of RegisterExternalDomainResponse
   */
  registerExternalDomain = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiRegisterExternalDomainRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/external-domains`
  }, unmarshalRegisterExternalDomainResponse);

  /**
   * Delete an external domain. Delete an external domain name.
   *
   * @param request - The request {@link RegistrarApiDeleteExternalDomainRequest}
   * @returns A Promise of DeleteExternalDomainResponse
   */
  deleteExternalDomain = request => this.client.fetch({
    method: 'DELETE',
    path: `/domain/v2beta1/external-domains/${validatePathParam('domain', request.domain)}`
  }, unmarshalDeleteExternalDomainResponse);

  /**
   * Check if contacts are compatible with a domain or a TLD. Check whether
   * contacts are compatible with a domain or a TLD. If contacts are not
   * compatible with either the domain or the TLD, the information that needs to
   * be corrected is returned.
   *
   * @param request - The request
   *   {@link RegistrarApiCheckContactsCompatibilityRequest}
   * @returns A Promise of CheckContactsCompatibilityResponse
   */
  checkContactsCompatibility = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiCheckContactsCompatibilityRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/check-contacts-compatibility`
  }, unmarshalCheckContactsCompatibilityResponse);
  pageOfListContacts = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/contacts`,
    urlParams: urlParams(['domain', request.domain], ['email_status', request.emailStatus], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['role', request.role])
  }, unmarshalListContactsResponse);

  /**
   * List contacts. Retrieve the list of contacts and their associated domains
   * and roles. You can filter the list by domain name.
   *
   * @param request - The request {@link RegistrarApiListContactsRequest}
   * @returns A Promise of ListContactsResponse
   */
  listContacts = (request = {}) => enrichForPagination('contacts', this.pageOfListContacts, request);

  /**
   * Get a contact. Retrieve a contact's details from the registrar using the
   * given contact's ID.
   *
   * @param request - The request {@link RegistrarApiGetContactRequest}
   * @returns A Promise of Contact
   */
  getContact = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/contacts/${validatePathParam('contactId', request.contactId)}`
  }, unmarshalContact);

  /**
   * Update contact. Edit the contact's information.
   *
   * @param request - The request {@link RegistrarApiUpdateContactRequest}
   * @returns A Promise of Contact
   */
  updateContact = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiUpdateContactRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'PATCH',
    path: `/domain/v2beta1/contacts/${validatePathParam('contactId', request.contactId)}`
  }, unmarshalContact);
  pageOfListDomains = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/domains`,
    urlParams: urlParams(['domain', request.domain], ['is_external', request.isExternal], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['registrar', request.registrar], ['status', request.status])
  }, unmarshalListDomainsResponse$2);

  /**
   * List domains. Retrieve the list of domains you own.
   *
   * @param request - The request {@link RegistrarApiListDomainsRequest}
   * @returns A Promise of ListDomainsResponse
   */
  listDomains = (request = {}) => enrichForPagination('domains', this.pageOfListDomains, request);
  pageOfListRenewableDomains = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/renewable-domains`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListRenewableDomainsResponse);

  /**
   * List domains that can be renewed. Retrieve the list of domains you own that
   * can be renewed. You can also see the maximum renewal duration in years for
   * your domains that are renewable.
   *
   * @param request - The request {@link RegistrarApiListRenewableDomainsRequest}
   * @returns A Promise of ListRenewableDomainsResponse
   */
  listRenewableDomains = (request = {}) => enrichForPagination('domains', this.pageOfListRenewableDomains, request);

  /**
   * Get domain. Retrieve a specific domain and display the domain's
   * information.
   *
   * @param request - The request {@link RegistrarApiGetDomainRequest}
   * @returns A Promise of Domain
   */
  getDomain = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}`
  }, unmarshalDomain$2);

  /**
   * Waits for {@link Domain} to be in a final state.
   *
   * @param request - The request {@link RegistrarApiGetDomainRequest}
   * @param options - The waiting options
   * @returns A Promise of Domain
   */
  waitForDomain = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!DOMAIN_TRANSIENT_STATUSES$2.includes(res.status))), this.getDomain, request, options);

  /**
   * Update a domain's contacts. Update contacts for a specific domain or create
   * a new contact.<br/> If you add the same contact for multiple roles (owner,
   * administrative, technical), only one ID will be created and used for all of
   * the roles.
   *
   * @param request - The request {@link RegistrarApiUpdateDomainRequest}
   * @returns A Promise of Domain
   */
  updateDomain = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiUpdateDomainRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'PATCH',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}`
  }, unmarshalDomain$2);

  /**
   * Lock the transfer of a domain. Lock the transfer of a domain. This means
   * that the domain cannot be transferred and the authorization code cannot be
   * requested to your current registrar.
   *
   * @param request - The request {@link RegistrarApiLockDomainTransferRequest}
   * @returns A Promise of Domain
   */
  lockDomainTransfer = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/lock-transfer`
  }, unmarshalDomain$2);

  /**
   * Unlock the transfer of a domain. Unlock the transfer of a domain. This
   * means that the domain can be transferred and the authorization code can be
   * requested to your current registrar.
   *
   * @param request - The request {@link RegistrarApiUnlockDomainTransferRequest}
   * @returns A Promise of Domain
   */
  unlockDomainTransfer = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/unlock-transfer`
  }, unmarshalDomain$2);

  /**
   * Enable auto renew. Enable the `auto renew` feature for a domain. This means
   * the domain will be automatically renewed before its expiry date.
   *
   * @param request - The request
   *   {@link RegistrarApiEnableDomainAutoRenewRequest}
   * @returns A Promise of Domain
   */
  enableDomainAutoRenew = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/enable-auto-renew`
  }, unmarshalDomain$2);

  /**
   * Disable auto renew. Disable the `auto renew` feature for a domain. This
   * means the domain will not be renewed before its expiry date.
   *
   * @param request - The request
   *   {@link RegistrarApiDisableDomainAutoRenewRequest}
   * @returns A Promise of Domain
   */
  disableDomainAutoRenew = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/disable-auto-renew`
  }, unmarshalDomain$2);

  /**
   * Get a domain's authorization code. Retrieve the authorization code to
   * tranfer an unlocked domain. The output returns an error if the domain is
   * locked. Some TLDs may have a different procedure to retrieve the
   * authorization code. In that case, the information displays in the message
   * field.
   *
   * @param request - The request {@link RegistrarApiGetDomainAuthCodeRequest}
   * @returns A Promise of GetDomainAuthCodeResponse
   */
  getDomainAuthCode = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/auth-code`
  }, unmarshalGetDomainAuthCodeResponse);

  /**
   * Update domain DNSSEC. If your domain has the default Scaleway NS and uses
   * another registrar, you have to update the DS record manually.
   *
   * @param request - The request {@link RegistrarApiEnableDomainDNSSECRequest}
   * @returns A Promise of Domain
   */
  enableDomainDNSSEC = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiEnableDomainDNSSECRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/enable-dnssec`
  }, unmarshalDomain$2);

  /**
   * Disable a domain's DNSSEC. Disable DNSSEC for a domain.
   *
   * @param request - The request {@link RegistrarApiDisableDomainDNSSECRequest}
   * @returns A Promise of Domain
   */
  disableDomainDNSSEC = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/disable-dnssec`
  }, unmarshalDomain$2);

  /**
   * Search available domains. Search a domain or a maximum of 10 domains that
   * are available.
   *
   * If the TLD list is empty or not set, the search returns the results from
   * the most popular TLDs.
   *
   * @param request - The request
   *   {@link RegistrarApiSearchAvailableDomainsRequest}
   * @returns A Promise of SearchAvailableDomainsResponse
   */
  searchAvailableDomains = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/search-domains`,
    urlParams: urlParams(['domains', request.domains], ['strict_search', request.strictSearch], ['tlds', request.tlds])
  }, unmarshalSearchAvailableDomainsResponse);
  pageOfListTlds = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/tlds`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['tlds', request.tlds])
  }, unmarshalListTldsResponse);

  /**
   * List TLD offers. Retrieve the list of TLDs and offers associated with them.
   *
   * @param request - The request {@link RegistrarApiListTldsRequest}
   * @returns A Promise of ListTldsResponse
   */
  listTlds = (request = {}) => enrichForPagination('tlds', this.pageOfListTlds, request);

  /**
   * Create a hostname for a domain. Create a hostname for a domain with glue
   * IPs.
   *
   * @param request - The request {@link RegistrarApiCreateDomainHostRequest}
   * @returns A Promise of Host
   */
  createDomainHost = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiCreateDomainHostRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'POST',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/hosts`
  }, unmarshalHost);
  pageOfListDomainHosts = request => this.client.fetch({
    method: 'GET',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/hosts`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDomainHostsResponse);

  /**
   * List a domain's hostnames. List a domain's hostnames using their glue IPs.
   *
   * @param request - The request {@link RegistrarApiListDomainHostsRequest}
   * @returns A Promise of ListDomainHostsResponse
   */
  listDomainHosts = request => enrichForPagination('hosts', this.pageOfListDomainHosts, request);

  /**
   * Update a domain's hostname. Update a domain's hostname with glue IPs.
   *
   * @param request - The request {@link RegistrarApiUpdateDomainHostRequest}
   * @returns A Promise of Host
   */
  updateDomainHost = request => this.client.fetch({
    body: JSON.stringify(marshalRegistrarApiUpdateDomainHostRequest(request, this.client.settings)),
    headers: jsonContentHeaders$o,
    method: 'PATCH',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/hosts/${validatePathParam('name', request.name)}`
  }, unmarshalHost);

  /**
   * Delete a domain's hostname.
   *
   * @param request - The request {@link RegistrarApiDeleteDomainHostRequest}
   * @returns A Promise of Host
   */
  deleteDomainHost = request => this.client.fetch({
    method: 'DELETE',
    path: `/domain/v2beta1/domains/${validatePathParam('domain', request.domain)}/hosts/${validatePathParam('name', request.name)}`
  }, unmarshalHost);
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$n = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$o,
  DNS_ZONE_TRANSIENT_STATUSES: DNS_ZONE_TRANSIENT_STATUSES,
  DOMAIN_FEATURE_TRANSIENT_STATUSES: DOMAIN_FEATURE_TRANSIENT_STATUSES,
  DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES: DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES,
  DOMAIN_TRANSIENT_STATUSES: DOMAIN_TRANSIENT_STATUSES$2,
  HOST_TRANSIENT_STATUSES: HOST_TRANSIENT_STATUSES,
  RegistrarAPI: RegistrarAPI,
  SSL_CERTIFICATE_TRANSIENT_STATUSES: SSL_CERTIFICATE_TRANSIENT_STATUSES,
  TASK_TRANSIENT_STATUSES: TASK_TRANSIENT_STATUSES$1
});

var index$q = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v2beta1: index_gen$n
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link FlexibleIPStatus}. */
const FLEXIBLE_IP_TRANSIENT_STATUSES = ['updating', 'detaching'];

/** Lists transient statutes of the enum {@link MACAddressStatus}. */
const MAC_ADDRESS_TRANSIENT_STATUSES = ['updating', 'deleting'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalMACAddress = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'MACAddress' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    macAddress: data.mac_address,
    macType: data.mac_type,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalFlexibleIP = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'FlexibleIP' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    id: data.id,
    ipAddress: data.ip_address,
    macAddress: data.mac_address ? unmarshalMACAddress(data.mac_address) : undefined,
    organizationId: data.organization_id,
    projectId: data.project_id,
    reverse: data.reverse,
    serverId: data.server_id,
    status: data.status,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalAttachFlexibleIPsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AttachFlexibleIPsResponse' failed as data isn't a dictionary.`);
  }
  return {
    flexibleIps: unmarshalArrayOfObject(data.flexible_ips, unmarshalFlexibleIP),
    totalCount: data.total_count
  };
};
const unmarshalDetachFlexibleIPsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DetachFlexibleIPsResponse' failed as data isn't a dictionary.`);
  }
  return {
    flexibleIps: unmarshalArrayOfObject(data.flexible_ips, unmarshalFlexibleIP),
    totalCount: data.total_count
  };
};
const unmarshalListFlexibleIPsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListFlexibleIPsResponse' failed as data isn't a dictionary.`);
  }
  return {
    flexibleIps: unmarshalArrayOfObject(data.flexible_ips, unmarshalFlexibleIP),
    totalCount: data.total_count
  };
};
const marshalAttachFlexibleIPRequest = (request, defaults) => ({
  fips_ids: request.fipsIds,
  server_id: request.serverId
});
const marshalCreateFlexibleIPRequest = (request, defaults) => ({
  description: request.description,
  is_ipv6: request.isIpv6,
  project_id: request.projectId ?? defaults.defaultProjectId,
  reverse: request.reverse,
  server_id: request.serverId,
  tags: request.tags
});
const marshalDetachFlexibleIPRequest = (request, defaults) => ({
  fips_ids: request.fipsIds
});
const marshalDuplicateMACAddrRequest = (request, defaults) => ({
  duplicate_from_fip_id: request.duplicateFromFipId
});
const marshalGenerateMACAddrRequest = (request, defaults) => ({
  mac_type: request.macType
});
const marshalMoveMACAddrRequest = (request, defaults) => ({
  dst_fip_id: request.dstFipId
});
const marshalUpdateFlexibleIPRequest = (request, defaults) => ({
  description: request.description,
  reverse: request.reverse,
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$n = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Elastic Metal - Flexible IP API. */
let API$n = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'nl-ams-1'];

  /**
   * Create a new flexible IP. Generate a new flexible IP within a given zone,
   * specifying its configuration including Project ID and description.
   *
   * @param request - The request {@link CreateFlexibleIPRequest}
   * @returns A Promise of FlexibleIP
   */
  createFlexibleIP = request => this.client.fetch({
    body: JSON.stringify(marshalCreateFlexibleIPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$n,
    method: 'POST',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips`
  }, unmarshalFlexibleIP);

  /**
   * Get an existing flexible IP. Retrieve information about an existing
   * flexible IP, specified by its ID and zone. Its full details, including
   * Project ID, description and status, are returned in the response object.
   *
   * @param request - The request {@link GetFlexibleIPRequest}
   * @returns A Promise of FlexibleIP
   */
  getFlexibleIP = request => this.client.fetch({
    method: 'GET',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/${validatePathParam('fipId', request.fipId)}`
  }, unmarshalFlexibleIP);

  /**
   * Waits for {@link FlexibleIP} to be in a final state.
   *
   * @param request - The request {@link GetFlexibleIPRequest}
   * @param options - The waiting options
   * @returns A Promise of FlexibleIP
   */
  waitForFlexibleIP = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!FLEXIBLE_IP_TRANSIENT_STATUSES.includes(res.status))), this.getFlexibleIP, request, options);
  pageOfListFlexibleIPs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['server_ids', request.serverIds], ['status', request.status], ['tags', request.tags])
  }, unmarshalListFlexibleIPsResponse);

  /**
   * List flexible IPs. List all flexible IPs within a given zone.
   *
   * @param request - The request {@link ListFlexibleIPsRequest}
   * @returns A Promise of ListFlexibleIPsResponse
   */
  listFlexibleIPs = (request = {}) => enrichForPagination('flexibleIps', this.pageOfListFlexibleIPs, request);

  /**
   * Update an existing flexible IP. Update the parameters of an existing
   * flexible IP, specified by its ID and zone. These parameters include tags
   * and description.
   *
   * @param request - The request {@link UpdateFlexibleIPRequest}
   * @returns A Promise of FlexibleIP
   */
  updateFlexibleIP = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateFlexibleIPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$n,
    method: 'PATCH',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/${validatePathParam('fipId', request.fipId)}`
  }, unmarshalFlexibleIP);

  /**
   * Delete an existing flexible IP. Delete an existing flexible IP, specified
   * by its ID and zone. Note that deleting a flexible IP is permanent and
   * cannot be undone.
   *
   * @param request - The request {@link DeleteFlexibleIPRequest}
   */
  deleteFlexibleIP = request => this.client.fetch({
    method: 'DELETE',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/${validatePathParam('fipId', request.fipId)}`
  });

  /**
   * Attach an existing flexible IP to a server. Attach an existing flexible IP
   * to a specified Elastic Metal server.
   *
   * @param request - The request {@link AttachFlexibleIPRequest}
   * @returns A Promise of AttachFlexibleIPsResponse
   */
  attachFlexibleIP = request => this.client.fetch({
    body: JSON.stringify(marshalAttachFlexibleIPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$n,
    method: 'POST',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/attach`
  }, unmarshalAttachFlexibleIPsResponse);

  /**
   * Detach an existing flexible IP from a server. Detach an existing flexible
   * IP from a specified Elastic Metal server.
   *
   * @param request - The request {@link DetachFlexibleIPRequest}
   * @returns A Promise of DetachFlexibleIPsResponse
   */
  detachFlexibleIP = request => this.client.fetch({
    body: JSON.stringify(marshalDetachFlexibleIPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$n,
    method: 'POST',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/detach`
  }, unmarshalDetachFlexibleIPsResponse);

  /**
   * Generate a virtual MAC address on an existing flexible IP. Generate a
   * virtual MAC (Media Access Control) address on an existing flexible IP.
   *
   * @param request - The request {@link GenerateMACAddrRequest}
   * @returns A Promise of FlexibleIP
   */
  generateMACAddr = request => this.client.fetch({
    body: JSON.stringify(marshalGenerateMACAddrRequest(request, this.client.settings)),
    headers: jsonContentHeaders$n,
    method: 'POST',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/${validatePathParam('fipId', request.fipId)}/mac`
  }, unmarshalFlexibleIP);

  /**
   * Duplicate a virtual MAC address to another flexible IP. Duplicate a virtual
   * MAC address from a given flexible IP to another flexible IP attached to the
   * same server.
   *
   * @param request - The request {@link DuplicateMACAddrRequest}
   * @returns A Promise of FlexibleIP
   */
  duplicateMACAddr = request => this.client.fetch({
    body: JSON.stringify(marshalDuplicateMACAddrRequest(request, this.client.settings)),
    headers: jsonContentHeaders$n,
    method: 'POST',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/${validatePathParam('fipId', request.fipId)}/mac/duplicate`
  }, unmarshalFlexibleIP);

  /**
   * Relocate an existing virtual MAC address to a different flexible IP.
   * Relocate a virtual MAC (Media Access Control) address from an existing
   * flexible IP to a different flexible IP.
   *
   * @param request - The request {@link MoveMACAddrRequest}
   * @returns A Promise of FlexibleIP
   */
  moveMACAddr = request => this.client.fetch({
    body: JSON.stringify(marshalMoveMACAddrRequest(request, this.client.settings)),
    headers: jsonContentHeaders$n,
    method: 'POST',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/${validatePathParam('fipId', request.fipId)}/mac/move`
  }, unmarshalFlexibleIP);

  /**
   * Detach a given virtual MAC address from an existing flexible IP. Detach a
   * given MAC (Media Access Control) address from an existing flexible IP.
   *
   * @param request - The request {@link DeleteMACAddrRequest}
   */
  deleteMACAddr = request => this.client.fetch({
    method: 'DELETE',
    path: `/flexible-ip/v1alpha1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/fips/${validatePathParam('fipId', request.fipId)}/mac`
  });
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const ListFlexibleIPsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};

var validationRules_gen$b = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ListFlexibleIPsRequest: ListFlexibleIPsRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$m = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$n,
  FLEXIBLE_IP_TRANSIENT_STATUSES: FLEXIBLE_IP_TRANSIENT_STATUSES,
  MAC_ADDRESS_TRANSIENT_STATUSES: MAC_ADDRESS_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen$b
});

var index$p = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$m
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link CronStatus}. */
const CRON_TRANSIENT_STATUSES = ['deleting', 'creating', 'pending'];

/** Lists transient statutes of the enum {@link DomainStatus}. */
const DOMAIN_TRANSIENT_STATUSES$1 = ['deleting', 'creating', 'pending'];

/** Lists transient statutes of the enum {@link FunctionStatus}. */
const FUNCTION_TRANSIENT_STATUSES = ['deleting', 'creating', 'pending'];

/** Lists transient statutes of the enum {@link NamespaceStatus}. */
const NAMESPACE_TRANSIENT_STATUSES$1 = ['deleting', 'creating', 'pending'];

/** Lists transient statutes of the enum {@link TokenStatus}. */
const TOKEN_TRANSIENT_STATUSES = ['deleting', 'creating'];

/** Lists transient statutes of the enum {@link TriggerStatus}. */
const TRIGGER_TRANSIENT_STATUSES = ['deleting', 'creating', 'pending'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalCron = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Cron' failed as data isn't a dictionary.`);
  }
  return {
    args: data.args,
    functionId: data.function_id,
    id: data.id,
    name: data.name,
    schedule: data.schedule,
    status: data.status
  };
};
const unmarshalDomain$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Domain' failed as data isn't a dictionary.`);
  }
  return {
    errorMessage: data.error_message,
    functionId: data.function_id,
    hostname: data.hostname,
    id: data.id,
    status: data.status,
    url: data.url
  };
};
const unmarshalSecretHashedValue = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SecretHashedValue' failed as data isn't a dictionary.`);
  }
  return {
    hashedValue: data.hashed_value,
    key: data.key
  };
};
const unmarshalFunction = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Function' failed as data isn't a dictionary.`);
  }
  return {
    buildMessage: data.build_message,
    cpuLimit: data.cpu_limit,
    description: data.description,
    domainName: data.domain_name,
    environmentVariables: data.environment_variables,
    errorMessage: data.error_message,
    handler: data.handler,
    httpOption: data.http_option,
    id: data.id,
    maxScale: data.max_scale,
    memoryLimit: data.memory_limit,
    minScale: data.min_scale,
    name: data.name,
    namespaceId: data.namespace_id,
    privacy: data.privacy,
    region: data.region,
    runtime: data.runtime,
    runtimeMessage: data.runtime_message,
    secretEnvironmentVariables: unmarshalArrayOfObject(data.secret_environment_variables, unmarshalSecretHashedValue),
    status: data.status,
    timeout: data.timeout
  };
};
const unmarshalNamespace$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Namespace' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    environmentVariables: data.environment_variables,
    errorMessage: data.error_message,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    region: data.region,
    registryEndpoint: data.registry_endpoint,
    registryNamespaceId: data.registry_namespace_id,
    secretEnvironmentVariables: unmarshalArrayOfObject(data.secret_environment_variables, unmarshalSecretHashedValue),
    status: data.status
  };
};
const unmarshalToken = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Token' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    expiresAt: unmarshalDate(data.expires_at),
    functionId: data.function_id,
    id: data.id,
    namespaceId: data.namespace_id,
    publicKey: data.public_key,
    status: data.status,
    token: data.token
  };
};
const unmarshalTriggerMnqNatsClientConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TriggerMnqNatsClientConfig' failed as data isn't a dictionary.`);
  }
  return {
    mnqCredentialId: data.mnq_credential_id,
    mnqNatsAccountId: data.mnq_nats_account_id,
    mnqProjectId: data.mnq_project_id,
    mnqRegion: data.mnq_region,
    subject: data.subject
  };
};
const unmarshalTriggerMnqSqsClientConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TriggerMnqSqsClientConfig' failed as data isn't a dictionary.`);
  }
  return {
    mnqCredentialId: data.mnq_credential_id,
    mnqProjectId: data.mnq_project_id,
    mnqRegion: data.mnq_region,
    queue: data.queue
  };
};
const unmarshalTriggerSqsClientConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TriggerSqsClientConfig' failed as data isn't a dictionary.`);
  }
  return {
    accessKey: data.access_key,
    endpoint: data.endpoint,
    queueUrl: data.queue_url,
    secretKey: data.secret_key
  };
};
const unmarshalTrigger = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Trigger' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    errorMessage: data.error_message,
    functionId: data.function_id,
    id: data.id,
    inputType: data.input_type,
    name: data.name,
    scwNatsConfig: data.scw_nats_config ? unmarshalTriggerMnqNatsClientConfig(data.scw_nats_config) : undefined,
    scwSqsConfig: data.scw_sqs_config ? unmarshalTriggerMnqSqsClientConfig(data.scw_sqs_config) : undefined,
    sqsConfig: data.sqs_config ? unmarshalTriggerSqsClientConfig(data.sqs_config) : undefined,
    status: data.status
  };
};
const unmarshalDownloadURL = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DownloadURL' failed as data isn't a dictionary.`);
  }
  return {
    headers: data.headers,
    url: data.url
  };
};
const unmarshalListCronsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListCronsResponse' failed as data isn't a dictionary.`);
  }
  return {
    crons: unmarshalArrayOfObject(data.crons, unmarshalCron),
    totalCount: data.total_count
  };
};
const unmarshalListDomainsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary.`);
  }
  return {
    domains: unmarshalArrayOfObject(data.domains, unmarshalDomain$1),
    totalCount: data.total_count
  };
};
const unmarshalRuntime = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Runtime' failed as data isn't a dictionary.`);
  }
  return {
    codeSample: data.code_sample,
    defaultHandler: data.default_handler,
    extension: data.extension,
    implementation: data.implementation,
    language: data.language,
    logoUrl: data.logo_url,
    name: data.name,
    status: data.status,
    statusMessage: data.status_message,
    version: data.version
  };
};
const unmarshalListFunctionRuntimesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListFunctionRuntimesResponse' failed as data isn't a dictionary.`);
  }
  return {
    runtimes: unmarshalArrayOfObject(data.runtimes, unmarshalRuntime),
    totalCount: data.total_count
  };
};
const unmarshalListFunctionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListFunctionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    functions: unmarshalArrayOfObject(data.functions, unmarshalFunction),
    totalCount: data.total_count
  };
};
const unmarshalListNamespacesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNamespacesResponse' failed as data isn't a dictionary.`);
  }
  return {
    namespaces: unmarshalArrayOfObject(data.namespaces, unmarshalNamespace$1),
    totalCount: data.total_count
  };
};
const unmarshalListTokensResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTokensResponse' failed as data isn't a dictionary.`);
  }
  return {
    tokens: unmarshalArrayOfObject(data.tokens, unmarshalToken),
    totalCount: data.total_count
  };
};
const unmarshalListTriggersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTriggersResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    triggers: unmarshalArrayOfObject(data.triggers, unmarshalTrigger)
  };
};
const unmarshalUploadURL = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UploadURL' failed as data isn't a dictionary.`);
  }
  return {
    headers: data.headers,
    url: data.url
  };
};
const marshalCreateCronRequest = (request, defaults) => ({
  args: request.args,
  function_id: request.functionId,
  name: request.name,
  schedule: request.schedule
});
const marshalCreateDomainRequest$1 = (request, defaults) => ({
  function_id: request.functionId,
  hostname: request.hostname
});
const marshalSecret = (request, defaults) => ({
  key: request.key,
  value: request.value
});
const marshalCreateFunctionRequest = (request, defaults) => ({
  description: request.description,
  environment_variables: request.environmentVariables,
  handler: request.handler,
  http_option: request.httpOption,
  max_scale: request.maxScale,
  memory_limit: request.memoryLimit,
  min_scale: request.minScale,
  name: request.name || randomName('fn'),
  namespace_id: request.namespaceId,
  privacy: request.privacy,
  runtime: request.runtime,
  secret_environment_variables: request.secretEnvironmentVariables !== undefined ? request.secretEnvironmentVariables.map(elt => marshalSecret(elt)) : undefined,
  timeout: request.timeout
});
const marshalCreateNamespaceRequest$1 = (request, defaults) => ({
  description: request.description,
  environment_variables: request.environmentVariables,
  name: request.name || randomName('ns'),
  project_id: request.projectId ?? defaults.defaultProjectId,
  secret_environment_variables: request.secretEnvironmentVariables !== undefined ? request.secretEnvironmentVariables.map(elt => marshalSecret(elt)) : undefined
});
const marshalCreateTokenRequest = (request, defaults) => ({
  description: request.description,
  expires_at: request.expiresAt,
  ...resolveOneOf([{
    param: 'function_id',
    value: request.functionId
  }, {
    param: 'namespace_id',
    value: request.namespaceId
  }])
});
const marshalCreateTriggerRequestMnqNatsClientConfig = (request, defaults) => ({
  mnq_nats_account_id: request.mnqNatsAccountId,
  mnq_project_id: request.mnqProjectId,
  mnq_region: request.mnqRegion,
  subject: request.subject
});
const marshalCreateTriggerRequestMnqSqsClientConfig = (request, defaults) => ({
  mnq_project_id: request.mnqProjectId,
  mnq_region: request.mnqRegion,
  queue: request.queue
});
const marshalCreateTriggerRequestSqsClientConfig = (request, defaults) => ({
  access_key: request.accessKey,
  endpoint: request.endpoint,
  queue_url: request.queueUrl,
  secret_key: request.secretKey
});
const marshalCreateTriggerRequest = (request, defaults) => ({
  description: request.description,
  function_id: request.functionId,
  name: request.name,
  ...resolveOneOf([{
    param: 'scw_sqs_config',
    value: request.scwSqsConfig !== undefined ? marshalCreateTriggerRequestMnqSqsClientConfig(request.scwSqsConfig) : undefined
  }, {
    param: 'scw_nats_config',
    value: request.scwNatsConfig !== undefined ? marshalCreateTriggerRequestMnqNatsClientConfig(request.scwNatsConfig) : undefined
  }, {
    param: 'sqs_config',
    value: request.sqsConfig !== undefined ? marshalCreateTriggerRequestSqsClientConfig(request.sqsConfig) : undefined
  }])
});
const marshalUpdateCronRequest = (request, defaults) => ({
  args: request.args,
  function_id: request.functionId,
  name: request.name,
  schedule: request.schedule
});
const marshalUpdateFunctionRequest = (request, defaults) => ({
  description: request.description,
  environment_variables: request.environmentVariables,
  handler: request.handler,
  http_option: request.httpOption,
  max_scale: request.maxScale,
  memory_limit: request.memoryLimit,
  min_scale: request.minScale,
  privacy: request.privacy,
  redeploy: request.redeploy,
  runtime: request.runtime,
  secret_environment_variables: request.secretEnvironmentVariables !== undefined ? request.secretEnvironmentVariables.map(elt => marshalSecret(elt)) : undefined,
  timeout: request.timeout
});
const marshalUpdateNamespaceRequest$1 = (request, defaults) => ({
  description: request.description,
  environment_variables: request.environmentVariables,
  secret_environment_variables: request.secretEnvironmentVariables !== undefined ? request.secretEnvironmentVariables.map(elt => marshalSecret(elt)) : undefined
});
const marshalUpdateTriggerRequestSqsClientConfig = (request, defaults) => ({
  access_key: request.accessKey,
  secret_key: request.secretKey
});
const marshalUpdateTriggerRequest = (request, defaults) => ({
  description: request.description,
  name: request.name,
  ...resolveOneOf([{
    param: 'sqs_config',
    value: request.sqsConfig !== undefined ? marshalUpdateTriggerRequestSqsClientConfig(request.sqsConfig) : undefined
  }])
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$m = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Serverless Functions API. */
let API$m = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];
  pageOfListNamespaces = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListNamespacesResponse$1);

  /**
   * List all your namespaces. List all existing namespaces in the specified
   * region.
   *
   * @param request - The request {@link ListNamespacesRequest}
   * @returns A Promise of ListNamespacesResponse
   */
  listNamespaces = (request = {}) => enrichForPagination('namespaces', this.pageOfListNamespaces, request);

  /**
   * Get a namespace. Get the namespace associated with the specified ID.
   *
   * @param request - The request {@link GetNamespaceRequest}
   * @returns A Promise of Namespace
   */
  getNamespace = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace$1);

  /**
   * Waits for {@link Namespace} to be in a final state.
   *
   * @param request - The request {@link GetNamespaceRequest}
   * @param options - The waiting options
   * @returns A Promise of Namespace
   */
  waitForNamespace = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!NAMESPACE_TRANSIENT_STATUSES$1.includes(res.status))), this.getNamespace, request, options);

  /**
   * Create a new namespace. Create a new namespace in a specified Organization
   * or Project.
   *
   * @param request - The request {@link CreateNamespaceRequest}
   * @returns A Promise of Namespace
   */
  createNamespace = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateNamespaceRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'POST',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces`
  }, unmarshalNamespace$1);

  /**
   * Update an existing namespace. Update the namespace associated with the
   * specified ID.
   *
   * @param request - The request {@link UpdateNamespaceRequest}
   * @returns A Promise of Namespace
   */
  updateNamespace = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateNamespaceRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'PATCH',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace$1);

  /**
   * Delete an existing namespace. Delete the namespace associated with the
   * specified ID.
   *
   * @param request - The request {@link DeleteNamespaceRequest}
   * @returns A Promise of Namespace
   */
  deleteNamespace = request => this.client.fetch({
    method: 'DELETE',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace$1);
  pageOfListFunctions = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/functions`,
    urlParams: urlParams(['name', request.name], ['namespace_id', request.namespaceId], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListFunctionsResponse);

  /**
   * List all your functions.
   *
   * @param request - The request {@link ListFunctionsRequest}
   * @returns A Promise of ListFunctionsResponse
   */
  listFunctions = request => enrichForPagination('functions', this.pageOfListFunctions, request);

  /**
   * Get a function. Get the function associated with the specified ID.
   *
   * @param request - The request {@link GetFunctionRequest}
   * @returns A Promise of Function
   */
  getFunction = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/functions/${validatePathParam('functionId', request.functionId)}`
  }, unmarshalFunction);

  /**
   * Waits for {@link Function} to be in a final state.
   *
   * @param request - The request {@link GetFunctionRequest}
   * @param options - The waiting options
   * @returns A Promise of Function
   */
  waitForFunction = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!FUNCTION_TRANSIENT_STATUSES.includes(res.status))), this.getFunction, request, options);

  /**
   * Create a new function. Create a new function in the specified region for a
   * specified Organization or Project.
   *
   * @param request - The request {@link CreateFunctionRequest}
   * @returns A Promise of Function
   */
  createFunction = request => this.client.fetch({
    body: JSON.stringify(marshalCreateFunctionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'POST',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/functions`
  }, unmarshalFunction);

  /**
   * Update an existing function. Update the function associated with the
   * specified ID.
   *
   * @param request - The request {@link UpdateFunctionRequest}
   * @returns A Promise of Function
   */
  updateFunction = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateFunctionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'PATCH',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/functions/${validatePathParam('functionId', request.functionId)}`
  }, unmarshalFunction);

  /**
   * Delete a function. Delete the function associated with the specified ID.
   *
   * @param request - The request {@link DeleteFunctionRequest}
   * @returns A Promise of Function
   */
  deleteFunction = request => this.client.fetch({
    method: 'DELETE',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/functions/${validatePathParam('functionId', request.functionId)}`
  }, unmarshalFunction);

  /**
   * Deploy a function. Deploy a function associated with the specified ID.
   *
   * @param request - The request {@link DeployFunctionRequest}
   * @returns A Promise of Function
   */
  deployFunction = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$m,
    method: 'POST',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/functions/${validatePathParam('functionId', request.functionId)}/deploy`
  }, unmarshalFunction);

  /**
   * List function runtimes. List available function runtimes.
   *
   * @param request - The request {@link ListFunctionRuntimesRequest}
   * @returns A Promise of ListFunctionRuntimesResponse
   */
  listFunctionRuntimes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/runtimes`
  }, unmarshalListFunctionRuntimesResponse);

  /**
   * Get an upload URL of a function. Get an upload URL of a function associated
   * with the specified ID.
   *
   * @param request - The request {@link GetFunctionUploadURLRequest}
   * @returns A Promise of UploadURL
   */
  getFunctionUploadURL = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/functions/${validatePathParam('functionId', request.functionId)}/upload-url`,
    urlParams: urlParams(['content_length', request.contentLength])
  }, unmarshalUploadURL);

  /**
   * Get a download URL of a function. Get a download URL for a function
   * associated with the specified ID.
   *
   * @param request - The request {@link GetFunctionDownloadURLRequest}
   * @returns A Promise of DownloadURL
   */
  getFunctionDownloadURL = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/functions/${validatePathParam('functionId', request.functionId)}/download-url`
  }, unmarshalDownloadURL);
  pageOfListCrons = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons`,
    urlParams: urlParams(['function_id', request.functionId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListCronsResponse);

  /**
   * List all crons. List all the cronjobs in a specified region.
   *
   * @param request - The request {@link ListCronsRequest}
   * @returns A Promise of ListCronsResponse
   */
  listCrons = request => enrichForPagination('crons', this.pageOfListCrons, request);

  /**
   * Get a cron. Get the cron associated with the specified ID.
   *
   * @param request - The request {@link GetCronRequest}
   * @returns A Promise of Cron
   */
  getCron = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons/${validatePathParam('cronId', request.cronId)}`
  }, unmarshalCron);

  /**
   * Waits for {@link Cron} to be in a final state.
   *
   * @param request - The request {@link GetCronRequest}
   * @param options - The waiting options
   * @returns A Promise of Cron
   */
  waitForCron = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!CRON_TRANSIENT_STATUSES.includes(res.status))), this.getCron, request, options);

  /**
   * Create a new cron. Create a new cronjob for a function with the specified
   * ID.
   *
   * @param request - The request {@link CreateCronRequest}
   * @returns A Promise of Cron
   */
  createCron = request => this.client.fetch({
    body: JSON.stringify(marshalCreateCronRequest(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'POST',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons`
  }, unmarshalCron);

  /**
   * Update an existing cron. Update the cron associated with the specified ID.
   *
   * @param request - The request {@link UpdateCronRequest}
   * @returns A Promise of Cron
   */
  updateCron = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateCronRequest(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'PATCH',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons/${validatePathParam('cronId', request.cronId)}`
  }, unmarshalCron);

  /**
   * Delete an existing cron. Delete the cron associated with the specified ID.
   *
   * @param request - The request {@link DeleteCronRequest}
   * @returns A Promise of Cron
   */
  deleteCron = request => this.client.fetch({
    method: 'DELETE',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/crons/${validatePathParam('cronId', request.cronId)}`
  }, unmarshalCron);
  pageOfListDomains = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains`,
    urlParams: urlParams(['function_id', request.functionId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDomainsResponse$1);

  /**
   * List all domain name bindings. List all domain name bindings in a specified
   * region.
   *
   * @param request - The request {@link ListDomainsRequest}
   * @returns A Promise of ListDomainsResponse
   */
  listDomains = request => enrichForPagination('domains', this.pageOfListDomains, request);

  /**
   * Get a domain name binding. Get a domain name binding for the function with
   * the specified ID.
   *
   * @param request - The request {@link GetDomainRequest}
   * @returns A Promise of Domain
   */
  getDomain = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domainId', request.domainId)}`
  }, unmarshalDomain$1);

  /**
   * Waits for {@link Domain} to be in a final state.
   *
   * @param request - The request {@link GetDomainRequest}
   * @param options - The waiting options
   * @returns A Promise of Domain
   */
  waitForDomain = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!DOMAIN_TRANSIENT_STATUSES$1.includes(res.status))), this.getDomain, request, options);

  /**
   * Create a domain name binding. Create a domain name binding for the function
   * with the specified ID.
   *
   * @param request - The request {@link CreateDomainRequest}
   * @returns A Promise of Domain
   */
  createDomain = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDomainRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'POST',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains`
  }, unmarshalDomain$1);

  /**
   * Delete a domain name binding. Delete a domain name binding for the function
   * with the specified ID.
   *
   * @param request - The request {@link DeleteDomainRequest}
   * @returns A Promise of Domain
   */
  deleteDomain = request => this.client.fetch({
    method: 'DELETE',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domainId', request.domainId)}`
  }, unmarshalDomain$1);

  /**
   * Create a JWT token. Deprecated in favor of CreateToken.
   *
   * @deprecated
   * @param request - The request {@link IssueJWTRequest}
   * @returns A Promise of Token
   */
  issueJWT = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/issue-jwt`,
    urlParams: urlParams(['expires_at', request.expiresAt], ...Object.entries(resolveOneOf([{
      param: 'function_id',
      value: request.functionId
    }, {
      param: 'namespace_id',
      value: request.namespaceId
    }])))
  }, unmarshalToken);

  /**
   * Create a new revocable token.
   *
   * @param request - The request {@link CreateTokenRequest}
   * @returns A Promise of Token
   */
  createToken = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateTokenRequest(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'POST',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens`
  }, unmarshalToken);

  /**
   * Get a token.
   *
   * @param request - The request {@link GetTokenRequest}
   * @returns A Promise of Token
   */
  getToken = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens/${validatePathParam('tokenId', request.tokenId)}`
  }, unmarshalToken);

  /**
   * Waits for {@link Token} to be in a final state.
   *
   * @param request - The request {@link GetTokenRequest}
   * @param options - The waiting options
   * @returns A Promise of Token
   */
  waitForToken = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!TOKEN_TRANSIENT_STATUSES.includes(res.status))), this.getToken, request, options);
  pageOfListTokens = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens`,
    urlParams: urlParams(['function_id', request.functionId], ['namespace_id', request.namespaceId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListTokensResponse);

  /**
   * List all tokens.
   *
   * @param request - The request {@link ListTokensRequest}
   * @returns A Promise of ListTokensResponse
   */
  listTokens = (request = {}) => enrichForPagination('tokens', this.pageOfListTokens, request);

  /**
   * Delete a token.
   *
   * @param request - The request {@link DeleteTokenRequest}
   * @returns A Promise of Token
   */
  deleteToken = request => this.client.fetch({
    method: 'DELETE',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tokens/${validatePathParam('tokenId', request.tokenId)}`
  }, unmarshalToken);

  /**
   * Create a trigger. Create a new trigger for a specified function.
   *
   * @param request - The request {@link CreateTriggerRequest}
   * @returns A Promise of Trigger
   */
  createTrigger = request => this.client.fetch({
    body: JSON.stringify(marshalCreateTriggerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'POST',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers`
  }, unmarshalTrigger);

  /**
   * Get a trigger. Get a trigger with a specified ID.
   *
   * @param request - The request {@link GetTriggerRequest}
   * @returns A Promise of Trigger
   */
  getTrigger = request => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers/${validatePathParam('triggerId', request.triggerId)}`
  }, unmarshalTrigger);

  /**
   * Waits for {@link Trigger} to be in a final state.
   *
   * @param request - The request {@link GetTriggerRequest}
   * @param options - The waiting options
   * @returns A Promise of Trigger
   */
  waitForTrigger = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!TRIGGER_TRANSIENT_STATUSES.includes(res.status))), this.getTrigger, request, options);
  pageOfListTriggers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ...Object.entries(resolveOneOf([{
      param: 'function_id',
      value: request.functionId
    }, {
      param: 'namespace_id',
      value: request.namespaceId
    }, {
      default: this.client.settings.defaultProjectId,
      param: 'project_id',
      value: request.projectId
    }])))
  }, unmarshalListTriggersResponse);

  /**
   * List all triggers. List all triggers belonging to a specified Organization
   * or Project.
   *
   * @param request - The request {@link ListTriggersRequest}
   * @returns A Promise of ListTriggersResponse
   */
  listTriggers = (request = {}) => enrichForPagination('triggers', this.pageOfListTriggers, request);

  /**
   * Update a trigger. Update a trigger with a specified ID.
   *
   * @param request - The request {@link UpdateTriggerRequest}
   * @returns A Promise of Trigger
   */
  updateTrigger = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateTriggerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$m,
    method: 'PATCH',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers/${validatePathParam('triggerId', request.triggerId)}`
  }, unmarshalTrigger);

  /**
   * Delete a trigger. Delete a trigger with a specified ID.
   *
   * @param request - The request {@link DeleteTriggerRequest}
   * @returns A Promise of Trigger
   */
  deleteTrigger = request => this.client.fetch({
    method: 'DELETE',
    path: `/functions/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/triggers/${validatePathParam('triggerId', request.triggerId)}`
  }, unmarshalTrigger);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateTriggerRequest = {
  description: {
    maxLength: 255
  },
  name: {
    maxLength: 50,
    minLength: 1
  }
};
const CreateTriggerRequestMnqNatsClientConfig = {
  mnqRegion: {
    maxLength: 20,
    minLength: 1
  },
  subject: {
    maxLength: 100,
    minLength: 1,
    pattern: /^[^\s]+$/
  }
};
const CreateTriggerRequestMnqSqsClientConfig = {
  mnqRegion: {
    maxLength: 20,
    minLength: 1
  },
  queue: {
    maxLength: 100,
    minLength: 1
  }
};
const UpdateTriggerRequest = {
  description: {
    maxLength: 255
  },
  name: {
    maxLength: 50,
    minLength: 1
  }
};

var validationRules_gen$a = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateTriggerRequest: CreateTriggerRequest,
  CreateTriggerRequestMnqNatsClientConfig: CreateTriggerRequestMnqNatsClientConfig,
  CreateTriggerRequestMnqSqsClientConfig: CreateTriggerRequestMnqSqsClientConfig,
  UpdateTriggerRequest: UpdateTriggerRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$l = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$m,
  CRON_TRANSIENT_STATUSES: CRON_TRANSIENT_STATUSES,
  DOMAIN_TRANSIENT_STATUSES: DOMAIN_TRANSIENT_STATUSES$1,
  FUNCTION_TRANSIENT_STATUSES: FUNCTION_TRANSIENT_STATUSES,
  NAMESPACE_TRANSIENT_STATUSES: NAMESPACE_TRANSIENT_STATUSES$1,
  TOKEN_TRANSIENT_STATUSES: TOKEN_TRANSIENT_STATUSES,
  TRIGGER_TRANSIENT_STATUSES: TRIGGER_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen$a
});

var index$o = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1beta1: index_gen$l
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalJWT = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'JWT' failed as data isn't a dictionary.`);
  }
  return {
    audienceId: data.audience_id,
    createdAt: unmarshalDate(data.created_at),
    expiresAt: unmarshalDate(data.expires_at),
    ip: data.ip,
    issuerId: data.issuer_id,
    jti: data.jti,
    updatedAt: unmarshalDate(data.updated_at),
    userAgent: data.user_agent
  };
};
const unmarshalAPIKey = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'APIKey' failed as data isn't a dictionary.`);
  }
  return {
    accessKey: data.access_key,
    applicationId: data.application_id,
    createdAt: unmarshalDate(data.created_at),
    creationIp: data.creation_ip,
    defaultProjectId: data.default_project_id,
    description: data.description,
    editable: data.editable,
    expiresAt: unmarshalDate(data.expires_at),
    secretKey: data.secret_key,
    updatedAt: unmarshalDate(data.updated_at),
    userId: data.user_id
  };
};
const unmarshalApplication = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Application' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    editable: data.editable,
    id: data.id,
    name: data.name,
    nbApiKeys: data.nb_api_keys,
    organizationId: data.organization_id,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalGroup = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Group' failed as data isn't a dictionary.`);
  }
  return {
    applicationIds: data.application_ids,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    userIds: data.user_ids
  };
};
const unmarshalLog = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Log' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    bearerId: data.bearer_id,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    ip: data.ip,
    organizationId: data.organization_id,
    resourceId: data.resource_id,
    resourceType: data.resource_type,
    userAgent: data.user_agent
  };
};
const unmarshalPolicy = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Policy' failed as data isn't a dictionary.`);
  }
  return {
    applicationId: data.application_id,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    editable: data.editable,
    groupId: data.group_id,
    id: data.id,
    name: data.name,
    nbPermissionSets: data.nb_permission_sets,
    nbRules: data.nb_rules,
    nbScopes: data.nb_scopes,
    noPrincipal: data.no_principal,
    organizationId: data.organization_id,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    userId: data.user_id
  };
};
const unmarshalQuotum = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Quotum' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    limit: data.limit,
    name: data.name,
    prettyName: data.pretty_name,
    unit: data.unit,
    unlimited: data.unlimited
  };
};
const unmarshalSSHKey = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SSHKey' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    disabled: data.disabled,
    fingerprint: data.fingerprint,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    publicKey: data.public_key,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalUser$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'User' failed as data isn't a dictionary.`);
  }
  return {
    accountRootUserId: data.account_root_user_id,
    createdAt: unmarshalDate(data.created_at),
    deletable: data.deletable,
    email: data.email,
    id: data.id,
    lastLoginAt: unmarshalDate(data.last_login_at),
    mfa: data.mfa,
    organizationId: data.organization_id,
    status: data.status,
    tags: data.tags,
    twoFactorEnabled: data.two_factor_enabled,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalEncodedJWT = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EncodedJWT' failed as data isn't a dictionary.`);
  }
  return {
    jwt: data.jwt ? unmarshalJWT(data.jwt) : undefined,
    renewToken: data.renew_token,
    token: data.token
  };
};
const unmarshalListAPIKeysResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListAPIKeysResponse' failed as data isn't a dictionary.`);
  }
  return {
    apiKeys: unmarshalArrayOfObject(data.api_keys, unmarshalAPIKey),
    totalCount: data.total_count
  };
};
const unmarshalListApplicationsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListApplicationsResponse' failed as data isn't a dictionary.`);
  }
  return {
    applications: unmarshalArrayOfObject(data.applications, unmarshalApplication),
    totalCount: data.total_count
  };
};
const unmarshalListGroupsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListGroupsResponse' failed as data isn't a dictionary.`);
  }
  return {
    groups: unmarshalArrayOfObject(data.groups, unmarshalGroup),
    totalCount: data.total_count
  };
};
const unmarshalListJWTsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListJWTsResponse' failed as data isn't a dictionary.`);
  }
  return {
    jwts: unmarshalArrayOfObject(data.jwts, unmarshalJWT),
    totalCount: data.total_count
  };
};
const unmarshalListLogsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListLogsResponse' failed as data isn't a dictionary.`);
  }
  return {
    logs: unmarshalArrayOfObject(data.logs, unmarshalLog),
    totalCount: data.total_count
  };
};
const unmarshalPermissionSet = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PermissionSet' failed as data isn't a dictionary.`);
  }
  return {
    categories: data.categories,
    description: data.description,
    id: data.id,
    name: data.name,
    scopeType: data.scope_type
  };
};
const unmarshalListPermissionSetsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPermissionSetsResponse' failed as data isn't a dictionary.`);
  }
  return {
    permissionSets: unmarshalArrayOfObject(data.permission_sets, unmarshalPermissionSet),
    totalCount: data.total_count
  };
};
const unmarshalListPoliciesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPoliciesResponse' failed as data isn't a dictionary.`);
  }
  return {
    policies: unmarshalArrayOfObject(data.policies, unmarshalPolicy),
    totalCount: data.total_count
  };
};
const unmarshalListQuotaResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListQuotaResponse' failed as data isn't a dictionary.`);
  }
  return {
    quota: unmarshalArrayOfObject(data.quota, unmarshalQuotum),
    totalCount: data.total_count
  };
};
const unmarshalRule = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Rule' failed as data isn't a dictionary.`);
  }
  return {
    accountRootUserId: data.account_root_user_id,
    id: data.id,
    organizationId: data.organization_id,
    permissionSetNames: data.permission_set_names,
    permissionSetsScopeType: data.permission_sets_scope_type,
    projectIds: data.project_ids
  };
};
const unmarshalListRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalRule),
    totalCount: data.total_count
  };
};
const unmarshalListSSHKeysResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSSHKeysResponse' failed as data isn't a dictionary.`);
  }
  return {
    sshKeys: unmarshalArrayOfObject(data.ssh_keys, unmarshalSSHKey),
    totalCount: data.total_count
  };
};
const unmarshalListUsersResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListUsersResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    users: unmarshalArrayOfObject(data.users, unmarshalUser$1)
  };
};
const unmarshalSetRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalRule)
  };
};
const marshalAddGroupMemberRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'user_id',
    value: request.userId
  }, {
    param: 'application_id',
    value: request.applicationId
  }])
});
const marshalAddGroupMembersRequest = (request, defaults) => ({
  application_ids: request.applicationIds,
  user_ids: request.userIds
});
const marshalCreateAPIKeyRequest = (request, defaults) => ({
  default_project_id: request.defaultProjectId,
  description: request.description,
  expires_at: request.expiresAt,
  ...resolveOneOf([{
    param: 'application_id',
    value: request.applicationId
  }, {
    param: 'user_id',
    value: request.userId
  }])
});
const marshalCreateApplicationRequest = (request, defaults) => ({
  description: request.description,
  name: request.name || randomName('app'),
  organization_id: request.organizationId ?? defaults.defaultOrganizationId,
  tags: request.tags
});
const marshalCreateGroupRequest = (request, defaults) => ({
  description: request.description,
  name: request.name || randomName('grp'),
  organization_id: request.organizationId ?? defaults.defaultOrganizationId,
  tags: request.tags
});
const marshalCreateJWTRequest = (request, defaults) => ({
  referrer: request.referrer,
  user_id: request.userId
});
const marshalRuleSpecs = (request, defaults) => ({
  permission_set_names: request.permissionSetNames,
  ...resolveOneOf([{
    param: 'project_ids',
    value: request.projectIds
  }, {
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalCreatePolicyRequest = (request, defaults) => ({
  description: request.description,
  name: request.name || randomName('pol'),
  organization_id: request.organizationId ?? defaults.defaultOrganizationId,
  rules: request.rules !== undefined ? request.rules.map(elt => marshalRuleSpecs(elt)) : undefined,
  tags: request.tags,
  ...resolveOneOf([{
    param: 'user_id',
    value: request.userId
  }, {
    param: 'group_id',
    value: request.groupId
  }, {
    param: 'application_id',
    value: request.applicationId
  }, {
    param: 'no_principal',
    value: request.noPrincipal
  }])
});
const marshalCreateSSHKeyRequest = (request, defaults) => ({
  name: request.name || randomName('key'),
  project_id: request.projectId ?? defaults.defaultProjectId,
  public_key: request.publicKey
});
const marshalCreateUserRequest$1 = (request, defaults) => ({
  email: request.email,
  organization_id: request.organizationId ?? defaults.defaultOrganizationId,
  tags: request.tags
});
const marshalRemoveGroupMemberRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'user_id',
    value: request.userId
  }, {
    param: 'application_id',
    value: request.applicationId
  }])
});
const marshalSetGroupMembersRequest = (request, defaults) => ({
  application_ids: request.applicationIds,
  user_ids: request.userIds
});
const marshalSetRulesRequest = (request, defaults) => ({
  policy_id: request.policyId,
  rules: request.rules.map(elt => marshalRuleSpecs(elt))
});
const marshalUpdateAPIKeyRequest = (request, defaults) => ({
  default_project_id: request.defaultProjectId,
  description: request.description
});
const marshalUpdateApplicationRequest = (request, defaults) => ({
  description: request.description,
  name: request.name,
  tags: request.tags
});
const marshalUpdateGroupRequest = (request, defaults) => ({
  description: request.description,
  name: request.name,
  tags: request.tags
});
const marshalUpdatePolicyRequest = (request, defaults) => ({
  description: request.description,
  name: request.name,
  tags: request.tags,
  ...resolveOneOf([{
    param: 'user_id',
    value: request.userId
  }, {
    param: 'group_id',
    value: request.groupId
  }, {
    param: 'application_id',
    value: request.applicationId
  }, {
    param: 'no_principal',
    value: request.noPrincipal
  }])
});
const marshalUpdateSSHKeyRequest = (request, defaults) => ({
  disabled: request.disabled,
  name: request.name
});
const marshalUpdateUserRequest$1 = (request, defaults) => ({
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$l = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** IAM API. */
let API$l = class API extends API$z {
  pageOfListSSHKeys = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/ssh-keys`,
    urlParams: urlParams(['disabled', request.disabled], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListSSHKeysResponse);

  /**
   * List SSH keys. List SSH keys. By default, the SSH keys listed are ordered
   * by creation date in ascending order. This can be modified via the
   * `order_by` field. You can define additional parameters for your query such
   * as `organization_id`, `name`, `project_id` and `disabled`.
   *
   * @param request - The request {@link ListSSHKeysRequest}
   * @returns A Promise of ListSSHKeysResponse
   */
  listSSHKeys = (request = {}) => enrichForPagination('sshKeys', this.pageOfListSSHKeys, request);

  /**
   * Create an SSH key. Add a new SSH key to a Scaleway Project. You must
   * specify the `name`, `public_key` and `project_id`.
   *
   * @param request - The request {@link CreateSSHKeyRequest}
   * @returns A Promise of SSHKey
   */
  createSSHKey = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSSHKeyRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/ssh-keys`
  }, unmarshalSSHKey);

  /**
   * Get an SSH key. Retrieve information about a given SSH key, specified by
   * the `ssh_key_id` parameter. The SSH key's full details, including `id`,
   * `name`, `public_key`, and `project_id` are returned in the response.
   *
   * @param request - The request {@link GetSSHKeyRequest}
   * @returns A Promise of SSHKey
   */
  getSSHKey = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/ssh-keys/${validatePathParam('sshKeyId', request.sshKeyId)}`
  }, unmarshalSSHKey);

  /**
   * Update an SSH key. Update the parameters of an SSH key, including `name`
   * and `disable`.
   *
   * @param request - The request {@link UpdateSSHKeyRequest}
   * @returns A Promise of SSHKey
   */
  updateSSHKey = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSSHKeyRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'PATCH',
    path: `/iam/v1alpha1/ssh-keys/${validatePathParam('sshKeyId', request.sshKeyId)}`
  }, unmarshalSSHKey);

  /**
   * Delete an SSH key. Delete a given SSH key, specified by the `ssh_key_id`.
   * Deleting an SSH is permanent, and cannot be undone. Note that you might
   * need to update any configurations that used the SSH key.
   *
   * @param request - The request {@link DeleteSSHKeyRequest}
   */
  deleteSSHKey = request => this.client.fetch({
    method: 'DELETE',
    path: `/iam/v1alpha1/ssh-keys/${validatePathParam('sshKeyId', request.sshKeyId)}`
  });
  pageOfListUsers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/users`,
    urlParams: urlParams(['mfa', request.mfa], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['tag', request.tag], ['user_ids', request.userIds])
  }, unmarshalListUsersResponse$1);

  /**
   * List users of an Organization. List the users of an Organization. By
   * default, the users listed are ordered by creation date in ascending order.
   * This can be modified via the `order_by` field. You must define the
   * `organization_id` in the query path of your request. You can also define
   * additional parameters for your query such as `user_ids`.
   *
   * @param request - The request {@link ListUsersRequest}
   * @returns A Promise of ListUsersResponse
   */
  listUsers = (request = {}) => enrichForPagination('users', this.pageOfListUsers, request);

  /**
   * Get a given user. Retrieve information about a user, specified by the
   * `user_id` parameter. The user's full details, including `id`, `email`,
   * `organization_id`, `status` and `mfa` are returned in the response.
   *
   * @param request - The request {@link GetUserRequest}
   * @returns A Promise of User
   */
  getUser = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/users/${validatePathParam('userId', request.userId)}`
  }, unmarshalUser$1);

  /**
   * Update a user. Update the parameters of a user, including `tags`.
   *
   * @param request - The request {@link UpdateUserRequest}
   * @returns A Promise of User
   */
  updateUser = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateUserRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'PATCH',
    path: `/iam/v1alpha1/users/${validatePathParam('userId', request.userId)}`
  }, unmarshalUser$1);

  /**
   * Delete a guest user from an Organization. Remove a user from an
   * Organization in which they are a guest. You must define the `user_id` in
   * your request. Note that removing a user from an Organization automatically
   * deletes their API keys, and any policies directly attached to them become
   * orphaned.
   *
   * @param request - The request {@link DeleteUserRequest}
   */
  deleteUser = request => this.client.fetch({
    method: 'DELETE',
    path: `/iam/v1alpha1/users/${validatePathParam('userId', request.userId)}`
  });

  /**
   * Create a new user. Create a new user. You must define the `organization_id`
   * and the `email` in your request.
   *
   * @param request - The request {@link CreateUserRequest}
   * @returns A Promise of User
   */
  createUser = request => this.client.fetch({
    body: JSON.stringify(marshalCreateUserRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/users`
  }, unmarshalUser$1);
  pageOfListApplications = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/applications`,
    urlParams: urlParams(['application_ids', request.applicationIds], ['editable', request.editable], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['tag', request.tag])
  }, unmarshalListApplicationsResponse);

  /**
   * List applications of an Organization. List the applications of an
   * Organization. By default, the applications listed are ordered by creation
   * date in ascending order. This can be modified via the `order_by` field. You
   * must define the `organization_id` in the query path of your request. You
   * can also define additional parameters for your query such as
   * `application_ids`.
   *
   * @param request - The request {@link ListApplicationsRequest}
   * @returns A Promise of ListApplicationsResponse
   */
  listApplications = (request = {}) => enrichForPagination('applications', this.pageOfListApplications, request);

  /**
   * Create a new application. Create a new application. You must define the
   * `name` parameter in the request.
   *
   * @param request - The request {@link CreateApplicationRequest}
   * @returns A Promise of Application
   */
  createApplication = request => this.client.fetch({
    body: JSON.stringify(marshalCreateApplicationRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/applications`
  }, unmarshalApplication);

  /**
   * Get a given application. Retrieve information about an application,
   * specified by the `application_id` parameter. The application's full
   * details, including `id`, `email`, `organization_id`, `status` and
   * `two_factor_enabled` are returned in the response.
   *
   * @param request - The request {@link GetApplicationRequest}
   * @returns A Promise of Application
   */
  getApplication = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/applications/${validatePathParam('applicationId', request.applicationId)}`
  }, unmarshalApplication);

  /**
   * Update an application. Update the parameters of an application, including
   * `name` and `description`.
   *
   * @param request - The request {@link UpdateApplicationRequest}
   * @returns A Promise of Application
   */
  updateApplication = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateApplicationRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'PATCH',
    path: `/iam/v1alpha1/applications/${validatePathParam('applicationId', request.applicationId)}`
  }, unmarshalApplication);

  /**
   * Delete an application. Delete an application. Note that this action is
   * irreversible and will automatically delete the application's API keys.
   * Policies attached to users and applications via this group will no longer
   * apply.
   *
   * @param request - The request {@link DeleteApplicationRequest}
   */
  deleteApplication = request => this.client.fetch({
    method: 'DELETE',
    path: `/iam/v1alpha1/applications/${validatePathParam('applicationId', request.applicationId)}`
  });
  pageOfListGroups = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/groups`,
    urlParams: urlParams(['application_ids', request.applicationIds], ['group_ids', request.groupIds], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['tag', request.tag], ['user_ids', request.userIds])
  }, unmarshalListGroupsResponse);

  /**
   * List groups. List groups. By default, the groups listed are ordered by
   * creation date in ascending order. This can be modified via the `order_by`
   * field. You can define additional parameters to filter your query. Use
   * `user_ids` or `application_ids` to list all groups certain users or
   * applications belong to.
   *
   * @param request - The request {@link ListGroupsRequest}
   * @returns A Promise of ListGroupsResponse
   */
  listGroups = (request = {}) => enrichForPagination('groups', this.pageOfListGroups, request);

  /**
   * Create a group. Create a new group. You must define the `name` and
   * `organization_id` parameters in the request.
   *
   * @param request - The request {@link CreateGroupRequest}
   * @returns A Promise of Group
   */
  createGroup = request => this.client.fetch({
    body: JSON.stringify(marshalCreateGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/groups`
  }, unmarshalGroup);

  /**
   * Get a group. Retrive information about a given group, specified by the
   * `group_id` parameter. The group's full details, including `user_ids` and
   * `application_ids` are returned in the response.
   *
   * @param request - The request {@link GetGroupRequest}
   * @returns A Promise of Group
   */
  getGroup = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/groups/${validatePathParam('groupId', request.groupId)}`
  }, unmarshalGroup);

  /**
   * Update a group. Update the parameters of group, including `name` and
   * `description`.
   *
   * @param request - The request {@link UpdateGroupRequest}
   * @returns A Promise of Group
   */
  updateGroup = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'PATCH',
    path: `/iam/v1alpha1/groups/${validatePathParam('groupId', request.groupId)}`
  }, unmarshalGroup);

  /**
   * Overwrite users and applications of a group. Overwrite users and
   * applications configuration in a group. Any information that you add using
   * this command will overwrite the previous configuration.
   *
   * @param request - The request {@link SetGroupMembersRequest}
   * @returns A Promise of Group
   */
  setGroupMembers = request => this.client.fetch({
    body: JSON.stringify(marshalSetGroupMembersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'PUT',
    path: `/iam/v1alpha1/groups/${validatePathParam('groupId', request.groupId)}/members`
  }, unmarshalGroup);

  /**
   * Add a user or an application to a group. Add a user or an application to a
   * group. You can specify a `user_id` and and `application_id` in the body of
   * your request. Note that you can only add one of each per request.
   *
   * @param request - The request {@link AddGroupMemberRequest}
   * @returns A Promise of Group
   */
  addGroupMember = request => this.client.fetch({
    body: JSON.stringify(marshalAddGroupMemberRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/groups/${validatePathParam('groupId', request.groupId)}/add-member`
  }, unmarshalGroup);

  /**
   * Add multiple users and applications to a group. Add multiple users and
   * applications to a group in a single call. You can specify an array of
   * `user_id`s and `application_id`s. Note that any existing users and
   * applications in the group will remain. To add new users/applications and
   * delete pre-existing ones, use the [Overwrite users and applications of a
   * group](#path-groups-overwrite-users-and-applications-of-a-group) method.
   *
   * @param request - The request {@link AddGroupMembersRequest}
   * @returns A Promise of Group
   */
  addGroupMembers = request => this.client.fetch({
    body: JSON.stringify(marshalAddGroupMembersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/groups/${validatePathParam('groupId', request.groupId)}/add-members`
  }, unmarshalGroup);

  /**
   * Remove a user or an application from a group. Remove a user or an
   * application from a group. You can specify a `user_id` and and
   * `application_id` in the body of your request. Note that you can only remove
   * one of each per request. Removing a user from a group means that any
   * permissions given to them via the group (i.e. from an attached policy) will
   * no longer apply. Be sure you want to remove these permissions from the user
   * before proceeding.
   *
   * @param request - The request {@link RemoveGroupMemberRequest}
   * @returns A Promise of Group
   */
  removeGroupMember = request => this.client.fetch({
    body: JSON.stringify(marshalRemoveGroupMemberRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/groups/${validatePathParam('groupId', request.groupId)}/remove-member`
  }, unmarshalGroup);

  /**
   * Delete a group. Delete a group. Note that this action is irreversible and
   * could delete permissions for group members. Policies attached to users and
   * applications via this group will no longer apply.
   *
   * @param request - The request {@link DeleteGroupRequest}
   */
  deleteGroup = request => this.client.fetch({
    method: 'DELETE',
    path: `/iam/v1alpha1/groups/${validatePathParam('groupId', request.groupId)}`
  });
  pageOfListPolicies = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/policies`,
    urlParams: urlParams(['application_ids', request.applicationIds], ['editable', request.editable], ['group_ids', request.groupIds], ['no_principal', request.noPrincipal], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['policy_ids', request.policyIds], ['policy_name', request.policyName], ['tag', request.tag], ['user_ids', request.userIds])
  }, unmarshalListPoliciesResponse);

  /**
   * List policies of an Organization. List the policies of an Organization. By
   * default, the policies listed are ordered by creation date in ascending
   * order. This can be modified via the `order_by` field. You must define the
   * `organization_id` in the query path of your request. You can also define
   * additional parameters to filter your query, such as `user_ids`,
   * `groups_ids`, `application_ids`, and `policy_name`.
   *
   * @param request - The request {@link ListPoliciesRequest}
   * @returns A Promise of ListPoliciesResponse
   */
  listPolicies = (request = {}) => enrichForPagination('policies', this.pageOfListPolicies, request);

  /**
   * Create a new policy. Create a new application. You must define the `name`
   * parameter in the request. You can specify parameters such as `user_id`,
   * `groups_id`, `application_id`, `no_principal`, `rules` and its child
   * attributes.
   *
   * @param request - The request {@link CreatePolicyRequest}
   * @returns A Promise of Policy
   */
  createPolicy = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePolicyRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/policies`
  }, unmarshalPolicy);

  /**
   * Get an existing policy. Retrieve information about a policy, speficified by
   * the `policy_id` parameter. The policy's full details, including `id`,
   * `name`, `organization_id`, `nb_rules` and `nb_scopes`, `nb_permission_sets`
   * are returned in the response.
   *
   * @param request - The request {@link GetPolicyRequest}
   * @returns A Promise of Policy
   */
  getPolicy = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/policies/${validatePathParam('policyId', request.policyId)}`
  }, unmarshalPolicy);

  /**
   * Update an existing policy. Update the parameters of a policy, including
   * `name`, `description`, `user_id`, `group_id`, `application_id` and
   * `no_principal`.
   *
   * @param request - The request {@link UpdatePolicyRequest}
   * @returns A Promise of Policy
   */
  updatePolicy = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePolicyRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'PATCH',
    path: `/iam/v1alpha1/policies/${validatePathParam('policyId', request.policyId)}`
  }, unmarshalPolicy);

  /**
   * Delete a policy. Delete a policy. You must define specify the `policy_id`
   * parameter in your request. Note that when deleting a policy, all
   * permissions it gives to its principal (user, group or application) will be
   * revoked.
   *
   * @param request - The request {@link DeletePolicyRequest}
   */
  deletePolicy = request => this.client.fetch({
    method: 'DELETE',
    path: `/iam/v1alpha1/policies/${validatePathParam('policyId', request.policyId)}`
  });

  /**
   * Clone a policy. Clone a policy. You must define specify the `policy_id`
   * parameter in your request.
   *
   * @param request - The request {@link ClonePolicyRequest}
   * @returns A Promise of Policy
   */
  clonePolicy = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/policies/${validatePathParam('policyId', request.policyId)}/clone`
  }, unmarshalPolicy);

  /**
   * Set rules of a given policy. Overwrite the rules of a given policy. Any
   * information that you add using this command will overwrite the previous
   * configuration. If you include some of the rules you already had in your
   * previous configuration in your new one, but you change their order, the new
   * order of display will apply. While policy rules are ordered, they have no
   * impact on the access logic of IAM because rules are allow-only.
   *
   * @param request - The request {@link SetRulesRequest}
   * @returns A Promise of SetRulesResponse
   */
  setRules = request => this.client.fetch({
    body: JSON.stringify(marshalSetRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'PUT',
    path: `/iam/v1alpha1/rules`
  }, unmarshalSetRulesResponse);
  pageOfListRules = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/rules`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['policy_id', request.policyId])
  }, unmarshalListRulesResponse);

  /**
   * List rules of a given policy. List the rules of a given policy. By default,
   * the rules listed are ordered by creation date in ascending order. This can
   * be modified via the `order_by` field. You must define the `policy_id` in
   * the query path of your request.
   *
   * @param request - The request {@link ListRulesRequest}
   * @returns A Promise of ListRulesResponse
   */
  listRules = request => enrichForPagination('rules', this.pageOfListRules, request);
  pageOfListPermissionSets = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/permission-sets`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListPermissionSetsResponse);

  /**
   * List permission sets. List permission sets available for given
   * Organization. You must define the `organization_id` in the query path of
   * your request.
   *
   * @param request - The request {@link ListPermissionSetsRequest}
   * @returns A Promise of ListPermissionSetsResponse
   */
  listPermissionSets = (request = {}) => enrichForPagination('permissionSets', this.pageOfListPermissionSets, request);
  pageOfListAPIKeys = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/api-keys`,
    urlParams: urlParams(['access_key', request.accessKey], ['access_keys', request.accessKeys], ['bearer_id', request.bearerId], ['bearer_type', request.bearerType], ['description', request.description], ['editable', request.editable], ['expired', request.expired], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ...Object.entries(resolveOneOf([{
      param: 'application_id',
      value: request.applicationId
    }, {
      param: 'user_id',
      value: request.userId
    }])))
  }, unmarshalListAPIKeysResponse);

  /**
   * List API keys. List API keys. By default, the API keys listed are ordered
   * by creation date in ascending order. This can be modified via the
   * `order_by` field. You can define additional parameters for your query such
   * as `editable`, `expired`, `access_key` and `bearer_id`.
   *
   * @param request - The request {@link ListAPIKeysRequest}
   * @returns A Promise of ListAPIKeysResponse
   */
  listAPIKeys = (request = {}) => enrichForPagination('apiKeys', this.pageOfListAPIKeys, request);

  /**
   * Create an API key. Create an API key. You must specify the `application_id`
   * or the `user_id` and the description. You can also specify the
   * `default_project_id` which is the Project ID of your preferred Project, to
   * use with Object Storage. The `access_key` and `secret_key` values are
   * returned in the response. Note that he secret key is only showed once. Make
   * sure that you copy and store both keys somewhere safe.
   *
   * @param request - The request {@link CreateAPIKeyRequest}
   * @returns A Promise of APIKey
   */
  createAPIKey = request => this.client.fetch({
    body: JSON.stringify(marshalCreateAPIKeyRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/api-keys`
  }, unmarshalAPIKey);

  /**
   * Get an API key. Retrive information about an API key, specified by the
   * `access_key` parameter. The API key's details, including either the
   * `user_id` or `application_id` of its bearer are returned in the response.
   * Note that the string value for the `secret_key` is nullable, and therefore
   * is not displayed in the response. The `secret_key` value is only displayed
   * upon API key creation.
   *
   * @param request - The request {@link GetAPIKeyRequest}
   * @returns A Promise of APIKey
   */
  getAPIKey = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/api-keys/${validatePathParam('accessKey', request.accessKey)}`
  }, unmarshalAPIKey);

  /**
   * Update an API key. Update the parameters of an API key, including
   * `default_project_id` and `description`.
   *
   * @param request - The request {@link UpdateAPIKeyRequest}
   * @returns A Promise of APIKey
   */
  updateAPIKey = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateAPIKeyRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'PATCH',
    path: `/iam/v1alpha1/api-keys/${validatePathParam('accessKey', request.accessKey)}`
  }, unmarshalAPIKey);

  /**
   * Delete an API key. Delete an API key. Note that this action is irreversible
   * and cannot be undone. Make sure you update any configurations using the API
   * keys you delete.
   *
   * @param request - The request {@link DeleteAPIKeyRequest}
   */
  deleteAPIKey = request => this.client.fetch({
    method: 'DELETE',
    path: `/iam/v1alpha1/api-keys/${validatePathParam('accessKey', request.accessKey)}`
  });
  pageOfListQuota = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/quota`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['quotum_names', request.quotumNames])
  }, unmarshalListQuotaResponse);

  /**
   * List all quotas in the Organization. List all product and features quota
   * for an Organization, with their associated limits. By default, the quota
   * listed are ordered by creation date in ascending order. This can be
   * modified via the `order_by` field. You must define the `organization_id` in
   * the query path of your request.
   *
   * @param request - The request {@link ListQuotaRequest}
   * @returns A Promise of ListQuotaResponse
   */
  listQuota = (request = {}) => enrichForPagination('quota', this.pageOfListQuota, request);

  /**
   * Get a quota in the Organization. Retrieve information about a resource
   * quota, specified by the `quotum_name` parameter. The quota's `limit`, or
   * whether it is unlimited, is returned in the response.
   *
   * @param request - The request {@link GetQuotumRequest}
   * @returns A Promise of Quotum
   */
  getQuotum = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/quota/${validatePathParam('quotumName', request.quotumName)}`,
    urlParams: urlParams(['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId])
  }, unmarshalQuotum);
  pageOfListJWTs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/jwts`,
    urlParams: urlParams(['audience_id', request.audienceId], ['expired', request.expired], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListJWTsResponse);

  /**
   * List JWTs.
   *
   * @param request - The request {@link ListJWTsRequest}
   * @returns A Promise of ListJWTsResponse
   */
  listJWTs = (request = {}) => enrichForPagination('jwts', this.pageOfListJWTs, request);

  /**
   * Create a JWT.
   *
   * @param request - The request {@link CreateJWTRequest}
   * @returns A Promise of EncodedJWT
   */
  createJWT = request => this.client.fetch({
    body: JSON.stringify(marshalCreateJWTRequest(request, this.client.settings)),
    headers: jsonContentHeaders$l,
    method: 'POST',
    path: `/iam/v1alpha1/jwts`
  }, unmarshalEncodedJWT);

  /**
   * Get a JWT.
   *
   * @param request - The request {@link GetJWTRequest}
   * @returns A Promise of JWT
   */
  getJWT = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/jwts/${validatePathParam('jti', request.jti)}`
  }, unmarshalJWT);

  /**
   * Delete a JWT.
   *
   * @param request - The request {@link DeleteJWTRequest}
   */
  deleteJWT = request => this.client.fetch({
    method: 'DELETE',
    path: `/iam/v1alpha1/jwts/${validatePathParam('jti', request.jti)}`
  });
  pageOfListLogs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/logs`,
    urlParams: urlParams(['action', request.action], ['created_after', request.createdAfter], ['created_before', request.createdBefore], ['order_by', request.orderBy], ['organization_id', request.organizationId ?? this.client.settings.defaultOrganizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['resource_type', request.resourceType], ['search', request.search])
  }, unmarshalListLogsResponse);

  /**
   * List logs. List logs available for given Organization. You must define the
   * `organization_id` in the query path of your request.
   *
   * @param request - The request {@link ListLogsRequest}
   * @returns A Promise of ListLogsResponse
   */
  listLogs = (request = {}) => enrichForPagination('logs', this.pageOfListLogs, request);

  /**
   * Get a log. Retrieve information about a log, specified by the `log_id`
   * parameter. The log's full details, including `id`, `ip`, `user_agent`,
   * `action`, `bearer_id`, `resource_type` and `resource_id` are returned in
   * the response.
   *
   * @param request - The request {@link GetLogRequest}
   * @returns A Promise of Log
   */
  getLog = request => this.client.fetch({
    method: 'GET',
    path: `/iam/v1alpha1/logs/${validatePathParam('logId', request.logId)}`
  }, unmarshalLog);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateAPIKeyRequest = {
  description: {
    ignoreEmpty: true,
    maxLength: 200
  }
};
const CreateApplicationRequest = {
  description: {
    ignoreEmpty: true,
    maxLength: 200
  },
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\(\)\._\- ]+$/
  }
};
const CreateGroupRequest = {
  description: {
    ignoreEmpty: true,
    maxLength: 200
  },
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\(\)\._\- ]+$/
  }
};
const CreatePolicyRequest = {
  description: {
    ignoreEmpty: true,
    maxLength: 200
  },
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\(\)\._\- ]+$/
  }
};
const CreateSSHKeyRequest = {
  name: {
    ignoreEmpty: true,
    maxLength: 1000
  },
  publicKey: {
    maxLength: 65000,
    minLength: 1
  }
};
const GetQuotumRequest = {
  quotumName: {
    minLength: 1
  }
};
const ListAPIKeysRequest = {
  description: {
    maxLength: 200
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListApplicationsRequest = {
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\(\)\._\- ]+$/
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListGroupsRequest = {
  name: {
    minLength: 1
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListJWTsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListLogsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListPermissionSetsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListPoliciesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  },
  policyName: {
    minLength: 1
  }
};
const ListQuotaRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListRulesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListSSHKeysRequest = {
  name: {
    maxLength: 1000,
    minLength: 1
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const ListUsersRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 100
  }
};
const UpdateAPIKeyRequest = {
  description: {
    maxLength: 200
  }
};
const UpdateApplicationRequest = {
  description: {
    maxLength: 200
  },
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\(\)\._\- ]+$/
  }
};
const UpdateGroupRequest = {
  description: {
    maxLength: 200
  },
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\(\)\._\- ]+$/
  }
};
const UpdatePolicyRequest = {
  description: {
    maxLength: 200
  },
  name: {
    maxLength: 64,
    minLength: 1,
    pattern: /^[a-zA-Z0-9\(\)\._\- ]+$/
  }
};
const UpdateSSHKeyRequest = {
  name: {
    maxLength: 1000
  }
};

var validationRules_gen$9 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateAPIKeyRequest: CreateAPIKeyRequest,
  CreateApplicationRequest: CreateApplicationRequest,
  CreateGroupRequest: CreateGroupRequest,
  CreatePolicyRequest: CreatePolicyRequest,
  CreateSSHKeyRequest: CreateSSHKeyRequest,
  GetQuotumRequest: GetQuotumRequest,
  ListAPIKeysRequest: ListAPIKeysRequest,
  ListApplicationsRequest: ListApplicationsRequest,
  ListGroupsRequest: ListGroupsRequest,
  ListJWTsRequest: ListJWTsRequest,
  ListLogsRequest: ListLogsRequest,
  ListPermissionSetsRequest: ListPermissionSetsRequest,
  ListPoliciesRequest: ListPoliciesRequest,
  ListQuotaRequest: ListQuotaRequest,
  ListRulesRequest: ListRulesRequest,
  ListSSHKeysRequest: ListSSHKeysRequest,
  ListUsersRequest: ListUsersRequest,
  UpdateAPIKeyRequest: UpdateAPIKeyRequest,
  UpdateApplicationRequest: UpdateApplicationRequest,
  UpdateGroupRequest: UpdateGroupRequest,
  UpdatePolicyRequest: UpdatePolicyRequest,
  UpdateSSHKeyRequest: UpdateSSHKeyRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$k = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$l,
  ValidationRules: validationRules_gen$9
});

var index$n = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$k
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalPrivateNIC = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrivateNIC' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    macAddress: data.mac_address,
    privateNetworkId: data.private_network_id,
    serverId: data.server_id,
    state: data.state,
    tags: data.tags
  };
};
const unmarshalServerSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerSummary' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name
  };
};
const unmarshalBootscript = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Bootscript' failed as data isn't a dictionary.`);
  }
  return {
    arch: data.arch,
    bootcmdargs: data.bootcmdargs,
    default: data.default,
    dtb: data.dtb,
    id: data.id,
    initrd: data.initrd,
    kernel: data.kernel,
    organization: data.organization,
    project: data.project,
    public: data.public,
    title: data.title,
    zone: data.zone
  };
};
const unmarshalVolume$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Volume' failed as data isn't a dictionary.`);
  }
  return {
    creationDate: unmarshalDate(data.creation_date),
    exportUri: data.export_uri,
    id: data.id,
    modificationDate: unmarshalDate(data.modification_date),
    name: data.name,
    organization: data.organization,
    project: data.project,
    server: data.server ? unmarshalServerSummary(data.server) : undefined,
    size: data.size,
    state: data.state,
    tags: data.tags,
    volumeType: data.volume_type,
    zone: data.zone
  };
};
const unmarshalVolumeSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'VolumeSummary' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name,
    size: data.size,
    volumeType: data.volume_type
  };
};
const unmarshalImage$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Image' failed as data isn't a dictionary.`);
  }
  return {
    arch: data.arch,
    creationDate: unmarshalDate(data.creation_date),
    defaultBootscript: data.default_bootscript ? unmarshalBootscript(data.default_bootscript) : undefined,
    extraVolumes: unmarshalMapOfObject(data.extra_volumes, unmarshalVolume$2),
    fromServer: data.from_server,
    id: data.id,
    modificationDate: unmarshalDate(data.modification_date),
    name: data.name,
    organization: data.organization,
    project: data.project,
    public: data.public,
    rootVolume: data.root_volume ? unmarshalVolumeSummary(data.root_volume) : undefined,
    state: data.state,
    tags: data.tags,
    zone: data.zone
  };
};
const unmarshalPlacementGroup = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PlacementGroup' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name,
    organization: data.organization,
    policyMode: data.policy_mode,
    policyRespected: data.policy_respected,
    policyType: data.policy_type,
    project: data.project,
    tags: data.tags,
    zone: data.zone
  };
};
const unmarshalSecurityGroupSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SecurityGroupSummary' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name
  };
};
const unmarshalServerIp = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerIp' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    dynamic: data.dynamic,
    family: data.family,
    gateway: data.gateway,
    id: data.id,
    ipamId: data.ipam_id,
    netmask: data.netmask,
    provisioningMode: data.provisioning_mode,
    state: data.state,
    tags: data.tags
  };
};
const unmarshalServerIpv6 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerIpv6' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    gateway: data.gateway,
    netmask: data.netmask
  };
};
const unmarshalServerLocation = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerLocation' failed as data isn't a dictionary.`);
  }
  return {
    clusterId: data.cluster_id,
    hypervisorId: data.hypervisor_id,
    nodeId: data.node_id,
    platformId: data.platform_id,
    zoneId: data.zone_id
  };
};
const unmarshalServerMaintenance = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerMaintenance' failed as data isn't a dictionary.`);
  }
  return {
    reason: data.reason,
    startDate: unmarshalDate(data.start_date)
  };
};
const unmarshalVolumeServer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'VolumeServer' failed as data isn't a dictionary.`);
  }
  return {
    boot: data.boot,
    creationDate: unmarshalDate(data.creation_date),
    exportUri: data.export_uri,
    id: data.id,
    modificationDate: unmarshalDate(data.modification_date),
    name: data.name,
    organization: data.organization,
    project: data.project,
    server: data.server ? unmarshalServerSummary(data.server) : undefined,
    size: data.size,
    state: data.state,
    volumeType: data.volume_type,
    zone: data.zone
  };
};
const unmarshalServer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Server' failed as data isn't a dictionary.`);
  }
  return {
    allowedActions: data.allowed_actions,
    arch: data.arch,
    bootType: data.boot_type,
    bootscript: data.bootscript ? unmarshalBootscript(data.bootscript) : undefined,
    commercialType: data.commercial_type,
    creationDate: unmarshalDate(data.creation_date),
    dynamicIpRequired: data.dynamic_ip_required,
    enableIpv6: data.enable_ipv6,
    hostname: data.hostname,
    id: data.id,
    image: data.image ? unmarshalImage$2(data.image) : undefined,
    ipv6: data.ipv6 ? unmarshalServerIpv6(data.ipv6) : undefined,
    location: data.location ? unmarshalServerLocation(data.location) : undefined,
    macAddress: data.mac_address,
    maintenances: unmarshalArrayOfObject(data.maintenances, unmarshalServerMaintenance),
    modificationDate: unmarshalDate(data.modification_date),
    name: data.name,
    organization: data.organization,
    placementGroup: data.placement_group ? unmarshalPlacementGroup(data.placement_group) : undefined,
    privateIp: data.private_ip,
    privateNics: unmarshalArrayOfObject(data.private_nics, unmarshalPrivateNIC),
    project: data.project,
    protected: data.protected,
    publicIp: data.public_ip ? unmarshalServerIp(data.public_ip) : undefined,
    publicIps: unmarshalArrayOfObject(data.public_ips, unmarshalServerIp),
    routedIpEnabled: data.routed_ip_enabled,
    securityGroup: data.security_group ? unmarshalSecurityGroupSummary(data.security_group) : undefined,
    state: data.state,
    stateDetail: data.state_detail,
    tags: data.tags,
    volumes: unmarshalMapOfObject(data.volumes, unmarshalVolumeServer),
    zone: data.zone
  };
};
const unmarshalAttachServerVolumeResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AttachServerVolumeResponse' failed as data isn't a dictionary.`);
  }
  return {
    server: data.server ? unmarshalServer(data.server) : undefined
  };
};
const unmarshalCreateImageResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateImageResponse' failed as data isn't a dictionary.`);
  }
  return {
    image: data.image ? unmarshalImage$2(data.image) : undefined
  };
};
const unmarshalIp$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Ip' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    id: data.id,
    ipamId: data.ipam_id,
    organization: data.organization,
    prefix: data.prefix,
    project: data.project,
    reverse: data.reverse,
    server: data.server ? unmarshalServerSummary(data.server) : undefined,
    state: data.state,
    tags: data.tags,
    type: data.type,
    zone: data.zone
  };
};
const unmarshalCreateIpResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateIpResponse' failed as data isn't a dictionary.`);
  }
  return {
    ip: data.ip ? unmarshalIp$1(data.ip) : undefined
  };
};
const unmarshalCreatePlacementGroupResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreatePlacementGroupResponse' failed as data isn't a dictionary.`);
  }
  return {
    placementGroup: data.placement_group ? unmarshalPlacementGroup(data.placement_group) : undefined
  };
};
const unmarshalCreatePrivateNICResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreatePrivateNICResponse' failed as data isn't a dictionary.`);
  }
  return {
    privateNic: data.private_nic ? unmarshalPrivateNIC(data.private_nic) : undefined
  };
};
const unmarshalSecurityGroup = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SecurityGroup' failed as data isn't a dictionary.`);
  }
  return {
    creationDate: unmarshalDate(data.creation_date),
    description: data.description,
    enableDefaultSecurity: data.enable_default_security,
    id: data.id,
    inboundDefaultPolicy: data.inbound_default_policy,
    modificationDate: unmarshalDate(data.modification_date),
    name: data.name,
    organization: data.organization,
    organizationDefault: data.organization_default,
    outboundDefaultPolicy: data.outbound_default_policy,
    project: data.project,
    projectDefault: data.project_default,
    servers: unmarshalArrayOfObject(data.servers, unmarshalServerSummary),
    state: data.state,
    stateful: data.stateful,
    tags: data.tags,
    zone: data.zone
  };
};
const unmarshalCreateSecurityGroupResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateSecurityGroupResponse' failed as data isn't a dictionary.`);
  }
  return {
    securityGroup: data.security_group ? unmarshalSecurityGroup(data.security_group) : undefined
  };
};
const unmarshalSecurityGroupRule = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SecurityGroupRule' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    destPortFrom: data.dest_port_from,
    destPortTo: data.dest_port_to,
    direction: data.direction,
    editable: data.editable,
    id: data.id,
    ipRange: data.ip_range,
    position: data.position,
    protocol: data.protocol,
    zone: data.zone
  };
};
const unmarshalCreateSecurityGroupRuleResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateSecurityGroupRuleResponse' failed as data isn't a dictionary.`);
  }
  return {
    rule: data.rule ? unmarshalSecurityGroupRule(data.rule) : undefined
  };
};
const unmarshalCreateServerResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateServerResponse' failed as data isn't a dictionary.`);
  }
  return {
    server: data.server ? unmarshalServer(data.server) : undefined
  };
};
const unmarshalSnapshotBaseVolume = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SnapshotBaseVolume' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name
  };
};
const unmarshalSnapshot$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Snapshot' failed as data isn't a dictionary.`);
  }
  return {
    baseVolume: data.base_volume ? unmarshalSnapshotBaseVolume(data.base_volume) : undefined,
    creationDate: unmarshalDate(data.creation_date),
    errorReason: data.error_reason,
    id: data.id,
    modificationDate: unmarshalDate(data.modification_date),
    name: data.name,
    organization: data.organization,
    project: data.project,
    size: data.size,
    state: data.state,
    tags: data.tags,
    volumeType: data.volume_type,
    zone: data.zone
  };
};
const unmarshalTask = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Task' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    hrefFrom: data.href_from,
    hrefResult: data.href_result,
    id: data.id,
    progress: data.progress,
    startedAt: unmarshalDate(data.started_at),
    status: data.status,
    terminatedAt: unmarshalDate(data.terminated_at),
    zone: data.zone
  };
};
const unmarshalCreateSnapshotResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateSnapshotResponse' failed as data isn't a dictionary.`);
  }
  return {
    snapshot: data.snapshot ? unmarshalSnapshot$1(data.snapshot) : undefined,
    task: data.task ? unmarshalTask(data.task) : undefined
  };
};
const unmarshalCreateVolumeResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateVolumeResponse' failed as data isn't a dictionary.`);
  }
  return {
    volume: data.volume ? unmarshalVolume$2(data.volume) : undefined
  };
};
const unmarshalDetachServerVolumeResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DetachServerVolumeResponse' failed as data isn't a dictionary.`);
  }
  return {
    server: data.server ? unmarshalServer(data.server) : undefined
  };
};
const unmarshalExportSnapshotResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ExportSnapshotResponse' failed as data isn't a dictionary.`);
  }
  return {
    task: data.task ? unmarshalTask(data.task) : undefined
  };
};
const unmarshalGetBootscriptResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetBootscriptResponse' failed as data isn't a dictionary.`);
  }
  return {
    bootscript: data.bootscript ? unmarshalBootscript(data.bootscript) : undefined
  };
};
const unmarshalDashboard = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Dashboard' failed as data isn't a dictionary.`);
  }
  return {
    imagesCount: data.images_count,
    ipsCount: data.ips_count,
    ipsUnused: data.ips_unused,
    placementGroupsCount: data.placement_groups_count,
    privateNicsCount: data.private_nics_count,
    runningServersCount: data.running_servers_count,
    securityGroupsCount: data.security_groups_count,
    serversByTypes: data.servers_by_types,
    serversCount: data.servers_count,
    snapshotsCount: data.snapshots_count,
    volumesBSsdCount: data.volumes_b_ssd_count,
    volumesBSsdTotalSize: data.volumes_b_ssd_total_size,
    volumesCount: data.volumes_count,
    volumesLSsdCount: data.volumes_l_ssd_count,
    volumesLSsdTotalSize: data.volumes_l_ssd_total_size
  };
};
const unmarshalGetDashboardResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetDashboardResponse' failed as data isn't a dictionary.`);
  }
  return {
    dashboard: data.dashboard ? unmarshalDashboard(data.dashboard) : undefined
  };
};
const unmarshalGetImageResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetImageResponse' failed as data isn't a dictionary.`);
  }
  return {
    image: data.image ? unmarshalImage$2(data.image) : undefined
  };
};
const unmarshalGetIpResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetIpResponse' failed as data isn't a dictionary.`);
  }
  return {
    ip: data.ip ? unmarshalIp$1(data.ip) : undefined
  };
};
const unmarshalGetPlacementGroupResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetPlacementGroupResponse' failed as data isn't a dictionary.`);
  }
  return {
    placementGroup: data.placement_group ? unmarshalPlacementGroup(data.placement_group) : undefined
  };
};
const unmarshalPlacementGroupServer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PlacementGroupServer' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name,
    policyRespected: data.policy_respected
  };
};
const unmarshalGetPlacementGroupServersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetPlacementGroupServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalPlacementGroupServer)
  };
};
const unmarshalGetPrivateNICResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetPrivateNICResponse' failed as data isn't a dictionary.`);
  }
  return {
    privateNic: data.private_nic ? unmarshalPrivateNIC(data.private_nic) : undefined
  };
};
const unmarshalGetSecurityGroupResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetSecurityGroupResponse' failed as data isn't a dictionary.`);
  }
  return {
    securityGroup: data.security_group ? unmarshalSecurityGroup(data.security_group) : undefined
  };
};
const unmarshalGetSecurityGroupRuleResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetSecurityGroupRuleResponse' failed as data isn't a dictionary.`);
  }
  return {
    rule: data.rule ? unmarshalSecurityGroupRule(data.rule) : undefined
  };
};
const unmarshalGetServerResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetServerResponse' failed as data isn't a dictionary.`);
  }
  return {
    server: data.server ? unmarshalServer(data.server) : undefined
  };
};
const unmarshalGetServerTypesAvailabilityResponseAvailability = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetServerTypesAvailabilityResponseAvailability' failed as data isn't a dictionary.`);
  }
  return {
    availability: data.availability
  };
};
const unmarshalGetServerTypesAvailabilityResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetServerTypesAvailabilityResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalMapOfObject(data.servers, unmarshalGetServerTypesAvailabilityResponseAvailability),
    totalCount: data.total_count
  };
};
const unmarshalGetSnapshotResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetSnapshotResponse' failed as data isn't a dictionary.`);
  }
  return {
    snapshot: data.snapshot ? unmarshalSnapshot$1(data.snapshot) : undefined
  };
};
const unmarshalGetVolumeResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetVolumeResponse' failed as data isn't a dictionary.`);
  }
  return {
    volume: data.volume ? unmarshalVolume$2(data.volume) : undefined
  };
};
const unmarshalListBootscriptsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListBootscriptsResponse' failed as data isn't a dictionary.`);
  }
  return {
    bootscripts: unmarshalArrayOfObject(data.bootscripts, unmarshalBootscript),
    totalCount: data.total_count
  };
};
const unmarshalListImagesResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListImagesResponse' failed as data isn't a dictionary.`);
  }
  return {
    images: unmarshalArrayOfObject(data.images, unmarshalImage$2),
    totalCount: data.total_count
  };
};
const unmarshalListIpsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListIpsResponse' failed as data isn't a dictionary.`);
  }
  return {
    ips: unmarshalArrayOfObject(data.ips, unmarshalIp$1),
    totalCount: data.total_count
  };
};
const unmarshalListPlacementGroupsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPlacementGroupsResponse' failed as data isn't a dictionary.`);
  }
  return {
    placementGroups: unmarshalArrayOfObject(data.placement_groups, unmarshalPlacementGroup),
    totalCount: data.total_count
  };
};
const unmarshalListPrivateNICsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPrivateNICsResponse' failed as data isn't a dictionary.`);
  }
  return {
    privateNics: unmarshalArrayOfObject(data.private_nics, unmarshalPrivateNIC),
    totalCount: data.total_count
  };
};
const unmarshalListSecurityGroupRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSecurityGroupRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalSecurityGroupRule),
    totalCount: data.total_count
  };
};
const unmarshalListSecurityGroupsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSecurityGroupsResponse' failed as data isn't a dictionary.`);
  }
  return {
    securityGroups: unmarshalArrayOfObject(data.security_groups, unmarshalSecurityGroup),
    totalCount: data.total_count
  };
};
const unmarshalListServerActionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServerActionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    actions: data.actions
  };
};
const unmarshalListServerUserDataResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServerUserDataResponse' failed as data isn't a dictionary.`);
  }
  return {
    userData: data.user_data
  };
};
const unmarshalListServersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalServer),
    totalCount: data.total_count
  };
};
const unmarshalServerTypeNetworkInterface = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerTypeNetworkInterface' failed as data isn't a dictionary.`);
  }
  return {
    internalBandwidth: data.internal_bandwidth,
    internetBandwidth: data.internet_bandwidth
  };
};
const unmarshalServerTypeVolumeConstraintSizes = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerTypeVolumeConstraintSizes' failed as data isn't a dictionary.`);
  }
  return {
    maxSize: data.max_size,
    minSize: data.min_size
  };
};
const unmarshalServerTypeCapabilities = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerTypeCapabilities' failed as data isn't a dictionary.`);
  }
  return {
    blockStorage: data.block_storage,
    bootTypes: data.boot_types
  };
};
const unmarshalServerTypeNetwork = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerTypeNetwork' failed as data isn't a dictionary.`);
  }
  return {
    interfaces: unmarshalArrayOfObject(data.interfaces, unmarshalServerTypeNetworkInterface),
    ipv6Support: data.ipv6_support,
    sumInternalBandwidth: data.sum_internal_bandwidth,
    sumInternetBandwidth: data.sum_internet_bandwidth
  };
};
const unmarshalServerTypeVolumeConstraintsByType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerTypeVolumeConstraintsByType' failed as data isn't a dictionary.`);
  }
  return {
    lSsd: data.l_ssd ? unmarshalServerTypeVolumeConstraintSizes(data.l_ssd) : undefined
  };
};
const unmarshalServerType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerType' failed as data isn't a dictionary.`);
  }
  return {
    altNames: data.alt_names,
    arch: data.arch,
    baremetal: data.baremetal,
    capabilities: data.capabilities ? unmarshalServerTypeCapabilities(data.capabilities) : undefined,
    gpu: data.gpu,
    hourlyPrice: data.hourly_price,
    monthlyPrice: data.monthly_price,
    ncpus: data.ncpus,
    network: data.network ? unmarshalServerTypeNetwork(data.network) : undefined,
    perVolumeConstraint: data.per_volume_constraint ? unmarshalServerTypeVolumeConstraintsByType(data.per_volume_constraint) : undefined,
    ram: data.ram,
    scratchStorageMaxSize: data.scratch_storage_max_size,
    volumesConstraint: data.volumes_constraint ? unmarshalServerTypeVolumeConstraintSizes(data.volumes_constraint) : undefined
  };
};
const unmarshalListServersTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListServersTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalMapOfObject(data.servers, unmarshalServerType),
    totalCount: data.total_count
  };
};
const unmarshalListSnapshotsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSnapshotsResponse' failed as data isn't a dictionary.`);
  }
  return {
    snapshots: unmarshalArrayOfObject(data.snapshots, unmarshalSnapshot$1),
    totalCount: data.total_count
  };
};
const unmarshalListVolumesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListVolumesResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    volumes: unmarshalArrayOfObject(data.volumes, unmarshalVolume$2)
  };
};
const unmarshalVolumeTypeCapabilities = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'VolumeTypeCapabilities' failed as data isn't a dictionary.`);
  }
  return {
    snapshot: data.snapshot
  };
};
const unmarshalVolumeTypeConstraints = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'VolumeTypeConstraints' failed as data isn't a dictionary.`);
  }
  return {
    max: data.max,
    min: data.min
  };
};
const unmarshalVolumeType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'VolumeType' failed as data isn't a dictionary.`);
  }
  return {
    capabilities: data.capabilities ? unmarshalVolumeTypeCapabilities(data.capabilities) : undefined,
    constraints: data.constraints ? unmarshalVolumeTypeConstraints(data.constraints) : undefined,
    displayName: data.display_name
  };
};
const unmarshalListVolumesTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListVolumesTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    volumes: unmarshalMapOfObject(data.volumes, unmarshalVolumeType)
  };
};
const unmarshalMigrationPlan = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'MigrationPlan' failed as data isn't a dictionary.`);
  }
  return {
    snapshots: unmarshalArrayOfObject(data.snapshots, unmarshalSnapshot$1),
    validationKey: data.validation_key,
    volume: data.volume ? unmarshalVolume$2(data.volume) : undefined
  };
};
const unmarshalServerActionResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServerActionResponse' failed as data isn't a dictionary.`);
  }
  return {
    task: data.task ? unmarshalTask(data.task) : undefined
  };
};
const unmarshalSetImageResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetImageResponse' failed as data isn't a dictionary.`);
  }
  return {
    image: data.image ? unmarshalImage$2(data.image) : undefined
  };
};
const unmarshalSetPlacementGroupResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetPlacementGroupResponse' failed as data isn't a dictionary.`);
  }
  return {
    placementGroup: data.placement_group ? unmarshalPlacementGroup(data.placement_group) : undefined
  };
};
const unmarshalSetPlacementGroupServersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetPlacementGroupServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalPlacementGroupServer)
  };
};
const unmarshalSetSecurityGroupResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetSecurityGroupResponse' failed as data isn't a dictionary.`);
  }
  return {
    securityGroup: data.security_group ? unmarshalSecurityGroup(data.security_group) : undefined
  };
};
const unmarshalSetSecurityGroupRuleResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetSecurityGroupRuleResponse' failed as data isn't a dictionary.`);
  }
  return {
    rule: data.rule ? unmarshalSecurityGroupRule(data.rule) : undefined
  };
};
const unmarshalSetSecurityGroupRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetSecurityGroupRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalSecurityGroupRule)
  };
};
const unmarshalSetServerResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetServerResponse' failed as data isn't a dictionary.`);
  }
  return {
    server: data.server ? unmarshalServer(data.server) : undefined
  };
};
const unmarshalSetSnapshotResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetSnapshotResponse' failed as data isn't a dictionary.`);
  }
  return {
    snapshot: data.snapshot ? unmarshalSnapshot$1(data.snapshot) : undefined
  };
};
const unmarshalUpdateImageResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateImageResponse' failed as data isn't a dictionary.`);
  }
  return {
    image: data.image ? unmarshalImage$2(data.image) : undefined
  };
};
const unmarshalUpdateIpResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateIpResponse' failed as data isn't a dictionary.`);
  }
  return {
    ip: data.ip ? unmarshalIp$1(data.ip) : undefined
  };
};
const unmarshalUpdatePlacementGroupResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdatePlacementGroupResponse' failed as data isn't a dictionary.`);
  }
  return {
    placementGroup: data.placement_group ? unmarshalPlacementGroup(data.placement_group) : undefined
  };
};
const unmarshalUpdatePlacementGroupServersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdatePlacementGroupServersResponse' failed as data isn't a dictionary.`);
  }
  return {
    servers: unmarshalArrayOfObject(data.servers, unmarshalPlacementGroupServer)
  };
};
const unmarshalUpdateSecurityGroupResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateSecurityGroupResponse' failed as data isn't a dictionary.`);
  }
  return {
    securityGroup: data.security_group ? unmarshalSecurityGroup(data.security_group) : undefined
  };
};
const unmarshalUpdateSecurityGroupRuleResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateSecurityGroupRuleResponse' failed as data isn't a dictionary.`);
  }
  return {
    rule: data.rule ? unmarshalSecurityGroupRule(data.rule) : undefined
  };
};
const unmarshalUpdateServerResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateServerResponse' failed as data isn't a dictionary.`);
  }
  return {
    server: data.server ? unmarshalServer(data.server) : undefined
  };
};
const unmarshalUpdateSnapshotResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateSnapshotResponse' failed as data isn't a dictionary.`);
  }
  return {
    snapshot: data.snapshot ? unmarshalSnapshot$1(data.snapshot) : undefined
  };
};
const unmarshalUpdateVolumeResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpdateVolumeResponse' failed as data isn't a dictionary.`);
  }
  return {
    volume: data.volume ? unmarshalVolume$2(data.volume) : undefined
  };
};
const marshalApplyBlockMigrationRequest = (request, defaults) => ({
  validation_key: request.validationKey,
  ...resolveOneOf([{
    param: 'volume_id',
    value: request.volumeId
  }, {
    param: 'snapshot_id',
    value: request.snapshotId
  }])
});
const marshalAttachServerVolumeRequest = (request, defaults) => ({
  boot: request.boot,
  volume_id: request.volumeId,
  volume_type: request.volumeType
});
const marshalVolumeTemplate = (request, defaults) => ({
  id: request.id,
  name: request.name,
  size: request.size,
  volume_type: request.volumeType,
  ...resolveOneOf([{
    param: 'project',
    value: request.project
  }, {
    param: 'organization',
    value: request.organization
  }])
});
const marshalCreateImageRequest = (request, defaults) => ({
  arch: request.arch,
  default_bootscript: request.defaultBootscript,
  extra_volumes: request.extraVolumes !== undefined ? Object.entries(request.extraVolumes).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: marshalVolumeTemplate(value)
  }), {}) : undefined,
  name: request.name || randomName('img'),
  public: request.public,
  root_volume: request.rootVolume,
  tags: request.tags,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project',
    value: request.project
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization',
    value: request.organization
  }])
});
const marshalCreateIpRequest$1 = (request, defaults) => ({
  server: request.server,
  tags: request.tags,
  type: request.type,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project',
    value: request.project
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization',
    value: request.organization
  }])
});
const marshalCreatePlacementGroupRequest = (request, defaults) => ({
  name: request.name || randomName('pg'),
  policy_mode: request.policyMode,
  policy_type: request.policyType,
  tags: request.tags,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project',
    value: request.project
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization',
    value: request.organization
  }])
});
const marshalCreatePrivateNICRequest = (request, defaults) => ({
  ip_ids: request.ipIds,
  private_network_id: request.privateNetworkId,
  tags: request.tags
});
const marshalCreateSecurityGroupRequest = (request, defaults) => ({
  description: request.description,
  enable_default_security: request.enableDefaultSecurity,
  inbound_default_policy: request.inboundDefaultPolicy,
  name: request.name || randomName('sg'),
  outbound_default_policy: request.outboundDefaultPolicy,
  stateful: request.stateful,
  tags: request.tags,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project',
    value: request.project
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization',
    value: request.organization
  }]),
  ...resolveOneOf([{
    param: 'organization_default',
    value: request.organizationDefault
  }, {
    param: 'project_default',
    value: request.projectDefault
  }])
});
const marshalCreateSecurityGroupRuleRequest = (request, defaults) => ({
  action: request.action,
  dest_port_from: request.destPortFrom,
  dest_port_to: request.destPortTo,
  direction: request.direction,
  editable: request.editable,
  ip_range: request.ipRange,
  position: request.position,
  protocol: request.protocol
});
const marshalVolumeServerTemplate = (request, defaults) => ({
  base_snapshot: request.baseSnapshot,
  boot: request.boot,
  id: request.id,
  name: request.name,
  organization: request.organization,
  project: request.project,
  size: request.size,
  volume_type: request.volumeType
});
const marshalCreateServerRequest = (request, defaults) => ({
  boot_type: request.bootType,
  bootscript: request.bootscript,
  commercial_type: request.commercialType,
  dynamic_ip_required: request.dynamicIpRequired,
  enable_ipv6: request.enableIpv6,
  image: request.image,
  name: request.name || randomName('srv'),
  placement_group: request.placementGroup,
  public_ip: request.publicIp,
  public_ips: request.publicIps,
  routed_ip_enabled: request.routedIpEnabled,
  security_group: request.securityGroup,
  tags: request.tags,
  volumes: request.volumes !== undefined ? Object.entries(request.volumes).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: marshalVolumeServerTemplate(value)
  }), {}) : undefined,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project',
    value: request.project
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization',
    value: request.organization
  }])
});
const marshalCreateSnapshotRequest$1 = (request, defaults) => ({
  bucket: request.bucket,
  key: request.key,
  name: request.name || randomName('snp'),
  size: request.size,
  tags: request.tags,
  volume_id: request.volumeId,
  volume_type: request.volumeType,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project',
    value: request.project
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization',
    value: request.organization
  }])
});
const marshalCreateVolumeRequest$1 = (request, defaults) => ({
  name: request.name || randomName('vol'),
  tags: request.tags,
  volume_type: request.volumeType,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project',
    value: request.project
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization',
    value: request.organization
  }]),
  ...resolveOneOf([{
    param: 'size',
    value: request.size
  }, {
    param: 'base_snapshot',
    value: request.baseSnapshot
  }])
});
const marshalDetachServerVolumeRequest = (request, defaults) => ({
  volume_id: request.volumeId
});
const marshalExportSnapshotRequest = (request, defaults) => ({
  bucket: request.bucket,
  key: request.key
});
const marshalPlanBlockMigrationRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'volume_id',
    value: request.volumeId
  }, {
    param: 'snapshot_id',
    value: request.snapshotId
  }])
});
const marshalServerActionRequestVolumeBackupTemplate = (request, defaults) => ({
  volume_type: request.volumeType
});
const marshalServerActionRequest = (request, defaults) => ({
  action: request.action,
  name: request.name,
  volumes: request.volumes !== undefined ? Object.entries(request.volumes).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: marshalServerActionRequestVolumeBackupTemplate(value)
  }), {}) : undefined
});
const marshalServerSummary = (request, defaults) => ({
  id: request.id,
  name: request.name
});
const marshalBootscript = (request, defaults) => ({
  arch: request.arch,
  bootcmdargs: request.bootcmdargs,
  default: request.default,
  dtb: request.dtb,
  id: request.id,
  initrd: request.initrd,
  kernel: request.kernel,
  organization: request.organization,
  project: request.project,
  public: request.public,
  title: request.title,
  zone: request.zone
});
const marshalVolume = (request, defaults) => ({
  creation_date: request.creationDate,
  export_uri: request.exportUri,
  id: request.id,
  modification_date: request.modificationDate,
  name: request.name,
  organization: request.organization,
  project: request.project,
  server: request.server !== undefined ? marshalServerSummary(request.server) : undefined,
  size: request.size,
  state: request.state,
  tags: request.tags,
  volume_type: request.volumeType,
  zone: request.zone
});
const marshalVolumeSummary = (request, defaults) => ({
  id: request.id,
  name: request.name,
  size: request.size,
  volume_type: request.volumeType
});
const marshalSetImageRequest = (request, defaults) => ({
  arch: request.arch,
  creation_date: request.creationDate,
  default_bootscript: request.defaultBootscript !== undefined ? marshalBootscript(request.defaultBootscript) : undefined,
  extra_volumes: request.extraVolumes !== undefined ? Object.entries(request.extraVolumes).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: marshalVolume(value)
  }), {}) : undefined,
  from_server: request.fromServer,
  modification_date: request.modificationDate,
  name: request.name,
  organization: request.organization ?? defaults.defaultOrganizationId,
  project: request.project ?? defaults.defaultProjectId,
  public: request.public,
  root_volume: request.rootVolume !== undefined ? marshalVolumeSummary(request.rootVolume) : undefined,
  state: request.state,
  tags: request.tags
});
const marshalSetPlacementGroupRequest = (request, defaults) => ({
  name: request.name,
  organization: request.organization ?? defaults.defaultOrganizationId,
  policy_mode: request.policyMode,
  policy_type: request.policyType,
  project: request.project ?? defaults.defaultProjectId,
  tags: request.tags
});
const marshalSetPlacementGroupServersRequest = (request, defaults) => ({
  servers: request.servers
});
const marshalSetSecurityGroupRequest = (request, defaults) => ({
  creation_date: request.creationDate,
  description: request.description,
  enable_default_security: request.enableDefaultSecurity,
  inbound_default_policy: request.inboundDefaultPolicy,
  modification_date: request.modificationDate,
  name: request.name,
  organization: request.organization ?? defaults.defaultOrganizationId,
  organization_default: request.organizationDefault,
  outbound_default_policy: request.outboundDefaultPolicy,
  project: request.project ?? defaults.defaultProjectId,
  project_default: request.projectDefault,
  servers: request.servers !== undefined ? request.servers.map(elt => marshalServerSummary(elt)) : undefined,
  stateful: request.stateful,
  tags: request.tags
});
const marshalSetSecurityGroupRuleRequest = (request, defaults) => ({
  action: request.action,
  dest_port_from: request.destPortFrom,
  dest_port_to: request.destPortTo,
  direction: request.direction,
  editable: request.editable,
  id: request.id,
  ip_range: request.ipRange,
  position: request.position,
  protocol: request.protocol
});
const marshalSetSecurityGroupRulesRequestRule = (request, defaults) => ({
  action: request.action,
  dest_port_from: request.destPortFrom,
  dest_port_to: request.destPortTo,
  direction: request.direction,
  editable: request.editable,
  id: request.id,
  ip_range: request.ipRange,
  position: request.position,
  protocol: request.protocol,
  zone: request.zone
});
const marshalSetSecurityGroupRulesRequest = (request, defaults) => ({
  rules: request.rules !== undefined ? request.rules.map(elt => marshalSetSecurityGroupRulesRequestRule(elt)) : undefined
});
const marshalImage = (request, defaults) => ({
  arch: request.arch,
  creation_date: request.creationDate,
  default_bootscript: request.defaultBootscript !== undefined ? marshalBootscript(request.defaultBootscript) : undefined,
  extra_volumes: Object.entries(request.extraVolumes).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: marshalVolume(value)
  }), {}),
  from_server: request.fromServer,
  id: request.id,
  modification_date: request.modificationDate,
  name: request.name,
  organization: request.organization,
  project: request.project,
  public: request.public,
  root_volume: request.rootVolume !== undefined ? marshalVolumeSummary(request.rootVolume) : undefined,
  state: request.state,
  tags: request.tags,
  zone: request.zone
});
const marshalPlacementGroup = (request, defaults) => ({
  id: request.id,
  name: request.name,
  organization: request.organization,
  policy_mode: request.policyMode,
  policy_respected: request.policyRespected,
  policy_type: request.policyType,
  project: request.project,
  tags: request.tags,
  zone: request.zone
});
const marshalPrivateNIC = (request, defaults) => ({
  id: request.id,
  mac_address: request.macAddress,
  private_network_id: request.privateNetworkId,
  server_id: request.serverId,
  state: request.state,
  tags: request.tags
});
const marshalSecurityGroupSummary = (request, defaults) => ({
  id: request.id,
  name: request.name
});
const marshalServerIp = (request, defaults) => ({
  address: request.address,
  dynamic: request.dynamic,
  family: request.family,
  gateway: request.gateway,
  id: request.id,
  ipam_id: request.ipamId,
  netmask: request.netmask,
  provisioning_mode: request.provisioningMode,
  state: request.state,
  tags: request.tags
});
const marshalServerIpv6 = (request, defaults) => ({
  address: request.address,
  gateway: request.gateway,
  netmask: request.netmask
});
const marshalServerLocation = (request, defaults) => ({
  cluster_id: request.clusterId,
  hypervisor_id: request.hypervisorId,
  node_id: request.nodeId,
  platform_id: request.platformId,
  zone_id: request.zoneId
});
const marshalServerMaintenance = (request, defaults) => ({
  reason: request.reason,
  start_date: request.startDate
});
const marshalSetServerRequest = (request, defaults) => ({
  allowed_actions: request.allowedActions !== undefined ? request.allowedActions : undefined,
  arch: request.arch,
  boot_type: request.bootType,
  bootscript: request.bootscript !== undefined ? marshalBootscript(request.bootscript) : undefined,
  commercial_type: request.commercialType,
  creation_date: request.creationDate,
  dynamic_ip_required: request.dynamicIpRequired,
  enable_ipv6: request.enableIpv6,
  hostname: request.hostname,
  image: request.image !== undefined ? marshalImage(request.image) : undefined,
  ipv6: request.ipv6 !== undefined ? marshalServerIpv6(request.ipv6) : undefined,
  location: request.location !== undefined ? marshalServerLocation(request.location) : undefined,
  maintenances: request.maintenances !== undefined ? request.maintenances.map(elt => marshalServerMaintenance(elt)) : undefined,
  modification_date: request.modificationDate,
  name: request.name,
  organization: request.organization ?? defaults.defaultOrganizationId,
  placement_group: request.placementGroup !== undefined ? marshalPlacementGroup(request.placementGroup) : undefined,
  private_ip: request.privateIp,
  private_nics: request.privateNics !== undefined ? request.privateNics.map(elt => marshalPrivateNIC(elt)) : undefined,
  project: request.project ?? defaults.defaultProjectId,
  protected: request.protected,
  public_ip: request.publicIp !== undefined ? marshalServerIp(request.publicIp) : undefined,
  public_ips: request.publicIps !== undefined ? request.publicIps.map(elt => marshalServerIp(elt)) : undefined,
  routed_ip_enabled: request.routedIpEnabled,
  security_group: request.securityGroup !== undefined ? marshalSecurityGroupSummary(request.securityGroup) : undefined,
  state: request.state,
  state_detail: request.stateDetail,
  tags: request.tags,
  volumes: request.volumes !== undefined ? Object.entries(request.volumes).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: marshalVolume(value)
  }), {}) : undefined
});
const marshalSnapshotBaseVolume = (request, defaults) => ({
  id: request.id,
  name: request.name
});
const marshalSetSnapshotRequest = (request, defaults) => ({
  base_volume: request.baseVolume !== undefined ? marshalSnapshotBaseVolume(request.baseVolume) : undefined,
  creation_date: request.creationDate,
  id: request.id,
  modification_date: request.modificationDate,
  name: request.name,
  organization: request.organization ?? defaults.defaultOrganizationId,
  project: request.project ?? defaults.defaultProjectId,
  size: request.size,
  state: request.state,
  tags: request.tags,
  volume_type: request.volumeType
});
const marshalVolumeImageUpdateTemplate = (request, defaults) => ({
  id: request.id
});
const marshalUpdateImageRequest$1 = (request, defaults) => ({
  arch: request.arch,
  extra_volumes: request.extraVolumes !== undefined ? Object.entries(request.extraVolumes).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: marshalVolumeImageUpdateTemplate(value)
  }), {}) : undefined,
  name: request.name,
  public: request.public,
  tags: request.tags
});
const marshalUpdateIpRequest$1 = (request, defaults) => ({
  reverse: request.reverse,
  server: request.server,
  tags: request.tags,
  type: request.type
});
const marshalUpdatePlacementGroupRequest = (request, defaults) => ({
  name: request.name,
  policy_mode: request.policyMode,
  policy_type: request.policyType,
  tags: request.tags
});
const marshalUpdatePlacementGroupServersRequest = (request, defaults) => ({
  servers: request.servers
});
const marshalUpdatePrivateNICRequest = (request, defaults) => ({
  tags: request.tags
});
const marshalUpdateSecurityGroupRequest = (request, defaults) => ({
  description: request.description,
  enable_default_security: request.enableDefaultSecurity,
  inbound_default_policy: request.inboundDefaultPolicy,
  name: request.name,
  organization_default: request.organizationDefault,
  outbound_default_policy: request.outboundDefaultPolicy,
  project_default: request.projectDefault,
  stateful: request.stateful,
  tags: request.tags
});
const marshalUpdateSecurityGroupRuleRequest = (request, defaults) => ({
  action: request.action,
  dest_port_from: request.destPortFrom,
  dest_port_to: request.destPortTo,
  direction: request.direction,
  ip_range: request.ipRange,
  position: request.position,
  protocol: request.protocol
});
const marshalSecurityGroupTemplate = (request, defaults) => ({
  id: request.id,
  name: request.name
});
const marshalUpdateServerRequest = (request, defaults) => ({
  boot_type: request.bootType,
  bootscript: request.bootscript,
  commercial_type: request.commercialType,
  dynamic_ip_required: request.dynamicIpRequired,
  enable_ipv6: request.enableIpv6,
  name: request.name,
  placement_group: request.placementGroup,
  private_nics: request.privateNics,
  protected: request.protected,
  public_ips: request.publicIps,
  routed_ip_enabled: request.routedIpEnabled,
  security_group: request.securityGroup !== undefined ? marshalSecurityGroupTemplate(request.securityGroup) : undefined,
  tags: request.tags,
  volumes: request.volumes !== undefined ? Object.entries(request.volumes).reduce((acc, [key, value]) => ({
    ...acc,
    [key]: marshalVolumeServerTemplate(value)
  }), {}) : undefined
});
const marshalUpdateSnapshotRequest$1 = (request, defaults) => ({
  name: request.name,
  tags: request.tags
});
const marshalUpdateVolumeRequest$1 = (request, defaults) => ({
  name: request.name,
  size: request.size,
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$k = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Instance API. */
let API$k = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'fr-par-3', 'nl-ams-1', 'nl-ams-2', 'nl-ams-3', 'pl-waw-1', 'pl-waw-2', 'pl-waw-3'];

  /**
   * Get availability. Get availability for all Instance types.
   *
   * @param request - The request {@link GetServerTypesAvailabilityRequest}
   * @returns A Promise of GetServerTypesAvailabilityResponse
   */
  getServerTypesAvailability = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/products/servers/availability`,
    urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize])
  }, unmarshalGetServerTypesAvailabilityResponse);

  /**
   * List Instance types. List available Instance types and their technical
   * details.
   *
   * @param request - The request {@link ListServersTypesRequest}
   * @returns A Promise of ListServersTypesResponse
   */
  listServersTypes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/products/servers`,
    urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize])
  }, unmarshalListServersTypesResponse);

  /**
   * List volume types. List all volume types and their technical details.
   *
   * @param request - The request {@link ListVolumesTypesRequest}
   * @returns A Promise of ListVolumesTypesResponse
   */
  listVolumesTypes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/products/volumes`,
    urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize])
  }, unmarshalListVolumesTypesResponse);
  pageOfListServers = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`,
    urlParams: urlParams(['commercial_type', request.commercialType], ['name', request.name], ['order', request.order], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['private_ip', request.privateIp], ['private_network', request.privateNetwork], ['private_networks', request.privateNetworks && request.privateNetworks.length > 0 ? request.privateNetworks.join(',') : undefined], ['private_nic_mac_address', request.privateNicMacAddress], ['project', request.project], ['servers', request.servers && request.servers.length > 0 ? request.servers.join(',') : undefined], ['state', request.state], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined], ['with_ip', request.withIp], ['without_ip', request.withoutIp])
  }, unmarshalListServersResponse);

  /**
   * List all Instances. List all Instances in a specified Availability Zone,
   * e.g. `fr-par-1`.
   *
   * @param request - The request {@link ListServersRequest}
   * @returns A Promise of ListServersResponse
   */
  listServers = (request = {}) => enrichForPagination('servers', this.pageOfListServers, request);
  _createServer = request => this.client.fetch({
    body: JSON.stringify(marshalCreateServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers`
  }, unmarshalCreateServerResponse);

  /**
   * Delete an Instance. Delete the Instance with the specified ID.
   *
   * @param request - The request {@link DeleteServerRequest}
   */
  deleteServer = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  });

  /**
   * Get an Instance. Get the details of a specified Instance.
   *
   * @param request - The request {@link GetServerRequest}
   * @returns A Promise of GetServerResponse
   */
  getServer = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalGetServerResponse);
  _setServer = request => this.client.fetch({
    body: JSON.stringify(marshalSetServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('id', request.id)}`
  }, unmarshalSetServerResponse);
  _updateServer = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateServerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}`
  }, unmarshalUpdateServerResponse);

  /**
   * List Instance actions. List all actions (e.g. power on, power off, reboot)
   * that can currently be performed on an Instance.
   *
   * @param request - The request {@link ListServerActionsRequest}
   * @returns A Promise of ListServerActionsResponse
   */
  listServerActions = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/action`
  }, unmarshalListServerActionsResponse);

  /**
   * Perform action. Perform an action on an Instance. Available actions are:
   * `poweron`: Start a stopped Instance. `poweroff`: Fully stop the Instance
   * and release the hypervisor slot. `stop_in_place`: Stop the Instance, but
   * keep the slot on the hypervisor. `reboot`: Stop the instance and restart
   * it. `backup`: Create an image with all the volumes of an Instance.
   * `terminate`: Delete the Instance along with all attached volumes.
   * `enable_routed_ip`: Migrate the Instance to the new network stack.
   *
   * Keep in mind that terminating an Instance will result in the deletion of
   * all attached volumes, including local and block storage. If you want to
   * preserve your local volumes, you should use the `archive` action instead of
   * `terminate`. Similarly, if you want to keep your block storage volumes, you
   * must first detach them before issuing the `terminate` command. For more
   * information, read the [Volumes](#path-volumes-list-volumes) documentation.
   *
   * @param request - The request {@link ServerActionRequest}
   * @returns A Promise of ServerActionResponse
   */
  serverAction = request => this.client.fetch({
    body: JSON.stringify(marshalServerActionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/action`
  }, unmarshalServerActionResponse);

  /**
   * List user data. List all user data keys registered on a specified Instance.
   *
   * @param request - The request {@link ListServerUserDataRequest}
   * @returns A Promise of ListServerUserDataResponse
   */
  listServerUserData = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/user_data`
  }, unmarshalListServerUserDataResponse);

  /**
   * Delete user data. Delete the specified key from an Instance's user data.
   *
   * @param request - The request {@link DeleteServerUserDataRequest}
   */
  deleteServerUserData = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/user_data/${validatePathParam('key', request.key)}`
  });
  attachServerVolume = request => this.client.fetch({
    body: JSON.stringify(marshalAttachServerVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/attach-volume`
  }, unmarshalAttachServerVolumeResponse);
  detachServerVolume = request => this.client.fetch({
    body: JSON.stringify(marshalDetachServerVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/detach-volume`
  }, unmarshalDetachServerVolumeResponse);
  pageOfListImages = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images`,
    urlParams: urlParams(['arch', request.arch], ['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['project', request.project], ['public', request.public], ['tags', request.tags])
  }, unmarshalListImagesResponse$2);

  /**
   * List Instance images. List all existing Instance images.
   *
   * @param request - The request {@link ListImagesRequest}
   * @returns A Promise of ListImagesResponse
   */
  listImages = (request = {}) => enrichForPagination('images', this.pageOfListImages, request);

  /**
   * Get an Instance image. Get details of an image with the specified ID.
   *
   * @param request - The request {@link GetImageRequest}
   * @returns A Promise of GetImageResponse
   */
  getImage = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images/${validatePathParam('imageId', request.imageId)}`
  }, unmarshalGetImageResponse);

  /**
   * Create an Instance image. Create an Instance image from the specified
   * snapshot ID.
   *
   * @param request - The request {@link CreateImageRequest}
   * @returns A Promise of CreateImageResponse
   */
  createImage = request => this.client.fetch({
    body: JSON.stringify(marshalCreateImageRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images`
  }, unmarshalCreateImageResponse);
  _setImage = request => this.client.fetch({
    body: JSON.stringify(marshalSetImageRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images/${validatePathParam('id', request.id)}`
  }, unmarshalSetImageResponse);

  /**
   * Update image. Update the properties of an image.
   *
   * @param request - The request {@link UpdateImageRequest}
   * @returns A Promise of UpdateImageResponse
   */
  updateImage = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateImageRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images/${validatePathParam('imageId', request.imageId)}`
  }, unmarshalUpdateImageResponse);

  /**
   * Delete an Instance image. Delete the image with the specified ID.
   *
   * @param request - The request {@link DeleteImageRequest}
   */
  deleteImage = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/images/${validatePathParam('imageId', request.imageId)}`
  });
  pageOfListSnapshots = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots`,
    urlParams: urlParams(['base_volume_id', request.baseVolumeId], ['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['project', request.project], ['tags', request.tags])
  }, unmarshalListSnapshotsResponse$1);

  /**
   * List snapshots. List all snapshots of an Organization in a specified
   * Availability Zone.
   *
   * @param request - The request {@link ListSnapshotsRequest}
   * @returns A Promise of ListSnapshotsResponse
   */
  listSnapshots = (request = {}) => enrichForPagination('snapshots', this.pageOfListSnapshots, request);

  /**
   * Create a snapshot from a specified volume or from a QCOW2 file. Create a
   * snapshot from a specified volume or from a QCOW2 file in a specified
   * Availability Zone.
   *
   * @param request - The request {@link CreateSnapshotRequest}
   * @returns A Promise of CreateSnapshotResponse
   */
  createSnapshot = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateSnapshotRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots`
  }, unmarshalCreateSnapshotResponse);

  /**
   * Get a snapshot. Get details of a snapshot with the specified ID.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @returns A Promise of GetSnapshotResponse
   */
  getSnapshot = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalGetSnapshotResponse);
  _setSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalSetSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSetSnapshotResponse);

  /**
   * Update a snapshot. Update the properties of a snapshot.
   *
   * @param request - The request {@link UpdateSnapshotRequest}
   * @returns A Promise of UpdateSnapshotResponse
   */
  updateSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSnapshotRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalUpdateSnapshotResponse);

  /**
   * Delete a snapshot. Delete the snapshot with the specified ID.
   *
   * @param request - The request {@link DeleteSnapshotRequest}
   */
  deleteSnapshot = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  });

  /**
   * Export a snapshot. Export a snapshot to a specified S3 bucket in the same
   * region.
   *
   * @param request - The request {@link ExportSnapshotRequest}
   * @returns A Promise of ExportSnapshotResponse
   */
  exportSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalExportSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}/export`
  }, unmarshalExportSnapshotResponse);
  pageOfListVolumes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes`,
    urlParams: urlParams(['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['project', request.project], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined], ['volume_type', request.volumeType])
  }, unmarshalListVolumesResponse$1);

  /**
   * List volumes. List volumes in the specified Availability Zone. You can
   * filter the output by volume type.
   *
   * @param request - The request {@link ListVolumesRequest}
   * @returns A Promise of ListVolumesResponse
   */
  listVolumes = (request = {}) => enrichForPagination('volumes', this.pageOfListVolumes, request);

  /**
   * Create a volume. Create a volume of a specified type in an Availability
   * Zone.
   *
   * @param request - The request {@link CreateVolumeRequest}
   * @returns A Promise of CreateVolumeResponse
   */
  createVolume = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateVolumeRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes`
  }, unmarshalCreateVolumeResponse);

  /**
   * Get a volume. Get details of a volume with the specified ID.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @returns A Promise of GetVolumeResponse
   */
  getVolume = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalGetVolumeResponse);

  /**
   * Update a volume. Replace the name and/or size properties of a volume
   * specified by its ID, with the specified value(s). Any volume name can be
   * changed, however only `b_ssd` volumes can currently be increased in size.
   *
   * @param request - The request {@link UpdateVolumeRequest}
   * @returns A Promise of UpdateVolumeResponse
   */
  updateVolume = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateVolumeRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalUpdateVolumeResponse);

  /**
   * Delete a volume. Delete the volume with the specified ID.
   *
   * @param request - The request {@link DeleteVolumeRequest}
   */
  deleteVolume = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  });
  pageOfListSecurityGroups = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups`,
    urlParams: urlParams(['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['project', request.project], ['project_default', request.projectDefault], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined])
  }, unmarshalListSecurityGroupsResponse);

  /**
   * List security groups. List all existing security groups.
   *
   * @param request - The request {@link ListSecurityGroupsRequest}
   * @returns A Promise of ListSecurityGroupsResponse
   */
  listSecurityGroups = (request = {}) => enrichForPagination('securityGroups', this.pageOfListSecurityGroups, request);

  /**
   * Create a security group. Create a security group with a specified name and
   * description.
   *
   * @param request - The request {@link CreateSecurityGroupRequest}
   * @returns A Promise of CreateSecurityGroupResponse
   */
  createSecurityGroup = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSecurityGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups`
  }, unmarshalCreateSecurityGroupResponse);

  /**
   * Get a security group. Get the details of a security group with the
   * specified ID.
   *
   * @param request - The request {@link GetSecurityGroupRequest}
   * @returns A Promise of GetSecurityGroupResponse
   */
  getSecurityGroup = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}`
  }, unmarshalGetSecurityGroupResponse);

  /**
   * Delete a security group. Delete a security group with the specified ID.
   *
   * @param request - The request {@link DeleteSecurityGroupRequest}
   */
  deleteSecurityGroup = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}`
  });
  _setSecurityGroup = request => this.client.fetch({
    body: JSON.stringify(marshalSetSecurityGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('id', request.id)}`
  }, unmarshalSetSecurityGroupResponse);

  /**
   * Update a security group. Update the properties of security group.
   *
   * @param request - The request {@link UpdateSecurityGroupRequest}
   * @returns A Promise of UpdateSecurityGroupResponse
   */
  updateSecurityGroup = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSecurityGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}`
  }, unmarshalUpdateSecurityGroupResponse);

  /**
   * Get default rules. Lists the default rules applied to all the security
   * groups.
   *
   * @param request - The request {@link ListDefaultSecurityGroupRulesRequest}
   * @returns A Promise of ListSecurityGroupRulesResponse
   */
  listDefaultSecurityGroupRules = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/default/rules`
  }, unmarshalListSecurityGroupRulesResponse);
  pageOfListSecurityGroupRules = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules`,
    urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize])
  }, unmarshalListSecurityGroupRulesResponse);

  /**
   * List rules. List the rules of the a specified security group ID.
   *
   * @param request - The request {@link ListSecurityGroupRulesRequest}
   * @returns A Promise of ListSecurityGroupRulesResponse
   */
  listSecurityGroupRules = request => enrichForPagination('rules', this.pageOfListSecurityGroupRules, request);

  /**
   * Create rule. Create a rule in the specified security group ID.
   *
   * @param request - The request {@link CreateSecurityGroupRuleRequest}
   * @returns A Promise of CreateSecurityGroupRuleResponse
   */
  createSecurityGroupRule = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSecurityGroupRuleRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules`
  }, unmarshalCreateSecurityGroupRuleResponse);

  /**
   * Update all the rules of a security group. Replaces the existing rules of
   * the security group with the rules provided. This endpoint supports the
   * update of existing rules, creation of new rules and deletion of existing
   * rules when they are not passed in the request.
   *
   * @param request - The request {@link SetSecurityGroupRulesRequest}
   * @returns A Promise of SetSecurityGroupRulesResponse
   */
  setSecurityGroupRules = request => this.client.fetch({
    body: JSON.stringify(marshalSetSecurityGroupRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules`
  }, unmarshalSetSecurityGroupRulesResponse);

  /**
   * Delete rule. Delete a security group rule with the specified ID.
   *
   * @param request - The request {@link DeleteSecurityGroupRuleRequest}
   */
  deleteSecurityGroupRule = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules/${validatePathParam('securityGroupRuleId', request.securityGroupRuleId)}`
  });

  /**
   * Get rule. Get details of a security group rule with the specified ID.
   *
   * @param request - The request {@link GetSecurityGroupRuleRequest}
   * @returns A Promise of GetSecurityGroupRuleResponse
   */
  getSecurityGroupRule = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules/${validatePathParam('securityGroupRuleId', request.securityGroupRuleId)}`
  }, unmarshalGetSecurityGroupRuleResponse);
  _setSecurityGroupRule = request => this.client.fetch({
    body: JSON.stringify(marshalSetSecurityGroupRuleRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules/${validatePathParam('securityGroupRuleId', request.securityGroupRuleId)}`
  }, unmarshalSetSecurityGroupRuleResponse);

  /**
   * Update security group rule. Update the properties of a rule from a
   * specified security group.
   *
   * @param request - The request {@link UpdateSecurityGroupRuleRequest}
   * @returns A Promise of UpdateSecurityGroupRuleResponse
   */
  updateSecurityGroupRule = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSecurityGroupRuleRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/security_groups/${validatePathParam('securityGroupId', request.securityGroupId)}/rules/${validatePathParam('securityGroupRuleId', request.securityGroupRuleId)}`
  }, unmarshalUpdateSecurityGroupRuleResponse);
  pageOfListPlacementGroups = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups`,
    urlParams: urlParams(['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['project', request.project], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined])
  }, unmarshalListPlacementGroupsResponse);

  /**
   * List placement groups. List all placement groups in a specified
   * Availability Zone.
   *
   * @param request - The request {@link ListPlacementGroupsRequest}
   * @returns A Promise of ListPlacementGroupsResponse
   */
  listPlacementGroups = (request = {}) => enrichForPagination('placementGroups', this.pageOfListPlacementGroups, request);

  /**
   * Create a placement group. Create a new placement group in a specified
   * Availability Zone.
   *
   * @param request - The request {@link CreatePlacementGroupRequest}
   * @returns A Promise of CreatePlacementGroupResponse
   */
  createPlacementGroup = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreatePlacementGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups`
  }, unmarshalCreatePlacementGroupResponse);

  /**
   * Get a placement group. Get the specified placement group.
   *
   * @param request - The request {@link GetPlacementGroupRequest}
   * @returns A Promise of GetPlacementGroupResponse
   */
  getPlacementGroup = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}`
  }, unmarshalGetPlacementGroupResponse);

  /**
   * Set placement group. Set all parameters of the specified placement group.
   *
   * @param request - The request {@link SetPlacementGroupRequest}
   * @returns A Promise of SetPlacementGroupResponse
   */
  setPlacementGroup = request => this.client.fetch({
    body: JSON.stringify(marshalSetPlacementGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}`
  }, unmarshalSetPlacementGroupResponse);

  /**
   * Update a placement group. Update one or more parameter of the specified
   * placement group.
   *
   * @param request - The request {@link UpdatePlacementGroupRequest}
   * @returns A Promise of UpdatePlacementGroupResponse
   */
  updatePlacementGroup = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePlacementGroupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}`
  }, unmarshalUpdatePlacementGroupResponse);

  /**
   * Delete the specified placement group.
   *
   * @param request - The request {@link DeletePlacementGroupRequest}
   */
  deletePlacementGroup = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}`
  });

  /**
   * Get placement group servers. Get all Instances belonging to the specified
   * placement group.
   *
   * @param request - The request {@link GetPlacementGroupServersRequest}
   * @returns A Promise of GetPlacementGroupServersResponse
   */
  getPlacementGroupServers = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}/servers`
  }, unmarshalGetPlacementGroupServersResponse);

  /**
   * Set placement group servers. Set all Instances belonging to the specified
   * placement group.
   *
   * @param request - The request {@link SetPlacementGroupServersRequest}
   * @returns A Promise of SetPlacementGroupServersResponse
   */
  setPlacementGroupServers = request => this.client.fetch({
    body: JSON.stringify(marshalSetPlacementGroupServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PUT',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}/servers`
  }, unmarshalSetPlacementGroupServersResponse);

  /**
   * Update placement group servers. Update all Instances belonging to the
   * specified placement group.
   *
   * @param request - The request {@link UpdatePlacementGroupServersRequest}
   * @returns A Promise of UpdatePlacementGroupServersResponse
   */
  updatePlacementGroupServers = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePlacementGroupServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/placement_groups/${validatePathParam('placementGroupId', request.placementGroupId)}/servers`
  }, unmarshalUpdatePlacementGroupServersResponse);
  pageOfListIps = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips`,
    urlParams: urlParams(['name', request.name], ['organization', request.organization], ['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['project', request.project], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined], ['type', request.type])
  }, unmarshalListIpsResponse$1);

  /**
   * List all flexible IPs. List all flexible IPs in a specified zone.
   *
   * @param request - The request {@link ListIpsRequest}
   * @returns A Promise of ListIpsResponse
   */
  listIps = (request = {}) => enrichForPagination('ips', this.pageOfListIps, request);

  /**
   * Reserve a flexible IP. Reserve a flexible IP and attach it to the specified
   * Instance.
   *
   * @param request - The request {@link CreateIpRequest}
   * @returns A Promise of CreateIpResponse
   */
  createIp = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateIpRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips`
  }, unmarshalCreateIpResponse);

  /**
   * Get a flexible IP. Get details of an IP with the specified ID or address.
   *
   * @param request - The request {@link GetIpRequest}
   * @returns A Promise of GetIpResponse
   */
  getIp = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ip', request.ip)}`
  }, unmarshalGetIpResponse);

  /**
   * Update a flexible IP. Update a flexible IP in the specified zone with the
   * specified ID.
   *
   * @param request - The request {@link UpdateIpRequest}
   * @returns A Promise of UpdateIpResponse
   */
  updateIp = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateIpRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ip', request.ip)}`
  }, unmarshalUpdateIpResponse);

  /**
   * Delete a flexible IP. Delete the IP with the specified ID.
   *
   * @param request - The request {@link DeleteIpRequest}
   */
  deleteIp = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ip', request.ip)}`
  });
  pageOfListPrivateNICs = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics`,
    urlParams: urlParams(['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['tags', request.tags && request.tags.length > 0 ? request.tags.join(',') : undefined])
  }, unmarshalListPrivateNICsResponse);

  /**
   * List all private NICs. List all private NICs of a specified Instance.
   *
   * @param request - The request {@link ListPrivateNICsRequest}
   * @returns A Promise of ListPrivateNICsResponse
   */
  listPrivateNICs = request => enrichForPagination('privateNics', this.pageOfListPrivateNICs, request);

  /**
   * Create a private NIC connecting an Instance to a Private Network.
   *
   * @param request - The request {@link CreatePrivateNICRequest}
   * @returns A Promise of CreatePrivateNICResponse
   */
  createPrivateNIC = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePrivateNICRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics`
  }, unmarshalCreatePrivateNICResponse);

  /**
   * Get a private NIC. Get private NIC properties.
   *
   * @param request - The request {@link GetPrivateNICRequest}
   * @returns A Promise of GetPrivateNICResponse
   */
  getPrivateNIC = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics/${validatePathParam('privateNicId', request.privateNicId)}`
  }, unmarshalGetPrivateNICResponse);

  /**
   * Update a private NIC. Update one or more parameter(s) of a specified
   * private NIC.
   *
   * @param request - The request {@link UpdatePrivateNICRequest}
   * @returns A Promise of PrivateNIC
   */
  updatePrivateNIC = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePrivateNICRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics/${validatePathParam('privateNicId', request.privateNicId)}`
  }, unmarshalPrivateNIC);

  /**
   * Delete a private NIC.
   *
   * @param request - The request {@link DeletePrivateNICRequest}
   */
  deletePrivateNIC = request => this.client.fetch({
    method: 'DELETE',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/private_nics/${validatePathParam('privateNicId', request.privateNicId)}`
  });
  pageOfListBootscripts = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/bootscripts`,
    urlParams: urlParams(['arch', request.arch], ['default', request.default], ['page', request.page], ['per_page', request.perPage ?? this.client.settings.defaultPageSize], ['public', request.public], ['title', request.title])
  }, unmarshalListBootscriptsResponse);

  /**
   * List bootscripts.
   *
   * @deprecated
   * @param request - The request {@link ListBootscriptsRequest}
   * @returns A Promise of ListBootscriptsResponse
   */
  listBootscripts = (request = {}) => enrichForPagination('bootscripts', this.pageOfListBootscripts, request);

  /**
   * Get bootscripts. Get details of a bootscript with the specified ID.
   *
   * @deprecated
   * @param request - The request {@link GetBootscriptRequest}
   * @returns A Promise of GetBootscriptResponse
   */
  getBootscript = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/bootscripts/${validatePathParam('bootscriptId', request.bootscriptId)}`
  }, unmarshalGetBootscriptResponse);
  getDashboard = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dashboard`,
    urlParams: urlParams(['organization', request.organization], ['project', request.project])
  }, unmarshalGetDashboardResponse);

  /**
   * Get a volume or snapshot's migration plan. Given a volume or snapshot,
   * returns the migration plan for a call to the "Apply a migration plan"
   * endpoint. This plan will include zero or one volume, and zero or more
   * snapshots, which will need to be migrated together. This endpoint does not
   * perform the actual migration itself, the "Apply a migration plan" endpoint
   * must be used. The validation_key value returned by this endpoint must be
   * provided to the call to the "Apply a migration plan" endpoint to confirm
   * that all resources listed in the plan should be migrated.
   *
   * @param request - The request {@link PlanBlockMigrationRequest}
   * @returns A Promise of MigrationPlan
   */
  planBlockMigration = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalPlanBlockMigrationRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/block-migration/plan`
  }, unmarshalMigrationPlan);

  /**
   * Migrate a volume and/or snapshots to SBS (Scaleway Block Storage). To be
   * used, the call to this endpoint must be preceded by a call to the "Plan a
   * migration" endpoint. To migrate all resources mentioned in the migration
   * plan, the validation_key returned in the plan must be provided.
   *
   * @param request - The request {@link ApplyBlockMigrationRequest}
   */
  applyBlockMigration = request => this.client.fetch({
    body: JSON.stringify(marshalApplyBlockMigrationRequest(request, this.client.settings)),
    headers: jsonContentHeaders$k,
    method: 'POST',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/block-migration/apply`
  });
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link ImageState}. */
const IMAGE_TRANSIENT_STATUSES$1 = ['creating'];

/** Lists transient statutes of the enum {@link IpState}. */
const IP_TRANSIENT_STATUSES = ['pending'];

/** Lists transient statutes of the enum {@link PrivateNICState}. */
const PRIVATE_NIC_TRANSIENT_STATUSES = ['syncing'];

/** Lists transient statutes of the enum {@link SecurityGroupState}. */
const SECURITY_GROUP_TRANSIENT_STATUSES = ['syncing'];

/** Lists transient statutes of the enum {@link ServerIpState}. */
const SERVER_IP_TRANSIENT_STATUSES = ['pending'];

/** Lists transient statutes of the enum {@link ServerState}. */
const SERVER_TRANSIENT_STATUSES = ['starting', 'stopping'];

/** Lists transient statutes of the enum {@link SnapshotState}. */
const SNAPSHOT_TRANSIENT_STATUSES$1 = ['snapshotting', 'importing', 'exporting'];

/** Lists transient statutes of the enum {@link TaskStatus}. */
const TASK_TRANSIENT_STATUSES = ['pending', 'started', 'retry'];

/** Lists transient statutes of the enum {@link VolumeServerState}. */
const VOLUME_SERVER_TRANSIENT_STATUSES = ['snapshotting', 'fetching', 'resizing', 'saving', 'hotsyncing'];

/** Lists transient statutes of the enum {@link VolumeState}. */
const VOLUME_TRANSIENT_STATUSES = ['snapshotting', 'fetching', 'resizing', 'saving', 'hotsyncing'];

const validateNotUndefined = obj => {
  if (obj === undefined) throw new TypeError(`object was found undefined`);
  return obj;
};
class InstanceV1UtilsAPI extends API$k {
  /**
   * Waits for {@link Image} to be in a final state.
   *
   * @param request - The request {@link GetImageRequest}
   * @param options - The waiting options
   * @returns A Promise of Image
   */
  waitForImage = (request, options) => tryAtIntervals(async () => {
    const value = await this.getImage(request).then(res => res.image);
    return {
      done: !IMAGE_TRANSIENT_STATUSES$1.includes(value.state),
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);

  /**
   * Waits for {@link PrivateNIC} to be in a final state.
   *
   * @param request - The request {@link GetPrivateNICRequest}
   * @param options - The waiting options
   * @returns A Promise of PrivateNIC
   */
  waitForPrivateNIC = (request, options) => tryAtIntervals(async () => {
    const value = await this.getPrivateNIC(request).then(res => res.privateNic);
    return {
      done: !PRIVATE_NIC_TRANSIENT_STATUSES.includes(value.state),
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);

  /**
   * Waits for {@link Server} to be in a final state.
   *
   * @param request - The request {@link GetServerRequest}
   * @param options - The waiting options
   * @returns A Promise of Server
   */
  waitForServer = (request, options) => tryAtIntervals(async () => {
    const value = await this.getServer(request).then(res => res.server);
    return {
      done: !SERVER_TRANSIENT_STATUSES.includes(value.state),
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);

  /**
   * Waits for {@link Server} to be in a final state.
   *
   * @param request - The request {@link GetServerRequest}
   * @param options - The waiting options
   * @returns A Promise of Server
   */
  waitForSnapshot = (request, options) => tryAtIntervals(async () => {
    const value = await this.getSnapshot(request).then(res => res.snapshot);
    return {
      done: !SNAPSHOT_TRANSIENT_STATUSES$1.includes(value.state),
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);

  /**
   * Waits for {@link Volume} to be in a final state.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @param options - The waiting options
   * @returns A Promise of Volume
   */
  waitForVolume = (request, options) => tryAtIntervals(async () => {
    const value = await this.getVolume(request).then(res => res.volume);
    return {
      done: !VOLUME_TRANSIENT_STATUSES.includes(value.state),
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);

  /**
   * Updates a server.
   *
   * @param request - The request {@link UpdateServerRequest}
   * @returns A Promise of UpdateServerResponse
   */
  updateServer = request => this._updateServer(request);

  /**
   * Creates a server.
   *
   * @param request - The request {@link CreateServerRequest}
   * @returns A Promise of CreateServerResponse
   */
  createServer = request => this._createServer(request);

  /**
   * Starts an action and wait for the server to be in the correct "terminal
   * state" expected by this action.
   *
   * @param request - The request {@link ServerActionRequest}
   * @returns A Promise of Server
   */
  serverActionAndWait = async (request, options) => {
    const finalServer = await this.serverAction(request).then(() => this.waitForServer({
      serverId: request.serverId,
      zone: request.zone
    }, options));

    // Check the action was properly executed.
    let expectedState;
    switch (request.action) {
      case 'poweron':
      case 'reboot':
        expectedState = 'running';
        break;
      case 'poweroff':
        expectedState = 'stopped';
        break;
      case 'stop_in_place':
        expectedState = 'stopped in place';
        break;
    }
    if (expectedState && finalServer.state !== expectedState) {
      throw new Error(`expected state ${expectedState} but found ${finalServer.state}: ${finalServer.stateDetail}`);
    }
    return finalServer;
  };

  /**
   * Attaches a volume to a server.
   *
   * @param request - The request {@link AttachVolumeRequest}
   * @returns A Promise of AttachVolumeResponse
   */
  attachVolume = async request => {
    const volumes = await this.getServer({
      serverId: request.serverId,
      zone: request.zone
    }).then(res => validateNotUndefined(res.server?.volumes));
    const newVolumes = {};
    for (const [key, server] of Object.entries(volumes)) {
      newVolumes[key] = {
        id: server.id,
        name: server.name
      };
    }

    // We loop through all the possible volume keys (0 to len(volumes))
    // to find a non existing key and assign it to the requested volume.
    // A key should always be found. However we return an error if no keys were found.
    let found = false;
    const volumesLength = Object.keys(volumes).length;
    for (let index = 0; index <= volumesLength; index += 1) {
      const key = index.toString();
      if (!(key in newVolumes)) {
        newVolumes[key] = {
          id: request.volumeId,
          name: request.volumeId // name is ignored on this PATCH
        };
        found = true;
        break;
      }
    }
    if (!found) {
      throw new Error(`could not find key to attach volume ${request.volumeId}`);
    }

    // Update server
    return this.updateServer({
      serverId: request.serverId,
      volumes: newVolumes,
      zone: request.zone
    }).then(obj => obj);
  };

  /**
   * Detaches a volume from a server.
   *
   * @param request - The request {@link DetachVolumeRequest}
   * @returns A Promise of DetachVolumeResponse
   */
  detachVolume = async request => {
    // Get server and related volumes.
    const server = await this.getVolume({
      volumeId: request.volumeId,
      zone: request.zone
    }).then(res => validateNotUndefined(res.volume?.server?.id)).then(serverId => this.getServer({
      serverId,
      zone: request.zone
    })).then(res => validateNotUndefined(res.server));

    // Remove volume.
    const newVolumes = {};
    for (const [key, volume] of Object.entries(server.volumes)) {
      if (volume.id !== request.volumeId) {
        newVolumes[key] = {
          id: volume.id,
          name: volume.name
        };
      }
    }

    // Update server.
    return this.updateServer({
      serverId: server.id,
      volumes: newVolumes,
      zone: request.zone
    }).then(obj => obj);
  };

  /**
   * Get the content of a user data on a server for the given key.
   *
   * @param request - The request {@link GetServerUserDataRequest}
   * @returns The content of the key
   */
  getServerUserData = request => this.client.fetch({
    method: 'GET',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/user_data/${validatePathParam('key', request.key)}`
  });

  /**
   * Sets the content of a user data on a server for the given key.
   *
   * @param request - The request {@link SetServerUserDataRequest}
   */
  setServerUserData = request => this.client.fetch({
    body: request.content,
    headers: {
      'Content-Type': 'text/plain'
    },
    method: 'PATCH',
    path: `/instance/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/servers/${validatePathParam('serverId', request.serverId)}/user_data/${validatePathParam('key', request.key)}`
  });
}

var index$m = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: InstanceV1UtilsAPI,
  IMAGE_TRANSIENT_STATUSES: IMAGE_TRANSIENT_STATUSES$1,
  IP_TRANSIENT_STATUSES: IP_TRANSIENT_STATUSES,
  PRIVATE_NIC_TRANSIENT_STATUSES: PRIVATE_NIC_TRANSIENT_STATUSES,
  SECURITY_GROUP_TRANSIENT_STATUSES: SECURITY_GROUP_TRANSIENT_STATUSES,
  SERVER_IP_TRANSIENT_STATUSES: SERVER_IP_TRANSIENT_STATUSES,
  SERVER_TRANSIENT_STATUSES: SERVER_TRANSIENT_STATUSES,
  SNAPSHOT_TRANSIENT_STATUSES: SNAPSHOT_TRANSIENT_STATUSES$1,
  TASK_TRANSIENT_STATUSES: TASK_TRANSIENT_STATUSES,
  VOLUME_SERVER_TRANSIENT_STATUSES: VOLUME_SERVER_TRANSIENT_STATUSES,
  VOLUME_TRANSIENT_STATUSES: VOLUME_TRANSIENT_STATUSES
});

var index$l = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index$m
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link HubStatus}. */
const HUB_TRANSIENT_STATUSES = ['enabling', 'disabling'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalDeviceMessageFiltersRule = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeviceMessageFiltersRule' failed as data isn't a dictionary.`);
  }
  return {
    policy: data.policy,
    topics: data.topics
  };
};
const unmarshalDeviceMessageFilters = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeviceMessageFilters' failed as data isn't a dictionary.`);
  }
  return {
    publish: data.publish ? unmarshalDeviceMessageFiltersRule(data.publish) : undefined,
    subscribe: data.subscribe ? unmarshalDeviceMessageFiltersRule(data.subscribe) : undefined
  };
};
const unmarshalDevice = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Device' failed as data isn't a dictionary.`);
  }
  return {
    allowInsecure: data.allow_insecure,
    allowMultipleConnections: data.allow_multiple_connections,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    hasCustomCertificate: data.has_custom_certificate,
    hubId: data.hub_id,
    id: data.id,
    isConnected: data.is_connected,
    lastActivityAt: unmarshalDate(data.last_activity_at),
    messageFilters: data.message_filters ? unmarshalDeviceMessageFilters(data.message_filters) : undefined,
    name: data.name,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalNetwork = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Network' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    endpoint: data.endpoint,
    hubId: data.hub_id,
    id: data.id,
    name: data.name,
    topicPrefix: data.topic_prefix,
    type: data.type
  };
};
const unmarshalHubTwinsGraphiteConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HubTwinsGraphiteConfig' failed as data isn't a dictionary.`);
  }
  return {
    pushUri: data.push_uri
  };
};
const unmarshalHub = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Hub' failed as data isn't a dictionary.`);
  }
  return {
    connectedDeviceCount: data.connected_device_count,
    createdAt: unmarshalDate(data.created_at),
    deviceCount: data.device_count,
    disableEvents: data.disable_events,
    enableDeviceAutoProvisioning: data.enable_device_auto_provisioning,
    enabled: data.enabled,
    endpoint: data.endpoint,
    eventsTopicPrefix: data.events_topic_prefix,
    hasCustomCa: data.has_custom_ca,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    productPlan: data.product_plan,
    projectId: data.project_id,
    region: data.region,
    status: data.status,
    twinsGraphiteConfig: data.twins_graphite_config ? unmarshalHubTwinsGraphiteConfig(data.twins_graphite_config) : undefined,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalCertificate$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Certificate' failed as data isn't a dictionary.`);
  }
  return {
    crt: data.crt,
    key: data.key
  };
};
const unmarshalCreateDeviceResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateDeviceResponse' failed as data isn't a dictionary.`);
  }
  return {
    certificate: data.certificate ? unmarshalCertificate$1(data.certificate) : undefined,
    device: data.device ? unmarshalDevice(data.device) : undefined
  };
};
const unmarshalCreateNetworkResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateNetworkResponse' failed as data isn't a dictionary.`);
  }
  return {
    network: data.network ? unmarshalNetwork(data.network) : undefined,
    secret: data.secret
  };
};
const unmarshalGetDeviceCertificateResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetDeviceCertificateResponse' failed as data isn't a dictionary.`);
  }
  return {
    certificatePem: data.certificate_pem,
    device: data.device ? unmarshalDevice(data.device) : undefined
  };
};
const unmarshalGetDeviceMetricsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetDeviceMetricsResponse' failed as data isn't a dictionary.`);
  }
  return {
    metrics: unmarshalArrayOfObject(data.metrics, unmarshalTimeSeries)
  };
};
const unmarshalGetHubCAResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetHubCAResponse' failed as data isn't a dictionary.`);
  }
  return {
    caCertPem: data.ca_cert_pem
  };
};
const unmarshalGetHubMetricsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GetHubMetricsResponse' failed as data isn't a dictionary.`);
  }
  return {
    metrics: unmarshalArrayOfObject(data.metrics, unmarshalTimeSeries)
  };
};
const unmarshalListDevicesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDevicesResponse' failed as data isn't a dictionary.`);
  }
  return {
    devices: unmarshalArrayOfObject(data.devices, unmarshalDevice),
    totalCount: data.total_count
  };
};
const unmarshalListHubsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListHubsResponse' failed as data isn't a dictionary.`);
  }
  return {
    hubs: unmarshalArrayOfObject(data.hubs, unmarshalHub),
    totalCount: data.total_count
  };
};
const unmarshalListNetworksResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNetworksResponse' failed as data isn't a dictionary.`);
  }
  return {
    networks: unmarshalArrayOfObject(data.networks, unmarshalNetwork),
    totalCount: data.total_count
  };
};
const unmarshalRouteSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RouteSummary' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    hubId: data.hub_id,
    id: data.id,
    name: data.name,
    topic: data.topic,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListRoutesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRoutesResponse' failed as data isn't a dictionary.`);
  }
  return {
    routes: unmarshalArrayOfObject(data.routes, unmarshalRouteSummary),
    totalCount: data.total_count
  };
};
const unmarshalListTwinDocumentsResponseDocumentSummary = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTwinDocumentsResponseDocumentSummary' failed as data isn't a dictionary.`);
  }
  return {
    documentName: data.document_name
  };
};
const unmarshalListTwinDocumentsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTwinDocumentsResponse' failed as data isn't a dictionary.`);
  }
  return {
    documents: unmarshalArrayOfObject(data.documents, unmarshalListTwinDocumentsResponseDocumentSummary)
  };
};
const unmarshalRenewDeviceCertificateResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RenewDeviceCertificateResponse' failed as data isn't a dictionary.`);
  }
  return {
    certificate: data.certificate ? unmarshalCertificate$1(data.certificate) : undefined,
    device: data.device ? unmarshalDevice(data.device) : undefined
  };
};
const unmarshalRouteDatabaseConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RouteDatabaseConfig' failed as data isn't a dictionary.`);
  }
  return {
    dbname: data.dbname,
    engine: data.engine,
    host: data.host,
    password: data.password,
    port: data.port,
    query: data.query,
    username: data.username
  };
};
const unmarshalRouteRestConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RouteRestConfig' failed as data isn't a dictionary.`);
  }
  return {
    headers: data.headers,
    uri: data.uri,
    verb: data.verb
  };
};
const unmarshalRouteS3Config = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RouteS3Config' failed as data isn't a dictionary.`);
  }
  return {
    bucketName: data.bucket_name,
    bucketRegion: data.bucket_region,
    objectPrefix: data.object_prefix,
    strategy: data.strategy
  };
};
const unmarshalRoute$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Route' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    dbConfig: data.db_config ? unmarshalRouteDatabaseConfig(data.db_config) : undefined,
    hubId: data.hub_id,
    id: data.id,
    name: data.name,
    restConfig: data.rest_config ? unmarshalRouteRestConfig(data.rest_config) : undefined,
    s3Config: data.s3_config ? unmarshalRouteS3Config(data.s3_config) : undefined,
    topic: data.topic,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalSetDeviceCertificateResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetDeviceCertificateResponse' failed as data isn't a dictionary.`);
  }
  return {
    certificatePem: data.certificate_pem,
    device: data.device ? unmarshalDevice(data.device) : undefined
  };
};
const unmarshalTwinDocument = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TwinDocument' failed as data isn't a dictionary.`);
  }
  return {
    data: data.data,
    documentName: data.document_name,
    twinId: data.twin_id,
    version: data.version
  };
};
const marshalDeviceMessageFiltersRule = (request, defaults) => ({
  policy: request.policy,
  topics: request.topics
});
const marshalDeviceMessageFilters = (request, defaults) => ({
  publish: request.publish !== undefined ? marshalDeviceMessageFiltersRule(request.publish) : undefined,
  subscribe: request.subscribe !== undefined ? marshalDeviceMessageFiltersRule(request.subscribe) : undefined
});
const marshalCreateDeviceRequest = (request, defaults) => ({
  allow_insecure: request.allowInsecure,
  allow_multiple_connections: request.allowMultipleConnections,
  description: request.description,
  hub_id: request.hubId,
  message_filters: request.messageFilters !== undefined ? marshalDeviceMessageFilters(request.messageFilters) : undefined,
  name: request.name || randomName('device')
});
const marshalHubTwinsGraphiteConfig = (request, defaults) => ({
  push_uri: request.pushUri
});
const marshalCreateHubRequest = (request, defaults) => ({
  disable_events: request.disableEvents,
  events_topic_prefix: request.eventsTopicPrefix,
  name: request.name || randomName('hub'),
  product_plan: request.productPlan,
  project_id: request.projectId ?? defaults.defaultProjectId,
  ...resolveOneOf([{
    param: 'twins_graphite_config',
    value: request.twinsGraphiteConfig !== undefined ? marshalHubTwinsGraphiteConfig(request.twinsGraphiteConfig) : undefined
  }])
});
const marshalCreateNetworkRequest = (request, defaults) => ({
  hub_id: request.hubId,
  name: request.name || randomName('network'),
  topic_prefix: request.topicPrefix,
  type: request.type
});
const marshalCreateRouteRequestDatabaseConfig = (request, defaults) => ({
  dbname: request.dbname,
  engine: request.engine,
  host: request.host,
  password: request.password,
  port: request.port,
  query: request.query,
  username: request.username
});
const marshalCreateRouteRequestRestConfig = (request, defaults) => ({
  headers: request.headers,
  uri: request.uri,
  verb: request.verb
});
const marshalCreateRouteRequestS3Config = (request, defaults) => ({
  bucket_name: request.bucketName,
  bucket_region: request.bucketRegion,
  object_prefix: request.objectPrefix,
  strategy: request.strategy
});
const marshalCreateRouteRequest$1 = (request, defaults) => ({
  hub_id: request.hubId,
  name: request.name || randomName('route'),
  topic: request.topic,
  ...resolveOneOf([{
    param: 's3_config',
    value: request.s3Config !== undefined ? marshalCreateRouteRequestS3Config(request.s3Config) : undefined
  }, {
    param: 'db_config',
    value: request.dbConfig !== undefined ? marshalCreateRouteRequestDatabaseConfig(request.dbConfig) : undefined
  }, {
    param: 'rest_config',
    value: request.restConfig !== undefined ? marshalCreateRouteRequestRestConfig(request.restConfig) : undefined
  }])
});
const marshalPatchTwinDocumentRequest = (request, defaults) => ({
  data: request.data,
  version: request.version
});
const marshalPutTwinDocumentRequest = (request, defaults) => ({
  data: request.data,
  version: request.version
});
const marshalSetDeviceCertificateRequest = (request, defaults) => ({
  certificate_pem: request.certificatePem
});
const marshalSetHubCARequest = (request, defaults) => ({
  ca_cert_pem: request.caCertPem,
  challenge_cert_pem: request.challengeCertPem
});
const marshalUpdateDeviceRequest = (request, defaults) => ({
  allow_insecure: request.allowInsecure,
  allow_multiple_connections: request.allowMultipleConnections,
  description: request.description,
  hub_id: request.hubId,
  message_filters: request.messageFilters !== undefined ? marshalDeviceMessageFilters(request.messageFilters) : undefined
});
const marshalUpdateHubRequest = (request, defaults) => ({
  disable_events: request.disableEvents,
  enable_device_auto_provisioning: request.enableDeviceAutoProvisioning,
  events_topic_prefix: request.eventsTopicPrefix,
  name: request.name,
  product_plan: request.productPlan,
  ...resolveOneOf([{
    param: 'twins_graphite_config',
    value: request.twinsGraphiteConfig !== undefined ? marshalHubTwinsGraphiteConfig(request.twinsGraphiteConfig) : undefined
  }])
});
const marshalUpdateRouteRequestDatabaseConfig = (request, defaults) => ({
  dbname: request.dbname,
  engine: request.engine,
  host: request.host,
  password: request.password,
  port: request.port,
  query: request.query,
  username: request.username
});
const marshalUpdateRouteRequestRestConfig = (request, defaults) => ({
  headers: request.headers,
  uri: request.uri,
  verb: request.verb
});
const marshalUpdateRouteRequestS3Config = (request, defaults) => ({
  bucket_name: request.bucketName,
  bucket_region: request.bucketRegion,
  object_prefix: request.objectPrefix,
  strategy: request.strategy
});
const marshalUpdateRouteRequest$1 = (request, defaults) => ({
  name: request.name,
  topic: request.topic,
  ...resolveOneOf([{
    param: 's3_config',
    value: request.s3Config !== undefined ? marshalUpdateRouteRequestS3Config(request.s3Config) : undefined
  }, {
    param: 'db_config',
    value: request.dbConfig !== undefined ? marshalUpdateRouteRequestDatabaseConfig(request.dbConfig) : undefined
  }, {
    param: 'rest_config',
    value: request.restConfig !== undefined ? marshalUpdateRouteRequestRestConfig(request.restConfig) : undefined
  }])
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$j = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * IoT Hub API.
 *
 * This API allows you to manage IoT hubs and devices.
 */
let API$j = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par'];
  pageOfListHubs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListHubsResponse);

  /**
   * List hubs. List all Hubs in the specified zone. By default, returned Hubs
   * are ordered by creation date in ascending order, though this can be
   * modified via the `order_by` field.
   *
   * @param request - The request {@link ListHubsRequest}
   * @returns A Promise of ListHubsResponse
   */
  listHubs = (request = {}) => enrichForPagination('hubs', this.pageOfListHubs, request);

  /**
   * Create a hub. Create a new Hub in the targeted region, specifying its
   * configuration including name and product plan.
   *
   * @param request - The request {@link CreateHubRequest}
   * @returns A Promise of Hub
   */
  createHub = request => this.client.fetch({
    body: JSON.stringify(marshalCreateHubRequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs`
  }, unmarshalHub);

  /**
   * Get a hub. Retrieve information about an existing IoT Hub, specified by its
   * Hub ID. Its full details, including name, status and endpoint, are returned
   * in the response object.
   *
   * @param request - The request {@link GetHubRequest}
   * @returns A Promise of Hub
   */
  getHub = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs/${validatePathParam('hubId', request.hubId)}`
  }, unmarshalHub);

  /**
   * Waits for {@link Hub} to be in a final state.
   *
   * @param request - The request {@link GetHubRequest}
   * @param options - The waiting options
   * @returns A Promise of Hub
   */
  waitForHub = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!HUB_TRANSIENT_STATUSES.includes(res.status))), this.getHub, request, options);

  /**
   * Update a hub. Update the parameters of an existing IoT Hub, specified by
   * its Hub ID.
   *
   * @param request - The request {@link UpdateHubRequest}
   * @returns A Promise of Hub
   */
  updateHub = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateHubRequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'PATCH',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs/${validatePathParam('hubId', request.hubId)}`
  }, unmarshalHub);

  /**
   * Enable a hub. Enable an existing IoT Hub, specified by its Hub ID.
   *
   * @param request - The request {@link EnableHubRequest}
   * @returns A Promise of Hub
   */
  enableHub = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs/${validatePathParam('hubId', request.hubId)}/enable`
  }, unmarshalHub);

  /**
   * Disable a hub. Disable an existing IoT Hub, specified by its Hub ID.
   *
   * @param request - The request {@link DisableHubRequest}
   * @returns A Promise of Hub
   */
  disableHub = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs/${validatePathParam('hubId', request.hubId)}/disable`
  }, unmarshalHub);

  /**
   * Delete a hub. Delete an existing IoT Hub, specified by its Hub ID. Deleting
   * a Hub is permanent, and cannot be undone.
   *
   * @param request - The request {@link DeleteHubRequest}
   */
  deleteHub = request => this.client.fetch({
    method: 'DELETE',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs/${validatePathParam('hubId', request.hubId)}`,
    urlParams: urlParams(['delete_devices', request.deleteDevices])
  });

  /**
   * Get a hub's metrics. Get the metrics of an existing IoT Hub, specified by
   * its Hub ID.
   *
   * @deprecated
   * @param request - The request {@link GetHubMetricsRequest}
   * @returns A Promise of GetHubMetricsResponse
   */
  getHubMetrics = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs/${validatePathParam('hubId', request.hubId)}/metrics`,
    urlParams: urlParams(['start_date', request.startDate])
  }, unmarshalGetHubMetricsResponse);

  /**
   * Set the certificate authority of a hub. Set a particular PEM-encoded
   * certificate, specified by the Hub ID.
   *
   * @param request - The request {@link SetHubCARequest}
   * @returns A Promise of Hub
   */
  setHubCA = request => this.client.fetch({
    body: JSON.stringify(marshalSetHubCARequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs/${validatePathParam('hubId', request.hubId)}/ca`
  }, unmarshalHub);

  /**
   * Get the certificate authority of a hub. Get information for a particular
   * PEM-encoded certificate, specified by the Hub ID.
   *
   * @param request - The request {@link GetHubCARequest}
   * @returns A Promise of GetHubCAResponse
   */
  getHubCA = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hubs/${validatePathParam('hubId', request.hubId)}/ca`
  }, unmarshalGetHubCAResponse);
  pageOfListDevices = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices`,
    urlParams: urlParams(['allow_insecure', request.allowInsecure], ['hub_id', request.hubId], ['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['status', request.status])
  }, unmarshalListDevicesResponse);

  /**
   * List devices. List all devices in the specified region. By default,
   * returned devices are ordered by creation date in ascending order, though
   * this can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListDevicesRequest}
   * @returns A Promise of ListDevicesResponse
   */
  listDevices = (request = {}) => enrichForPagination('devices', this.pageOfListDevices, request);

  /**
   * Add a device. Attach a device to a given Hub.
   *
   * @param request - The request {@link CreateDeviceRequest}
   * @returns A Promise of CreateDeviceResponse
   */
  createDevice = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDeviceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices`
  }, unmarshalCreateDeviceResponse);

  /**
   * Get a device. Retrieve information about an existing device, specified by
   * its device ID. Its full details, including name, status and ID, are
   * returned in the response object.
   *
   * @param request - The request {@link GetDeviceRequest}
   * @returns A Promise of Device
   */
  getDevice = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}`
  }, unmarshalDevice);

  /**
   * Update a device. Update the parameters of an existing device, specified by
   * its device ID.
   *
   * @param request - The request {@link UpdateDeviceRequest}
   * @returns A Promise of Device
   */
  updateDevice = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDeviceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'PATCH',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}`
  }, unmarshalDevice);

  /**
   * Enable a device. Enable a specific device, specified by its device ID.
   *
   * @param request - The request {@link EnableDeviceRequest}
   * @returns A Promise of Device
   */
  enableDevice = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}/enable`
  }, unmarshalDevice);

  /**
   * Disable a device. Disable an existing device, specified by its device ID.
   *
   * @param request - The request {@link DisableDeviceRequest}
   * @returns A Promise of Device
   */
  disableDevice = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}/disable`
  }, unmarshalDevice);

  /**
   * Renew a device certificate. Renew the certificate of an existing device,
   * specified by its device ID.
   *
   * @param request - The request {@link RenewDeviceCertificateRequest}
   * @returns A Promise of RenewDeviceCertificateResponse
   */
  renewDeviceCertificate = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}/renew-certificate`
  }, unmarshalRenewDeviceCertificateResponse);

  /**
   * Set a custom certificate on a device. Switch the existing certificate of a
   * given device with an EM-encoded custom certificate.
   *
   * @param request - The request {@link SetDeviceCertificateRequest}
   * @returns A Promise of SetDeviceCertificateResponse
   */
  setDeviceCertificate = request => this.client.fetch({
    body: JSON.stringify(marshalSetDeviceCertificateRequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'PUT',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}/certificate`
  }, unmarshalSetDeviceCertificateResponse);

  /**
   * Get a device's certificate. Get information for a particular PEM-encoded
   * certificate, specified by the device ID. The response returns full details
   * of the device, including its type of certificate.
   *
   * @param request - The request {@link GetDeviceCertificateRequest}
   * @returns A Promise of GetDeviceCertificateResponse
   */
  getDeviceCertificate = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}/certificate`
  }, unmarshalGetDeviceCertificateResponse);

  /**
   * Remove a device. Remove a specific device from the specific Hub it is
   * attached to.
   *
   * @param request - The request {@link DeleteDeviceRequest}
   */
  deleteDevice = request => this.client.fetch({
    method: 'DELETE',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}`
  });

  /**
   * Get a device's metrics. Get the metrics of an existing device, specified by
   * its device ID.
   *
   * @deprecated
   * @param request - The request {@link GetDeviceMetricsRequest}
   * @returns A Promise of GetDeviceMetricsResponse
   */
  getDeviceMetrics = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/devices/${validatePathParam('deviceId', request.deviceId)}/metrics`,
    urlParams: urlParams(['start_date', request.startDate])
  }, unmarshalGetDeviceMetricsResponse);
  pageOfListRoutes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes`,
    urlParams: urlParams(['hub_id', request.hubId], ['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListRoutesResponse$1);

  /**
   * List routes. List all routes in the specified region. By default, returned
   * routes are ordered by creation date in ascending order, though this can be
   * modified via the `order_by` field.
   *
   * @param request - The request {@link ListRoutesRequest}
   * @returns A Promise of ListRoutesResponse
   */
  listRoutes = (request = {}) => enrichForPagination('routes', this.pageOfListRoutes, request);

  /**
   * Create a route. Multiple kinds of routes can be created, such as:
   *
   * - Database Route Create a route that will record subscribed MQTT messages
   *   into your database. <b>You need to manage the database by yourself</b>.
   * - REST Route. Create a route that will call a REST API on received subscribed
   *   MQTT messages.
   * - S3 Routes. Create a route that will put subscribed MQTT messages into an S3
   *   bucket. You need to create the bucket yourself and grant write access.
   *   Granting can be done with s3cmd (`s3cmd setacl s3://<my-bucket>
   *   --acl-grant=write:555c69c3-87d0-4bf8-80f1-99a2f757d031:555c69c3-87d0-4bf8-80f1-99a2f757d031`).
   *
   * @param request - The request {@link CreateRouteRequest}
   * @returns A Promise of Route
   */
  createRoute = request => this.client.fetch({
    body: JSON.stringify(marshalCreateRouteRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes`
  }, unmarshalRoute$1);

  /**
   * Update a route. Update the parameters of an existing route, specified by
   * its route ID.
   *
   * @param request - The request {@link UpdateRouteRequest}
   * @returns A Promise of Route
   */
  updateRoute = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateRouteRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'PATCH',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes/${validatePathParam('routeId', request.routeId)}`
  }, unmarshalRoute$1);

  /**
   * Get a route. Get information for a particular route, specified by the route
   * ID. The response returns full details of the route, including its type, the
   * topic it subscribes to and its configuration.
   *
   * @param request - The request {@link GetRouteRequest}
   * @returns A Promise of Route
   */
  getRoute = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes/${validatePathParam('routeId', request.routeId)}`
  }, unmarshalRoute$1);

  /**
   * Delete a route. Delete an existing route, specified by its route ID.
   * Deleting a route is permanent, and cannot be undone.
   *
   * @param request - The request {@link DeleteRouteRequest}
   */
  deleteRoute = request => this.client.fetch({
    method: 'DELETE',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes/${validatePathParam('routeId', request.routeId)}`
  });
  pageOfListNetworks = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/networks`,
    urlParams: urlParams(['hub_id', request.hubId], ['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['topic_prefix', request.topicPrefix])
  }, unmarshalListNetworksResponse);

  /**
   * List the networks.
   *
   * @param request - The request {@link ListNetworksRequest}
   * @returns A Promise of ListNetworksResponse
   */
  listNetworks = (request = {}) => enrichForPagination('networks', this.pageOfListNetworks, request);

  /**
   * Create a new network. Create a new network for an existing hub. Beside the
   * default network, you can add networks for different data providers.
   * Possible network types are Sigfox and REST.
   *
   * @param request - The request {@link CreateNetworkRequest}
   * @returns A Promise of CreateNetworkResponse
   */
  createNetwork = request => this.client.fetch({
    body: JSON.stringify(marshalCreateNetworkRequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'POST',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/networks`
  }, unmarshalCreateNetworkResponse);

  /**
   * Retrieve a specific network. Retrieve an existing network, specified by its
   * network ID. The response returns full details of the network, including its
   * type, the topic prefix and its endpoint.
   *
   * @param request - The request {@link GetNetworkRequest}
   * @returns A Promise of Network
   */
  getNetwork = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/networks/${validatePathParam('networkId', request.networkId)}`
  }, unmarshalNetwork);

  /**
   * Delete a Network. Delete an existing network, specified by its network ID.
   * Deleting a network is permanent, and cannot be undone.
   *
   * @param request - The request {@link DeleteNetworkRequest}
   */
  deleteNetwork = request => this.client.fetch({
    method: 'DELETE',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/networks/${validatePathParam('networkId', request.networkId)}`
  });

  /**
   * BETA - Get a Cloud Twin Document.
   *
   * @param request - The request {@link GetTwinDocumentRequest}
   * @returns A Promise of TwinDocument
   */
  getTwinDocument = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/twins/${validatePathParam('twinId', request.twinId)}/documents/${validatePathParam('documentName', request.documentName)}`
  }, unmarshalTwinDocument);

  /**
   * BETA - Update a Cloud Twin Document.
   *
   * @param request - The request {@link PutTwinDocumentRequest}
   * @returns A Promise of TwinDocument
   */
  putTwinDocument = request => this.client.fetch({
    body: JSON.stringify(marshalPutTwinDocumentRequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'PUT',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/twins/${validatePathParam('twinId', request.twinId)}/documents/${validatePathParam('documentName', request.documentName)}`
  }, unmarshalTwinDocument);

  /**
   * BETA - Patch a Cloud Twin Document.
   *
   * @param request - The request {@link PatchTwinDocumentRequest}
   * @returns A Promise of TwinDocument
   */
  patchTwinDocument = request => this.client.fetch({
    body: JSON.stringify(marshalPatchTwinDocumentRequest(request, this.client.settings)),
    headers: jsonContentHeaders$j,
    method: 'PATCH',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/twins/${validatePathParam('twinId', request.twinId)}/documents/${validatePathParam('documentName', request.documentName)}`
  }, unmarshalTwinDocument);

  /**
   * BETA - Delete a Cloud Twin Document.
   *
   * @param request - The request {@link DeleteTwinDocumentRequest}
   */
  deleteTwinDocument = request => this.client.fetch({
    method: 'DELETE',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/twins/${validatePathParam('twinId', request.twinId)}/documents/${validatePathParam('documentName', request.documentName)}`
  });

  /**
   * BETA - List the documents of a Cloud Twin.
   *
   * @param request - The request {@link ListTwinDocumentsRequest}
   * @returns A Promise of ListTwinDocumentsResponse
   */
  listTwinDocuments = request => this.client.fetch({
    method: 'GET',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/twins/${validatePathParam('twinId', request.twinId)}`
  }, unmarshalListTwinDocumentsResponse);

  /**
   * BETA - Delete all the documents of a Cloud Twin.
   *
   * @param request - The request {@link DeleteTwinDocumentsRequest}
   */
  deleteTwinDocuments = request => this.client.fetch({
    method: 'DELETE',
    path: `/iot/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/twins/${validatePathParam('twinId', request.twinId)}`
  });
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$j = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$j,
  HUB_TRANSIENT_STATUSES: HUB_TRANSIENT_STATUSES
});

var index$k = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$j
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalResource = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Resource' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    macAddress: data.mac_address,
    name: data.name,
    type: data.type
  };
};
const unmarshalReverse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Reverse' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    hostname: data.hostname
  };
};
const unmarshalSource = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Source' failed as data isn't a dictionary.`);
  }
  return {
    privateNetworkId: data.private_network_id,
    subnetId: data.subnet_id,
    zonal: data.zonal
  };
};
const unmarshalIP$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'IP' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    isIpv6: data.is_ipv6,
    projectId: data.project_id,
    region: data.region,
    resource: data.resource ? unmarshalResource(data.resource) : undefined,
    reverses: unmarshalArrayOfObject(data.reverses, unmarshalReverse),
    source: unmarshalSource(data.source),
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalListIPsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListIPsResponse' failed as data isn't a dictionary.`);
  }
  return {
    ips: unmarshalArrayOfObject(data.ips, unmarshalIP$1),
    totalCount: data.total_count
  };
};
const marshalSource = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'zonal',
    value: request.zonal
  }, {
    param: 'private_network_id',
    value: request.privateNetworkId
  }, {
    param: 'subnet_id',
    value: request.subnetId
  }])
});
const marshalBookIPRequest = (request, defaults) => ({
  address: request.address,
  is_ipv6: request.isIpv6,
  project_id: request.projectId ?? defaults.defaultProjectId,
  source: marshalSource(request.source),
  tags: request.tags
});
const marshalReverse = (request, defaults) => ({
  address: request.address,
  hostname: request.hostname
});
const marshalUpdateIPRequest$1 = (request, defaults) => ({
  reverses: request.reverses !== undefined ? request.reverses.map(elt => marshalReverse(elt)) : undefined,
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$i = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * IPAM API.
 *
 * This API allows you to manage IP addresses with Scaleway's IP Address
 * Management tool.
 */
let API$i = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];

  /**
   * Book a new IP. Book a new IP from the specified source. Currently IPs can
   * only be booked from a Private Network.
   *
   * @param request - The request {@link BookIPRequest}
   * @returns A Promise of IP
   */
  bookIP = request => this.client.fetch({
    body: JSON.stringify(marshalBookIPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$i,
    method: 'POST',
    path: `/ipam/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips`
  }, unmarshalIP$1);

  /**
   * Release an IP. Release an IP not currently attached to a resource, and
   * returns it to the available IP pool.
   *
   * @param request - The request {@link ReleaseIPRequest}
   */
  releaseIP = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$i,
    method: 'DELETE',
    path: `/ipam/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips/${validatePathParam('ipId', request.ipId)}`
  });

  /**
   * Get an IP. Retrieve details of an existing IP, specified by its IP ID.
   *
   * @param request - The request {@link GetIPRequest}
   * @returns A Promise of IP
   */
  getIP = request => this.client.fetch({
    method: 'GET',
    path: `/ipam/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIP$1);

  /**
   * Update an IP. Update parameters including tags of the specified IP.
   *
   * @param request - The request {@link UpdateIPRequest}
   * @returns A Promise of IP
   */
  updateIP = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateIPRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$i,
    method: 'PATCH',
    path: `/ipam/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIP$1);
  pageOfListIPs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/ipam/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips`,
    urlParams: urlParams(['attached', request.attached], ['is_ipv6', request.isIpv6], ['mac_address', request.macAddress], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['resource_id', request.resourceId], ['resource_name', request.resourceName], ['resource_type', request.resourceType], ['tags', request.tags], ...Object.entries(resolveOneOf([{
      param: 'zonal',
      value: request.zonal
    }, {
      param: 'private_network_id',
      value: request.privateNetworkId
    }])))
  }, unmarshalListIPsResponse$1);

  /**
   * List existing IPs. List existing IPs in the specified region using various
   * filters. For example, you can filter for IPs within a specified Private
   * Network, or for public IPs within a specified Project. By default, the IPs
   * returned in the list are ordered by creation date in ascending order,
   * though this can be modified via the order_by field.
   *
   * @param request - The request {@link ListIPsRequest}
   * @returns A Promise of ListIPsResponse
   */
  listIPs = (request = {}) => enrichForPagination('ips', this.pageOfListIPs, request);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const ListIPsRequest = {
  resourceName: {
    maxLength: 63,
    minLength: 2
  }
};

var validationRules_gen$8 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ListIPsRequest: ListIPsRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$i = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$i,
  ValidationRules: validationRules_gen$8
});

var index$j = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$i
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link NameStatus}. */
const NAME_TRANSIENT_STATUSES = ['queued', 'publishing'];

/** Lists transient statutes of the enum {@link PinStatus}. */
const PIN_TRANSIENT_STATUSES = ['queued', 'pinning'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalName = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Name' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    key: data.key,
    name: data.name,
    nameId: data.name_id,
    projectId: data.project_id,
    region: data.region,
    status: data.status,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    value: data.value
  };
};
const unmarshalPinCIDMeta = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PinCIDMeta' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id
  };
};
const unmarshalPinCID = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PinCID' failed as data isn't a dictionary.`);
  }
  return {
    cid: data.cid,
    meta: data.meta ? unmarshalPinCIDMeta(data.meta) : undefined,
    name: data.name,
    origins: data.origins
  };
};
const unmarshalPinInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PinInfo' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    progress: data.progress,
    size: data.size,
    statusDetails: data.status_details,
    url: data.url
  };
};
const unmarshalPin = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Pin' failed as data isn't a dictionary.`);
  }
  return {
    cid: data.cid ? unmarshalPinCID(data.cid) : undefined,
    createdAt: unmarshalDate(data.created_at),
    delegates: data.delegates,
    info: data.info ? unmarshalPinInfo(data.info) : undefined,
    pinId: data.pin_id,
    status: data.status
  };
};
const unmarshalVolume$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Volume' failed as data isn't a dictionary.`);
  }
  return {
    countPin: data.count_pin,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    projectId: data.project_id,
    region: data.region,
    size: data.size,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalExportKeyNameResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ExportKeyNameResponse' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    nameId: data.name_id,
    privateKey: data.private_key,
    projectId: data.project_id,
    publicKey: data.public_key,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListNamesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNamesResponse' failed as data isn't a dictionary.`);
  }
  return {
    names: unmarshalArrayOfObject(data.names, unmarshalName),
    totalCount: data.total_count
  };
};
const unmarshalListPinsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPinsResponse' failed as data isn't a dictionary.`);
  }
  return {
    pins: unmarshalArrayOfObject(data.pins, unmarshalPin),
    totalCount: data.total_count
  };
};
const unmarshalListVolumesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListVolumesResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    volumes: unmarshalArrayOfObject(data.volumes, unmarshalVolume$1)
  };
};
const unmarshalReplacePinResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ReplacePinResponse' failed as data isn't a dictionary.`);
  }
  return {
    pin: data.pin ? unmarshalPin(data.pin) : undefined
  };
};
const marshalCreatePinByCIDRequest = (request, defaults) => ({
  cid: request.cid,
  name: request.name,
  origins: request.origins,
  volume_id: request.volumeId
});
const marshalCreatePinByURLRequest = (request, defaults) => ({
  name: request.name,
  url: request.url,
  volume_id: request.volumeId
});
const marshalCreateVolumeRequest = (request, defaults) => ({
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalIpnsApiCreateNameRequest = (request, defaults) => ({
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId,
  value: request.value
});
const marshalIpnsApiImportKeyNameRequest = (request, defaults) => ({
  name: request.name,
  private_key: request.privateKey,
  project_id: request.projectId ?? defaults.defaultProjectId,
  value: request.value
});
const marshalIpnsApiUpdateNameRequest = (request, defaults) => ({
  name: request.name,
  tags: request.tags,
  value: request.value
});
const marshalReplacePinRequest = (request, defaults) => ({
  cid: request.cid,
  name: request.name,
  origins: request.origins,
  volume_id: request.volumeId
});
const marshalUpdateVolumeRequest = (request, defaults) => ({
  name: request.name,
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$h = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** IPFS Pinning service API. */
let API$h = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];

  /**
   * Create a new volume. Create a new volume from a Project ID. Volume is
   * identified by an ID and used to host pin references. Volume is personal (at
   * least to your organization) even if IPFS blocks and CID are available to
   * anyone. Should be the first command you made because every pin must be
   * attached to a volume.
   *
   * @param request - The request {@link CreateVolumeRequest}
   * @returns A Promise of Volume
   */
  createVolume = request => this.client.fetch({
    body: JSON.stringify(marshalCreateVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders$h,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes`
  }, unmarshalVolume$1);

  /**
   * Get information about a volume. Retrieve information about a specific
   * volume.
   *
   * @param request - The request {@link GetVolumeRequest}
   * @returns A Promise of Volume
   */
  getVolume = request => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalVolume$1);
  pageOfListVolumes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListVolumesResponse);

  /**
   * List all volumes by a Project ID. Retrieve information about all volumes
   * from a Project ID.
   *
   * @param request - The request {@link ListVolumesRequest}
   * @returns A Promise of ListVolumesResponse
   */
  listVolumes = (request = {}) => enrichForPagination('volumes', this.pageOfListVolumes, request);

  /**
   * Update volume information. Update volume information (tag, name...).
   *
   * @param request - The request {@link UpdateVolumeRequest}
   * @returns A Promise of Volume
   */
  updateVolume = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateVolumeRequest(request, this.client.settings)),
    headers: jsonContentHeaders$h,
    method: 'PATCH',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  }, unmarshalVolume$1);

  /**
   * Delete an existing volume. Delete a volume by its ID and every pin attached
   * to this volume. This process can take a while to conclude, depending on the
   * size of your pinned content.
   *
   * @param request - The request {@link DeleteVolumeRequest}
   */
  deleteVolume = request => this.client.fetch({
    method: 'DELETE',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/volumes/${validatePathParam('volumeId', request.volumeId)}`
  });

  /**
   * Create a pin by URL. Will fetch and store the content pointed by the
   * provided URL. The content must be available on the public IPFS network. The
   * content (IPFS blocks) will be host by the pinning service until pin
   * deletion. From that point, any other IPFS peer can fetch and host your
   * content: Make sure to pin public or encrypted content. Many pin requests
   * (from different users) can target the same CID. A pin is defined by its ID
   * (UUID), its status (queued, pinning, pinned or failed) and target CID.
   *
   * @param request - The request {@link CreatePinByURLRequest}
   * @returns A Promise of Pin
   */
  createPinByURL = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePinByURLRequest(request, this.client.settings)),
    headers: jsonContentHeaders$h,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/create-by-url`
  }, unmarshalPin);

  /**
   * Create a pin by CID. Will fetch and store the content pointed by the
   * provided CID. The content must be available on the public IPFS network. The
   * content (IPFS blocks) will be host by the pinning service until pin
   * deletion. From that point, any other IPFS peer can fetch and host your
   * content: Make sure to pin public or encrypted content. Many pin requests
   * (from different users) can target the same CID. A pin is defined by its ID
   * (UUID), its status (queued, pinning, pinned or failed) and target CID.
   *
   * @param request - The request {@link CreatePinByCIDRequest}
   * @returns A Promise of Pin
   */
  createPinByCID = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePinByCIDRequest(request, this.client.settings)),
    headers: jsonContentHeaders$h,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/create-by-cid`
  }, unmarshalPin);

  /**
   * Replace pin by CID. Deletes the given resource ID and pins the new CID in
   * its place. Will fetch and store the content pointed by the provided CID.
   * The content must be available on the public IPFS network. The content (IPFS
   * blocks) is hosted by the pinning service until the pin is deleted. While
   * the content is available any other IPFS peer can fetch and host your
   * content. For this reason, we recommend that you pin either public or
   * encrypted content. Several different pin requests can target the same CID.
   * A pin is defined by its ID (UUID), its status (queued, pinning, pinned or
   * failed) and target CID.
   *
   * @param request - The request {@link ReplacePinRequest}
   * @returns A Promise of ReplacePinResponse
   */
  replacePin = request => this.client.fetch({
    body: JSON.stringify(marshalReplacePinRequest(request, this.client.settings)),
    headers: jsonContentHeaders$h,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/${validatePathParam('pinId', request.pinId)}/replace`
  }, unmarshalReplacePinResponse);

  /**
   * Get pin information. Retrieve information about the provided **pin ID**,
   * such as status, last modification, and CID.
   *
   * @param request - The request {@link GetPinRequest}
   * @returns A Promise of Pin
   */
  getPin = request => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/${validatePathParam('pinId', request.pinId)}`,
    urlParams: urlParams(['volume_id', request.volumeId])
  }, unmarshalPin);

  /**
   * Waits for {@link Pin} to be in a final state.
   *
   * @param request - The request {@link GetPinRequest}
   * @param options - The waiting options
   * @returns A Promise of Pin
   */
  waitForPin = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!PIN_TRANSIENT_STATUSES.includes(res.status))), this.getPin, request, options);
  pageOfListPins = request => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['status', request.status], ['volume_id', request.volumeId])
  }, unmarshalListPinsResponse);

  /**
   * List all pins within a volume. Retrieve information about all pins within a
   * volume.
   *
   * @param request - The request {@link ListPinsRequest}
   * @returns A Promise of ListPinsResponse
   */
  listPins = request => enrichForPagination('pins', this.pageOfListPins, request);

  /**
   * Create an unpin request. An unpin request means that you no longer own the
   * content. This content can therefore be removed and no longer provided on
   * the IPFS network.
   *
   * @param request - The request {@link DeletePinRequest}
   */
  deletePin = request => this.client.fetch({
    method: 'DELETE',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pins/${validatePathParam('pinId', request.pinId)}`,
    urlParams: urlParams(['volume_id', request.volumeId])
  });
};

/** IPFS Naming service API. */
class IpnsAPI extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];

  /**
   * Create a new name. You can use the `ipns key` command to list and generate
   * more names and their respective keys.
   *
   * @param request - The request {@link IpnsApiCreateNameRequest}
   * @returns A Promise of Name
   */
  createName = request => this.client.fetch({
    body: JSON.stringify(marshalIpnsApiCreateNameRequest(request, this.client.settings)),
    headers: jsonContentHeaders$h,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/names`
  }, unmarshalName);

  /**
   * Get information about a name. Retrieve information about a specific name.
   *
   * @param request - The request {@link IpnsApiGetNameRequest}
   * @returns A Promise of Name
   */
  getName = request => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/names/${validatePathParam('nameId', request.nameId)}`
  }, unmarshalName);

  /**
   * Waits for {@link Name} to be in a final state.
   *
   * @param request - The request {@link IpnsApiGetNameRequest}
   * @param options - The waiting options
   * @returns A Promise of Name
   */
  waitForName = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!NAME_TRANSIENT_STATUSES.includes(res.status))), this.getName, request, options);

  /**
   * Delete an existing name. Delete a name by its ID.
   *
   * @param request - The request {@link IpnsApiDeleteNameRequest}
   */
  deleteName = request => this.client.fetch({
    method: 'DELETE',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/names/${validatePathParam('nameId', request.nameId)}`
  });
  pageOfListNames = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/names`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListNamesResponse);

  /**
   * List all names by a Project ID. Retrieve information about all names from a
   * Project ID.
   *
   * @param request - The request {@link IpnsApiListNamesRequest}
   * @returns A Promise of ListNamesResponse
   */
  listNames = (request = {}) => enrichForPagination('names', this.pageOfListNames, request);

  /**
   * Update name information. Update name information (CID, tag, name...).
   *
   * @param request - The request {@link IpnsApiUpdateNameRequest}
   * @returns A Promise of Name
   */
  updateName = request => this.client.fetch({
    body: JSON.stringify(marshalIpnsApiUpdateNameRequest(request, this.client.settings)),
    headers: jsonContentHeaders$h,
    method: 'PATCH',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/names/${validatePathParam('nameId', request.nameId)}`
  }, unmarshalName);

  /**
   * Export your private key. Export a private key by its ID.
   *
   * @param request - The request {@link IpnsApiExportKeyNameRequest}
   * @returns A Promise of ExportKeyNameResponse
   */
  exportKeyName = request => this.client.fetch({
    method: 'GET',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/names/${validatePathParam('nameId', request.nameId)}/export-key`
  }, unmarshalExportKeyNameResponse);

  /**
   * Import your private key. Import a private key.
   *
   * @param request - The request {@link IpnsApiImportKeyNameRequest}
   * @returns A Promise of Name
   */
  importKeyName = request => this.client.fetch({
    body: JSON.stringify(marshalIpnsApiImportKeyNameRequest(request, this.client.settings)),
    headers: jsonContentHeaders$h,
    method: 'POST',
    path: `/ipfs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/names/import-key`
  }, unmarshalName);
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$h = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$h,
  IpnsAPI: IpnsAPI,
  NAME_TRANSIENT_STATUSES: NAME_TRANSIENT_STATUSES,
  PIN_TRANSIENT_STATUSES: PIN_TRANSIENT_STATUSES
});

var index$i = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$h
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalCronSchedule = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CronSchedule' failed as data isn't a dictionary.`);
  }
  return {
    schedule: data.schedule,
    timezone: data.timezone
  };
};
const unmarshalJobDefinition = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'JobDefinition' failed as data isn't a dictionary.`);
  }
  return {
    command: data.command,
    cpuLimit: data.cpu_limit,
    createdAt: unmarshalDate(data.created_at),
    cronSchedule: data.cron_schedule ? unmarshalCronSchedule(data.cron_schedule) : undefined,
    description: data.description,
    environmentVariables: data.environment_variables,
    id: data.id,
    imageUri: data.image_uri,
    jobTimeout: data.job_timeout,
    localStorageCapacity: data.local_storage_capacity,
    memoryLimit: data.memory_limit,
    name: data.name,
    projectId: data.project_id,
    region: data.region,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalJobRun = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'JobRun' failed as data isn't a dictionary.`);
  }
  return {
    command: data.command,
    cpuLimit: data.cpu_limit,
    createdAt: unmarshalDate(data.created_at),
    environmentVariables: data.environment_variables,
    errorMessage: data.error_message,
    exitCode: data.exit_code,
    id: data.id,
    jobDefinitionId: data.job_definition_id,
    localStorageCapacity: data.local_storage_capacity,
    memoryLimit: data.memory_limit,
    region: data.region,
    runDuration: data.run_duration,
    startedAt: unmarshalDate(data.started_at),
    state: data.state,
    terminatedAt: unmarshalDate(data.terminated_at),
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListJobDefinitionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListJobDefinitionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    jobDefinitions: unmarshalArrayOfObject(data.job_definitions, unmarshalJobDefinition),
    totalCount: data.total_count
  };
};
const unmarshalListJobRunsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListJobRunsResponse' failed as data isn't a dictionary.`);
  }
  return {
    jobRuns: unmarshalArrayOfObject(data.job_runs, unmarshalJobRun),
    totalCount: data.total_count
  };
};
const unmarshalStartJobDefinitionResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'StartJobDefinitionResponse' failed as data isn't a dictionary.`);
  }
  return {
    jobRuns: unmarshalArrayOfObject(data.job_runs, unmarshalJobRun)
  };
};
const marshalCreateJobDefinitionRequestCronScheduleConfig = (request, defaults) => ({
  schedule: request.schedule,
  timezone: request.timezone
});
const marshalCreateJobDefinitionRequest = (request, defaults) => ({
  command: request.command,
  cpu_limit: request.cpuLimit,
  cron_schedule: request.cronSchedule !== undefined ? marshalCreateJobDefinitionRequestCronScheduleConfig(request.cronSchedule) : undefined,
  description: request.description,
  environment_variables: request.environmentVariables !== undefined ? request.environmentVariables : undefined,
  image_uri: request.imageUri,
  job_timeout: request.jobTimeout,
  local_storage_capacity: request.localStorageCapacity,
  memory_limit: request.memoryLimit,
  name: request.name || randomName('job'),
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalStartJobDefinitionRequest = (request, defaults) => ({
  command: request.command,
  environment_variables: request.environmentVariables,
  replicas: request.replicas
});
const marshalUpdateJobDefinitionRequestCronScheduleConfig = (request, defaults) => ({
  schedule: request.schedule,
  timezone: request.timezone
});
const marshalUpdateJobDefinitionRequest = (request, defaults) => ({
  command: request.command,
  cpu_limit: request.cpuLimit,
  cron_schedule: request.cronSchedule !== undefined ? marshalUpdateJobDefinitionRequestCronScheduleConfig(request.cronSchedule) : undefined,
  description: request.description,
  environment_variables: request.environmentVariables,
  image_uri: request.imageUri,
  job_timeout: request.jobTimeout,
  local_storage_capacity: request.localStorageCapacity,
  memory_limit: request.memoryLimit,
  name: request.name
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$g = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Serverless Jobs API. */
let API$g = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];

  /**
   * Create a new job definition in a specified Project.
   *
   * @param request - The request {@link CreateJobDefinitionRequest}
   * @returns A Promise of JobDefinition
   */
  createJobDefinition = request => this.client.fetch({
    body: JSON.stringify(marshalCreateJobDefinitionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$g,
    method: 'POST',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-definitions`
  }, unmarshalJobDefinition);

  /**
   * Get a job definition by its unique identifier.
   *
   * @param request - The request {@link GetJobDefinitionRequest}
   * @returns A Promise of JobDefinition
   */
  getJobDefinition = request => this.client.fetch({
    method: 'GET',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-definitions/${validatePathParam('jobDefinitionId', request.jobDefinitionId)}`
  }, unmarshalJobDefinition);
  pageOfListJobDefinitions = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-definitions`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListJobDefinitionsResponse);

  /**
   * List all your job definitions with filters.
   *
   * @param request - The request {@link ListJobDefinitionsRequest}
   * @returns A Promise of ListJobDefinitionsResponse
   */
  listJobDefinitions = (request = {}) => enrichForPagination('jobDefinitions', this.pageOfListJobDefinitions, request);

  /**
   * Update an existing job definition associated with the specified unique
   * identifier.
   *
   * @param request - The request {@link UpdateJobDefinitionRequest}
   * @returns A Promise of JobDefinition
   */
  updateJobDefinition = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateJobDefinitionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$g,
    method: 'PATCH',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-definitions/${validatePathParam('jobDefinitionId', request.jobDefinitionId)}`
  }, unmarshalJobDefinition);

  /**
   * Delete an exsisting job definition by its unique identifier.
   *
   * @param request - The request {@link DeleteJobDefinitionRequest}
   */
  deleteJobDefinition = request => this.client.fetch({
    method: 'DELETE',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-definitions/${validatePathParam('jobDefinitionId', request.jobDefinitionId)}`
  });

  /**
   * Run an existing job definition by its unique identifier. This will create a
   * new job run.
   *
   * @param request - The request {@link StartJobDefinitionRequest}
   * @returns A Promise of StartJobDefinitionResponse
   */
  startJobDefinition = request => this.client.fetch({
    body: JSON.stringify(marshalStartJobDefinitionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$g,
    method: 'POST',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-definitions/${validatePathParam('jobDefinitionId', request.jobDefinitionId)}/start`
  }, unmarshalStartJobDefinitionResponse);

  /**
   * Get a job run by its unique identifier.
   *
   * @param request - The request {@link GetJobRunRequest}
   * @returns A Promise of JobRun
   */
  getJobRun = request => this.client.fetch({
    method: 'GET',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-runs/${validatePathParam('jobRunId', request.jobRunId)}`
  }, unmarshalJobRun);

  /**
   * Stop a job run by its unique identifier.
   *
   * @param request - The request {@link StopJobRunRequest}
   * @returns A Promise of JobRun
   */
  stopJobRun = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$g,
    method: 'POST',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-runs/${validatePathParam('jobRunId', request.jobRunId)}/stop`
  }, unmarshalJobRun);
  pageOfListJobRuns = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/serverless-jobs/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/job-runs`,
    urlParams: urlParams(['job_definition_id', request.jobDefinitionId], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListJobRunsResponse);

  /**
   * List all job runs with filters.
   *
   * @param request - The request {@link ListJobRunsRequest}
   * @returns A Promise of ListJobRunsResponse
   */
  listJobRuns = (request = {}) => enrichForPagination('jobRuns', this.pageOfListJobRuns, request);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link JobRunState}. */
const JOB_RUN_TRANSIENT_STATUSES = ['queued', 'scheduled', 'running'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateJobDefinitionRequest = {
  cpuLimit: {
    greaterThan: 0
  },
  imageUri: {
    pattern: /^((?:(?:(?:[a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9-]*[a-zA-Z0-9])(?:\.(?:[a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9-]*[a-zA-Z0-9]))*|\[(?:[a-fA-F0-9:]+)\])(?::[0-9]+)?\/)?[a-z0-9]+(?:(?:[._]|__|[-]+)[a-z0-9]+)*(?:\/[a-z0-9]+(?:(?:[._]|__|[-]+)[a-z0-9]+)*)*)(?::([\w][\w.-]{0,127}))?(?:@([A-Za-z][A-Za-z0-9]*(?:[-_+.][A-Za-z][A-Za-z0-9]*)*[:][[:xdigit:]]{32,}))?$/
  },
  localStorageCapacity: {
    greaterThan: 0
  },
  memoryLimit: {
    greaterThan: 0
  },
  name: {
    pattern: /^[A-Za-z0-9-_]{3,50}$/
  }
};
const CreateJobDefinitionRequestCronScheduleConfig = {
  schedule: {
    maxLength: 255,
    minLength: 1
  },
  timezone: {
    maxLength: 255,
    minLength: 1
  }
};
const CronSchedule = {
  schedule: {
    maxLength: 255,
    minLength: 1
  },
  timezone: {
    maxLength: 255,
    minLength: 1
  }
};
const ListJobDefinitionsRequest = {
  page: {
    greaterThanOrEqual: 1
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 1000
  }
};
const ListJobRunsRequest = {
  page: {
    greaterThanOrEqual: 1
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 1000
  }
};
const StartJobDefinitionRequest = {
  replicas: {
    greaterThan: 0
  }
};
const UpdateJobDefinitionRequest = {
  cpuLimit: {
    greaterThan: 0
  },
  imageUri: {
    pattern: /^((?:(?:(?:[a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9-]*[a-zA-Z0-9])(?:\.(?:[a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9-]*[a-zA-Z0-9]))*|\[(?:[a-fA-F0-9:]+)\])(?::[0-9]+)?\/)?[a-z0-9]+(?:(?:[._]|__|[-]+)[a-z0-9]+)*(?:\/[a-z0-9]+(?:(?:[._]|__|[-]+)[a-z0-9]+)*)*)(?::([\w][\w.-]{0,127}))?(?:@([A-Za-z][A-Za-z0-9]*(?:[-_+.][A-Za-z][A-Za-z0-9]*)*[:][[:xdigit:]]{32,}))?$/
  },
  localStorageCapacity: {
    greaterThan: 0
  },
  memoryLimit: {
    greaterThan: 0
  },
  name: {
    pattern: /^[A-Za-z0-9-_]{3,50}$/
  }
};
const UpdateJobDefinitionRequestCronScheduleConfig = {
  schedule: {
    maxLength: 255,
    minLength: 1
  },
  timezone: {
    maxLength: 255,
    minLength: 1
  }
};

var validationRules_gen$7 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateJobDefinitionRequest: CreateJobDefinitionRequest,
  CreateJobDefinitionRequestCronScheduleConfig: CreateJobDefinitionRequestCronScheduleConfig,
  CronSchedule: CronSchedule,
  ListJobDefinitionsRequest: ListJobDefinitionsRequest,
  ListJobRunsRequest: ListJobRunsRequest,
  StartJobDefinitionRequest: StartJobDefinitionRequest,
  UpdateJobDefinitionRequest: UpdateJobDefinitionRequest,
  UpdateJobDefinitionRequestCronScheduleConfig: UpdateJobDefinitionRequestCronScheduleConfig
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$g = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$g,
  JOB_RUN_TRANSIENT_STATUSES: JOB_RUN_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen$7
});

var index$h = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$g
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link ClusterStatus}. */
const CLUSTER_TRANSIENT_STATUSES$1 = ['creating', 'deleting', 'updating'];

/** Lists transient statutes of the enum {@link NodeStatus}. */
const NODE_TRANSIENT_STATUSES = ['creating', 'deleting', 'rebooting', 'upgrading', 'starting', 'registering'];

/** Lists transient statutes of the enum {@link PoolStatus}. */
const POOL_TRANSIENT_STATUSES = ['deleting', 'scaling', 'upgrading'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalPoolUpgradePolicy = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PoolUpgradePolicy' failed as data isn't a dictionary.`);
  }
  return {
    maxSurge: data.max_surge,
    maxUnavailable: data.max_unavailable
  };
};
const unmarshalPool = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Pool' failed as data isn't a dictionary.`);
  }
  return {
    autohealing: data.autohealing,
    autoscaling: data.autoscaling,
    clusterId: data.cluster_id,
    containerRuntime: data.container_runtime,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    kubeletArgs: data.kubelet_args,
    maxSize: data.max_size,
    minSize: data.min_size,
    name: data.name,
    nodeType: data.node_type,
    placementGroupId: data.placement_group_id,
    publicIpDisabled: data.public_ip_disabled,
    region: data.region,
    rootVolumeSize: data.root_volume_size,
    rootVolumeType: data.root_volume_type,
    size: data.size,
    status: data.status,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    upgradePolicy: data.upgrade_policy ? unmarshalPoolUpgradePolicy(data.upgrade_policy) : undefined,
    version: data.version,
    zone: data.zone
  };
};
const unmarshalVersion$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Version' failed as data isn't a dictionary.`);
  }
  return {
    availableAdmissionPlugins: data.available_admission_plugins,
    availableCnis: data.available_cnis,
    availableContainerRuntimes: data.available_container_runtimes,
    availableFeatureGates: data.available_feature_gates,
    availableKubeletArgs: data.available_kubelet_args,
    label: data.label,
    name: data.name,
    region: data.region
  };
};
const unmarshalMaintenanceWindow = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'MaintenanceWindow' failed as data isn't a dictionary.`);
  }
  return {
    day: data.day,
    startHour: data.start_hour
  };
};
const unmarshalClusterAutoUpgrade = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClusterAutoUpgrade' failed as data isn't a dictionary.`);
  }
  return {
    enabled: data.enabled,
    maintenanceWindow: data.maintenance_window ? unmarshalMaintenanceWindow(data.maintenance_window) : undefined
  };
};
const unmarshalClusterAutoscalerConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClusterAutoscalerConfig' failed as data isn't a dictionary.`);
  }
  return {
    balanceSimilarNodeGroups: data.balance_similar_node_groups,
    estimator: data.estimator,
    expander: data.expander,
    expendablePodsPriorityCutoff: data.expendable_pods_priority_cutoff,
    ignoreDaemonsetsUtilization: data.ignore_daemonsets_utilization,
    maxGracefulTerminationSec: data.max_graceful_termination_sec,
    scaleDownDelayAfterAdd: data.scale_down_delay_after_add,
    scaleDownDisabled: data.scale_down_disabled,
    scaleDownUnneededTime: data.scale_down_unneeded_time,
    scaleDownUtilizationThreshold: data.scale_down_utilization_threshold
  };
};
const unmarshalClusterOpenIDConnectConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClusterOpenIDConnectConfig' failed as data isn't a dictionary.`);
  }
  return {
    clientId: data.client_id,
    groupsClaim: data.groups_claim,
    groupsPrefix: data.groups_prefix,
    issuerUrl: data.issuer_url,
    requiredClaim: data.required_claim,
    usernameClaim: data.username_claim,
    usernamePrefix: data.username_prefix
  };
};
const unmarshalCluster$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Cluster' failed as data isn't a dictionary.`);
  }
  return {
    admissionPlugins: data.admission_plugins,
    apiserverCertSans: data.apiserver_cert_sans,
    autoUpgrade: data.auto_upgrade ? unmarshalClusterAutoUpgrade(data.auto_upgrade) : undefined,
    autoscalerConfig: data.autoscaler_config ? unmarshalClusterAutoscalerConfig(data.autoscaler_config) : undefined,
    clusterUrl: data.cluster_url,
    cni: data.cni,
    commitmentEndsAt: unmarshalDate(data.commitment_ends_at),
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    dnsWildcard: data.dns_wildcard,
    featureGates: data.feature_gates,
    id: data.id,
    name: data.name,
    openIdConnectConfig: data.open_id_connect_config ? unmarshalClusterOpenIDConnectConfig(data.open_id_connect_config) : undefined,
    organizationId: data.organization_id,
    privateNetworkId: data.private_network_id,
    projectId: data.project_id,
    region: data.region,
    routedIpEnabled: data.routed_ip_enabled,
    status: data.status,
    tags: data.tags,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    upgradeAvailable: data.upgrade_available,
    version: data.version
  };
};
const unmarshalNode = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Node' failed as data isn't a dictionary.`);
  }
  return {
    clusterId: data.cluster_id,
    conditions: data.conditions ? data.conditions : undefined,
    createdAt: unmarshalDate(data.created_at),
    errorMessage: data.error_message,
    id: data.id,
    name: data.name,
    poolId: data.pool_id,
    providerId: data.provider_id,
    publicIpV4: data.public_ip_v4,
    publicIpV6: data.public_ip_v6,
    region: data.region,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalExternalNodeCoreV1Taint = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ExternalNodeCoreV1Taint' failed as data isn't a dictionary.`);
  }
  return {
    effect: data.effect,
    key: data.key,
    value: data.value
  };
};
const unmarshalExternalNode = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ExternalNode' failed as data isn't a dictionary.`);
  }
  return {
    clusterCa: data.cluster_ca,
    clusterUrl: data.cluster_url,
    cniPluginsVersion: data.cni_plugins_version,
    containerdVersion: data.containerd_version,
    externalIp: data.external_ip,
    id: data.id,
    kubeToken: data.kube_token,
    kubeletConfig: data.kubelet_config,
    name: data.name,
    nodeLabels: data.node_labels,
    nodeTaints: unmarshalArrayOfObject(data.node_taints, unmarshalExternalNodeCoreV1Taint),
    poolVersion: data.pool_version,
    runcVersion: data.runc_version
  };
};
const unmarshalClusterType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClusterType' failed as data isn't a dictionary.`);
  }
  return {
    auditLogsSupported: data.audit_logs_supported,
    availability: data.availability,
    commitmentDelay: data.commitment_delay,
    dedicated: data.dedicated,
    maxNodes: data.max_nodes,
    memory: data.memory,
    name: data.name,
    resiliency: data.resiliency,
    sla: data.sla
  };
};
const unmarshalListClusterAvailableTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListClusterAvailableTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    clusterTypes: unmarshalArrayOfObject(data.cluster_types, unmarshalClusterType),
    totalCount: data.total_count
  };
};
const unmarshalListClusterAvailableVersionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListClusterAvailableVersionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    versions: unmarshalArrayOfObject(data.versions, unmarshalVersion$1)
  };
};
const unmarshalListClusterTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListClusterTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    clusterTypes: unmarshalArrayOfObject(data.cluster_types, unmarshalClusterType),
    totalCount: data.total_count
  };
};
const unmarshalListClustersResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListClustersResponse' failed as data isn't a dictionary.`);
  }
  return {
    clusters: unmarshalArrayOfObject(data.clusters, unmarshalCluster$1),
    totalCount: data.total_count
  };
};
const unmarshalListNodesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNodesResponse' failed as data isn't a dictionary.`);
  }
  return {
    nodes: unmarshalArrayOfObject(data.nodes, unmarshalNode),
    totalCount: data.total_count
  };
};
const unmarshalListPoolsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPoolsResponse' failed as data isn't a dictionary.`);
  }
  return {
    pools: unmarshalArrayOfObject(data.pools, unmarshalPool),
    totalCount: data.total_count
  };
};
const unmarshalListVersionsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListVersionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    versions: unmarshalArrayOfObject(data.versions, unmarshalVersion$1)
  };
};
const marshalMaintenanceWindow = (request, defaults) => ({
  day: request.day,
  start_hour: request.startHour
});
const marshalCreateClusterRequestPoolConfigUpgradePolicy = (request, defaults) => ({
  max_surge: request.maxSurge,
  max_unavailable: request.maxUnavailable
});
const marshalCreateClusterRequestAutoUpgrade = (request, defaults) => ({
  enable: request.enable,
  maintenance_window: request.maintenanceWindow !== undefined ? marshalMaintenanceWindow(request.maintenanceWindow) : undefined
});
const marshalCreateClusterRequestAutoscalerConfig = (request, defaults) => ({
  balance_similar_node_groups: request.balanceSimilarNodeGroups,
  estimator: request.estimator,
  expander: request.expander,
  expendable_pods_priority_cutoff: request.expendablePodsPriorityCutoff,
  ignore_daemonsets_utilization: request.ignoreDaemonsetsUtilization,
  max_graceful_termination_sec: request.maxGracefulTerminationSec,
  scale_down_delay_after_add: request.scaleDownDelayAfterAdd,
  scale_down_disabled: request.scaleDownDisabled,
  scale_down_unneeded_time: request.scaleDownUnneededTime,
  scale_down_utilization_threshold: request.scaleDownUtilizationThreshold
});
const marshalCreateClusterRequestOpenIDConnectConfig = (request, defaults) => ({
  client_id: request.clientId,
  groups_claim: request.groupsClaim,
  groups_prefix: request.groupsPrefix,
  issuer_url: request.issuerUrl,
  required_claim: request.requiredClaim,
  username_claim: request.usernameClaim,
  username_prefix: request.usernamePrefix
});
const marshalCreateClusterRequestPoolConfig = (request, defaults) => ({
  autohealing: request.autohealing,
  autoscaling: request.autoscaling,
  container_runtime: request.containerRuntime,
  kubelet_args: request.kubeletArgs,
  max_size: request.maxSize,
  min_size: request.minSize,
  name: request.name,
  node_type: request.nodeType,
  placement_group_id: request.placementGroupId,
  public_ip_disabled: request.publicIpDisabled,
  root_volume_size: request.rootVolumeSize,
  root_volume_type: request.rootVolumeType,
  size: request.size,
  tags: request.tags,
  upgrade_policy: request.upgradePolicy !== undefined ? marshalCreateClusterRequestPoolConfigUpgradePolicy(request.upgradePolicy) : undefined,
  zone: request.zone
});
const marshalCreateClusterRequest$1 = (request, defaults) => ({
  admission_plugins: request.admissionPlugins,
  apiserver_cert_sans: request.apiserverCertSans,
  auto_upgrade: request.autoUpgrade !== undefined ? marshalCreateClusterRequestAutoUpgrade(request.autoUpgrade) : undefined,
  autoscaler_config: request.autoscalerConfig !== undefined ? marshalCreateClusterRequestAutoscalerConfig(request.autoscalerConfig) : undefined,
  cni: request.cni,
  description: request.description,
  feature_gates: request.featureGates,
  name: request.name || randomName('k8s'),
  open_id_connect_config: request.openIdConnectConfig !== undefined ? marshalCreateClusterRequestOpenIDConnectConfig(request.openIdConnectConfig) : undefined,
  pools: request.pools !== undefined ? request.pools.map(elt => marshalCreateClusterRequestPoolConfig(elt)) : undefined,
  private_network_id: request.privateNetworkId,
  tags: request.tags,
  type: request.type,
  version: request.version,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalCreatePoolRequestUpgradePolicy = (request, defaults) => ({
  max_surge: request.maxSurge,
  max_unavailable: request.maxUnavailable
});
const marshalCreatePoolRequest = (request, defaults) => ({
  autohealing: request.autohealing,
  autoscaling: request.autoscaling,
  container_runtime: request.containerRuntime,
  kubelet_args: request.kubeletArgs !== undefined ? request.kubeletArgs : undefined,
  max_size: request.maxSize,
  min_size: request.minSize,
  name: request.name || randomName('pool'),
  node_type: request.nodeType,
  placement_group_id: request.placementGroupId,
  public_ip_disabled: request.publicIpDisabled,
  root_volume_size: request.rootVolumeSize,
  root_volume_type: request.rootVolumeType,
  size: request.size,
  tags: request.tags,
  upgrade_policy: request.upgradePolicy !== undefined ? marshalCreatePoolRequestUpgradePolicy(request.upgradePolicy) : undefined,
  zone: request.zone ?? defaults.defaultZone
});
const marshalMigrateToPrivateNetworkClusterRequest = (request, defaults) => ({
  private_network_id: request.privateNetworkId
});
const marshalSetClusterTypeRequest = (request, defaults) => ({
  type: request.type
});
const marshalUpdateClusterRequestAutoUpgrade = (request, defaults) => ({
  enable: request.enable,
  maintenance_window: request.maintenanceWindow !== undefined ? marshalMaintenanceWindow(request.maintenanceWindow) : undefined
});
const marshalUpdateClusterRequestAutoscalerConfig = (request, defaults) => ({
  balance_similar_node_groups: request.balanceSimilarNodeGroups,
  estimator: request.estimator,
  expander: request.expander,
  expendable_pods_priority_cutoff: request.expendablePodsPriorityCutoff,
  ignore_daemonsets_utilization: request.ignoreDaemonsetsUtilization,
  max_graceful_termination_sec: request.maxGracefulTerminationSec,
  scale_down_delay_after_add: request.scaleDownDelayAfterAdd,
  scale_down_disabled: request.scaleDownDisabled,
  scale_down_unneeded_time: request.scaleDownUnneededTime,
  scale_down_utilization_threshold: request.scaleDownUtilizationThreshold
});
const marshalUpdateClusterRequestOpenIDConnectConfig = (request, defaults) => ({
  client_id: request.clientId,
  groups_claim: request.groupsClaim,
  groups_prefix: request.groupsPrefix,
  issuer_url: request.issuerUrl,
  required_claim: request.requiredClaim,
  username_claim: request.usernameClaim,
  username_prefix: request.usernamePrefix
});
const marshalUpdateClusterRequest$1 = (request, defaults) => ({
  admission_plugins: request.admissionPlugins,
  apiserver_cert_sans: request.apiserverCertSans,
  auto_upgrade: request.autoUpgrade !== undefined ? marshalUpdateClusterRequestAutoUpgrade(request.autoUpgrade) : undefined,
  autoscaler_config: request.autoscalerConfig !== undefined ? marshalUpdateClusterRequestAutoscalerConfig(request.autoscalerConfig) : undefined,
  description: request.description,
  feature_gates: request.featureGates,
  name: request.name,
  open_id_connect_config: request.openIdConnectConfig !== undefined ? marshalUpdateClusterRequestOpenIDConnectConfig(request.openIdConnectConfig) : undefined,
  tags: request.tags
});
const marshalUpdatePoolRequestUpgradePolicy = (request, defaults) => ({
  max_surge: request.maxSurge,
  max_unavailable: request.maxUnavailable
});
const marshalUpdatePoolRequest = (request, defaults) => ({
  autohealing: request.autohealing,
  autoscaling: request.autoscaling,
  kubelet_args: request.kubeletArgs,
  max_size: request.maxSize,
  min_size: request.minSize,
  size: request.size,
  tags: request.tags,
  upgrade_policy: request.upgradePolicy !== undefined ? marshalUpdatePoolRequestUpgradePolicy(request.upgradePolicy) : undefined
});
const marshalUpgradeClusterRequest = (request, defaults) => ({
  upgrade_pools: request.upgradePools,
  version: request.version
});
const marshalUpgradePoolRequest = (request, defaults) => ({
  version: request.version
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$f = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Kubernetes API. */
let API$f = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];
  pageOfListClusters = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['private_network_id', request.privateNetworkId], ['project_id', request.projectId], ['status', request.status], ['type', request.type])
  }, unmarshalListClustersResponse$1);

  /**
   * List Clusters. List all existing Kubernetes clusters in a specific region.
   *
   * @param request - The request {@link ListClustersRequest}
   * @returns A Promise of ListClustersResponse
   */
  listClusters = (request = {}) => enrichForPagination('clusters', this.pageOfListClusters, request);

  /**
   * Create a new Cluster. Create a new Kubernetes cluster in a Scaleway region.
   *
   * @param request - The request {@link CreateClusterRequest}
   * @returns A Promise of Cluster
   */
  createCluster = request => this.client.fetch({
    body: JSON.stringify(marshalCreateClusterRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters`
  }, unmarshalCluster$1);

  /**
   * Get a Cluster. Retrieve information about a specific Kubernetes cluster.
   *
   * @param request - The request {@link GetClusterRequest}
   * @returns A Promise of Cluster
   */
  getCluster = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}`
  }, unmarshalCluster$1);

  /**
   * Waits for {@link Cluster} to be in a final state.
   *
   * @param request - The request {@link GetClusterRequest}
   * @param options - The waiting options
   * @returns A Promise of Cluster
   */
  waitForCluster = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!CLUSTER_TRANSIENT_STATUSES$1.includes(res.status))), this.getCluster, request, options);

  /**
   * Update a Cluster. Update information on a specific Kubernetes cluster. You
   * can update details such as its name, description, tags and configuration.
   * To upgrade a cluster, you will need to use the dedicated endpoint.
   *
   * @param request - The request {@link UpdateClusterRequest}
   * @returns A Promise of Cluster
   */
  updateCluster = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateClusterRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$f,
    method: 'PATCH',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}`
  }, unmarshalCluster$1);

  /**
   * Delete a Cluster. Delete a specific Kubernetes cluster and all its
   * associated pools and nodes. Note that this method will not delete any Load
   * Balancer or Block Volume that are associated with the cluster.
   *
   * @param request - The request {@link DeleteClusterRequest}
   * @returns A Promise of Cluster
   */
  deleteCluster = request => this.client.fetch({
    method: 'DELETE',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}`,
    urlParams: urlParams(['with_additional_resources', request.withAdditionalResources])
  }, unmarshalCluster$1);

  /**
   * Upgrade a Cluster. Upgrade a specific Kubernetes cluster and possibly its
   * associated pools to a specific and supported Kubernetes version.
   *
   * @param request - The request {@link UpgradeClusterRequest}
   * @returns A Promise of Cluster
   */
  upgradeCluster = request => this.client.fetch({
    body: JSON.stringify(marshalUpgradeClusterRequest(request, this.client.settings)),
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/upgrade`
  }, unmarshalCluster$1);

  /**
   * Change the Cluster type. Change the type of a specific Kubernetes cluster.
   * To see the possible values you can enter for the `type` field, [list
   * available cluster
   * types](#path-clusters-list-available-cluster-types-for-a-cluster).
   *
   * @param request - The request {@link SetClusterTypeRequest}
   * @returns A Promise of Cluster
   */
  setClusterType = request => this.client.fetch({
    body: JSON.stringify(marshalSetClusterTypeRequest(request, this.client.settings)),
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/set-type`
  }, unmarshalCluster$1);

  /**
   * List available versions for a Cluster. List the versions that a specific
   * Kubernetes cluster is allowed to upgrade to. Results will include every
   * patch version greater than the current patch, as well as one minor version
   * ahead of the current version. Any upgrade skipping a minor version will not
   * work.
   *
   * @param request - The request {@link ListClusterAvailableVersionsRequest}
   * @returns A Promise of ListClusterAvailableVersionsResponse
   */
  listClusterAvailableVersions = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/available-versions`
  }, unmarshalListClusterAvailableVersionsResponse);

  /**
   * List available cluster types for a cluster. List the cluster types that a
   * specific Kubernetes cluster is allowed to switch to.
   *
   * @param request - The request {@link ListClusterAvailableTypesRequest}
   * @returns A Promise of ListClusterAvailableTypesResponse
   */
  listClusterAvailableTypes = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/available-types`
  }, unmarshalListClusterAvailableTypesResponse);
  _getClusterKubeConfig = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/kubeconfig`,
    urlParams: urlParams(['dl', 1], ['redacted', request.redacted]),
    responseType: 'blob'
  });

  /**
   * Reset the admin token of a Cluster. Reset the admin token for a specific
   * Kubernetes cluster. This will revoke the old admin token (which will not be
   * usable afterwards) and create a new one. Note that you will need to
   * download kubeconfig again to keep interacting with the cluster.
   *
   * @param request - The request {@link ResetClusterAdminTokenRequest}
   */
  resetClusterAdminToken = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/reset-admin-token`
  });

  /**
   * Migrate an existing cluster to a Private Network cluster. Migrate a cluster
   * that was created before the release of Private Network clusters to a new
   * one with a Private Network.
   *
   * @param request - The request {@link MigrateToPrivateNetworkClusterRequest}
   * @returns A Promise of Cluster
   */
  migrateToPrivateNetworkCluster = request => this.client.fetch({
    body: JSON.stringify(marshalMigrateToPrivateNetworkClusterRequest(request, this.client.settings)),
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/migrate-to-private-network`
  }, unmarshalCluster$1);

  /**
   * Migrate a cluster to Routed IPs. Migrate the nodes of an existing cluster
   * to Routed IPs and enable Routed IPs for all future nodes.
   *
   * @param request - The request {@link MigrateClusterToRoutedIPsRequest}
   * @returns A Promise of Cluster
   */
  migrateClusterToRoutedIPs = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/migrate-to-routed-ips`
  }, unmarshalCluster$1);
  pageOfListPools = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/pools`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['status', request.status])
  }, unmarshalListPoolsResponse);

  /**
   * List Pools in a Cluster. List all the existing pools for a specific
   * Kubernetes cluster.
   *
   * @param request - The request {@link ListPoolsRequest}
   * @returns A Promise of ListPoolsResponse
   */
  listPools = request => enrichForPagination('pools', this.pageOfListPools, request);

  /**
   * Create a new Pool in a Cluster. Create a new pool in a specific Kubernetes
   * cluster.
   *
   * @param request - The request {@link CreatePoolRequest}
   * @returns A Promise of Pool
   */
  createPool = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePoolRequest(request, this.client.settings)),
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/pools`
  }, unmarshalPool);

  /**
   * Get a Pool in a Cluster. Retrieve details about a specific pool in a
   * Kubernetes cluster.
   *
   * @param request - The request {@link GetPoolRequest}
   * @returns A Promise of Pool
   */
  getPool = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pools/${validatePathParam('poolId', request.poolId)}`
  }, unmarshalPool);

  /**
   * Waits for {@link Pool} to be in a final state.
   *
   * @param request - The request {@link GetPoolRequest}
   * @param options - The waiting options
   * @returns A Promise of Pool
   */
  waitForPool = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!POOL_TRANSIENT_STATUSES.includes(res.status))), this.getPool, request, options);

  /**
   * Upgrade a Pool in a Cluster. Upgrade the Kubernetes version of a specific
   * pool. Note that it only works if the targeted version matches the cluster's
   * version.
   *
   * @param request - The request {@link UpgradePoolRequest}
   * @returns A Promise of Pool
   */
  upgradePool = request => this.client.fetch({
    body: JSON.stringify(marshalUpgradePoolRequest(request, this.client.settings)),
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pools/${validatePathParam('poolId', request.poolId)}/upgrade`
  }, unmarshalPool);

  /**
   * Update a Pool in a Cluster. Update the attributes of a specific pool, such
   * as its desired size, autoscaling settings, and tags.
   *
   * @param request - The request {@link UpdatePoolRequest}
   * @returns A Promise of Pool
   */
  updatePool = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePoolRequest(request, this.client.settings)),
    headers: jsonContentHeaders$f,
    method: 'PATCH',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pools/${validatePathParam('poolId', request.poolId)}`
  }, unmarshalPool);

  /**
   * Delete a Pool in a Cluster. Delete a specific pool from a cluster. Note
   * that all the pool's nodes will also be deleted.
   *
   * @param request - The request {@link DeletePoolRequest}
   * @returns A Promise of Pool
   */
  deletePool = request => this.client.fetch({
    method: 'DELETE',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pools/${validatePathParam('poolId', request.poolId)}`
  }, unmarshalPool);

  /**
   * Create a Kosmos node. Retrieve metadata for a Kosmos node. This method is
   * not intended to be called by end users but rather programmatically by the
   * kapsule-node-agent.
   *
   * @param request - The request {@link CreateExternalNodeRequest}
   * @returns A Promise of ExternalNode
   */
  createExternalNode = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/pools/${validatePathParam('poolId', request.poolId)}/external-nodes`
  }, unmarshalExternalNode);
  pageOfListNodes = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/clusters/${validatePathParam('clusterId', request.clusterId)}/nodes`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['pool_id', request.poolId], ['status', request.status])
  }, unmarshalListNodesResponse);

  /**
   * List Nodes in a Cluster. List all the existing nodes for a specific
   * Kubernetes cluster.
   *
   * @param request - The request {@link ListNodesRequest}
   * @returns A Promise of ListNodesResponse
   */
  listNodes = request => enrichForPagination('nodes', this.pageOfListNodes, request);

  /**
   * Get a Node in a Cluster. Retrieve details about a specific Kubernetes Node.
   *
   * @param request - The request {@link GetNodeRequest}
   * @returns A Promise of Node
   */
  getNode = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nodes/${validatePathParam('nodeId', request.nodeId)}`
  }, unmarshalNode);

  /**
   * Waits for {@link Node} to be in a final state.
   *
   * @param request - The request {@link GetNodeRequest}
   * @param options - The waiting options
   * @returns A Promise of Node
   */
  waitForNode = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!NODE_TRANSIENT_STATUSES.includes(res.status))), this.getNode, request, options);

  /**
   * Replace a Node in a Cluster. Replace a specific Node. The node will first
   * be cordoned (scheduling will be disabled on it). The existing pods on the
   * node will then be drained and rescheduled onto another schedulable node.
   * Note that when there is not enough space to reschedule all the pods (such
   * as in a one-node cluster), disruption of your applications can be
   * expected.
   *
   * @deprecated
   * @param request - The request {@link ReplaceNodeRequest}
   * @returns A Promise of Node
   */
  replaceNode = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nodes/${validatePathParam('nodeId', request.nodeId)}/replace`
  }, unmarshalNode);

  /**
   * Reboot a Node in a Cluster. Reboot a specific Node. The node will first be
   * cordoned (scheduling will be disabled on it). The existing pods on the node
   * will then be drained and rescheduled onto another schedulable node. Note
   * that when there is not enough space to reschedule all the pods (such as in
   * a one-node cluster), disruption of your applications can be expected.
   *
   * @param request - The request {@link RebootNodeRequest}
   * @returns A Promise of Node
   */
  rebootNode = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$f,
    method: 'POST',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nodes/${validatePathParam('nodeId', request.nodeId)}/reboot`
  }, unmarshalNode);

  /**
   * Delete a Node in a Cluster. Delete a specific Node. Note that when there is
   * not enough space to reschedule all the pods (such as in a one-node
   * cluster), disruption of your applications can be expected.
   *
   * @param request - The request {@link DeleteNodeRequest}
   * @returns A Promise of Node
   */
  deleteNode = request => this.client.fetch({
    method: 'DELETE',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nodes/${validatePathParam('nodeId', request.nodeId)}`,
    urlParams: urlParams(['replace', request.replace], ['skip_drain', request.skipDrain])
  }, unmarshalNode);

  /**
   * List all available Versions. List all available versions for the creation
   * of a new Kubernetes cluster.
   *
   * @param request - The request {@link ListVersionsRequest}
   * @returns A Promise of ListVersionsResponse
   */
  listVersions = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/versions`
  }, unmarshalListVersionsResponse$1);

  /**
   * Get a Version. Retrieve a specific Kubernetes version and its details.
   *
   * @param request - The request {@link GetVersionRequest}
   * @returns A Promise of Version
   */
  getVersion = request => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/versions/${validatePathParam('versionName', request.versionName)}`
  }, unmarshalVersion$1);
  pageOfListClusterTypes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/k8s/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/cluster-types`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListClusterTypesResponse);

  /**
   * List cluster types. List available cluster types and their technical
   * details.
   *
   * @param request - The request {@link ListClusterTypesRequest}
   * @returns A Promise of ListClusterTypesResponse
   */
  listClusterTypes = (request = {}) => enrichForPagination('clusterTypes', this.pageOfListClusterTypes, request);
};

class K8SUtilsAPI extends API$f {
  /**
   * Get configuration of a kube cluster.
   *
   * @param request - The request {@link GetClusterKubeConfigRequest}
   * @returns A Promise of Blob
   */
  getClusterKubeConfig = request => this._getClusterKubeConfig(request);
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateClusterRequest = {
  name: {
    minLength: 1
  }
};
const CreateClusterRequestAutoscalerConfig = {
  scaleDownUtilizationThreshold: {
    greaterThan: 0,
    lessThan: 1
  }
};
const CreateClusterRequestOpenIDConnectConfig = {
  clientId: {
    minLength: 1
  },
  groupsPrefix: {
    minLength: 1
  },
  usernameClaim: {
    minLength: 1
  },
  usernamePrefix: {
    minLength: 1
  }
};
const CreateClusterRequestPoolConfig = {
  name: {
    minLength: 1
  }
};
const CreatePoolRequest = {
  name: {
    minLength: 1
  }
};
const ListClusterTypesRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListClustersRequest = {
  name: {
    minLength: 1
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListNodesRequest = {
  name: {
    minLength: 1
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListPoolsRequest = {
  name: {
    minLength: 1
  },
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const MaintenanceWindow = {
  startHour: {
    lessThanOrEqual: 23
  }
};
const UpdateClusterRequest = {
  name: {
    minLength: 1
  }
};
const UpdateClusterRequestAutoscalerConfig = {
  scaleDownUtilizationThreshold: {
    greaterThan: 0,
    lessThan: 1
  }
};

var validationRules_gen$6 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateClusterRequest: CreateClusterRequest,
  CreateClusterRequestAutoscalerConfig: CreateClusterRequestAutoscalerConfig,
  CreateClusterRequestOpenIDConnectConfig: CreateClusterRequestOpenIDConnectConfig,
  CreateClusterRequestPoolConfig: CreateClusterRequestPoolConfig,
  CreatePoolRequest: CreatePoolRequest,
  ListClusterTypesRequest: ListClusterTypesRequest,
  ListClustersRequest: ListClustersRequest,
  ListNodesRequest: ListNodesRequest,
  ListPoolsRequest: ListPoolsRequest,
  MaintenanceWindow: MaintenanceWindow,
  UpdateClusterRequest: UpdateClusterRequest,
  UpdateClusterRequestAutoscalerConfig: UpdateClusterRequestAutoscalerConfig
});

var index$g = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: K8SUtilsAPI,
  CLUSTER_TRANSIENT_STATUSES: CLUSTER_TRANSIENT_STATUSES$1,
  NODE_TRANSIENT_STATUSES: NODE_TRANSIENT_STATUSES,
  POOL_TRANSIENT_STATUSES: POOL_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen$6
});

var index$f = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index$g
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link CertificateStatus}. */
const CERTIFICATE_TRANSIENT_STATUSES = ['pending'];

/** Lists transient statutes of the enum {@link InstanceStatus}. */
const INSTANCE_TRANSIENT_STATUSES$1 = ['pending', 'migrating'];

/** Lists transient statutes of the enum {@link LbStatus}. */
const LB_TRANSIENT_STATUSES = ['pending', 'migrating', 'to_create', 'creating', 'to_delete', 'deleting'];

/** Lists transient statutes of the enum {@link PrivateNetworkStatus}. */
const PRIVATE_NETWORK_TRANSIENT_STATUSES = ['pending'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalIp = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Ip' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    ipAddress: data.ip_address,
    lbId: data.lb_id,
    organizationId: data.organization_id,
    projectId: data.project_id,
    region: data.region,
    reverse: data.reverse,
    zone: data.zone
  };
};
const unmarshalSubscriberEmailConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SubscriberEmailConfig' failed as data isn't a dictionary.`);
  }
  return {
    email: data.email
  };
};
const unmarshalSubscriberWebhookConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SubscriberWebhookConfig' failed as data isn't a dictionary.`);
  }
  return {
    uri: data.uri
  };
};
const unmarshalSubscriber = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Subscriber' failed as data isn't a dictionary.`);
  }
  return {
    emailConfig: data.email_config ? unmarshalSubscriberEmailConfig(data.email_config) : undefined,
    id: data.id,
    name: data.name,
    webhookConfig: data.webhook_config ? unmarshalSubscriberWebhookConfig(data.webhook_config) : undefined
  };
};
const unmarshalHealthCheckHttpConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HealthCheckHttpConfig' failed as data isn't a dictionary.`);
  }
  return {
    code: data.code,
    hostHeader: data.host_header,
    method: data.method,
    uri: data.uri
  };
};
const unmarshalHealthCheckHttpsConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HealthCheckHttpsConfig' failed as data isn't a dictionary.`);
  }
  return {
    code: data.code,
    hostHeader: data.host_header,
    method: data.method,
    sni: data.sni,
    uri: data.uri
  };
};
const unmarshalHealthCheckLdapConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HealthCheckLdapConfig' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalHealthCheckMysqlConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HealthCheckMysqlConfig' failed as data isn't a dictionary.`);
  }
  return {
    user: data.user
  };
};
const unmarshalHealthCheckPgsqlConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HealthCheckPgsqlConfig' failed as data isn't a dictionary.`);
  }
  return {
    user: data.user
  };
};
const unmarshalHealthCheckRedisConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HealthCheckRedisConfig' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalHealthCheckTcpConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HealthCheckTcpConfig' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalHealthCheck = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HealthCheck' failed as data isn't a dictionary.`);
  }
  return {
    checkDelay: data.check_delay,
    checkMaxRetries: data.check_max_retries,
    checkSendProxy: data.check_send_proxy,
    checkTimeout: data.check_timeout,
    httpConfig: data.http_config ? unmarshalHealthCheckHttpConfig(data.http_config) : undefined,
    httpsConfig: data.https_config ? unmarshalHealthCheckHttpsConfig(data.https_config) : undefined,
    ldapConfig: data.ldap_config ? unmarshalHealthCheckLdapConfig(data.ldap_config) : undefined,
    mysqlConfig: data.mysql_config ? unmarshalHealthCheckMysqlConfig(data.mysql_config) : undefined,
    pgsqlConfig: data.pgsql_config ? unmarshalHealthCheckPgsqlConfig(data.pgsql_config) : undefined,
    port: data.port,
    redisConfig: data.redis_config ? unmarshalHealthCheckRedisConfig(data.redis_config) : undefined,
    tcpConfig: data.tcp_config ? unmarshalHealthCheckTcpConfig(data.tcp_config) : undefined,
    transientCheckDelay: data.transient_check_delay
  };
};
const unmarshalInstance$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Instance' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    ipAddress: data.ip_address,
    region: data.region,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalLb = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Lb' failed as data isn't a dictionary.`);
  }
  return {
    backendCount: data.backend_count,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    frontendCount: data.frontend_count,
    id: data.id,
    instances: unmarshalArrayOfObject(data.instances, unmarshalInstance$1),
    ip: unmarshalArrayOfObject(data.ip, unmarshalIp),
    name: data.name,
    organizationId: data.organization_id,
    privateNetworkCount: data.private_network_count,
    projectId: data.project_id,
    region: data.region,
    routeCount: data.route_count,
    sslCompatibilityLevel: data.ssl_compatibility_level,
    status: data.status,
    subscriber: data.subscriber ? unmarshalSubscriber(data.subscriber) : undefined,
    tags: data.tags,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalBackend = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Backend' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    failoverHost: data.failover_host,
    forwardPort: data.forward_port,
    forwardPortAlgorithm: data.forward_port_algorithm,
    forwardProtocol: data.forward_protocol,
    healthCheck: data.health_check ? unmarshalHealthCheck(data.health_check) : undefined,
    id: data.id,
    ignoreSslServerVerify: data.ignore_ssl_server_verify,
    lb: data.lb ? unmarshalLb(data.lb) : undefined,
    maxConnections: data.max_connections,
    maxRetries: data.max_retries,
    name: data.name,
    onMarkedDownAction: data.on_marked_down_action,
    pool: data.pool,
    proxyProtocol: data.proxy_protocol,
    redispatchAttemptCount: data.redispatch_attempt_count,
    sendProxyV2: data.send_proxy_v2,
    sslBridging: data.ssl_bridging,
    stickySessions: data.sticky_sessions,
    stickySessionsCookieName: data.sticky_sessions_cookie_name,
    timeoutConnect: data.timeout_connect,
    timeoutQueue: data.timeout_queue,
    timeoutServer: data.timeout_server,
    timeoutTunnel: data.timeout_tunnel,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalCertificate = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Certificate' failed as data isn't a dictionary.`);
  }
  return {
    commonName: data.common_name,
    createdAt: unmarshalDate(data.created_at),
    fingerprint: data.fingerprint,
    id: data.id,
    lb: data.lb ? unmarshalLb(data.lb) : undefined,
    name: data.name,
    notValidAfter: unmarshalDate(data.not_valid_after),
    notValidBefore: unmarshalDate(data.not_valid_before),
    status: data.status,
    statusDetails: data.status_details,
    subjectAlternativeName: data.subject_alternative_name,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalFrontend = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Frontend' failed as data isn't a dictionary.`);
  }
  return {
    backend: data.backend ? unmarshalBackend(data.backend) : undefined,
    certificate: data.certificate ? unmarshalCertificate(data.certificate) : undefined,
    certificateIds: data.certificate_ids,
    createdAt: unmarshalDate(data.created_at),
    enableHttp3: data.enable_http3,
    id: data.id,
    inboundPort: data.inbound_port,
    lb: data.lb ? unmarshalLb(data.lb) : undefined,
    name: data.name,
    timeoutClient: data.timeout_client,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalAclActionRedirect = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AclActionRedirect' failed as data isn't a dictionary.`);
  }
  return {
    code: data.code,
    target: data.target,
    type: data.type
  };
};
const unmarshalAclAction = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AclAction' failed as data isn't a dictionary.`);
  }
  return {
    redirect: data.redirect ? unmarshalAclActionRedirect(data.redirect) : undefined,
    type: data.type
  };
};
const unmarshalAclMatch = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AclMatch' failed as data isn't a dictionary.`);
  }
  return {
    httpFilter: data.http_filter,
    httpFilterOption: data.http_filter_option,
    httpFilterValue: data.http_filter_value,
    invert: data.invert,
    ipSubnet: data.ip_subnet
  };
};
const unmarshalAcl = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Acl' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action ? unmarshalAclAction(data.action) : undefined,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    frontend: data.frontend ? unmarshalFrontend(data.frontend) : undefined,
    id: data.id,
    index: data.index,
    match: data.match ? unmarshalAclMatch(data.match) : undefined,
    name: data.name,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalPrivateNetworkDHCPConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrivateNetworkDHCPConfig' failed as data isn't a dictionary.`);
  }
  return {
    ipId: data.ip_id
  };
};
const unmarshalPrivateNetworkIpamConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrivateNetworkIpamConfig' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalPrivateNetworkStaticConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrivateNetworkStaticConfig' failed as data isn't a dictionary.`);
  }
  return {
    ipAddress: data.ip_address ? data.ip_address : undefined
  };
};
const unmarshalPrivateNetwork$3 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrivateNetwork' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    dhcpConfig: data.dhcp_config ? unmarshalPrivateNetworkDHCPConfig(data.dhcp_config) : undefined,
    ipamConfig: data.ipam_config ? unmarshalPrivateNetworkIpamConfig(data.ipam_config) : undefined,
    ipamIds: data.ipam_ids,
    lb: data.lb ? unmarshalLb(data.lb) : undefined,
    privateNetworkId: data.private_network_id,
    staticConfig: data.static_config ? unmarshalPrivateNetworkStaticConfig(data.static_config) : undefined,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalRouteMatch = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RouteMatch' failed as data isn't a dictionary.`);
  }
  return {
    hostHeader: data.host_header,
    sni: data.sni
  };
};
const unmarshalRoute = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Route' failed as data isn't a dictionary.`);
  }
  return {
    backendId: data.backend_id,
    createdAt: unmarshalDate(data.created_at),
    frontendId: data.frontend_id,
    id: data.id,
    match: data.match ? unmarshalRouteMatch(data.match) : undefined,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalBackendServerStats = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BackendServerStats' failed as data isn't a dictionary.`);
  }
  return {
    backendId: data.backend_id,
    instanceId: data.instance_id,
    ip: data.ip,
    lastHealthCheckStatus: data.last_health_check_status,
    serverState: data.server_state,
    serverStateChangedAt: unmarshalDate(data.server_state_changed_at)
  };
};
const unmarshalLbStats = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'LbStats' failed as data isn't a dictionary.`);
  }
  return {
    backendServersStats: unmarshalArrayOfObject(data.backend_servers_stats, unmarshalBackendServerStats)
  };
};
const unmarshalListAclResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListAclResponse' failed as data isn't a dictionary.`);
  }
  return {
    acls: unmarshalArrayOfObject(data.acls, unmarshalAcl),
    totalCount: data.total_count
  };
};
const unmarshalListBackendStatsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListBackendStatsResponse' failed as data isn't a dictionary.`);
  }
  return {
    backendServersStats: unmarshalArrayOfObject(data.backend_servers_stats, unmarshalBackendServerStats),
    totalCount: data.total_count
  };
};
const unmarshalListBackendsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListBackendsResponse' failed as data isn't a dictionary.`);
  }
  return {
    backends: unmarshalArrayOfObject(data.backends, unmarshalBackend),
    totalCount: data.total_count
  };
};
const unmarshalListCertificatesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListCertificatesResponse' failed as data isn't a dictionary.`);
  }
  return {
    certificates: unmarshalArrayOfObject(data.certificates, unmarshalCertificate),
    totalCount: data.total_count
  };
};
const unmarshalListFrontendsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListFrontendsResponse' failed as data isn't a dictionary.`);
  }
  return {
    frontends: unmarshalArrayOfObject(data.frontends, unmarshalFrontend),
    totalCount: data.total_count
  };
};
const unmarshalListIpsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListIpsResponse' failed as data isn't a dictionary.`);
  }
  return {
    ips: unmarshalArrayOfObject(data.ips, unmarshalIp),
    totalCount: data.total_count
  };
};
const unmarshalListLbPrivateNetworksResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListLbPrivateNetworksResponse' failed as data isn't a dictionary.`);
  }
  return {
    privateNetwork: unmarshalArrayOfObject(data.private_network, unmarshalPrivateNetwork$3),
    totalCount: data.total_count
  };
};
const unmarshalLbType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'LbType' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    name: data.name,
    region: data.region,
    stockStatus: data.stock_status,
    zone: data.zone
  };
};
const unmarshalListLbTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListLbTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    lbTypes: unmarshalArrayOfObject(data.lb_types, unmarshalLbType),
    totalCount: data.total_count
  };
};
const unmarshalListLbsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListLbsResponse' failed as data isn't a dictionary.`);
  }
  return {
    lbs: unmarshalArrayOfObject(data.lbs, unmarshalLb),
    totalCount: data.total_count
  };
};
const unmarshalListRoutesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListRoutesResponse' failed as data isn't a dictionary.`);
  }
  return {
    routes: unmarshalArrayOfObject(data.routes, unmarshalRoute),
    totalCount: data.total_count
  };
};
const unmarshalListSubscriberResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSubscriberResponse' failed as data isn't a dictionary.`);
  }
  return {
    subscribers: unmarshalArrayOfObject(data.subscribers, unmarshalSubscriber),
    totalCount: data.total_count
  };
};
const unmarshalSetAclsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetAclsResponse' failed as data isn't a dictionary.`);
  }
  return {
    acls: unmarshalArrayOfObject(data.acls, unmarshalAcl),
    totalCount: data.total_count
  };
};
const marshalAddBackendServersRequest = (request, defaults) => ({
  server_ip: request.serverIp
});
const marshalPrivateNetworkDHCPConfig = (request, defaults) => ({
  ip_id: request.ipId
});
const marshalPrivateNetworkIpamConfig = (request, defaults) => ({});
const marshalPrivateNetworkStaticConfig = (request, defaults) => ({
  ip_address: request.ipAddress
});
const marshalAttachPrivateNetworkRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'static_config',
    value: request.staticConfig !== undefined ? marshalPrivateNetworkStaticConfig(request.staticConfig) : undefined
  }, {
    param: 'dhcp_config',
    value: request.dhcpConfig !== undefined ? marshalPrivateNetworkDHCPConfig(request.dhcpConfig) : undefined
  }, {
    param: 'ipam_config',
    value: request.ipamConfig !== undefined ? marshalPrivateNetworkIpamConfig(request.ipamConfig) : undefined
  }])
});
const marshalAclActionRedirect = (request, defaults) => ({
  code: request.code,
  target: request.target,
  type: request.type
});
const marshalAclAction = (request, defaults) => ({
  redirect: request.redirect !== undefined ? marshalAclActionRedirect(request.redirect) : undefined,
  type: request.type
});
const marshalAclMatch = (request, defaults) => ({
  http_filter: request.httpFilter,
  http_filter_option: request.httpFilterOption,
  http_filter_value: request.httpFilterValue,
  invert: request.invert,
  ip_subnet: request.ipSubnet
});
const marshalCreateAclRequest = (request, defaults) => ({
  action: marshalAclAction(request.action),
  description: request.description,
  index: request.index,
  match: request.match !== undefined ? marshalAclMatch(request.match) : undefined,
  name: request.name || randomName('acl')
});
const marshalHealthCheckHttpConfig = (request, defaults) => ({
  code: request.code,
  host_header: request.hostHeader,
  method: request.method,
  uri: request.uri
});
const marshalHealthCheckHttpsConfig = (request, defaults) => ({
  code: request.code,
  host_header: request.hostHeader,
  method: request.method,
  sni: request.sni,
  uri: request.uri
});
const marshalHealthCheckLdapConfig = (request, defaults) => ({});
const marshalHealthCheckMysqlConfig = (request, defaults) => ({
  user: request.user
});
const marshalHealthCheckPgsqlConfig = (request, defaults) => ({
  user: request.user
});
const marshalHealthCheckRedisConfig = (request, defaults) => ({});
const marshalHealthCheckTcpConfig = (request, defaults) => ({});
const marshalHealthCheck = (request, defaults) => ({
  check_delay: request.checkDelay,
  check_max_retries: request.checkMaxRetries,
  check_send_proxy: request.checkSendProxy,
  check_timeout: request.checkTimeout,
  port: request.port,
  transient_check_delay: request.transientCheckDelay,
  ...resolveOneOf([{
    param: 'tcp_config',
    value: request.tcpConfig !== undefined ? marshalHealthCheckTcpConfig(request.tcpConfig) : undefined
  }, {
    param: 'mysql_config',
    value: request.mysqlConfig !== undefined ? marshalHealthCheckMysqlConfig(request.mysqlConfig) : undefined
  }, {
    param: 'pgsql_config',
    value: request.pgsqlConfig !== undefined ? marshalHealthCheckPgsqlConfig(request.pgsqlConfig) : undefined
  }, {
    param: 'ldap_config',
    value: request.ldapConfig !== undefined ? marshalHealthCheckLdapConfig(request.ldapConfig) : undefined
  }, {
    param: 'redis_config',
    value: request.redisConfig !== undefined ? marshalHealthCheckRedisConfig(request.redisConfig) : undefined
  }, {
    param: 'http_config',
    value: request.httpConfig !== undefined ? marshalHealthCheckHttpConfig(request.httpConfig) : undefined
  }, {
    param: 'https_config',
    value: request.httpsConfig !== undefined ? marshalHealthCheckHttpsConfig(request.httpsConfig) : undefined
  }])
});
const marshalCreateBackendRequest = (request, defaults) => ({
  failover_host: request.failoverHost,
  forward_port: request.forwardPort,
  forward_port_algorithm: request.forwardPortAlgorithm,
  forward_protocol: request.forwardProtocol,
  health_check: marshalHealthCheck(request.healthCheck),
  ignore_ssl_server_verify: request.ignoreSslServerVerify,
  max_connections: request.maxConnections,
  max_retries: request.maxRetries,
  name: request.name || randomName('lbb'),
  on_marked_down_action: request.onMarkedDownAction,
  proxy_protocol: request.proxyProtocol,
  redispatch_attempt_count: request.redispatchAttemptCount,
  send_proxy_v2: request.sendProxyV2,
  server_ip: request.serverIp,
  ssl_bridging: request.sslBridging,
  sticky_sessions: request.stickySessions,
  sticky_sessions_cookie_name: request.stickySessionsCookieName,
  timeout_connect: request.timeoutConnect,
  timeout_queue: request.timeoutQueue,
  timeout_server: request.timeoutServer,
  timeout_tunnel: request.timeoutTunnel
});
const marshalCreateCertificateRequestCustomCertificate = (request, defaults) => ({
  certificate_chain: request.certificateChain
});
const marshalCreateCertificateRequestLetsencryptConfig = (request, defaults) => ({
  common_name: request.commonName,
  subject_alternative_name: request.subjectAlternativeName
});
const marshalCreateCertificateRequest = (request, defaults) => ({
  name: request.name || randomName('certificate'),
  ...resolveOneOf([{
    param: 'letsencrypt',
    value: request.letsencrypt !== undefined ? marshalCreateCertificateRequestLetsencryptConfig(request.letsencrypt) : undefined
  }, {
    param: 'custom_certificate',
    value: request.customCertificate !== undefined ? marshalCreateCertificateRequestCustomCertificate(request.customCertificate) : undefined
  }])
});
const marshalCreateFrontendRequest = (request, defaults) => ({
  backend_id: request.backendId,
  certificate_id: request.certificateId,
  certificate_ids: request.certificateIds,
  enable_http3: request.enableHttp3,
  inbound_port: request.inboundPort,
  name: request.name || randomName('lbf'),
  timeout_client: request.timeoutClient
});
const marshalCreateIpRequest = (request, defaults) => ({
  is_ipv6: request.isIpv6,
  reverse: request.reverse,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalCreateLbRequest = (request, defaults) => ({
  assign_flexible_ip: request.assignFlexibleIp,
  assign_flexible_ipv6: request.assignFlexibleIpv6,
  description: request.description,
  ip_id: request.ipId,
  ip_ids: request.ipIds,
  name: request.name || randomName('lb'),
  ssl_compatibility_level: request.sslCompatibilityLevel,
  tags: request.tags,
  type: request.type,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalRouteMatch = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'sni',
    value: request.sni
  }, {
    param: 'host_header',
    value: request.hostHeader
  }])
});
const marshalCreateRouteRequest = (request, defaults) => ({
  backend_id: request.backendId,
  frontend_id: request.frontendId,
  match: request.match !== undefined ? marshalRouteMatch(request.match) : undefined
});
const marshalSubscriberEmailConfig = (request, defaults) => ({
  email: request.email
});
const marshalSubscriberWebhookConfig = (request, defaults) => ({
  uri: request.uri
});
const marshalCreateSubscriberRequest = (request, defaults) => ({
  name: request.name,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }]),
  ...resolveOneOf([{
    param: 'email_config',
    value: request.emailConfig !== undefined ? marshalSubscriberEmailConfig(request.emailConfig) : undefined
  }, {
    param: 'webhook_config',
    value: request.webhookConfig !== undefined ? marshalSubscriberWebhookConfig(request.webhookConfig) : undefined
  }])
});
const marshalMigrateLbRequest = (request, defaults) => ({
  type: request.type
});
const marshalRemoveBackendServersRequest = (request, defaults) => ({
  server_ip: request.serverIp
});
const marshalSetBackendServersRequest = (request, defaults) => ({
  server_ip: request.serverIp
});
const marshalSubscribeToLbRequest = (request, defaults) => ({
  subscriber_id: request.subscriberId
});
const marshalUpdateAclRequest = (request, defaults) => ({
  action: marshalAclAction(request.action),
  description: request.description,
  index: request.index,
  match: request.match !== undefined ? marshalAclMatch(request.match) : undefined,
  name: request.name
});
const marshalUpdateBackendRequest = (request, defaults) => ({
  failover_host: request.failoverHost,
  forward_port: request.forwardPort,
  forward_port_algorithm: request.forwardPortAlgorithm,
  forward_protocol: request.forwardProtocol,
  ignore_ssl_server_verify: request.ignoreSslServerVerify,
  max_connections: request.maxConnections,
  max_retries: request.maxRetries,
  name: request.name,
  on_marked_down_action: request.onMarkedDownAction,
  proxy_protocol: request.proxyProtocol,
  redispatch_attempt_count: request.redispatchAttemptCount,
  send_proxy_v2: request.sendProxyV2,
  ssl_bridging: request.sslBridging,
  sticky_sessions: request.stickySessions,
  sticky_sessions_cookie_name: request.stickySessionsCookieName,
  timeout_connect: request.timeoutConnect,
  timeout_queue: request.timeoutQueue,
  timeout_server: request.timeoutServer,
  timeout_tunnel: request.timeoutTunnel
});
const marshalUpdateCertificateRequest = (request, defaults) => ({
  name: request.name
});
const marshalUpdateFrontendRequest = (request, defaults) => ({
  backend_id: request.backendId,
  certificate_id: request.certificateId,
  certificate_ids: request.certificateIds,
  enable_http3: request.enableHttp3,
  inbound_port: request.inboundPort,
  name: request.name,
  timeout_client: request.timeoutClient
});
const marshalUpdateHealthCheckRequest = (request, defaults) => ({
  check_delay: request.checkDelay,
  check_max_retries: request.checkMaxRetries,
  check_send_proxy: request.checkSendProxy,
  check_timeout: request.checkTimeout,
  port: request.port,
  transient_check_delay: request.transientCheckDelay,
  ...resolveOneOf([{
    param: 'tcp_config',
    value: request.tcpConfig !== undefined ? marshalHealthCheckTcpConfig(request.tcpConfig) : undefined
  }, {
    param: 'mysql_config',
    value: request.mysqlConfig !== undefined ? marshalHealthCheckMysqlConfig(request.mysqlConfig) : undefined
  }, {
    param: 'pgsql_config',
    value: request.pgsqlConfig !== undefined ? marshalHealthCheckPgsqlConfig(request.pgsqlConfig) : undefined
  }, {
    param: 'ldap_config',
    value: request.ldapConfig !== undefined ? marshalHealthCheckLdapConfig(request.ldapConfig) : undefined
  }, {
    param: 'redis_config',
    value: request.redisConfig !== undefined ? marshalHealthCheckRedisConfig(request.redisConfig) : undefined
  }, {
    param: 'http_config',
    value: request.httpConfig !== undefined ? marshalHealthCheckHttpConfig(request.httpConfig) : undefined
  }, {
    param: 'https_config',
    value: request.httpsConfig !== undefined ? marshalHealthCheckHttpsConfig(request.httpsConfig) : undefined
  }])
});
const marshalUpdateIpRequest = (request, defaults) => ({
  lb_id: request.lbId,
  reverse: request.reverse
});
const marshalUpdateLbRequest = (request, defaults) => ({
  description: request.description,
  name: request.name,
  ssl_compatibility_level: request.sslCompatibilityLevel,
  tags: request.tags
});
const marshalUpdateRouteRequest = (request, defaults) => ({
  backend_id: request.backendId,
  match: request.match !== undefined ? marshalRouteMatch(request.match) : undefined
});
const marshalUpdateSubscriberRequest = (request, defaults) => ({
  name: request.name,
  ...resolveOneOf([{
    param: 'email_config',
    value: request.emailConfig !== undefined ? marshalSubscriberEmailConfig(request.emailConfig) : undefined
  }, {
    param: 'webhook_config',
    value: request.webhookConfig !== undefined ? marshalSubscriberWebhookConfig(request.webhookConfig) : undefined
  }])
});
const marshalZonedApiAddBackendServersRequest = (request, defaults) => ({
  server_ip: request.serverIp
});
const marshalZonedApiAttachPrivateNetworkRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'static_config',
    value: request.staticConfig !== undefined ? marshalPrivateNetworkStaticConfig(request.staticConfig) : undefined
  }, {
    param: 'dhcp_config',
    value: request.dhcpConfig !== undefined ? marshalPrivateNetworkDHCPConfig(request.dhcpConfig) : undefined
  }, {
    param: 'ipam_config',
    value: request.ipamConfig !== undefined ? marshalPrivateNetworkIpamConfig(request.ipamConfig) : undefined
  }])
});
const marshalZonedApiCreateAclRequest = (request, defaults) => ({
  action: marshalAclAction(request.action),
  description: request.description,
  index: request.index,
  match: request.match !== undefined ? marshalAclMatch(request.match) : undefined,
  name: request.name || randomName('acl')
});
const marshalZonedApiCreateBackendRequest = (request, defaults) => ({
  failover_host: request.failoverHost,
  forward_port: request.forwardPort,
  forward_port_algorithm: request.forwardPortAlgorithm,
  forward_protocol: request.forwardProtocol,
  health_check: marshalHealthCheck(request.healthCheck),
  ignore_ssl_server_verify: request.ignoreSslServerVerify,
  max_connections: request.maxConnections,
  max_retries: request.maxRetries,
  name: request.name || randomName('lbb'),
  on_marked_down_action: request.onMarkedDownAction,
  proxy_protocol: request.proxyProtocol,
  redispatch_attempt_count: request.redispatchAttemptCount,
  send_proxy_v2: request.sendProxyV2,
  server_ip: request.serverIp,
  ssl_bridging: request.sslBridging,
  sticky_sessions: request.stickySessions,
  sticky_sessions_cookie_name: request.stickySessionsCookieName,
  timeout_connect: request.timeoutConnect,
  timeout_queue: request.timeoutQueue,
  timeout_server: request.timeoutServer,
  timeout_tunnel: request.timeoutTunnel
});
const marshalZonedApiCreateCertificateRequest = (request, defaults) => ({
  name: request.name || randomName('certificate'),
  ...resolveOneOf([{
    param: 'letsencrypt',
    value: request.letsencrypt !== undefined ? marshalCreateCertificateRequestLetsencryptConfig(request.letsencrypt) : undefined
  }, {
    param: 'custom_certificate',
    value: request.customCertificate !== undefined ? marshalCreateCertificateRequestCustomCertificate(request.customCertificate) : undefined
  }])
});
const marshalZonedApiCreateFrontendRequest = (request, defaults) => ({
  backend_id: request.backendId,
  certificate_id: request.certificateId,
  certificate_ids: request.certificateIds,
  enable_http3: request.enableHttp3,
  inbound_port: request.inboundPort,
  name: request.name || randomName('lbf'),
  timeout_client: request.timeoutClient
});
const marshalZonedApiCreateIpRequest = (request, defaults) => ({
  is_ipv6: request.isIpv6,
  reverse: request.reverse,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalZonedApiCreateLbRequest = (request, defaults) => ({
  assign_flexible_ip: request.assignFlexibleIp,
  assign_flexible_ipv6: request.assignFlexibleIpv6,
  description: request.description,
  ip_id: request.ipId,
  ip_ids: request.ipIds,
  name: request.name || randomName('lb'),
  ssl_compatibility_level: request.sslCompatibilityLevel,
  tags: request.tags,
  type: request.type,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalZonedApiCreateRouteRequest = (request, defaults) => ({
  backend_id: request.backendId,
  frontend_id: request.frontendId,
  match: request.match !== undefined ? marshalRouteMatch(request.match) : undefined
});
const marshalZonedApiCreateSubscriberRequest = (request, defaults) => ({
  name: request.name,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }]),
  ...resolveOneOf([{
    param: 'email_config',
    value: request.emailConfig !== undefined ? marshalSubscriberEmailConfig(request.emailConfig) : undefined
  }, {
    param: 'webhook_config',
    value: request.webhookConfig !== undefined ? marshalSubscriberWebhookConfig(request.webhookConfig) : undefined
  }])
});
const marshalZonedApiMigrateLbRequest = (request, defaults) => ({
  type: request.type
});
const marshalZonedApiRemoveBackendServersRequest = (request, defaults) => ({
  server_ip: request.serverIp
});
const marshalAclSpec = (request, defaults) => ({
  action: marshalAclAction(request.action),
  description: request.description,
  index: request.index,
  match: request.match !== undefined ? marshalAclMatch(request.match) : undefined,
  name: request.name
});
const marshalZonedApiSetAclsRequest = (request, defaults) => ({
  acls: request.acls.map(elt => marshalAclSpec(elt))
});
const marshalZonedApiSetBackendServersRequest = (request, defaults) => ({
  server_ip: request.serverIp
});
const marshalZonedApiSubscribeToLbRequest = (request, defaults) => ({
  subscriber_id: request.subscriberId
});
const marshalZonedApiUpdateAclRequest = (request, defaults) => ({
  action: marshalAclAction(request.action),
  description: request.description,
  index: request.index,
  match: request.match !== undefined ? marshalAclMatch(request.match) : undefined,
  name: request.name
});
const marshalZonedApiUpdateBackendRequest = (request, defaults) => ({
  failover_host: request.failoverHost,
  forward_port: request.forwardPort,
  forward_port_algorithm: request.forwardPortAlgorithm,
  forward_protocol: request.forwardProtocol,
  ignore_ssl_server_verify: request.ignoreSslServerVerify,
  max_connections: request.maxConnections,
  max_retries: request.maxRetries,
  name: request.name,
  on_marked_down_action: request.onMarkedDownAction,
  proxy_protocol: request.proxyProtocol,
  redispatch_attempt_count: request.redispatchAttemptCount,
  send_proxy_v2: request.sendProxyV2,
  ssl_bridging: request.sslBridging,
  sticky_sessions: request.stickySessions,
  sticky_sessions_cookie_name: request.stickySessionsCookieName,
  timeout_connect: request.timeoutConnect,
  timeout_queue: request.timeoutQueue,
  timeout_server: request.timeoutServer,
  timeout_tunnel: request.timeoutTunnel
});
const marshalZonedApiUpdateCertificateRequest = (request, defaults) => ({
  name: request.name
});
const marshalZonedApiUpdateFrontendRequest = (request, defaults) => ({
  backend_id: request.backendId,
  certificate_id: request.certificateId,
  certificate_ids: request.certificateIds,
  enable_http3: request.enableHttp3,
  inbound_port: request.inboundPort,
  name: request.name,
  timeout_client: request.timeoutClient
});
const marshalZonedApiUpdateHealthCheckRequest = (request, defaults) => ({
  check_delay: request.checkDelay,
  check_max_retries: request.checkMaxRetries,
  check_send_proxy: request.checkSendProxy,
  check_timeout: request.checkTimeout,
  port: request.port,
  transient_check_delay: request.transientCheckDelay,
  ...resolveOneOf([{
    param: 'tcp_config',
    value: request.tcpConfig !== undefined ? marshalHealthCheckTcpConfig(request.tcpConfig) : undefined
  }, {
    param: 'mysql_config',
    value: request.mysqlConfig !== undefined ? marshalHealthCheckMysqlConfig(request.mysqlConfig) : undefined
  }, {
    param: 'pgsql_config',
    value: request.pgsqlConfig !== undefined ? marshalHealthCheckPgsqlConfig(request.pgsqlConfig) : undefined
  }, {
    param: 'ldap_config',
    value: request.ldapConfig !== undefined ? marshalHealthCheckLdapConfig(request.ldapConfig) : undefined
  }, {
    param: 'redis_config',
    value: request.redisConfig !== undefined ? marshalHealthCheckRedisConfig(request.redisConfig) : undefined
  }, {
    param: 'http_config',
    value: request.httpConfig !== undefined ? marshalHealthCheckHttpConfig(request.httpConfig) : undefined
  }, {
    param: 'https_config',
    value: request.httpsConfig !== undefined ? marshalHealthCheckHttpsConfig(request.httpsConfig) : undefined
  }])
});
const marshalZonedApiUpdateIpRequest = (request, defaults) => ({
  lb_id: request.lbId,
  reverse: request.reverse
});
const marshalZonedApiUpdateLbRequest = (request, defaults) => ({
  description: request.description,
  name: request.name,
  ssl_compatibility_level: request.sslCompatibilityLevel,
  tags: request.tags
});
const marshalZonedApiUpdateRouteRequest = (request, defaults) => ({
  backend_id: request.backendId,
  match: request.match !== undefined ? marshalRouteMatch(request.match) : undefined
});
const marshalZonedApiUpdateSubscriberRequest = (request, defaults) => ({
  name: request.name,
  ...resolveOneOf([{
    param: 'email_config',
    value: request.emailConfig !== undefined ? marshalSubscriberEmailConfig(request.emailConfig) : undefined
  }, {
    param: 'webhook_config',
    value: request.webhookConfig !== undefined ? marshalSubscriberWebhookConfig(request.webhookConfig) : undefined
  }])
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$e = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Load Balancer API.
 *
 * This API allows you to manage your Scaleway Load Balancer services.
 */
class ZonedAPI extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'nl-ams-1', 'nl-ams-2', 'nl-ams-3', 'pl-waw-1', 'pl-waw-2', 'pl-waw-3'];
  pageOfListLbs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalListLbsResponse);

  /**
   * List Load Balancers. List all Load Balancers in the specified zone, for a
   * Scaleway Organization or Scaleway Project. By default, the Load Balancers
   * returned in the list are ordered by creation date in ascending order,
   * though this can be modified via the `order_by` field.
   *
   * @param request - The request {@link ZonedApiListLbsRequest}
   * @returns A Promise of ListLbsResponse
   */
  listLbs = (request = {}) => enrichForPagination('lbs', this.pageOfListLbs, request);

  /**
   * Create a Load Balancer. Create a new Load Balancer. Note that the Load
   * Balancer will be created without frontends or backends; these must be
   * created separately via the dedicated endpoints.
   *
   * @param request - The request {@link ZonedApiCreateLbRequest}
   * @returns A Promise of Lb
   */
  createLb = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiCreateLbRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs`
  }, unmarshalLb);

  /**
   * Get a Load Balancer. Retrieve information about an existing Load Balancer,
   * specified by its Load Balancer ID. Its full details, including name, status
   * and IP address, are returned in the response object.
   *
   * @param request - The request {@link ZonedApiGetLbRequest}
   * @returns A Promise of Lb
   */
  getLb = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}`
  }, unmarshalLb);

  /**
   * Waits for {@link Lb} to be in a final state.
   *
   * @param request - The request {@link ZonedApiGetLbRequest}
   * @param options - The waiting options
   * @returns A Promise of Lb
   */
  waitForLb = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!LB_TRANSIENT_STATUSES.includes(res.status))), this.getLb, request, options);

  /**
   * Update a Load Balancer. Update the parameters of an existing Load Balancer,
   * specified by its Load Balancer ID. Note that the request type is PUT and
   * not PATCH. You must set all parameters.
   *
   * @param request - The request {@link ZonedApiUpdateLbRequest}
   * @returns A Promise of Lb
   */
  updateLb = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateLbRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}`
  }, unmarshalLb);

  /**
   * Delete a Load Balancer. Delete an existing Load Balancer, specified by its
   * Load Balancer ID. Deleting a Load Balancer is permanent, and cannot be
   * undone. The Load Balancer's flexible IP address can either be deleted with
   * the Load Balancer, or kept in your account for future use.
   *
   * @param request - The request {@link ZonedApiDeleteLbRequest}
   */
  deleteLb = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}`,
    urlParams: urlParams(['release_ip', request.releaseIp])
  });

  /**
   * Migrate a Load Balancer. Migrate an existing Load Balancer from one
   * commercial type to another. Allows you to scale your Load Balancer up or
   * down in terms of bandwidth or multi-cloud provision.
   *
   * @param request - The request {@link ZonedApiMigrateLbRequest}
   * @returns A Promise of Lb
   */
  migrateLb = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiMigrateLbRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/migrate`
  }, unmarshalLb);
  pageOfListIPs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips`,
    urlParams: urlParams(['ip_address', request.ipAddress], ['ip_type', request.ipType], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListIpsResponse);

  /**
   * List IP addresses. List the Load Balancer flexible IP addresses held in the
   * account (filtered by Organization ID or Project ID). It is also possible to
   * search for a specific IP address.
   *
   * @param request - The request {@link ZonedApiListIPsRequest}
   * @returns A Promise of ListIpsResponse
   */
  listIPs = (request = {}) => enrichForPagination('ips', this.pageOfListIPs, request);

  /**
   * Create an IP address. Create a new Load Balancer flexible IP address, in
   * the specified Scaleway Project. This can be attached to new Load Balancers
   * created in the future.
   *
   * @param request - The request {@link ZonedApiCreateIpRequest}
   * @returns A Promise of Ip
   */
  createIp = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiCreateIpRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips`
  }, unmarshalIp);

  /**
   * Get an IP address. Retrieve the full details of a Load Balancer flexible IP
   * address.
   *
   * @param request - The request {@link ZonedApiGetIpRequest}
   * @returns A Promise of Ip
   */
  getIp = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIp);

  /**
   * Delete an IP address. Delete a Load Balancer flexible IP address. This
   * action is irreversible, and cannot be undone.
   *
   * @param request - The request {@link ZonedApiReleaseIpRequest}
   */
  releaseIp = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ipId', request.ipId)}`
  });

  /**
   * Update an IP address. Update the reverse DNS of a Load Balancer flexible IP
   * address.
   *
   * @param request - The request {@link ZonedApiUpdateIpRequest}
   * @returns A Promise of Ip
   */
  updateIp = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateIpRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PATCH',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIp);
  pageOfListBackends = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/backends`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListBackendsResponse);

  /**
   * List the backends of a given Load Balancer. List all the backends of a Load
   * Balancer, specified by its Load Balancer ID. By default, results are
   * returned in ascending order by the creation date of each backend. The
   * response is an array of backend objects, containing full details of each
   * one including their configuration parameters such as protocol, port and
   * forwarding algorithm.
   *
   * @param request - The request {@link ZonedApiListBackendsRequest}
   * @returns A Promise of ListBackendsResponse
   */
  listBackends = request => enrichForPagination('backends', this.pageOfListBackends, request);

  /**
   * Create a backend for a given Load Balancer. Create a new backend for a
   * given Load Balancer, specifying its full configuration including protocol,
   * port and forwarding algorithm.
   *
   * @param request - The request {@link ZonedApiCreateBackendRequest}
   * @returns A Promise of Backend
   */
  createBackend = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiCreateBackendRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/backends`
  }, unmarshalBackend);

  /**
   * Get a backend of a given Load Balancer. Get the full details of a given
   * backend, specified by its backend ID. The response contains the backend's
   * full configuration parameters including protocol, port and forwarding
   * algorithm.
   *
   * @param request - The request {@link ZonedApiGetBackendRequest}
   * @returns A Promise of Backend
   */
  getBackend = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/backends/${validatePathParam('backendId', request.backendId)}`
  }, unmarshalBackend);

  /**
   * Update a backend of a given Load Balancer. Update a backend of a given Load
   * Balancer, specified by its backend ID. Note that the request type is PUT
   * and not PATCH. You must set all parameters.
   *
   * @param request - The request {@link ZonedApiUpdateBackendRequest}
   * @returns A Promise of Backend
   */
  updateBackend = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateBackendRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/backends/${validatePathParam('backendId', request.backendId)}`
  }, unmarshalBackend);

  /**
   * Delete a backend of a given Load Balancer. Delete a backend of a given Load
   * Balancer, specified by its backend ID. This action is irreversible and
   * cannot be undone.
   *
   * @param request - The request {@link ZonedApiDeleteBackendRequest}
   */
  deleteBackend = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/backends/${validatePathParam('backendId', request.backendId)}`
  });

  /**
   * Add a set of backend servers to a given backend. For a given backend
   * specified by its backend ID, add a set of backend servers (identified by
   * their IP addresses) it should forward traffic to. These will be appended to
   * any existing set of backend servers for this backend.
   *
   * @param request - The request {@link ZonedApiAddBackendServersRequest}
   * @returns A Promise of Backend
   */
  addBackendServers = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiAddBackendServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/backends/${validatePathParam('backendId', request.backendId)}/servers`
  }, unmarshalBackend);

  /**
   * Remove a set of servers for a given backend. For a given backend specified
   * by its backend ID, remove the specified backend servers (identified by
   * their IP addresses) so that it no longer forwards traffic to them.
   *
   * @param request - The request {@link ZonedApiRemoveBackendServersRequest}
   * @returns A Promise of Backend
   */
  removeBackendServers = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiRemoveBackendServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/backends/${validatePathParam('backendId', request.backendId)}/servers`
  }, unmarshalBackend);

  /**
   * Define all backend servers for a given backend. For a given backend
   * specified by its backend ID, define the set of backend servers (identified
   * by their IP addresses) that it should forward traffic to. Any existing
   * backend servers configured for this backend will be removed.
   *
   * @param request - The request {@link ZonedApiSetBackendServersRequest}
   * @returns A Promise of Backend
   */
  setBackendServers = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiSetBackendServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/backends/${validatePathParam('backendId', request.backendId)}/servers`
  }, unmarshalBackend);

  /**
   * Update a health check for a given backend. Update the configuration of the
   * health check performed by a given backend to verify the health of its
   * backend servers, identified by its backend ID. Note that the request type
   * is PUT and not PATCH. You must set all parameters.
   *
   * @param request - The request {@link ZonedApiUpdateHealthCheckRequest}
   * @returns A Promise of HealthCheck
   */
  updateHealthCheck = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateHealthCheckRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/backends/${validatePathParam('backendId', request.backendId)}/healthcheck`
  }, unmarshalHealthCheck);
  pageOfListFrontends = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/frontends`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListFrontendsResponse);

  /**
   * List frontends of a given Load Balancer. List all the frontends of a Load
   * Balancer, specified by its Load Balancer ID. By default, results are
   * returned in ascending order by the creation date of each frontend. The
   * response is an array of frontend objects, containing full details of each
   * one including the port they listen on and the backend they are attached
   * to.
   *
   * @param request - The request {@link ZonedApiListFrontendsRequest}
   * @returns A Promise of ListFrontendsResponse
   */
  listFrontends = request => enrichForPagination('frontends', this.pageOfListFrontends, request);

  /**
   * Create a frontend in a given Load Balancer. Create a new frontend for a
   * given Load Balancer, specifying its configuration including the port it
   * should listen on and the backend to attach it to.
   *
   * @param request - The request {@link ZonedApiCreateFrontendRequest}
   * @returns A Promise of Frontend
   */
  createFrontend = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiCreateFrontendRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/frontends`
  }, unmarshalFrontend);

  /**
   * Get a frontend. Get the full details of a given frontend, specified by its
   * frontend ID. The response contains the frontend's full configuration
   * parameters including the backend it is attached to, the port it listens on,
   * and any certificates it has.
   *
   * @param request - The request {@link ZonedApiGetFrontendRequest}
   * @returns A Promise of Frontend
   */
  getFrontend = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/frontends/${validatePathParam('frontendId', request.frontendId)}`
  }, unmarshalFrontend);

  /**
   * Update a frontend. Update a given frontend, specified by its frontend ID.
   * You can update configuration parameters including its name and the port it
   * listens on. Note that the request type is PUT and not PATCH. You must set
   * all parameters.
   *
   * @param request - The request {@link ZonedApiUpdateFrontendRequest}
   * @returns A Promise of Frontend
   */
  updateFrontend = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateFrontendRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/frontends/${validatePathParam('frontendId', request.frontendId)}`
  }, unmarshalFrontend);

  /**
   * Delete a frontend. Delete a given frontend, specified by its frontend ID.
   * This action is irreversible and cannot be undone.
   *
   * @param request - The request {@link ZonedApiDeleteFrontendRequest}
   */
  deleteFrontend = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/frontends/${validatePathParam('frontendId', request.frontendId)}`
  });
  pageOfListRoutes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/routes`,
    urlParams: urlParams(['frontend_id', request.frontendId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListRoutesResponse);

  /**
   * List all routes. List all routes for a given frontend. The response is an
   * array of routes, each one with a specified backend to direct to if a
   * certain condition is matched (based on the value of the SNI field or HTTP
   * Host header).
   *
   * @param request - The request {@link ZonedApiListRoutesRequest}
   * @returns A Promise of ListRoutesResponse
   */
  listRoutes = (request = {}) => enrichForPagination('routes', this.pageOfListRoutes, request);

  /**
   * Create a route. Create a new route on a given frontend. To configure a
   * route, specify the backend to direct to if a certain condition is matched
   * (based on the value of the SNI field or HTTP Host header).
   *
   * @param request - The request {@link ZonedApiCreateRouteRequest}
   * @returns A Promise of Route
   */
  createRoute = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiCreateRouteRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/routes`
  }, unmarshalRoute);

  /**
   * Get a route. Retrieve information about an existing route, specified by its
   * route ID. Its full details, origin frontend, target backend and match
   * condition, are returned in the response object.
   *
   * @param request - The request {@link ZonedApiGetRouteRequest}
   * @returns A Promise of Route
   */
  getRoute = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/routes/${validatePathParam('routeId', request.routeId)}`
  }, unmarshalRoute);

  /**
   * Update a route. Update the configuration of an existing route, specified by
   * its route ID.
   *
   * @param request - The request {@link ZonedApiUpdateRouteRequest}
   * @returns A Promise of Route
   */
  updateRoute = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateRouteRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/routes/${validatePathParam('routeId', request.routeId)}`
  }, unmarshalRoute);

  /**
   * Delete a route. Delete an existing route, specified by its route ID.
   * Deleting a route is permanent, and cannot be undone.
   *
   * @param request - The request {@link ZonedApiDeleteRouteRequest}
   */
  deleteRoute = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/routes/${validatePathParam('routeId', request.routeId)}`
  });

  /**
   * Get usage statistics of a given Load Balancer.
   *
   * @deprecated
   * @param request - The request {@link ZonedApiGetLbStatsRequest}
   * @returns A Promise of LbStats
   */
  getLbStats = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/stats`,
    urlParams: urlParams(['backend_id', request.backendId])
  }, unmarshalLbStats);
  pageOfListBackendStats = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/backend-stats`,
    urlParams: urlParams(['backend_id', request.backendId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListBackendStatsResponse);

  /**
   * List backend server statistics. List information about your backend
   * servers, including their state and the result of their last health check.
   *
   * @param request - The request {@link ZonedApiListBackendStatsRequest}
   * @returns A Promise of ListBackendStatsResponse
   */
  listBackendStats = request => enrichForPagination('backendServersStats', this.pageOfListBackendStats, request);
  pageOfListAcls = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/frontends/${validatePathParam('frontendId', request.frontendId)}/acls`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListAclResponse);

  /**
   * List ACLs for a given frontend. List the ACLs for a given frontend,
   * specified by its frontend ID. The response is an array of ACL objects, each
   * one representing an ACL that denies or allows traffic based on certain
   * conditions.
   *
   * @param request - The request {@link ZonedApiListAclsRequest}
   * @returns A Promise of ListAclResponse
   */
  listAcls = request => enrichForPagination('acls', this.pageOfListAcls, request);

  /**
   * Create an ACL for a given frontend. Create a new ACL for a given frontend.
   * Each ACL must have a name, an action to perform (allow or deny), and a
   * match rule (the action is carried out when the incoming traffic matches the
   * rule).
   *
   * @param request - The request {@link ZonedApiCreateAclRequest}
   * @returns A Promise of Acl
   */
  createAcl = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiCreateAclRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/frontends/${validatePathParam('frontendId', request.frontendId)}/acls`
  }, unmarshalAcl);

  /**
   * Get an ACL. Get information for a particular ACL, specified by its ACL ID.
   * The response returns full details of the ACL, including its name, action,
   * match rule and frontend.
   *
   * @param request - The request {@link ZonedApiGetAclRequest}
   * @returns A Promise of Acl
   */
  getAcl = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/acls/${validatePathParam('aclId', request.aclId)}`
  }, unmarshalAcl);

  /**
   * Update an ACL. Update a particular ACL, specified by its ACL ID. You can
   * update details including its name, action and match rule.
   *
   * @param request - The request {@link ZonedApiUpdateAclRequest}
   * @returns A Promise of Acl
   */
  updateAcl = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateAclRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/acls/${validatePathParam('aclId', request.aclId)}`
  }, unmarshalAcl);

  /**
   * Delete an ACL. Delete an ACL, specified by its ACL ID. Deleting an ACL is
   * irreversible and cannot be undone.
   *
   * @param request - The request {@link ZonedApiDeleteAclRequest}
   */
  deleteAcl = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/acls/${validatePathParam('aclId', request.aclId)}`
  });

  /**
   * Define all ACLs for a given frontend. For a given frontend specified by its
   * frontend ID, define and add the complete set of ACLS for that frontend. Any
   * existing ACLs on this frontend will be removed.
   *
   * @param request - The request {@link ZonedApiSetAclsRequest}
   * @returns A Promise of SetAclsResponse
   */
  setAcls = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiSetAclsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/frontends/${validatePathParam('frontendId', request.frontendId)}/acls`
  }, unmarshalSetAclsResponse);

  /**
   * Create an SSL/TLS certificate. Generate a new SSL/TLS certificate for a
   * given Load Balancer. You can choose to create a Let's Encrypt certificate,
   * or import a custom certificate.
   *
   * @param request - The request {@link ZonedApiCreateCertificateRequest}
   * @returns A Promise of Certificate
   */
  createCertificate = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiCreateCertificateRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/certificates`
  }, unmarshalCertificate);
  pageOfListCertificates = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/certificates`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListCertificatesResponse);

  /**
   * List all SSL/TLS certificates on a given Load Balancer. List all the
   * SSL/TLS certificates on a given Load Balancer. The response is an array of
   * certificate objects, which are by default listed in ascending order of
   * creation date.
   *
   * @param request - The request {@link ZonedApiListCertificatesRequest}
   * @returns A Promise of ListCertificatesResponse
   */
  listCertificates = request => enrichForPagination('certificates', this.pageOfListCertificates, request);

  /**
   * Get an SSL/TLS certificate. Get information for a particular SSL/TLS
   * certificate, specified by its certificate ID. The response returns full
   * details of the certificate, including its type, main domain name, and
   * alternative domain names.
   *
   * @param request - The request {@link ZonedApiGetCertificateRequest}
   * @returns A Promise of Certificate
   */
  getCertificate = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/certificates/${validatePathParam('certificateId', request.certificateId)}`
  }, unmarshalCertificate);

  /**
   * Waits for {@link Certificate} to be in a final state.
   *
   * @param request - The request {@link ZonedApiGetCertificateRequest}
   * @param options - The waiting options
   * @returns A Promise of Certificate
   */
  waitForCertificate = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!CERTIFICATE_TRANSIENT_STATUSES.includes(res.status))), this.getCertificate, request, options);

  /**
   * Update an SSL/TLS certificate. Update the name of a particular SSL/TLS
   * certificate, specified by its certificate ID.
   *
   * @param request - The request {@link ZonedApiUpdateCertificateRequest}
   * @returns A Promise of Certificate
   */
  updateCertificate = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateCertificateRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/certificates/${validatePathParam('certificateId', request.certificateId)}`
  }, unmarshalCertificate);

  /**
   * Delete an SSL/TLS certificate. Delete an SSL/TLS certificate, specified by
   * its certificate ID. Deleting a certificate is irreversible and cannot be
   * undone.
   *
   * @param request - The request {@link ZonedApiDeleteCertificateRequest}
   */
  deleteCertificate = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/certificates/${validatePathParam('certificateId', request.certificateId)}`
  });
  pageOfListLbTypes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lb-types`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListLbTypesResponse);

  /**
   * List all Load Balancer offer types. List all the different commercial Load
   * Balancer types. The response includes an array of offer types, each with a
   * name, description, and information about its stock availability.
   *
   * @param request - The request {@link ZonedApiListLbTypesRequest}
   * @returns A Promise of ListLbTypesResponse
   */
  listLbTypes = (request = {}) => enrichForPagination('lbTypes', this.pageOfListLbTypes, request);

  /**
   * Create a subscriber. Create a new subscriber, either with an email
   * configuration or a webhook configuration, for a specified Scaleway
   * Project.
   *
   * @param request - The request {@link ZonedApiCreateSubscriberRequest}
   * @returns A Promise of Subscriber
   */
  createSubscriber = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiCreateSubscriberRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/subscribers`
  }, unmarshalSubscriber);

  /**
   * Get a subscriber. Retrieve information about an existing subscriber,
   * specified by its subscriber ID. Its full details, including name and
   * email/webhook configuration, are returned in the response object.
   *
   * @param request - The request {@link ZonedApiGetSubscriberRequest}
   * @returns A Promise of Subscriber
   */
  getSubscriber = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/subscribers/${validatePathParam('subscriberId', request.subscriberId)}`
  }, unmarshalSubscriber);
  pageOfListSubscriber = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/subscribers`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListSubscriberResponse);

  /**
   * List all subscribers. List all subscribers to Load Balancer alerts. By
   * default, returns all subscribers to Load Balancer alerts for the
   * Organization associated with the authentication token used for the
   * request.
   *
   * @param request - The request {@link ZonedApiListSubscriberRequest}
   * @returns A Promise of ListSubscriberResponse
   */
  listSubscriber = (request = {}) => enrichForPagination('subscribers', this.pageOfListSubscriber, request);

  /**
   * Update a subscriber. Update the parameters of a given subscriber (e.g.
   * name, webhook configuration, email configuration), specified by its
   * subscriber ID.
   *
   * @param request - The request {@link ZonedApiUpdateSubscriberRequest}
   * @returns A Promise of Subscriber
   */
  updateSubscriber = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiUpdateSubscriberRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/subscribers/${validatePathParam('subscriberId', request.subscriberId)}`
  }, unmarshalSubscriber);

  /**
   * Delete a subscriber. Delete an existing subscriber, specified by its
   * subscriber ID. Deleting a subscriber is permanent, and cannot be undone.
   *
   * @param request - The request {@link ZonedApiDeleteSubscriberRequest}
   */
  deleteSubscriber = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lb/subscription/${validatePathParam('subscriberId', request.subscriberId)}`
  });

  /**
   * Subscribe a subscriber to alerts for a given Load Balancer. Subscribe an
   * existing subscriber to alerts for a given Load Balancer.
   *
   * @param request - The request {@link ZonedApiSubscribeToLbRequest}
   * @returns A Promise of Lb
   */
  subscribeToLb = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiSubscribeToLbRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lb/${validatePathParam('lbId', request.lbId)}/subscribe`
  }, unmarshalLb);

  /**
   * Unsubscribe a subscriber from alerts for a given Load Balancer. Unsubscribe
   * a subscriber from alerts for a given Load Balancer. The subscriber is not
   * deleted, and can be resubscribed in the future if necessary.
   *
   * @param request - The request {@link ZonedApiUnsubscribeFromLbRequest}
   * @returns A Promise of Lb
   */
  unsubscribeFromLb = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lb/${validatePathParam('lbId', request.lbId)}/unsubscribe`
  }, unmarshalLb);
  pageOfListLbPrivateNetworks = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/private-networks`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListLbPrivateNetworksResponse);

  /**
   * List Private Networks attached to a Load Balancer. List the Private
   * Networks attached to a given Load Balancer, specified by its Load Balancer
   * ID. The response is an array of Private Network objects, giving information
   * including the status, configuration, name and creation date of each Private
   * Network.
   *
   * @param request - The request {@link ZonedApiListLbPrivateNetworksRequest}
   * @returns A Promise of ListLbPrivateNetworksResponse
   */
  listLbPrivateNetworks = request => enrichForPagination('privateNetwork', this.pageOfListLbPrivateNetworks, request);

  /**
   * Attach a Load Balancer to a Private Network. Attach a specified Load
   * Balancer to a specified Private Network, defining a static or DHCP
   * configuration for the Load Balancer on the network.
   *
   * @param request - The request {@link ZonedApiAttachPrivateNetworkRequest}
   * @returns A Promise of PrivateNetwork
   */
  attachPrivateNetwork = request => this.client.fetch({
    body: JSON.stringify(marshalZonedApiAttachPrivateNetworkRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}/attach`
  }, unmarshalPrivateNetwork$3);

  /**
   * Detach Load Balancer from Private Network. Detach a specified Load Balancer
   * from a specified Private Network.
   *
   * @param request - The request {@link ZonedApiDetachPrivateNetworkRequest}
   */
  detachPrivateNetwork = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/lbs/${validatePathParam('lbId', request.lbId)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}/detach`
  });
}

/**
 * Load balancer API.
 *
 * This API allows you to manage your load balancer service.
 */
let API$e = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];
  pageOfListLbs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalListLbsResponse);

  /**
   * List load balancers.
   *
   * @param request - The request {@link ListLbsRequest}
   * @returns A Promise of ListLbsResponse
   */
  listLbs = (request = {}) => enrichForPagination('lbs', this.pageOfListLbs, request);

  /**
   * Create a load balancer.
   *
   * @param request - The request {@link CreateLbRequest}
   * @returns A Promise of Lb
   */
  createLb = request => this.client.fetch({
    body: JSON.stringify(marshalCreateLbRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs`
  }, unmarshalLb);

  /**
   * Get a load balancer.
   *
   * @param request - The request {@link GetLbRequest}
   * @returns A Promise of Lb
   */
  getLb = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}`
  }, unmarshalLb);

  /**
   * Waits for {@link Lb} to be in a final state.
   *
   * @param request - The request {@link GetLbRequest}
   * @param options - The waiting options
   * @returns A Promise of Lb
   */
  waitForLb = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!LB_TRANSIENT_STATUSES.includes(res.status))), this.getLb, request, options);

  /**
   * Update a load balancer.
   *
   * @param request - The request {@link UpdateLbRequest}
   * @returns A Promise of Lb
   */
  updateLb = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateLbRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}`
  }, unmarshalLb);

  /**
   * Delete a load balancer.
   *
   * @param request - The request {@link DeleteLbRequest}
   */
  deleteLb = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}`,
    urlParams: urlParams(['release_ip', request.releaseIp])
  });

  /**
   * Migrate a load balancer.
   *
   * @param request - The request {@link MigrateLbRequest}
   * @returns A Promise of Lb
   */
  migrateLb = request => this.client.fetch({
    body: JSON.stringify(marshalMigrateLbRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/migrate`
  }, unmarshalLb);
  pageOfListIPs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips`,
    urlParams: urlParams(['ip_address', request.ipAddress], ['ip_type', request.ipType], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListIpsResponse);

  /**
   * List IPs.
   *
   * @param request - The request {@link ListIPsRequest}
   * @returns A Promise of ListIpsResponse
   */
  listIPs = (request = {}) => enrichForPagination('ips', this.pageOfListIPs, request);

  /**
   * Create an IP.
   *
   * @param request - The request {@link CreateIpRequest}
   * @returns A Promise of Ip
   */
  createIp = request => this.client.fetch({
    body: JSON.stringify(marshalCreateIpRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips`
  }, unmarshalIp);

  /**
   * Get an IP.
   *
   * @param request - The request {@link GetIpRequest}
   * @returns A Promise of Ip
   */
  getIp = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIp);

  /**
   * Delete an IP.
   *
   * @param request - The request {@link ReleaseIpRequest}
   */
  releaseIp = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips/${validatePathParam('ipId', request.ipId)}`
  });

  /**
   * Update an IP.
   *
   * @param request - The request {@link UpdateIpRequest}
   * @returns A Promise of Ip
   */
  updateIp = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateIpRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PATCH',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIp);
  pageOfListBackends = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/backends`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListBackendsResponse);

  /**
   * List backends in a given load balancer.
   *
   * @param request - The request {@link ListBackendsRequest}
   * @returns A Promise of ListBackendsResponse
   */
  listBackends = request => enrichForPagination('backends', this.pageOfListBackends, request);

  /**
   * Create a backend in a given load balancer.
   *
   * @param request - The request {@link CreateBackendRequest}
   * @returns A Promise of Backend
   */
  createBackend = request => this.client.fetch({
    body: JSON.stringify(marshalCreateBackendRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/backends`
  }, unmarshalBackend);

  /**
   * Get a backend in a given load balancer.
   *
   * @param request - The request {@link GetBackendRequest}
   * @returns A Promise of Backend
   */
  getBackend = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backends/${validatePathParam('backendId', request.backendId)}`
  }, unmarshalBackend);

  /**
   * Update a backend in a given load balancer.
   *
   * @param request - The request {@link UpdateBackendRequest}
   * @returns A Promise of Backend
   */
  updateBackend = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateBackendRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backends/${validatePathParam('backendId', request.backendId)}`
  }, unmarshalBackend);

  /**
   * Delete a backend in a given load balancer.
   *
   * @param request - The request {@link DeleteBackendRequest}
   */
  deleteBackend = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backends/${validatePathParam('backendId', request.backendId)}`
  });

  /**
   * Add a set of servers in a given backend.
   *
   * @param request - The request {@link AddBackendServersRequest}
   * @returns A Promise of Backend
   */
  addBackendServers = request => this.client.fetch({
    body: JSON.stringify(marshalAddBackendServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backends/${validatePathParam('backendId', request.backendId)}/servers`
  }, unmarshalBackend);

  /**
   * Remove a set of servers for a given backend.
   *
   * @param request - The request {@link RemoveBackendServersRequest}
   * @returns A Promise of Backend
   */
  removeBackendServers = request => this.client.fetch({
    body: JSON.stringify(marshalRemoveBackendServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backends/${validatePathParam('backendId', request.backendId)}/servers`
  }, unmarshalBackend);

  /**
   * Define all servers in a given backend.
   *
   * @param request - The request {@link SetBackendServersRequest}
   * @returns A Promise of Backend
   */
  setBackendServers = request => this.client.fetch({
    body: JSON.stringify(marshalSetBackendServersRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backends/${validatePathParam('backendId', request.backendId)}/servers`
  }, unmarshalBackend);

  /**
   * Update an health check for a given backend.
   *
   * @param request - The request {@link UpdateHealthCheckRequest}
   * @returns A Promise of HealthCheck
   */
  updateHealthCheck = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateHealthCheckRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backends/${validatePathParam('backendId', request.backendId)}/healthcheck`
  }, unmarshalHealthCheck);
  pageOfListFrontends = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/frontends`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListFrontendsResponse);

  /**
   * List frontends in a given load balancer.
   *
   * @param request - The request {@link ListFrontendsRequest}
   * @returns A Promise of ListFrontendsResponse
   */
  listFrontends = request => enrichForPagination('frontends', this.pageOfListFrontends, request);

  /**
   * Create a frontend in a given load balancer.
   *
   * @param request - The request {@link CreateFrontendRequest}
   * @returns A Promise of Frontend
   */
  createFrontend = request => this.client.fetch({
    body: JSON.stringify(marshalCreateFrontendRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/frontends`
  }, unmarshalFrontend);

  /**
   * Get a frontend.
   *
   * @param request - The request {@link GetFrontendRequest}
   * @returns A Promise of Frontend
   */
  getFrontend = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/frontends/${validatePathParam('frontendId', request.frontendId)}`
  }, unmarshalFrontend);

  /**
   * Update a frontend.
   *
   * @param request - The request {@link UpdateFrontendRequest}
   * @returns A Promise of Frontend
   */
  updateFrontend = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateFrontendRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/frontends/${validatePathParam('frontendId', request.frontendId)}`
  }, unmarshalFrontend);

  /**
   * Delete a frontend.
   *
   * @param request - The request {@link DeleteFrontendRequest}
   */
  deleteFrontend = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/frontends/${validatePathParam('frontendId', request.frontendId)}`
  });
  pageOfListRoutes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes`,
    urlParams: urlParams(['frontend_id', request.frontendId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListRoutesResponse);

  /**
   * List all backend redirections.
   *
   * @param request - The request {@link ListRoutesRequest}
   * @returns A Promise of ListRoutesResponse
   */
  listRoutes = (request = {}) => enrichForPagination('routes', this.pageOfListRoutes, request);

  /**
   * Create a backend redirection.
   *
   * @param request - The request {@link CreateRouteRequest}
   * @returns A Promise of Route
   */
  createRoute = request => this.client.fetch({
    body: JSON.stringify(marshalCreateRouteRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes`
  }, unmarshalRoute);

  /**
   * Get single backend redirection.
   *
   * @param request - The request {@link GetRouteRequest}
   * @returns A Promise of Route
   */
  getRoute = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes/${validatePathParam('routeId', request.routeId)}`
  }, unmarshalRoute);

  /**
   * Edit a backend redirection.
   *
   * @param request - The request {@link UpdateRouteRequest}
   * @returns A Promise of Route
   */
  updateRoute = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateRouteRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes/${validatePathParam('routeId', request.routeId)}`
  }, unmarshalRoute);

  /**
   * Delete a backend redirection.
   *
   * @param request - The request {@link DeleteRouteRequest}
   */
  deleteRoute = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/routes/${validatePathParam('routeId', request.routeId)}`
  });

  /**
   * Get usage statistics of a given load balancer.
   *
   * @deprecated
   * @param request - The request {@link GetLbStatsRequest}
   * @returns A Promise of LbStats
   */
  getLbStats = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/stats`,
    urlParams: urlParams(['backend_id', request.backendId])
  }, unmarshalLbStats);
  pageOfListBackendStats = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/backend-stats`,
    urlParams: urlParams(['backend_id', request.backendId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListBackendStatsResponse);

  /**
   * List backend server statistics.
   *
   * @param request - The request {@link ListBackendStatsRequest}
   * @returns A Promise of ListBackendStatsResponse
   */
  listBackendStats = request => enrichForPagination('backendServersStats', this.pageOfListBackendStats, request);
  pageOfListAcls = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/frontends/${validatePathParam('frontendId', request.frontendId)}/acls`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListAclResponse);

  /**
   * List ACL for a given frontend.
   *
   * @param request - The request {@link ListAclsRequest}
   * @returns A Promise of ListAclResponse
   */
  listAcls = request => enrichForPagination('acls', this.pageOfListAcls, request);

  /**
   * Create an ACL for a given frontend.
   *
   * @param request - The request {@link CreateAclRequest}
   * @returns A Promise of Acl
   */
  createAcl = request => this.client.fetch({
    body: JSON.stringify(marshalCreateAclRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/frontends/${validatePathParam('frontendId', request.frontendId)}/acls`
  }, unmarshalAcl);

  /**
   * Get an ACL.
   *
   * @param request - The request {@link GetAclRequest}
   * @returns A Promise of Acl
   */
  getAcl = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/acls/${validatePathParam('aclId', request.aclId)}`
  }, unmarshalAcl);

  /**
   * Update an ACL.
   *
   * @param request - The request {@link UpdateAclRequest}
   * @returns A Promise of Acl
   */
  updateAcl = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateAclRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/acls/${validatePathParam('aclId', request.aclId)}`
  }, unmarshalAcl);

  /**
   * Delete an ACL.
   *
   * @param request - The request {@link DeleteAclRequest}
   */
  deleteAcl = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/acls/${validatePathParam('aclId', request.aclId)}`
  });

  /**
   * Create a TLS certificate. Generate a new TLS certificate using Let's
   * Encrypt or import your certificate.
   *
   * @param request - The request {@link CreateCertificateRequest}
   * @returns A Promise of Certificate
   */
  createCertificate = request => this.client.fetch({
    body: JSON.stringify(marshalCreateCertificateRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/certificates`
  }, unmarshalCertificate);
  pageOfListCertificates = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/certificates`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListCertificatesResponse);

  /**
   * List all TLS certificates on a given load balancer.
   *
   * @param request - The request {@link ListCertificatesRequest}
   * @returns A Promise of ListCertificatesResponse
   */
  listCertificates = request => enrichForPagination('certificates', this.pageOfListCertificates, request);

  /**
   * Get a TLS certificate.
   *
   * @param request - The request {@link GetCertificateRequest}
   * @returns A Promise of Certificate
   */
  getCertificate = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/certificates/${validatePathParam('certificateId', request.certificateId)}`
  }, unmarshalCertificate);

  /**
   * Waits for {@link Certificate} to be in a final state.
   *
   * @param request - The request {@link GetCertificateRequest}
   * @param options - The waiting options
   * @returns A Promise of Certificate
   */
  waitForCertificate = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!CERTIFICATE_TRANSIENT_STATUSES.includes(res.status))), this.getCertificate, request, options);

  /**
   * Update a TLS certificate.
   *
   * @param request - The request {@link UpdateCertificateRequest}
   * @returns A Promise of Certificate
   */
  updateCertificate = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateCertificateRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/certificates/${validatePathParam('certificateId', request.certificateId)}`
  }, unmarshalCertificate);

  /**
   * Delete a TLS certificate.
   *
   * @param request - The request {@link DeleteCertificateRequest}
   */
  deleteCertificate = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/certificates/${validatePathParam('certificateId', request.certificateId)}`
  });
  pageOfListLbTypes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lb-types`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListLbTypesResponse);

  /**
   * List all load balancer offer type.
   *
   * @param request - The request {@link ListLbTypesRequest}
   * @returns A Promise of ListLbTypesResponse
   */
  listLbTypes = (request = {}) => enrichForPagination('lbTypes', this.pageOfListLbTypes, request);

  /**
   * Create a subscriber, webhook or email.
   *
   * @param request - The request {@link CreateSubscriberRequest}
   * @returns A Promise of Subscriber
   */
  createSubscriber = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSubscriberRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/subscribers`
  }, unmarshalSubscriber);

  /**
   * Get a subscriber.
   *
   * @param request - The request {@link GetSubscriberRequest}
   * @returns A Promise of Subscriber
   */
  getSubscriber = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/subscribers/${validatePathParam('subscriberId', request.subscriberId)}`
  }, unmarshalSubscriber);
  pageOfListSubscriber = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/subscribers`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListSubscriberResponse);

  /**
   * List all subscriber.
   *
   * @param request - The request {@link ListSubscriberRequest}
   * @returns A Promise of ListSubscriberResponse
   */
  listSubscriber = (request = {}) => enrichForPagination('subscribers', this.pageOfListSubscriber, request);

  /**
   * Update a subscriber.
   *
   * @param request - The request {@link UpdateSubscriberRequest}
   * @returns A Promise of Subscriber
   */
  updateSubscriber = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSubscriberRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'PUT',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/subscribers/${validatePathParam('subscriberId', request.subscriberId)}`
  }, unmarshalSubscriber);

  /**
   * Delete a subscriber.
   *
   * @param request - The request {@link DeleteSubscriberRequest}
   */
  deleteSubscriber = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lb/subscriber/${validatePathParam('subscriberId', request.subscriberId)}`
  });

  /**
   * Subscribe a subscriber to a given load balancer.
   *
   * @param request - The request {@link SubscribeToLbRequest}
   * @returns A Promise of Lb
   */
  subscribeToLb = request => this.client.fetch({
    body: JSON.stringify(marshalSubscribeToLbRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lb/${validatePathParam('lbId', request.lbId)}/subscribe`
  }, unmarshalLb);

  /**
   * Unsubscribe a subscriber from a given load balancer.
   *
   * @param request - The request {@link UnsubscribeFromLbRequest}
   * @returns A Promise of Lb
   */
  unsubscribeFromLb = request => this.client.fetch({
    method: 'DELETE',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lb/${validatePathParam('lbId', request.lbId)}/unsubscribe`
  }, unmarshalLb);
  pageOfListLbPrivateNetworks = request => this.client.fetch({
    method: 'GET',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/private-networks`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListLbPrivateNetworksResponse);

  /**
   * List attached private network of load balancer.
   *
   * @param request - The request {@link ListLbPrivateNetworksRequest}
   * @returns A Promise of ListLbPrivateNetworksResponse
   */
  listLbPrivateNetworks = request => enrichForPagination('privateNetwork', this.pageOfListLbPrivateNetworks, request);

  /**
   * Add load balancer on instance private network.
   *
   * @param request - The request {@link AttachPrivateNetworkRequest}
   * @returns A Promise of PrivateNetwork
   */
  attachPrivateNetwork = request => this.client.fetch({
    body: JSON.stringify(marshalAttachPrivateNetworkRequest(request, this.client.settings)),
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}/attach`
  }, unmarshalPrivateNetwork$3);

  /**
   * Remove load balancer of private network.
   *
   * @param request - The request {@link DetachPrivateNetworkRequest}
   */
  detachPrivateNetwork = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$e,
    method: 'POST',
    path: `/lb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/lbs/${validatePathParam('lbId', request.lbId)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}/detach`
  });
};

class LbV1UtilsAPI extends API$e {
  /**
   * Waits for all private networks of a load balancer to be in a final state.
   *
   * @param request - The request {@link WaitForLbPrivateNetworksRequest}
   * @param options - The waiting options
   * @returns A Promise of ListLbPrivateNetworksResponse
   */
  waitForLbPrivateNetworks = (request, options) => tryAtIntervals(async () => {
    const value = await this.listLbPrivateNetworks({
      lbId: request.lbId,
      region: request.region
    }).all().then(list => ({
      privateNetwork: list,
      totalCount: list.length
    }));
    return {
      done: value.privateNetwork.find(elt => PRIVATE_NETWORK_TRANSIENT_STATUSES.includes(elt.status)) === undefined,
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);

  /**
   * Waits for all instances of a load balancer to be in a final state.
   *
   * @param request - The request {@link GetLbRequest}
   * @param options - The waiting options
   * @returns A Promise of Lb
   */
  waitForLbInstances = (request, options) => tryAtIntervals(async () => {
    const value = await this.getLb(request);
    return {
      done: !LB_TRANSIENT_STATUSES.includes(value.status) && value.instances.find(elt => INSTANCE_TRANSIENT_STATUSES$1.includes(elt.status)) === undefined,
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);
}
class LbZonedV1UtilsAPI extends ZonedAPI {
  /**
   * Waits for all private networks of a (zoned) load balancer to be in a final
   * state.
   *
   * @param request - The request {@link ZonedWaitForLbPrivateNetworksRequest}
   * @param options - The waiting options
   * @returns A Promise of ListLbPrivateNetworksResponse
   */
  waitForLbPrivateNetworks = (request, options) => tryAtIntervals(async () => {
    const value = await this.listLbPrivateNetworks({
      lbId: request.lbId,
      zone: request.zone
    }).all().then(list => ({
      privateNetwork: list,
      totalCount: list.length
    }));
    return {
      done: value.privateNetwork.find(elt => PRIVATE_NETWORK_TRANSIENT_STATUSES.includes(elt.status)) === undefined,
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);

  /**
   * Waits for all instances of a (zoned) load balancer to be in a final state.
   *
   * @param request - The request {@link GetLbRequest}
   * @param options - The waiting options
   * @returns A Promise of Lb
   */
  waitForLbInstances = (request, options) => tryAtIntervals(async () => {
    const value = await this.getLb(request);
    return {
      done: !LB_TRANSIENT_STATUSES.includes(value.status) && value.instances.find(elt => INSTANCE_TRANSIENT_STATUSES$1.includes(elt.status)) === undefined,
      value
    };
  }, createExponentialBackoffStrategy(options?.minDelay ?? 1, options?.maxDelay ?? 30), options?.timeout);
}

var index$e = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: LbV1UtilsAPI,
  CERTIFICATE_TRANSIENT_STATUSES: CERTIFICATE_TRANSIENT_STATUSES,
  INSTANCE_TRANSIENT_STATUSES: INSTANCE_TRANSIENT_STATUSES$1,
  LB_TRANSIENT_STATUSES: LB_TRANSIENT_STATUSES,
  PRIVATE_NETWORK_TRANSIENT_STATUSES: PRIVATE_NETWORK_TRANSIENT_STATUSES,
  ZonedAPI: LbZonedV1UtilsAPI
});

var index$d = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index$e
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link DeploymentStatus}. */
const DEPLOYMENT_TRANSIENT_STATUSES = ['creating', 'deploying', 'deleting'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalEndpointPrivateNetworkDetails$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EndpointPrivateNetworkDetails' failed as data isn't a dictionary.`);
  }
  return {
    privateNetworkId: data.private_network_id
  };
};
const unmarshalEndpointPublicAccessDetails = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EndpointPublicAccessDetails' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalEndpoint$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Endpoint' failed as data isn't a dictionary.`);
  }
  return {
    disableAuth: data.disable_auth,
    id: data.id,
    privateNetwork: data.private_network ? unmarshalEndpointPrivateNetworkDetails$1(data.private_network) : undefined,
    publicAccess: data.public_access ? unmarshalEndpointPublicAccessDetails(data.public_access) : undefined,
    url: data.url
  };
};
const unmarshalDeployment = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Deployment' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    endpoints: unmarshalArrayOfObject(data.endpoints, unmarshalEndpoint$2),
    errorMessage: data.error_message,
    id: data.id,
    maxSize: data.max_size,
    minSize: data.min_size,
    modelName: data.model_name,
    name: data.name,
    nodeType: data.node_type,
    projectId: data.project_id,
    region: data.region,
    size: data.size,
    status: data.status,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalModelS3Model = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ModelS3Model' failed as data isn't a dictionary.`);
  }
  return {
    nodeType: data.node_type,
    pythonDependencies: data.python_dependencies,
    s3Url: data.s3_url,
    tritonServerVersion: data.triton_server_version
  };
};
const unmarshalModel = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Model' failed as data isn't a dictionary.`);
  }
  return {
    compatibleNodeTypes: data.compatible_node_types,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    hasEula: data.has_eula,
    id: data.id,
    isPublic: data.is_public,
    name: data.name,
    projectId: data.project_id,
    provider: data.provider,
    quantizationLevel: data.quantization_level,
    region: data.region,
    s3Model: data.s3_model ? unmarshalModelS3Model(data.s3_model) : undefined,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalACLRule$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ACLRule' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    id: data.id,
    ip: data.ip
  };
};
const unmarshalAddDeploymentACLRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AddDeploymentACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$2)
  };
};
const unmarshalEula = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Eula' failed as data isn't a dictionary.`);
  }
  return {
    content: data.content
  };
};
const unmarshalListDeploymentACLRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDeploymentACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$2),
    totalCount: data.total_count
  };
};
const unmarshalListDeploymentsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDeploymentsResponse' failed as data isn't a dictionary.`);
  }
  return {
    deployments: unmarshalArrayOfObject(data.deployments, unmarshalDeployment),
    totalCount: data.total_count
  };
};
const unmarshalListModelsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListModelsResponse' failed as data isn't a dictionary.`);
  }
  return {
    models: unmarshalArrayOfObject(data.models, unmarshalModel),
    totalCount: data.total_count
  };
};
const unmarshalNodeType$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NodeType' failed as data isn't a dictionary.`);
  }
  return {
    beta: data.beta,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    disabled: data.disabled,
    gpus: data.gpus,
    memory: data.memory,
    name: data.name,
    region: data.region,
    stockStatus: data.stock_status,
    updatedAt: unmarshalDate(data.updated_at),
    vcpus: data.vcpus,
    vram: data.vram
  };
};
const unmarshalListNodeTypesResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNodeTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    nodeTypes: unmarshalArrayOfObject(data.node_types, unmarshalNodeType$2),
    totalCount: data.total_count
  };
};
const unmarshalSetDeploymentACLRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetDeploymentACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$2)
  };
};
const marshalACLRuleRequest$1 = (request, defaults) => ({
  description: request.description,
  ip: request.ip
});
const marshalAddDeploymentACLRulesRequest = (request, defaults) => ({
  acls: request.acls !== undefined ? request.acls.map(elt => marshalACLRuleRequest$1(elt)) : undefined
});
const marshalEndpointSpecPrivateNetwork$1 = (request, defaults) => ({
  private_network_id: request.privateNetworkId
});
const marshalEndpointSpecPublic = (request, defaults) => ({});
const marshalEndpointSpec$2 = (request, defaults) => ({
  disable_auth: request.disableAuth,
  ...resolveOneOf([{
    param: 'public',
    value: request.public !== undefined ? marshalEndpointSpecPublic(request.public) : undefined
  }, {
    param: 'private_network',
    value: request.privateNetwork !== undefined ? marshalEndpointSpecPrivateNetwork$1(request.privateNetwork) : undefined
  }])
});
const marshalCreateDeploymentRequest = (request, defaults) => ({
  accept_eula: request.acceptEula,
  endpoints: request.endpoints.map(elt => marshalEndpointSpec$2(elt)),
  max_size: request.maxSize,
  min_size: request.minSize,
  model_name: request.modelName,
  name: request.name || randomName('llm'),
  node_type: request.nodeType,
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags
});
const marshalCreateEndpointRequest$1 = (request, defaults) => ({
  deployment_id: request.deploymentId,
  endpoint: marshalEndpointSpec$2(request.endpoint)
});
const marshalSetDeploymentACLRulesRequest = (request, defaults) => ({
  acls: request.acls !== undefined ? request.acls.map(elt => marshalACLRuleRequest$1(elt)) : undefined
});
const marshalUpdateDeploymentRequest = (request, defaults) => ({
  max_size: request.maxSize,
  min_size: request.minSize,
  name: request.name,
  tags: request.tags
});
const marshalUpdateEndpointRequest$1 = (request, defaults) => ({
  disable_auth: request.disableAuth
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$d = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** LLM Inference API. */
let API$d = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par'];
  pageOfListDeployments = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalListDeploymentsResponse);

  /**
   * List inference deployments. List all your inference deployments.
   *
   * @param request - The request {@link ListDeploymentsRequest}
   * @returns A Promise of ListDeploymentsResponse
   */
  listDeployments = (request = {}) => enrichForPagination('deployments', this.pageOfListDeployments, request);

  /**
   * Get a deployment. Get the deployment for the given ID.
   *
   * @param request - The request {@link GetDeploymentRequest}
   * @returns A Promise of Deployment
   */
  getDeployment = request => this.client.fetch({
    method: 'GET',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments/${validatePathParam('deploymentId', request.deploymentId)}`
  }, unmarshalDeployment);

  /**
   * Waits for {@link Deployment} to be in a final state.
   *
   * @param request - The request {@link GetDeploymentRequest}
   * @param options - The waiting options
   * @returns A Promise of Deployment
   */
  waitForDeployment = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!DEPLOYMENT_TRANSIENT_STATUSES.includes(res.status))), this.getDeployment, request, options);

  /**
   * Create a deployment. Create a new inference deployment related to a
   * specific model.
   *
   * @param request - The request {@link CreateDeploymentRequest}
   * @returns A Promise of Deployment
   */
  createDeployment = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDeploymentRequest(request, this.client.settings)),
    headers: jsonContentHeaders$d,
    method: 'POST',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments`
  }, unmarshalDeployment);

  /**
   * Update a deployment. Update an existing inference deployment.
   *
   * @param request - The request {@link UpdateDeploymentRequest}
   * @returns A Promise of Deployment
   */
  updateDeployment = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDeploymentRequest(request, this.client.settings)),
    headers: jsonContentHeaders$d,
    method: 'PATCH',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments/${validatePathParam('deploymentId', request.deploymentId)}`
  }, unmarshalDeployment);

  /**
   * Delete a deployment. Delete an existing inference deployment.
   *
   * @param request - The request {@link DeleteDeploymentRequest}
   * @returns A Promise of Deployment
   */
  deleteDeployment = request => this.client.fetch({
    method: 'DELETE',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments/${validatePathParam('deploymentId', request.deploymentId)}`
  }, unmarshalDeployment);

  /**
   * Get the CA certificate. Get the CA certificate used for the deployment of
   * private endpoints. The CA certificate will be returned as a PEM file.
   *
   * @param request - The request {@link GetDeploymentCertificateRequest}
   * @returns A Promise of Blob
   */
  getDeploymentCertificate = request => this.client.fetch({
    method: 'GET',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments/${validatePathParam('deploymentId', request.deploymentId)}/certificate`,
    urlParams: urlParams(['dl', 1]),
    responseType: 'blob'
  });

  /**
   * Create an endpoint. Create a new Endpoint related to a specific deployment.
   *
   * @param request - The request {@link CreateEndpointRequest}
   * @returns A Promise of Endpoint
   */
  createEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalCreateEndpointRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$d,
    method: 'POST',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints`
  }, unmarshalEndpoint$2);

  /**
   * Update an endpoint. Update an existing Endpoint.
   *
   * @param request - The request {@link UpdateEndpointRequest}
   * @returns A Promise of Endpoint
   */
  updateEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateEndpointRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$d,
    method: 'PATCH',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  }, unmarshalEndpoint$2);

  /**
   * Delete an endpoint. Delete an existing Endpoint.
   *
   * @param request - The request {@link DeleteEndpointRequest}
   */
  deleteEndpoint = request => this.client.fetch({
    method: 'DELETE',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  });
  pageOfListDeploymentACLRules = request => this.client.fetch({
    method: 'GET',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments/${validatePathParam('deploymentId', request.deploymentId)}/acls`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDeploymentACLRulesResponse);

  /**
   * List your ACLs. List ACLs for a specific deployment.
   *
   * @param request - The request {@link ListDeploymentACLRulesRequest}
   * @returns A Promise of ListDeploymentACLRulesResponse
   */
  listDeploymentACLRules = request => enrichForPagination('rules', this.pageOfListDeploymentACLRules, request);

  /**
   * Add new ACLs. Add new ACL rules for a specific deployment.
   *
   * @param request - The request {@link AddDeploymentACLRulesRequest}
   * @returns A Promise of AddDeploymentACLRulesResponse
   */
  addDeploymentACLRules = request => this.client.fetch({
    body: JSON.stringify(marshalAddDeploymentACLRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$d,
    method: 'POST',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments/${validatePathParam('deploymentId', request.deploymentId)}/acls`
  }, unmarshalAddDeploymentACLRulesResponse);

  /**
   * Set new ACL. Set new ACL rules for a specific deployment.
   *
   * @param request - The request {@link SetDeploymentACLRulesRequest}
   * @returns A Promise of SetDeploymentACLRulesResponse
   */
  setDeploymentACLRules = request => this.client.fetch({
    body: JSON.stringify(marshalSetDeploymentACLRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$d,
    method: 'PUT',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deployments/${validatePathParam('deploymentId', request.deploymentId)}/acls`
  }, unmarshalSetDeploymentACLRulesResponse);

  /**
   * Delete an exising ACL.
   *
   * @param request - The request {@link DeleteDeploymentACLRuleRequest}
   */
  deleteDeploymentACLRule = request => this.client.fetch({
    method: 'DELETE',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/acls/${validatePathParam('aclId', request.aclId)}`
  });
  pageOfListModels = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/models`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalListModelsResponse);

  /**
   * List models. List all available LLM models.
   *
   * @param request - The request {@link ListModelsRequest}
   * @returns A Promise of ListModelsResponse
   */
  listModels = (request = {}) => enrichForPagination('models', this.pageOfListModels, request);

  /**
   * Get a model. Get the model for the given ID.
   *
   * @param request - The request {@link GetModelRequest}
   * @returns A Promise of Model
   */
  getModel = request => this.client.fetch({
    method: 'GET',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/models/${validatePathParam('modelId', request.modelId)}`
  }, unmarshalModel);
  getModelEula = request => this.client.fetch({
    method: 'GET',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/models/${validatePathParam('modelId', request.modelId)}/eula`
  }, unmarshalEula);
  pageOfListNodeTypes = request => this.client.fetch({
    method: 'GET',
    path: `/llm-inference/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/node-types`,
    urlParams: urlParams(['include_disabled_types', request.includeDisabledTypes], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListNodeTypesResponse$2);

  /**
   * List available node types. List all available node types. By default, the
   * node types returned in the list are ordered by creation date in ascending
   * order, though this can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListNodeTypesRequest}
   * @returns A Promise of ListNodeTypesResponse
   */
  listNodeTypes = request => enrichForPagination('nodeTypes', this.pageOfListNodeTypes, request);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateDeploymentRequest = {
  maxSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 50
  },
  minSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 50
  },
  modelName: {
    maxLength: 255,
    minLength: 0,
    pattern: /^[\w-_]+\/[\w-_.]+(:[\w]+)?$/
  },
  name: {
    maxLength: 255,
    minLength: 1,
    pattern: /^[A-Za-z0-9-_]+$/
  },
  nodeType: {
    maxLength: 64,
    minLength: 1
  }
};
const ListDeploymentsRequest = {
  name: {
    maxLength: 255,
    minLength: 1
  }
};
const ListModelsRequest = {
  name: {
    maxLength: 255,
    minLength: 1
  }
};
const UpdateDeploymentRequest = {
  maxSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 50
  },
  minSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 50
  },
  name: {
    maxLength: 255,
    minLength: 1
  }
};

var validationRules_gen$5 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateDeploymentRequest: CreateDeploymentRequest,
  ListDeploymentsRequest: ListDeploymentsRequest,
  ListModelsRequest: ListModelsRequest,
  UpdateDeploymentRequest: UpdateDeploymentRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$f = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$d,
  DEPLOYMENT_TRANSIENT_STATUSES: DEPLOYMENT_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen$5
});

var index$c = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1beta1: index_gen$f
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalCategory = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Category' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    id: data.id,
    name: data.name
  };
};
const unmarshalImage$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Image' failed as data isn't a dictionary.`);
  }
  return {
    categories: data.categories,
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    id: data.id,
    label: data.label,
    logo: data.logo,
    name: data.name,
    updatedAt: unmarshalDate(data.updated_at),
    validUntil: unmarshalDate(data.valid_until)
  };
};
const unmarshalLocalImage = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'LocalImage' failed as data isn't a dictionary.`);
  }
  return {
    arch: data.arch,
    compatibleCommercialTypes: data.compatible_commercial_types,
    id: data.id,
    label: data.label,
    type: data.type,
    zone: data.zone
  };
};
const unmarshalVersion = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Version' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    publishedAt: unmarshalDate(data.published_at),
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListCategoriesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListCategoriesResponse' failed as data isn't a dictionary.`);
  }
  return {
    categories: unmarshalArrayOfObject(data.categories, unmarshalCategory),
    totalCount: data.total_count
  };
};
const unmarshalListImagesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListImagesResponse' failed as data isn't a dictionary.`);
  }
  return {
    images: unmarshalArrayOfObject(data.images, unmarshalImage$1),
    totalCount: data.total_count
  };
};
const unmarshalListLocalImagesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListLocalImagesResponse' failed as data isn't a dictionary.`);
  }
  return {
    localImages: unmarshalArrayOfObject(data.local_images, unmarshalLocalImage),
    totalCount: data.total_count
  };
};
const unmarshalListVersionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListVersionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    versions: unmarshalArrayOfObject(data.versions, unmarshalVersion)
  };
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
/** Marketplace API. */
let API$c = class API extends API$z {
  pageOfListImages = request => this.client.fetch({
    method: 'GET',
    path: `/marketplace/v2/images`,
    urlParams: urlParams(['arch', request.arch], ['category', request.category], ['include_eol', request.includeEol], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListImagesResponse$1);

  /**
   * List marketplace images. List all available images on the marketplace,
   * their UUID, CPU architecture and description.
   *
   * @param request - The request {@link ListImagesRequest}
   * @returns A Promise of ListImagesResponse
   */
  listImages = request => enrichForPagination('images', this.pageOfListImages, request);

  /**
   * Get a specific marketplace image. Get detailed information about a
   * marketplace image, specified by its `image_id` (UUID format).
   *
   * @param request - The request {@link GetImageRequest}
   * @returns A Promise of Image
   */
  getImage = request => this.client.fetch({
    method: 'GET',
    path: `/marketplace/v2/images/${validatePathParam('imageId', request.imageId)}`
  }, unmarshalImage$1);
  pageOfListVersions = request => this.client.fetch({
    method: 'GET',
    path: `/marketplace/v2/versions`,
    urlParams: urlParams(['image_id', request.imageId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListVersionsResponse);

  /**
   * List versions of an Image. Get a list of all available version of an image,
   * specified by its `image_id` (UUID format).
   *
   * @param request - The request {@link ListVersionsRequest}
   * @returns A Promise of ListVersionsResponse
   */
  listVersions = request => enrichForPagination('versions', this.pageOfListVersions, request);

  /**
   * Get a specific image version. Get information such as the name, creation
   * date, last update and published date for an image version specified by its
   * `version_id` (UUID format).
   *
   * @param request - The request {@link GetVersionRequest}
   * @returns A Promise of Version
   */
  getVersion = request => this.client.fetch({
    method: 'GET',
    path: `/marketplace/v2/versions/${validatePathParam('versionId', request.versionId)}`
  }, unmarshalVersion);
  pageOfListLocalImages = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/marketplace/v2/local-images`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['type', request.type], ['zone', request.zone], ...Object.entries(resolveOneOf([{
      param: 'image_id',
      value: request.imageId
    }, {
      param: 'version_id',
      value: request.versionId
    }, {
      param: 'image_label',
      value: request.imageLabel
    }])))
  }, unmarshalListLocalImagesResponse);

  /**
   * List local images from a specific image or version. List information about
   * local images in a specific Availability Zone, specified by its `image_id`
   * (UUID format), `version_id` (UUID format) or `image_label`. Only one of
   * these three parameters may be set.
   *
   * @param request - The request {@link ListLocalImagesRequest}
   * @returns A Promise of ListLocalImagesResponse
   */
  listLocalImages = (request = {}) => enrichForPagination('localImages', this.pageOfListLocalImages, request);

  /**
   * Get a specific local image by ID. Get detailed information about a local
   * image, including compatible commercial types, supported architecture,
   * labels and the Availability Zone of the image, specified by its
   * `local_image_id` (UUID format).
   *
   * @param request - The request {@link GetLocalImageRequest}
   * @returns A Promise of LocalImage
   */
  getLocalImage = request => this.client.fetch({
    method: 'GET',
    path: `/marketplace/v2/local-images/${validatePathParam('localImageId', request.localImageId)}`
  }, unmarshalLocalImage);
  pageOfListCategories = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/marketplace/v2/categories`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListCategoriesResponse);

  /**
   * List existing image categories. Get a list of all existing categories. The
   * output can be paginated.
   *
   * @param request - The request {@link ListCategoriesRequest}
   * @returns A Promise of ListCategoriesResponse
   */
  listCategories = (request = {}) => enrichForPagination('categories', this.pageOfListCategories, request);

  /**
   * Get a specific category. Get information about a specific category of the
   * marketplace catalog, specified by its `category_id` (UUID format).
   *
   * @param request - The request {@link GetCategoryRequest}
   * @returns A Promise of Category
   */
  getCategory = request => this.client.fetch({
    method: 'GET',
    path: `/marketplace/v2/categories/${validatePathParam('categoryId', request.categoryId)}`
  }, unmarshalCategory);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$e = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$c
});

var index$b = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v2: index_gen$e
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalNatsAccount = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NatsAccount' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    endpoint: data.endpoint,
    id: data.id,
    name: data.name,
    projectId: data.project_id,
    region: data.region,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalFile = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'File' failed as data isn't a dictionary.`);
  }
  return {
    content: data.content,
    name: data.name
  };
};
const unmarshalNatsCredentials = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NatsCredentials' failed as data isn't a dictionary.`);
  }
  return {
    checksum: data.checksum,
    createdAt: unmarshalDate(data.created_at),
    credentials: data.credentials ? unmarshalFile(data.credentials) : undefined,
    id: data.id,
    name: data.name,
    natsAccountId: data.nats_account_id,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalSnsPermissions = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SnsPermissions' failed as data isn't a dictionary.`);
  }
  return {
    canManage: data.can_manage,
    canPublish: data.can_publish,
    canReceive: data.can_receive
  };
};
const unmarshalSnsCredentials = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SnsCredentials' failed as data isn't a dictionary.`);
  }
  return {
    accessKey: data.access_key,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    permissions: data.permissions ? unmarshalSnsPermissions(data.permissions) : undefined,
    projectId: data.project_id,
    region: data.region,
    secretChecksum: data.secret_checksum,
    secretKey: data.secret_key,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalSqsPermissions = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SqsPermissions' failed as data isn't a dictionary.`);
  }
  return {
    canManage: data.can_manage,
    canPublish: data.can_publish,
    canReceive: data.can_receive
  };
};
const unmarshalSqsCredentials = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SqsCredentials' failed as data isn't a dictionary.`);
  }
  return {
    accessKey: data.access_key,
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    permissions: data.permissions ? unmarshalSqsPermissions(data.permissions) : undefined,
    projectId: data.project_id,
    region: data.region,
    secretChecksum: data.secret_checksum,
    secretKey: data.secret_key,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListNatsAccountsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNatsAccountsResponse' failed as data isn't a dictionary.`);
  }
  return {
    natsAccounts: unmarshalArrayOfObject(data.nats_accounts, unmarshalNatsAccount),
    totalCount: data.total_count
  };
};
const unmarshalListNatsCredentialsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNatsCredentialsResponse' failed as data isn't a dictionary.`);
  }
  return {
    natsCredentials: unmarshalArrayOfObject(data.nats_credentials, unmarshalNatsCredentials),
    totalCount: data.total_count
  };
};
const unmarshalListSnsCredentialsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSnsCredentialsResponse' failed as data isn't a dictionary.`);
  }
  return {
    snsCredentials: unmarshalArrayOfObject(data.sns_credentials, unmarshalSnsCredentials),
    totalCount: data.total_count
  };
};
const unmarshalListSqsCredentialsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSqsCredentialsResponse' failed as data isn't a dictionary.`);
  }
  return {
    sqsCredentials: unmarshalArrayOfObject(data.sqs_credentials, unmarshalSqsCredentials),
    totalCount: data.total_count
  };
};
const unmarshalSnsInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SnsInfo' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    projectId: data.project_id,
    region: data.region,
    snsEndpointUrl: data.sns_endpoint_url,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalSqsInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SqsInfo' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    projectId: data.project_id,
    region: data.region,
    sqsEndpointUrl: data.sqs_endpoint_url,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const marshalNatsApiCreateNatsAccountRequest = (request, defaults) => ({
  name: request.name || randomName('mnq'),
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalNatsApiCreateNatsCredentialsRequest = (request, defaults) => ({
  name: request.name || randomName('mnq'),
  nats_account_id: request.natsAccountId
});
const marshalNatsApiUpdateNatsAccountRequest = (request, defaults) => ({
  name: request.name
});
const marshalSnsApiActivateSnsRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalSnsPermissions = (request, defaults) => ({
  can_manage: request.canManage,
  can_publish: request.canPublish,
  can_receive: request.canReceive
});
const marshalSnsApiCreateSnsCredentialsRequest = (request, defaults) => ({
  name: request.name || randomName('mnq_sns'),
  permissions: request.permissions !== undefined ? marshalSnsPermissions(request.permissions) : undefined,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalSnsApiDeactivateSnsRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalSnsApiUpdateSnsCredentialsRequest = (request, defaults) => ({
  name: request.name,
  permissions: request.permissions !== undefined ? marshalSnsPermissions(request.permissions) : undefined
});
const marshalSqsApiActivateSqsRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalSqsPermissions = (request, defaults) => ({
  can_manage: request.canManage,
  can_publish: request.canPublish,
  can_receive: request.canReceive
});
const marshalSqsApiCreateSqsCredentialsRequest = (request, defaults) => ({
  name: request.name || randomName('mnq_sqs'),
  permissions: request.permissions !== undefined ? marshalSqsPermissions(request.permissions) : undefined,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalSqsApiDeactivateSqsRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalSqsApiUpdateSqsCredentialsRequest = (request, defaults) => ({
  name: request.name,
  permissions: request.permissions !== undefined ? marshalSqsPermissions(request.permissions) : undefined
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$c = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Messaging and Queuing NATS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing NATS accounts.
 */
class NatsAPI extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams'];

  /**
   * Create a NATS account. Create a NATS account associated with a Project.
   *
   * @param request - The request {@link NatsApiCreateNatsAccountRequest}
   * @returns A Promise of NatsAccount
   */
  createNatsAccount = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalNatsApiCreateNatsAccountRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-accounts`
  }, unmarshalNatsAccount);

  /**
   * Delete a NATS account. Delete a NATS account, specified by its NATS account
   * ID. Note that deleting a NATS account is irreversible, and any credentials,
   * streams, consumer and stored messages belonging to this NATS account will
   * also be deleted.
   *
   * @param request - The request {@link NatsApiDeleteNatsAccountRequest}
   */
  deleteNatsAccount = request => this.client.fetch({
    method: 'DELETE',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-accounts/${validatePathParam('natsAccountId', request.natsAccountId)}`
  });

  /**
   * Update the name of a NATS account. Update the name of a NATS account,
   * specified by its NATS account ID.
   *
   * @param request - The request {@link NatsApiUpdateNatsAccountRequest}
   * @returns A Promise of NatsAccount
   */
  updateNatsAccount = request => this.client.fetch({
    body: JSON.stringify(marshalNatsApiUpdateNatsAccountRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'PATCH',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-accounts/${validatePathParam('natsAccountId', request.natsAccountId)}`
  }, unmarshalNatsAccount);

  /**
   * Get a NATS account. Retrieve information about an existing NATS account
   * identified by its NATS account ID. Its full details, including name and
   * endpoint, are returned in the response.
   *
   * @param request - The request {@link NatsApiGetNatsAccountRequest}
   * @returns A Promise of NatsAccount
   */
  getNatsAccount = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-accounts/${validatePathParam('natsAccountId', request.natsAccountId)}`
  }, unmarshalNatsAccount);
  pageOfListNatsAccounts = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-accounts`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListNatsAccountsResponse);

  /**
   * List NATS accounts. List all NATS accounts in the specified region, for a
   * Scaleway Organization or Project. By default, the NATS accounts returned in
   * the list are ordered by creation date in ascending order, though this can
   * be modified via the `order_by` field.
   *
   * @param request - The request {@link NatsApiListNatsAccountsRequest}
   * @returns A Promise of ListNatsAccountsResponse
   */
  listNatsAccounts = (request = {}) => enrichForPagination('natsAccounts', this.pageOfListNatsAccounts, request);

  /**
   * Create NATS credentials. Create a set of credentials for a NATS account,
   * specified by its NATS account ID.
   *
   * @param request - The request {@link NatsApiCreateNatsCredentialsRequest}
   * @returns A Promise of NatsCredentials
   */
  createNatsCredentials = request => this.client.fetch({
    body: JSON.stringify(marshalNatsApiCreateNatsCredentialsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-credentials`
  }, unmarshalNatsCredentials);

  /**
   * Delete NATS credentials. Delete a set of credentials, specified by their
   * credentials ID. Deleting credentials is irreversible and cannot be undone.
   * The credentials can no longer be used to access the NATS account, and
   * active connections using this credentials will be closed.
   *
   * @param request - The request {@link NatsApiDeleteNatsCredentialsRequest}
   */
  deleteNatsCredentials = request => this.client.fetch({
    method: 'DELETE',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-credentials/${validatePathParam('natsCredentialsId', request.natsCredentialsId)}`
  });

  /**
   * Get NATS credentials. Retrieve an existing set of credentials, identified
   * by the `nats_credentials_id`. The credentials themselves are NOT returned,
   * only their metadata (NATS account ID, credentials name, etc), are returned
   * in the response.
   *
   * @param request - The request {@link NatsApiGetNatsCredentialsRequest}
   * @returns A Promise of NatsCredentials
   */
  getNatsCredentials = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-credentials/${validatePathParam('natsCredentialsId', request.natsCredentialsId)}`
  }, unmarshalNatsCredentials);
  pageOfListNatsCredentials = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/nats-credentials`,
    urlParams: urlParams(['nats_account_id', request.natsAccountId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListNatsCredentialsResponse);

  /**
   * List NATS credentials. List existing credentials in the specified NATS
   * account. The response contains only the metadata for the credentials, not
   * the credentials themselves, which are only returned after a **Create
   * Credentials** call.
   *
   * @param request - The request {@link NatsApiListNatsCredentialsRequest}
   * @returns A Promise of ListNatsCredentialsResponse
   */
  listNatsCredentials = (request = {}) => enrichForPagination('natsCredentials', this.pageOfListNatsCredentials, request);
}

/**
 * Messaging and Queuing SNS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing SNS brokers.
 */
class SnsAPI extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams'];

  /**
   * Activate SNS. Activate SNS for the specified Project ID. SNS must be
   * activated before any usage. Activating SNS does not trigger any billing,
   * and you can deactivate at any time.
   *
   * @param request - The request {@link SnsApiActivateSnsRequest}
   * @returns A Promise of SnsInfo
   */
  activateSns = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalSnsApiActivateSnsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/activate-sns`
  }, unmarshalSnsInfo);

  /**
   * Get SNS info. Retrieve the SNS information of the specified Project ID.
   * Informations include the activation status and the SNS API endpoint URL.
   *
   * @param request - The request {@link SnsApiGetSnsInfoRequest}
   * @returns A Promise of SnsInfo
   */
  getSnsInfo = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-info`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalSnsInfo);

  /**
   * Deactivate SNS. Deactivate SNS for the specified Project ID.You must delete
   * all topics and credentials before this call or you need to set the
   * force_delete parameter.
   *
   * @param request - The request {@link SnsApiDeactivateSnsRequest}
   * @returns A Promise of SnsInfo
   */
  deactivateSns = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalSnsApiDeactivateSnsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deactivate-sns`
  }, unmarshalSnsInfo);

  /**
   * Create SNS credentials. Create a set of credentials for SNS, specified by a
   * Project ID. Credentials give the bearer access to topics, and the level of
   * permissions can be defined granularly.
   *
   * @param request - The request {@link SnsApiCreateSnsCredentialsRequest}
   * @returns A Promise of SnsCredentials
   */
  createSnsCredentials = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalSnsApiCreateSnsCredentialsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-credentials`
  }, unmarshalSnsCredentials);

  /**
   * Delete SNS credentials. Delete a set of SNS credentials, specified by their
   * credentials ID. Deleting credentials is irreversible and cannot be undone.
   * The credentials can then no longer be used to access SNS.
   *
   * @param request - The request {@link SnsApiDeleteSnsCredentialsRequest}
   */
  deleteSnsCredentials = request => this.client.fetch({
    method: 'DELETE',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-credentials/${validatePathParam('snsCredentialsId', request.snsCredentialsId)}`
  });

  /**
   * Update SNS credentials. Update a set of SNS credentials. You can update the
   * credentials' name, or their permissions.
   *
   * @param request - The request {@link SnsApiUpdateSnsCredentialsRequest}
   * @returns A Promise of SnsCredentials
   */
  updateSnsCredentials = request => this.client.fetch({
    body: JSON.stringify(marshalSnsApiUpdateSnsCredentialsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'PATCH',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-credentials/${validatePathParam('snsCredentialsId', request.snsCredentialsId)}`
  }, unmarshalSnsCredentials);

  /**
   * Get SNS credentials. Retrieve an existing set of credentials, identified by
   * the `credentials_id`. The credentials themselves, as well as their metadata
   * (name, project ID etc), are returned in the response.
   *
   * @param request - The request {@link SnsApiGetSnsCredentialsRequest}
   * @returns A Promise of SnsCredentials
   */
  getSnsCredentials = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-credentials/${validatePathParam('snsCredentialsId', request.snsCredentialsId)}`
  }, unmarshalSnsCredentials);
  pageOfListSnsCredentials = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sns-credentials`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListSnsCredentialsResponse);

  /**
   * List SNS credentials. List existing SNS credentials in the specified
   * region. The response contains only the metadata for the credentials, not
   * the credentials themselves.
   *
   * @param request - The request {@link SnsApiListSnsCredentialsRequest}
   * @returns A Promise of ListSnsCredentialsResponse
   */
  listSnsCredentials = (request = {}) => enrichForPagination('snsCredentials', this.pageOfListSnsCredentials, request);
}

/**
 * Messaging and Queuing SQS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing SQS brokers.
 */
class SqsAPI extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams'];

  /**
   * Activate SQS. Activate SQS for the specified Project ID. SQS must be
   * activated before any usage such as creating credentials and queues.
   * Activating SQS does not trigger any billing, and you can deactivate at any
   * time.
   *
   * @param request - The request {@link SqsApiActivateSqsRequest}
   * @returns A Promise of SqsInfo
   */
  activateSqs = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalSqsApiActivateSqsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/activate-sqs`
  }, unmarshalSqsInfo);

  /**
   * Get SQS info. Retrieve the SQS information of the specified Project ID.
   * Informations include the activation status and the SQS API endpoint URL.
   *
   * @param request - The request {@link SqsApiGetSqsInfoRequest}
   * @returns A Promise of SqsInfo
   */
  getSqsInfo = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-info`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalSqsInfo);

  /**
   * Deactivate SQS. Deactivate SQS for the specified Project ID. You must
   * delete all queues and credentials before this call or you need to set the
   * force_delete parameter.
   *
   * @param request - The request {@link SqsApiDeactivateSqsRequest}
   * @returns A Promise of SqsInfo
   */
  deactivateSqs = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalSqsApiDeactivateSqsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/deactivate-sqs`
  }, unmarshalSqsInfo);

  /**
   * Create SQS credentials. Create a set of credentials for SQS, specified by a
   * Project ID. Credentials give the bearer access to queues, and the level of
   * permissions can be defined granularly.
   *
   * @param request - The request {@link SqsApiCreateSqsCredentialsRequest}
   * @returns A Promise of SqsCredentials
   */
  createSqsCredentials = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalSqsApiCreateSqsCredentialsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'POST',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-credentials`
  }, unmarshalSqsCredentials);

  /**
   * Delete SQS credentials. Delete a set of SQS credentials, specified by their
   * credentials ID. Deleting credentials is irreversible and cannot be undone.
   * The credentials can then no longer be used to access SQS.
   *
   * @param request - The request {@link SqsApiDeleteSqsCredentialsRequest}
   */
  deleteSqsCredentials = request => this.client.fetch({
    method: 'DELETE',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-credentials/${validatePathParam('sqsCredentialsId', request.sqsCredentialsId)}`
  });

  /**
   * Update SQS credentials. Update a set of SQS credentials. You can update the
   * credentials' name, or their permissions.
   *
   * @param request - The request {@link SqsApiUpdateSqsCredentialsRequest}
   * @returns A Promise of SqsCredentials
   */
  updateSqsCredentials = request => this.client.fetch({
    body: JSON.stringify(marshalSqsApiUpdateSqsCredentialsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$c,
    method: 'PATCH',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-credentials/${validatePathParam('sqsCredentialsId', request.sqsCredentialsId)}`
  }, unmarshalSqsCredentials);

  /**
   * Get SQS credentials. Retrieve an existing set of credentials, identified by
   * the `credentials_id`. The credentials themselves, as well as their metadata
   * (name, project ID etc), are returned in the response.
   *
   * @param request - The request {@link SqsApiGetSqsCredentialsRequest}
   * @returns A Promise of SqsCredentials
   */
  getSqsCredentials = request => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-credentials/${validatePathParam('sqsCredentialsId', request.sqsCredentialsId)}`
  }, unmarshalSqsCredentials);
  pageOfListSqsCredentials = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/mnq/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/sqs-credentials`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListSqsCredentialsResponse);

  /**
   * List SQS credentials. List existing SQS credentials in the specified
   * region. The response contains only the metadata for the credentials, not
   * the credentials themselves.
   *
   * @param request - The request {@link SqsApiListSqsCredentialsRequest}
   * @returns A Promise of ListSqsCredentialsResponse
   */
  listSqsCredentials = (request = {}) => enrichForPagination('sqsCredentials', this.pageOfListSqsCredentials, request);
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const NatsAccount = {
  name: {
    maxLength: 64,
    minLength: 1
  }
};
const NatsApiCreateNatsAccountRequest = {
  name: {
    maxLength: 64,
    minLength: 1
  }
};
const NatsApiCreateNatsCredentialsRequest = {
  name: {
    maxLength: 500,
    minLength: 1
  }
};
const NatsApiUpdateNatsAccountRequest = {
  name: {
    maxLength: 64,
    minLength: 1
  }
};
const NatsCredentials = {
  name: {
    maxLength: 500,
    minLength: 1
  }
};
const SnsApiCreateSnsCredentialsRequest = {
  name: {
    maxLength: 500,
    minLength: 1
  }
};
const SnsApiUpdateSnsCredentialsRequest = {
  name: {
    maxLength: 500,
    minLength: 1
  }
};
const SnsCredentials = {
  accessKey: {
    minLength: 1
  },
  name: {
    maxLength: 500,
    minLength: 1
  }
};
const SqsApiCreateSqsCredentialsRequest = {
  name: {
    maxLength: 500,
    minLength: 1
  }
};
const SqsApiUpdateSqsCredentialsRequest = {
  name: {
    maxLength: 500,
    minLength: 1
  }
};
const SqsCredentials = {
  accessKey: {
    minLength: 1
  },
  name: {
    maxLength: 500,
    minLength: 1
  }
};

var validationRules_gen$4 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  NatsAccount: NatsAccount,
  NatsApiCreateNatsAccountRequest: NatsApiCreateNatsAccountRequest,
  NatsApiCreateNatsCredentialsRequest: NatsApiCreateNatsCredentialsRequest,
  NatsApiUpdateNatsAccountRequest: NatsApiUpdateNatsAccountRequest,
  NatsCredentials: NatsCredentials,
  SnsApiCreateSnsCredentialsRequest: SnsApiCreateSnsCredentialsRequest,
  SnsApiUpdateSnsCredentialsRequest: SnsApiUpdateSnsCredentialsRequest,
  SnsCredentials: SnsCredentials,
  SqsApiCreateSqsCredentialsRequest: SqsApiCreateSqsCredentialsRequest,
  SqsApiUpdateSqsCredentialsRequest: SqsApiUpdateSqsCredentialsRequest,
  SqsCredentials: SqsCredentials
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$d = /*#__PURE__*/Object.freeze({
  __proto__: null,
  NatsAPI: NatsAPI,
  SnsAPI: SnsAPI,
  SqsAPI: SqsAPI,
  ValidationRules: validationRules_gen$4
});

var index$a = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1beta1: index_gen$d
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link DatabaseBackupStatus}. */
const DATABASE_BACKUP_TRANSIENT_STATUSES = ['creating', 'restoring', 'deleting', 'exporting'];

/** Lists transient statutes of the enum {@link InstanceLogStatus}. */
const INSTANCE_LOG_TRANSIENT_STATUSES = ['creating'];

/** Lists transient statutes of the enum {@link InstanceStatus}. */
const INSTANCE_TRANSIENT_STATUSES = ['provisioning', 'configuring', 'deleting', 'autohealing', 'initializing', 'backuping', 'snapshotting', 'restarting'];

/** Lists transient statutes of the enum {@link MaintenanceStatus}. */
const MAINTENANCE_TRANSIENT_STATUSES = ['ongoing'];

/** Lists transient statutes of the enum {@link ReadReplicaStatus}. */
const READ_REPLICA_TRANSIENT_STATUSES = ['provisioning', 'initializing', 'deleting', 'configuring', 'promoting'];

/** Lists transient statutes of the enum {@link SnapshotStatus}. */
const SNAPSHOT_TRANSIENT_STATUSES = ['creating', 'restoring', 'deleting'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalEndpointDirectAccessDetails = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EndpointDirectAccessDetails' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalEndpointLoadBalancerDetails = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EndpointLoadBalancerDetails' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalEndpointPrivateNetworkDetails = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EndpointPrivateNetworkDetails' failed as data isn't a dictionary.`);
  }
  return {
    privateNetworkId: data.private_network_id,
    provisioningMode: data.provisioning_mode,
    serviceIp: data.service_ip,
    zone: data.zone
  };
};
const unmarshalEndpoint$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Endpoint' failed as data isn't a dictionary.`);
  }
  return {
    directAccess: data.direct_access ? unmarshalEndpointDirectAccessDetails(data.direct_access) : undefined,
    hostname: data.hostname,
    id: data.id,
    ip: data.ip,
    loadBalancer: data.load_balancer ? unmarshalEndpointLoadBalancerDetails(data.load_balancer) : undefined,
    name: data.name,
    port: data.port,
    privateNetwork: data.private_network ? unmarshalEndpointPrivateNetworkDetails(data.private_network) : undefined
  };
};
const unmarshalMaintenance = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Maintenance' failed as data isn't a dictionary.`);
  }
  return {
    closedAt: unmarshalDate(data.closed_at),
    forcedAt: unmarshalDate(data.forced_at),
    reason: data.reason,
    startsAt: unmarshalDate(data.starts_at),
    status: data.status,
    stopsAt: unmarshalDate(data.stops_at)
  };
};
const unmarshalReadReplica = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ReadReplica' failed as data isn't a dictionary.`);
  }
  return {
    endpoints: unmarshalArrayOfObject(data.endpoints, unmarshalEndpoint$1),
    id: data.id,
    instanceId: data.instance_id,
    region: data.region,
    sameZone: data.same_zone,
    status: data.status
  };
};
const unmarshalDatabaseBackup$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DatabaseBackup' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    databaseName: data.database_name,
    downloadUrl: data.download_url,
    downloadUrlExpiresAt: unmarshalDate(data.download_url_expires_at),
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    instanceId: data.instance_id,
    instanceName: data.instance_name,
    name: data.name,
    region: data.region,
    sameRegion: data.same_region,
    size: data.size,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalDatabase$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Database' failed as data isn't a dictionary.`);
  }
  return {
    managed: data.managed,
    name: data.name,
    owner: data.owner,
    size: data.size
  };
};
const unmarshalInstanceLog = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'InstanceLog' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    downloadUrl: data.download_url,
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    nodeName: data.node_name,
    region: data.region,
    status: data.status
  };
};
const unmarshalBackupSchedule = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BackupSchedule' failed as data isn't a dictionary.`);
  }
  return {
    disabled: data.disabled,
    frequency: data.frequency,
    nextRunAt: unmarshalDate(data.next_run_at),
    retention: data.retention
  };
};
const unmarshalInstanceSetting = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'InstanceSetting' failed as data isn't a dictionary.`);
  }
  return {
    name: data.name,
    value: data.value
  };
};
const unmarshalLogsPolicy = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'LogsPolicy' failed as data isn't a dictionary.`);
  }
  return {
    maxAgeRetention: data.max_age_retention,
    totalDiskRetention: data.total_disk_retention
  };
};
const unmarshalUpgradableVersion = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'UpgradableVersion' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    minorVersion: data.minor_version,
    name: data.name,
    version: data.version
  };
};
const unmarshalVolume = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Volume' failed as data isn't a dictionary.`);
  }
  return {
    class: data.class,
    size: data.size,
    type: data.type
  };
};
const unmarshalInstance = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Instance' failed as data isn't a dictionary.`);
  }
  return {
    backupSameRegion: data.backup_same_region,
    backupSchedule: data.backup_schedule ? unmarshalBackupSchedule(data.backup_schedule) : undefined,
    createdAt: unmarshalDate(data.created_at),
    endpoint: data.endpoint ? unmarshalEndpoint$1(data.endpoint) : undefined,
    endpoints: unmarshalArrayOfObject(data.endpoints, unmarshalEndpoint$1),
    engine: data.engine,
    id: data.id,
    initSettings: unmarshalArrayOfObject(data.init_settings, unmarshalInstanceSetting),
    isHaCluster: data.is_ha_cluster,
    logsPolicy: data.logs_policy ? unmarshalLogsPolicy(data.logs_policy) : undefined,
    maintenances: unmarshalArrayOfObject(data.maintenances, unmarshalMaintenance),
    name: data.name,
    nodeType: data.node_type,
    organizationId: data.organization_id,
    projectId: data.project_id,
    readReplicas: unmarshalArrayOfObject(data.read_replicas, unmarshalReadReplica),
    region: data.region,
    settings: unmarshalArrayOfObject(data.settings, unmarshalInstanceSetting),
    status: data.status,
    tags: data.tags,
    upgradableVersion: unmarshalArrayOfObject(data.upgradable_version, unmarshalUpgradableVersion),
    volume: data.volume ? unmarshalVolume(data.volume) : undefined
  };
};
const unmarshalPrivilege = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Privilege' failed as data isn't a dictionary.`);
  }
  return {
    databaseName: data.database_name,
    permission: data.permission,
    userName: data.user_name
  };
};
const unmarshalSnapshotVolumeType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SnapshotVolumeType' failed as data isn't a dictionary.`);
  }
  return {
    class: data.class,
    type: data.type
  };
};
const unmarshalSnapshot = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Snapshot' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    instanceId: data.instance_id,
    instanceName: data.instance_name,
    name: data.name,
    nodeType: data.node_type,
    region: data.region,
    size: data.size,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at),
    volumeType: data.volume_type ? unmarshalSnapshotVolumeType(data.volume_type) : undefined
  };
};
const unmarshalUser = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'User' failed as data isn't a dictionary.`);
  }
  return {
    isAdmin: data.is_admin,
    name: data.name
  };
};
const unmarshalACLRule$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ACLRule' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    description: data.description,
    direction: data.direction,
    ip: data.ip,
    port: data.port,
    protocol: data.protocol
  };
};
const unmarshalAddInstanceACLRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AddInstanceACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$1)
  };
};
const unmarshalAddInstanceSettingsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AddInstanceSettingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    settings: unmarshalArrayOfObject(data.settings, unmarshalInstanceSetting)
  };
};
const unmarshalDeleteInstanceACLRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeleteInstanceACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$1)
  };
};
const unmarshalDeleteInstanceSettingsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeleteInstanceSettingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    settings: unmarshalArrayOfObject(data.settings, unmarshalInstanceSetting)
  };
};
const unmarshalInstanceMetrics = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'InstanceMetrics' failed as data isn't a dictionary.`);
  }
  return {
    timeseries: unmarshalArrayOfObject(data.timeseries, unmarshalTimeSeries)
  };
};
const unmarshalListDatabaseBackupsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDatabaseBackupsResponse' failed as data isn't a dictionary.`);
  }
  return {
    databaseBackups: unmarshalArrayOfObject(data.database_backups, unmarshalDatabaseBackup$1),
    totalCount: data.total_count
  };
};
const unmarshalEngineSetting = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EngineSetting' failed as data isn't a dictionary.`);
  }
  return {
    defaultValue: data.default_value,
    description: data.description,
    floatMax: data.float_max,
    floatMin: data.float_min,
    hotConfigurable: data.hot_configurable,
    intMax: data.int_max,
    intMin: data.int_min,
    name: data.name,
    propertyType: data.property_type,
    stringConstraint: data.string_constraint,
    unit: data.unit
  };
};
const unmarshalEngineVersion = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EngineVersion' failed as data isn't a dictionary.`);
  }
  return {
    availableInitSettings: unmarshalArrayOfObject(data.available_init_settings, unmarshalEngineSetting),
    availableSettings: unmarshalArrayOfObject(data.available_settings, unmarshalEngineSetting),
    beta: data.beta,
    disabled: data.disabled,
    endOfLife: unmarshalDate(data.end_of_life),
    name: data.name,
    version: data.version
  };
};
const unmarshalDatabaseEngine = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DatabaseEngine' failed as data isn't a dictionary.`);
  }
  return {
    logoUrl: data.logo_url,
    name: data.name,
    region: data.region,
    versions: unmarshalArrayOfObject(data.versions, unmarshalEngineVersion)
  };
};
const unmarshalListDatabaseEnginesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDatabaseEnginesResponse' failed as data isn't a dictionary.`);
  }
  return {
    engines: unmarshalArrayOfObject(data.engines, unmarshalDatabaseEngine),
    totalCount: data.total_count
  };
};
const unmarshalListDatabasesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDatabasesResponse' failed as data isn't a dictionary.`);
  }
  return {
    databases: unmarshalArrayOfObject(data.databases, unmarshalDatabase$1),
    totalCount: data.total_count
  };
};
const unmarshalListInstanceACLRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstanceACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$1),
    totalCount: data.total_count
  };
};
const unmarshalListInstanceLogsDetailsResponseInstanceLogDetail = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstanceLogsDetailsResponseInstanceLogDetail' failed as data isn't a dictionary.`);
  }
  return {
    logName: data.log_name,
    size: data.size
  };
};
const unmarshalListInstanceLogsDetailsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstanceLogsDetailsResponse' failed as data isn't a dictionary.`);
  }
  return {
    details: unmarshalArrayOfObject(data.details, unmarshalListInstanceLogsDetailsResponseInstanceLogDetail)
  };
};
const unmarshalListInstanceLogsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstanceLogsResponse' failed as data isn't a dictionary.`);
  }
  return {
    instanceLogs: unmarshalArrayOfObject(data.instance_logs, unmarshalInstanceLog)
  };
};
const unmarshalListInstancesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListInstancesResponse' failed as data isn't a dictionary.`);
  }
  return {
    instances: unmarshalArrayOfObject(data.instances, unmarshalInstance),
    totalCount: data.total_count
  };
};
const unmarshalNodeTypeVolumeConstraintSizes = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NodeTypeVolumeConstraintSizes' failed as data isn't a dictionary.`);
  }
  return {
    maxSize: data.max_size,
    minSize: data.min_size
  };
};
const unmarshalNodeTypeVolumeType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NodeTypeVolumeType' failed as data isn't a dictionary.`);
  }
  return {
    chunkSize: data.chunk_size,
    class: data.class,
    description: data.description,
    maxSize: data.max_size,
    minSize: data.min_size,
    type: data.type
  };
};
const unmarshalNodeType$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NodeType' failed as data isn't a dictionary.`);
  }
  return {
    availableVolumeTypes: unmarshalArrayOfObject(data.available_volume_types, unmarshalNodeTypeVolumeType),
    beta: data.beta,
    description: data.description,
    disabled: data.disabled,
    generation: data.generation,
    instanceRange: data.instance_range,
    isBssdCompatible: data.is_bssd_compatible,
    isHaRequired: data.is_ha_required,
    memory: data.memory,
    name: data.name,
    region: data.region,
    stockStatus: data.stock_status,
    vcpus: data.vcpus,
    volumeConstraint: data.volume_constraint ? unmarshalNodeTypeVolumeConstraintSizes(data.volume_constraint) : undefined
  };
};
const unmarshalListNodeTypesResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNodeTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    nodeTypes: unmarshalArrayOfObject(data.node_types, unmarshalNodeType$1),
    totalCount: data.total_count
  };
};
const unmarshalListPrivilegesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPrivilegesResponse' failed as data isn't a dictionary.`);
  }
  return {
    privileges: unmarshalArrayOfObject(data.privileges, unmarshalPrivilege),
    totalCount: data.total_count
  };
};
const unmarshalListSnapshotsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSnapshotsResponse' failed as data isn't a dictionary.`);
  }
  return {
    snapshots: unmarshalArrayOfObject(data.snapshots, unmarshalSnapshot),
    totalCount: data.total_count
  };
};
const unmarshalListUsersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListUsersResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    users: unmarshalArrayOfObject(data.users, unmarshalUser)
  };
};
const unmarshalPrepareInstanceLogsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrepareInstanceLogsResponse' failed as data isn't a dictionary.`);
  }
  return {
    instanceLogs: unmarshalArrayOfObject(data.instance_logs, unmarshalInstanceLog)
  };
};
const unmarshalSetInstanceACLRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetInstanceACLRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    rules: unmarshalArrayOfObject(data.rules, unmarshalACLRule$1)
  };
};
const unmarshalSetInstanceSettingsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetInstanceSettingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    settings: unmarshalArrayOfObject(data.settings, unmarshalInstanceSetting)
  };
};
const marshalACLRuleRequest = (request, defaults) => ({
  description: request.description,
  ip: request.ip
});
const marshalAddInstanceACLRulesRequest = (request, defaults) => ({
  rules: request.rules.map(elt => marshalACLRuleRequest(elt))
});
const marshalInstanceSetting = (request, defaults) => ({
  name: request.name,
  value: request.value
});
const marshalAddInstanceSettingsRequest = (request, defaults) => ({
  settings: request.settings.map(elt => marshalInstanceSetting(elt))
});
const marshalCloneInstanceRequest = (request, defaults) => ({
  name: request.name,
  node_type: request.nodeType
});
const marshalCreateDatabaseBackupRequest = (request, defaults) => ({
  database_name: request.databaseName,
  expires_at: request.expiresAt,
  instance_id: request.instanceId,
  name: request.name || randomName('bkp')
});
const marshalCreateDatabaseRequest$1 = (request, defaults) => ({
  name: request.name
});
const marshalEndpointSpecPrivateNetworkIpamConfig = (request, defaults) => ({});
const marshalEndpointSpecLoadBalancer = (request, defaults) => ({});
const marshalEndpointSpecPrivateNetwork = (request, defaults) => ({
  private_network_id: request.privateNetworkId,
  ...resolveOneOf([{
    param: 'service_ip',
    value: request.serviceIp
  }, {
    param: 'ipam_config',
    value: request.ipamConfig !== undefined ? marshalEndpointSpecPrivateNetworkIpamConfig(request.ipamConfig) : undefined
  }])
});
const marshalEndpointSpec$1 = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'load_balancer',
    value: request.loadBalancer !== undefined ? marshalEndpointSpecLoadBalancer(request.loadBalancer) : undefined
  }, {
    param: 'private_network',
    value: request.privateNetwork !== undefined ? marshalEndpointSpecPrivateNetwork(request.privateNetwork) : undefined
  }])
});
const marshalCreateEndpointRequest = (request, defaults) => ({
  endpoint_spec: request.endpointSpec !== undefined ? marshalEndpointSpec$1(request.endpointSpec) : undefined
});
const marshalCreateInstanceFromSnapshotRequest = (request, defaults) => ({
  instance_name: request.instanceName,
  is_ha_cluster: request.isHaCluster,
  node_type: request.nodeType
});
const marshalCreateInstanceRequest = (request, defaults) => ({
  backup_same_region: request.backupSameRegion,
  disable_backup: request.disableBackup,
  engine: request.engine,
  init_endpoints: request.initEndpoints !== undefined ? request.initEndpoints.map(elt => marshalEndpointSpec$1(elt)) : undefined,
  init_settings: request.initSettings !== undefined ? request.initSettings.map(elt => marshalInstanceSetting(elt)) : undefined,
  is_ha_cluster: request.isHaCluster,
  name: request.name || randomName('ins'),
  node_type: request.nodeType,
  password: request.password,
  tags: request.tags,
  user_name: request.userName,
  volume_size: request.volumeSize,
  volume_type: request.volumeType,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalReadReplicaEndpointSpecPrivateNetworkIpamConfig = (request, defaults) => ({});
const marshalReadReplicaEndpointSpecDirectAccess = (request, defaults) => ({});
const marshalReadReplicaEndpointSpecPrivateNetwork = (request, defaults) => ({
  private_network_id: request.privateNetworkId,
  ...resolveOneOf([{
    param: 'service_ip',
    value: request.serviceIp
  }, {
    param: 'ipam_config',
    value: request.ipamConfig !== undefined ? marshalReadReplicaEndpointSpecPrivateNetworkIpamConfig(request.ipamConfig) : undefined
  }])
});
const marshalReadReplicaEndpointSpec = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'direct_access',
    value: request.directAccess !== undefined ? marshalReadReplicaEndpointSpecDirectAccess(request.directAccess) : undefined
  }, {
    param: 'private_network',
    value: request.privateNetwork !== undefined ? marshalReadReplicaEndpointSpecPrivateNetwork(request.privateNetwork) : undefined
  }])
});
const marshalCreateReadReplicaEndpointRequest = (request, defaults) => ({
  endpoint_spec: request.endpointSpec.map(elt => marshalReadReplicaEndpointSpec(elt))
});
const marshalCreateReadReplicaRequest = (request, defaults) => ({
  endpoint_spec: request.endpointSpec !== undefined ? request.endpointSpec.map(elt => marshalReadReplicaEndpointSpec(elt)) : undefined,
  instance_id: request.instanceId,
  same_zone: request.sameZone
});
const marshalCreateSnapshotRequest = (request, defaults) => ({
  expires_at: request.expiresAt,
  name: request.name || randomName('snp')
});
const marshalCreateUserRequest = (request, defaults) => ({
  is_admin: request.isAdmin,
  name: request.name,
  password: request.password
});
const marshalDeleteInstanceACLRulesRequest = (request, defaults) => ({
  acl_rule_ips: request.aclRuleIps
});
const marshalDeleteInstanceSettingsRequest = (request, defaults) => ({
  setting_names: request.settingNames
});
const marshalMigrateEndpointRequest = (request, defaults) => ({
  instance_id: request.instanceId
});
const marshalPrepareInstanceLogsRequest = (request, defaults) => ({
  end_date: request.endDate,
  start_date: request.startDate
});
const marshalPurgeInstanceLogsRequest = (request, defaults) => ({
  log_name: request.logName
});
const marshalRestoreDatabaseBackupRequest = (request, defaults) => ({
  database_name: request.databaseName,
  instance_id: request.instanceId
});
const marshalSetInstanceACLRulesRequest = (request, defaults) => ({
  rules: request.rules.map(elt => marshalACLRuleRequest(elt))
});
const marshalSetInstanceSettingsRequest = (request, defaults) => ({
  settings: request.settings.map(elt => marshalInstanceSetting(elt))
});
const marshalSetPrivilegeRequest = (request, defaults) => ({
  database_name: request.databaseName,
  permission: request.permission,
  user_name: request.userName
});
const marshalUpdateDatabaseBackupRequest = (request, defaults) => ({
  expires_at: request.expiresAt,
  name: request.name
});
const marshalLogsPolicy = (request, defaults) => ({
  max_age_retention: request.maxAgeRetention,
  total_disk_retention: request.totalDiskRetention
});
const marshalUpdateInstanceRequest = (request, defaults) => ({
  backup_same_region: request.backupSameRegion,
  backup_schedule_frequency: request.backupScheduleFrequency,
  backup_schedule_retention: request.backupScheduleRetention,
  backup_schedule_start_hour: request.backupScheduleStartHour,
  is_backup_schedule_disabled: request.isBackupScheduleDisabled,
  logs_policy: request.logsPolicy !== undefined ? marshalLogsPolicy(request.logsPolicy) : undefined,
  name: request.name,
  tags: request.tags
});
const marshalUpdateSnapshotRequest = (request, defaults) => ({
  expires_at: request.expiresAt,
  name: request.name
});
const marshalUpdateUserRequest = (request, defaults) => ({
  is_admin: request.isAdmin,
  password: request.password
});
const marshalUpgradeInstanceRequestMajorUpgradeWorkflow = (request, defaults) => ({
  upgradable_version_id: request.upgradableVersionId,
  with_endpoints: request.withEndpoints
});
const marshalUpgradeInstanceRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'node_type',
    value: request.nodeType
  }, {
    param: 'enable_ha',
    value: request.enableHa
  }, {
    param: 'volume_size',
    value: request.volumeSize
  }, {
    param: 'volume_type',
    value: request.volumeType
  }, {
    param: 'upgradable_version_id',
    value: request.upgradableVersionId
  }, {
    param: 'major_upgrade_workflow',
    value: request.majorUpgradeWorkflow !== undefined ? marshalUpgradeInstanceRequestMajorUpgradeWorkflow(request.majorUpgradeWorkflow) : undefined
  }])
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$b = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Managed Database for PostgreSQL and MySQL API. */
let API$b = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];
  pageOfListDatabaseEngines = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/database-engines`,
    urlParams: urlParams(['name', request.name], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['version', request.version])
  }, unmarshalListDatabaseEnginesResponse);

  /**
   * List available database engines. List the PostgreSQL and MySQL database
   * engines available at Scaleway.
   *
   * @param request - The request {@link ListDatabaseEnginesRequest}
   * @returns A Promise of ListDatabaseEnginesResponse
   */
  listDatabaseEngines = (request = {}) => enrichForPagination('engines', this.pageOfListDatabaseEngines, request);
  pageOfListNodeTypes = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/node-types`,
    urlParams: urlParams(['include_disabled_types', request.includeDisabledTypes], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListNodeTypesResponse$1);

  /**
   * List available node types. List all available node types. By default, the
   * node types returned in the list are ordered by creation date in ascending
   * order, though this can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListNodeTypesRequest}
   * @returns A Promise of ListNodeTypesResponse
   */
  listNodeTypes = request => enrichForPagination('nodeTypes', this.pageOfListNodeTypes, request);
  pageOfListDatabaseBackups = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups`,
    urlParams: urlParams(['instance_id', request.instanceId], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListDatabaseBackupsResponse$1);

  /**
   * List database backups. List all backups in a specified region, for a given
   * Scaleway Organization or Scaleway Project. By default, the backups listed
   * are ordered by creation date in ascending order. This can be modified via
   * the `order_by` field.
   *
   * @param request - The request {@link ListDatabaseBackupsRequest}
   * @returns A Promise of ListDatabaseBackupsResponse
   */
  listDatabaseBackups = (request = {}) => enrichForPagination('databaseBackups', this.pageOfListDatabaseBackups, request);

  /**
   * Create a database backup. Create a new backup. You must set the
   * `instance_id`, `database_name`, `name` and `expires_at` parameters.
   *
   * @param request - The request {@link CreateDatabaseBackupRequest}
   * @returns A Promise of DatabaseBackup
   */
  createDatabaseBackup = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDatabaseBackupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups`
  }, unmarshalDatabaseBackup$1);

  /**
   * Get a database backup. Retrieve information about a given backup, specified
   * by its database backup ID and region. Full details about the backup, like
   * size, URL and expiration date, are returned in the response.
   *
   * @param request - The request {@link GetDatabaseBackupRequest}
   * @returns A Promise of DatabaseBackup
   */
  getDatabaseBackup = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups/${validatePathParam('databaseBackupId', request.databaseBackupId)}`
  }, unmarshalDatabaseBackup$1);

  /**
   * Waits for {@link DatabaseBackup} to be in a final state.
   *
   * @param request - The request {@link GetDatabaseBackupRequest}
   * @param options - The waiting options
   * @returns A Promise of DatabaseBackup
   */
  waitForDatabaseBackup = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!DATABASE_BACKUP_TRANSIENT_STATUSES.includes(res.status))), this.getDatabaseBackup, request, options);

  /**
   * Update a database backup. Update the parameters of a backup, including name
   * and expiration date.
   *
   * @param request - The request {@link UpdateDatabaseBackupRequest}
   * @returns A Promise of DatabaseBackup
   */
  updateDatabaseBackup = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDatabaseBackupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'PATCH',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups/${validatePathParam('databaseBackupId', request.databaseBackupId)}`
  }, unmarshalDatabaseBackup$1);

  /**
   * Delete a database backup. Delete a backup, specified by its database backup
   * ID and region. Deleting a backup is permanent, and cannot be undone.
   *
   * @param request - The request {@link DeleteDatabaseBackupRequest}
   * @returns A Promise of DatabaseBackup
   */
  deleteDatabaseBackup = request => this.client.fetch({
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups/${validatePathParam('databaseBackupId', request.databaseBackupId)}`
  }, unmarshalDatabaseBackup$1);

  /**
   * Restore a database backup. Launch the process of restoring database backup.
   * You must specify the `instance_id` of the Database Instance of destination,
   * where the backup will be restored. Note that large database backups can
   * take up to several hours to restore.
   *
   * @param request - The request {@link RestoreDatabaseBackupRequest}
   * @returns A Promise of DatabaseBackup
   */
  restoreDatabaseBackup = request => this.client.fetch({
    body: JSON.stringify(marshalRestoreDatabaseBackupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups/${validatePathParam('databaseBackupId', request.databaseBackupId)}/restore`
  }, unmarshalDatabaseBackup$1);

  /**
   * Export a database backup. Export a backup, specified by the
   * `database_backup_id` and the `region` parameters. The download URL is
   * returned in the response.
   *
   * @param request - The request {@link ExportDatabaseBackupRequest}
   * @returns A Promise of DatabaseBackup
   */
  exportDatabaseBackup = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups/${validatePathParam('databaseBackupId', request.databaseBackupId)}/export`
  }, unmarshalDatabaseBackup$1);

  /**
   * Upgrade a Database Instance. Upgrade your current Database Instance
   * specifications like node type, high availability, volume, or the database
   * engine version. Note that upon upgrade the `enable_ha` parameter can only
   * be set to `true`.
   *
   * @param request - The request {@link UpgradeInstanceRequest}
   * @returns A Promise of Instance
   */
  upgradeInstance = request => this.client.fetch({
    body: JSON.stringify(marshalUpgradeInstanceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/upgrade`
  }, unmarshalInstance);
  pageOfListInstances = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalListInstancesResponse);

  /**
   * List Database Instances. List all Database Instances in the specified
   * region, for a given Scaleway Organization or Scaleway Project. By default,
   * the Database Instances returned in the list are ordered by creation date in
   * ascending order, though this can be modified via the order_by field. You
   * can define additional parameters for your query, such as `tags` and `name`.
   * For the `name` parameter, the value you include will be checked against the
   * whole name string to see if it includes the string you put in the
   * parameter.
   *
   * @param request - The request {@link ListInstancesRequest}
   * @returns A Promise of ListInstancesResponse
   */
  listInstances = (request = {}) => enrichForPagination('instances', this.pageOfListInstances, request);

  /**
   * Get a Database Instance. Retrieve information about a given Database
   * Instance, specified by the `region` and `instance_id` parameters. Its full
   * details, including name, status, IP address and port, are returned in the
   * response object.
   *
   * @param request - The request {@link GetInstanceRequest}
   * @returns A Promise of Instance
   */
  getInstance = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}`
  }, unmarshalInstance);

  /**
   * Waits for {@link Instance} to be in a final state.
   *
   * @param request - The request {@link GetInstanceRequest}
   * @param options - The waiting options
   * @returns A Promise of Instance
   */
  waitForInstance = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!INSTANCE_TRANSIENT_STATUSES.includes(res.status))), this.getInstance, request, options);

  /**
   * Create a Database Instance. Create a new Database Instance. You must set
   * the `engine`, `user_name`, `password` and `node_type` parameters.
   * Optionally, you can specify the volume type and size.
   *
   * @param request - The request {@link CreateInstanceRequest}
   * @returns A Promise of Instance
   */
  createInstance = request => this.client.fetch({
    body: JSON.stringify(marshalCreateInstanceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances`
  }, unmarshalInstance);

  /**
   * Update a Database Instance. Update the parameters of a Database Instance,
   * including name, tags and backup schedule details.
   *
   * @param request - The request {@link UpdateInstanceRequest}
   * @returns A Promise of Instance
   */
  updateInstance = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateInstanceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'PATCH',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}`
  }, unmarshalInstance);

  /**
   * Delete a Database Instance. Delete a given Database Instance, specified by
   * the `region` and `instance_id` parameters. Deleting a Database Instance is
   * permanent, and cannot be undone. Note that upon deletion all your data will
   * be lost.
   *
   * @param request - The request {@link DeleteInstanceRequest}
   * @returns A Promise of Instance
   */
  deleteInstance = request => this.client.fetch({
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}`
  }, unmarshalInstance);

  /**
   * Clone a Database Instance. Clone a given Database Instance, specified by
   * the `region` and `instance_id` parameters. The clone feature allows you to
   * create a new Database Instance from an existing one. The clone includes all
   * existing databases, users and permissions. You can create a clone on a
   * Database Instance bigger than your current one.
   *
   * @param request - The request {@link CloneInstanceRequest}
   * @returns A Promise of Instance
   */
  cloneInstance = request => this.client.fetch({
    body: JSON.stringify(marshalCloneInstanceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/clone`
  }, unmarshalInstance);

  /**
   * Restart Database Instance. Restart a given Database Instance, specified by
   * the `region` and `instance_id` parameters. The status of the Database
   * Instance returned in the response.
   *
   * @param request - The request {@link RestartInstanceRequest}
   * @returns A Promise of Instance
   */
  restartInstance = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/restart`
  }, unmarshalInstance);

  /**
   * Get the TLS certificate of a Database Instance. Retrieve information about
   * the TLS certificate of a given Database Instance. Details like name and
   * content are returned in the response.
   *
   * @param request - The request {@link GetInstanceCertificateRequest}
   * @returns A Promise of Blob
   */
  getInstanceCertificate = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/certificate`,
    urlParams: urlParams(['dl', 1]),
    responseType: 'blob'
  });

  /**
   * Renew the TLS certificate of a Database Instance. Renew a TLS for a
   * Database Instance. Renewing a certificate means that you will not be able
   * to connect to your Database Instance using the previous certificate. You
   * will also need to download and update the new certificate for all database
   * clients.
   *
   * @param request - The request {@link RenewInstanceCertificateRequest}
   */
  renewInstanceCertificate = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/renew-certificate`
  });

  /**
   * Get Database Instance metrics. Retrieve the time series metrics of a given
   * Database Instance. You can define the period from which to retrieve metrics
   * by specifying the `start_date` and `end_date`.
   *
   * @param request - The request {@link GetInstanceMetricsRequest}
   * @returns A Promise of InstanceMetrics
   */
  getInstanceMetrics = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/metrics`,
    urlParams: urlParams(['end_date', request.endDate], ['metric_name', request.metricName], ['start_date', request.startDate])
  }, unmarshalInstanceMetrics);

  /**
   * Create a Read Replica. Create a new Read Replica of a Database Instance.
   * You must specify the `region` and the `instance_id`. You can only create a
   * maximum of 3 Read Replicas per Database Instance.
   *
   * @param request - The request {@link CreateReadReplicaRequest}
   * @returns A Promise of ReadReplica
   */
  createReadReplica = request => this.client.fetch({
    body: JSON.stringify(marshalCreateReadReplicaRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas`
  }, unmarshalReadReplica);

  /**
   * Get a Read Replica. Retrieve information about a Database Instance Read
   * Replica. Full details about the Read Replica, like `endpoints`, `status`
   * and `region` are returned in the response.
   *
   * @param request - The request {@link GetReadReplicaRequest}
   * @returns A Promise of ReadReplica
   */
  getReadReplica = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}`
  }, unmarshalReadReplica);

  /**
   * Waits for {@link ReadReplica} to be in a final state.
   *
   * @param request - The request {@link GetReadReplicaRequest}
   * @param options - The waiting options
   * @returns A Promise of ReadReplica
   */
  waitForReadReplica = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!READ_REPLICA_TRANSIENT_STATUSES.includes(res.status))), this.getReadReplica, request, options);

  /**
   * Delete a Read Replica. Delete a Read Replica of a Database Instance. You
   * must specify the `region` and `read_replica_id` parameters of the Read
   * Replica you want to delete.
   *
   * @param request - The request {@link DeleteReadReplicaRequest}
   * @returns A Promise of ReadReplica
   */
  deleteReadReplica = request => this.client.fetch({
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}`
  }, unmarshalReadReplica);

  /**
   * Resync a Read Replica. When you resync a Read Replica, first it is reset,
   * then its data is resynchronized from the primary node. Your Read Replica
   * remains unavailable during the resync process. The duration of this process
   * is proportional to the size of your Database Instance. The configured
   * endpoints do not change.
   *
   * @param request - The request {@link ResetReadReplicaRequest}
   * @returns A Promise of ReadReplica
   */
  resetReadReplica = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}/reset`
  }, unmarshalReadReplica);

  /**
   * Promote a Read Replica. Promote a Read Replica to Database Instance
   * automatically.
   *
   * @param request - The request {@link PromoteReadReplicaRequest}
   * @returns A Promise of Instance
   */
  promoteReadReplica = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}/promote`
  }, unmarshalInstance);

  /**
   * Create an endpoint for a Read Replica. Create a new endpoint for a Read
   * Replica. Read Replicas can have at most one direct access and one Private
   * Network endpoint.
   *
   * @param request - The request {@link CreateReadReplicaEndpointRequest}
   * @returns A Promise of ReadReplica
   */
  createReadReplicaEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalCreateReadReplicaEndpointRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/read-replicas/${validatePathParam('readReplicaId', request.readReplicaId)}/endpoints`
  }, unmarshalReadReplica);

  /**
   * Prepare logs of a Database Instance. Prepare your Database Instance logs.
   * You can define the `start_date` and `end_date` parameters for your query.
   * The download URL is returned in the response. Logs are recorded from 00h00
   * to 23h59 and then aggregated in a `.log` file once a day. Therefore, even
   * if you specify a timeframe from which you want to get the logs, you will
   * receive logs from the full 24 hours.
   *
   * @param request - The request {@link PrepareInstanceLogsRequest}
   * @returns A Promise of PrepareInstanceLogsResponse
   */
  prepareInstanceLogs = request => this.client.fetch({
    body: JSON.stringify(marshalPrepareInstanceLogsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/prepare-logs`
  }, unmarshalPrepareInstanceLogsResponse);

  /**
   * List available logs of a Database Instance. List the available logs of a
   * Database Instance. By default, the logs returned in the list are ordered by
   * creation date in ascending order, though this can be modified via the
   * order_by field.
   *
   * @param request - The request {@link ListInstanceLogsRequest}
   * @returns A Promise of ListInstanceLogsResponse
   */
  listInstanceLogs = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/logs`,
    urlParams: urlParams(['order_by', request.orderBy])
  }, unmarshalListInstanceLogsResponse);

  /**
   * Get given logs of a Database Instance. Retrieve information about the logs
   * of a Database Instance. Specify the `instance_log_id` and `region` in your
   * request to get information such as `download_url`, `status`, `expires_at`
   * and `created_at` about your logs in the response.
   *
   * @param request - The request {@link GetInstanceLogRequest}
   * @returns A Promise of InstanceLog
   */
  getInstanceLog = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/logs/${validatePathParam('instanceLogId', request.instanceLogId)}`
  }, unmarshalInstanceLog);

  /**
   * Waits for {@link InstanceLog} to be in a final state.
   *
   * @param request - The request {@link GetInstanceLogRequest}
   * @param options - The waiting options
   * @returns A Promise of InstanceLog
   */
  waitForInstanceLog = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!INSTANCE_LOG_TRANSIENT_STATUSES.includes(res.status))), this.getInstanceLog, request, options);

  /**
   * Purge remote Database Instance logs. Purge a given remote log from a
   * Database Instance. You can specify the `log_name` of the log you wish to
   * clean from your Database Instance.
   *
   * @param request - The request {@link PurgeInstanceLogsRequest}
   */
  purgeInstanceLogs = request => this.client.fetch({
    body: JSON.stringify(marshalPurgeInstanceLogsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/purge-logs`
  });

  /**
   * List remote Database Instance logs details. List remote log details. By
   * default, the details returned in the list are ordered by creation date in
   * ascending order, though this can be modified via the order_by field.
   *
   * @param request - The request {@link ListInstanceLogsDetailsRequest}
   * @returns A Promise of ListInstanceLogsDetailsResponse
   */
  listInstanceLogsDetails = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/logs-details`
  }, unmarshalListInstanceLogsDetailsResponse);

  /**
   * Add Database Instance advanced settings. Add an advanced setting to a
   * Database Instance. You must set the `name` and the `value` of each
   * setting.
   *
   * @param request - The request {@link AddInstanceSettingsRequest}
   * @returns A Promise of AddInstanceSettingsResponse
   */
  addInstanceSettings = request => this.client.fetch({
    body: JSON.stringify(marshalAddInstanceSettingsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/settings`
  }, unmarshalAddInstanceSettingsResponse);

  /**
   * Delete Database Instance advanced settings. Delete an advanced setting in a
   * Database Instance. You must specify the names of the settings you want to
   * delete in the request.
   *
   * @param request - The request {@link DeleteInstanceSettingsRequest}
   * @returns A Promise of DeleteInstanceSettingsResponse
   */
  deleteInstanceSettings = request => this.client.fetch({
    body: JSON.stringify(marshalDeleteInstanceSettingsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/settings`
  }, unmarshalDeleteInstanceSettingsResponse);

  /**
   * Set Database Instance advanced settings. Update an advanced setting for a
   * Database Instance. Settings added upon database engine initalization can
   * only be defined once, and cannot, therefore, be updated.
   *
   * @param request - The request {@link SetInstanceSettingsRequest}
   * @returns A Promise of SetInstanceSettingsResponse
   */
  setInstanceSettings = request => this.client.fetch({
    body: JSON.stringify(marshalSetInstanceSettingsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'PUT',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/settings`
  }, unmarshalSetInstanceSettingsResponse);
  pageOfListInstanceACLRules = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/acls`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListInstanceACLRulesResponse);

  /**
   * List ACL rules of a Database Instance. List the ACL rules for a given
   * Database Instance. The response is an array of ACL objects, each one
   * representing an ACL that denies, allows or redirects traffic based on
   * certain conditions.
   *
   * @param request - The request {@link ListInstanceACLRulesRequest}
   * @returns A Promise of ListInstanceACLRulesResponse
   */
  listInstanceACLRules = request => enrichForPagination('rules', this.pageOfListInstanceACLRules, request);

  /**
   * Add an ACL rule to a Database Instance. Add an additional ACL rule to a
   * Database Instance.
   *
   * @param request - The request {@link AddInstanceACLRulesRequest}
   * @returns A Promise of AddInstanceACLRulesResponse
   */
  addInstanceACLRules = request => this.client.fetch({
    body: JSON.stringify(marshalAddInstanceACLRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/acls`
  }, unmarshalAddInstanceACLRulesResponse);

  /**
   * Set ACL rules for a Database Instance. Replace all the ACL rules of a
   * Database Instance.
   *
   * @param request - The request {@link SetInstanceACLRulesRequest}
   * @returns A Promise of SetInstanceACLRulesResponse
   */
  setInstanceACLRules = request => this.client.fetch({
    body: JSON.stringify(marshalSetInstanceACLRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'PUT',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/acls`
  }, unmarshalSetInstanceACLRulesResponse);

  /**
   * Delete ACL rules of a Database Instance. Delete one or more ACL rules of a
   * Database Instance.
   *
   * @param request - The request {@link DeleteInstanceACLRulesRequest}
   * @returns A Promise of DeleteInstanceACLRulesResponse
   */
  deleteInstanceACLRules = request => this.client.fetch({
    body: JSON.stringify(marshalDeleteInstanceACLRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/acls`
  }, unmarshalDeleteInstanceACLRulesResponse);
  pageOfListUsers = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/users`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListUsersResponse);

  /**
   * List users of a Database Instance. List all users of a given Database
   * Instance. By default, the users returned in the list are ordered by
   * creation date in ascending order, though this can be modified via the
   * order_by field.
   *
   * @param request - The request {@link ListUsersRequest}
   * @returns A Promise of ListUsersResponse
   */
  listUsers = request => enrichForPagination('users', this.pageOfListUsers, request);

  /**
   * Create a user for a Database Instance. Create a new user for a Database
   * Instance. You must define the `name`, `password` and `is_admin`
   * parameters.
   *
   * @param request - The request {@link CreateUserRequest}
   * @returns A Promise of User
   */
  createUser = request => this.client.fetch({
    body: JSON.stringify(marshalCreateUserRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/users`
  }, unmarshalUser);

  /**
   * Update a user on a Database Instance. Update the parameters of a user on a
   * Database Instance. You can update the `password` and `is_admin` parameters,
   * but you cannot change the name of the user.
   *
   * @param request - The request {@link UpdateUserRequest}
   * @returns A Promise of User
   */
  updateUser = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateUserRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'PATCH',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/users/${validatePathParam('name', request.name)}`
  }, unmarshalUser);

  /**
   * Delete a user on a Database Instance. Delete a given user on a Database
   * Instance. You must specify, in the endpoint, the `region`, `instance_id`
   * and `name` parameters of the user you want to delete.
   *
   * @param request - The request {@link DeleteUserRequest}
   */
  deleteUser = request => this.client.fetch({
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/users/${validatePathParam('name', request.name)}`
  });
  pageOfListDatabases = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/databases`,
    urlParams: urlParams(['managed', request.managed], ['name', request.name], ['order_by', request.orderBy], ['owner', request.owner], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListDatabasesResponse$1);

  /**
   * List databases in a Database Instance. List all databases of a given
   * Database Instance. By default, the databases returned in the list are
   * ordered by creation date in ascending order, though this can be modified
   * via the order_by field. You can define additional parameters for your
   * query, such as `name`, `managed` and `owner`.
   *
   * @param request - The request {@link ListDatabasesRequest}
   * @returns A Promise of ListDatabasesResponse
   */
  listDatabases = request => enrichForPagination('databases', this.pageOfListDatabases, request);

  /**
   * Create a database in a Database Instance. Create a new database. You must
   * define the `name` parameter in the request.
   *
   * @param request - The request {@link CreateDatabaseRequest}
   * @returns A Promise of Database
   */
  createDatabase = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDatabaseRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/databases`
  }, unmarshalDatabase$1);

  /**
   * Delete a database in a Database Instance. Delete a given database on a
   * Database Instance. You must specify, in the endpoint, the `region`,
   * `instance_id` and `name` parameters of the database you want to delete.
   *
   * @param request - The request {@link DeleteDatabaseRequest}
   */
  deleteDatabase = request => this.client.fetch({
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/databases/${validatePathParam('name', request.name)}`
  });
  pageOfListPrivileges = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/privileges`,
    urlParams: urlParams(['database_name', request.databaseName], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['user_name', request.userName])
  }, unmarshalListPrivilegesResponse);

  /**
   * List user privileges for a database. List privileges of a user on a
   * database. By default, the details returned in the list are ordered by
   * creation date in ascending order, though this can be modified via the
   * order_by field. You can define additional parameters for your query, such
   * as `database_name` and `user_name`.
   *
   * @param request - The request {@link ListPrivilegesRequest}
   * @returns A Promise of ListPrivilegesResponse
   */
  listPrivileges = request => enrichForPagination('privileges', this.pageOfListPrivileges, request);

  /**
   * Set user privileges for a database. Set the privileges of a user on a
   * database. You must define `database_name`, `user_name` and `permission` in
   * the request body.
   *
   * @param request - The request {@link SetPrivilegeRequest}
   * @returns A Promise of Privilege
   */
  setPrivilege = request => this.client.fetch({
    body: JSON.stringify(marshalSetPrivilegeRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'PUT',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/privileges`
  }, unmarshalPrivilege);
  pageOfListSnapshots = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots`,
    urlParams: urlParams(['instance_id', request.instanceId], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListSnapshotsResponse);

  /**
   * List snapshots. List snapshots. You can include the `instance_id` or
   * `project_id` in your query to get the list of snapshots for specific
   * Database Instances and/or Projects. By default, the details returned in the
   * list are ordered by creation date in ascending order, though this can be
   * modified via the `order_by` field.
   *
   * @param request - The request {@link ListSnapshotsRequest}
   * @returns A Promise of ListSnapshotsResponse
   */
  listSnapshots = (request = {}) => enrichForPagination('snapshots', this.pageOfListSnapshots, request);

  /**
   * Get a Database Instance snapshot. Retrieve information about a given
   * snapshot, specified by its `snapshot_id` and `region`. Full details about
   * the snapshot, like size and expiration date, are returned in the response.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  getSnapshot = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot);

  /**
   * Waits for {@link Snapshot} to be in a final state.
   *
   * @param request - The request {@link GetSnapshotRequest}
   * @param options - The waiting options
   * @returns A Promise of Snapshot
   */
  waitForSnapshot = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!SNAPSHOT_TRANSIENT_STATUSES.includes(res.status))), this.getSnapshot, request, options);

  /**
   * Create a Database Instance snapshot. Create a new snapshot of a Database
   * Instance. You must define the `name` parameter in the request.
   *
   * @param request - The request {@link CreateSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  createSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/snapshots`
  }, unmarshalSnapshot);

  /**
   * Update a Database Instance snapshot. Update the parameters of a snapshot of
   * a Database Instance. You can update the `name` and `expires_at`
   * parameters.
   *
   * @param request - The request {@link UpdateSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  updateSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'PATCH',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot);

  /**
   * Delete a Database Instance snapshot. Delete a given snapshot of a Database
   * Instance. You must specify, in the endpoint, the `region` and `snapshot_id`
   * parameters of the snapshot you want to delete.
   *
   * @param request - The request {@link DeleteSnapshotRequest}
   * @returns A Promise of Snapshot
   */
  deleteSnapshot = request => this.client.fetch({
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}`
  }, unmarshalSnapshot);

  /**
   * Create a new Database Instance from a snapshot. Restore a snapshot. When
   * you restore a snapshot, a new Instance is created and billed to your
   * account. Note that is possible to select a larger node type for your new
   * Database Instance. However, the Block volume size will be the same as the
   * size of the restored snapshot. All Instance settings will be restored if
   * you chose a node type with the same or more memory size than the initial
   * Instance. Settings will be reset to the default if your node type has less
   * memory.
   *
   * @param request - The request {@link CreateInstanceFromSnapshotRequest}
   * @returns A Promise of Instance
   */
  createInstanceFromSnapshot = request => this.client.fetch({
    body: JSON.stringify(marshalCreateInstanceFromSnapshotRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/snapshots/${validatePathParam('snapshotId', request.snapshotId)}/create-instance`
  }, unmarshalInstance);

  /**
   * Create a new Database Instance endpoint. Create a new endpoint for a
   * Database Instance. You can add `load_balancer` and `private_network`
   * specifications to the body of the request.
   *
   * @param request - The request {@link CreateEndpointRequest}
   * @returns A Promise of Endpoint
   */
  createEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalCreateEndpointRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/endpoints`
  }, unmarshalEndpoint$1);

  /**
   * Delete a Database Instance endpoint. Delete the endpoint of a Database
   * Instance. You must specify the `region` and `endpoint_id` parameters of the
   * endpoint you want to delete. Note that might need to update any environment
   * configurations that point to the deleted endpoint.
   *
   * @param request - The request {@link DeleteEndpointRequest}
   */
  deleteEndpoint = request => this.client.fetch({
    method: 'DELETE',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  });

  /**
   * Get a Database Instance endpoint. Retrieve information about a Database
   * Instance endpoint. Full details about the endpoint, like `ip`, `port`,
   * `private_network` and `load_balancer` specifications are returned in the
   * response.
   *
   * @param request - The request {@link GetEndpointRequest}
   * @returns A Promise of Endpoint
   */
  getEndpoint = request => this.client.fetch({
    method: 'GET',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  }, unmarshalEndpoint$1);

  /**
   * Migrate an existing instance endpoint to another instance.
   *
   * @param request - The request {@link MigrateEndpointRequest}
   * @returns A Promise of Endpoint
   */
  migrateEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalMigrateEndpointRequest(request, this.client.settings)),
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/endpoints/${validatePathParam('endpointId', request.endpointId)}/migrate`
  }, unmarshalEndpoint$1);

  /**
   * Apply Database Instance maintenance. Apply maintenance tasks to your
   * Database Instance. This will trigger pending maintenance tasks to start in
   * your Database Instance and can generate service interruption. Maintenance
   * tasks can be applied between `starts_at` and `stops_at` times, and are run
   * directly by Scaleway at `forced_at` timestamp.
   *
   * @param request - The request {@link ApplyInstanceMaintenanceRequest}
   * @returns A Promise of Maintenance
   */
  applyInstanceMaintenance = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$b,
    method: 'POST',
    path: `/rdb/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/instances/${validatePathParam('instanceId', request.instanceId)}/apply-maintenance`
  }, unmarshalMaintenance);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const UpdateInstanceRequest = {
  backupScheduleStartHour: {
    lessThanOrEqual: 23
  }
};

var validationRules_gen$3 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  UpdateInstanceRequest: UpdateInstanceRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$c = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$b,
  DATABASE_BACKUP_TRANSIENT_STATUSES: DATABASE_BACKUP_TRANSIENT_STATUSES,
  INSTANCE_LOG_TRANSIENT_STATUSES: INSTANCE_LOG_TRANSIENT_STATUSES,
  INSTANCE_TRANSIENT_STATUSES: INSTANCE_TRANSIENT_STATUSES,
  MAINTENANCE_TRANSIENT_STATUSES: MAINTENANCE_TRANSIENT_STATUSES,
  READ_REPLICA_TRANSIENT_STATUSES: READ_REPLICA_TRANSIENT_STATUSES,
  SNAPSHOT_TRANSIENT_STATUSES: SNAPSHOT_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen$3
});

var index$9 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$c
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link ClusterStatus}. */
const CLUSTER_TRANSIENT_STATUSES = ['provisioning', 'configuring', 'deleting', 'autohealing', 'initializing'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalACLRule = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ACLRule' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    id: data.id,
    ipCidr: data.ip_cidr
  };
};
const unmarshalPrivateNetwork$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrivateNetwork' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    provisioningMode: data.provisioning_mode,
    serviceIps: data.service_ips,
    zone: data.zone
  };
};
const unmarshalPublicNetwork = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PublicNetwork' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalEndpoint = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Endpoint' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    ips: data.ips,
    port: data.port,
    privateNetwork: data.private_network ? unmarshalPrivateNetwork$2(data.private_network) : undefined,
    publicNetwork: data.public_network ? unmarshalPublicNetwork(data.public_network) : undefined
  };
};
const unmarshalClusterSetting = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClusterSetting' failed as data isn't a dictionary.`);
  }
  return {
    name: data.name,
    value: data.value
  };
};
const unmarshalCluster = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Cluster' failed as data isn't a dictionary.`);
  }
  return {
    aclRules: unmarshalArrayOfObject(data.acl_rules, unmarshalACLRule),
    clusterSettings: unmarshalArrayOfObject(data.cluster_settings, unmarshalClusterSetting),
    clusterSize: data.cluster_size,
    createdAt: unmarshalDate(data.created_at),
    endpoints: unmarshalArrayOfObject(data.endpoints, unmarshalEndpoint),
    id: data.id,
    name: data.name,
    nodeType: data.node_type,
    projectId: data.project_id,
    status: data.status,
    tags: data.tags,
    tlsEnabled: data.tls_enabled,
    updatedAt: unmarshalDate(data.updated_at),
    upgradableVersions: data.upgradable_versions,
    userName: data.user_name,
    version: data.version,
    zone: data.zone
  };
};
const unmarshalAddAclRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AddAclRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    aclRules: unmarshalArrayOfObject(data.acl_rules, unmarshalACLRule),
    totalCount: data.total_count
  };
};
const unmarshalAddEndpointsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AddEndpointsResponse' failed as data isn't a dictionary.`);
  }
  return {
    endpoints: unmarshalArrayOfObject(data.endpoints, unmarshalEndpoint),
    totalCount: data.total_count
  };
};
const unmarshalClusterMetricsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClusterMetricsResponse' failed as data isn't a dictionary.`);
  }
  return {
    timeseries: unmarshalArrayOfObject(data.timeseries, unmarshalTimeSeries)
  };
};
const unmarshalClusterSettingsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClusterSettingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    settings: unmarshalArrayOfObject(data.settings, unmarshalClusterSetting)
  };
};
const unmarshalAvailableClusterSetting = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AvailableClusterSetting' failed as data isn't a dictionary.`);
  }
  return {
    defaultValue: data.default_value,
    deprecated: data.deprecated,
    description: data.description,
    maxValue: data.max_value,
    minValue: data.min_value,
    name: data.name,
    regex: data.regex,
    type: data.type
  };
};
const unmarshalClusterVersion = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ClusterVersion' failed as data isn't a dictionary.`);
  }
  return {
    availableSettings: unmarshalArrayOfObject(data.available_settings, unmarshalAvailableClusterSetting),
    endOfLifeAt: unmarshalDate(data.end_of_life_at),
    logoUrl: data.logo_url,
    version: data.version,
    zone: data.zone
  };
};
const unmarshalListClusterVersionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListClusterVersionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    versions: unmarshalArrayOfObject(data.versions, unmarshalClusterVersion)
  };
};
const unmarshalListClustersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListClustersResponse' failed as data isn't a dictionary.`);
  }
  return {
    clusters: unmarshalArrayOfObject(data.clusters, unmarshalCluster),
    totalCount: data.total_count
  };
};
const unmarshalNodeType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'NodeType' failed as data isn't a dictionary.`);
  }
  return {
    beta: data.beta,
    description: data.description,
    disabled: data.disabled,
    memory: data.memory,
    name: data.name,
    stockStatus: data.stock_status,
    vcpus: data.vcpus,
    zone: data.zone
  };
};
const unmarshalListNodeTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNodeTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    nodeTypes: unmarshalArrayOfObject(data.node_types, unmarshalNodeType),
    totalCount: data.total_count
  };
};
const unmarshalSetAclRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetAclRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    aclRules: unmarshalArrayOfObject(data.acl_rules, unmarshalACLRule)
  };
};
const unmarshalSetEndpointsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetEndpointsResponse' failed as data isn't a dictionary.`);
  }
  return {
    endpoints: unmarshalArrayOfObject(data.endpoints, unmarshalEndpoint)
  };
};
const marshalACLRuleSpec = (request, defaults) => ({
  description: request.description,
  ip_cidr: request.ipCidr
});
const marshalAddAclRulesRequest = (request, defaults) => ({
  acl_rules: request.aclRules.map(elt => marshalACLRuleSpec(elt))
});
const marshalClusterSetting = (request, defaults) => ({
  name: request.name,
  value: request.value
});
const marshalAddClusterSettingsRequest = (request, defaults) => ({
  settings: request.settings.map(elt => marshalClusterSetting(elt))
});
const marshalEndpointSpecPrivateNetworkSpecIpamConfig = (request, defaults) => ({});
const marshalEndpointSpecPrivateNetworkSpec = (request, defaults) => ({
  id: request.id,
  ipam_config: request.ipamConfig !== undefined ? marshalEndpointSpecPrivateNetworkSpecIpamConfig(request.ipamConfig) : undefined,
  service_ips: request.serviceIps
});
const marshalEndpointSpecPublicNetworkSpec = (request, defaults) => ({});
const marshalEndpointSpec = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'private_network',
    value: request.privateNetwork !== undefined ? marshalEndpointSpecPrivateNetworkSpec(request.privateNetwork) : undefined
  }, {
    param: 'public_network',
    value: request.publicNetwork !== undefined ? marshalEndpointSpecPublicNetworkSpec(request.publicNetwork) : undefined
  }])
});
const marshalAddEndpointsRequest = (request, defaults) => ({
  endpoints: request.endpoints.map(elt => marshalEndpointSpec(elt))
});
const marshalCreateClusterRequest = (request, defaults) => ({
  acl_rules: request.aclRules !== undefined ? request.aclRules.map(elt => marshalACLRuleSpec(elt)) : undefined,
  cluster_settings: request.clusterSettings !== undefined ? request.clusterSettings.map(elt => marshalClusterSetting(elt)) : undefined,
  cluster_size: request.clusterSize,
  endpoints: request.endpoints !== undefined ? request.endpoints.map(elt => marshalEndpointSpec(elt)) : undefined,
  name: request.name || randomName('ins'),
  node_type: request.nodeType,
  password: request.password,
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags,
  tls_enabled: request.tlsEnabled,
  user_name: request.userName,
  version: request.version
});
const marshalMigrateClusterRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'version',
    value: request.version
  }, {
    param: 'node_type',
    value: request.nodeType
  }, {
    param: 'cluster_size',
    value: request.clusterSize
  }])
});
const marshalSetAclRulesRequest = (request, defaults) => ({
  acl_rules: request.aclRules.map(elt => marshalACLRuleSpec(elt))
});
const marshalSetClusterSettingsRequest = (request, defaults) => ({
  settings: request.settings.map(elt => marshalClusterSetting(elt))
});
const marshalSetEndpointsRequest = (request, defaults) => ({
  endpoints: request.endpoints.map(elt => marshalEndpointSpec(elt))
});
const marshalUpdateClusterRequest = (request, defaults) => ({
  name: request.name,
  password: request.password,
  tags: request.tags,
  user_name: request.userName
});
const marshalUpdateEndpointRequest = (request, defaults) => ({
  ...resolveOneOf([{
    param: 'private_network',
    value: request.privateNetwork !== undefined ? marshalEndpointSpecPrivateNetworkSpec(request.privateNetwork) : undefined
  }, {
    param: 'public_network',
    value: request.publicNetwork !== undefined ? marshalEndpointSpecPublicNetworkSpec(request.publicNetwork) : undefined
  }])
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$a = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Managed Database for Redis™ API. */
let API$a = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'nl-ams-1', 'nl-ams-2', 'pl-waw-1', 'pl-waw-2'];

  /**
   * Create a Redis™ Database Instance. Create a new Redis™ Database Instance
   * (Redis™ cluster). You must set the `zone`, `project_id`, `version`,
   * `node_type`, `user_name` and `password` parameters. Optionally you can
   * define `acl_rules`, `endpoints`, `tls_enabled` and `cluster_settings`.
   *
   * @param request - The request {@link CreateClusterRequest}
   * @returns A Promise of Cluster
   */
  createCluster = request => this.client.fetch({
    body: JSON.stringify(marshalCreateClusterRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'POST',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters`
  }, unmarshalCluster);

  /**
   * Update a Redis™ Database Instance. Update the parameters of a Redis™
   * Database Instance (Redis™ cluster), including `name`, `tags`, `user_name`
   * and `password`.
   *
   * @param request - The request {@link UpdateClusterRequest}
   * @returns A Promise of Cluster
   */
  updateCluster = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateClusterRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'PATCH',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}`
  }, unmarshalCluster);

  /**
   * Get a Redis™ Database Instance. Retrieve information about a Redis™
   * Database Instance (Redis™ cluster). Specify the `cluster_id` and `region`
   * in your request to get information such as `id`, `status`, `version`,
   * `tls_enabled`, `cluster_settings`, `upgradable_versions` and `endpoints`
   * about your cluster in the response.
   *
   * @param request - The request {@link GetClusterRequest}
   * @returns A Promise of Cluster
   */
  getCluster = request => this.client.fetch({
    method: 'GET',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}`
  }, unmarshalCluster);

  /**
   * Waits for {@link Cluster} to be in a final state.
   *
   * @param request - The request {@link GetClusterRequest}
   * @param options - The waiting options
   * @returns A Promise of Cluster
   */
  waitForCluster = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!CLUSTER_TRANSIENT_STATUSES.includes(res.status))), this.getCluster, request, options);
  pageOfListClusters = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['tags', request.tags], ['version', request.version])
  }, unmarshalListClustersResponse);

  /**
   * List Redis™ Database Instances. List all Redis™ Database Instances (Redis™
   * cluster) in the specified zone. By default, the Database Instances returned
   * in the list are ordered by creation date in ascending order, though this
   * can be modified via the order_by field. You can define additional
   * parameters for your query, such as `tags`, `name`, `organization_id` and
   * `version`.
   *
   * @param request - The request {@link ListClustersRequest}
   * @returns A Promise of ListClustersResponse
   */
  listClusters = (request = {}) => enrichForPagination('clusters', this.pageOfListClusters, request);

  /**
   * Scale up a Redis™ Database Instance. Upgrade your Redis™ Database Instance,
   * either by upgrading to a bigger node type (vertical scaling) or by adding
   * more nodes to your Database Instance to increase your number of endpoints
   * and distribute cache (horizontal scaling, available for clusters only).
   * Note that scaling horizontally your Redis™ Database Instance will not renew
   * its TLS certificate. In order to refresh the TLS certificate, you must use
   * the Renew TLS certificate endpoint.
   *
   * @param request - The request {@link MigrateClusterRequest}
   * @returns A Promise of Cluster
   */
  migrateCluster = request => this.client.fetch({
    body: JSON.stringify(marshalMigrateClusterRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'POST',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/migrate`
  }, unmarshalCluster);

  /**
   * Delete a Redis™ Database Instance. Delete a Redis™ Database Instance
   * (Redis™ cluster), specified by the `region` and `cluster_id` parameters.
   * Deleting a Database Instance is permanent, and cannot be undone. Note that
   * upon deletion all your data will be lost.
   *
   * @param request - The request {@link DeleteClusterRequest}
   * @returns A Promise of Cluster
   */
  deleteCluster = request => this.client.fetch({
    method: 'DELETE',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}`
  }, unmarshalCluster);

  /**
   * Get metrics of a Redis™ Database Instance. Retrieve the metrics of a Redis™
   * Database Instance (Redis™ cluster). You can define the period from which to
   * retrieve metrics by specifying the `start_date` and `end_date`.
   *
   * @param request - The request {@link GetClusterMetricsRequest}
   * @returns A Promise of ClusterMetricsResponse
   */
  getClusterMetrics = request => this.client.fetch({
    method: 'GET',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/metrics`,
    urlParams: urlParams(['end_at', request.endAt], ['metric_name', request.metricName], ['start_at', request.startAt])
  }, unmarshalClusterMetricsResponse);
  pageOfListNodeTypes = request => this.client.fetch({
    method: 'GET',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/node-types`,
    urlParams: urlParams(['include_disabled_types', request.includeDisabledTypes], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListNodeTypesResponse);

  /**
   * List available node types. List all available node types. By default, the
   * node types returned in the list are ordered by creation date in ascending
   * order, though this can be modified via the `order_by` field.
   *
   * @param request - The request {@link ListNodeTypesRequest}
   * @returns A Promise of ListNodeTypesResponse
   */
  listNodeTypes = request => enrichForPagination('nodeTypes', this.pageOfListNodeTypes, request);
  pageOfListClusterVersions = request => this.client.fetch({
    method: 'GET',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/cluster-versions`,
    urlParams: urlParams(['include_beta', request.includeBeta], ['include_deprecated', request.includeDeprecated], ['include_disabled', request.includeDisabled], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['version', request.version])
  }, unmarshalListClusterVersionsResponse);

  /**
   * List available Redis™ versions. List the Redis™ database engine versions
   * available. You can define additional parameters for your query, such as
   * `include_disabled`, `include_beta`, `include_deprecated` and `version`.
   *
   * @param request - The request {@link ListClusterVersionsRequest}
   * @returns A Promise of ListClusterVersionsResponse
   */
  listClusterVersions = request => enrichForPagination('versions', this.pageOfListClusterVersions, request);

  /**
   * Get the TLS certificate of a cluster. Retrieve information about the TLS
   * certificate of a Redis™ Database Instance (Redis™ cluster). Details like
   * name and content are returned in the response.
   *
   * @param request - The request {@link GetClusterCertificateRequest}
   * @returns A Promise of Blob
   */
  getClusterCertificate = request => this.client.fetch({
    method: 'GET',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/certificate`,
    urlParams: urlParams(['dl', 1]),
    responseType: 'blob'
  });

  /**
   * Renew the TLS certificate of a cluster. Renew a TLS certificate for a
   * Redis™ Database Instance (Redis™ cluster). Renewing a certificate means
   * that you will not be able to connect to your Database Instance using the
   * previous certificate. You will also need to download and update the new
   * certificate for all database clients.
   *
   * @param request - The request {@link RenewClusterCertificateRequest}
   * @returns A Promise of Cluster
   */
  renewClusterCertificate = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$a,
    method: 'POST',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/renew-certificate`
  }, unmarshalCluster);

  /**
   * Add advanced settings. Add an advanced setting to a Redis™ Database
   * Instance (Redis™ cluster). You must set the `name` and the `value` of each
   * setting.
   *
   * @param request - The request {@link AddClusterSettingsRequest}
   * @returns A Promise of ClusterSettingsResponse
   */
  addClusterSettings = request => this.client.fetch({
    body: JSON.stringify(marshalAddClusterSettingsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'POST',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/settings`
  }, unmarshalClusterSettingsResponse);

  /**
   * Delete advanced setting. Delete an advanced setting in a Redis™ Database
   * Instance (Redis™ cluster). You must specify the names of the settings you
   * want to delete in the request body.
   *
   * @param request - The request {@link DeleteClusterSettingRequest}
   * @returns A Promise of Cluster
   */
  deleteClusterSetting = request => this.client.fetch({
    method: 'DELETE',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/settings/${validatePathParam('settingName', request.settingName)}`
  }, unmarshalCluster);

  /**
   * Set advanced settings. Update an advanced setting for a Redis™ Database
   * Instance (Redis™ cluster). Settings added upon database engine
   * initalization can only be defined once, and cannot, therefore, be updated.
   *
   * @param request - The request {@link SetClusterSettingsRequest}
   * @returns A Promise of ClusterSettingsResponse
   */
  setClusterSettings = request => this.client.fetch({
    body: JSON.stringify(marshalSetClusterSettingsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'PUT',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/settings`
  }, unmarshalClusterSettingsResponse);

  /**
   * Set ACL rules for a cluster. Replace all the ACL rules of a Redis™ Database
   * Instance (Redis™ cluster).
   *
   * @param request - The request {@link SetAclRulesRequest}
   * @returns A Promise of SetAclRulesResponse
   */
  setAclRules = request => this.client.fetch({
    body: JSON.stringify(marshalSetAclRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'PUT',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/acls`
  }, unmarshalSetAclRulesResponse);

  /**
   * Add ACL rules for a cluster. Add an additional ACL rule to a Redis™
   * Database Instance (Redis™ cluster).
   *
   * @param request - The request {@link AddAclRulesRequest}
   * @returns A Promise of AddAclRulesResponse
   */
  addAclRules = request => this.client.fetch({
    body: JSON.stringify(marshalAddAclRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'POST',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/acls`
  }, unmarshalAddAclRulesResponse);

  /**
   * Delete an ACL rule for a cluster. Delete an ACL rule of a Redis™ Database
   * Instance (Redis™ cluster). You must specify the `acl_id` of the rule you
   * want to delete in your request.
   *
   * @param request - The request {@link DeleteAclRuleRequest}
   * @returns A Promise of Cluster
   */
  deleteAclRule = request => this.client.fetch({
    method: 'DELETE',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/acls/${validatePathParam('aclId', request.aclId)}`
  }, unmarshalCluster);

  /**
   * Get an ACL rule. Retrieve information about an ACL rule of a Redis™
   * Database Instance (Redis™ cluster). You must specify the `acl_id` of the
   * rule in your request.
   *
   * @param request - The request {@link GetAclRuleRequest}
   * @returns A Promise of ACLRule
   */
  getAclRule = request => this.client.fetch({
    method: 'GET',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/acls/${validatePathParam('aclId', request.aclId)}`
  }, unmarshalACLRule);

  /**
   * Set endpoints for a cluster. Update an endpoint for a Redis™ Database
   * Instance (Redis™ cluster). You must specify the `cluster_id` and the
   * `endpoints` parameters in your request.
   *
   * @param request - The request {@link SetEndpointsRequest}
   * @returns A Promise of SetEndpointsResponse
   */
  setEndpoints = request => this.client.fetch({
    body: JSON.stringify(marshalSetEndpointsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'PUT',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/endpoints`
  }, unmarshalSetEndpointsResponse);

  /**
   * Add endpoints for a cluster. Add a new endpoint for a Redis™ Database
   * Instance (Redis™ cluster). You can add `private_network` or
   * `public_network` specifications to the body of the request.
   *
   * @param request - The request {@link AddEndpointsRequest}
   * @returns A Promise of AddEndpointsResponse
   */
  addEndpoints = request => this.client.fetch({
    body: JSON.stringify(marshalAddEndpointsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'POST',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/clusters/${validatePathParam('clusterId', request.clusterId)}/endpoints`
  }, unmarshalAddEndpointsResponse);

  /**
   * Delete an endpoint for a cluster. Delete the endpoint of a Redis™ Database
   * Instance (Redis™ cluster). You must specify the `region` and `endpoint_id`
   * parameters of the endpoint you want to delete. Note that might need to
   * update any environment configurations that point to the deleted endpoint.
   *
   * @param request - The request {@link DeleteEndpointRequest}
   * @returns A Promise of Cluster
   */
  deleteEndpoint = request => this.client.fetch({
    method: 'DELETE',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  }, unmarshalCluster);

  /**
   * Get an endpoint. Retrieve information about a Redis™ Database Instance
   * (Redis™ cluster) endpoint. Full details about the endpoint, like `ips`,
   * `port`, `private_network` and `public_network` specifications are returned
   * in the response.
   *
   * @param request - The request {@link GetEndpointRequest}
   * @returns A Promise of Endpoint
   */
  getEndpoint = request => this.client.fetch({
    method: 'GET',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  }, unmarshalEndpoint);

  /**
   * Update an endpoint. Update information about a Redis™ Database Instance
   * (Redis™ cluster) endpoint. Full details about the endpoint, like `ips`,
   * `port`, `private_network` and `public_network` specifications are returned
   * in the response.
   *
   * @param request - The request {@link UpdateEndpointRequest}
   * @returns A Promise of Endpoint
   */
  updateEndpoint = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateEndpointRequest(request, this.client.settings)),
    headers: jsonContentHeaders$a,
    method: 'PATCH',
    path: `/redis/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/endpoints/${validatePathParam('endpointId', request.endpointId)}`
  }, unmarshalEndpoint);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$b = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$a,
  CLUSTER_TRANSIENT_STATUSES: CLUSTER_TRANSIENT_STATUSES
});

var index$8 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$b
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link ImageStatus}. */
const IMAGE_TRANSIENT_STATUSES = ['deleting'];

/** Lists transient statutes of the enum {@link NamespaceStatus}. */
const NAMESPACE_TRANSIENT_STATUSES = ['deleting'];

/** Lists transient statutes of the enum {@link TagStatus}. */
const TAG_TRANSIENT_STATUSES = ['deleting'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalImage = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Image' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    namespaceId: data.namespace_id,
    size: data.size,
    status: data.status,
    statusMessage: data.status_message,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    visibility: data.visibility
  };
};
const unmarshalNamespace = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Namespace' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    endpoint: data.endpoint,
    id: data.id,
    imageCount: data.image_count,
    isPublic: data.is_public,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    region: data.region,
    size: data.size,
    status: data.status,
    statusMessage: data.status_message,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalTag = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Tag' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    digest: data.digest,
    id: data.id,
    imageId: data.image_id,
    name: data.name,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListImagesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListImagesResponse' failed as data isn't a dictionary.`);
  }
  return {
    images: unmarshalArrayOfObject(data.images, unmarshalImage),
    totalCount: data.total_count
  };
};
const unmarshalListNamespacesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListNamespacesResponse' failed as data isn't a dictionary.`);
  }
  return {
    namespaces: unmarshalArrayOfObject(data.namespaces, unmarshalNamespace),
    totalCount: data.total_count
  };
};
const unmarshalListTagsResponse$2 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTagsResponse' failed as data isn't a dictionary.`);
  }
  return {
    tags: unmarshalArrayOfObject(data.tags, unmarshalTag),
    totalCount: data.total_count
  };
};
const marshalCreateNamespaceRequest = (request, defaults) => ({
  description: request.description,
  is_public: request.isPublic,
  name: request.name || randomName('ns'),
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalUpdateImageRequest = (request, defaults) => ({
  visibility: request.visibility
});
const marshalUpdateNamespaceRequest = (request, defaults) => ({
  description: request.description,
  is_public: request.isPublic
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$9 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Container Registry API. */
let API$9 = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];
  pageOfListNamespaces = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListNamespacesResponse);

  /**
   * List namespaces. List all namespaces in a specified region. By default, the
   * namespaces listed are ordered by creation date in ascending order. This can
   * be modified via the order_by field. You can also define additional
   * parameters for your query, such as the `instance_id` and `project_id`
   * parameters.
   *
   * @param request - The request {@link ListNamespacesRequest}
   * @returns A Promise of ListNamespacesResponse
   */
  listNamespaces = (request = {}) => enrichForPagination('namespaces', this.pageOfListNamespaces, request);

  /**
   * Get a namespace. Retrieve information about a given namespace, specified by
   * its `namespace_id` and region. Full details about the namespace, such as
   * `description`, `project_id`, `status`, `endpoint`, `is_public`, `size`, and
   * `image_count` are returned in the response.
   *
   * @param request - The request {@link GetNamespaceRequest}
   * @returns A Promise of Namespace
   */
  getNamespace = request => this.client.fetch({
    method: 'GET',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace);

  /**
   * Waits for {@link Namespace} to be in a final state.
   *
   * @param request - The request {@link GetNamespaceRequest}
   * @param options - The waiting options
   * @returns A Promise of Namespace
   */
  waitForNamespace = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!NAMESPACE_TRANSIENT_STATUSES.includes(res.status))), this.getNamespace, request, options);

  /**
   * Create a namespace. Create a new Container Registry namespace. You must
   * specify the namespace name and region in which you want it to be created.
   * Optionally, you can specify the `project_id` and `is_public` in the request
   * payload.
   *
   * @param request - The request {@link CreateNamespaceRequest}
   * @returns A Promise of Namespace
   */
  createNamespace = request => this.client.fetch({
    body: JSON.stringify(marshalCreateNamespaceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$9,
    method: 'POST',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces`
  }, unmarshalNamespace);

  /**
   * Update a namespace. Update the parameters of a given namespace, specified
   * by its `namespace_id` and `region`. You can update the `description` and
   * `is_public` parameters.
   *
   * @param request - The request {@link UpdateNamespaceRequest}
   * @returns A Promise of Namespace
   */
  updateNamespace = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateNamespaceRequest(request, this.client.settings)),
    headers: jsonContentHeaders$9,
    method: 'PATCH',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace);

  /**
   * Delete a namespace. Delete a given namespace. You must specify, in the
   * endpoint, the `region` and `namespace_id` parameters of the namespace you
   * want to delete.
   *
   * @param request - The request {@link DeleteNamespaceRequest}
   * @returns A Promise of Namespace
   */
  deleteNamespace = request => this.client.fetch({
    method: 'DELETE',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/namespaces/${validatePathParam('namespaceId', request.namespaceId)}`
  }, unmarshalNamespace);
  pageOfListImages = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/images`,
    urlParams: urlParams(['name', request.name], ['namespace_id', request.namespaceId], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListImagesResponse);

  /**
   * List images. List all images in a specified region. By default, the images
   * listed are ordered by creation date in ascending order. This can be
   * modified via the order_by field. You can also define additional parameters
   * for your query, such as the `namespace_id` and `project_id` parameters.
   *
   * @param request - The request {@link ListImagesRequest}
   * @returns A Promise of ListImagesResponse
   */
  listImages = (request = {}) => enrichForPagination('images', this.pageOfListImages, request);

  /**
   * Get an image. Retrieve information about a given container image, specified
   * by its `image_id` and region. Full details about the image, such as `name`,
   * `namespace_id`, `status`, `visibility`, and `size` are returned in the
   * response.
   *
   * @param request - The request {@link GetImageRequest}
   * @returns A Promise of Image
   */
  getImage = request => this.client.fetch({
    method: 'GET',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/images/${validatePathParam('imageId', request.imageId)}`
  }, unmarshalImage);

  /**
   * Waits for {@link Image} to be in a final state.
   *
   * @param request - The request {@link GetImageRequest}
   * @param options - The waiting options
   * @returns A Promise of Image
   */
  waitForImage = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!IMAGE_TRANSIENT_STATUSES.includes(res.status))), this.getImage, request, options);

  /**
   * Update an image. Update the parameters of a given image, specified by its
   * `image_id` and `region`. You can update the `visibility` parameter.
   *
   * @param request - The request {@link UpdateImageRequest}
   * @returns A Promise of Image
   */
  updateImage = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateImageRequest(request, this.client.settings)),
    headers: jsonContentHeaders$9,
    method: 'PATCH',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/images/${validatePathParam('imageId', request.imageId)}`
  }, unmarshalImage);

  /**
   * Delete an image. Delete a given image. You must specify, in the endpoint,
   * the `region` and `image_id` parameters of the image you want to delete.
   *
   * @param request - The request {@link DeleteImageRequest}
   * @returns A Promise of Image
   */
  deleteImage = request => this.client.fetch({
    method: 'DELETE',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/images/${validatePathParam('imageId', request.imageId)}`
  }, unmarshalImage);
  pageOfListTags = request => this.client.fetch({
    method: 'GET',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/images/${validatePathParam('imageId', request.imageId)}/tags`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListTagsResponse$2);

  /**
   * List tags. List all tags for a given image, specified by region. By
   * default, the tags listed are ordered by creation date in ascending order.
   * This can be modified via the order_by field. You can also define additional
   * parameters for your query, such as the `name`.
   *
   * @param request - The request {@link ListTagsRequest}
   * @returns A Promise of ListTagsResponse
   */
  listTags = request => enrichForPagination('tags', this.pageOfListTags, request);

  /**
   * Get a tag. Retrieve information about a given image tag, specified by its
   * `tag_id` and region. Full details about the tag, such as `name`,
   * `image_id`, `status`, and `digest` are returned in the response.
   *
   * @param request - The request {@link GetTagRequest}
   * @returns A Promise of Tag
   */
  getTag = request => this.client.fetch({
    method: 'GET',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tags/${validatePathParam('tagId', request.tagId)}`
  }, unmarshalTag);

  /**
   * Waits for {@link Tag} to be in a final state.
   *
   * @param request - The request {@link GetTagRequest}
   * @param options - The waiting options
   * @returns A Promise of Tag
   */
  waitForTag = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!TAG_TRANSIENT_STATUSES.includes(res.status))), this.getTag, request, options);

  /**
   * Delete a tag. Delete a given image tag. You must specify, in the endpoint,
   * the `region` and `tag_id` parameters of the tag you want to delete.
   *
   * @param request - The request {@link DeleteTagRequest}
   * @returns A Promise of Tag
   */
  deleteTag = request => this.client.fetch({
    method: 'DELETE',
    path: `/registry/v1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tags/${validatePathParam('tagId', request.tagId)}`,
    urlParams: urlParams(['force', request.force])
  }, unmarshalTag);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$a = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$9,
  IMAGE_TRANSIENT_STATUSES: IMAGE_TRANSIENT_STATUSES,
  NAMESPACE_TRANSIENT_STATUSES: NAMESPACE_TRANSIENT_STATUSES,
  TAG_TRANSIENT_STATUSES: TAG_TRANSIENT_STATUSES
});

var index$7 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$a
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalFolder = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Folder' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    path: data.path,
    projectId: data.project_id,
    region: data.region
  };
};
const unmarshalEphemeralProperties$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EphemeralProperties' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    expiresAt: unmarshalDate(data.expires_at),
    expiresOnceAccessed: data.expires_once_accessed
  };
};
const unmarshalSecretVersion$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SecretVersion' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    ephemeralProperties: data.ephemeral_properties ? unmarshalEphemeralProperties$1(data.ephemeral_properties) : undefined,
    isLatest: data.is_latest,
    revision: data.revision,
    secretId: data.secret_id,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalEphemeralPolicy$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EphemeralPolicy' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    expiresOnceAccessed: data.expires_once_accessed,
    timeToLive: data.time_to_live
  };
};
const unmarshalSecret$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Secret' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    ephemeralPolicy: data.ephemeral_policy ? unmarshalEphemeralPolicy$1(data.ephemeral_policy) : undefined,
    id: data.id,
    isManaged: data.is_managed,
    isProtected: data.is_protected,
    name: data.name,
    path: data.path,
    projectId: data.project_id,
    region: data.region,
    status: data.status,
    tags: data.tags,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    versionCount: data.version_count
  };
};
const unmarshalAccessSecretVersionResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AccessSecretVersionResponse' failed as data isn't a dictionary.`);
  }
  return {
    data: data.data,
    dataCrc32: data.data_crc32,
    revision: data.revision,
    secretId: data.secret_id
  };
};
const unmarshalListFoldersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListFoldersResponse' failed as data isn't a dictionary.`);
  }
  return {
    folders: unmarshalArrayOfObject(data.folders, unmarshalFolder),
    totalCount: data.total_count
  };
};
const unmarshalListSecretVersionsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSecretVersionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    versions: unmarshalArrayOfObject(data.versions, unmarshalSecretVersion$1)
  };
};
const unmarshalListSecretsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSecretsResponse' failed as data isn't a dictionary.`);
  }
  return {
    secrets: unmarshalArrayOfObject(data.secrets, unmarshalSecret$1),
    totalCount: data.total_count
  };
};
const unmarshalListTagsResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTagsResponse' failed as data isn't a dictionary.`);
  }
  return {
    tags: data.tags,
    totalCount: data.total_count
  };
};
const marshalAddSecretOwnerRequest$1 = (request, defaults) => ({
  product: request.product,
  product_name: request.productName
});
const marshalCreateFolderRequest = (request, defaults) => ({
  name: request.name,
  path: request.path,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalEphemeralPolicy$1 = (request, defaults) => ({
  action: request.action,
  expires_once_accessed: request.expiresOnceAccessed,
  time_to_live: request.timeToLive
});
const marshalCreateSecretRequest$1 = (request, defaults) => ({
  description: request.description,
  ephemeral_policy: request.ephemeralPolicy !== undefined ? marshalEphemeralPolicy$1(request.ephemeralPolicy) : undefined,
  is_protected: request.isProtected,
  name: request.name,
  path: request.path,
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags,
  type: request.type
});
const marshalPasswordGenerationParams = (request, defaults) => ({
  additional_chars: request.additionalChars,
  length: request.length,
  no_digits: request.noDigits,
  no_lowercase_letters: request.noLowercaseLetters,
  no_uppercase_letters: request.noUppercaseLetters
});
const marshalCreateSecretVersionRequest$1 = (request, defaults) => ({
  data: request.data,
  data_crc32: request.dataCrc32,
  description: request.description,
  disable_previous: request.disablePrevious,
  password_generation: request.passwordGeneration !== undefined ? marshalPasswordGenerationParams(request.passwordGeneration) : undefined
});
const marshalGeneratePasswordRequest = (request, defaults) => ({
  additional_chars: request.additionalChars,
  description: request.description,
  disable_previous: request.disablePrevious,
  length: request.length,
  no_digits: request.noDigits,
  no_lowercase_letters: request.noLowercaseLetters,
  no_uppercase_letters: request.noUppercaseLetters
});
const marshalUpdateSecretRequest$1 = (request, defaults) => ({
  description: request.description,
  ephemeral_policy: request.ephemeralPolicy !== undefined ? marshalEphemeralPolicy$1(request.ephemeralPolicy) : undefined,
  name: request.name,
  path: request.path,
  tags: request.tags
});
const marshalEphemeralProperties$1 = (request, defaults) => ({
  action: request.action,
  expires_at: request.expiresAt,
  expires_once_accessed: request.expiresOnceAccessed
});
const marshalUpdateSecretVersionRequest$1 = (request, defaults) => ({
  description: request.description,
  ephemeral_properties: request.ephemeralProperties !== undefined ? marshalEphemeralProperties$1(request.ephemeralProperties) : undefined
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$8 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Secret Manager API.
 *
 * Secret Manager API. This API allows you to conveniently store, access and
 * share sensitive data.
 */
let API$8 = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];

  /**
   * Create a secret. You must specify the `region` to create a secret.
   *
   * @param request - The request {@link CreateSecretRequest}
   * @returns A Promise of Secret
   */
  createSecret = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSecretRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets`
  }, unmarshalSecret$1);

  /**
   * Create folder.
   *
   * @param request - The request {@link CreateFolderRequest}
   * @returns A Promise of Folder
   */
  createFolder = request => this.client.fetch({
    body: JSON.stringify(marshalCreateFolderRequest(request, this.client.settings)),
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/folders`
  }, unmarshalFolder);

  /**
   * Get metadata using the secret's ID. Retrieve the metadata of a secret
   * specified by the `region` and `secret_id` parameters.
   *
   * @param request - The request {@link GetSecretRequest}
   * @returns A Promise of Secret
   */
  getSecret = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}`
  }, unmarshalSecret$1);

  /**
   * Get metadata using the secret's name. Retrieve the metadata of a secret
   * specified by the `region` and `secret_name` parameters.
   *
   * GetSecretByName usage is now deprecated.
   *
   * Scaleway recommends that you use the `ListSecrets` request with the `name`
   * filter.
   *
   * @deprecated
   * @param request - The request {@link GetSecretByNameRequest}
   * @returns A Promise of Secret
   */
  getSecretByName = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets-by-name/${validatePathParam('secretName', request.secretName)}`,
    urlParams: urlParams(['project_id', request.projectId])
  }, unmarshalSecret$1);

  /**
   * Update metadata of a secret. Edit a secret's metadata such as name, tag(s),
   * description and ephemeral policy. The secret to update is specified by the
   * `secret_id` and `region` parameters.
   *
   * @param request - The request {@link UpdateSecretRequest}
   * @returns A Promise of Secret
   */
  updateSecret = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSecretRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$8,
    method: 'PATCH',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}`
  }, unmarshalSecret$1);
  pageOfListSecrets = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets`,
    urlParams: urlParams(['is_ephemeral', request.isEphemeral], ['is_managed', request.isManaged], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['path', request.path], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalListSecretsResponse$1);

  /**
   * List secrets. Retrieve the list of secrets created within an Organization
   * and/or Project. You must specify either the `organization_id` or the
   * `project_id` and the `region`.
   *
   * @param request - The request {@link ListSecretsRequest}
   * @returns A Promise of ListSecretsResponse
   */
  listSecrets = (request = {}) => enrichForPagination('secrets', this.pageOfListSecrets, request);
  pageOfListFolders = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/folders`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['path', request.path], ['project_id', request.projectId])
  }, unmarshalListFoldersResponse);

  /**
   * List folders. Retrieve the list of folders created within a Project.
   *
   * @param request - The request {@link ListFoldersRequest}
   * @returns A Promise of ListFoldersResponse
   */
  listFolders = (request = {}) => enrichForPagination('folders', this.pageOfListFolders, request);

  /**
   * Delete a secret. Delete a given secret specified by the `region` and
   * `secret_id` parameters.
   *
   * @param request - The request {@link DeleteSecretRequest}
   */
  deleteSecret = request => this.client.fetch({
    method: 'DELETE',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}`
  });

  /**
   * Delete a given folder specified by the `region` and `folder_id` parameters.
   *
   * @param request - The request {@link DeleteFolderRequest}
   */
  deleteFolder = request => this.client.fetch({
    method: 'DELETE',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/folders/${validatePathParam('folderId', request.folderId)}`
  });

  /**
   * Protect a secret. Protect a given secret specified by the `secret_id`
   * parameter. A protected secret can be read and modified but cannot be
   * deleted.
   *
   * @param request - The request {@link ProtectSecretRequest}
   * @returns A Promise of Secret
   */
  protectSecret = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/protect`
  }, unmarshalSecret$1);

  /**
   * Unprotect a secret. Unprotect a given secret specified by the `secret_id`
   * parameter. An unprotected secret can be read, modified and deleted.
   *
   * @param request - The request {@link UnprotectSecretRequest}
   * @returns A Promise of Secret
   */
  unprotectSecret = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/unprotect`
  }, unmarshalSecret$1);

  /**
   * Allow a product to use the secret.
   *
   * @param request - The request {@link AddSecretOwnerRequest}
   */
  addSecretOwner = request => this.client.fetch({
    body: JSON.stringify(marshalAddSecretOwnerRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/add-owner`
  });

  /**
   * Create a version. Create a version of a given secret specified by the
   * `region` and `secret_id` parameters.
   *
   * @param request - The request {@link CreateSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  createSecretVersion = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSecretVersionRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions`
  }, unmarshalSecretVersion$1);

  /**
   * Generate a password in a new version. Generate a password for the given
   * secret specified by the `region` and `secret_id` parameters. This will also
   * create a new version of the secret that will store the password.
   *
   * @param request - The request {@link GeneratePasswordRequest}
   * @returns A Promise of SecretVersion
   */
  generatePassword = request => this.client.fetch({
    body: JSON.stringify(marshalGeneratePasswordRequest(request, this.client.settings)),
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/generate-password`
  }, unmarshalSecretVersion$1);

  /**
   * Get metadata of a secret's version using the secret's ID. Retrieve the
   * metadata of a secret's given version specified by the `region`, `secret_id`
   * and `revision` parameters.
   *
   * @param request - The request {@link GetSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  getSecretVersion = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}`
  }, unmarshalSecretVersion$1);

  /**
   * Get metadata of a secret's version using the secret's name. Retrieve the
   * metadata of a secret's given version specified by the `region`,
   * `secret_name`, `revision` and `project_id` parameters.
   *
   * This method is deprecated.
   *
   * Scaleway recommends that you use the `ListSecrets` request with the `name`
   * filter to specify the secret version desired, then use the
   * `GetSecretVersion` request.
   *
   * @deprecated
   * @param request - The request {@link GetSecretVersionByNameRequest}
   * @returns A Promise of SecretVersion
   */
  getSecretVersionByName = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets-by-name/${validatePathParam('secretName', request.secretName)}/versions/${validatePathParam('revision', request.revision)}`,
    urlParams: urlParams(['project_id', request.projectId])
  }, unmarshalSecretVersion$1);

  /**
   * Update metadata of a version. Edit the metadata of a secret's given
   * version, specified by the `region`, `secret_id` and `revision` parameters.
   *
   * @param request - The request {@link UpdateSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  updateSecretVersion = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSecretVersionRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$8,
    method: 'PATCH',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}`
  }, unmarshalSecretVersion$1);
  pageOfListSecretVersions = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['status', request.status])
  }, unmarshalListSecretVersionsResponse$1);

  /**
   * List versions of a secret using the secret's ID. Retrieve the list of a
   * given secret's versions specified by the `secret_id` and `region`
   * parameters.
   *
   * @param request - The request {@link ListSecretVersionsRequest}
   * @returns A Promise of ListSecretVersionsResponse
   */
  listSecretVersions = request => enrichForPagination('versions', this.pageOfListSecretVersions, request);
  pageOfListSecretVersionsByName = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets-by-name/${validatePathParam('secretName', request.secretName)}/versions`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['status', request.status])
  }, unmarshalListSecretVersionsResponse$1);

  /**
   * List versions of a secret using the secret's name. Retrieve the list of a
   * given secret's versions specified by the `secret_name`,`region` and
   * `project_id` parameters.
   *
   * This method is deprecated.
   *
   * Scaleway recommends that you use the `ListSecrets` request with the `name`
   * filter to specify the secret version desired, then use the
   * `ListSecretVersions` request.
   *
   * @deprecated
   * @param request - The request {@link ListSecretVersionsByNameRequest}
   * @returns A Promise of ListSecretVersionsResponse
   */
  listSecretVersionsByName = request => enrichForPagination('versions', this.pageOfListSecretVersionsByName, request);

  /**
   * Enable a version. Make a specific version accessible. You must specify the
   * `region`, `secret_id` and `revision` parameters.
   *
   * @param request - The request {@link EnableSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  enableSecretVersion = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}/enable`
  }, unmarshalSecretVersion$1);

  /**
   * Disable a version. Make a specific version inaccessible. You must specify
   * the `region`, `secret_id` and `revision` parameters.
   *
   * @param request - The request {@link DisableSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  disableSecretVersion = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}/disable`
  }, unmarshalSecretVersion$1);

  /**
   * Access a secret's version using the secret's ID. Access sensitive data in a
   * secret's version specified by the `region`, `secret_id` and `revision`
   * parameters.
   *
   * @param request - The request {@link AccessSecretVersionRequest}
   * @returns A Promise of AccessSecretVersionResponse
   */
  accessSecretVersion = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}/access`
  }, unmarshalAccessSecretVersionResponse$1);

  /**
   * Access a secret's version using the secret's name. Access sensitive data in
   * a secret's version specified by the `region`, `secret_name`, `revision` and
   * `project_id` parameters.
   *
   * This method is deprecated.
   *
   * Scaleway recommends that you use the `ListSecrets` request with the `name`
   * filter to specify the secret version desired, then use the
   * `AccessSecretVersion` request.
   *
   * @deprecated
   * @param request - The request {@link AccessSecretVersionByNameRequest}
   * @returns A Promise of AccessSecretVersionResponse
   */
  accessSecretVersionByName = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets-by-name/${validatePathParam('secretName', request.secretName)}/versions/${validatePathParam('revision', request.revision)}/access`,
    urlParams: urlParams(['project_id', request.projectId])
  }, unmarshalAccessSecretVersionResponse$1);

  /**
   * Delete a version. Delete a secret's version and the sensitive data
   * contained in it. Deleting a version is permanent and cannot be undone.
   *
   * @param request - The request {@link DestroySecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  destroySecretVersion = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$8,
    method: 'POST',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}/destroy`
  }, unmarshalSecretVersion$1);
  pageOfListTags = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tags`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListTagsResponse$1);

  /**
   * List tags. List all tags associated with secrets within a given Project.
   *
   * @param request - The request {@link ListTagsRequest}
   * @returns A Promise of ListTagsResponse
   */
  listTags = (request = {}) => enrichForPagination('tags', this.pageOfListTags, request);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$9 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$8
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalEphemeralProperties = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EphemeralProperties' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    expiresAt: unmarshalDate(data.expires_at),
    expiresOnceAccessed: data.expires_once_accessed
  };
};
const unmarshalSecretVersion = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SecretVersion' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    ephemeralProperties: data.ephemeral_properties ? unmarshalEphemeralProperties(data.ephemeral_properties) : undefined,
    latest: data.latest,
    revision: data.revision,
    secretId: data.secret_id,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalEphemeralPolicy = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EphemeralPolicy' failed as data isn't a dictionary.`);
  }
  return {
    action: data.action,
    expiresOnceAccessed: data.expires_once_accessed,
    timeToLive: data.time_to_live
  };
};
const unmarshalSecret = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Secret' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    description: data.description,
    ephemeralPolicy: data.ephemeral_policy ? unmarshalEphemeralPolicy(data.ephemeral_policy) : undefined,
    id: data.id,
    managed: data.managed,
    name: data.name,
    path: data.path,
    projectId: data.project_id,
    protected: data.protected,
    region: data.region,
    status: data.status,
    tags: data.tags,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    versionCount: data.version_count
  };
};
const unmarshalAccessSecretVersionResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AccessSecretVersionResponse' failed as data isn't a dictionary.`);
  }
  return {
    data: data.data,
    dataCrc32: data.data_crc32,
    revision: data.revision,
    secretId: data.secret_id,
    type: data.type
  };
};
const unmarshalBrowseSecretsResponseItemFolderDetails = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BrowseSecretsResponseItemFolderDetails' failed as data isn't a dictionary.`);
  }
  return {};
};
const unmarshalBrowseSecretsResponseItemSecretDetails = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BrowseSecretsResponseItemSecretDetails' failed as data isn't a dictionary.`);
  }
  return {
    ephemeralPolicy: data.ephemeral_policy ? unmarshalEphemeralPolicy(data.ephemeral_policy) : undefined,
    id: data.id,
    protected: data.protected,
    tags: data.tags,
    versionCount: data.version_count
  };
};
const unmarshalBrowseSecretsResponseItem = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BrowseSecretsResponseItem' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    folder: data.folder ? unmarshalBrowseSecretsResponseItemFolderDetails(data.folder) : undefined,
    name: data.name,
    secret: data.secret ? unmarshalBrowseSecretsResponseItemSecretDetails(data.secret) : undefined,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalBrowseSecretsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'BrowseSecretsResponse' failed as data isn't a dictionary.`);
  }
  return {
    currentPath: data.current_path,
    items: unmarshalArrayOfObject(data.items, unmarshalBrowseSecretsResponseItem),
    totalCount: data.total_count
  };
};
const unmarshalListSecretVersionsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSecretVersionsResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    versions: unmarshalArrayOfObject(data.versions, unmarshalSecretVersion)
  };
};
const unmarshalListSecretsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListSecretsResponse' failed as data isn't a dictionary.`);
  }
  return {
    secrets: unmarshalArrayOfObject(data.secrets, unmarshalSecret),
    totalCount: data.total_count
  };
};
const unmarshalListTagsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListTagsResponse' failed as data isn't a dictionary.`);
  }
  return {
    tags: data.tags,
    totalCount: data.total_count
  };
};
const marshalAddSecretOwnerRequest = (request, defaults) => ({
  product: request.product
});
const marshalEphemeralPolicy = (request, defaults) => ({
  action: request.action,
  expires_once_accessed: request.expiresOnceAccessed,
  time_to_live: request.timeToLive
});
const marshalCreateSecretRequest = (request, defaults) => ({
  description: request.description,
  ephemeral_policy: request.ephemeralPolicy !== undefined ? marshalEphemeralPolicy(request.ephemeralPolicy) : undefined,
  name: request.name,
  path: request.path,
  project_id: request.projectId ?? defaults.defaultProjectId,
  protected: request.protected,
  tags: request.tags,
  type: request.type
});
const marshalCreateSecretVersionRequest = (request, defaults) => ({
  data: request.data,
  data_crc32: request.dataCrc32,
  description: request.description,
  disable_previous: request.disablePrevious
});
const marshalUpdateSecretRequest = (request, defaults) => ({
  description: request.description,
  ephemeral_policy: request.ephemeralPolicy !== undefined ? marshalEphemeralPolicy(request.ephemeralPolicy) : undefined,
  name: request.name,
  path: request.path,
  tags: request.tags
});
const marshalEphemeralProperties = (request, defaults) => ({
  action: request.action,
  expires_at: request.expiresAt,
  expires_once_accessed: request.expiresOnceAccessed
});
const marshalUpdateSecretVersionRequest = (request, defaults) => ({
  description: request.description,
  ephemeral_properties: request.ephemeralProperties !== undefined ? marshalEphemeralProperties(request.ephemeralProperties) : undefined
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$7 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Secret Manager API.
 *
 * This API allows you to conveniently store, access and share sensitive data
 * such as passwords, API keys and certificates.
 */
let API$7 = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];

  /**
   * Create a secret. Create a secret in a given region specified by the
   * `region` parameter.
   *
   * @param request - The request {@link CreateSecretRequest}
   * @returns A Promise of Secret
   */
  createSecret = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSecretRequest(request, this.client.settings)),
    headers: jsonContentHeaders$7,
    method: 'POST',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets`
  }, unmarshalSecret);

  /**
   * Get metadata using the secret's ID. Retrieve the metadata of a secret
   * specified by the `region` and `secret_id` parameters.
   *
   * @param request - The request {@link GetSecretRequest}
   * @returns A Promise of Secret
   */
  getSecret = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}`
  }, unmarshalSecret);

  /**
   * Update metadata of a secret. Edit a secret's metadata such as name, tag(s),
   * description and ephemeral policy. The secret to update is specified by the
   * `secret_id` and `region` parameters.
   *
   * @param request - The request {@link UpdateSecretRequest}
   * @returns A Promise of Secret
   */
  updateSecret = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSecretRequest(request, this.client.settings)),
    headers: jsonContentHeaders$7,
    method: 'PATCH',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}`
  }, unmarshalSecret);

  /**
   * Delete a secret. Delete a given secret specified by the `region` and
   * `secret_id` parameters.
   *
   * @param request - The request {@link DeleteSecretRequest}
   */
  deleteSecret = request => this.client.fetch({
    method: 'DELETE',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}`
  });
  pageOfListSecrets = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets`,
    urlParams: urlParams(['ephemeral', request.ephemeral], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['path', request.path], ['project_id', request.projectId], ['tags', request.tags], ['type', request.type])
  }, unmarshalListSecretsResponse);

  /**
   * List secrets. Retrieve the list of secrets created within an Organization
   * and/or Project. You must specify either the `organization_id` or the
   * `project_id` and the `region`.
   *
   * @param request - The request {@link ListSecretsRequest}
   * @returns A Promise of ListSecretsResponse
   */
  listSecrets = (request = {}) => enrichForPagination('secrets', this.pageOfListSecrets, request);

  /**
   * Browse secrets. Retrieve the list of secrets and folders for the given
   * prefix. You must specify either the `organization_id` or the `project_id`
   * and the `region`.
   *
   * @param request - The request {@link BrowseSecretsRequest}
   * @returns A Promise of BrowseSecretsResponse
   */
  browseSecrets = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/browse`,
    urlParams: urlParams(['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['prefix', request.prefix], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalBrowseSecretsResponse);

  /**
   * Enable secret protection. Enable secret protection for a given secret
   * specified by the `secret_id` parameter. Enabling secret protection means
   * that your secret can be read and modified, but it cannot be deleted.
   *
   * @param request - The request {@link ProtectSecretRequest}
   * @returns A Promise of Secret
   */
  protectSecret = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$7,
    method: 'POST',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/protect`
  }, unmarshalSecret);

  /**
   * Disable secret protection. Disable secret protection for a given secret
   * specified by the `secret_id` parameter. Disabling secret protection means
   * that your secret can be read, modified and deleted.
   *
   * @param request - The request {@link UnprotectSecretRequest}
   * @returns A Promise of Secret
   */
  unprotectSecret = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$7,
    method: 'POST',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/unprotect`
  }, unmarshalSecret);

  /**
   * Allow a product to use the secret.
   *
   * @param request - The request {@link AddSecretOwnerRequest}
   */
  addSecretOwner = request => this.client.fetch({
    body: JSON.stringify(marshalAddSecretOwnerRequest(request, this.client.settings)),
    headers: jsonContentHeaders$7,
    method: 'POST',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/add-owner`
  });

  /**
   * Create a version. Create a version of a given secret specified by the
   * `region` and `secret_id` parameters.
   *
   * @param request - The request {@link CreateSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  createSecretVersion = request => this.client.fetch({
    body: JSON.stringify(marshalCreateSecretVersionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$7,
    method: 'POST',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions`
  }, unmarshalSecretVersion);

  /**
   * Get metadata of a secret's version using the secret's ID. Retrieve the
   * metadata of a secret's given version specified by the `region`, `secret_id`
   * and `revision` parameters.
   *
   * @param request - The request {@link GetSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  getSecretVersion = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}`
  }, unmarshalSecretVersion);

  /**
   * Update metadata of a version. Edit the metadata of a secret's given
   * version, specified by the `region`, `secret_id` and `revision` parameters.
   *
   * @param request - The request {@link UpdateSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  updateSecretVersion = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateSecretVersionRequest(request, this.client.settings)),
    headers: jsonContentHeaders$7,
    method: 'PATCH',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}`
  }, unmarshalSecretVersion);

  /**
   * Delete a version. Delete a secret's version and the sensitive data
   * contained in it. Deleting a version is permanent and cannot be undone.
   *
   * @param request - The request {@link DeleteSecretVersionRequest}
   */
  deleteSecretVersion = request => this.client.fetch({
    method: 'DELETE',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}`
  });
  pageOfListSecretVersions = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['status', request.status])
  }, unmarshalListSecretVersionsResponse);

  /**
   * List versions of a secret using the secret's ID. Retrieve the list of a
   * given secret's versions specified by the `secret_id` and `region`
   * parameters.
   *
   * @param request - The request {@link ListSecretVersionsRequest}
   * @returns A Promise of ListSecretVersionsResponse
   */
  listSecretVersions = request => enrichForPagination('versions', this.pageOfListSecretVersions, request);

  /**
   * Access a secret's version using the secret's ID. Access sensitive data in a
   * secret's version specified by the `region`, `secret_id` and `revision`
   * parameters.
   *
   * @param request - The request {@link AccessSecretVersionRequest}
   * @returns A Promise of AccessSecretVersionResponse
   */
  accessSecretVersion = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}/access`
  }, unmarshalAccessSecretVersionResponse);

  /**
   * Access a secret's version using the secret's name and path. Access
   * sensitive data in a secret's version specified by the `region`,
   * `secret_name`, `secret_path` and `revision` parameters.
   *
   * @param request - The request {@link AccessSecretVersionByPathRequest}
   * @returns A Promise of AccessSecretVersionResponse
   */
  accessSecretVersionByPath = request => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets-by-path/versions/${validatePathParam('revision', request.revision)}/access`,
    urlParams: urlParams(['project_id', request.projectId ?? this.client.settings.defaultProjectId], ['secret_name', request.secretName], ['secret_path', request.secretPath])
  }, unmarshalAccessSecretVersionResponse);

  /**
   * Enable a version. Make a specific version accessible. You must specify the
   * `region`, `secret_id` and `revision` parameters.
   *
   * @param request - The request {@link EnableSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  enableSecretVersion = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$7,
    method: 'POST',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}/enable`
  }, unmarshalSecretVersion);

  /**
   * Disable a version. Make a specific version inaccessible. You must specify
   * the `region`, `secret_id` and `revision` parameters.
   *
   * @param request - The request {@link DisableSecretVersionRequest}
   * @returns A Promise of SecretVersion
   */
  disableSecretVersion = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$7,
    method: 'POST',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/secrets/${validatePathParam('secretId', request.secretId)}/versions/${validatePathParam('revision', request.revision)}/disable`
  }, unmarshalSecretVersion);
  pageOfListTags = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/secret-manager/v1beta1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/tags`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListTagsResponse);

  /**
   * List tags. List all tags associated with secrets within a given Project.
   *
   * @param request - The request {@link ListTagsRequest}
   * @returns A Promise of ListTagsResponse
   */
  listTags = (request = {}) => enrichForPagination('tags', this.pageOfListTags, request);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const DatabaseCredentials = {
  port: {
    pattern: /^[0-9]+$/
  }
};

var validationRules_gen$2 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  DatabaseCredentials: DatabaseCredentials
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$8 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$7,
  ValidationRules: validationRules_gen$2
});

var index$6 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$9,
  v1beta1: index_gen$8
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link DatabaseStatus}. */
const DATABASE_TRANSIENT_STATUSES = ['creating', 'deleting', 'restoring'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalDatabaseBackup = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DatabaseBackup' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    databaseId: data.database_id,
    downloadUrl: data.download_url,
    downloadUrlExpiresAt: unmarshalDate(data.download_url_expires_at),
    expiresAt: unmarshalDate(data.expires_at),
    id: data.id,
    organizationId: data.organization_id,
    projectId: data.project_id,
    region: data.region,
    size: data.size,
    status: data.status
  };
};
const unmarshalDatabase = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Database' failed as data isn't a dictionary.`);
  }
  return {
    cpuCurrent: data.cpu_current,
    cpuMax: data.cpu_max,
    cpuMin: data.cpu_min,
    createdAt: unmarshalDate(data.created_at),
    endpoint: data.endpoint,
    engineMajorVersion: data.engine_major_version,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    region: data.region,
    started: data.started,
    status: data.status
  };
};
const unmarshalListDatabaseBackupsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDatabaseBackupsResponse' failed as data isn't a dictionary.`);
  }
  return {
    backups: unmarshalArrayOfObject(data.backups, unmarshalDatabaseBackup),
    totalCount: data.total_count
  };
};
const unmarshalListDatabasesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDatabasesResponse' failed as data isn't a dictionary.`);
  }
  return {
    databases: unmarshalArrayOfObject(data.databases, unmarshalDatabase),
    totalCount: data.total_count
  };
};
const marshalCreateDatabaseRequest = (request, defaults) => ({
  cpu_max: request.cpuMax,
  cpu_min: request.cpuMin,
  from_backup_id: request.fromBackupId,
  name: request.name,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalRestoreDatabaseFromBackupRequest = (request, defaults) => ({
  backup_id: request.backupId
});
const marshalUpdateDatabaseRequest = (request, defaults) => ({
  cpu_max: request.cpuMax,
  cpu_min: request.cpuMin
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$6 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Serverless SQL Databases API.
 *
 * This API allows you to manage your Serverless SQL DB databases.
 */
let API$6 = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par'];

  /**
   * Create a new Serverless SQL Database. You must provide the following
   * parameters: `organization_id`, `project_id`, `name`, `cpu_min`, `cpu_max`.
   * You can also provide `from_backup_id` to create a database from a backup.
   *
   * @param request - The request {@link CreateDatabaseRequest}
   * @returns A Promise of Database
   */
  createDatabase = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDatabaseRequest(request, this.client.settings)),
    headers: jsonContentHeaders$6,
    method: 'POST',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/databases`
  }, unmarshalDatabase);

  /**
   * Get a database information. Retrieve information about your Serverless SQL
   * Database. You must provide the `database_id` parameter.
   *
   * @param request - The request {@link GetDatabaseRequest}
   * @returns A Promise of Database
   */
  getDatabase = request => this.client.fetch({
    method: 'GET',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/databases/${validatePathParam('databaseId', request.databaseId)}`
  }, unmarshalDatabase);

  /**
   * Waits for {@link Database} to be in a final state.
   *
   * @param request - The request {@link GetDatabaseRequest}
   * @param options - The waiting options
   * @returns A Promise of Database
   */
  waitForDatabase = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!DATABASE_TRANSIENT_STATUSES.includes(res.status))), this.getDatabase, request, options);

  /**
   * Delete a database. Deletes a database. You must provide the `database_id`
   * parameter. All data stored in the database will be permanently deleted.
   *
   * @param request - The request {@link DeleteDatabaseRequest}
   * @returns A Promise of Database
   */
  deleteDatabase = request => this.client.fetch({
    method: 'DELETE',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/databases/${validatePathParam('databaseId', request.databaseId)}`
  }, unmarshalDatabase);
  pageOfListDatabases = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/databases`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize], ['project_id', request.projectId ?? this.client.settings.defaultProjectId])
  }, unmarshalListDatabasesResponse);

  /**
   * List your Serverless SQL Databases. List all Serverless SQL Databases for a
   * given Scaleway Organization or Scaleway Project. By default, the databases
   * returned in the list are ordered by creation date in ascending order,
   * though this can be modified via the order_by field. For the `name`
   * parameter, the value you include will be checked against the whole name
   * string to see if it includes the string you put in the parameter.
   *
   * @param request - The request {@link ListDatabasesRequest}
   * @returns A Promise of ListDatabasesResponse
   */
  listDatabases = (request = {}) => enrichForPagination('databases', this.pageOfListDatabases, request);

  /**
   * Update database information. Update CPU limits of your Serverless SQL
   * Database. You must provide the `database_id` parameter.
   *
   * @param request - The request {@link UpdateDatabaseRequest}
   * @returns A Promise of Database
   */
  updateDatabase = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDatabaseRequest(request, this.client.settings)),
    headers: jsonContentHeaders$6,
    method: 'PATCH',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/databases/${validatePathParam('databaseId', request.databaseId)}`
  }, unmarshalDatabase);

  /**
   * Restore a database from a backup. Restore a database from a backup. You
   * must provide the `backup_id` parameter.
   *
   * @param request - The request {@link RestoreDatabaseFromBackupRequest}
   * @returns A Promise of Database
   */
  restoreDatabaseFromBackup = request => this.client.fetch({
    body: JSON.stringify(marshalRestoreDatabaseFromBackupRequest(request, this.client.settings)),
    headers: jsonContentHeaders$6,
    method: 'POST',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/databases/${validatePathParam('databaseId', request.databaseId)}/restore`
  }, unmarshalDatabase);

  /**
   * Get a database backup information. Retrieve information about your
   * Serverless SQL Database backup. You must provide the `backup_id`
   * parameter.
   *
   * @param request - The request {@link GetDatabaseBackupRequest}
   * @returns A Promise of DatabaseBackup
   */
  getDatabaseBackup = request => this.client.fetch({
    method: 'GET',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups/${validatePathParam('backupId', request.backupId)}`
  }, unmarshalDatabaseBackup);
  pageOfListDatabaseBackups = request => this.client.fetch({
    method: 'GET',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups`,
    urlParams: urlParams(['database_id', request.databaseId], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize], ['project_id', request.projectId])
  }, unmarshalListDatabaseBackupsResponse);

  /**
   * List your Serverless SQL Database backups. List all Serverless SQL Database
   * backups for a given Scaleway Project or Database. By default, the backups
   * returned in the list are ordered by creation date in descending order,
   * though this can be modified via the order_by field.
   *
   * @param request - The request {@link ListDatabaseBackupsRequest}
   * @returns A Promise of ListDatabaseBackupsResponse
   */
  listDatabaseBackups = request => enrichForPagination('backups', this.pageOfListDatabaseBackups, request);

  /**
   * Export a database backup. Export a database backup providing a download
   * link once the export process is completed. You must provide the `backup_id`
   * parameter.
   *
   * @param request - The request {@link ExportDatabaseBackupRequest}
   * @returns A Promise of DatabaseBackup
   */
  exportDatabaseBackup = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$6,
    method: 'POST',
    path: `/serverless-sqldb/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/backups/${validatePathParam('backupId', request.backupId)}/export`
  }, unmarshalDatabaseBackup);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const CreateDatabaseRequest = {
  cpuMax: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 15
  },
  cpuMin: {
    lessThanOrEqual: 15
  },
  fromBackupId: {
    ignoreEmpty: true
  },
  name: {
    maxLength: 63,
    minLength: 1
  }
};
const Database = {
  cpuCurrent: {
    lessThanOrEqual: 15
  },
  cpuMax: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 15
  },
  cpuMin: {
    lessThanOrEqual: 15
  }
};
const ListDatabaseBackupsRequest = {
  page: {
    greaterThanOrEqual: 1
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 1000
  }
};
const ListDatabasesRequest = {
  page: {
    greaterThanOrEqual: 1
  },
  pageSize: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 1000
  }
};
const UpdateDatabaseRequest = {
  cpuMax: {
    greaterThanOrEqual: 1,
    lessThanOrEqual: 15
  },
  cpuMin: {
    lessThanOrEqual: 15
  }
};

var validationRules_gen$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  CreateDatabaseRequest: CreateDatabaseRequest,
  Database: Database,
  ListDatabaseBackupsRequest: ListDatabaseBackupsRequest,
  ListDatabasesRequest: ListDatabasesRequest,
  UpdateDatabaseRequest: UpdateDatabaseRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$7 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$6,
  DATABASE_TRANSIENT_STATUSES: DATABASE_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen$1
});

var index$5 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$7
});

var index_gen$6 = /*#__PURE__*/Object.freeze({
  __proto__: null
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link HumanStatus}. */
const HUMAN_TRANSIENT_STATUSES = ['running'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalHuman = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Human' failed as data isn't a dictionary.`);
  }
  return {
    altitudeInMeter: data.altitude_in_meter,
    altitudeInMillimeter: data.altitude_in_millimeter,
    createdAt: unmarshalDate(data.created_at),
    eyesColor: data.eyes_color,
    fingersCount: data.fingers_count,
    hairCount: data.hair_count,
    height: data.height,
    id: data.id,
    isHappy: data.is_happy,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    shoeSize: data.shoe_size,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalListHumansResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListHumansResponse' failed as data isn't a dictionary.`);
  }
  return {
    humans: unmarshalArrayOfObject(data.humans, unmarshalHuman),
    totalCount: data.total_count
  };
};
const unmarshalRegisterResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'RegisterResponse' failed as data isn't a dictionary.`);
  }
  return {
    accessKey: data.access_key,
    secretKey: data.secret_key
  };
};
const marshalCreateHumanRequest = (request, defaults) => ({
  altitude_in_meter: request.altitudeInMeter,
  altitude_in_millimeter: request.altitudeInMillimeter,
  eyes_color: request.eyesColor,
  fingers_count: request.fingersCount,
  hair_count: request.hairCount,
  height: request.height,
  is_happy: request.isHappy,
  name: request.name,
  shoe_size: request.shoeSize,
  ...resolveOneOf([{
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }, {
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }])
});
const marshalRegisterRequest = (request, defaults) => ({
  username: request.username
});
const marshalUpdateHumanRequest = (request, defaults) => ({
  altitude_in_meter: request.altitudeInMeter,
  altitude_in_millimeter: request.altitudeInMillimeter,
  eyes_color: request.eyesColor,
  fingers_count: request.fingersCount,
  hair_count: request.hairCount,
  height: request.height,
  is_happy: request.isHappy,
  name: request.name,
  shoe_size: request.shoeSize
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$5 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/**
 * Fake API.
 *
 * No Auth Service for end-to-end testing.
 */
let API$5 = class API extends API$z {
  /**
   * Register a user. Register a human and return a access-key and a secret-key
   * that must be used in all other commands.
   *
   * Hint: you can use other test commands by setting the SCW_SECRET_KEY env
   * variable.
   *
   * @param request - The request {@link RegisterRequest}
   * @returns A Promise of RegisterResponse
   */
  register = request => this.client.fetch({
    body: JSON.stringify(marshalRegisterRequest(request, this.client.settings)),
    headers: jsonContentHeaders$5,
    method: 'POST',
    path: `/test/v1/register`
  }, unmarshalRegisterResponse);
  pageOfListHumans = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/test/v1/humans`,
    urlParams: urlParams(['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListHumansResponse);

  /**
   * List all your humans.
   *
   * @param request - The request {@link ListHumansRequest}
   * @returns A Promise of ListHumansResponse
   */
  listHumans = (request = {}) => enrichForPagination('humans', this.pageOfListHumans, request);

  /**
   * Get human details. Get the human details associated with the given id.
   *
   * @param request - The request {@link GetHumanRequest}
   * @returns A Promise of Human
   */
  getHuman = request => this.client.fetch({
    method: 'GET',
    path: `/test/v1/humans/${validatePathParam('humanId', request.humanId)}`
  }, unmarshalHuman);

  /**
   * Waits for {@link Human} to be in a final state.
   *
   * @param request - The request {@link GetHumanRequest}
   * @param options - The waiting options
   * @returns A Promise of Human
   */
  waitForHuman = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!HUMAN_TRANSIENT_STATUSES.includes(res.status))), this.getHuman, request, options);

  /**
   * Create a new human.
   *
   * @param request - The request {@link CreateHumanRequest}
   * @returns A Promise of Human
   */
  createHuman = request => this.client.fetch({
    body: JSON.stringify(marshalCreateHumanRequest(request, this.client.settings)),
    headers: jsonContentHeaders$5,
    method: 'POST',
    path: `/test/v1/humans`
  }, unmarshalHuman);

  /**
   * Update an existing human. Update the human associated with the given id.
   *
   * @param request - The request {@link UpdateHumanRequest}
   * @returns A Promise of Human
   */
  updateHuman = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateHumanRequest(request, this.client.settings)),
    headers: jsonContentHeaders$5,
    method: 'PATCH',
    path: `/test/v1/humans/${validatePathParam('humanId', request.humanId)}`
  }, unmarshalHuman);

  /**
   * Delete an existing human. Delete the human associated with the given id.
   *
   * @param request - The request {@link DeleteHumanRequest}
   * @returns A Promise of Human
   */
  deleteHuman = request => this.client.fetch({
    method: 'DELETE',
    path: `/test/v1/humans/${validatePathParam('humanId', request.humanId)}`
  }, unmarshalHuman);

  /**
   * Start a 1h running for the given human. Start a one hour running for the
   * given human.
   *
   * @param request - The request {@link RunHumanRequest}
   * @returns A Promise of Human
   */
  runHuman = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$5,
    method: 'POST',
    path: `/test/v1/humans/${validatePathParam('humanId', request.humanId)}/run`
  }, unmarshalHuman);

  /**
   * Make a human smoke.
   *
   * @deprecated
   * @param request - The request {@link SmokeHumanRequest}
   * @returns A Promise of Human
   */
  smokeHuman = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$5,
    method: 'POST',
    path: `/test/v1/humans/${validatePathParam('humanId', request.humanId)}/smoke`
  }, unmarshalHuman);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$5 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$5,
  HUMAN_TRANSIENT_STATUSES: HUMAN_TRANSIENT_STATUSES
});

var index$4 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$5
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link DomainStatus}. */
const DOMAIN_TRANSIENT_STATUSES = ['pending'];

/** Lists transient statutes of the enum {@link EmailStatus}. */
const EMAIL_TRANSIENT_STATUSES = ['new', 'sending'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalEmailTry = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'EmailTry' failed as data isn't a dictionary.`);
  }
  return {
    code: data.code,
    message: data.message,
    rank: data.rank,
    triedAt: unmarshalDate(data.tried_at)
  };
};
const unmarshalEmail = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Email' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    flags: data.flags,
    id: data.id,
    lastTries: unmarshalArrayOfObject(data.last_tries, unmarshalEmailTry),
    mailFrom: data.mail_from,
    mailRcpt: data.mail_rcpt,
    messageId: data.message_id,
    projectId: data.project_id,
    rcptTo: data.rcpt_to,
    rcptType: data.rcpt_type,
    status: data.status,
    statusDetails: data.status_details,
    subject: data.subject,
    tryCount: data.try_count,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalDomainRecordsDMARC = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecordsDMARC' failed as data isn't a dictionary.`);
  }
  return {
    name: data.name,
    value: data.value
  };
};
const unmarshalDomainRecords = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainRecords' failed as data isn't a dictionary.`);
  }
  return {
    dmarc: data.dmarc ? unmarshalDomainRecordsDMARC(data.dmarc) : undefined
  };
};
const unmarshalDomainReputation = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainReputation' failed as data isn't a dictionary.`);
  }
  return {
    previousScore: data.previous_score,
    previousScoredAt: unmarshalDate(data.previous_scored_at),
    score: data.score,
    scoredAt: unmarshalDate(data.scored_at),
    status: data.status
  };
};
const unmarshalDomainStatistics = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainStatistics' failed as data isn't a dictionary.`);
  }
  return {
    canceledCount: data.canceled_count,
    failedCount: data.failed_count,
    sentCount: data.sent_count,
    totalCount: data.total_count
  };
};
const unmarshalDomain = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Domain' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    dkimConfig: data.dkim_config,
    id: data.id,
    lastError: data.last_error,
    lastValidAt: unmarshalDate(data.last_valid_at),
    name: data.name,
    nextCheckAt: unmarshalDate(data.next_check_at),
    organizationId: data.organization_id,
    projectId: data.project_id,
    records: data.records ? unmarshalDomainRecords(data.records) : undefined,
    region: data.region,
    reputation: data.reputation ? unmarshalDomainReputation(data.reputation) : undefined,
    revokedAt: unmarshalDate(data.revoked_at),
    spfConfig: data.spf_config,
    statistics: data.statistics ? unmarshalDomainStatistics(data.statistics) : undefined,
    status: data.status
  };
};
const unmarshalCreateEmailResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'CreateEmailResponse' failed as data isn't a dictionary.`);
  }
  return {
    emails: unmarshalArrayOfObject(data.emails, unmarshalEmail)
  };
};
const unmarshalDomainLastStatusDkimRecord = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainLastStatusDkimRecord' failed as data isn't a dictionary.`);
  }
  return {
    error: data.error,
    lastValidAt: unmarshalDate(data.last_valid_at),
    status: data.status
  };
};
const unmarshalDomainLastStatusDmarcRecord = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainLastStatusDmarcRecord' failed as data isn't a dictionary.`);
  }
  return {
    error: data.error,
    lastValidAt: unmarshalDate(data.last_valid_at),
    status: data.status
  };
};
const unmarshalDomainLastStatusSpfRecord = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainLastStatusSpfRecord' failed as data isn't a dictionary.`);
  }
  return {
    error: data.error,
    lastValidAt: unmarshalDate(data.last_valid_at),
    status: data.status
  };
};
const unmarshalDomainLastStatus = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DomainLastStatus' failed as data isn't a dictionary.`);
  }
  return {
    dkimRecord: data.dkim_record ? unmarshalDomainLastStatusDkimRecord(data.dkim_record) : undefined,
    dmarcRecord: data.dmarc_record ? unmarshalDomainLastStatusDmarcRecord(data.dmarc_record) : undefined,
    domainId: data.domain_id,
    domainName: data.domain_name,
    spfRecord: data.spf_record ? unmarshalDomainLastStatusSpfRecord(data.spf_record) : undefined
  };
};
const unmarshalListDomainsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary.`);
  }
  return {
    domains: unmarshalArrayOfObject(data.domains, unmarshalDomain),
    totalCount: data.total_count
  };
};
const unmarshalListEmailsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListEmailsResponse' failed as data isn't a dictionary.`);
  }
  return {
    emails: unmarshalArrayOfObject(data.emails, unmarshalEmail),
    totalCount: data.total_count
  };
};
const unmarshalStatistics = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Statistics' failed as data isn't a dictionary.`);
  }
  return {
    canceledCount: data.canceled_count,
    failedCount: data.failed_count,
    newCount: data.new_count,
    sendingCount: data.sending_count,
    sentCount: data.sent_count,
    totalCount: data.total_count
  };
};
const marshalCreateDomainRequest = (request, defaults) => ({
  accept_tos: request.acceptTos,
  domain_name: request.domainName,
  project_id: request.projectId ?? defaults.defaultProjectId
});
const marshalCreateEmailRequestAddress = (request, defaults) => ({
  email: request.email,
  name: request.name
});
const marshalCreateEmailRequestAttachment = (request, defaults) => ({
  content: request.content,
  name: request.name,
  type: request.type
});
const marshalCreateEmailRequestHeader = (request, defaults) => ({
  key: request.key,
  value: request.value
});
const marshalCreateEmailRequest = (request, defaults) => ({
  additional_headers: request.additionalHeaders !== undefined ? request.additionalHeaders.map(elt => marshalCreateEmailRequestHeader(elt)) : undefined,
  attachments: request.attachments !== undefined ? request.attachments.map(elt => marshalCreateEmailRequestAttachment(elt)) : undefined,
  bcc: request.bcc !== undefined ? request.bcc.map(elt => marshalCreateEmailRequestAddress(elt)) : undefined,
  cc: request.cc !== undefined ? request.cc.map(elt => marshalCreateEmailRequestAddress(elt)) : undefined,
  from: request.from !== undefined ? marshalCreateEmailRequestAddress(request.from) : undefined,
  html: request.html,
  project_id: request.projectId ?? defaults.defaultProjectId,
  send_before: request.sendBefore,
  subject: request.subject,
  text: request.text,
  to: request.to !== undefined ? request.to.map(elt => marshalCreateEmailRequestAddress(elt)) : undefined
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$4 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Transactional Email API. */
let API$4 = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par'];

  /**
   * Send an email. You must specify the `region`, the sender and the
   * recipient's information and the `project_id` to send an email from a
   * checked domain. The subject of the email must contain at least 6
   * characters.
   *
   * @param request - The request {@link CreateEmailRequest}
   * @returns A Promise of CreateEmailResponse
   */
  createEmail = request => this.client.fetch({
    body: JSON.stringify(marshalCreateEmailRequest(request, this.client.settings)),
    headers: jsonContentHeaders$4,
    method: 'POST',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/emails`
  }, unmarshalCreateEmailResponse);

  /**
   * Get an email. Retrieve information about a specific email using the
   * `email_id` and `region` parameters.
   *
   * @param request - The request {@link GetEmailRequest}
   * @returns A Promise of Email
   */
  getEmail = request => this.client.fetch({
    method: 'GET',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/emails/${validatePathParam('emailId', request.emailId)}`
  }, unmarshalEmail);

  /**
   * Waits for {@link Email} to be in a final state.
   *
   * @param request - The request {@link GetEmailRequest}
   * @param options - The waiting options
   * @returns A Promise of Email
   */
  waitForEmail = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!EMAIL_TRANSIENT_STATUSES.includes(res.status))), this.getEmail, request, options);
  pageOfListEmails = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/emails`,
    urlParams: urlParams(['domain_id', request.domainId], ['flags', request.flags], ['mail_from', request.mailFrom], ['mail_rcpt', request.mailRcpt], ['mail_to', request.mailTo], ['message_id', request.messageId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['search', request.search], ['since', request.since], ['statuses', request.statuses], ['subject', request.subject], ['until', request.until])
  }, unmarshalListEmailsResponse);

  /**
   * List emails. Retrieve the list of emails sent from a specific domain or for
   * a specific Project or Organization. You must specify the `region`.
   *
   * @param request - The request {@link ListEmailsRequest}
   * @returns A Promise of ListEmailsResponse
   */
  listEmails = (request = {}) => enrichForPagination('emails', this.pageOfListEmails, request);

  /**
   * Email statuses. Get information on your emails' statuses.
   *
   * @param request - The request {@link GetStatisticsRequest}
   * @returns A Promise of Statistics
   */
  getStatistics = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/statistics`,
    urlParams: urlParams(['domain_id', request.domainId], ['mail_from', request.mailFrom], ['project_id', request.projectId], ['since', request.since], ['until', request.until])
  }, unmarshalStatistics);

  /**
   * Cancel an email. You can cancel the sending of an email if it has not been
   * sent yet. You must specify the `region` and the `email_id` of the email you
   * want to cancel.
   *
   * @param request - The request {@link CancelEmailRequest}
   * @returns A Promise of Email
   */
  cancelEmail = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$4,
    method: 'POST',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/emails/${validatePathParam('emailId', request.emailId)}/cancel`
  }, unmarshalEmail);

  /**
   * Register a domain in a project. You must specify the `region`, `project_id`
   * and `domain_name` to register a domain in a specific Project.
   *
   * @param request - The request {@link CreateDomainRequest}
   * @returns A Promise of Domain
   */
  createDomain = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDomainRequest(request, this.client.settings)),
    headers: jsonContentHeaders$4,
    method: 'POST',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains`
  }, unmarshalDomain);

  /**
   * Get information about a domain. Retrieve information about a specific
   * domain using the `region` and `domain_id` parameters. Monitor your domain's
   * reputation and improve **average** and **bad** reputation statuses, using
   * your domain's **Email activity** tab on the [Scaleway
   * console](https://console.scaleway.com/transactional-email/domains) to get a
   * more detailed report. Check out our [dedicated
   * documentation](https://www.scaleway.com/en/docs/managed-services/transactional-email/reference-content/understanding-tem-reputation-score/)
   * to improve your domain's reputation.
   *
   * @param request - The request {@link GetDomainRequest}
   * @returns A Promise of Domain
   */
  getDomain = request => this.client.fetch({
    method: 'GET',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domainId', request.domainId)}`
  }, unmarshalDomain);

  /**
   * Waits for {@link Domain} to be in a final state.
   *
   * @param request - The request {@link GetDomainRequest}
   * @param options - The waiting options
   * @returns A Promise of Domain
   */
  waitForDomain = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!DOMAIN_TRANSIENT_STATUSES.includes(res.status))), this.getDomain, request, options);
  pageOfListDomains = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains`,
    urlParams: urlParams(['name', request.name], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['status', request.status])
  }, unmarshalListDomainsResponse);

  /**
   * List domains. Retrieve domains in a specific Project or in a specific
   * Organization using the `region` parameter.
   *
   * @param request - The request {@link ListDomainsRequest}
   * @returns A Promise of ListDomainsResponse
   */
  listDomains = (request = {}) => enrichForPagination('domains', this.pageOfListDomains, request);

  /**
   * Delete a domain. You must specify the domain you want to delete by the
   * `region` and `domain_id`. Deleting a domain is permanent and cannot be
   * undone.
   *
   * @param request - The request {@link RevokeDomainRequest}
   * @returns A Promise of Domain
   */
  revokeDomain = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$4,
    method: 'POST',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domainId', request.domainId)}/revoke`
  }, unmarshalDomain);

  /**
   * Domain DNS check. Perform an immediate DNS check of a domain using the
   * `region` and `domain_id` parameters.
   *
   * @param request - The request {@link CheckDomainRequest}
   * @returns A Promise of Domain
   */
  checkDomain = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$4,
    method: 'POST',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domainId', request.domainId)}/check`
  }, unmarshalDomain);

  /**
   * Display SPF and DKIM records status and potential errors. Display SPF and
   * DKIM records status and potential errors, including the found records to
   * make debugging easier.
   *
   * @param request - The request {@link GetDomainLastStatusRequest}
   * @returns A Promise of DomainLastStatus
   */
  getDomainLastStatus = request => this.client.fetch({
    method: 'GET',
    path: `/transactional-email/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domainId', request.domainId)}/verification`
  }, unmarshalDomainLastStatus);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$4 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$4,
  DOMAIN_TRANSIENT_STATUSES: DOMAIN_TRANSIENT_STATUSES,
  EMAIL_TRANSIENT_STATUSES: EMAIL_TRANSIENT_STATUSES
});

var index$3 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen$4
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalPrivateNetwork$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrivateNetwork' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    subnets: data.subnets,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalListPrivateNetworksResponse$1 = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPrivateNetworksResponse' failed as data isn't a dictionary.`);
  }
  return {
    privateNetworks: unmarshalArrayOfObject(data.private_networks, unmarshalPrivateNetwork$1),
    totalCount: data.total_count
  };
};
const marshalCreatePrivateNetworkRequest$1 = (request, defaults) => ({
  name: request.name || randomName('pn'),
  project_id: request.projectId ?? defaults.defaultProjectId,
  subnets: request.subnets,
  tags: request.tags
});
const marshalUpdatePrivateNetworkRequest$1 = (request, defaults) => ({
  name: request.name,
  subnets: request.subnets,
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$3 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** VPC API. */
let API$3 = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'fr-par-3', 'nl-ams-1', 'nl-ams-2', 'nl-ams-3', 'pl-waw-1', 'pl-waw-2'];
  pageOfListPrivateNetworks = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/private-networks`,
    urlParams: urlParams(['include_regional', request.includeRegional], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['private_network_ids', request.privateNetworkIds], ['project_id', request.projectId], ['tags', request.tags])
  }, unmarshalListPrivateNetworksResponse$1);

  /**
   * List Private Networks. List existing Private Networks in a specified
   * Availability Zone. By default, the Private Networks returned in the list
   * are ordered by creation date in ascending order, though this can be
   * modified via the order_by field.
   *
   * @param request - The request {@link ListPrivateNetworksRequest}
   * @returns A Promise of ListPrivateNetworksResponse
   */
  listPrivateNetworks = (request = {}) => enrichForPagination('privateNetworks', this.pageOfListPrivateNetworks, request);

  /**
   * Create a Private Network. Create a new Private Network. Once created, you
   * can attach Scaleway resources in the same Availability Zone.
   *
   * @param request - The request {@link CreatePrivateNetworkRequest}
   * @returns A Promise of PrivateNetwork
   */
  createPrivateNetwork = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreatePrivateNetworkRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$3,
    method: 'POST',
    path: `/vpc/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/private-networks`
  }, unmarshalPrivateNetwork$1);

  /**
   * Get a Private Network. Retrieve information about an existing Private
   * Network, specified by its Private Network ID. Its full details are returned
   * in the response object.
   *
   * @param request - The request {@link GetPrivateNetworkRequest}
   * @returns A Promise of PrivateNetwork
   */
  getPrivateNetwork = request => this.client.fetch({
    method: 'GET',
    path: `/vpc/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}`
  }, unmarshalPrivateNetwork$1);

  /**
   * Update Private Network. Update parameters (such as name or tags) of an
   * existing Private Network, specified by its Private Network ID.
   *
   * @param request - The request {@link UpdatePrivateNetworkRequest}
   * @returns A Promise of PrivateNetwork
   */
  updatePrivateNetwork = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePrivateNetworkRequest$1(request, this.client.settings)),
    headers: jsonContentHeaders$3,
    method: 'PATCH',
    path: `/vpc/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}`
  }, unmarshalPrivateNetwork$1);

  /**
   * Delete a Private Network. Delete an existing Private Network. Note that you
   * must first detach all resources from the network, in order to delete it.
   *
   * @param request - The request {@link DeletePrivateNetworkRequest}
   */
  deletePrivateNetwork = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}`
  });
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$3 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$3
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalSubnet = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Subnet' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    subnet: data.subnet,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalPrivateNetwork = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PrivateNetwork' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    dhcpEnabled: data.dhcp_enabled,
    id: data.id,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    region: data.region,
    subnets: unmarshalArrayOfObject(data.subnets, unmarshalSubnet),
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    vpcId: data.vpc_id
  };
};
const unmarshalVPC = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'VPC' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    id: data.id,
    isDefault: data.is_default,
    name: data.name,
    organizationId: data.organization_id,
    privateNetworkCount: data.private_network_count,
    projectId: data.project_id,
    region: data.region,
    routingEnabled: data.routing_enabled,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at)
  };
};
const unmarshalAddSubnetsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'AddSubnetsResponse' failed as data isn't a dictionary.`);
  }
  return {
    subnets: data.subnets
  };
};
const unmarshalDeleteSubnetsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DeleteSubnetsResponse' failed as data isn't a dictionary.`);
  }
  return {
    subnets: data.subnets
  };
};
const unmarshalListPrivateNetworksResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPrivateNetworksResponse' failed as data isn't a dictionary.`);
  }
  return {
    privateNetworks: unmarshalArrayOfObject(data.private_networks, unmarshalPrivateNetwork),
    totalCount: data.total_count
  };
};
const unmarshalListVPCsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListVPCsResponse' failed as data isn't a dictionary.`);
  }
  return {
    totalCount: data.total_count,
    vpcs: unmarshalArrayOfObject(data.vpcs, unmarshalVPC)
  };
};
const unmarshalSetSubnetsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetSubnetsResponse' failed as data isn't a dictionary.`);
  }
  return {
    subnets: data.subnets
  };
};
const marshalAddSubnetsRequest = (request, defaults) => ({
  subnets: request.subnets
});
const marshalCreatePrivateNetworkRequest = (request, defaults) => ({
  name: request.name || randomName('pn'),
  project_id: request.projectId ?? defaults.defaultProjectId,
  subnets: request.subnets,
  tags: request.tags,
  vpc_id: request.vpcId
});
const marshalCreateVPCRequest = (request, defaults) => ({
  enable_routing: request.enableRouting,
  name: request.name || randomName('vpc'),
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags
});
const marshalDeleteSubnetsRequest = (request, defaults) => ({
  subnets: request.subnets
});
const marshalMigrateZonalPrivateNetworksRequest = (request, defaults) => ({
  private_network_ids: request.privateNetworkIds,
  ...resolveOneOf([{
    default: defaults.defaultOrganizationId,
    param: 'organization_id',
    value: request.organizationId
  }, {
    default: defaults.defaultProjectId,
    param: 'project_id',
    value: request.projectId
  }])
});
const marshalSetSubnetsRequest = (request, defaults) => ({
  subnets: request.subnets
});
const marshalUpdatePrivateNetworkRequest = (request, defaults) => ({
  name: request.name,
  tags: request.tags
});
const marshalUpdateVPCRequest = (request, defaults) => ({
  name: request.name,
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$2 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** VPC API. */
let API$2 = class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams', 'pl-waw'];
  pageOfListVPCs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/vpcs`,
    urlParams: urlParams(['is_default', request.isDefault], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['routing_enabled', request.routingEnabled], ['tags', request.tags])
  }, unmarshalListVPCsResponse);

  /**
   * List VPCs. List existing VPCs in the specified region.
   *
   * @param request - The request {@link ListVPCsRequest}
   * @returns A Promise of ListVPCsResponse
   */
  listVPCs = (request = {}) => enrichForPagination('vpcs', this.pageOfListVPCs, request);

  /**
   * Create a VPC. Create a new VPC in the specified region.
   *
   * @param request - The request {@link CreateVPCRequest}
   * @returns A Promise of VPC
   */
  createVPC = request => this.client.fetch({
    body: JSON.stringify(marshalCreateVPCRequest(request, this.client.settings)),
    headers: jsonContentHeaders$2,
    method: 'POST',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/vpcs`
  }, unmarshalVPC);

  /**
   * Get a VPC. Retrieve details of an existing VPC, specified by its VPC ID.
   *
   * @param request - The request {@link GetVPCRequest}
   * @returns A Promise of VPC
   */
  getVPC = request => this.client.fetch({
    method: 'GET',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/vpcs/${validatePathParam('vpcId', request.vpcId)}`
  }, unmarshalVPC);

  /**
   * Update VPC. Update parameters including name and tags of the specified VPC.
   *
   * @param request - The request {@link UpdateVPCRequest}
   * @returns A Promise of VPC
   */
  updateVPC = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateVPCRequest(request, this.client.settings)),
    headers: jsonContentHeaders$2,
    method: 'PATCH',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/vpcs/${validatePathParam('vpcId', request.vpcId)}`
  }, unmarshalVPC);

  /**
   * Delete a VPC. Delete a VPC specified by its VPC ID.
   *
   * @param request - The request {@link DeleteVPCRequest}
   */
  deleteVPC = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/vpcs/${validatePathParam('vpcId', request.vpcId)}`
  });
  pageOfListPrivateNetworks = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks`,
    urlParams: urlParams(['dhcp_enabled', request.dhcpEnabled], ['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['private_network_ids', request.privateNetworkIds], ['project_id', request.projectId], ['tags', request.tags], ['vpc_id', request.vpcId])
  }, unmarshalListPrivateNetworksResponse);

  /**
   * List Private Networks. List existing Private Networks in the specified
   * region. By default, the Private Networks returned in the list are ordered
   * by creation date in ascending order, though this can be modified via the
   * order_by field.
   *
   * @param request - The request {@link ListPrivateNetworksRequest}
   * @returns A Promise of ListPrivateNetworksResponse
   */
  listPrivateNetworks = (request = {}) => enrichForPagination('privateNetworks', this.pageOfListPrivateNetworks, request);

  /**
   * Create a Private Network. Create a new Private Network. Once created, you
   * can attach Scaleway resources which are in the same region.
   *
   * @param request - The request {@link CreatePrivateNetworkRequest}
   * @returns A Promise of PrivateNetwork
   */
  createPrivateNetwork = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreatePrivateNetworkRequest(request, this.client.settings)),
    headers: jsonContentHeaders$2,
    method: 'POST',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks`
  }, unmarshalPrivateNetwork);

  /**
   * Get a Private Network. Retrieve information about an existing Private
   * Network, specified by its Private Network ID. Its full details are returned
   * in the response object.
   *
   * @param request - The request {@link GetPrivateNetworkRequest}
   * @returns A Promise of PrivateNetwork
   */
  getPrivateNetwork = request => this.client.fetch({
    method: 'GET',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}`
  }, unmarshalPrivateNetwork);

  /**
   * Update Private Network. Update parameters (such as name or tags) of an
   * existing Private Network, specified by its Private Network ID.
   *
   * @param request - The request {@link UpdatePrivateNetworkRequest}
   * @returns A Promise of PrivateNetwork
   */
  updatePrivateNetwork = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePrivateNetworkRequest(request, this.client.settings)),
    headers: jsonContentHeaders$2,
    method: 'PATCH',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}`
  }, unmarshalPrivateNetwork);

  /**
   * Delete a Private Network. Delete an existing Private Network. Note that you
   * must first detach all resources from the network, in order to delete it.
   *
   * @param request - The request {@link DeletePrivateNetworkRequest}
   */
  deletePrivateNetwork = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}`
  });

  /**
   * Migrate Private Networks from zoned to regional. Transform multiple
   * existing zoned Private Networks (scoped to a single Availability Zone) into
   * regional Private Networks, scoped to an entire region. You can transform
   * one or many Private Networks (specified by their Private Network IDs)
   * within a single Scaleway Organization or Project, with the same call.
   *
   * @param request - The request {@link MigrateZonalPrivateNetworksRequest}
   */
  migrateZonalPrivateNetworks = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalMigrateZonalPrivateNetworksRequest(request, this.client.settings)),
    headers: jsonContentHeaders$2,
    method: 'POST',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks/migrate-zonal`
  });

  /**
   * Enable DHCP on a Private Network. Enable DHCP managed on an existing
   * Private Network. Note that you will not be able to deactivate it
   * afterwards.
   *
   * @param request - The request {@link EnableDHCPRequest}
   * @returns A Promise of PrivateNetwork
   */
  enableDHCP = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$2,
    method: 'POST',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}/enable-dhcp`
  }, unmarshalPrivateNetwork);

  /**
   * Enable routing on a VPC. Enable routing on an existing VPC. Note that you
   * will not be able to deactivate it afterwards.
   *
   * @param request - The request {@link EnableRoutingRequest}
   * @returns A Promise of VPC
   */
  enableRouting = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$2,
    method: 'POST',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/vpcs/${validatePathParam('vpcId', request.vpcId)}/enable-routing`
  }, unmarshalVPC);

  /**
   * Set the subnets of a Private Network. Set subnets for an existing Private
   * Network. Note that the method is PUT and not PATCH. Any existing subnets
   * will be removed in favor of the new specified set of subnets.
   *
   * @param request - The request {@link SetSubnetsRequest}
   * @returns A Promise of SetSubnetsResponse
   */
  setSubnets = request => this.client.fetch({
    body: JSON.stringify(marshalSetSubnetsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$2,
    method: 'PUT',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}/subnets`
  }, unmarshalSetSubnetsResponse);

  /**
   * Add subnets to a Private Network. Add new subnets to an existing Private
   * Network.
   *
   * @param request - The request {@link AddSubnetsRequest}
   * @returns A Promise of AddSubnetsResponse
   */
  addSubnets = request => this.client.fetch({
    body: JSON.stringify(marshalAddSubnetsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$2,
    method: 'POST',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}/subnets`
  }, unmarshalAddSubnetsResponse);

  /**
   * Delete subnets from a Private Network. Delete the specified subnets from a
   * Private Network.
   *
   * @param request - The request {@link DeleteSubnetsRequest}
   * @returns A Promise of DeleteSubnetsResponse
   */
  deleteSubnets = request => this.client.fetch({
    body: JSON.stringify(marshalDeleteSubnetsRequest(request, this.client.settings)),
    headers: jsonContentHeaders$2,
    method: 'DELETE',
    path: `/vpc/v2/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/private-networks/${validatePathParam('privateNetworkId', request.privateNetworkId)}/subnets`
  }, unmarshalDeleteSubnetsResponse);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$2 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$2
});

var index$2 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$3,
  v2: index_gen$2
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link GatewayNetworkStatus}. */
const GATEWAY_NETWORK_TRANSIENT_STATUSES = ['attaching', 'configuring', 'detaching'];

/** Lists transient statutes of the enum {@link GatewayStatus}. */
const GATEWAY_TRANSIENT_STATUSES = ['allocating', 'configuring', 'stopping', 'deleting'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalDHCP = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DHCP' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    createdAt: unmarshalDate(data.created_at),
    dnsLocalName: data.dns_local_name,
    dnsSearch: data.dns_search,
    dnsServersOverride: data.dns_servers_override,
    enableDynamic: data.enable_dynamic,
    id: data.id,
    organizationId: data.organization_id,
    poolHigh: data.pool_high,
    poolLow: data.pool_low,
    projectId: data.project_id,
    pushDefaultRoute: data.push_default_route,
    pushDnsServer: data.push_dns_server,
    rebindTimer: data.rebind_timer,
    renewTimer: data.renew_timer,
    subnet: data.subnet,
    updatedAt: unmarshalDate(data.updated_at),
    validLifetime: data.valid_lifetime,
    zone: data.zone
  };
};
const unmarshalIpamConfig = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'IpamConfig' failed as data isn't a dictionary.`);
  }
  return {
    ipamIpId: data.ipam_ip_id,
    pushDefaultRoute: data.push_default_route
  };
};
const unmarshalGatewayNetwork = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GatewayNetwork' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    createdAt: unmarshalDate(data.created_at),
    dhcp: data.dhcp ? unmarshalDHCP(data.dhcp) : undefined,
    enableDhcp: data.enable_dhcp,
    enableMasquerade: data.enable_masquerade,
    gatewayId: data.gateway_id,
    id: data.id,
    ipamConfig: data.ipam_config ? unmarshalIpamConfig(data.ipam_config) : undefined,
    macAddress: data.mac_address,
    privateNetworkId: data.private_network_id,
    status: data.status,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalIP = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'IP' failed as data isn't a dictionary.`);
  }
  return {
    address: data.address,
    createdAt: unmarshalDate(data.created_at),
    gatewayId: data.gateway_id,
    id: data.id,
    organizationId: data.organization_id,
    projectId: data.project_id,
    reverse: data.reverse,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalDHCPEntry = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DHCPEntry' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    gatewayNetworkId: data.gateway_network_id,
    hostname: data.hostname,
    id: data.id,
    ipAddress: data.ip_address,
    macAddress: data.mac_address,
    type: data.type,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalGatewayType = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'GatewayType' failed as data isn't a dictionary.`);
  }
  return {
    bandwidth: data.bandwidth,
    name: data.name,
    zone: data.zone
  };
};
const unmarshalGateway = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Gateway' failed as data isn't a dictionary.`);
  }
  return {
    bastionEnabled: data.bastion_enabled,
    bastionPort: data.bastion_port,
    canUpgradeTo: data.can_upgrade_to,
    createdAt: unmarshalDate(data.created_at),
    gatewayNetworks: unmarshalArrayOfObject(data.gateway_networks, unmarshalGatewayNetwork),
    id: data.id,
    ip: data.ip ? unmarshalIP(data.ip) : undefined,
    ipMobilityEnabled: data.ip_mobility_enabled,
    isLegacy: data.is_legacy,
    name: data.name,
    organizationId: data.organization_id,
    projectId: data.project_id,
    smtpEnabled: data.smtp_enabled,
    status: data.status,
    tags: data.tags,
    type: data.type ? unmarshalGatewayType(data.type) : undefined,
    updatedAt: unmarshalDate(data.updated_at),
    upstreamDnsServers: data.upstream_dns_servers,
    version: data.version,
    zone: data.zone
  };
};
const unmarshalPATRule = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'PATRule' failed as data isn't a dictionary.`);
  }
  return {
    createdAt: unmarshalDate(data.created_at),
    gatewayId: data.gateway_id,
    id: data.id,
    privateIp: data.private_ip,
    privatePort: data.private_port,
    protocol: data.protocol,
    publicPort: data.public_port,
    updatedAt: unmarshalDate(data.updated_at),
    zone: data.zone
  };
};
const unmarshalListDHCPEntriesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDHCPEntriesResponse' failed as data isn't a dictionary.`);
  }
  return {
    dhcpEntries: unmarshalArrayOfObject(data.dhcp_entries, unmarshalDHCPEntry),
    totalCount: data.total_count
  };
};
const unmarshalListDHCPsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListDHCPsResponse' failed as data isn't a dictionary.`);
  }
  return {
    dhcps: unmarshalArrayOfObject(data.dhcps, unmarshalDHCP),
    totalCount: data.total_count
  };
};
const unmarshalListGatewayNetworksResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListGatewayNetworksResponse' failed as data isn't a dictionary.`);
  }
  return {
    gatewayNetworks: unmarshalArrayOfObject(data.gateway_networks, unmarshalGatewayNetwork),
    totalCount: data.total_count
  };
};
const unmarshalListGatewayTypesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListGatewayTypesResponse' failed as data isn't a dictionary.`);
  }
  return {
    types: unmarshalArrayOfObject(data.types, unmarshalGatewayType)
  };
};
const unmarshalListGatewaysResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListGatewaysResponse' failed as data isn't a dictionary.`);
  }
  return {
    gateways: unmarshalArrayOfObject(data.gateways, unmarshalGateway),
    totalCount: data.total_count
  };
};
const unmarshalListIPsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListIPsResponse' failed as data isn't a dictionary.`);
  }
  return {
    ips: unmarshalArrayOfObject(data.ips, unmarshalIP),
    totalCount: data.total_count
  };
};
const unmarshalListPATRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListPATRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    patRules: unmarshalArrayOfObject(data.pat_rules, unmarshalPATRule),
    totalCount: data.total_count
  };
};
const unmarshalSetDHCPEntriesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetDHCPEntriesResponse' failed as data isn't a dictionary.`);
  }
  return {
    dhcpEntries: unmarshalArrayOfObject(data.dhcp_entries, unmarshalDHCPEntry)
  };
};
const unmarshalSetPATRulesResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'SetPATRulesResponse' failed as data isn't a dictionary.`);
  }
  return {
    patRules: unmarshalArrayOfObject(data.pat_rules, unmarshalPATRule)
  };
};
const marshalCreateDHCPRequest = (request, defaults) => ({
  address: request.address,
  dns_local_name: request.dnsLocalName,
  dns_search: request.dnsSearch,
  dns_servers_override: request.dnsServersOverride,
  enable_dynamic: request.enableDynamic,
  pool_high: request.poolHigh,
  pool_low: request.poolLow,
  project_id: request.projectId ?? defaults.defaultProjectId,
  push_default_route: request.pushDefaultRoute,
  push_dns_server: request.pushDnsServer,
  rebind_timer: request.rebindTimer,
  renew_timer: request.renewTimer,
  subnet: request.subnet,
  valid_lifetime: request.validLifetime
});
const marshalCreateDHCPEntryRequest = (request, defaults) => ({
  gateway_network_id: request.gatewayNetworkId,
  ip_address: request.ipAddress,
  mac_address: request.macAddress
});
const marshalCreateGatewayNetworkRequestIpamConfig = (request, defaults) => ({
  ipam_ip_id: request.ipamIpId,
  push_default_route: request.pushDefaultRoute
});
const marshalCreateGatewayNetworkRequest = (request, defaults) => ({
  enable_dhcp: request.enableDhcp,
  enable_masquerade: request.enableMasquerade,
  gateway_id: request.gatewayId,
  private_network_id: request.privateNetworkId,
  ...resolveOneOf([{
    param: 'dhcp_id',
    value: request.dhcpId
  }, {
    param: 'dhcp',
    value: request.dhcp !== undefined ? marshalCreateDHCPRequest(request.dhcp, defaults) : undefined
  }, {
    param: 'address',
    value: request.address
  }, {
    param: 'ipam_config',
    value: request.ipamConfig !== undefined ? marshalCreateGatewayNetworkRequestIpamConfig(request.ipamConfig) : undefined
  }])
});
const marshalCreateGatewayRequest = (request, defaults) => ({
  bastion_port: request.bastionPort,
  enable_bastion: request.enableBastion,
  enable_smtp: request.enableSmtp,
  ip_id: request.ipId,
  name: request.name || randomName('gw'),
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags,
  type: request.type,
  upstream_dns_servers: request.upstreamDnsServers
});
const marshalCreateIPRequest = (request, defaults) => ({
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags
});
const marshalCreatePATRuleRequest = (request, defaults) => ({
  gateway_id: request.gatewayId,
  private_ip: request.privateIp,
  private_port: request.privatePort,
  protocol: request.protocol,
  public_port: request.publicPort
});
const marshalSetDHCPEntriesRequestEntry = (request, defaults) => ({
  ip_address: request.ipAddress,
  mac_address: request.macAddress
});
const marshalSetDHCPEntriesRequest = (request, defaults) => ({
  dhcp_entries: request.dhcpEntries !== undefined ? request.dhcpEntries.map(elt => marshalSetDHCPEntriesRequestEntry(elt)) : undefined,
  gateway_network_id: request.gatewayNetworkId
});
const marshalSetPATRulesRequestRule = (request, defaults) => ({
  private_ip: request.privateIp,
  private_port: request.privatePort,
  protocol: request.protocol,
  public_port: request.publicPort
});
const marshalSetPATRulesRequest = (request, defaults) => ({
  gateway_id: request.gatewayId,
  pat_rules: request.patRules.map(elt => marshalSetPATRulesRequestRule(elt))
});
const marshalUpdateDHCPEntryRequest = (request, defaults) => ({
  ip_address: request.ipAddress
});
const marshalUpdateDHCPRequest = (request, defaults) => ({
  address: request.address,
  dns_local_name: request.dnsLocalName,
  dns_search: request.dnsSearch,
  dns_servers_override: request.dnsServersOverride,
  enable_dynamic: request.enableDynamic,
  pool_high: request.poolHigh,
  pool_low: request.poolLow,
  push_default_route: request.pushDefaultRoute,
  push_dns_server: request.pushDnsServer,
  rebind_timer: request.rebindTimer,
  renew_timer: request.renewTimer,
  subnet: request.subnet,
  valid_lifetime: request.validLifetime
});
const marshalUpdateGatewayNetworkRequestIpamConfig = (request, defaults) => ({
  ipam_ip_id: request.ipamIpId,
  push_default_route: request.pushDefaultRoute
});
const marshalUpdateGatewayNetworkRequest = (request, defaults) => ({
  enable_dhcp: request.enableDhcp,
  enable_masquerade: request.enableMasquerade,
  ...resolveOneOf([{
    param: 'dhcp_id',
    value: request.dhcpId
  }, {
    param: 'address',
    value: request.address
  }, {
    param: 'ipam_config',
    value: request.ipamConfig !== undefined ? marshalUpdateGatewayNetworkRequestIpamConfig(request.ipamConfig) : undefined
  }])
});
const marshalUpdateGatewayRequest = (request, defaults) => ({
  bastion_port: request.bastionPort,
  enable_bastion: request.enableBastion,
  enable_smtp: request.enableSmtp,
  name: request.name,
  tags: request.tags,
  upstream_dns_servers: request.upstreamDnsServers
});
const marshalUpdateIPRequest = (request, defaults) => ({
  gateway_id: request.gatewayId,
  reverse: request.reverse,
  tags: request.tags
});
const marshalUpdatePATRuleRequest = (request, defaults) => ({
  private_ip: request.privateIp,
  private_port: request.privatePort,
  protocol: request.protocol,
  public_port: request.publicPort
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders$1 = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Public Gateways API. */
let API$1 = class API extends API$z {
  /** Lists the available zones of the API. */
  static LOCALITIES = ['fr-par-1', 'fr-par-2', 'nl-ams-1', 'nl-ams-2', 'nl-ams-3', 'pl-waw-1', 'pl-waw-2', 'pl-waw-3'];
  pageOfListGateways = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateways`,
    urlParams: urlParams(['name', request.name], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['private_network_id', request.privateNetworkId], ['project_id', request.projectId], ['status', request.status], ['tags', request.tags], ['type', request.type])
  }, unmarshalListGatewaysResponse);

  /**
   * List Public Gateways. List Public Gateways in a given Scaleway Organization
   * or Project. By default, results are displayed in ascending order of
   * creation date.
   *
   * @param request - The request {@link ListGatewaysRequest}
   * @returns A Promise of ListGatewaysResponse
   */
  listGateways = (request = {}) => enrichForPagination('gateways', this.pageOfListGateways, request);

  /**
   * Get a Public Gateway. Get details of a Public Gateway, specified by its
   * gateway ID. The response object contains full details of the gateway,
   * including its **name**, **type**, **status** and more.
   *
   * @param request - The request {@link GetGatewayRequest}
   * @returns A Promise of Gateway
   */
  getGateway = request => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateways/${validatePathParam('gatewayId', request.gatewayId)}`
  }, unmarshalGateway);

  /**
   * Waits for {@link Gateway} to be in a final state.
   *
   * @param request - The request {@link GetGatewayRequest}
   * @param options - The waiting options
   * @returns A Promise of Gateway
   */
  waitForGateway = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!GATEWAY_TRANSIENT_STATUSES.includes(res.status))), this.getGateway, request, options);

  /**
   * Create a Public Gateway. Create a new Public Gateway in the specified
   * Scaleway Project, defining its **name**, **type** and other configuration
   * details such as whether to enable SSH bastion.
   *
   * @param request - The request {@link CreateGatewayRequest}
   * @returns A Promise of Gateway
   */
  createGateway = request => this.client.fetch({
    body: JSON.stringify(marshalCreateGatewayRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateways`
  }, unmarshalGateway);

  /**
   * Update a Public Gateway. Update the parameters of an existing Public
   * Gateway, for example, its **name**, **tags**, **SSH bastion
   * configuration**, and **DNS servers**.
   *
   * @param request - The request {@link UpdateGatewayRequest}
   * @returns A Promise of Gateway
   */
  updateGateway = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateGatewayRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'PATCH',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateways/${validatePathParam('gatewayId', request.gatewayId)}`
  }, unmarshalGateway);

  /**
   * Delete a Public Gateway. Delete an existing Public Gateway, specified by
   * its gateway ID. This action is irreversible.
   *
   * @param request - The request {@link DeleteGatewayRequest}
   */
  deleteGateway = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateways/${validatePathParam('gatewayId', request.gatewayId)}`,
    urlParams: urlParams(['cleanup_dhcp', request.cleanupDhcp])
  });

  /**
   * Upgrade a Public Gateway to the latest version. Upgrade a given Public
   * Gateway to the newest software version. This applies the latest bugfixes
   * and features to your Public Gateway, but its service will be interrupted
   * during the update.
   *
   * @param request - The request {@link UpgradeGatewayRequest}
   * @returns A Promise of Gateway
   */
  upgradeGateway = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateways/${validatePathParam('gatewayId', request.gatewayId)}/upgrade`
  }, unmarshalGateway);

  /**
   * Upgrade a Public Gateway to IP mobility. Upgrade a Public Gateway to IP
   * mobility (move from NAT IP to routed IP). This is idempotent: repeated
   * calls after the first will return no error but have no effect.
   *
   * @param request - The request {@link EnableIPMobilityRequest}
   */
  enableIPMobility = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateways/${validatePathParam('gatewayId', request.gatewayId)}/enable-ip-mobility`
  });
  pageOfListGatewayNetworks = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateway-networks`,
    urlParams: urlParams(['dhcp_id', request.dhcpId], ['enable_masquerade', request.enableMasquerade], ['gateway_id', request.gatewayId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['private_network_id', request.privateNetworkId], ['status', request.status])
  }, unmarshalListGatewayNetworksResponse);

  /**
   * List Public Gateway connections to Private Networks. List the connections
   * between Public Gateways and Private Networks (a connection = a
   * GatewayNetwork). You can choose to filter by `gateway-id` to list all
   * Private Networks attached to the specified Public Gateway, or by
   * `private_network_id` to list all Public Gateways attached to the specified
   * Private Network. Other query parameters are also available. The result is
   * an array of GatewayNetwork objects, each giving details of the connection
   * between a given Public Gateway and a given Private Network.
   *
   * @param request - The request {@link ListGatewayNetworksRequest}
   * @returns A Promise of ListGatewayNetworksResponse
   */
  listGatewayNetworks = (request = {}) => enrichForPagination('gatewayNetworks', this.pageOfListGatewayNetworks, request);

  /**
   * Get a Public Gateway connection to a Private Network. Get details of a
   * given connection between a Public Gateway and a Private Network (this
   * connection = a GatewayNetwork), specified by its `gateway_network_id`. The
   * response object contains details of the connection including the IDs of the
   * Public Gateway and Private Network, the dates the connection was
   * created/updated and its configuration settings.
   *
   * @param request - The request {@link GetGatewayNetworkRequest}
   * @returns A Promise of GatewayNetwork
   */
  getGatewayNetwork = request => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateway-networks/${validatePathParam('gatewayNetworkId', request.gatewayNetworkId)}`
  }, unmarshalGatewayNetwork);

  /**
   * Waits for {@link GatewayNetwork} to be in a final state.
   *
   * @param request - The request {@link GetGatewayNetworkRequest}
   * @param options - The waiting options
   * @returns A Promise of GatewayNetwork
   */
  waitForGatewayNetwork = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!GATEWAY_NETWORK_TRANSIENT_STATUSES.includes(res.status))), this.getGatewayNetwork, request, options);

  /**
   * Attach a Public Gateway to a Private Network. Attach a specific Public
   * Gateway to a specific Private Network (create a GatewayNetwork). You can
   * configure parameters for the connection including DHCP settings, whether to
   * enable masquerade (dynamic NAT), and more.
   *
   * @param request - The request {@link CreateGatewayNetworkRequest}
   * @returns A Promise of GatewayNetwork
   */
  createGatewayNetwork = request => this.client.fetch({
    body: JSON.stringify(marshalCreateGatewayNetworkRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateway-networks`
  }, unmarshalGatewayNetwork);

  /**
   * Update a Public Gateway's connection to a Private Network. Update the
   * configuration parameters of a connection between a given Public Gateway and
   * Private Network (the connection = a GatewayNetwork). Updatable parameters
   * include DHCP settings and whether to enable traffic masquerade (dynamic
   * NAT).
   *
   * @param request - The request {@link UpdateGatewayNetworkRequest}
   * @returns A Promise of GatewayNetwork
   */
  updateGatewayNetwork = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateGatewayNetworkRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'PATCH',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateway-networks/${validatePathParam('gatewayNetworkId', request.gatewayNetworkId)}`
  }, unmarshalGatewayNetwork);

  /**
   * Detach a Public Gateway from a Private Network. Detach a given Public
   * Gateway from a given Private Network, i.e. delete a GatewayNetwork
   * specified by a gateway_network_id.
   *
   * @param request - The request {@link DeleteGatewayNetworkRequest}
   */
  deleteGatewayNetwork = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateway-networks/${validatePathParam('gatewayNetworkId', request.gatewayNetworkId)}`,
    urlParams: urlParams(['cleanup_dhcp', request.cleanupDhcp])
  });
  pageOfListDHCPs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcps`,
    urlParams: urlParams(['address', request.address], ['has_address', request.hasAddress], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId])
  }, unmarshalListDHCPsResponse);

  /**
   * List DHCP configurations. List DHCP configurations, optionally filtering by
   * Organization, Project, Public Gateway IP address or more. The response is
   * an array of DHCP configuration objects, each identified by a DHCP ID and
   * containing configuration settings for the assignment of IP addresses to
   * devices on a Private Network attached to a Public Gateway. Note that the
   * response does not contain the IDs of any Private Network / Public Gateway
   * the configuration is attached to. Use the `List Public Gateway connections
   * to Private Networks` method for that purpose, filtering on DHCP ID.
   *
   * @param request - The request {@link ListDHCPsRequest}
   * @returns A Promise of ListDHCPsResponse
   */
  listDHCPs = (request = {}) => enrichForPagination('dhcps', this.pageOfListDHCPs, request);

  /**
   * Get a DHCP configuration. Get a DHCP configuration object, identified by
   * its DHCP ID. The response object contains configuration settings for the
   * assignment of IP addresses to devices on a Private Network attached to a
   * Public Gateway. Note that the response does not contain the IDs of any
   * Private Network / Public Gateway the configuration is attached to. Use the
   * `List Public Gateway connections to Private Networks` method for that
   * purpose, filtering on DHCP ID.
   *
   * @param request - The request {@link GetDHCPRequest}
   * @returns A Promise of DHCP
   */
  getDHCP = request => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcps/${validatePathParam('dhcpId', request.dhcpId)}`
  }, unmarshalDHCP);

  /**
   * Create a DHCP configuration. Create a new DHCP configuration object,
   * containing settings for the assignment of IP addresses to devices on a
   * Private Network attached to a Public Gateway. The response object includes
   * the ID of the DHCP configuration object. You can use this ID as part of a
   * call to `Create a Public Gateway connection to a Private Network` or
   * `Update a Public Gateway connection to a Private Network` to directly apply
   * this DHCP configuration.
   *
   * @param request - The request {@link CreateDHCPRequest}
   * @returns A Promise of DHCP
   */
  createDHCP = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDHCPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcps`
  }, unmarshalDHCP);

  /**
   * Update a DHCP configuration. Update a DHCP configuration object, identified
   * by its DHCP ID.
   *
   * @param request - The request {@link UpdateDHCPRequest}
   * @returns A Promise of DHCP
   */
  updateDHCP = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDHCPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'PATCH',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcps/${validatePathParam('dhcpId', request.dhcpId)}`
  }, unmarshalDHCP);

  /**
   * Delete a DHCP configuration. Delete a DHCP configuration object, identified
   * by its DHCP ID. Note that you cannot delete a DHCP configuration object
   * that is currently being used by a Gateway Network.
   *
   * @param request - The request {@link DeleteDHCPRequest}
   */
  deleteDHCP = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcps/${validatePathParam('dhcpId', request.dhcpId)}`
  });
  pageOfListDHCPEntries = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcp-entries`,
    urlParams: urlParams(['gateway_network_id', request.gatewayNetworkId], ['hostname', request.hostname], ['ip_address', request.ipAddress], ['mac_address', request.macAddress], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['type', request.type])
  }, unmarshalListDHCPEntriesResponse);

  /**
   * List DHCP entries. List DHCP entries, whether dynamically assigned and/or
   * statically reserved. DHCP entries can be filtered by the Gateway Network
   * they are on, their MAC address, IP address, type or hostname.
   *
   * @param request - The request {@link ListDHCPEntriesRequest}
   * @returns A Promise of ListDHCPEntriesResponse
   */
  listDHCPEntries = (request = {}) => enrichForPagination('dhcpEntries', this.pageOfListDHCPEntries, request);

  /**
   * Get a DHCP entry. Get a DHCP entry, specified by its DHCP entry ID.
   *
   * @param request - The request {@link GetDHCPEntryRequest}
   * @returns A Promise of DHCPEntry
   */
  getDHCPEntry = request => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcp-entries/${validatePathParam('dhcpEntryId', request.dhcpEntryId)}`
  }, unmarshalDHCPEntry);

  /**
   * Create a DHCP entry. Create a static DHCP reservation, specifying the
   * Gateway Network for the reservation, the MAC address of the target device
   * and the IP address to assign this device. The response is a DHCP entry
   * object, confirming the ID and configuration details of the static DHCP
   * reservation.
   *
   * @param request - The request {@link CreateDHCPEntryRequest}
   * @returns A Promise of DHCPEntry
   */
  createDHCPEntry = request => this.client.fetch({
    body: JSON.stringify(marshalCreateDHCPEntryRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcp-entries`
  }, unmarshalDHCPEntry);

  /**
   * Update a DHCP entry. Update the IP address for a DHCP entry, specified by
   * its DHCP entry ID. You can update an existing DHCP entry of any type
   * (`reservation` (static), `lease` (dynamic) or `unknown`), but in manually
   * updating the IP address the entry will necessarily be of type `reservation`
   * after the update.
   *
   * @param request - The request {@link UpdateDHCPEntryRequest}
   * @returns A Promise of DHCPEntry
   */
  updateDHCPEntry = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateDHCPEntryRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'PATCH',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcp-entries/${validatePathParam('dhcpEntryId', request.dhcpEntryId)}`
  }, unmarshalDHCPEntry);

  /**
   * Set all DHCP reservations on a Gateway Network. Set the list of DHCP
   * reservations attached to a Gateway Network. Reservations are identified by
   * their MAC address, and will sync the current DHCP entry list to the given
   * list, creating, updating or deleting DHCP entries accordingly.
   *
   * @param request - The request {@link SetDHCPEntriesRequest}
   * @returns A Promise of SetDHCPEntriesResponse
   */
  setDHCPEntries = request => this.client.fetch({
    body: JSON.stringify(marshalSetDHCPEntriesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'PUT',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcp-entries`
  }, unmarshalSetDHCPEntriesResponse);

  /**
   * Delete a DHCP entry. Delete a static DHCP reservation, identified by its
   * DHCP entry ID. Note that you cannot delete DHCP entries of type `lease`,
   * these are deleted automatically when their time-to-live expires.
   *
   * @param request - The request {@link DeleteDHCPEntryRequest}
   */
  deleteDHCPEntry = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/dhcp-entries/${validatePathParam('dhcpEntryId', request.dhcpEntryId)}`
  });
  pageOfListPATRules = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/pat-rules`,
    urlParams: urlParams(['gateway_id', request.gatewayId], ['order_by', request.orderBy], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['private_ip', request.privateIp], ['protocol', request.protocol])
  }, unmarshalListPATRulesResponse);

  /**
   * List PAT rules. List PAT rules. You can filter by gateway ID to list all
   * PAT rules for a particular gateway, or filter for PAT rules targeting a
   * specific IP address or using a specific protocol.
   *
   * @param request - The request {@link ListPATRulesRequest}
   * @returns A Promise of ListPATRulesResponse
   */
  listPATRules = (request = {}) => enrichForPagination('patRules', this.pageOfListPATRules, request);

  /**
   * Get a PAT rule. Get a PAT rule, specified by its PAT rule ID. The response
   * object gives full details of the PAT rule, including the Public Gateway it
   * belongs to and the configuration settings in terms of public / private
   * ports, private IP and protocol.
   *
   * @param request - The request {@link GetPATRuleRequest}
   * @returns A Promise of PATRule
   */
  getPATRule = request => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/pat-rules/${validatePathParam('patRuleId', request.patRuleId)}`
  }, unmarshalPATRule);

  /**
   * Create a PAT rule. Create a new PAT rule on a specified Public Gateway,
   * defining the protocol to use, public port to listen on, and private port /
   * IP address to map to.
   *
   * @param request - The request {@link CreatePATRuleRequest}
   * @returns A Promise of PATRule
   */
  createPATRule = request => this.client.fetch({
    body: JSON.stringify(marshalCreatePATRuleRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/pat-rules`
  }, unmarshalPATRule);

  /**
   * Update a PAT rule. Update a PAT rule, specified by its PAT rule ID.
   * Configuration settings including private/public port, private IP address
   * and protocol can all be updated.
   *
   * @param request - The request {@link UpdatePATRuleRequest}
   * @returns A Promise of PATRule
   */
  updatePATRule = request => this.client.fetch({
    body: JSON.stringify(marshalUpdatePATRuleRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'PATCH',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/pat-rules/${validatePathParam('patRuleId', request.patRuleId)}`
  }, unmarshalPATRule);

  /**
   * Set all PAT rules. Set a definitive list of PAT rules attached to a Public
   * Gateway. Each rule is identified by its public port and protocol. This will
   * sync the current PAT rule list on the gateway with the new list, creating,
   * updating or deleting PAT rules accordingly.
   *
   * @param request - The request {@link SetPATRulesRequest}
   * @returns A Promise of SetPATRulesResponse
   */
  setPATRules = request => this.client.fetch({
    body: JSON.stringify(marshalSetPATRulesRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'PUT',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/pat-rules`
  }, unmarshalSetPATRulesResponse);

  /**
   * Delete a PAT rule. Delete a PAT rule, identified by its PAT rule ID. This
   * action is irreversible.
   *
   * @param request - The request {@link DeletePATRuleRequest}
   */
  deletePATRule = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/pat-rules/${validatePathParam('patRuleId', request.patRuleId)}`
  });

  /**
   * List Public Gateway types. List the different Public Gateway commercial
   * offer types available at Scaleway. The response is an array of objects
   * describing the name and technical details of each available gateway type.
   *
   * @param request - The request {@link ListGatewayTypesRequest}
   * @returns A Promise of ListGatewayTypesResponse
   */
  listGatewayTypes = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateway-types`
  }, unmarshalListGatewayTypesResponse);
  pageOfListIPs = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips`,
    urlParams: urlParams(['is_free', request.isFree], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['reverse', request.reverse], ['tags', request.tags])
  }, unmarshalListIPsResponse);

  /**
   * List IPs. List Public Gateway flexible IP addresses. A number of filter
   * options are available for limiting results in the response.
   *
   * @param request - The request {@link ListIPsRequest}
   * @returns A Promise of ListIPsResponse
   */
  listIPs = (request = {}) => enrichForPagination('ips', this.pageOfListIPs, request);

  /**
   * Get an IP. Get details of a Public Gateway flexible IP address, identified
   * by its IP ID. The response object contains information including which (if
   * any) Public Gateway using this IP address, the reverse and various other
   * metadata.
   *
   * @param request - The request {@link GetIPRequest}
   * @returns A Promise of IP
   */
  getIP = request => this.client.fetch({
    method: 'GET',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIP);

  /**
   * Reserve an IP. Create (reserve) a new flexible IP address that can be used
   * for a Public Gateway in a specified Scaleway Project.
   *
   * @param request - The request {@link CreateIPRequest}
   * @returns A Promise of IP
   */
  createIP = (request = {}) => this.client.fetch({
    body: JSON.stringify(marshalCreateIPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips`
  }, unmarshalIP);

  /**
   * Update an IP. Update details of an existing flexible IP address, including
   * its tags, reverse and the Public Gateway it is assigned to.
   *
   * @param request - The request {@link UpdateIPRequest}
   * @returns A Promise of IP
   */
  updateIP = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateIPRequest(request, this.client.settings)),
    headers: jsonContentHeaders$1,
    method: 'PATCH',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ipId', request.ipId)}`
  }, unmarshalIP);

  /**
   * Delete an IP. Delete a flexible IP address from your account. This action
   * is irreversible.
   *
   * @param request - The request {@link DeleteIPRequest}
   */
  deleteIP = request => this.client.fetch({
    method: 'DELETE',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/ips/${validatePathParam('ipId', request.ipId)}`
  });

  /**
   * Refresh a Public Gateway's SSH keys. Refresh the SSH keys of a given Public
   * Gateway, specified by its gateway ID. This adds any new SSH keys in the
   * gateway's Scaleway Project to the gateway itself.
   *
   * @param request - The request {@link RefreshSSHKeysRequest}
   * @returns A Promise of Gateway
   */
  refreshSSHKeys = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders$1,
    method: 'POST',
    path: `/vpc-gw/v1/zones/${validatePathParam('zone', request.zone ?? this.client.settings.defaultZone)}/gateways/${validatePathParam('gatewayId', request.gatewayId)}/refresh-ssh-keys`
  }, unmarshalGateway);
};

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API$1,
  GATEWAY_NETWORK_TRANSIENT_STATUSES: GATEWAY_NETWORK_TRANSIENT_STATUSES,
  GATEWAY_TRANSIENT_STATUSES: GATEWAY_TRANSIENT_STATUSES
});

var index$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1: index_gen$1
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

/** Lists transient statutes of the enum {@link HostingStatus}. */
const HOSTING_TRANSIENT_STATUSES = ['delivering', 'deleting', 'migrating'];

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const unmarshalHostingCpanelUrls = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HostingCpanelUrls' failed as data isn't a dictionary.`);
  }
  return {
    dashboard: data.dashboard,
    webmail: data.webmail
  };
};
const unmarshalHostingOption = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'HostingOption' failed as data isn't a dictionary.`);
  }
  return {
    id: data.id,
    name: data.name
  };
};
const unmarshalHosting = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Hosting' failed as data isn't a dictionary.`);
  }
  return {
    controlPanelName: data.control_panel_name,
    cpanelUrls: data.cpanel_urls ? unmarshalHostingCpanelUrls(data.cpanel_urls) : undefined,
    createdAt: unmarshalDate(data.created_at),
    dnsStatus: data.dns_status,
    domain: data.domain,
    id: data.id,
    ipv4: data.ipv4,
    ipv6: data.ipv6,
    offerEndOfLife: data.offer_end_of_life,
    offerId: data.offer_id,
    offerName: data.offer_name,
    oneTimePassword: data.one_time_password,
    options: unmarshalArrayOfObject(data.options, unmarshalHostingOption),
    organizationId: data.organization_id,
    platformGroup: data.platform_group,
    platformHostname: data.platform_hostname,
    platformNumber: data.platform_number,
    projectId: data.project_id,
    protected: data.protected,
    region: data.region,
    status: data.status,
    tags: data.tags,
    updatedAt: unmarshalDate(data.updated_at),
    username: data.username
  };
};
const unmarshalDnsRecord = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DnsRecord' failed as data isn't a dictionary.`);
  }
  return {
    name: data.name,
    priority: data.priority,
    status: data.status,
    ttl: data.ttl,
    type: data.type,
    value: data.value
  };
};
const unmarshalNameserver = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Nameserver' failed as data isn't a dictionary.`);
  }
  return {
    hostname: data.hostname,
    isDefault: data.is_default,
    status: data.status
  };
};
const unmarshalDnsRecords = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'DnsRecords' failed as data isn't a dictionary.`);
  }
  return {
    nameServers: unmarshalArrayOfObject(data.name_servers, unmarshalNameserver),
    records: unmarshalArrayOfObject(data.records, unmarshalDnsRecord),
    status: data.status
  };
};
const unmarshalControlPanel = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ControlPanel' failed as data isn't a dictionary.`);
  }
  return {
    available: data.available,
    logoUrl: data.logo_url,
    name: data.name
  };
};
const unmarshalListControlPanelsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListControlPanelsResponse' failed as data isn't a dictionary.`);
  }
  return {
    controlPanels: unmarshalArrayOfObject(data.control_panels, unmarshalControlPanel),
    totalCount: data.total_count
  };
};
const unmarshalListHostingsResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListHostingsResponse' failed as data isn't a dictionary.`);
  }
  return {
    hostings: unmarshalArrayOfObject(data.hostings, unmarshalHosting),
    totalCount: data.total_count
  };
};
const unmarshalOfferProduct = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'OfferProduct' failed as data isn't a dictionary.`);
  }
  return {
    databasesQuota: data.databases_quota,
    emailAccountsQuota: data.email_accounts_quota,
    emailStorageQuota: data.email_storage_quota,
    hostingStorageQuota: data.hosting_storage_quota,
    maxAddonDomains: data.max_addon_domains,
    name: data.name,
    option: data.option,
    ram: data.ram,
    supportIncluded: data.support_included,
    vCpu: data.v_cpu
  };
};
const unmarshalOffer = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Offer' failed as data isn't a dictionary.`);
  }
  return {
    available: data.available,
    billingOperationPath: data.billing_operation_path,
    controlPanelName: data.control_panel_name,
    endOfLife: data.end_of_life,
    id: data.id,
    price: data.price ? unmarshalMoney(data.price) : undefined,
    product: data.product ? unmarshalOfferProduct(data.product) : undefined,
    quotaWarnings: data.quota_warnings
  };
};
const unmarshalListOffersResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ListOffersResponse' failed as data isn't a dictionary.`);
  }
  return {
    offers: unmarshalArrayOfObject(data.offers, unmarshalOffer)
  };
};
const unmarshalResetHostingPasswordResponse = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ResetHostingPasswordResponse' failed as data isn't a dictionary.`);
  }
  return {
    password: data.password
  };
};
const unmarshalSession = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Session' failed as data isn't a dictionary.`);
  }
  return {
    url: data.url
  };
};
const marshalCreateHostingRequestDomainConfiguration = (request, defaults) => ({
  update_all_records: request.updateAllRecords,
  update_mail_record: request.updateMailRecord,
  update_nameservers: request.updateNameservers,
  update_web_record: request.updateWebRecord
});
const marshalCreateHostingRequest = (request, defaults) => ({
  domain: request.domain,
  domain_configuration: request.domainConfiguration !== undefined ? marshalCreateHostingRequestDomainConfiguration(request.domainConfiguration) : undefined,
  email: request.email,
  language: request.language,
  offer_id: request.offerId,
  option_ids: request.optionIds,
  project_id: request.projectId ?? defaults.defaultProjectId,
  tags: request.tags
});
const marshalUpdateHostingRequest = (request, defaults) => ({
  email: request.email,
  offer_id: request.offerId,
  option_ids: request.optionIds,
  protected: request.protected,
  tags: request.tags
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.
const jsonContentHeaders = {
  'Content-Type': 'application/json; charset=utf-8'
};

/** Web Hosting API. */
class API extends API$z {
  /** Lists the available regions of the API. */
  static LOCALITIES = ['fr-par', 'nl-ams'];

  /**
   * Order a Web Hosting plan. Order a Web Hosting plan, specifying the offer
   * type required via the `offer_id` parameter.
   *
   * @param request - The request {@link CreateHostingRequest}
   * @returns A Promise of Hosting
   */
  createHosting = request => this.client.fetch({
    body: JSON.stringify(marshalCreateHostingRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hostings`
  }, unmarshalHosting);
  pageOfListHostings = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hostings`,
    urlParams: urlParams(['control_panels', request.controlPanels], ['domain', request.domain], ['order_by', request.orderBy], ['organization_id', request.organizationId], ['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize], ['project_id', request.projectId], ['statuses', request.statuses], ['tags', request.tags])
  }, unmarshalListHostingsResponse);

  /**
   * List all Web Hosting plans. List all of your existing Web Hosting plans.
   * Various filters are available to limit the results, including filtering by
   * domain, status, tag and Project ID.
   *
   * @param request - The request {@link ListHostingsRequest}
   * @returns A Promise of ListHostingsResponse
   */
  listHostings = (request = {}) => enrichForPagination('hostings', this.pageOfListHostings, request);

  /**
   * Get a Web Hosting plan. Get the details of one of your existing Web Hosting
   * plans, specified by its `hosting_id`.
   *
   * @param request - The request {@link GetHostingRequest}
   * @returns A Promise of Hosting
   */
  getHosting = request => this.client.fetch({
    method: 'GET',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hostings/${validatePathParam('hostingId', request.hostingId)}`
  }, unmarshalHosting);

  /**
   * Waits for {@link Hosting} to be in a final state.
   *
   * @param request - The request {@link GetHostingRequest}
   * @param options - The waiting options
   * @returns A Promise of Hosting
   */
  waitForHosting = (request, options) => waitForResource(options?.stop ?? (res => Promise.resolve(!HOSTING_TRANSIENT_STATUSES.includes(res.status))), this.getHosting, request, options);

  /**
   * Update a Web Hosting plan. Update the details of one of your existing Web
   * Hosting plans, specified by its `hosting_id`. You can update parameters
   * including the contact email address, tags, options and offer.
   *
   * @param request - The request {@link UpdateHostingRequest}
   * @returns A Promise of Hosting
   */
  updateHosting = request => this.client.fetch({
    body: JSON.stringify(marshalUpdateHostingRequest(request, this.client.settings)),
    headers: jsonContentHeaders,
    method: 'PATCH',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hostings/${validatePathParam('hostingId', request.hostingId)}`
  }, unmarshalHosting);

  /**
   * Delete a Web Hosting plan. Delete a Web Hosting plan, specified by its
   * `hosting_id`. Note that deletion is not immediate: it will take place at
   * the end of the calendar month, after which time your Web Hosting plan and
   * all its data (files and emails) will be irreversibly lost.
   *
   * @param request - The request {@link DeleteHostingRequest}
   * @returns A Promise of Hosting
   */
  deleteHosting = request => this.client.fetch({
    method: 'DELETE',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hostings/${validatePathParam('hostingId', request.hostingId)}`
  }, unmarshalHosting);

  /**
   * Restore a Web Hosting plan. When you [delete a Web Hosting
   * plan](#path-hostings-delete-a-hosting), definitive deletion does not take
   * place until the end of the calendar month. In the time between initiating
   * the deletion, and definitive deletion at the end of the month, you can
   * choose to **restore** the Web Hosting plan, using this endpoint and
   * specifying its `hosting_id`.
   *
   * @param request - The request {@link RestoreHostingRequest}
   * @returns A Promise of Hosting
   */
  restoreHosting = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hostings/${validatePathParam('hostingId', request.hostingId)}/restore`
  }, unmarshalHosting);

  /**
   * Get DNS records. Get the set of DNS records of a specified domain
   * associated with a Web Hosting plan.
   *
   * @param request - The request {@link GetDomainDnsRecordsRequest}
   * @returns A Promise of DnsRecords
   */
  getDomainDnsRecords = request => this.client.fetch({
    method: 'GET',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/domains/${validatePathParam('domain', request.domain)}/dns-records`
  }, unmarshalDnsRecords);

  /**
   * List all offers. List the different Web Hosting offers, and their options,
   * available to order from Scaleway.
   *
   * @param request - The request {@link ListOffersRequest}
   * @returns A Promise of ListOffersResponse
   */
  listOffers = request => this.client.fetch({
    method: 'GET',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/offers`,
    urlParams: urlParams(['control_panels', request.controlPanels], ['hosting_id', request.hostingId], ['only_options', request.onlyOptions], ['order_by', request.orderBy], ['without_options', request.withoutOptions])
  }, unmarshalListOffersResponse);
  pageOfListControlPanels = (request = {}) => this.client.fetch({
    method: 'GET',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/control-panels`,
    urlParams: urlParams(['page', request.page], ['page_size', request.pageSize ?? this.client.settings.defaultPageSize])
  }, unmarshalListControlPanelsResponse);

  /**
   * "List the control panels type: cpanel or plesk.".
   *
   * @param request - The request {@link ListControlPanelsRequest}
   * @returns A Promise of ListControlPanelsResponse
   */
  listControlPanels = (request = {}) => enrichForPagination('controlPanels', this.pageOfListControlPanels, request);

  /**
   * Create a user session.
   *
   * @param request - The request {@link CreateSessionRequest}
   * @returns A Promise of Session
   */
  createSession = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hostings/${validatePathParam('hostingId', request.hostingId)}/sessions`
  }, unmarshalSession);
  resetHostingPassword = request => this.client.fetch({
    body: '{}',
    headers: jsonContentHeaders,
    method: 'POST',
    path: `/webhosting/v1alpha1/regions/${validatePathParam('region', request.region ?? this.client.settings.defaultRegion)}/hostings/${validatePathParam('hostingId', request.hostingId)}/reset-password`
  }, unmarshalResetHostingPasswordResponse);
}

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

const ListControlPanelsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};
const ListHostingsRequest = {
  page: {
    greaterThan: 0
  },
  pageSize: {
    greaterThan: 0,
    lessThanOrEqual: 100
  }
};

var validationRules_gen = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ListControlPanelsRequest: ListControlPanelsRequest,
  ListHostingsRequest: ListHostingsRequest
});

// This file was automatically generated. DO NOT EDIT.
// If you have any remark or suggestion do not hesitate to open an issue.

var index_gen = /*#__PURE__*/Object.freeze({
  __proto__: null,
  API: API,
  HOSTING_TRANSIENT_STATUSES: HOSTING_TRANSIENT_STATUSES,
  ValidationRules: validationRules_gen
});

var index = /*#__PURE__*/Object.freeze({
  __proto__: null,
  v1alpha1: index_gen
});

exports.API = API$z;
exports.Account = index$A;
exports.AppleSilicon = index$z;
exports.BareMetal = index$x;
exports.Billing = index$w;
exports.Block = index$v;
exports.Cockpit = index$u;
exports.Container = index$t;
exports.Dedibox = index$s;
exports.DocumentDB = index$r;
exports.Domain = index$q;
exports.Errors = index$B;
exports.FlexibleIP = index$p;
exports.Function = index$o;
exports.IAM = index$n;
exports.IOT = index$k;
exports.IPAM = index$j;
exports.IPFS = index$i;
exports.Instance = index$l;
exports.Jobs = index$h;
exports.K8S = index$f;
exports.LB = index$d;
exports.LLMInference = index$c;
exports.MNQ = index$a;
exports.Marketplace = index$b;
exports.RDB = index$9;
exports.Redis = index$8;
exports.Registry = index$7;
exports.Secret = index$6;
exports.ServerlessSQLDB = index$5;
exports.Std = index_gen$6;
exports.Test = index$4;
exports.TransactionalEmail = index$3;
exports.VPC = index$2;
exports.VPCGW = index$1;
exports.Webhosting = index;
exports.addAsyncHeaderInterceptor = addAsyncHeaderInterceptor;
exports.authenticateWithSessionToken = authenticateWithSessionToken;
exports.createAdvancedClient = createAdvancedClient;
exports.createClient = createClient;
exports.createExponentialBackoffStrategy = createExponentialBackoffStrategy;
exports.enableConsoleLogger = enableConsoleLogger;
exports.enrichForPagination = enrichForPagination;
exports.isJSONObject = isJSONObject;
exports.marshalMoney = marshalMoney;
exports.marshalScwFile = marshalScwFile;
exports.marshalTimeSeries = marshalTimeSeries;
exports.resolveOneOf = resolveOneOf;
exports.setLogger = setLogger;
exports.tryAtIntervals = tryAtIntervals;
exports.unmarshalArrayOfObject = unmarshalArrayOfObject;
exports.unmarshalDate = unmarshalDate;
exports.unmarshalMapOfObject = unmarshalMapOfObject;
exports.unmarshalMoney = unmarshalMoney;
exports.unmarshalScwFile = unmarshalScwFile;
exports.unmarshalServiceInfo = unmarshalServiceInfo;
exports.unmarshalTimeSeries = unmarshalTimeSeries;
exports.unmarshalTimeSeriesPoint = unmarshalTimeSeriesPoint;
exports.urlParams = urlParams;
exports.validatePathParam = validatePathParam;
exports.waitForResource = waitForResource;
exports.withAdditionalInterceptors = withAdditionalInterceptors;
exports.withDefaultPageSize = withDefaultPageSize;
exports.withHTTPClient = withHTTPClient;
exports.withProfile = withProfile;
exports.withUserAgent = withUserAgent;
exports.withUserAgentSuffix = withUserAgentSuffix;
